/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */

package org.xquark.extractor;

/** This class is a implementation of XMLDriver */
import org.xquark.extractor.common.Log;
import org.xquark.xml.xdbc.*;

public class ExtractorDriver implements XMLDriver {
    private static final String RCSRevision = "$Revision: 1.4 $";
    private static final String RCSName     = "$Name:  $";
    static ExtractorDriver instance = null;
    static {
        try {
            createInstance();
        }
        catch (Exception ex) {
            Log.exception(ex);
        }
        try {
            Class.forName("oracle.jdbc.driver.OracleDriver");
        }
        catch (Exception ex)  {
            Log.exception(ex);
        }
        try {
            Class.forName("com.microsoft.jdbc.sqlserver.SQLServerDriver");
        }
        catch (Exception ex) {
            Log.exception(ex);
        }
        try {
            Class.forName("com.progress.sql.jdbc.JdbcProgressDriver");
        }
        catch (Exception ex) {
            Log.exception(ex);
        }
        try {
            Class.forName("com.sybase.jdbc2.jdbc.SybDriver");
        }
        catch (Exception ex) {
            Log.exception(ex);
        }
    }

    private ExtractorDriver() throws XMLDBCException {
        XMLDriverManager.registerDriver(this);
    }

    public static ExtractorDriver createInstance() throws XMLDBCException {
        if (null == instance) {
            instance = new ExtractorDriver();
        }
        return instance;
    }

    /**
     * @param uri
     * @return org.xquark.xml.xdbc.XMLDataSource
     * @throws org.xquark.xml.xdbc.XMLDBCException
     * @roseuid 3BD14246021C
     */

    public XMLDataSource getDataSource(String uri) throws XMLDBCException {

        XMLDBCUri xmlDbcUri = null ;
        try {
            xmlDbcUri = new XMLDBCUri(uri);
        }
        catch (XMLDBCException e) {
            return null ;
        }
        if (xmlDbcUri.getProtocol().equalsIgnoreCase("xdbc") &&
        xmlDbcUri.getVendor().equalsIgnoreCase("xquark") &&
        xmlDbcUri.getDataSourceType().equalsIgnoreCase("extractor")) {
            return new Extractor(xmlDbcUri);
        }
        else {
            return null;
        }
    }

    protected  void finalize() {
    }
}
