/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */

package org.xquark.extractor;

import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;

import org.xquark.extractor.common.Debug;
import org.xquark.extractor.common.MessageLibrary;
import org.xquark.extractor.common.Trace;


	class LookAheadJdbcResultSet {
		private static final String RCSRevision = "$Revision: 1.3 $";
		private static final String RCSName     = "$Name:  $";

		private  int  BOR       = 0;     /* begin of resultset*/
		private  int  BUFFER    = 1;
		private  int  RESULTSET = 2;

		private ResultSet   _resultset;
		private List        _buffer = new ArrayList();
		private List        _attributeNameList = new ArrayList();
		private int         _attributeNumber;

		private boolean _bufferNotEmpty;
		private boolean _resultSetNotEmpty;
		private int     _cursor;

		public LookAheadJdbcResultSet( ResultSet resultset) throws SQLException {
			Trace.enter(this, "LookAheadSqlResultSet()");
			_resultset = resultset;

			ResultSetMetaData _rsm = _resultset.getMetaData();

			_attributeNumber = _rsm.getColumnCount();
			for (int i = 1; i <= _attributeNumber; i++) {
				_attributeNameList.add(_rsm.getColumnName(i));
			}

			_bufferNotEmpty = _resultset.next();
			if (_bufferNotEmpty) {
				refreshBuffer();
				_resultSetNotEmpty = _resultset.next();
			}
			_cursor = BOR;
			Trace.exit(this, "LookAheadSqlResultSet()");
		}

		public boolean next()  throws SQLException  {
			Trace.enter(this, "next()");
			boolean retVal ;
			if (BOR == _cursor) {
				_cursor = BUFFER;
			}
			else {
				Debug.assertTrue(BUFFER == _cursor, "BUFFER == _cursor");
				if (_resultSetNotEmpty) {
					refreshBuffer();
					_resultSetNotEmpty = _resultset.next();
				}
				else {
					_resultset.close();
					_resultset = null;
					_bufferNotEmpty = false;
				}
			}
			Trace.exit(this, "next()");
			return _bufferNotEmpty;
		}

		public boolean hasNext() {
			Trace.enter(this, "hasNext()");
			boolean retVal = false;

			if (BUFFER ==_cursor) {
				 retVal = _resultSetNotEmpty;
			}
			else if (BOR ==_cursor) {
				retVal = _bufferNotEmpty;
			} else {
				Debug.assertTrue(false,MessageLibrary.getMessage("IE_ERR"));
			}

			Trace.exit(this, "hasNext()");
			return retVal;
		}

		public boolean lookAhead() {
			Trace.enter(this, "lookAhead()");
			boolean retVal = false;
			if (BOR == _cursor) {
				retVal = _bufferNotEmpty;
				//org.xquark.xml.misc.Debug.print(this,"IN	lookAhead (BOF == _cursor)::retVal -> "+retVal);
				if (retVal) {
					_cursor = BUFFER;
				}
			}
			else if (BUFFER == _cursor) {
				retVal = _resultSetNotEmpty;
				//org.xquark.xml.misc.Debug.print(this,"IN	lookAhead (BUFFER == _cursor)::retVal -> "+retVal);
				if (retVal) {
					_cursor = RESULTSET;
				}
			}
			else {
				org.xquark.extractor.common.Debug.assertTrue(false, "Logic error");
			}

			Trace.exit(this, "lookAhead()");
			return retVal;
		}

		public boolean goBack() {
			Trace.enter(this, "goBack()");
			boolean retVal= false;
			if (BUFFER == _cursor) {
				_cursor = BOR;
				retVal = true ;
			}
			else if (RESULTSET == _cursor) {
				_cursor = BUFFER;
				retVal = true ;
			}
			else if (BOR == _cursor) {
				retVal = false ;
			}

			Trace.exit(this, "goBack()");
			return retVal;
		}

		public long getLong(String name)  throws SQLException {
			Trace.enter(this, "getLong(String name)");
			long retVal = 0;
			if ( BUFFER == _cursor) {
				retVal = getLongFromBuffer(name);
			}
			else {
				try {
					retVal = _resultset.getLong(name);
				}
				catch (Exception e) {}
			}
			Trace.exit(this, "getLong(String name)");
			return retVal;
		}

		public String getString(String name)  throws SQLException {
			Trace.enter(this, "getSting(String name)");
			String retVal = null;
			if ( BUFFER == _cursor) {
				retVal = getStringFromBuffer(name);
			}
			else {
				try {
					retVal = _resultset.getString(name);
				}
				catch (Exception e) {}
			}
			Trace.exit(this, "getSting(String name)");
			return retVal;
		}

		private void refreshBuffer() throws SQLException {
			Trace.enter(this, "refreshBuffer()");
			_buffer.clear();
			for (int i = 1; i <= _attributeNumber; i++) {
				_buffer.add(_resultset.getString(i));
			}
			Trace.exit(this, "refreshBuffer()");
		}

		public String getStringFromBuffer(String name) {
			Trace.enter(this, "getStingFromBuffer(String name)");
			String retVal = null;
			int index = -1;
			for (int i = 0; i < _attributeNameList.size(); i++) {
				if (name.equalsIgnoreCase((String)_attributeNameList.get(i))) {
					index = i;
					break;
				}
			}
			if (-1 != index) {
				if (!_buffer.isEmpty()) // added   22/03/2002
					retVal = (String)_buffer.get(index);
			}
			Trace.exit(this, "getStingFromBuffer(String name)");
			return retVal;
		}

		public long getLongFromBuffer(String name) {
			Trace.enter(this, "getStingFromBuffer(String name)");
			long retVal = 0;
			int index = -1;
			for (int i = 0; i < _attributeNameList.size(); i++) {
				if (name.equalsIgnoreCase((String)_attributeNameList.get(i))) {
					index = i;
					break;
				}
			}
			if (-1 != index) {
				if (!_buffer.isEmpty()) // added   22/03/2002
					retVal = Long.parseLong((String)_buffer.get(index));
			}
			Trace.exit(this, "getStingFromBuffer(String name)");
			return retVal;
		}

		/** @todo this function is to be removed after debug  */
		public ResultSet getSqlResultSet() { return _resultset; }

//
//        public void write(DataOutputStream dos)  throws java.io.IOException {
//            Trace.enter(this,"write(DataOutputStream dos)");
//
//            String newLine = System.getProperty("line.separator");
//
//            dos.writeBytes("<LookAheadResultSet>");
//            dos.writeBytes(newLine);
//
//            dos.writeBytes("<header>");
//            for (int i = 0; i < _attributeNameList.size(); i++) {
//                Object obj = _attributeNameList.get(i);
//                if (null != obj) {
//                    dos.writeBytes((String)obj);
//                    dos.writeBytes(",\t");
//                }
//            }
//            dos.writeBytes("</header>");
//            dos.writeBytes(newLine);
//
//            try {
//                while (next()) {
//                    dos.writeBytes("<row>");
//                    for (int i = 0; i < _buffer.size(); i++) {
//                        Object obj = _buffer.get(i);
//                        if (null != obj) {
//                            dos.writeBytes((String)obj);
//                            dos.writeBytes(",\t");
//                        }
//                    }
//                    dos.writeBytes("</row>");
//                    dos.writeBytes(newLine);
//                }
//
//                dos.writeBytes("</LookAheadResultSet>");
//                dos.writeBytes(newLine);;
//            }
//            catch (Exception ex) {
//                ex.printStackTrace();
//            }
//
//            Trace.exit(this,"write(DataOutputStream dos)");
//        }
		public void close()  {
			try {
				if (null != _resultset) {
					_resultset.close();
				}
			}
			catch (Exception ex) {
			}
		}
	}

