/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */

package org.xquark.extractor;

import java.util.List;

import org.xquark.xml.xdbc.XMLDBCException;
import org.xquark.xquery.xdbc.XDBCResultSetInterface;

public class UnionResultSet implements XDBCResultSetInterface {

	private static final String RCSRevision = "$Revision: 1.3 $";
	private static final String RCSName = "$Name:  $";

	private List _sRSList = null;
	private int _sRSNum = 0;
	private SynchronizedResultSet _cRS = null;
	private int _cRSIndex = -1;

    public UnionResultSet() {
    }

    public UnionResultSet(List sRSList) {
		setSubResultSetList(sRSList);
    }

	public void setSubResultSetList(List sRSList) {
		_sRSList = sRSList;
		_sRSNum = _sRSList.size();
		_cRSIndex = 0;
		_cRS = (SynchronizedResultSet)_sRSList.get(_cRSIndex);
	}

	public String fetch(String path,int nodeAccessor,String loopID) throws XMLDBCException {
		return fetch(path,nodeAccessor);
	}
    public String fetch(String path,int nodeAccessor) throws XMLDBCException {
		return _cRS.fetch(path, nodeAccessor);
	}

	public void generate(String path,int nodeAccessor,String loopID) throws XMLDBCException{
		generate(path, nodeAccessor);
	}
    public void generate(String path,int nodeAccessor) throws XMLDBCException{
		_cRS.generate(path, nodeAccessor);
	}

    public void getIdentifiers(Object[] identifiers) throws XMLDBCException{
		_cRS.getIdentifiers(identifiers);
		shiftIdentifiers(_cRS, identifiers);
	}

    public void getNextIdentifiers(Object[] identifiers) throws XMLDBCException {
		if (_cRS.hasNext()) {
			_cRS.getNextIdentifiers(identifiers);
			shiftIdentifiers(_cRS, identifiers);
		}
		else {
			SynchronizedResultSet rs = null;
			for (int i = _cRSIndex +1 ; i < _sRSNum; i++) {
				rs = (SynchronizedResultSet)_sRSList.get(i);
				if (rs.hasNext()) {
					rs.getNextIdentifiers(identifiers);
					shiftIdentifiers(rs, identifiers);
					break;
				}
			}
		}
	}

    public boolean next() throws XMLDBCException{
		boolean retVal = false;
		retVal = _cRS.next();
		if (!retVal) {
			SynchronizedResultSet rs = null;
			for (int i = _cRSIndex +1 ; i < _sRSNum; i++) {
				rs = (SynchronizedResultSet)_sRSList.get(i);
				retVal = rs.next();
				if (retVal) {
					_cRSIndex = i;
					_cRS = rs;
					break;
				}
			}
		}
		return retVal;
	}

    public boolean hasNext() throws XMLDBCException{
		boolean retVal = false;
		retVal = _cRS.hasNext();
		if (!retVal) {
			SynchronizedResultSet rs = null;
			for (int i = _cRSIndex +1 ; i < _sRSNum; i++) {
				rs = (SynchronizedResultSet)_sRSList.get(i);
				retVal = rs.hasNext();
				if (retVal) {
					break;
				}
			}
		}
		return retVal;
	}
    /**
     * To set a content handler (SAX 2) to intercept events produce when retrieves results by SAX 2 way.
     * @param handler the content handler implementation (see org.xml.sax.ContentHandler)
     * @throws XDBCException if a database access error occurs.
     */
    public void setContentHandler(org.xml.sax.ContentHandler handler){
		SynchronizedResultSet rs = null;
		for (int i = 0; i < _sRSNum; i++) {
			rs = (SynchronizedResultSet)_sRSList.get(i);
			rs.setContentHandler(handler);
		}
	}

    public void setLexicalHandler(org.xml.sax.ext.LexicalHandler handler) {
		SynchronizedResultSet rs = null;
		for (int i = 0; i < _sRSNum; i++) {
			rs = (SynchronizedResultSet)_sRSList.get(i);
			rs.setLexicalHandler(handler);
		}
	}

    public void setErrorHandler(org.xml.sax.ErrorHandler handler){
		SynchronizedResultSet rs = null;
		for (int i = 0; i < _sRSNum; i++) {
			rs = (SynchronizedResultSet)_sRSList.get(i);
			rs.setErrorHandler(handler);
		}
	}

    public void close() throws XMLDBCException {
		SynchronizedResultSet rs = null;
		for (int i = 0; i < _sRSNum; i++) {
			rs = (SynchronizedResultSet)_sRSList.get(i);
			rs.close();
		}
	}

	private void shiftIdentifiers(SynchronizedResultSet rs, Object[] ids) throws XMLDBCException {
		int position = 0;
		for (int i = 0; i < _sRSNum; i++) {
			rs = (SynchronizedResultSet)_sRSList.get(i);
			if (rs != _cRS) {
				position += rs._aoThisId.length;
			}
			else {
				break;
			}
		}

		if (0 != position) {
			for (int i = 0; i < _cRS._aoThisId.length; i++) {
				ids[i + position] = ids[i];
				ids[i] = null;
			}
		}
	}
}
