/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */

package org.xquark.extractor;

/** This class encapsule URI for XMLConnection. it parse URI String and provide
 *  accessor methode for each field of the URI.
 */

import org.xquark.extractor.common.Trace;
import org.xquark.xml.xdbc.XMLDBCException;

public class XMLDBCUri
{

	private static final String RCSRevision = "$Revision: 1.3 $";
	private static final String RCSName     = "$Name:  $";

    private String _protocol;           /* "xdbc" */
    private String _vendor;             /* "exml" */
    private String _dataSourceType;     /* "extractor" or "repository" */
    private String _jdbcUri;
    private String  _configFileName;    /* config file name */

    /**
    @param UriString
    @roseuid 3BD17C6F0300
     */
    public XMLDBCUri(String UriString)
        throws XMLDBCException
    {
        parseUri(UriString);
    }

    /**
    @roseuid 3BD17BFA0239
     */
    public XMLDBCUri()
    {

    }

    private void parseUri(String uriString) throws XMLDBCException
    {
        Trace.enter(this, "parseUri(String uriString)", uriString);

        /** @todo  */
        int index = uriString.indexOf(":");
        if (-1 == index) {
            throw new XMLDBCException("URI string is not valid.");
        }
        _protocol = uriString.substring(0,index);

        int newIndex = uriString.indexOf(":", index+1);
        if (-1 == newIndex) {
            throw new XMLDBCException("URI string is not valid.");
        }
        _vendor = uriString.substring(index+1, newIndex);

        index = newIndex;
        newIndex = uriString.indexOf(":", index+1);
        if (-1 == newIndex) {
            throw new XMLDBCException("URI string is not valid.");
        }
        _dataSourceType = uriString.substring(index+1, newIndex);

        index = newIndex;
        newIndex = uriString.indexOf(":", index+1);
        if (-1 == newIndex) {
            throw new XMLDBCException("URI string is not valid.");
        }
        String token = uriString.substring(index+1, newIndex);

        if (token.equalsIgnoreCase("jdbc")) {
            _jdbcUri = uriString.substring(index+1, uriString.length());
        }
        else if (token.equalsIgnoreCase("file")) {
            _configFileName = uriString.substring(newIndex+1, uriString.length());
        }
        else {
            throw new XMLDBCException("URI string is not valid.");
        }

        Trace.exit(this, "parseUri(String uriString)", uriString);
    }


    /**
    Access method for the _protocol property.

    @return   the current value of the _protocol property
     */
    public String getProtocol()
    {
        return _protocol;
    }

    /**
    Sets the value of the _protocol property.

    @param aProtocol the new value of the _protocol property
     */
    public void setProtocol(String aProtocol)
    {
        _protocol = aProtocol;
    }

    /**
    Access method for the _vendor property.

    @return   the current value of the _vendor property
     */
    public String getVendor()
    {
        return _vendor;
    }

    /**
    Sets the value of the _vendor property.

    @param aVendor the new value of the _vendor property
     */
    public void setVendor(String aVendor)
    {
        _vendor = aVendor;
    }

    /**
    Access method for the _dataSourceType property.

    @return   the current value of the _dataSourceType property
     */
    public String getDataSourceType()
    {
        return _dataSourceType;
    }

    /**
    Sets the value of the _dataSourceType property.

    @param aDataSourceType the new value of the _dataSourceType property
     */
    public void setDataSourceType(String aDataSourceType)
    {
        _dataSourceType = aDataSourceType;
    }

    /**
    Access method for the _configFileName property.

    @return   the current value of the _configFileName property
     */
    public String getConfigFileName() {
        return _configFileName;
    }

    /**
    Sets the value of the _configFileName property.

    @param aJdbcUri the new value of the _configFileName property
     */
    public void setConfigFileName(String configFileName) {
        _configFileName = configFileName;
    }


    /**
    Access method for the _jdbcUri property.

    @return   the current value of the _jdbcUri property
     */
    public String getJdbcUri()
    {
        return _jdbcUri;
    }

    /**
    Sets the value of the _jdbcUri property.

    @param aJdbcUri the new value of the _jdbcUri property
     */
    public void setJdbcUri(String aJdbcUri)
    {
        _jdbcUri = aJdbcUri;
    }

    public boolean containsJDBCUri() { return null != _jdbcUri; }

    public boolean containsConfigFileName() { return null != _configFileName; }

    /**
    @return String
    @roseuid 3BD17BCE01FA
     */
    public String getUriString()
    {
        StringBuffer retVal = new StringBuffer();
        if (null != _protocol){
            retVal.append(_protocol);
            retVal.append(":");
        }
        if (null != _vendor){
            retVal.append(_vendor);
            retVal.append(":");
        }
        if (null != _dataSourceType){
            retVal.append(_dataSourceType);
            retVal.append(":");
        }

        if (null != _configFileName){
            retVal.append("file:");
            retVal.append(_configFileName);
        }

        if (null != _jdbcUri){
            retVal.append(_jdbcUri);
        }
        return retVal.toString();
    }

}
