/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */

package org.xquark.extractor.algebra;

import java.util.*;

import org.xquark.extractor.common.Trace;

/**
 * Static utilities for relational attributes
 */
public final class AlgebraTools {

    private static final String RCSRevision = "$Revision: 1.3 $";
    private static final String RCSName = "$Name:  $";

    public static List clone(List list) throws CloneNotSupportedException {
        List retVal = new ArrayList();

        try {
            for (int i = 0; i < list.size(); i++) {
                Object obj = list.get(i);

                if (obj instanceof Expression) {
                    retVal.add(((Expression) obj).clone());
                }
                else if (obj instanceof MapItem) {
                    retVal.add(((MapItem) obj).clone());
                }
                else if (obj instanceof SqlType) {
                    retVal.add(((SqlType) obj).clone());
                }
                else if (obj instanceof String) {
                    retVal.add(obj);
                }
                else if (null == obj) {
                    retVal.add(null);
                }
                else {
                    Trace.error(
                        "Tools",
                        "clone(List)",
                        "object to be cloned is not Expression"
                            + obj.toString());
                    throw new CloneNotSupportedException(
                        "Error in cloning ArrayList" + obj.toString());
                }
            }

        }
        catch (Exception ex) {
            throw new CloneNotSupportedException("Error in cloning ArrayList");
        }

        return retVal;
    }

    public static List clone(List list, Map clonedExprs)
        throws CloneNotSupportedException {
        if (null == list) {
            return null;
        }
        List retVal = new ArrayList();

        try {
            for (int i = 0; i < list.size(); i++) {
                Object obj = list.get(i);

                if (obj instanceof Expression) {
                    retVal.add(((Expression) obj).clone(clonedExprs));
                }
                else if (obj instanceof MapItem) {
                    retVal.add(((MapItem) obj).clone());
                }
                else if (obj instanceof SqlType) {
                    retVal.add(((SqlType) obj).clone());
                }
                else if (obj instanceof String) {
                    retVal.add(new String((String) obj));
                }
                else {
                    Trace.error(
                        "Tools",
                        "clone(List list, Map clonedExprs)",
                        "object to be cloned is not Expression"
                            + obj.toString());
                    throw new CloneNotSupportedException(
                        "Error in cloning ArrayList" + obj.toString());
                }
            }

        }
        catch (Exception ex) {
            throw new CloneNotSupportedException("Error in cloning ArrayList");
        }

        return retVal;
    }

    /* Compares deeply expression without considering order */
    public static boolean areExprListEquivalent(
        List exprList1,
        List exprList2) {
        int len1 = exprList1.size(), len2 = exprList2.size();
        boolean ret = (len1 == len2), found;
        LinkedList optList2 = new LinkedList(exprList2);
        // clone is performed for optimization
        // and matching expression are removed (linked list is more efficient)
        Expression expr;
        ListIterator it;

        /* Browse the reference list */
        for (int i = 0; ret && i < len1; i++) {
            expr = (Expression) exprList1.get(i);
            it = optList2.listIterator();
            found = false;
            /* browse the compared cloned list and remove item that matches */
            while (it.hasNext()) {
                if (expr.deepEquals(it.next())) {
                    it.remove();
                    found = true;
                }
            }
            ret &= found;
        }
        return ret;
    }
    
    public static void applyVisitorOnCollection(AlgebraVisitor visitor, Collection exprList) {
    	Iterator it = exprList.iterator();
    	while (it.hasNext()) {
    		((Expression)it.next()).accept(visitor);
    	}
    }

    //		/**
    //		 * create a Map where all visible table instances of <code>node<code> are mapped to
    //		 * the <code>node<code> itself.
    //		 */
    //		public Map createRelaceMapForRenameRelation(RenameRelation node) {
    //			Map retVal = new Hashtable();
    //
    //			Relation relation = (Relation) node;
    //			Set vti = relation.visibleTableInstances();
    //
    //			Iterator iter = vti.iterator();
    //			while (iter.hasNext()) {
    //				Object item = iter.next();
    //				retVal.put(item, node);
    //			}
    //		}

}
