/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */

package org.xquark.extractor.algebra;

import java.util.*;

import org.xquark.extractor.common.Trace;

public abstract class BinaryOperator extends Expression
{

    private static final String RCSRevision = "$Revision: 1.3 $";
    private static final String RCSName     = "$Name:  $";

    protected Expression _rightOperand;
    protected Expression _leftOperand;

    public BinaryOperator()
    {
    }
    /**
     * @param leftOperand
     * @param rightOperand
     * @roseuid 3AB7974D02F9
     */
    public BinaryOperator(Expression leftOperand, Expression rightOperand)
    {
        setLeftOperand ( leftOperand ) ;
        setRightOperand ( rightOperand ) ;
    }

    synchronized Object clone(Map clonedExprs) throws CloneNotSupportedException
    {
        Trace.enter(this, "clone(Map clonedExprs)");

        BinaryOperator retVal = (BinaryOperator)super.clone(clonedExprs);
        retVal.setLeftOperand((getLeftOperand() == null) ? null : (Expression)getLeftOperand().clone (clonedExprs));
        retVal.setRightOperand((getRightOperand() == null) ? null : (Expression) getRightOperand().clone (clonedExprs));

        clonedExprs.put(this, retVal);
        Trace.exit(this, "clone(Map clonedExprs)");
        return  retVal;

    }

//    public boolean equals (Object o)
//    {
//        if (o == this)
//            return true;
//        if (!(o instanceof BinOpArithmetic))
//            return false;
//        BinOpArithmetic p = (BinOpArithmetic) o;
//        if ( getOperator() != p.getOperator())
//            return false;
//
//        if ( !getLeftOperand().equals(p.getLeftOperand()))
//            return false;
//        if ( !getRightOperand().equals(p.getRightOperand()))
//            return false;
//    	return true;
//    }

    /**
     * @return org.xquark.extractor.algebra.Expression
     * @roseuid 3AB869570075
     */
    public Expression getLeftOperand()
    {
        return _leftOperand ;
    }

    /**
     * @return org.xquark.extractor.algebra.Expression
     * @roseuid 3AB8696A011D
     */
    public Expression getRightOperand()
    {
        return _rightOperand ;
    }

    /**
     * @param leftOperand
     * @roseuid 3AB8699D0211
     */
    public void setLeftOperand(Expression leftOperand)
    {
        _leftOperand = leftOperand ;
        _leftOperand.setFather ( this ) ;
    }

    /**
     * @param rightOperand
     * @roseuid 3AB869A80202
     */
    public void setRightOperand(Expression rightOperand)
    {
        _rightOperand = rightOperand ;
        _rightOperand.setFather ( this ) ;
    }

    public List getOperands()
    {
        ArrayList  retVal = new ArrayList () ;
        retVal.add( _leftOperand ) ;
        retVal.add( _rightOperand );
        return retVal ;
    }
    protected Set _visibleTableInstance = null;
    public Set visibleTableInstances() {
    	if (null == _visibleTableInstance) {
    		_visibleTableInstance = new HashSet();
    	}
    	_visibleTableInstance.clear();
    	_visibleTableInstance.addAll( ((Relation)getLeftOperand()).providedTableInstances());
    	_visibleTableInstance.addAll( ((Relation)getRightOperand()).providedTableInstances());
    	return _visibleTableInstance;
    }

	/**
	 * @see Expression#deepEquals(Object)
	 */
	public boolean deepEquals(Object o)
	{
		if (o instanceof BinaryOperator)
		{
			BinaryOperator cast = (BinaryOperator)o;
			return	 (_leftOperand.deepEquals(cast.getLeftOperand())
						&& _rightOperand.deepEquals(cast.getRightOperand()))
					||
					(_leftOperand.deepEquals(cast.getRightOperand())
						&& _rightOperand.deepEquals(cast.getLeftOperand()));
		}
		return false;
	}
}
