/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */

package org.xquark.extractor.algebra;

import org.xquark.extractor.common.SqlWrapperException;
import org.xquark.extractor.common.Trace;

public final class CollectionName {

    private static final String RCSRevision = "$Revision: 1.2 $";
    private static final String RCSName     = "$Name:  $";


    private String _dataSource = null;
    private String _catalog = null;
    private String _schema = null;
    private String _table = null;

    public CollectionName(String name) throws SqlWrapperException{
        Trace.enter(this, " Collection_Name(String name)", name);
        int index = name.indexOf(":");
        if (-1 == index) {
            throw new SqlWrapperException("Collection name string is not valid.");
        }
        _dataSource = name.substring(0,index);

        String [] steps = new String[3];
        int newIndex = -1;
        int stepCursor = 0;
        while (true) {
            newIndex = name.indexOf(".", index + 1);
            if (-1 != newIndex) {
                steps [stepCursor] = name.substring(index + 1, newIndex);
                if (0 ==steps [stepCursor].length()) {
                    throw new SqlWrapperException("Collection name string is not valid.");
                }
                index = newIndex;
                stepCursor ++;
            }
            else {
                steps [stepCursor] = name.substring(index + 1, name.length());
                if (0 ==steps [stepCursor].length()) {
                    throw new SqlWrapperException("Collection name string is not valid.");
                }
                stepCursor ++;
                break;
            }
        }
        if (1 == stepCursor) {
            _table = steps[0];
        }
        else if (2 == stepCursor) {
            _schema = steps[0];
            _table = steps[1];
        }
        else if (3 == stepCursor) {
            _catalog = steps[0];
            _schema = steps[1];
            _table = steps[2];
        }
        else {
            throw new SqlWrapperException("Collection name string is not valid.");
        }
        Trace.exit(this, " Collection_Name(String name)", name);
    }

    public String getDataSource() { return _dataSource;}

    public String getCatalog() { return _catalog;}

    public String getSchema() { return _schema; }
    public void setSchema(String schema) { _schema = schema; }

    public boolean dataSourceIsWildCard()
    {
        boolean retVal = false;
        if (null != _dataSource){
            if (_dataSource.equals("*")) {
                retVal = true;
            }
        }
        return retVal;
    }

    public boolean catalogIsWildCard()
    {
        boolean retVal = false;
        if (null != _catalog){
            if (_catalog.equals("*")) {
                retVal = true;
            }
        }
        return retVal;
    }

    public boolean schemaIsWildCard()
    {
        boolean retVal = false;
        if (null != _schema){
            if (_schema.equals("*")) {
                retVal = true;
            }
        }
        return retVal;
    }

    public boolean tableIsWildCard()
    {
        boolean retVal = false;
        if (null != _table){
            if (_table.equals("*")) {
                retVal = true;
            }
        }
        return retVal;
    }

    public String getTable() { return _table; }

    public void setTable(String table) { _table = table; }

    public String pprint() {
        Trace.enter(this, "pprint()");

        StringBuffer retVal = new StringBuffer();
        if (null != _dataSource) {
            retVal.append(_dataSource);
            retVal.append(":");
        }

        if (null != _catalog) {
            retVal.append(_catalog);
            retVal.append(".");
        }

        if (null != _schema) {
            retVal.append(_schema);
            retVal.append(".");
        }

        if (null != _table) {
            retVal.append(_table);
            retVal.append(".");
        }

        Trace.exit(this, "pprint()");
        return retVal.toString();
    }

    public static void main(String args[]) throws Exception {
        CollectionName cn = new CollectionName("extractor:pubs.dbo.jobs");
        System.out.println(cn.pprint());
    }

    public boolean equals(Object o) {
        if (o instanceof CollectionName) {
            CollectionName cast = (CollectionName) o;
            return _dataSource.equalsIgnoreCase(cast.getDataSource())
                && _catalog.equalsIgnoreCase(cast.getDataSource())
                && _schema.equalsIgnoreCase(cast.getSchema())
                && _table.equalsIgnoreCase(cast.getTable());
        }
        return false;
    }
}
