/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */

package org.xquark.extractor.algebra;

import java.util.*;

import org.xquark.extractor.common.Debug;
import org.xquark.extractor.common.SqlWrapperException;
import org.xquark.extractor.common.Trace;
import org.xquark.extractor.sql.SqlExpression;
import org.xquark.xquery.parser.XQueryExpression;

/**
import java.util.List;
import java.util.Iterator;
import org.xquark.extractor.common.*;
 */
public abstract class Expression implements Cloneable, Constants
{
	private static final String RCSRevision = "$Revision: 1.3 $";
	private static final String RCSName     = "$Name:  $";

	protected SqlType   _type;
	protected Expression _father;
	protected XQueryExpression _OrginalXExpr;
	protected Set _referredAttributes;
	protected Mapper _mapper;

	/**
	Access method for the _father property.

	@return   the current value of the _father property
	 */
	public org.xquark.extractor.algebra.Expression getFather()
	{
		return _father;
	}

	/**
	Sets the value of the _father property.

	@param aFather the new value of the _father property
	 */
	public void setFather(org.xquark.extractor.algebra.Expression aFather)
	{
		_father = aFather;
	}

	/**
	Access method for the _OrginalXExpr property.

	@return   the current value of the _OrginalXExpr property
	 */
	public XQueryExpression getOrginalXExpr()
	{
		return _OrginalXExpr;
	}

	/**
	Sets the value of the _OrginalXExpr property.

	@param aOrginalXExpr the new value of the _OrginalXExpr property
	 */
	public void setOrginalXExpr(XQueryExpression aOrginalXExpr)
	{
		_OrginalXExpr = aOrginalXExpr;
	}


	public void setType(SqlType type)
	{
		_type = type;
	}

	/**
	Access method for the _referredAttributes property.

	@return   the current value of the _referredAttributes property
	 */
	public Set getReferredAttributes()
	{
		return _referredAttributes;
	}

	/**
	Sets the value of the _referredAttributes property.

	@param aReferredAttributes the new value of the _referredAttributes property
	 */
	public void setReferredAttributes(Set aReferredAttributes)
	{
		//_referredAttributes = referredAttributes;
		_referredAttributes = aReferredAttributes;
	}

	/**
	Access method for the _mapper property.

	@return   the current value of the _mapper property
	 */
	public Mapper getMapper()
	{
		/* default implementation*/
		return _mapper;
	}

	/**
	Sets the value of the _mapper property.

	@param aMapper the new value of the _mapper property
	 */
	public void setMapper(Mapper aMapper)
	{
		//_mapper = mapper;
		_mapper = aMapper;
	}

	/**
	@return java.util.List
	@roseuid 3B2787A50176
	 */
	public abstract List getOperands();

	/**
	@return org.xquark.extractor.algebra.SqlType
	@roseuid 3B2778350087
	 */
	public SqlType getType()
	{
		return _type;
	}

	/**
	@return java.util.List
	@roseuid 3B27916B036C
	 */
	public List getChildren()
	{
		Debug.nyi("List getChildren()");
		return null;
	}

	/**
	@return java.lang.String
	@roseuid 3B27916B03DA
	 */
	public String pprint()
	{
		return this.getClass().getName();
	}

	public String toString() {
		return pprint();
	}

	/**
	@return Object
	@throws java.lang.CloneNotSupportedException
	@roseuid 3BD138AF01DE
	 */
	public synchronized Object clone() throws CloneNotSupportedException
	{
		Trace.enter(this, "clone()");

		Object retVal = null;
		Map clonedExprs = new HashMap(500);
		retVal = clone(clonedExprs);
		clonedExprs.clear();
		clonedExprs = null;

		Trace.exit(this, "clone(Map clonedExprs)");
		return  retVal;
	}

	synchronized Object clone(Map clonedExprs) throws CloneNotSupportedException
	{
		return super.clone();
	};

	/**
	@return String
	@roseuid 3BD138B00121
	 */
	public String getName()
	{
		return null;
	}



	/**
	@param oldOprnd
	@param newOprnd
	@roseuid 3BD138B003A2
	 */
	public boolean replaceChild(Expression oldChild, Expression newChild)
	{
		Debug.assertTrue(false,"NYI!! for " + this.pprint());
		return false;
	}

	public abstract SqlExpression accept (GenSqlVisitor visitor) throws SqlWrapperException;

	public abstract void accept (AlgebraVisitor visitor) throws SqlWrapperException;

	/**
	@return java.util.Set
	@roseuid 3BD138B2005C
	 */
	public Set getReferredTableInstances()
	{
		Set retVal = new HashSet();
		if (null != _referredAttributes) {
			Iterator iter = _referredAttributes.iterator();
			AttributeExpression item;
			while (iter.hasNext()) {
				item = (AttributeExpression)iter.next();
				retVal.add(item.getTableInstance());
			}
		}
		return retVal;
	}
	
	/**
	 * The semantics chosen for equals of algebra nodes is "the same algebra operator
	 * hierarchy (aliases ignored) referring to the same database objects".
	 * @param o the expression to be compared with this.
	 */
	public abstract boolean deepEquals(Object o);
}

