/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */

package org.xquark.extractor.algebra;

import java.sql.Date;
import java.text.DateFormat;
import java.util.Map;

import org.xquark.extractor.common.DatabaseManager;
import org.xquark.extractor.common.SqlWrapperException;
import org.xquark.extractor.common.Trace;
import org.xquark.extractor.sql.SqlExpression;

public final class LitDate extends Literal
{

    private static final String RCSRevision = "$Revision: 1.3 $";
    private static final String RCSName     = "$Name:  $";

    private Date _value;

    /**
     * @param value
     * @roseuid 3B276D400065
     */
    public LitDate(java.sql.Date value)
    {
        setValue ( value ) ;
    }

//    synchronized Object clone(Map clonedExprs) throws CloneNotSupportedException
//    {
//        Trace.enter(this, "clone(Map clonedExprs)");
//        LitDate retVal = (LitDate)super.clone();
//        retVal.setValue((getValue() == null) ? null : (java.sql.Date)getValue().clone ());
//        clonedExpr.put(this, retVal);Trace.exit(this, "clone(Map clonedExprs)");return retVal;
//        return  retVal;
//    }
    synchronized Object clone(Map clonedExprs) throws CloneNotSupportedException
    {
        Trace.enter(this, "clone(Map clonedExprs)");

        LitDate retVal = (LitDate)super.clone(clonedExprs);
        retVal.setValue((getValue() == null) ? null : (java.sql.Date)getValue().clone());

        clonedExprs.put(this, retVal);
        Trace.exit(this, "clone(Map clonedExprs)");

        return  retVal;
    }

    public Date getValue ()
    {
        return _value;
    }

    public void setValue (java.sql.Date value)
    {
         _value = value;
    }


    public String pprint  ( )
    {
        String retStr ;

        retStr = DateFormat.getDateInstance( DateFormat.SHORT ,DatabaseManager.getLocale () ).format(_value);
        retStr = "'" + retStr + "'" ;
        return  retStr ;
    }

    public SqlExpression accept (GenSqlVisitor visitor) throws SqlWrapperException
    {
        return visitor.visit(this);
    }

    public void accept (AlgebraVisitor visitor) throws SqlWrapperException
    {
        visitor.visit(this);
    }
    
	/**
	 * @see Expression
	 */
	public boolean deepEquals(Object o)
	{
		if (o instanceof LitDate)
		{
			LitDate cast = (LitDate)o;
			return _value.equals(cast.getValue());
		}
		return false;
	}
}
