/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */

package org.xquark.extractor.algebra;

import java.math.BigDecimal;
import java.sql.*;

import org.xquark.extractor.common.Debug;
import org.xquark.extractor.common.SqlWrapperException;
import org.xquark.extractor.common.Trace;
import org.xquark.extractor.sql.AbstractGenSqlVisitor;
import org.xquark.extractor.sql.SqlExpression;

public class PreEvaluationVisitor extends DefaultCompleteVisitor {

    private static final String RCSRevision = "$Revision: 1.3 $";
    private static final String RCSName     = "$Name:  $";


	AbstractGenSqlVisitor _genSqlVisitor = null;
	Statement _jdbcStatement = null;

    public PreEvaluationVisitor() {
    }

    public PreEvaluationVisitor(AbstractGenSqlVisitor genSqlVisitor, Statement jdbcStatement) {
		setGenSqlVisitor(genSqlVisitor);
		setJdbcStatement(jdbcStatement);
    }

	public void setGenSqlVisitor(AbstractGenSqlVisitor genSqlVisitor) {
	    _genSqlVisitor = genSqlVisitor;
	}

	public void setJdbcStatement(Statement jdbcStatement) {
	    _jdbcStatement = jdbcStatement;
	}

	public void visit(TempValue arg) throws SqlWrapperException {
		Trace.enter(this, "visit(TempValue arg)", arg);
		super.visit(arg);

		if (null == arg.getValue()) {
			/* evaluate this subtree*/
			try {
				Expression subTree = arg.getOperand();
				SqlTypeAtom sqlType = (SqlTypeAtom)subTree.getType();

				_genSqlVisitor.reinit();
				SqlExpression sql = subTree.accept(_genSqlVisitor);
				Trace.log(this, "visit(TempValue arg)", "SQL:" );
				Trace.log(sql.toSql() );
				ResultSet sqlResultSet = sql.Execute(_jdbcStatement);

				sqlResultSet.next();

				Literal litValue = null;

				Object value = sqlResultSet.getObject(1);
				if (null == value) {
					litValue = new LitNull();
				}
				else if (value instanceof Integer) {
					litValue = new LitInteger((Integer)value);
				}
				else if (value instanceof BigDecimal) {
					litValue = new LitDecimal((BigDecimal)value);
				}
				else if (value instanceof Float) {
					litValue = new LitFloat((Float)value);
				}
				else if (value instanceof Double) {
					litValue = new LitDouble((Double)value);
				}
				else if (value instanceof String) {
					litValue = new LitString((String)value);
				}
				else if (value instanceof Date) {
					litValue = new LitDate((Date)value);
				}
				else {
					Debug.nyi("creating a literal for " + value.getClass().getName());
				}

				arg.setValue(litValue);

				sqlResultSet.close();
			}
			catch (SQLException ex) {
				throw new SqlWrapperException(ex.getMessage(), ex);
			}
		}

		Trace.exit(this, "visit(TempValue arg)", arg);
	}
}
