/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */

package org.xquark.extractor.algebra;

import java.util.List;
import java.util.Set;

public interface Relation
{
    /**
	 * Browse recursively operands for {@link Table} stopping on projections and
	 * BinaryAlgebra operations which are returned in place.
     * Returns a set of {@link Relation} containing the same elements that
     * {@link #visibleTableInstances()} for joins and {@link UnaryAlgebra} (group,
     * sort, restrictions), and "this" for {@link BinaryAlgebra} (unions and
	 * other set operations), tables and projections.
     * @return a cached object to clone if modification needed. 
     */
    public Set providedTableInstances();

	/**
	 * Browse recursively operands returning union of {@link Relation#providedTableInstances()}.
	 * Returns a set of {@link Relation} containing the union of 
	 * {@link #providedTableInstances()} results on every operand and "this" for
	 * table nodes.
	 * To summerize, recursive search is performed almost as for {@link #providedTableInstances()}
	 * but one level (the operand level) is at least browsed so calling this
	 * method on a projection or a union will return the table(s) being
	 * projected contrary to {@link #providedTableInstances()}.
     * @return a cached object to clone if modification needed. 
	 */
    public Set visibleTableInstances();

    /**
     */
    public List getKeys();

    /**
     * Returns a list of items available in current subtree with a specified name.
     * Uses responsibility chain.
     * @param name the item name to be searched for.
     * @return List a list of {@link Expression}
     */
    public List nameTest(String name);

    /**
     * Browse relation attributes until finding a non-nullable one.
     * @return null if all relation attributes are nullable or relation has no attributes.
     */
    public AttributeExpression findNonNullAttribute();
}
