/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */

package org.xquark.extractor.algebra;

import java.util.List;
import java.util.Map;

import org.xquark.extractor.common.Debug;
import org.xquark.extractor.common.SqlWrapperException;
import org.xquark.extractor.common.Trace;

public final class RemoveProjectGoUpVisitor extends DefaultAlgebraVisitor {
	private static final String RCSRevision = "$Revision: 1.3 $";
	private static final String RCSName     = "$Name:  $";

	private RemoveProjectReplaceVisitor _rprv = new RemoveProjectReplaceVisitor();

	/* process of GoUp  end up at "_endNode"*/
	private Expression _endNode = null;

	public RemoveProjectGoUpVisitor(){
	}

	public RemoveProjectGoUpVisitor(Map replaceMap){
		setMap(replaceMap);
	}

//    public RemoveProjectReplaceVisitor getReplaceVisitor()
//    {
//        return _rprv;
//    }
//
//    public void setReplaceVisitor(RemoveProjectReplaceVisitor rprv)
//    {
//        _rprv = rprv;
//    }

	public void setMap(Map replaceMap) {
		_rprv.setMap(replaceMap);
	}

	public void setEndNode(Expression endNode) {
		_endNode = endNode;
	}

	public Expression getEndNode() {
		return _endNode;
	}

	public void visit(Expression arg) throws SqlWrapperException {
		Trace.enter(this,"visit(Expression arg)", arg);
		Debug.assertTrue(false, "NYI!! for " + arg.toString());
		Trace.exit(this,"visit(Expression arg)", arg);
	}

	public void visit(BinaryAlgebra arg) throws SqlWrapperException {
		Trace.enter(this,"visit(BinaryAlgebra arg)", arg);

		/* implementation for Merg, Union, Intersect and difference, but not for OuterJoin*/
		if (getEndNode() != arg) {
			arg.getFather().accept(this);
		}

		Trace.exit(this,"visit(BinaryAlgebra arg)", arg);
	}

//    public void visit(BinOpArithmetic arg) throws SqlWrapperException {
//        Trace.enter(this,"visit(BinOpArithmetic arg)", arg);
//        Trace.exit(this,"visit(BinOpArithmetic arg)", arg);
//    }
//
//    public void visit(BinOpBoolean arg) throws SqlWrapperException {
//        Trace.enter(this,"visit(BinOpBoolean arg)", arg);
//        Trace.exit(this,"visit(BinOpBoolean arg)", arg);
//    }
//
//    public void visit(BinOpCompare arg) throws SqlWrapperException {
//        Trace.enter(this,"visit(BinOpCompare arg)", arg);
//        Trace.exit(this,"visit(BinOpCompare arg)", arg);
//    }
//
//    public void visit(BinOpCompareAny arg) throws SqlWrapperException {
//        Trace.enter(this,"visit(BinOpCompareAny arg)", arg);
//        Trace.exit(this,"visit(BinOpCompareAny arg)", arg);
//    }

//    public void visit(BinOpDifference arg) throws SqlWrapperException {
//        Trace.enter(this,"visit(BinOpDifference arg)", arg);
//        Trace.exit(this,"visit(BinOpDifference arg)", arg);
//    }
//
//    public void visit(BinOpIntersect arg) throws SqlWrapperException {
//        Trace.enter(this,"visit(BinOpIntersect arg)", arg);
//        Trace.exit(this,"visit(BinOpIntersect arg)", arg);
//    }
//
//    public void visit(BinOpMerge arg) throws SqlWrapperException {
//        Trace.enter(this,"visit(BinOpMerge arg)", arg);
//        Trace.exit(this,"visit(BinOpMerge arg)", arg);
//    }

	public void visit(BinOpOuterJoin arg) throws SqlWrapperException {
		Trace.enter(this,"visit(BinOpOuterJoin arg)", arg);

		List list = arg.getPredicateList();
		Debug.assertTrue(null!=list,"null!=list");
		Expression expr = null ;
		for (int i = 0; i < list.size(); i++) {
			expr = (Expression)list.get(i);
			expr.accept(_rprv);
		}

		if (getEndNode() != arg) {
			arg.getFather().accept(this);
		}

		Trace.exit(this,"visit(BinOpOuterJoin arg)", arg);
	}

//    public void visit(BinOpUnion arg) throws SqlWrapperException {
//        Trace.enter(this,"visit(BinOpUnion arg)", arg);
//        Trace.exit(this,"visit(BinOpUnion arg)", arg);
//    }

	public void visit(FunAggregate arg) throws SqlWrapperException {
		Trace.enter(this,"visit(FunAggregate arg)", arg);
		Trace.exit(this,"visit(FunAggregate arg)", arg);
	}

//    public void visit(IfThenElse arg) throws SqlWrapperException {
//        Trace.enter(this,"visit(IfThenElse arg)", arg);
//        Trace.exit(this,"visit(IfThenElse arg)", arg);
//    }

	public void visit(Join arg) throws SqlWrapperException {
		Trace.enter(this,"visit(Join arg)", arg);

		List list = arg.getPredicateList();
		if (list != null) {
			Expression expr = null ;
			for (int i = 0; i < list.size(); i++) {
				expr = (Expression)list.get(i);
				expr.accept(_rprv);
			}
		}

		if (getEndNode() != arg) {
			arg.getFather().accept(this);
		}

		Trace.exit(this,"visit(Join arg)", arg);
	}

	public void visit(Literal arg) throws SqlWrapperException {
		Trace.enter(this,"visit(Literal arg)", arg);
		Trace.exit(this,"visit(Literal arg)", arg);
	}

//    public void visit(LitBoolean arg) throws SqlWrapperException {
//        Trace.enter(this,"visit(LitBoolean arg)", arg);
//        Trace.exit(this,"visit(LitBoolean arg)", arg);
//    }
//
//    public void visit(LitDate arg) throws SqlWrapperException {
//        Trace.enter(this,"visit(LitDate arg)", arg);
//        Trace.exit(this,"visit(LitDate arg)", arg);
//    }
//
//    public void visit(LitDouble arg) throws SqlWrapperException {
//        Trace.enter(this,"visit(LitDouble arg)", arg);
//        Trace.exit(this,"visit(LitDouble arg)", arg);
//    }
//
//    public void visit(LitFloat arg) throws SqlWrapperException {
//        Trace.enter(this,"visit(LitFloat arg)", arg);
//        Trace.exit(this,"visit(LitFloat arg)", arg);
//    }
//
//    public void visit(LitInteger arg) throws SqlWrapperException {
//        Trace.enter(this,"visit(LitInteger arg)", arg);
//        Trace.exit(this,"visit(LitInteger arg)", arg);
//    }
//
//    public void visit(LitString arg) throws SqlWrapperException {
//        Trace.enter(this,"visit(LitString arg)", arg);
//        Trace.exit(this,"visit(LitString arg)", arg);
//    }

	public void visit(RenameItem arg) throws SqlWrapperException {
		Trace.enter(this,"visit(RenameItem arg)", arg);

		Trace.exit(this,"visit(RenameItem arg)", arg);
	}

	public void visit(RenameRelation arg) throws SqlWrapperException {
		Trace.enter(this,"visit(RenameRelation arg)", arg);

		/* set "TableInstance" attribute to all AttributExpression*/
		_rprv.updateTableInstance(arg);
		Trace.message(this,"visit(RenameRelation arg)","updated replac map :" +_rprv.printMap());

		/* remove this node */
		Expression father = arg.getFather();
		boolean replaced = father.replaceChild(arg, arg.getOperand());
		Debug.assertTrue(replaced , "replaced");
		arg.setFather(null);
		Trace.message(this,"visit(RenameRelation arg)","RenameRelation removed");

		if (getEndNode() != arg) {
			father.accept(this);
		}

		Trace.exit(this,"visit(RenameRelation arg)", arg);
	}

//    public void visit(SortSpecification arg) throws SqlWrapperException {
//        Trace.enter(this,"visit(SortSpecification arg)", arg);
//
//        Trace.exit(this,"visit(SortSpecification arg)", arg);
//    }

//    public void visit(UnOpAggregate arg) throws SqlWrapperException {
//        Trace.enter(this,"visit(UnaryAtomicOp arg)", arg);
//        Trace.exit(this,"visit(UnaryAtomicOp arg)", arg);
//    }

	public void visit(UnaryAtomicOp arg) throws SqlWrapperException {
		Trace.enter(this,"visit(UnaryAtomicOp arg)", arg);

		if (getEndNode() != arg) {
			arg.getFather().accept(this);
		}

		Trace.exit(this,"visit(UnaryAtomicOp arg)", arg);
	}

//    public void visit(UnOpGroup arg) throws SqlWrapperException {
//        Trace.enter(this,"visit(UnOpGroup arg)", arg);
//        Trace.exit(this,"visit(UnOpGroup arg)", arg);
//    }

	public void visit(UnOpProject arg) throws SqlWrapperException {
		Trace.enter(this,"visit(UnOpProject arg)", arg);

		List list = arg.getItemList();
		Debug.assertTrue(null!=list,"null!=list");
		Expression expr = null ;
		for (int i = 0; i < list.size(); i++) {
			expr = (Expression)list.get(i);
			expr.accept(_rprv);
		}

		UnOpSort sort = arg.getAssociatedSort();
		if (null != sort) {
			SortSpecification spec = null;
			list = sort.getSortSpecificationList();
			for (int i = 0; i < list.size(); i++) {
				spec = (SortSpecification)list.get(i);
				expr = spec.getSortExpression();
				expr.accept(_rprv);
			}
		}

		if (getEndNode() != arg) {
			arg.getFather().accept(this);
		}

		Trace.exit(this,"visit(UnOpProject arg)", arg);
	}

	public void visit(UnOpRestrict arg) throws SqlWrapperException {
		Trace.enter(this,"visit(UnOpRestrict arg)", arg);

		List list = arg.getPredicateList();
		Debug.assertTrue(null!=list,"null!=list");
		Expression expr = null ;
		for (int i = 0; i < list.size(); i++) {
			expr = (Expression)list.get(i);
			expr.accept(_rprv);
		}

		if (getEndNode() != arg) {
			arg.getFather().accept(this);
		}

		Trace.exit(this,"visit(UnOpRestrict arg)", arg);
	}

	public void visit(UnOpSort arg) throws SqlWrapperException {
		Trace.enter(this,"visit(UnOpSort arg)", arg);

		List list = arg.getSortSpecificationList();
		Debug.assertTrue(null!=list,"null!=list");
		SortSpecification spec = null;
		Expression expr = null ;
		for (int i = 0; i < list.size(); i++) {
			spec = (SortSpecification)list.get(i);
			expr = spec.getSortExpression();
			expr.accept(_rprv);
		}

		if (getEndNode() != arg) {
			arg.getFather().accept(this);
		}

		Trace.exit(this,"visit(UnOpSort arg)", arg);
	}

	public String printMap(){
		return _rprv.printMap();
	}
}
