/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */

package org.xquark.extractor.algebra;

import java.util.*;

import org.xquark.extractor.common.Debug;
import org.xquark.extractor.common.SqlWrapperException;
import org.xquark.extractor.common.Trace;
import org.xquark.extractor.xfunctions.AfCast;
import org.xquark.extractor.xfunctions.XFunction;
import org.xquark.extractor.xfunctions.XfCast;

public final class RemoveProjectReplaceVisitor extends DefaultAlgebraVisitor
{
	private static final String RCSRevision = "$Revision: 1.3 $";
	private static final String RCSName     = "$Name:  $";


	public RemoveProjectReplaceVisitor() {
	}

	public RemoveProjectReplaceVisitor(Map map){
		setMap(map);
	}

	private Map _map;

	/**
	Access method for the _map property.

	@return   the current value of the _map property
	 */
	public Map getMap()
	{
		return _map;
	}

	/**
	Sets the value of the _map property.

	@param aMapping the new value of the _map property
	 */
	public void setMap(Map aMap)
	{
		_map = aMap;
	}

	public void updateTableInstance(RenameRelation tableInstance) {
		Trace.enter(this, "updateTableInstance", tableInstance);

		HashMap newMap = new HashMap();
		Set keySet = _map.keySet();
		Iterator iter = keySet.iterator();
		AttributeExpression item = null;
		Object value = null;
		while (iter.hasNext()) {
			item = (AttributeExpression)iter.next();
			value = _map.get(item);

			item.setTableInstance(tableInstance);
			newMap.put(item, value);
		}
		setMap(newMap);

		Trace.exit(this, "updateTableInstance", tableInstance);
	}

	public void visit(Expression arg) throws SqlWrapperException {
		Trace.enter(this,"visit(Expression arg)", arg);
		Debug.assertTrue(false, "NYI!! for " + arg.pprint());
		Trace.exit(this,"visit(Expression arg)", arg);
	}

	public void visit(Attribute arg) throws SqlWrapperException {
		Trace.enter(this,"visit(Attribute arg)", arg);
		Debug.assertTrue(false, "exceptional case:" + arg.toString());
		Trace.exit(this,"visit(Attribute arg)", arg);
	}

	public void visit(AttributeExpression arg) throws SqlWrapperException {
		Trace.enter(this,"visit(AttributeExpression arg)", arg);

		Expression replacer = (Expression)_map.get(arg);
		if (null != replacer) {
			arg.getFather().replaceChild(arg, replacer);
			Trace.message(this,"visit(AttributeExpression arg)",
				arg.pprint()+" replaced by " + replacer.pprint());
		}
		else {
			Object tableInstance = _map.get(arg.getTableInstance());
			if (null == tableInstance) {
			}
			else if (tableInstance instanceof NullPointer) {
				arg.setTableInstance(null);
			}
			else {
				arg.setTableInstance((RenameRelation)tableInstance);
			}
		}


		replacer = (Expression)_map.get(arg.getUnderlyinExpr());
		if (null != replacer) {
			arg.getFather().replaceChild(arg, replacer);
			Trace.message(this,"visit(AttributeExpression arg)",
				arg.pprint()+" replaced by " + replacer.pprint());
		}

		Trace.exit(this,"visit(AttributeExpression arg)", arg);
	}

//    public void visit(BinaryOperator arg) throws SqlWrapperException {
//        Trace.enter(this,"visit(BinaryOperator arg)", arg);
//        Trace.exit(this,"visit(BinaryOperator arg)", arg);
//    }

	public void visit(BinaryAtomicOp arg) throws SqlWrapperException {
		Trace.enter(this,"visit(BinaryAtomicOp arg)", arg);
		arg.getLeftOperand().accept(this);
		arg.getRightOperand().accept(this);
		Trace.exit(this,"visit(BinaryAtomicOp arg)", arg);
	}

	public void visit(BinaryAlgebra arg) throws SqlWrapperException {
		Trace.enter(this,"visit(BinaryAlgebra arg)", arg);
		/* implemention for Merg, Union, Difference and except, but not for BinOpOuterJoin */
		arg.getLeftOperand().accept(this);
		arg.getRightOperand().accept(this);
		Trace.exit(this,"visit(BinaryAlgebra arg)", arg);
	}


	public void visit(BinOpOuterJoin arg) throws SqlWrapperException {
		Trace.enter(this,"visit(BinOpOuterJoin arg)", arg);

		List list = arg.getPredicateList();
		Debug.assertTrue(null!=list,"null!=list");
		Expression expr = null ;
		for (int i = 0; i < list.size(); i++) {
			expr = (Expression)list.get(i);
			expr.accept(this);
		}

		arg.getLeftOperand().accept(this);
		arg.getRightOperand().accept(this);

		Trace.exit(this,"visit(BinOpOuterJoin arg)", arg);
	}

	public void visit(FunAggregate arg) throws SqlWrapperException {
		Trace.enter(this,"visit(FunAggregate arg)", arg);
		arg.getOperand().accept(this);
		Trace.exit(this,"visit(FunAggregate arg)", arg);
	}

//    public void visit(IfThenElse arg) throws SqlWrapperException {
//        Trace.enter(this,"visit(IfThenElse arg)", arg);
//        Trace.exit(this,"visit(IfThenElse arg)", arg);
//    }

	public void visit(Join arg) throws SqlWrapperException {
		Trace.enter(this,"visit(Join arg)", arg);

		List list = arg.getPredicateList();
		Debug.assertTrue(null!=list,"null!=list");
		Expression expr = null ;
		for (int i = 0; i < list.size(); i++) {
			expr = (Expression)list.get(i);
			expr.accept(this);
		}

		list = arg.getOperands();
		Debug.assertTrue(null!=list,"null!=list");
		expr = null ;
		for (int i = 0; i < list.size(); i++) {
			expr = (Expression)list.get(i);
			expr.accept(this);
		}

		Trace.exit(this,"visit(Join arg)", arg);
	}

	public void visit(Literal arg) throws SqlWrapperException {
		Trace.enter(this,"visit(Literal arg)", arg);
		Trace.exit(this,"visit(Literal arg)", arg);
	}

	public void visit(RenameRelation arg) throws SqlWrapperException {
		Trace.enter(this,"visit(RenameRelation arg)", arg);
		arg.getOperand().accept(this);
		Trace.exit(this,"visit(RenameRelation arg)", arg);
	}

	public void visit(Table arg) throws SqlWrapperException {
		Trace.enter(this,"visit(Table arg)", arg);
		Trace.exit(this,"visit(Table arg)", arg);
	}

//    public void visit(UnaryOperator arg) throws SqlWrapperException {
//        Trace.enter(this,"visit(UnaryOperator arg)", arg);
//        arg.getOperand().accept(this);
//        Trace.exit(this,"visit(UnaryOperator arg)", arg);
//    }

//    public void visit(UnaryAlgebra arg) throws SqlWrapperException {
//        Trace.enter(this,"visit(UnaryAlgebra arg)", arg);
//        Trace.exit(this,"visit(UnaryAlgebra arg)", arg);
//    }

	public void visit(UnaryAtomicOp arg) throws SqlWrapperException {
		Trace.enter(this,"visit(UnaryAtomicOp arg)", arg);
		arg.getOperand().accept(this);
		Trace.exit(this,"visit(UnaryAtomicOp arg)", arg);
	}

//    public void visit(UnOpAggregate arg) throws SqlWrapperException {
//        Trace.enter(this,"visit(UnaryAtomicOp arg)", arg);
//        Trace.exit(this,"visit(UnaryAtomicOp arg)", arg);
//    }

//    public void visit(UnOpExists arg) throws SqlWrapperException {
//        Trace.enter(this,"visit(UnOpExists arg)", arg);
//        arg.getOperand().accept(this);
//        Trace.exit(this,"visit(UnOpExists arg)", arg);
//    }

//    public void visit(UnOpGroup arg) throws SqlWrapperException {
//        Trace.enter(this,"visit(UnOpGroup arg)", arg);
//        Trace.exit(this,"visit(UnOpGroup arg)", arg);
//    }

//    public void visit(UnOpMinus arg) throws SqlWrapperException {
//        Trace.enter(this,"visit(UnOpMinus arg)", arg);
//        Trace.exit(this,"visit(UnOpMinus arg)", arg);
//    }

//    public void visit(UnOpNot arg) throws SqlWrapperException {
//        Trace.enter(this,"visit(UnOpNot arg)", arg);
//        Trace.exit(this,"visit(UnOpNot arg)", arg);
//    }

	public void visit(UnOpProject arg) throws SqlWrapperException {
		Trace.enter(this,"visit(UnOpProject arg)", arg);

		List list = arg.getItemList();
		Debug.assertTrue(null!=list,"null!=list");
		Expression expr = null ;
		for (int i = 0; i < list.size(); i++) {
			expr = (Expression)list.get(i);
			expr.accept(this);
		}

		arg.getOperand().accept(this);

		Trace.exit(this,"visit(UnOpProject arg)", arg);
	}

	public void visit(UnOpRestrict arg) throws SqlWrapperException {
		Trace.enter(this,"visit(UnOpRestrict arg)", arg);
		List list = arg.getPredicateList();
		Debug.assertTrue(null!=list,"null!=list");
		Expression expr = null ;
		for (int i = 0; i < list.size(); i++) {
			expr = (Expression)list.get(i);
			expr.accept(this);
		}

		arg.getOperand().accept(this);

		Trace.exit(this,"visit(UnOpRestrict arg)", arg);
	}

	public void visit(UnOpSort arg) throws SqlWrapperException {
		Trace.enter(this,"visit(UnOpSort arg)", arg);

		List list = arg.getSortSpecificationList();
		Debug.assertTrue(null!=list,"null!=list");
		SortSpecification spec = null;
		Expression expr = null ;
		for (int i = 0; i < list.size(); i++) {
			spec = (SortSpecification)list.get(i);
			expr = spec.getSortExpression();
			expr.accept(this);
		}

		arg.getOperand().accept(this);

		Trace.exit(this,"visit(UnOpSort arg)", arg);
	}

	public void visit(AfCast arg) throws SqlWrapperException {
		Trace.enter(this,"visit(AfCast arg)", arg);
		arg.getExpression().accept(this);
		Trace.exit(this,"visit(AfCast arg)", arg);
	}

	public void visit(XfCast arg) throws SqlWrapperException {
		Trace.enter(this,"visit(XfCast arg)", arg);
		arg.getExpression().accept(this);
		Trace.exit(this,"visit(XfCast arg)", arg);
	}

	public void visit(XFunction arg) throws SqlWrapperException {
		Trace.enter(this,"visit(XFunction arg)", arg);

		if (arg.getArgumentNumber()> 0) {
			List argList = arg.getArguments();
			Object argument = null;
			for (int i = 0; i < argList.size(); i++) {
				argument = argList.get(i);
				if (argument instanceof Expression) {
					((Expression)argument).accept(this);
				}
			}
		}

		Trace.exit(this,"visit(XFunction arg)", arg);
	}

	public String printMap() {
		Trace.enter(this, "pprint()");

		String newLine = System.getProperty("line.separator");
		StringBuffer retVal = new StringBuffer();
		retVal.append(newLine);
		retVal.append("<================ RemoveProjectReplaceVisitor ===========>");
		retVal.append(newLine);
		Object keyList[] = _map.keySet().toArray();
		Expression key = null;
		Object value = null;

		for (int i = 0; i < keyList.length; i++) {
			key = (Expression)keyList[i];
			retVal.append(Integer.toString(i));
			retVal.append("\t");

			retVal.append(key.pprint());
			retVal.append("\t");

			value = _map.get(key);
			if (value instanceof Expression) {
				retVal.append(((Expression)value).pprint());
			}
			else {
				retVal.append(value.toString());
			}
			retVal.append(newLine);
		}

		retVal.append("</================ RemoveProjectReplaceVisitor ===========>");

		Trace.exit(this, "pprint()");
		return retVal.toString();
	}

}
