/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */

package org.xquark.extractor.algebra;

import java.util.Map;

import org.xquark.extractor.common.SqlWrapperException;
import org.xquark.extractor.common.Trace;
import org.xquark.extractor.sql.SqlExpression;

public final class RenameItem extends UnaryAtomicOp
{
    private static final String RCSRevision = "$Revision: 1.3 $";
    private static final String RCSName     = "$Name:  $";

    protected String _name;

    /**
    @param Expr
    @param name
    @roseuid 3BD17C510284
     */
    public RenameItem(Expression Expr, String name)
    {
        super (Expr);
        setName (name);
    }

//    synchronized Object clone(Map clonedExprs) throws CloneNotSupportedException
//    {
//        Trace.enter(this, "clone(Map clonedExprs)");
//        RenameItem retVal = (RenameItem)super.clone();
//        retVal.setName((getName()== null) ? null : new String(getName()));
//        clonedExpr.put(this, retVal);Trace.exit(this, "clone(Map clonedExprs)");return retVal;
//        return  retVal;
//    }

    synchronized Object clone(Map clonedExprs) throws CloneNotSupportedException
    {
        Trace.enter(this, "clone(Map clonedExprs)");

        RenameItem retVal = (RenameItem)super.clone(clonedExprs);
        retVal.setName(getName());

        clonedExprs.put(this, retVal);
        Trace.exit(this, "clone(Map clonedExprs)");
        return retVal;
    }

    /**
    Access method for the _name property.

    @return   the current value of the _name property
     */
    public String getName()
    {
        return _name;
    }

    /**
    Sets the value of the _name property.

    @param aName the new value of the _name property
     */
    public void setName(String aName)
    {
        _name = aName;
    }

    public boolean replaceChild(Expression oldChild, Expression newChild)
    {
        Trace.enter(this, "replaceChild(Expression oldChild, Expression newChild)");

        boolean retVal = false;
        if (getOperand() == oldChild) {
            setOperand(newChild);
            retVal = true;
        }

        Trace.exit(this, "replaceChild(Expression oldChild, Expression newChild)");
        return retVal;
    }

    public String pprint ()
    {
        return "renameItem(" + getOperand().pprint() + " AS " + getName() + ")";
    }

    public SqlExpression accept (GenSqlVisitor visitor) throws SqlWrapperException
    {
        return visitor.visit(this);
    }

    public void accept (AlgebraVisitor visitor) throws SqlWrapperException
    {
        visitor.visit(this);
    }

	// NOTE; Expression#deepEquals(Object) not overridden cause alias names should
	// not be considered for expression equality
}
