/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */

package org.xquark.extractor.algebra;

import java.util.Map;

import org.xquark.extractor.common.SqlWrapperException;
import org.xquark.extractor.common.Trace;
import org.xquark.extractor.sql.SqlExpression;

public final class SortSpecification  extends Expression
{

    private static final String RCSRevision = "$Revision: 1.3 $";
    private static final String RCSName     = "$Name:  $";

    private Expression _sortExpression;
    private int _sortDirection;

    /**
     * @param sortExpression
     * @param sortDirection
     * @roseuid 3AB8681D0255
     */
    public SortSpecification(Expression sortExpression, int sortDirection)
    {
        setSortExpression ( sortExpression );
        _sortDirection = sortDirection ;
    }

//    synchronized Object clone(Map clonedExprs) throws CloneNotSupportedException
//    {
//        Trace.enter(this, "clone(Map clonedExprs)");
//        SortSpecification retVal = (SortSpecification)super.clone();
//        retVal.setSortExpression((getSortExpression() == null) ? null : (Expression)getSortExpression().clone ());
//        clonedExpr.put(this, retVal);Trace.exit(this, "clone(Map clonedExprs)");return retVal;
//        return  retVal;
//    }

    synchronized Object clone(Map clonedExprs) throws CloneNotSupportedException
    {
        Trace.enter(this, "clone(Map clonedExprs)");

        SortSpecification retVal = (SortSpecification)super.clone(clonedExprs);
        retVal.setSortExpression((getSortExpression() == null) ? null : (Expression)getSortExpression().clone (clonedExprs));
        clonedExprs.put(this, retVal);
        Trace.exit(this, "clone(Map clonedExprs)");
        return retVal;
    }

    /**
     * Access method for the _sortExpression property.
     *
     * @return   the current value of the _sortExpression property
     */
    public Expression getSortExpression()
    {
        return _sortExpression ;
    }

    public java.util.List getOperands() {
        return null;
    }

    /**
     * Sets the value of the _sortExpression property.
     *
     * @param aSortExpression the new value of the _sortExpression property
     */
    public void setSortExpression(Expression aSortExpression)
    {
        //sortExpression = aSortExpression;
        _sortExpression = aSortExpression;
        _sortExpression.setFather( this );
    }

    /**
     * @return int
     * @roseuid 3AB8687203DD
     */
    public int getSortDirection()
    {
        return _sortDirection ;
    }

    /**
     * @param sortDirection
     * @roseuid 3B27597200D3
     */
    public void setSortDirection(int sortDirection)
    {
        _sortDirection = sortDirection ;
    }

    public boolean replaceChild(Expression oldChild, Expression newChild)
    {
        Trace.enter(this, "replaceChild(Expression oldChild, Expression newChild)");

        boolean retVal = false;
        if (getSortExpression() == oldChild) {
            setSortExpression(newChild);
            retVal = true;
        }

        Trace.exit(this, "replaceChild(Expression oldChild, Expression newChild)");
        return retVal;
    }

	public String pprint() {
		String retVal = null;
		if (getSortDirection() == ASCENDING_ORDER) {
			return  getSortExpression().pprint() + " ASC";
		}
		else {
			return  getSortExpression().pprint() + " DESC";
		}
	}

    public SqlExpression accept (GenSqlVisitor visitor) throws SqlWrapperException
    {
        return visitor.visit(this);
    }

    public void accept (AlgebraVisitor visitor) throws SqlWrapperException
    {
        visitor.visit(this);
    }

	/**
	 * @see Expression#deepEquals(Object)
	 */
	public boolean deepEquals(Object o)
	{
		if (o instanceof SortSpecification)
		{
			SortSpecification cast = (SortSpecification)o;
			return	 _sortDirection == cast.getSortDirection()
					&& _sortExpression.deepEquals(cast.getSortExpression());
		}
		return false;
	}
}

