/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */

package org.xquark.extractor.algebra;

import java.sql.Types;

import org.xquark.extractor.common.Debug;
import org.xquark.extractor.common.Trace;
import org.xquark.extractor.metadata.DataType;
import org.xquark.extractor.metadata.DbPrimitiveType;


public final class SqlTypeAtom extends SqlType
{
   private static final String RCSRevision = "$Revision: 1.3 $";
	private static final String RCSName     = "$Name:  $";

	private DataType _type = null;
	private String  _name = null;
	private String _relationName = null;

	/**
	@param type
	@param name
	@roseuid 3BD145850305
	 */

	public SqlTypeAtom(DataType type, String name)
	{
		setType(type);
		setName(name);
	}

	public SqlTypeAtom(DataType type, String name, int multiplicity )
	{
		setType(type);
		setName(name);
		setMultiplicity(multiplicity);
	}

	public synchronized Object clone()
	{
		Trace.enter(this, "clone()");
		try {
			SqlTypeAtom retVal = (SqlTypeAtom)super.clone();
			retVal.setName((getName() == null) ? null : new String(getName()) );
			retVal.setType(getType() ==null ? null : (DataType)getType().clone());
			Trace.exit(this, "clone()");
			return  retVal;
		}
		catch (Exception ex) {
			Debug.assertTrue(false, "error in implementation of clone");
			Trace.exit(this, "clone()");
			return null;
		}
	}

	/**
	Access method for the _type property.

	@return   the current value of the _type property
	 */
	public DataType getType()
	{
		return _type;
	}

	/**
	Sets the value of the _type property.

	@param aType the new value of the _type property
	 */
	public void setType(DataType aType)
	{
		_type = aType;
	}

	/**
	Access method for the _name property.

	@return   the current value of the _name property
	 */
	public String getName()
	{
		return _name;
	}

	/**
	Sets the value of the _name property.

	@param aName the new value of the _name property
	 */
	public void setName(String aName)
	{
		_name = aName;
	}

	public void setRelationName(String relationName)
	{
		_relationName = relationName;
	}

	public String getRelationName()
	{
		return _relationName;
	}

	public boolean isRelation()
	{
		return false;
	}

	public boolean isTuple()
	{
		return false;
	}

	public boolean isColumn()
	{
		return MANY == _multiplicity;
	}

	public boolean isAtom()
	{
		return ONE == _multiplicity;
	}



	public  boolean isBoolean()
	{
		int typeCode = _type.getPrimitiveTypeCode();
		return Types.BIT == typeCode;
	};

	public  boolean isNumeric()
	{
		return  _type.isNumeric();
	};

	public  boolean isInteger()
	{
		return  _type.isInteger();
	};

	public  boolean isString()
	{
		return  _type.isString();
	};

	public  boolean isDataTime()
	{
		return _type.isDataTime();
	}

	public  boolean isNull()
	{
		return _type.isNull();
	}

	public boolean isOracleRowid()
	{
		return _type.getPrimitiveType().getCode() == DbPrimitiveType.ORACLE_ROWID;
	}

	public String pprint ()
	{
		String retVal = null;
		retVal = MANY == _multiplicity ? "Column" : "Value";
		retVal += "(" + ppprint()+")";

		return retVal;
	}

	String ppprint()
	{
		return _name + ":" + _type.getName();
	}

	public boolean isCompatibleTo(SqlType type)
	{
		boolean retVal;
		if ( type instanceof SqlTypeAtom ) {
			SqlTypeAtom tmpType = (SqlTypeAtom)type;
			retVal =  _type.isCompatibleTo(tmpType.getType());
		}
		else {
			retVal = false;
		}
		return retVal;
	}
}
