/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */

package org.xquark.extractor.algebra;

import java.util.*;

import org.xquark.extractor.common.Debug;
import org.xquark.extractor.common.SqlWrapperException;
import org.xquark.extractor.common.Trace;
import org.xquark.extractor.sql.SqlExpression;
import org.xquark.extractor.xfunctions.AfTrim;

public class UnOpProject extends UnaryAlgebra
{
    private static final String RCSRevision = "$Revision: 1.3 $";
	private static final String RCSName     = "$Name:  $";

	private static final String ATTRIBUTE_ALIAS_PREFIX = "a";

	protected boolean _distinct = false;
	protected List _itemList = null;
	protected List _itemNames = null;
	protected UnOpSort _associatedSort = null;

	static private int  _counter = 0;     /*a counter for generate unique name*/


	public UnOpProject()  {
	}

	public UnOpProject(Expression operand) {
		super(  operand  );
	}
	/**
	@param operand
	@param item
	@roseuid 3B2787EF03CB
	 */
	public UnOpProject(Expression operand, Expression item)
	{
		super(  operand  );
		addItem ( item ) ;
	}

	/**
	@param operand
	@param itemList
	@param distinct
	@roseuid 3B2787EF01F5
	 */
	public UnOpProject(Expression operand, List itemList, boolean distinct)
	{
		super ( operand );
		setItemList  ( itemList ) ;
		setDistinct ( distinct )  ;
	}

//    synchronized Object clone(Map clonedExprs) throws CloneNotSupportedException
//    {
//        Trace.enter(this, "clone(Map clonedExprs)");
//        UnOpProject retVal = (UnOpProject)super.clone();
//        retVal.setItemList(AlgebraTools.clone(_itemList));
//        clonedExpr.put(this, retVal);Trace.exit(this, "clone(Map clonedExprs)");return retVal;
//        return  retVal;
//    }

	synchronized Object clone(Map clonedExprs) throws CloneNotSupportedException
	{
		Trace.enter(this, "clone(Map clonedExprs)");
		UnOpProject retVal = (UnOpProject)super.clone(clonedExprs);
		if (null != _itemList) {
			List itemList =  AlgebraTools.clone(_itemList, clonedExprs);
			List itemNames =  AlgebraTools.clone(_itemNames);
			Expression item = null;
			for (int i = 0; i < itemList.size(); i++) {
				item = (Expression)itemList.get(i);
				item.setFather(retVal);
			}
			retVal._itemList = itemList;
			retVal._itemNames = itemNames;
		}

		clonedExprs.put(this, retVal);
		Trace.exit(this, "clone(Map clonedExprs)");
		return retVal;
	}

	/**
	Determines if the _distinct property is true.

	@return   <code>true<code> if the _distinct property is true
	 */
	public boolean getDistinct()
	{
		return _distinct ;
	}

	/**
	Sets the value of the _distinct property.

	@param aDistinct the new value of the _distinct property
	 */
	public void setDistinct(boolean aDistinct)
	{
		_distinct = aDistinct;
	}

	/**
	Access method for the _itemList property.

	@return   the current value of the _itemList property
	 */
	public List getItemList()
	{
		return _itemList ;
	}

//    public List getParameterList()
//    {
//        return getItemList();
//    }

	public void setAssociatedSort(UnOpSort associatedSort)
	{
		_associatedSort = associatedSort;
	}


	public UnOpSort getAssociatedSort()
	{
		return _associatedSort;
	}

	/**
	Sets the value of the _itemList property.

	@param aItemList the new value of the _itemList property
	 */
	public void setItemList(List itemList)
	{
		Trace.enter(this,"setItemList");

		_itemList = null;
		_itemNames = null;

		if (null==itemList) {
			return ;
		}
		addItemList(itemList);

		Trace.exit(this,"setItemList");
	}

	public void addItemList (List itemList)
	{
		Trace.enter(this,"addItemList");

		if (null==itemList) {
			return ;
		}

		Iterator iter = itemList.iterator();
		while (iter.hasNext()) {
			Expression item = (Expression)iter.next();
			addItem (item);
		}

		Trace.exit(this,"addItemList");
	}

	/**
	@param itemExpr
	@roseuid 3B2787F00323
	 */
	public Expression addItem(Expression itemExpr )
	{
		if (null!=getItemList()) {
			return insertItem (getItemList().size(),itemExpr);
		}
		else {
			return insertItem (0, itemExpr);
		}
	}

	public Expression setItem( int index, Expression itemExpr)
	{
		Trace.enter(this,"setItem( int index, Expression itemExpr)");
		Expression  retVal = null;
		_itemList.remove(index);
		_itemNames.remove(index);
		retVal = insertItem(index , itemExpr);

		Trace.exit(this,"setItem( int index, Expression itemExpr)");
		return retVal;
	}

	public Expression insertItem( int index, Expression itemExpr)
	{
		Trace.enter(this,"insertItem( int index, Expression itemExpr)");

		Expression retVal = null;

		if ( null == _itemList ) {
			_itemList = new ArrayList ();
			_itemNames = new ArrayList ();
		}


		if (itemExpr instanceof Rowid ) {
			itemExpr = new RenameItem(itemExpr,"OracleRowid");
		}

		if (itemExpr instanceof TupleExpression ) {
			addItemTupleExpression ((TupleExpression)itemExpr);
		}
		else if (itemExpr instanceof Relation ) {
			String name = getUniqueName(ATTRIBUTE_ALIAS_PREFIX);
			retVal = new RenameItem(itemExpr,name);
			retVal.setOrginalXExpr(itemExpr.getOrginalXExpr());
			_itemList.add(index,retVal);
			_itemNames.add(index,name);
		}
		else {
			String name = itemExpr.getName();
			if (null == name) {
				// when the new added item does not have a name, give it a anonymity
				name = getUniqueName(ATTRIBUTE_ALIAS_PREFIX);
				retVal = new RenameItem ( itemExpr, name);
				retVal.setOrginalXExpr(itemExpr.getOrginalXExpr());
				_itemNames.add(index,name);
				_itemList.add(index,retVal) ;
			}
			else {
				// when the new added item havs a name, check if it falls in conflict with the existing names
				if (!hasNameConflict(itemExpr)) {
					//no nameing conflict
					_itemNames.add(index,name);
					_itemList.add (index, itemExpr ) ;
					retVal = itemExpr;
				}
				else
				{
					//when there is a nameing conflict, rename the new added item
					_itemNames.add(index,name);
					name = getUniqueName(name);
					if (itemExpr instanceof RenameItem) {
						((RenameItem)itemExpr).setName(name);
						retVal = itemExpr;
					}
					else {
						retVal = new RenameItem ( itemExpr, name);
						retVal.setOrginalXExpr(itemExpr.getOrginalXExpr());
					}
					_itemList.add (index,retVal);
				}
			}
		}

		if (null != retVal) {
			retVal.setFather(this);
		}

		Trace.exit(this,"insertItem( int index, Expression itemExpr)");
		return retVal;
	}

	public void insertItemWithoutRename(int index, FunAggregate funAggregate) {
		Trace.enter(this,"insertItemWithoutRename(int index, FunAggregate funAggregate)");

		insertItem( index, funAggregate);
		/** @todo  */
//        if (null == _itemList) {
//            _itemList = new ArrayList ();
//        }
//
//        if (null == _itemNames) {
//            _itemNames = new ArrayList ();
//        }
//
//        _itemNames.add(index, null);/** @todo  for sybase ?? */
//        _itemList.add (index, funAggregate);

		Trace.exit(this,"insertItemWithoutRename(int index, FunAggregate funAggregate)");
	}



	private boolean hasNameConflict(Expression itemExpr)
	{
		Trace.enter(this, "hasNameConflict(Expression itemExpr)");
		boolean retVal = false;

		String newName = itemExpr.getName();
		if (null != _itemList && null!= itemExpr.getName()) {
			Expression item  = null;
			for (int i = 0; i < _itemList.size(); i++) {
				item = (Expression) _itemList.get(i);


				if (item.getName().equalsIgnoreCase(itemExpr.getName())) {
					retVal = true;
					break;
				}
//				String itemName = item.getName();
//
//				if (null != itemName &&
//					null != newName &&
//					itemName.equalsIgnoreCase(itemName)){
//                    retVal = true;
//                    break;
//				}
//				else if (null == itemName &&
//						null == newName ) {
//	               retVal = true;
//                    break;
//				}
			}
		}
		Trace.exit(this, "hasNameConflict(Expression itemExpr)");
		return retVal;
	}

	public boolean removeItem(Expression itemExpr)
	{

		Trace.enter(this,"removeItem");
		boolean retVal = false;

		int index = _itemList.indexOf(itemExpr);
		if (-1 != index){
			_itemList.remove(index);
			_itemNames.remove(index);
			retVal = true;
		}

		Trace.exit(this,"removeItem");
		return retVal;
	}

	private List addItemTupleExpression (TupleExpression tupleExpr)
	{
		Trace.enter(this,"addItemTupleExpression");

		List attrList = tupleExpr.getItemList();
		List retVal   = new ArrayList();

		Iterator iter = attrList.iterator();
		Expression item = null;
		while (iter.hasNext()) {
			item = (Expression)iter.next();
			retVal.add(addItem(item));
		}
		tupleExpr.setItemList(retVal);
		/* after above line , tupleExpr is the father of all these items
		* so ...*/
		for (int i = 0; i < retVal.size(); i++) {
			item = (Expression)retVal.get(i);
			item.setFather(this);
		}

		Trace.exit(this,"addItemTupleExpression");
		return retVal;
	}

	public List getKeys () {
		return _keys;
	}

	public void setKeys (List keys) {
		_keys = keys ;
	}

	public Expression SearchItem (Expression expr)
	{
		Trace.enter(this,"ArithmeticOperation");
		Expression retVal = null;
		Expression item ;
		for (int i = 0; i < _itemList.size(); i++) {
			item = (Expression)_itemList.get(i);
			if (item.equals(expr)) {
				retVal = item ;
				break;
			}
			else {
				item = getRealExpression(item);
				if (item.equals(expr)) {
					retVal = item ;
					break;
				}
			}
		}

		Trace.exit(this,"ArithmeticOperation");
		return retVal;
	}

	private Expression getRealExpression (Expression expr)
	{
		if (expr instanceof RenameItem) {
			return ((RenameItem)expr).getOperand();
		}
		else {
			return expr;
		}
	}

	public void arithmeticOperation(int operater,Expression leftOperand,Expression rightOperand)
	{
		Trace.enter(this,"ArithmeticOperation");

		Iterator iter = _itemList.iterator();
		Expression item ;
		List newItemList = new ArrayList();
		Expression  arOp ;
		while (iter.hasNext()) {
			item = getRealExpression((Expression)iter.next());
			if (null != leftOperand) {
				arOp = new BinOpArithmetic (operater, leftOperand, item);
			}
			else {
				arOp = new BinOpArithmetic (operater, item, rightOperand);
			}
			newItemList.add (arOp);
		}
		setItemList(newItemList);
		Trace.exit (this,"ArithmeticOperation");
	}

	private String getUniqueName(String name)
	{
		Trace.enter(this, "getUniqueName(String name)");
		String retVal = null;
		if (name.length()>20) {
			retVal = name.substring(0,19) + "_" + Integer.toString(_counter++);	// TODO: check that counter is reset at evey QDB execution for alias. See for temptable name.
		}
		else {
			retVal = name + Integer.toString(_counter++);;
		}

		Trace.exit(this, "getUniqueName(String name)");
		return retVal;
	}

//    /**
//    @param predicate
//    @param pTISet
//    @roseuid 3B2787F1036A
//     */
//    private void traitColumnExpression(Expression predicate, Set pTISet)
//    {
//        Trace.enter( this , "traitColumnExpression", predicate.pprint() );
//        boolean done = false ;
//        if ( predicate instanceof ColumnExpression )
//        {
//            ColumnExpression ce = (ColumnExpression) predicate ;
//
//            if ( pTISet.contains( ce.getTableInstance()) )
//            {
//                // trait the column expression belong to underlying table instance
//                Item    item ;
//                Expression      tmpExpr ;
//                ColumnExpression tmpCe ;
//
//                Iterator iter = _itemList.iterator();
//                while (iter.hasNext())
//                {
//                    item = (Item)iter.next();
//                    tmpExpr  = item.getExpression() ;
//                    if  ( tmpExpr instanceof ColumnExpression )
//                    {
//                        tmpCe = (ColumnExpression) tmpExpr ;
//                        if ( ce.equalsTo(tmpCe)  )
//                        {
//                            // correspondent Item found
//                            done = true ;
//                            if ( null != item.getName() )
//                            {
//                                ce.setColumn( item.getName());
//                            }
//                        }
//                    }
//                }
//                if ( ! done )   // // correspondent Item not found , add a item to projection
//                {
//                    ColumnExpression newCe = new ColumnExpression ( ce.getTableInstance() , ce.getColumn() ) ;
//                    _itemList.add( new Item (newCe, null )  ) ;
//                }
//            }
//        }
//        else
//        {
//            List   operands = predicate.getOperands() ;
//            if ( null != operands )
//            {
//                Iterator iter = operands.iterator();
//                Expression item ;
//                while (iter.hasNext())
//                {
//                    item = (Expression) iter.next();
//                    traitColumnExpression ( item , pTISet ) ;
//                }
//            }
//        Trace.exit( this , "traitColumnExpression");
//        }
//    }

	public void addNameTest (String name)
	{
		Trace.enter( this , "addNameTest");
		/** @todo!!!  */
		List itemList = ((Relation)_operand).nameTest(name);
		Expression item = null;

		AttributeExpression attrExpr = null;
		for (int i = 0; i < itemList.size(); i++) {
			item = (Expression)itemList.get(0);
			/* remove heading and tailing spaces */
			if (item instanceof Attribute ) {
				if ( ((Attribute)item).getPrimitiveType().isString() ) {
					item = new AfTrim(item);
				}
			}
			addItem(item);
		}
		Trace.exit( this , "addNameTest");
	}

	public List nameTest (String name)
	{
		Trace.enter(this,"nameTest", name);

		List retVal = null;

		if (name.equals("*")) {
			retVal = _itemList;
			// in this case , there will be a  problem when there are duplicated names
			Trace.warn(this,"nameTest","wild card used in nameTest");
		}
		else if (null == name) {
			retVal = new ArrayList ();
			for (int i = 0; i < _itemNames.size(); i++) {
				if (null == _itemNames.get(i)) {
					retVal.add (_itemList.get(i));
				}
			}
			if (0==retVal.size()) {
				retVal = null;
			}

		}
		else {
			// Name is not a wild card
			retVal = new ArrayList ();
			for (int i = 0; i < _itemNames.size(); i++) {
				if (name.equalsIgnoreCase((String)_itemNames.get(i))) {
					retVal.add (_itemList.get(i));
				}
			}
			if (0==retVal.size()) {
				retVal = null;
			}
		}

		Trace.exit (this,"nameTest",name);
		return retVal;
	}


//    public void removeItemName ()
//    {
//    /**
//     * this function is to be called in :
//     * case 3 eg. Collection("SQLWrapper1:items")/items/description/data()
//     * in this case there is only one item in UnOpProject
//     */
//        Trace.enter (this , "removeItemName");
//Debug.assertTrue(false,"this function seems to be inutile");
//
//        Debug.assertTrue(null!= _itemList,"null!= _itemList");
//        Expression  expr = (Expression)_itemList.get(0);
//        expr = new RenameItem (expr , getUniqueName("anonymity"));
//        _itemList.set(0,expr);
//
//        /** @todo the XQuery name for the item is to bu updated */
//        Trace.exit(this , "removeItemName");
//    }



	/**
	@param predicate
	@return boolean
	@roseuid 3B2787F200E1
	 */
	public boolean predicateUp(Expression predicate)
	{
		Trace.enter( this , "predicateUp");
		boolean retVal = false ;
//        if ( getDistinct () )
//        {
//            retVal = false ;
//        }
//        else
//        {
//            /**  there are two case to be traited before UnOpAggregete pass the
//             *   predicate  to his father :
//             *   1.the column expression referred in the predicate might be given
//             *   a new name . the name of the column expression should be replaced
//             *   2.if the column expression is not in itemList , it should be added
//             */
//            Set pTISet = providedTableInstances () ;
//            pTISet.add ( null ) ;
//            traitColumnExpression ( predicate , pTISet ) ;
//            retVal = super.predicateUp( predicate ) ;
//        }
//        Trace.exit ( this , "predicateUp");
		return retVal ;
	}

	public Mapper getMapper()
	{
		/* default implementation*/
		return _mapper;
	}

	public boolean replaceChild(Expression oldChild, Expression newChild)
	{
		Trace.enter(this, "replaceChild(Expression oldChild, Expression newChild)");
		boolean retVal = false;

		List list = getItemList();

		Expression item = null ;
		String itemName = null;
		RenameItem  renameItem = null;

		for (int i = 0; i < list.size(); i++) {
			item = (Expression)list.get(i);
			if (item.equals(oldChild)) {
				itemName = item.getName();
				Debug.assertTrue(null != itemName, "null != itemName");
				if (!itemName.equals(newChild.getName())) {
					/* keep the old name of this item so that the mapper can still works*/
					renameItem = new RenameItem(newChild,itemName);
					renameItem.setOrginalXExpr(item.getOrginalXExpr());
					list.set(i, renameItem);
					renameItem.setFather(this);
				}
				else {
					list.set(i, newChild);
					newChild.setFather(this);
				}
				retVal = true;
				break;
			}
		}

		if (getOperand().equals(oldChild)) {
			setOperand(newChild);
			retVal = true;
		}

		Trace.exit(this, "replaceChild(Expression oldChild, Expression newChild)");
		return retVal;
	}

	private Set _providedTableInstances = Collections.singleton(this);

	public Set providedTableInstances()
	{
		return _providedTableInstances;
	}
	
	public AttributeExpression findNonNullAttribute() {
		Iterator it = _itemList.iterator();
		Expression expr = null;
		AttributeExpression ret = null;
		
		while (it.hasNext() && ret == null) {
			expr = (Expression) it.next();
			if (expr instanceof AttributeExpression) {
				ret = (AttributeExpression)expr;
				Attribute att = (Attribute) ret.getUnderlyinExpr();
				if (!att.getAttribute().getNullable()) {
					ret = new AttributeExpression(ret.getTableInstance(), ret.getName());
					ret.setUnderlyingExpr(att);
				}
			}
			else if (expr instanceof RenameItem) {
				ret = new AttributeExpression(null, ((RenameItem)expr).getName());
			}
		}
		return ret;
	}

	public List getParameterList()
	{
		return getItemList();
	}

	static public Expression identicalExpr(List existingExprList, AttributeExpression newExpr){
		Expression retVal = null;
		Expression expr = null;
		for (int i = 0; i < existingExprList.size(); i++) {
			expr = (Expression)existingExprList.get(i);
			if (isIdentical(expr,newExpr)) {
				retVal = expr;
				break;
			}
		}
		return retVal;
	}

	static public boolean isIdentical(Expression existingExpr , AttributeExpression newExpr) {
		boolean retVal = true;
		if (existingExpr instanceof AttributeExpression) {
			 AttributeExpression attrExpr =  (AttributeExpression)existingExpr;
			 retVal = newExpr.getAttribute().equals(attrExpr.getAttribute());
			 if (retVal && null != newExpr.getTableInstance()) {
				 retVal = newExpr.getTableInstance().equals(attrExpr.getTableInstance());
			 }
			 else if (retVal && null == newExpr.getTableInstance()) {
				retVal = (null == attrExpr.getTableInstance());
			 }
		}
		else if (existingExpr instanceof RenameItem) {
			RenameItem renameItem = (RenameItem)existingExpr;
			retVal = isIdentical(renameItem.getOperand(), newExpr);
		}
		else {
			retVal = false;
		}
		return retVal;
	}

	public SqlExpression accept (GenSqlVisitor visitor) throws SqlWrapperException
	{
		return visitor.visit(this);
	}

	public void accept (AlgebraVisitor visitor) throws SqlWrapperException
	{
		visitor.visit(this);
	}

	/**
	 * @see Expression#deepEquals(Object)
	 */
	public boolean deepEquals(Object o)
	{
		if (o instanceof UnOpProject)
		{
			UnOpProject cast = (UnOpProject)o;
			return super.deepEquals(o)
					&& _distinct == cast.getDistinct()
					&& _associatedSort != null
					&& _associatedSort.deepEquals(cast.getAssociatedSort())
					&& AlgebraTools.areExprListEquivalent(_itemList, cast.getItemList()); 
		}
		return false;
	}
}


