/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */

package org.xquark.extractor.common;

import java.io.PrintStream;
import java.util.Date;

// QUESTION : Why trace is not activated with extractor ?
public class Log {

    public static final int MESSAGE = 0;
    public static final int WARN = 1;
    public static final int ERROR = 2;
    public static final int EXCEPTION = 3;

    private static final String RCSRevision = "$Revision: 1.3 $";
    private static final String RCSName = "$Name:  $";

    private static final String[] MESSAGE_ICONS =
        { "[(i)]", "[(?)]", "[(X)]", "[(E)]" };

    private static boolean enabled = false;
    private static boolean printTime = false;
    private static boolean redirected = false;

    public static void outputToStream(java.io.OutputStream out) {
        if (null != out) {
            redirected = true;
            PrintStream printStream = new PrintStream(out);
            System.setErr(printStream);
        }
    }

    public static void enable() {
        enabled = true;
    }

    public static void disable() {
        enabled = true;
    }

    public static void printTime(boolean on) {
        printTime = on;
    }

    private static void print(int msgType, String msg) {
        if (printTime) {
            Date now = new Date(System.currentTimeMillis());
            System.err.print(now.toString());
        }
        System.err.print("  " + MESSAGE_ICONS[msgType] + "  ");
        if (null != msg) {
            System.err.println(msg);
        } else {
            System.err.println();
        }
    }

    public static void message(String msg) {
        if (enabled) {
            print(MESSAGE, msg);
        }
    }

    public static void warn(String msg) {
        if (enabled) {
            print(WARN, msg);
        }
    }

    public static void error(String msg) {
        if (enabled) {
            print(ERROR, msg);
        }
    }

    public static void exception(Exception ex) {
        if (enabled) {
            print(EXCEPTION, ex.toString());
        }
    }

    public static void error(Exception ex) {
        if (enabled) {
            print(ERROR, MessageLibrary.getMessage("IE_EXCP"));
            ex.printStackTrace(System.err);
        }
    }

}
