/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */

package org.xquark.extractor.common;


import java.text.MessageFormat;
import java.util.Locale;
import java.util.ResourceBundle;


/**
 * This class holds a hash table of error messages. The hash key is the
 * error code and the value is the message.
 */
public class MessageLibrary
{

    private static final String RCSRevision = "$Revision: 1.3 $";
    private static final String RCSName     = "$Name:  $";

    private static MessageFormat _formatter = null ;
    private static ResourceBundle _bundle;

    public static void setup (String messageFileName,Locale locale )
    {
        try
        {
            _bundle = ResourceBundle.getBundle(messageFileName,locale);
        }
        catch(Exception ex)
        {
            throw new SqlWrapperException ( "Can not load message library file:"+messageFileName, ex);
        }
        _formatter = new MessageFormat("");
        _formatter.setLocale(locale);

    }

    public static String getMessage(String key)
    {
        if(_bundle==null)
            return "No message library available.";
        try
        {
            return key + ": " + _bundle.getString(key);
        }
        catch(Exception ex)
        {
            return key + ": " + "Message [" + key + "] not found.";
        }
    }

    public static String getMessage(String key, Object [] arguments)
    {
        String retVal = getMessage ( key );

         _formatter.applyPattern(retVal);
        retVal = _formatter.format(arguments);

        return retVal ;
    }

    public static String getMessage(String key, Object argument )
    {
        Object [] args = {argument};
        String retVal = getMessage (key, args);
        return retVal ;
    }

    public static String getMessage(String key, Object argument1, Object argument2)
    {
        Object [] args = {argument1, argument2};
        String retVal = getMessage (key, args);
        return retVal ;
    }

    public static void main ( String [] args) {
        MessageLibrary.setup( "org.xquark.extractor.common.resources.MessageLibrary" , new Locale("en","US"));
        String message ;
        Object [] as = {"=="} ;
        message = MessageLibrary.getMessage( "T_N_SUP_OPTR",as);
        System.out.println(message);

        message = MessageLibrary.getMessage( "T_N_SUP_FUN","contains");
        System.out.println(message);

        message = MessageLibrary.getMessage( "T_N_SUP_FUN","contains","==");
        System.out.println(message);
    }

}
