/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */

package org.xquark.extractor.common;


import java.io.PrintStream;
import java.util.*;


public class Trace {
    private static final String RCSRevision = "$Revision: 1.3 $";
    private static final String RCSName     = "$Name:  $";


    private static  boolean  TRACE_ALL_CLASSES = false ;
    private static  boolean  TRACE_NONE        = true ;

    private static  boolean  OUTPUT_ENTER_EXIT  = true ;
    private static  boolean  OUTPUT_MESSAGE     = true ;
    private static  boolean  OUTPUT_WARN        = true ;
    private static  boolean  OUTPUT_ERROR       = true ;

    private static boolean          redirected = false ;
    private static int              stackDepth = 0 ;
    private static String           spaces =
        new String ("                                            " +
                    "                                            " +
                    "                                            " ) ;

    private  static HashSet traceList = new HashSet();
    private  static Hashtable disabledFunctionList = new Hashtable();

    public static final int ENTER = 0 ;
    public static final int EXIT = 1 ;
    public static final int MESSAGE = 2;
    public static final int WARN = 3;
    public static final int ERROR = 4;
    public static final int LOG = 5;

	public static  int MAX_TRACE_LEVEL = -1;
    private static final String [] MESSAGE_ICONS = {"-->>" , "<<--" , "(i) ","(*) ","(X) ", "() "} ;

    private static  boolean  BRIEF_CLASS_NAME = true ;



	static {
		loadConfig();
	}

	public static void loadConfig() {
		ResourceBundle _bundle = null;
		try {
		    _bundle = ResourceBundle.getBundle("org.xquark.extractor.common.resources.Trace");
		}
		catch (Exception ex) {
			return ;
		}

		Enumeration enum = _bundle.getKeys();
		String propName = null;
		String propVal = null;
		while (enum.hasMoreElements()) {
			propName = (String)enum.nextElement();
			propVal = _bundle.getString(propName);

			if (propName.equals("Trace.TraceAll")) {
				if (propVal.equalsIgnoreCase("on")) enableAll();
			}
			else if (propName.equals("Trace.TraceDepth")) {
				MAX_TRACE_LEVEL = Integer.parseInt(propVal);
			}			
			else if (propName.equals("Trace.TraceNone")) {
				if (propVal.equalsIgnoreCase("on")) disableAll();
			}
			else if (propName.equals("Trace.EnterExit")) {
				if (propVal.equalsIgnoreCase("on")) OUTPUT_ENTER_EXIT = true;
			}
			else if (propName.equals("Trace.Message")) {
				if (propVal.equalsIgnoreCase("on")) OUTPUT_MESSAGE = true;
			}
		    else if (propName.equals("Trace.Warn")) {
				if (propVal.equalsIgnoreCase("on")) OUTPUT_WARN = true;
			}
			else if (propName.equals("Trace.Error")) {
				if (propVal.equalsIgnoreCase("on")) OUTPUT_ERROR = true;
			}
			else {
				if (propVal.equalsIgnoreCase("on")) traceClass(propName);
			}
		}
	}


    public static void enableAll ()
    {
        TRACE_ALL_CLASSES = true;
        TRACE_NONE         = false;

        OUTPUT_ENTER_EXIT = true ;
        OUTPUT_MESSAGE = true ;
        OUTPUT_WARN = true ;
        OUTPUT_ERROR = true ;
    }

    public static void disableAll ()
    {
        TRACE_ALL_CLASSES = false ;
        TRACE_NONE         = true;
    }

    public static void setBriefClassName ( boolean  brief )
    {
            BRIEF_CLASS_NAME = brief ;
    }

    public static void disableFunction (String functionName )
    {
        disabledFunctionList.put(functionName, functionName);
    }

    public static void traceClass (String classname)
    {
        traceList.add(classname);
        TRACE_NONE = false;
    }

    public static void unTraceClass(String classname)
    {
        traceList.remove(classname);
    }

    public static void setOutput (  boolean enter_exit ,
                                    boolean message ,
                                    boolean warn ,
                                    boolean error ) {
        OUTPUT_ENTER_EXIT   = enter_exit;
        OUTPUT_MESSAGE      = message;
        OUTPUT_WARN         = warn;
        OUTPUT_ERROR        = error;
    }

    public static void outputToStream ( java.io.OutputStream out )  {
        if ( null != out )
        {
            redirected = true ;
            PrintStream  printStream = new PrintStream ( out )  ;
            System.setErr( printStream );
        }
    }

    private  static boolean isEnabled ( Object obj ,String funcName , int traceType )   {
        if (TRACE_NONE) {
            return false;
        }

        if (LOG == traceType) {
            return true;
        }

        // check if the calling class is enabled .
        if ( ! TRACE_ALL_CLASSES ) {
            String tmpStr = getClassName( obj );
            if (!traceList.contains( tmpStr ))
                return false ;
        }
        if (  disabledFunctionList.get(funcName) != null )
            return false ;
        // chech if the traceType is enabled
        switch (traceType)
        {
            case ERROR :
                return OUTPUT_ERROR ;
            case WARN :
                return OUTPUT_WARN ;
            case MESSAGE :
                return OUTPUT_MESSAGE ;
            case ENTER :
                return OUTPUT_ENTER_EXIT ;
            case EXIT :
                return OUTPUT_ENTER_EXIT ;
            default :
                return true ;
        }
    }

    public static void enter ( Object caller, String funcName ,Object msg)    {
        if (isEnabled( caller , funcName ,ENTER ))
        {
			stackDepth++;
            print ( ENTER , caller , funcName , msg ) ;
        }
    }

    public static void enter ( Object caller, String funcName  )
    {
        if (isEnabled( caller , funcName ,ENTER )) {
			stackDepth++;
            print ( ENTER , caller , funcName , null ) ;
        }
    }

    private static void print (int msgType , Object caller, String funcName , Object msg )
    {
		if (stackDepth > MAX_TRACE_LEVEL && MAX_TRACE_LEVEL != -1) {
			return ;
		}

        if ( redirected ) {
            Date now = new Date (System.currentTimeMillis() );
            System.err.print ( now.toString () + "  " ) ;
        }
		if (null != caller && null != funcName) {
			 System.err.print( ident ( stackDepth )+MESSAGE_ICONS[msgType]);
			 if (BRIEF_CLASS_NAME) {
				 System.err.print(getShortClassName(caller));
			 }
			 else {
				 getClassName (caller);
			 }
			System.err.print("::"+funcName + "  //  ") ;
		}
		else {
	        System.err.print( MESSAGE_ICONS[msgType]) ;
		}

        if ( null != msg )
        {
            System.err.println(msg.toString());
        }
        else
        {
            System.err.println( );
        }
   }


    public static void exit ( Object caller, String funcName ,Object msg )
    {
        if (isEnabled( caller ,funcName ,EXIT ) ) {
            print ( EXIT , caller , funcName , msg ) ;
            stackDepth--;
        }
    }

    public static void exit ( Object caller, String funcName  )
    {
        if (isEnabled( caller ,funcName ,EXIT ) ) {
            print ( EXIT , caller , funcName , null ) ;
            stackDepth--;
        }
    }

    public  static void message ( Object caller, String funcName ,Object msg )
    {
        if (isEnabled( caller ,funcName ,MESSAGE) ) {
            print ( MESSAGE , caller , funcName , msg ) ;
        }
    }
    public  static void warn ( Object caller, String funcName ,Object msg )
    {
        if (isEnabled( caller ,funcName ,WARN) ) {
            print ( MESSAGE , caller , funcName , msg ) ;
        }
    }

    public  static void error ( Object caller, String funcName ,Object msg )
    {
        if (isEnabled( caller ,funcName ,WARN) ) {
            print ( WARN , caller , funcName , msg ) ;
        }
    }

    public  static void log ( Object caller, String funcName , Object msg )
    {
        if (isEnabled( caller ,funcName ,LOG) ) {
            print ( LOG , caller , funcName , msg ) ;
        }
    }

    public  static void log (Object obj)
    {
        if (isEnabled( null, null, LOG) ) {
            print ( LOG , null , null, obj.toString()) ;
        }
    }


    private static String ident ( int stackDepth ) {
        int ident = stackDepth;
        while (ident>40) {
            ident -= 15;
        }
        while (ident<0) {
            ident += 15;
        }
        return Integer.toString( stackDepth ) + spaces.substring(0, 2* ident )  ;
    }

    private static String getClassName(Object obj) {
        if(obj instanceof String)
            return   (String) obj;
        else
        {
            String  retVal = obj.getClass().getName();
            return retVal ;
        }
    }

	private static String getShortClassName(Object obj)	{
		String  className = getClassName(obj);
		int indexOfPoint = className.lastIndexOf('.');
		return className.substring(indexOfPoint+1);
    }
}
