/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */

package org.xquark.extractor.metadata;

import java.util.Properties;

import org.xquark.extractor.common.Constants;

/**
 * Handle case conversion for database object names used during metadata
 * contruction from JDBC information providing uniform later use.
 */
public class CustomizedEncoder implements QNameEncoder, Constants {

    private static final String RCSRevision = "$Revision: 1.3 $";
    private static final String RCSName = "$Name:  $";

    private Properties _substitutionMap = new Properties();
    private int _nameCase = CASE_MIXED;

    public CustomizedEncoder() {}

    public void addSubstitution(char aChar, String subtituteWith) {
        _substitutionMap.setProperty(String.valueOf(aChar), subtituteWith);
    }

    public void setSubstitutionMap(Properties map) {
        _substitutionMap = map;
    }

    public void setNameCase(int nameCase) {
        _nameCase = nameCase;
    }

    public String encode(String dbObjectName) {

        String retVal = null;
        if (null == _substitutionMap) {
            retVal = dbObjectName;
        } else {
            StringBuffer retBuf = new StringBuffer();

            char[] name = dbObjectName.toCharArray();
            String aChar = null;
            String substitution = null;
            for (int i = 0; i < name.length; i++) {
                aChar = String.valueOf(name[i]);
                substitution = _substitutionMap.getProperty(aChar);
                if (null == substitution) {
                    retBuf.append(aChar);
                } else {
                    retBuf.append(substitution);
                }
            }
            retVal = retBuf.toString();
        }

        if (CASE_MIXED == _nameCase) {
        } else if (CASE_LOWER == _nameCase) {
            retVal = retVal.toLowerCase();
        } else /* (CASE_UPPER == _nameCase) */ {
            retVal = retVal.toUpperCase();
        }

        return retVal;
    }
}
