/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */

package org.xquark.extractor.metadata;

import java.sql.Types;

public class DbPrimitiveType extends DataType
{

	private static final String RCSRevision = "$Revision: 1.3 $";
	private static final String RCSName     = "$Name:  $";

    private int _code;

    /* the type codes are definde by java.sql.Types , they are
     * ARRAY,  BIGINT,  BINARY,  BIT,  BLOB,  CHAR,  CLOB,  DATE,  DECIMAL,  DISTINCT,
     * DOUBLE,  FLOAT,  INTEGER,  JAVA_OBJECT,  LONGVARBINARY,  LONGVARCHAR,  NULL,
     * NUMERIC,  REAL,  SMALLINT,  STRUCT,  TIME,  TIMESTAMP,  TINYINT,  VARBINARY,  VARCHAR
    */
    /* extended type :
    *   ORACLE_ROWID =
    */
    public static final int  ORACLE_ROWID = 111;

    public static final int  BOOLEAN = 101;

    /**
    @param name
    @roseuid 3BD14DA801F4
     */
//    public DbPrimitiveType(String name)
//    {
//        setCode ( getCodeFromName (name));
//    }

    /**
    @param code
    @roseuid 3BD14D8B0093
     */
    public DbPrimitiveType(int code)
    {
        setCode (code);
    }

//    public Object clone () throws CloneNotSupportedException
//    {
//        DbPrimitiveType retVal = (DbPrimitiveType)super.clone();
//        return  retVal;
//    }

    /**
    Access method for the _name property.

    @return   the current value of the _name property
     */
    public String getName()
    {
        return getNameFromCode(_code);
    }

    public DbPrimitiveType getPrimitiveType()
    {
        return this;
    }

    public int getPrimitiveTypeCode()
    {
        return _code;
    }

    public boolean isPrimitive()
    {
        return true;
    }

    public boolean isUdt ()
    {
        return false;
    }

    /**
    Access method for the _code property.

    @return   the current value of the _code property
     */
    public int getCode()
    {
        return _code;
    }

    /**
    Sets the value of the _code property.

    @param aCode the new value of the _code property
     */
    public void setCode(int aCode) {
        _code = aCode;
    }

    public String name() {
        return getNameFromCode(_code);
    }
    /**
    @param code
    @return String
    @roseuid 3BD14FB5018A
     */
    protected static String getNameFromCode(int code) {
        String retVal = null;
        switch (code) {
            case Types.ARRAY:
                retVal = "ARRAY";
                break;
            case Types.BIGINT:
                retVal = "BIGINT";
                break;
            case Types.BINARY:
                retVal = "BINARY";
                break;
            case Types.BIT:
                retVal = "BIT";
                break;
            case Types.BLOB:
                retVal = "BLOB";
                break;
            case Types.CHAR:
                retVal = "CHAR";
                break;
            case Types.CLOB:
                retVal = "CLOB";
                break;
            case Types.DATE:
                retVal = "DATE";
                break;
            case Types.DECIMAL:
                retVal = "DECIMAL";
                break;
            case Types.DISTINCT:
                retVal = "DISTINCT";
                break;
            case Types.DOUBLE:
                retVal = "DOUBLE";
                break;
            case Types.FLOAT:
                retVal = "FLOAT";
                break;
            case Types.INTEGER:
                retVal = "INTEGER";
                break;
            case Types.JAVA_OBJECT:
                retVal = "JAVA_OBJECT";
                break;
            case Types.LONGVARBINARY:
                retVal = "LONGVARBINARY";
                break;
            case Types.LONGVARCHAR:
                retVal = "LONGVARCHAR";
                break;
            case Types.NULL:
                retVal = "NULL";
                break;
            case Types.NUMERIC:
                retVal = "NUMERIC";
                break;
            case Types.REAL:
                retVal = "REAL";
                break;
            case Types.SMALLINT:
                retVal = "SMALLINT";
                break;
            case Types.STRUCT:
                retVal = "STRUCT";
                break;
            case Types.TIME:
                retVal = "TIME";
                break;
            case Types.TIMESTAMP:
                retVal = "TIMESTAMP";
                break;
            case Types.TINYINT:
                retVal = "TINYINT";
                break;
            case Types.VARBINARY:
                retVal = "VARBINARY";
                break;
            case Types.VARCHAR:
                retVal = "VARCHAR";
                break;
            case ORACLE_ROWID:
                retVal = "ORACLE_ROWID";
                break;
            default:
                break;
        }

        return retVal;
    }


    public  boolean isNumeric()  {
        return  Types.INTEGER   == _code ||
                Types.DECIMAL   == _code ||
                Types.DOUBLE    == _code ||
                Types.FLOAT     == _code ||
                Types.REAL     == _code  ||
                Types.NUMERIC   == _code ||
                Types.SMALLINT  == _code ||
                Types.BIGINT    == _code ||
                Types.TINYINT   == _code ;
    }

    public  boolean isDecimal()  {
        return  Types.DECIMAL   == _code;
    }

    public  boolean isInteger() {
        return  Types.INTEGER == _code ||
                Types.SMALLINT== _code ||
                Types.BIGINT== _code ||
                Types.TINYINT == _code ;
    }

    public  boolean isString() {
        return  Types.CHAR  == _code ||
                Types.VARCHAR == _code ||
                Types.LONGVARCHAR == _code ;
    }

    public  boolean isDataTime() {
        return  Types.DATE == _code ||
                Types.TIMESTAMP == _code ||
                Types.TIME == _code ;
    }

    public  boolean isNull() {
        return  Types.NULL == _code;
    }

    public String pprint() {
        return DbPrimitiveType.getNameFromCode(_code) + "(" + Integer.toString(_code) + ")";
    }
}
