/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */

package org.xquark.extractor.metadata;


public class DbUdt extends DataType
{

	private static final String RCSRevision = "$Revision: 1.3 $";
	private static final String RCSName     = "$Name:  $";

    private String _name;
    protected DbPrimitiveType _primitiveType;

    /**
    Access method for the _name property.

    @return   the current value of the _name property
     */

    public DbUdt (DbPrimitiveType dbPrimitiveType, String name )
    {
        setPrimitiveType(dbPrimitiveType);
        setName(name);
    }

    public DbUdt (int primitiveTypeCode, String name )
    {
        setPrimitiveType( new DbPrimitiveType(primitiveTypeCode));
        setName(name);
    }

//    public Object clone () throws CloneNotSupportedException
//    {
//        DbUdt retVal = (DbUdt)super.clone();
//        return  retVal;
//    }

    public String getName()
    {
     return _name;
    }

    /**
    Sets the value of the _name property.

    @param aName the new value of the _name property
     */
    public void setName(String aName)
    {
        _name = aName;
    }

    /**
    Access method for the _primitiveType property.

    @return   the current value of the _primitiveType property
     */
    public DbPrimitiveType getPrimitiveType()
    {
     return _primitiveType;
    }

    /**
    Sets the value of the _primitiveType property.

    @param aPrimitiveType the new value of the _primitiveType property
     */
    public void setPrimitiveType(DbPrimitiveType aPrimitiveType)
    {
        _primitiveType = aPrimitiveType;
    }

    /**
    @return int
    @roseuid 3BD1694B000C
     */
    public int getPrimitiveTypeCode()
    {
     return _primitiveType.getCode();
    }

    /**
    @return boolean
    @roseuid 3BD1694B0020
     */
    public boolean isPrimitive()
    {
     return false;
    }

    /**
    @return boolean
    @roseuid 3BD1694B00C1
     */
    public boolean isUdt()
    {
     return true;
    }

    public boolean isNumeric()
    {
     return _primitiveType.isNumeric();
    }

    public boolean isDecimal()
    {
     return _primitiveType.isDecimal();
    }

    /**
    @return boolean
    @roseuid 3BD1694B00F3
     */
    public boolean isInteger()
    {
     return _primitiveType.isInteger();
    }

    /**
    @return boolean
    @roseuid 3BD1694B016B
     */
    public boolean isString()
    {
     return _primitiveType.isString();
    }

    /**
    @return boolean
    @roseuid 3BD1694B01CF
     */
    public boolean isDataTime()
    {
     return _primitiveType.isDataTime();
    }

    public  boolean isNull()
    {
        return  _primitiveType.isNull();
    }

    public String pprint()
    {
        return getName()+ "<" + _primitiveType.pprint() + ">";
    }
}
