/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */

package org.xquark.extractor.metadata;

import java.sql.ResultSet;
import java.sql.Types;

import org.xquark.extractor.common.MessageLibrary;
import org.xquark.extractor.common.MetaDataException;
import org.xquark.extractor.common.Trace;

public class MsSqlServerTypeInterpreter implements DbmsTypeInterpreter {

	private static final String RCSRevision = "$Revision: 1.3 $";
	private static final String RCSName     = "$Name:  $";


    public MsSqlServerTypeInterpreter() {
    }

    public DataType interprete(ResultSet resultSet) throws MetaDataException
    {
        Trace.enter(this,"interprete(ResultSet)");
        DataType retVal = null;

        String      type_name=null;
        int         data_type=0;
        int         pTypeCode=0;  // primitive type code defined by java.sql.Types

        /*  all column available in ResultSet :
         *  TABLE_CAT, 	TABLE_SCHEM, 	TABLE_NAME, 	COLUMN_NAME, 	DATA_TYPE, 	TYPE_NAME,
         *  COLUMN_SIZE, 	BUFFER_LENGTH, 	DECIMAL_DIGITS, 	NUM_PREC_RADIX, 	NULLABLE,
         *  REMARKS, 	COLUMN_DEF, 	SQL_DATA_TYPE, 	SQL_DATETIME_SUB, 	CHAR_OCTET_LENGTH,
         *  ORDINAL_POSITION, 	IS_NULLABLE, 	SS_DATA_TYPE
         */
        try {
            data_type = resultSet.getInt("DATA_TYPE");
            type_name = resultSet.getString ("TYPE_NAME");

            pTypeCode = getDbPrimitypeCode(pTypeCode);

            if ( Integer.MAX_VALUE != pTypeCode ){
                retVal = new DbPrimitiveType (pTypeCode);
            }
            else {
                 pTypeCode = getDbPrimitypeCode (data_type);
                 retVal = new DbUdt (pTypeCode,type_name);
            }
        }
        catch (Exception ex) {
            throw new MetaDataException (MessageLibrary.getMessage("M_SUP_DATATYPE",type_name), null);
        }
        Trace.exit(this,"interprete(ResultSet)");
        return retVal;
    }

    private int getDbPrimitypeCode (String type_name)
    {
        if (type_name.equalsIgnoreCase("bigint"))    { return  Types.BIGINT ;}
        if (type_name.equalsIgnoreCase("bigint identity"))    { return  Types.BIGINT;}// not in list

        if (type_name.equalsIgnoreCase("binary"))    { return  Types.BINARY;}
        if (type_name.equalsIgnoreCase("bit"))       { return  Types.BIT;}
        if (type_name.equalsIgnoreCase("char"))      { return  Types.CHAR;}
        if (type_name.equalsIgnoreCase("datetime"))  { return  Types.DATE;}
        if (type_name.equalsIgnoreCase("decimal" ))  { return  Types.DECIMAL;}
        if (type_name.equalsIgnoreCase("float"))     { return  Types.FLOAT;}
        if (type_name.equalsIgnoreCase("image"))     { return  Types.LONGVARBINARY;}
        if (type_name.equalsIgnoreCase("int"))       { return  Types.INTEGER;}
        if (type_name.equalsIgnoreCase("int identity"))      { return  Types.INTEGER;} // not in list

        if (type_name.equalsIgnoreCase("money"))     { return  Types.DECIMAL;}// not in list
        if (type_name.equalsIgnoreCase("nchar"))     { return  Types.CHAR;}
        if (type_name.equalsIgnoreCase("ntext"))     { return  Types.VARCHAR;}
        if (type_name.equalsIgnoreCase("numeric"))   { return  Types.NUMERIC;}
        if (type_name.equalsIgnoreCase("nvarchar"))  { return  Types.VARCHAR;}
        if (type_name.equalsIgnoreCase("real"))      { return  Types.REAL;}
        if (type_name.equalsIgnoreCase("smalldatetime")) { return  Types.DATE;}
        if (type_name.equalsIgnoreCase("smallint"))  { return  Types.SMALLINT;}
        if (type_name.equalsIgnoreCase("smallmoney")) { return  Types.DECIMAL;}
        if (type_name.equalsIgnoreCase("sql_variant")) { return  Types.VARBINARY;}// not in list
        if (type_name.equalsIgnoreCase("sysname"))   { return  Types.CHAR;}// not in list

        if (type_name.equalsIgnoreCase("text"))      { return  Types.LONGVARCHAR;}
        if (type_name.equalsIgnoreCase("timestamp")) { return  Types.TIMESTAMP;}
        if (type_name.equalsIgnoreCase("tinyint"))   { return  Types.TINYINT;}
        if (type_name.equalsIgnoreCase("uniqueidentifier")) { return  Types.CHAR;}
        if (type_name.equalsIgnoreCase("varbinary")) { return  Types.VARBINARY;}
        if (type_name.equalsIgnoreCase("varchar"))   { return  Types.VARCHAR;}

        if (type_name.equalsIgnoreCase("empid"))   {
            int i = 1;
        }
        if (type_name.equalsIgnoreCase("id"))   {
            int i = 1;
        }
        return Integer.MAX_VALUE;
    }

    private int getDbPrimitypeCode (int data_type)
    {
        if (data_type == -5)    { return  Types.BIGINT;}// not in list
        if (data_type == -2)    { return  Types.BINARY;}
        if (data_type == -7)       { return  Types.BIT;}
        if (data_type ==  1)      { return  Types.CHAR;}
        if (data_type == 93)  { return  Types.DATE;}
        if (data_type == 3)  { return  Types.DECIMAL;}
        if (data_type == 6)     { return  Types.FLOAT;}
        if (data_type == -4)     { return  Types.LONGVARBINARY;}
        if (data_type == 4)       { return  Types.INTEGER;}
//        if (type_name.equalsIgnoreCase("nchar"))     { return  Types.CHAR;}
//        if (type_name.equalsIgnoreCase("ntext"))     { return  Types.VARCHAR;}
        if (data_type == 2)   { return  Types.NUMERIC;}
//        if (type_name.equalsIgnoreCase("nvarchar"))  { return  Types.VARCHAR;}
        if (data_type == 7)      { return  Types.REAL;}
//        if (data_type == 93) { return  Types.DATE;}
        if (data_type == 5)  { return  Types.SMALLINT;}

//        if (type_name.equalsIgnoreCase("sql_variant")) { return  Types.VARBINARY;}// not in list
//        if (type_name.equalsIgnoreCase("sysname"))   { return  Types.CHAR;}// not in list

        if (data_type == -1)      { return  Types.LONGVARCHAR;}
        if (data_type == -2) { return  Types.TIMESTAMP;}
        if (data_type == -6)   { return  Types.TINYINT;}
//        if (type_name.equalsIgnoreCase("uniqueidentifier")) { return  Types.CHAR;}
        if (data_type == -3) { return  Types.VARBINARY;}
        if (data_type == 12)   { return  Types.VARCHAR;}

        return Integer.MAX_VALUE;
    }

}
