/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */

package org.xquark.extractor.mysql;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.xquark.extractor.algebra.*;
import org.xquark.extractor.common.Trace;
import org.xquark.extractor.mysql.sql.SqlCreateTempTableAs;
import org.xquark.extractor.sql.*;

public final class MySqlGenSqlVisitor extends AbstractGenSqlVisitor {
	
	private static final String RCSRevision = "$Revision: 1.2 $";
	private static final String RCSName = "$Name:  $";

    private int _nameCounter = 0;
    
    public MySqlGenSqlVisitor() {
        super((SqlFactory) MySqlFactory.getInstance());
    }


	public void reinit() {
		super.reinit();
		// QUESTION: reset counter to 0 ?
	}

    /**
    @param arg
    @return org.xquark.extractor.sql.SqlExpression
    @roseuid 3BD177C90034
     */
    public SqlExpression visit(UnOpAggregate arg)
    {
    	Trace.enter(this,"visit(UnOpAggregate arg)");
    
		SqlExpression retVal = visit((UnOpProject)arg);
    
    	Trace.exit(this,"visit(UnOpAggregate arg)");
    	return retVal;
    }

    public SqlExpression visit(Join arg)
    {
    	/* Create join clause */
		SqlExpression retVal = super.visit(arg);
    
		/*  add join predicate to select WHERE clause */
		SqlSelect select = (SqlSelect) _nestStack.peek();
		List aPredicateList = arg.getPredicateList();
		if (null !=  aPredicateList) {
			Iterator iter = aPredicateList.iterator();
			while (iter.hasNext()) {
				Expression aPredicate = (Expression)iter.next();
				SqlExpression sPredicate = aPredicate.accept(this);
				SqlExpression newWhereClause = whereClauseAddPredicate(select.getWhereClause(), sPredicate);
				select.setWhereClause(newWhereClause);
			}
		}
		else {
			Trace.warn( this , "visit" , "join operation without predicate");
		}
    	Trace.exit(this,"visit(Join arg)");
    	return retVal;
    }
	// TODO: generate also subquery for sorts...
	
    public SqlExpression visit(UnOpProject arg)
    {
    	Trace.enter(this,"visit(UnOpProject arg)");
		SqlExpression retVal;
    
		SqlSelect outerExpression = null;
		if (!_nestStack.isEmpty())
			outerExpression = (SqlSelect) _nestStack.firstElement();
		
		SqlSelect select = _factory.createSelect(arg);
		_nestStack.push(select);
    
		retVal = selectAddSelectClause(select,arg);	// After potential attributes renaming

    	/* if _nestStack not empty (subquery) create a temporary table (supposed
    	 * to remain only in FROMs */
    	if (outerExpression != null) {
			String tempTableName = getTemporaryTableName();
			SqlCreateTempTableAs sCreateTempTable = new SqlCreateTempTableAs(tempTableName, select);
			outerExpression.appendPrecedent(sCreateTempTable);
			retVal = _factory.createTable(tempTableName);
			
//    		/* Disconnect the inner table and rename the outer query reference to inner relation tables */
//    		Expression father = arg.getFather();	// save it before it is changed by the following instruction
//    		RenameRelation dummyRename = new RenameRelation(arg, tempTableName);
//			father.replaceChild(arg, dummyRename);
//			
//			// TODO: add a rename to the temp table    							
//    		outerExpression.getOriginalExpression().accept(_renameVisitor);
		}
		_nestStack.pop();
    	
    	Trace.exit(this,"visit(UnOpProject arg)");
    	return retVal;
    }

	protected SqlSelect selectAddFromClause(SqlSelect select, BinOpOuterJoin arg) {
        Trace.enter(
            this,
            "selectAddFromClause (SqlSelect select, BinOpOuterJoin arg)",
            arg);
    
		Expression [] relations = {arg.getLeftOperand(), arg.getRightOperand()};
		SqlExpression[] sOprnd = new SqlExpression[2];

		for (int i = 0; i < relations.length; i++) {
				// NOTE: a list of if (instanceof) used to check algebra integrity have
				// been removed because these tests make visitor automatic type switch worthless...
				sOprnd[i] = relations[i].accept(this);
		}

		/* Outer join predicate creation */
		List aPredicateList = arg.getPredicateList();
		List sPredicateList = null;
		if (null != aPredicateList) {
			/* translate predicates */
			sPredicateList = new ArrayList();
			Expression aPredicate = null;
			org.xquark.extractor.sql.SqlExpression sPredicate = null;

			for (int i = 0; i < aPredicateList.size(); i++) {
				aPredicate = (Expression)aPredicateList.get(i);
				sPredicate = aPredicate.accept(this);
				sPredicateList.add(sPredicate);
			}
		}

		/* Outer join clause creation */
		org.xquark.extractor.sql.SqlExpression retVal =
			new org.xquark.extractor.sql.SqlBinOpOuterJoin(
				sOprnd[0], sOprnd[1],
				arg.getJoinType(), sPredicateList);

        select.addFromClause(retVal);
    
        Trace.exit(
            this,
            "selectAddFromClause (SqlSelect select, BinOpOuterJoin arg)",
            arg);
        return select;
    }

    protected String getTemporaryTableName() {
    	return "extractorTempView" + Integer.toString(_nameCounter++) ;
    }
}
