/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */

package org.xquark.extractor.progress;

import java.util.*;

import org.xquark.extractor.algebra.*;
import org.xquark.extractor.common.Debug;
import org.xquark.extractor.common.SqlWrapperException;
import org.xquark.extractor.common.Trace;

/**
 *
 * To change the template for this generated type comment go to
 * Window>Preferences>Java>Code Generation>Code and Comments
 */
public class ProgressRemoveNestedAggregation extends RemoveNestedAggregation {

	protected Map _vt = new HashMap();

    protected AggregationVisitor _aggregationVisitor = new AggregationVisitor();

    public ProgressRemoveNestedAggregation() {
        super();
        // TODO Auto-generated constructor stub
    }

    /**
     * if this UnOpProject contains nested aggregation, transform the nested
     * aggregations into a outer join and a groupby.
     * @param arg
     */
    public void visit(UnOpProject arg) {
    	Trace.enter(this, "visit(UnOpProject arg)", arg);
    	super.visit((UnaryAlgebra)arg);
    	if (arg instanceof UnOpProject && !(arg instanceof UnOpAggregate)) {
    		List aggregationList = findAggregations(arg.getItemList());
    
    		/** TODO
    		 *  The aggregations contained in this project might have common
    		 *  base relations. In this case , several aggregation based on a same
    		 *  relation should by put in one aggregation.
    		 *  example :
    		 *  SELECT i.itemno ,
    		 *      (select max(b.bid) from bids b where b.itemno = i.itemno),
    		 *      (select count(b.bid) from bids b where b.itemno = i.itemno)
    		 *  from items i;
    		 *  In the above query , two nested aggregations should be put in one
    		 *  subquery :
    		 *  SELECT i.itemno ,
    		 *      (select max(b.bid) , count(b.bid) from bids b where b.itemno = i.itemno)
    		 *  from items i;
    		 */
    
    		if (aggregationList != null && !aggregationList.isEmpty()) {
    			try {
    				/* find out main relation */
    				Expression mainRelation = arg.getOperand();
    
    				if (true /** TODO  mainRelation is a complex algebra tree */) {
    //					mainRelation = new UnOpStore(mainRelation);
    //				    mainRelation = new RenameRelation(mainRelation, getRelationName(9));
    				}
    
    				/* Create a subtree (containing UnOpAggregate and UnOpgroup) for each aggregation */
    				List groupbyList = new ArrayList();
    				Expression groupby = null;
    				AggregatingExpression aggrExpr = null;
    				List predicateList = null;
    				for (int i = 0; i < aggregationList.size(); i++) {
    					aggrExpr = (AggregatingExpression)aggregationList.get(i);
    
    					/* added for optimization */
    					Expression translation = (Expression)_vt.get(aggrExpr._aggregation);
    					if (translation != null) {
    						aggrExpr._referringNode.replaceChild(aggrExpr._aggregation, translation);
    						continue;
    					}
    
    					predicateList = pickOutJoinPredicates(aggrExpr._aggregation);
    
    					Set tiSet = aggrExpr._aggregatedExpr.getReferredTableInstances();
    					Debug.assertTrue(null != tiSet && !tiSet.isEmpty(), "null != tiSet && !tiSet.isEmpty()");
    					boolean localExpression = aggrExpr._aggregation.visibleTableInstances().containsAll(tiSet);
    
    					if ((predicateList != null && !predicateList.isEmpty()) || !localExpression ) {
    						groupby = createGroupby (mainRelation, aggrExpr, predicateList);
    						groupbyList.add(groupby);
    					}
    					else {
    						/* the aggregation is a constant */
    						TempValue tempValue = new TempValue(aggrExpr._aggregation);
    						aggrExpr._referringNode.replaceChild(aggrExpr._aggregation, tempValue);
    					}
    				}
    
    				if (!groupbyList.isEmpty()) {
    					/* add keys to mainRelation, this keys are needed to outer join
    					*  mainRelation and groupby that will be created later.
    					*/
    					_addKeysVisitor.reinit();
    					mainRelation.accept(_addKeysVisitor);
    					List keys = ((Relation)mainRelation).getKeys();
    					RenameRelation renameRelation = null;
    					predicateList = null;
    
    					BinOpOuterJoin outerJoin = null;
    					for (int i = 0; i < groupbyList.size(); i++) {
    
    						renameRelation = (RenameRelation)groupbyList.get(i);
    						predicateList =  createJoinPredicateList(keys , renameRelation);
    						if (i == 0)
    							outerJoin = new BinOpOuterJoin(mainRelation, renameRelation, Constants.LEFT_OUTER_JOIN,predicateList);
    						else
    							outerJoin = new BinOpOuterJoin(outerJoin, renameRelation, Constants.LEFT_OUTER_JOIN, predicateList);
    					}
    
    					arg.setOperand(outerJoin);
    					SimplifyOuterJoinVisitor visitor = new SimplifyOuterJoinVisitor(); // QUESTION: What's this ?
    					outerJoin.accept(visitor);
    				}
    			}
    			catch (Exception ex) {
    				throw new SqlWrapperException(ex.getMessage(), ex);
    			}
    		}
    	}
    	Trace.exit(this, "visit(UnOpProject arg)", arg);
    }

    public void visit(UnOpRestrict arg) {
    	Trace.enter(this, "visit(UnOpRestrict arg)", arg);
    	super.visit((UnaryAlgebra)arg);
    
    	List aggregationList = findAggregations(arg.getPredicateList());
    
    	if (null != aggregationList && !aggregationList.isEmpty()) {
    		try {
    			/* find out main relation */
    			Expression mainRelation = arg.getOperand();
    
    			/* Create a subtree(containning UnOpAggregate and UnOpgroup) for each aggregation */
    			List groupbyList = new ArrayList();
    			Expression groupby = null;
    			AggregatingExpression aggrExpr = null;
    			List predicateList = null;
    			for (int i = 0; i < aggregationList.size(); i++) {
    				aggrExpr = (AggregatingExpression)aggregationList.get(i);
    
    				/* added for optimization */
    				Expression translation = (Expression)_vt.get(aggrExpr._aggregation);
    				if (null != translation) {
    					aggrExpr._referringNode.replaceChild(aggrExpr._aggregation, translation);
    					continue;
    				}
    
    				predicateList = pickOutJoinPredicates(aggrExpr._aggregation);
    
    				Set tiSet = aggrExpr._aggregatedExpr.getReferredTableInstances();
    				Debug.assertTrue(null != tiSet && !tiSet.isEmpty(), "null != tiSet && !tiSet.isEmpty()");
    				boolean localExpression = aggrExpr._aggregation.visibleTableInstances().containsAll(tiSet);
    
    				if ((null != predicateList && !predicateList.isEmpty()) || !localExpression ) {
    					groupby = createGroupby (mainRelation, aggrExpr, predicateList);
    					groupbyList.add(groupby);
    				}
    				else {
    					/* the aggregation is a constant */
    					TempValue tempValue = new TempValue(aggrExpr._aggregation);
    					aggrExpr._referringNode.replaceChild(aggrExpr._aggregation, tempValue);
    				}
    			}
    
    			if (!groupbyList.isEmpty()) {
    				/* add keys to mainRelation, this keys are needed to outer join
    				*  mainRelation and groupby that will be created later.
    				*/
    				_addKeysVisitor.reinit();
    				mainRelation.accept(_addKeysVisitor);
    				List keys = ((Relation)mainRelation).getKeys();
    				RenameRelation renameRelation = null;
    				predicateList = null;
    
    				BinOpOuterJoin outerJoin = null;
    				for (int i = 0; i < groupbyList.size(); i++) {
    					renameRelation = (RenameRelation)groupbyList.get(i);
    					predicateList =  createJoinPredicateList(keys , renameRelation);
    					if (0 == i) {
    						outerJoin = new BinOpOuterJoin(mainRelation, renameRelation, Constants.LEFT_OUTER_JOIN,predicateList);
    					}
    					else {
    						outerJoin = new BinOpOuterJoin(outerJoin, renameRelation, Constants.LEFT_OUTER_JOIN, predicateList);
    					}
    				}
    
    				arg.setOperand(outerJoin);
    				SimplifyOuterJoinVisitor visitor = new SimplifyOuterJoinVisitor();
    				outerJoin.accept(visitor);
    			}
    		}
    		catch (Exception ex) {
    			throw new SqlWrapperException(ex.getMessage(), ex);
    		}
    	}
    	Trace.exit(this, "visit(UnOpRestrict arg)", arg);
    }
    public void visit(Join arg) {
    	Trace.enter(this, "visit(Join arg)", arg);
    	super.visit(arg);
    
    	List aggregationList = findAggregations(arg.getPredicateList());
    
    	if (null != aggregationList && !aggregationList.isEmpty()) {
    		try {
    			/* find out main relation */
    			Expression mainRelation = arg;
    			Expression father = arg.getFather();
    
    			/* Create a subtree(containning UnOpAggregate and UnOpgroup) for each aggregation */
    			List groupbyList = new ArrayList();
    			Expression groupby = null;
    			AggregatingExpression aggrExpr = null;
    			List predicateList = null;
    			for (int i = 0; i < aggregationList.size(); i++) {
    				aggrExpr = (AggregatingExpression)aggregationList.get(i);
    
    				/* added for optimization */
    				Expression translation = (Expression)_vt.get(aggrExpr._aggregation);
    				if (null != translation) {
    					aggrExpr._referringNode.replaceChild(aggrExpr._aggregation, translation);
    					continue;
    				}
    
    				arg.removePredicate(aggrExpr._root);
    
    				predicateList = pickOutJoinPredicates(aggrExpr._aggregation);
    
    				Set tiSet = aggrExpr._aggregatedExpr.getReferredTableInstances();
    				Debug.assertTrue(null != tiSet && !tiSet.isEmpty(), "null != tiSet && !tiSet.isEmpty()");
    				boolean localExpression = aggrExpr._aggregation.visibleTableInstances().containsAll(tiSet);
    
    				if ((null != predicateList && !predicateList.isEmpty()) || !localExpression ) {
    					groupby = createGroupby (mainRelation, aggrExpr, predicateList);
    					groupbyList.add(groupby);
    				}
    				else {
    					/* the aggregation is a constant */
    					TempValue tempValue = new TempValue(aggrExpr._aggregation);
    					aggrExpr._referringNode.replaceChild(aggrExpr._aggregation, tempValue);
    				}
    			}
    
    			if (!groupbyList.isEmpty()) {
    				/* add keys to mainRelation, this keys are needed to outer join
    				*  mainRelation and groupby that will be created later.
    				*/
    				_addKeysVisitor.reinit();
    				mainRelation.accept(_addKeysVisitor);
    				List keys = ((Relation)mainRelation).getKeys();
    				RenameRelation renameRelation = null;
    				predicateList = null;
    
    				BinOpOuterJoin outerJoin = null;
    				for (int i = 0; i < groupbyList.size(); i++) {
    					renameRelation = (RenameRelation)groupbyList.get(i);
    					predicateList =  createJoinPredicateList(keys , renameRelation);
    					if (0 == i) {
    						outerJoin = new BinOpOuterJoin(mainRelation, renameRelation, Constants.LEFT_OUTER_JOIN,predicateList);
    					}
    					else {
    						outerJoin = new BinOpOuterJoin(outerJoin, renameRelation, Constants.LEFT_OUTER_JOIN, predicateList);
    					}
    				}
    
    				List list = new ArrayList();
    				for (int i = 0; i < aggregationList.size(); i++) {
    					list.add(((AggregatingExpression)aggregationList.get(i))._root);
    				}
    				UnOpRestrict restrict = new UnOpRestrict(outerJoin,list);
    				restrict.setOperand(outerJoin);
    
    				father.replaceChild(arg, restrict);
    
    				SimplifyOuterJoinVisitor visitor = new SimplifyOuterJoinVisitor();
    				outerJoin.accept(visitor);
    			}
    		}
    		catch (Exception ex) {
    			throw new SqlWrapperException(ex.getMessage(), ex);
    		}
    	}
    	Trace.exit(this, "visit(Join arg)", arg);
    }

    /**
     * This function
     * 1. create a goupby subtree for an UnOpAggregate
     * 2. update itemList in <code>project<code>
     *
     */
    protected RenameRelation createGroupby(Expression mainRelation, AggregatingExpression aggregatingExpr, List joinPredicateList)
    	throws CloneNotSupportedException {
    	Trace.enter(this, "createGroupby(...)");
    	RenameRelation retVal = null;
    
    
    	mainRelation = (Expression)mainRelation.clone();
    	UnOpAggregate aggregation = aggregatingExpr._aggregation;
    
    	/* create a join node */
    	Join join = new Join();
    	join.addOperand(mainRelation);
    	join.addOperand(aggregation.getOperand());
    	join.setPredicateList(joinPredicateList);
    
    	/* create a groupby node */
    	_addKeysVisitor.reinit();
    	mainRelation.accept(_addKeysVisitor);
    	List keysOfMainRelation = ((Relation)mainRelation).getKeys();
    	List clonedList = AlgebraTools.clone(keysOfMainRelation);
    	UnOpGroup group = new UnOpGroup(join, clonedList);
    
    	/* create aggregation node */
    	UnOpAggregate newAggregation = new UnOpAggregate();
    	newAggregation.setOperand(group);
    	newAggregation.addItem((Expression)((Expression)aggregation.getItemList().get(0)).clone());
    	newAggregation.setOrginalXExpr(aggregation.getOrginalXExpr());
    
    	_addKeysVisitor.reinit();
    	newAggregation.accept(_addKeysVisitor);
    
    	retVal = new RenameRelation(newAggregation);
    
    	/* modify the referencing node */
    	Expression aggregatedAttribute = (Expression)aggregation.getItemList().get(0);
    	AttributeExpression  correspondingAttribut = new AttributeExpression(retVal, aggregatedAttribute.getName());
    	correspondingAttribut.setUnderlyingExpr(aggregatedAttribute);
    	aggregatingExpr._referringNode.replaceChild(aggregatingExpr._aggregation, correspondingAttribut);
    
    	/* todo added for optimization */
    	RenameRelation ti = retVal;
    	RenameItem     aggr = (RenameItem)aggregation.getItemList().get(0);
    	AttributeExpression translation = new AttributeExpression(ti, aggr.getName());
    	translation.setUnderlyingExpr(aggr);
    	_vt.put(aggregatingExpr._aggregation, translation);
    
    	Trace.exit(this, "createGroupby(...)");
    	return retVal;
    }

    protected List createJoinPredicateList(List keys, RenameRelation renameRelation) throws CloneNotSupportedException {
    	Trace.enter(this, "createJoinPredicateList(List keys, RenameRelation renameRelation)");
    	List retVal = new ArrayList();
    
    	keys = AlgebraTools.clone(keys);
    	AttributeExpression keyItem = null;
    
    	_addKeysVisitor.reinit();
    	renameRelation.accept(_addKeysVisitor);
    	List keysInAggregation = AlgebraTools.clone(renameRelation.getKeys());
    	AttributeExpression keyInAggregation = null;
    
    	for (int i = 0; i < keys.size(); i++) {
    		keyItem = (AttributeExpression)keysInAggregation.get(i);
    		keyInAggregation =(AttributeExpression)keys.get(i);
    		retVal.add(new BinOpCompare(Constants.EQ_COMPOP,keyItem ,keyInAggregation));
    	}
    
    	Trace.exit(this, "createJoinPredicateList(List keys, RenameRelation renameRelation)");
    	return retVal;
    }

    /**
     * Browse a tree for {@link UnOpRestrict} and {@link Join} nodes and pick out
     * predicates referencing tables that are not in scope.
     * @param tree an expression.
     * @return a list of predicate expressions.
     */
    protected List pickOutJoinPredicates(Expression tree) {
    	Trace.enter(this, "pickOutJoinPredicats(Expression tree)", tree);
    	List retVal = new ArrayList();
    
    	/* find out UnOpRestrict nodes */
    	List joinPredicateList = new ArrayList();
    	_restrictionVisitor.reinit();
    	tree.accept(_restrictionVisitor);
    	List restrictionList = _restrictionVisitor.getRestrictions();
    
    	/* browse found nodes */
    	UnOpRestrict restrict = null;
    	Set vti = null;
    	List predicateList = null;
    	
    	for (int i = 0; i < restrictionList.size(); i++) {
    		Expression expr = (Expression)restrictionList.get(i);
    
    		vti = ((Relation)expr).visibleTableInstances();
    		if (expr instanceof PredicateHolder) {
    			predicateList = ((PredicateHolder)expr).getPredicateList();
    		}
    		else {
    			Debug.nyi("pickOutJoinPredicats(Expression tree)");
    		}
    
    		Expression predicate = null;
    		Set rti = null;
    		for (int j = predicateList.size() - 1; j >= 0; j--) {	// reverse order removes index problems caused by removal
    			predicate = (Expression)predicateList.get(j);
    			rti = predicate.getReferredTableInstances();
    			if (! vti.containsAll(rti)) { /** @todo  vti = null */
    				retVal.add(predicate);
    				((PredicateHolder)expr).removePredicate(predicate);
    			}
    		}
    	}
    
    	Trace.exit(this, "pickOutJoinPredicats(Expression tree)", tree);
    	return retVal;
    }

    /**
     * Find out if this project node contains aggregation in its itemList
     * @return a list of AggregatingExpression
     *
     */
    protected List findAggregations(List exprList) {
    	Trace.enter(this, "List findAggregations(List exprList)");
    	List retVal = new ArrayList();
    
    	Expression expr = null;
    	AggregatingExpression  aggrExpr = null;
    
    	if (null != exprList) {
    		int size = exprList.size();
    		for (int i = 0; i < size; i++) {
    			expr = (Expression)exprList.get(i);
    			_aggregationVisitor.reinit();
    			expr.accept(_aggregationVisitor);
    			if (_aggregationVisitor.isAggregation()) {
    
    				aggrExpr = new AggregatingExpression(expr,
    					_aggregationVisitor.getAggregation().getFather(),
    					_aggregationVisitor.getAggregation(),
    					_aggregationVisitor.getAggregatedExpression());
    				retVal.add(aggrExpr);
    			}
    		}
    	}
    
    	Trace.exit(this, "List findAggregations(List exprList)");
    	return retVal;
    }

    /**
     * An AggregatingExpression may be a simple aggregation, (i.e. <code>select
     * max(col1) from table1 where predicate1 <code>) or expression built on top
     * of a simple aggregation(i.e. <code> 2 * select max(col1) from table1
     * where predicate1 <code>). In this later case, AggregatingExpression indicates
     * important node in the tree.
     */
    protected class AggregatingExpression {
    
    	/**
    	 * Constructor.
         * @param root base expression that was scanned to find an aggregation.
         * Typically an item or a predicate.
         * @param referringNode The parent node of aggregation expression. Generally
         * a RenameItem node. But may be another operator (2*select(max)).
         * @param aggregation The aggregation expression (may not be an aggregation
         * function).
         * @param aggregatedExpr The argument of aggregation function.
         */
        public AggregatingExpression(Expression root,  Expression referringNode, UnOpAggregate aggregation, Expression aggregatedExpr) {
    		_root = root;
    		_referringNode = referringNode;
    		_aggregation = aggregation;
    		_aggregatedExpr = aggregatedExpr;
    	}
    
    	/**
         * Base expression.
         */
        public Expression      _root = null ;
    
    	/**
         * The parent node of aggregation expression.
         */
    	public Expression      _referringNode = null ;
    
    	/**
         * The aggregation expression.
         */
    	public UnOpAggregate   _aggregation = null;
        
    	/**
         * the argument of aggregation function.
         */
    	public Expression      _aggregatedExpr = null;
    }

    /**
     * this class scan a algebra tree and determine whether it is an aggregation
     */
    protected class AggregationVisitor extends DefaultSimpleVisitor {
    	private static final String RCSRevision = "$Revision: 1.2 $";
    	private static final String RCSName = "$Name:  $";
    
    	private UnOpAggregate _aggregation = null;
    
    	public AggregationVisitor () {
    	}
    
    	public void reinit() {
    		_aggregation = null;
    	}
    
    	public boolean isAggregation(){
    		return null != _aggregation;
    	}
    
    	public UnOpAggregate getAggregation(){
    		return _aggregation;
    	}
    
    	public Expression getAggregatedExpression(){
    		Expression retVal = null;
    		Expression item = (Expression)_aggregation.getItemList().get(0);
    		if (item instanceof RenameItem) {
    			item = ((RenameItem)item).getOperand();
    		}
    
    		Debug.assertTrue(item instanceof FunAggregate, "item instanceof FunAggregate");
    
    		FunAggregate  funAggregate = (FunAggregate)item;
    		retVal = funAggregate.getOperand();
    		return retVal;
    	}
    
    	public void visit(UnaryAlgebra arg) {
    	}
    
    	public void visit(BinaryAlgebra arg) {
    	}
    
    	public void visit(Join arg) {
    	}
    
    	public void visit(UnOpAggregate arg) {
    		_aggregation = arg;
    	}
    }

}
