/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */

package org.xquark.extractor.sql;

import java.sql.Types;
import java.util.*;

import org.xquark.extractor.algebra.*;
import org.xquark.extractor.algebra.Attribute;
import org.xquark.extractor.algebra.Table;
import org.xquark.extractor.common.*;
import org.xquark.extractor.metadata.*;
import org.xquark.extractor.xfunctions.*;

public class AbstractGenSqlVisitor  extends DefaultGenSqlVisitor  implements Constants
{
	private static final String RCSRevision = "$Revision: 1.3 $";
	private static final String RCSName     = "$Name:  $";

	protected SqlFactory _factory = null;

	protected MetaDataManager _metadataManager= null;

	// QUESTION: what's the use of this. peek() is never used and stack is only used by Sybase
	// to know if there are nested selects... A counter would be enough eventually ?
	protected Stack _nestStack = new Stack();

	/**
	@roseuid 3BD177BE002E
	 */
	protected AbstractGenSqlVisitor(SqlFactory aSqlFactory)
	{
		setSqlFactory(aSqlFactory);
	}


	/**
	Sets the value of the _sqlFactory property.

	@param aSqlFactory the new value of the _sqlFactory property
	 */
	public void setSqlFactory(SqlFactory aSqlFactory)
	{
	   Debug.assertTrue(null != aSqlFactory,"null != aSqlFactory");
		_factory = aSqlFactory;
	}

	public void setMetadataManager(MetaDataManager metadataManager) {
		_metadataManager = metadataManager;
	}

	public void reinit ()
	{
		_nestStack.clear();
	}

	/**
	@param arg
	@return org.xquark.extractor.sql.SqlExpression
	@roseuid 3BD177BE01F1
	 */
	public SqlExpression visit(Expression arg)
	{
		throw new SqlWrapperException(MessageLibrary.getMessage("T_N_SUP_EXPR",arg.pprint()),null);
	}


	/**
	@param arg
	@return org.xquark.extractor.sql.SqlExpression
	@roseuid 3BD177BF010C
	 */
	public SqlExpression visit(AttributeExpression arg)
	{
		Trace.enter(this, "visit(AttributeExpression arg)");

		SqlExpression retVal = null;
		String attrName = arg.getName();

		String tableInstanceName = null;
		RenameRelation  rr = arg.getTableInstance();
		if (rr != null)
			tableInstanceName = rr.getUniqueName();

		retVal = _factory.createAttributeExpression (tableInstanceName,attrName);
		Trace.exit(this,"visit(AttributeExpression arg)");
		return retVal;
	}

	/**
	@param arg
	@return org.xquark.extractor.sql.SqlExpression
	@roseuid 3BD177BF0288
	 */
	public SqlExpression visit(BinOpArithmetic arg)
	{
		Trace.enter(this,"visit(BinOpArithmetic arg)");

		SqlBinOpArithmetic  retVal = null;

		Expression      aLOprnd = arg.getLeftOperand();
		SqlExpression   sLOprnd = aLOprnd.accept(this);

		Expression      aROprnd = arg.getRightOperand();
		SqlExpression   sROprnd = aROprnd.accept(this);

		int oprtr = arg.getOperator();
		retVal = _factory.createBinOpArithmetic(oprtr,sLOprnd,sROprnd);

		Trace.exit(this,"visit(BinOpArithmetic arg)");
		return retVal;
	}

	/**
	@param arg
	@return org.xquark.extractor.sql.SqlExpression
	@roseuid 3BD177C00031
	 */
	public SqlExpression visit(BinOpBoolean arg)
	{
		Trace.enter(this,"visit(BinOpBoolean arg)");

		SqlBinOpBoolean  retVal = null;

		Expression      aLOprnd = arg.getLeftOperand();
		SqlExpression   sLOprnd = aLOprnd.accept(this);

		Expression      aROprnd = arg.getRightOperand();
		SqlExpression   sROprnd = aROprnd.accept(this);

		int oprtr = arg.getOperator();
		retVal = _factory.createBinOpBoolean(oprtr,sLOprnd,sROprnd);

		Trace.exit(this,"visit(BinOpBoolean arg)");
		return retVal;
	}

	/**
	@param arg
	@return org.xquark.extractor.sql.SqlExpression
	@roseuid 3BD177C001B7
	 */
	public SqlExpression visit(BinOpCompare arg)
	{
		Trace.enter(this,"visit(BinOpCompare arg)");
		SqlExpression  retVal = null;

		Expression      aLOprnd = arg.getLeftOperand();

		SqlType         lType = aLOprnd.getType();

		Expression      aROprnd = arg.getRightOperand();

		SqlType         rType = aROprnd.getType();

		if (lType.isString() && rType.isString()) {
			retVal = visitCompareString(arg);
		}
		else {
			SqlExpression   sLOprnd = aLOprnd.accept(this);
			SqlExpression   sROprnd = aROprnd.accept(this);
			int oprtr = arg.getOperator();
			retVal = _factory.createBinOpCompare(oprtr,sLOprnd,sROprnd);
		}

		Trace.exit(this,"visit(BinOpCompare arg)");
		return retVal;
	}

	protected SqlExpression visitCompareString(BinOpCompare arg)
	{
		Trace.enter(this,"visitCompareString(BinOpCompare arg)");
		SqlExpression  retVal = null;

		Expression      aLOprnd = arg.getLeftOperand();
		SqlExpression   sLOprnd = aLOprnd.accept(this);

		Expression      aROprnd = arg.getRightOperand();
		SqlExpression   sROprnd = aROprnd.accept(this);

		int oprtr = arg.getOperator();

		SqlExpression comp = null;
		SqlExpression eq = null;
		SqlExpression lenLOprnd = null;
		SqlExpression lenROprnd = null;
		SqlExpression lenL_lenR = null;
		SqlExpression and =  null;
		SqlExpression or =  null;

		switch (oprtr) {
			case LT_COMPOP:
				/* lOprnd < rOprnd ==> lOprnd< rOprnd or ( lOprnd = rOprnd and length(lOprnd) < length(rOprnd) ) */
			case GT_COMPOP:
				/* lOprnd > rOprnd ==> lOprnd > rOprnd or ( lOprnd = rOprnd and length(lOprnd) > length(rOprnd) ) */
				comp = _factory.createBinOpCompare(oprtr,sLOprnd,sROprnd);

				eq = _factory.createBinOpCompare(EQ_COMPOP,sLOprnd,sROprnd);

				lenLOprnd = _factory.createSfStringLength(sLOprnd);
				lenROprnd = _factory.createSfStringLength(sROprnd);
				lenL_lenR = _factory.createBinOpCompare(oprtr,lenLOprnd,lenROprnd);

				and =  _factory.createBinOpBoolean(AND,eq,lenL_lenR);

				or =  _factory.createBinOpBoolean(OR, comp, and);

				retVal = or;
				break;
			case GEQ_COMPOP:
			case LEQ_COMPOP:
			case NEQ_COMPOP:
				retVal = _factory.createBinOpCompare(oprtr,sLOprnd,sROprnd);
				break;
			case EQ_COMPOP:
				/* lOprnd = rOprnd ==> lOprnd = rOprnd and length(lOprnd) = length(rOprnd) ) */
				eq = _factory.createBinOpCompare(oprtr,sLOprnd,sROprnd);

				lenLOprnd = _factory.createSfStringLength(sLOprnd);
				lenROprnd = _factory.createSfStringLength(sROprnd);
				lenL_lenR = _factory.createBinOpCompare(oprtr,lenLOprnd,lenROprnd);

				and = _factory.createBinOpBoolean(AND,eq,lenL_lenR);
				retVal = and;
				break;
			default:
				throw new SqlWrapperException(MessageLibrary.getMessage("T_N_SUP_OPTR",COMPOPSSTRINGS[oprtr]));
		}
		Trace.exit(this,"visitCompareString(BinOpCompare arg)");
		return retVal;
	}

	/**
	@param arg
	@return org.xquark.extractor.sql.SqlExpression
	@roseuid 3BD177C0033E
	 */
	public SqlExpression visit(BinOpCompareAny arg)
	{
		Trace.enter(this,"visit(BinOpCompareAny arg)");

		SqlBinOpCompareAny  retVal = null;

		Expression      aLOprnd = arg.getLeftOperand();
		SqlExpression   sLOprnd = aLOprnd.accept(this);

		Expression      aROprnd = arg.getRightOperand();
		SqlExpression   sROprnd = aROprnd.accept(this);

		int oprtr = arg.getOperator();
		retVal = _factory.createBinOpCompareAny(oprtr,sLOprnd,sROprnd);

		Trace.exit(this,"visit(BinOpCompareAny arg)");
		return retVal;
	}

	/**
	@param arg
	@return org.xquark.extractor.sql.SqlExpression
	@roseuid 3BD177C100E7
	 */
	public SqlExpression visit(BinOpDifference arg)
	{
		throw new SqlWrapperException(MessageLibrary.getMessage("T_N_SUP_EXPR",arg),null);

	}

	/**
	@param arg
	@return org.xquark.extractor.sql.SqlExpression
	@roseuid 3BD177C1026D
	 */
	public SqlExpression visit(BinOpIntersect arg)
	{
		throw new SqlWrapperException(MessageLibrary.getMessage("T_N_SUP_EXPR",arg),null);

//        Trace.enter(this,"visit","BinOpIntersect");
//
//        Trace.exit(this,"visit","BinOpIntersect");  return null;
	}

	/**
	@param arg
	@return org.xquark.extractor.sql.SqlExpression
	@roseuid 3BD177C2000C
	 */
	public SqlExpression visit(BinOpMerge arg)
	{
		Trace.enter(this,"visit(BinOpMerge arg)");
		SqlBinOpUnion  retVal = null;

		Expression      aLOprnd = arg.getLeftOperand();
		SqlExpression   sLOprnd = aLOprnd.accept(this);

		Expression      aROprnd = arg.getRightOperand();
		SqlExpression   sROprnd = aROprnd.accept(this);

		retVal = _factory.createBinOpUnion (UNION_ALL,sLOprnd,sROprnd);
		Trace.exit(this,"visit(BinOpMerge arg)");
		return retVal;
	}


	/**
	@param arg
	@return org.xquark.extractor.sql.SqlExpression
	@roseuid 3BD177C20323
	 */
	public SqlExpression visit(BinOpUnion arg)
	{
		Trace.enter(this,"visit(BinOpUnion arg)");

		SqlExpression   sLOprnd = arg.getLeftOperand().accept(this);
		SqlExpression   sROprnd = arg.getRightOperand().accept(this);
		SqlBinOpUnion   retVal = _factory.createBinOpUnion(UNION,sLOprnd,sROprnd);

		Trace.exit(this,"visit(BinOpUnion arg)");
		return retVal;
	}

	/**
	@param arg
	@return org.xquark.extractor.sql.SqlExpression
	@roseuid 3BD177C300C1
	 */
	public SqlExpression visit(UnOpExists arg)
	{
		Trace.enter(this,"visit(UnOpExists arg)");

		SqlExpression   query = arg.getOperand().accept(this);
		SqlUnOpExists   retVal = _factory.createUnOpExists(query,arg.getNot());

		Trace.exit(this,"visit(UnOpExists arg)");
		return retVal;
	}

	public SqlExpression visit(UnOpIsNull arg) throws SqlWrapperException
	{
		Trace.enter(this, "visit(UnOpIsNull arg)");

		SqlExpression   operand = arg.getOperand().accept(this);
		SqlExpression   retVal = _factory.createUnOpIsNull(operand, arg.getNot());

		Trace.exit(this,"visit(UnOpIsNull arg)");
		return retVal;
	}

	/**
	@param arg
	@return org.xquark.extractor.sql.SqlExpression
	@roseuid 3BD177C30266
	 */
	public SqlExpression visit(FunAggregate arg)
	{
		Trace.enter(this,"visit(FunAggregate arg)");

		SqlExpression   sExpr = arg.getOperand().accept(this);
		SqlFunAggregate retVal = _factory.createFunAggregate(arg.getOperator(),sExpr,arg.getDistinct());

		Trace.exit(this,"visit(FunAggregate arg)");
		return retVal;
	}


	/**
	@param arg
	@return org.xquark.extractor.sql.SqlExpression
	@roseuid 3BD177C40023
	 */
	public SqlExpression visit(IfThenElse arg)
	{
		Trace.enter(this,"visit(IfThenElse arg)");
		SqlExpression retVal = null;
		SqlExpression   sIf = arg.getIf().accept(this);
		SqlExpression   sThen = arg.getThen().accept(this);
		SqlExpression   sElse = arg.getElse().accept(this);
		retVal = _factory.createIfThenElse(sIf,sThen,sElse);

		Trace.exit(this,"visit(IfThenElse arg)");
		return retVal;
	}

	// NOTE: we suppose no join would be heading an algebra tree.
	public SqlExpression visit(Join arg)
	{
		Trace.enter(this,"visit(Join arg)");

		List relations = arg.getOperands();
		if (null == relations) {
			throw new SqlWrapperException(MessageLibrary.getMessage("INT_LOG_ERR"),null);
		}

		RenameRelation  aRR;
		Expression      aRelation;
		SqlExpression   sRelation;
		int len = relations.size();
		
		/* Create join clause */
		SqlJoin sJoin = _factory.createJoin();

		for (int i = 0; i < len; i++) {
			/* generate FROM clause for SELECT */
			aRelation = (Expression) relations.get(i);
			// ADD   06/07/2003
			if (aRelation instanceof UnOpSort)
				aRelation = ((UnOpSort)aRelation).getOperand();	
			if (aRelation instanceof UnOpRestrict)
				aRelation = ((UnOpRestrict)aRelation).getOperand();	
			// end ADD  	
			aRR = (RenameRelation)aRelation;
			aRelation = aRR.getOperand();
			
			// NOTE: a list of if (instanceof) used to check algebra integrity have
			// been removed because these tests make visitor automatic type switch worthless...
			sRelation = aRelation.accept(this);
			sJoin.addOperand(_factory.createRenameRelation (sRelation, aRR.getUniqueName()));
		}
		
		Trace.exit(this,"visit(Join arg)");
		return sJoin;
	}

//	public SqlExpression visit(Join arg)
//	{
//		Trace.enter(this,"visit(Join arg)");
//
//		SqlSelect   retVal = _factory.createSelect();
//		_nestStack.push(retVal);
//		selectAddFromClause(retVal,arg);
//		_nestStack.pop();
//
//		Trace.exit(this,"visit(Join arg)");
//		return retVal;
//	}
//
	/**
	@param arg
	@return org.xquark.extractor.sql.SqlExpression
	@roseuid 3BD177C40362
	 */
	public SqlExpression visit(LitBoolean arg)
	{
//        throw new SqlWrapperException(MessageLibrary.getMessage("T_N_SUP_EXPR",arg),null);
		Trace.enter(this,"visit(LitBoolean arg)");

		SqlLitBoolean    retVal = null;
		retVal = _factory.createLitBoolean (arg.getValue());

		Trace.exit(this,"visit(LitBoolean arg)");
		return retVal;
	}

	/**
	@param arg
	@return org.xquark.extractor.sql.SqlExpression
	@roseuid 3BD177C5011E
	 */
	public SqlExpression visit(LitDate arg)
	{
		Trace.enter(this,"visit(LitDate arg)");

		SqlLitDate    retVal = null;
		retVal = _factory.createLitDate (arg.getValue());

		Trace.exit(this,"visit(LitDate arg)");
		return retVal;
	}


	public SqlExpression visit(LitDecimal arg)
	{
		Trace.enter(this,"visit(LitDecimal arg)");

		SqlLitDecimal    retVal = null;
		retVal = _factory.createLitDecimal(arg.getValue());

		Trace.exit(this,"visit(LitDecimal arg)");
		return retVal;
	}

	/**
	@param arg
	@return org.xquark.extractor.sql.SqlExpression
	@roseuid 3BD177C502B9
	 */
	public SqlExpression visit(LitDouble arg)
	{
		Trace.enter(this,"visit(LitDouble arg)", arg.pprint());

		SqlLitDouble    retVal = null;
		retVal = _factory.createLitDouble (arg.getValue());

//        Trace.message(this,"visit(LitDouble arg)", retVal.toSql(null));
		Trace.exit(this,"visit(LitDouble arg)", arg.pprint());
		return retVal;
	}

	/**
	@param arg
	@return org.xquark.extractor.sql.SqlExpression
	@roseuid 3BD177C60076
	 */
	public SqlExpression visit(LitFloat arg)
	{
		Trace.enter(this,"visit(LitFloat arg)");

		SqlLitFloat    retVal = null;
		retVal = _factory.createLitFloat (arg.getValue());

		Trace.exit(this,"visit(LitFloat arg)");
		return retVal;
	}

	/**
	@param arg
	@return org.xquark.extractor.sql.SqlExpression
	@roseuid 3BD177C60210
	 */
	public SqlExpression visit(LitInteger arg)
	{
		Trace.enter(this,"visit(LitInteger arg)");

		SqlLitInteger    retVal = null;
		retVal = _factory.createLitInteger (arg.getValue());

		Trace.exit(this,"visit(LitInteger arg)");
		return retVal;
	}

	public SqlExpression visit(LitList arg) {
		Trace.enter(this,"visit(LitList arg)");
		SqlLitList    retVal = null;
		List sExprList = new ArrayList(arg.getLitList().size());
		SqlExpression sExpr = null;
		for (int i = 0; i < arg.getLitList().size(); i++) {
			sExpr = ((Expression)arg.getLitList().get(i)).accept(this);
			sExprList.add(sExpr);
		}

		retVal = _factory.createLitList(sExprList);
		Trace.exit(this,"visit(LitList arg)");
		return retVal;
	}

	public SqlExpression visit(LitNull arg)
	{
		Trace.enter(this,"visit(LitNull arg)");

		SqlLitNull    retVal = _factory.createLitNull();

		Trace.exit(this,"visit(LitNull arg)");
		return retVal;
	}

	/**
	@param arg
	@return org.xquark.extractor.sql.SqlExpression
	@roseuid 3BD177C603B5
	 */
	public SqlExpression visit(LitString arg)
	{
		Trace.enter(this,"visit(LitString arg)");

		SqlLitString    retVal = null;
		retVal = _factory.createLitString (arg.getValue());

		Trace.exit(this,"visit(LitString arg)");
		return retVal;
	}

	/**
	@param arg
	@return org.xquark.extractor.sql.SqlExpression
	@roseuid 3BD177C70167
	 */
	public SqlExpression visit(RenameItem arg)
	{
		Trace.enter(this,"visit(RenameItem arg)");

		SqlRenameItem   retVal = null;
		SqlExpression   sExpr = arg.getOperand().accept(this);
		String          name  = arg.getName();
		retVal = _factory.createRenameItem (sExpr,name);

		Trace.exit(this,"visit(RenameItem arg)");
		return retVal;
	}

	/**
	@param arg
	@return org.xquark.extractor.sql.SqlExpression
	@roseuid 3BD177C7030C
	 */
	public SqlExpression visit(RenameRelation arg)
	{
		Trace.enter(this,"visit(RenameRelation arg)");

		SqlRenameRelation   retVal = null;
		SqlExpression   sExpr = arg.getOperand().accept(this);
		String          name  = arg.getUniqueName();
		retVal = _factory.createRenameRelation (sExpr,name);

		Trace.exit(this,"visit(RenameRelation arg)");
		return retVal;
	}

	/**
	@param arg
	@return org.xquark.extractor.sql.SqlExpression
	@roseuid 3BD177C800D3
	 */
	public SqlExpression visit(SortSpecification arg)
	{
		Trace.enter(this,"visit(SortSpecification arg)");

		Expression sortExpr = arg.getSortExpression();
//        if (sortExpr instanceof AttributeExpression) {
//            sortExpr = ((AttributeExpression)sortExpr).getUnderlyinExpr();
//        }
		SqlExpression   sqlSortExpr = sortExpr.accept(this);

		SqlSortSpecification   retVal = _factory.createSortSpecification(sqlSortExpr,arg.getSortDirection());

		Trace.exit(this,"visit(SortSpecification arg)");
		return retVal;
	}


	public SqlExpression visit(Table arg)
	{
		Trace.enter(this,"visit(Table arg)");

		SqlTable    retVal = _factory.createTable(arg.getCatalogName(), arg.getSchemaName(), arg.getName());

		Trace.exit(this,"visit(Table arg)");
		return retVal;
	}

	public SqlExpression visit(ConstantTempTable arg) {
		Trace.enter(this,"visit(ConstantTempTable arg)");

		SqlExpression retVal = null;
		List values = arg.getValues();
		Debug.assertTrue(null != values  && !values.isEmpty(), " TempTable is empty");

		SqlSelect   select = null;
		List        tuple = null;
		List        selectList = null;
		Expression  aAttr = null;
		SqlExpression sAttr = null;
		String      attrName = null;

		for (int i = 0; i < values.size(); i++) {
			select = _factory.createSelect(arg);

			tuple = (List)values.get(i);
			selectList = new ArrayList();
			for (int j = 0; j < tuple.size(); j++) {
				aAttr = (Expression)tuple.get(j);
				sAttr = aAttr.accept(this);
				if (0 ==i) {
					attrName = ((Attribute)arg.getAttributes().get(j)).getName();
					sAttr = _factory.createRenameItem(sAttr, attrName);
				}
				selectList.add(sAttr);
			}

			select.setSelectList(selectList);
			DummyTable dummyTable = new DummyTable();
			selectAddNextClause(select, dummyTable);
//			SqlExpression sTable = aDummyTable.accept(this);
//			if (null != sTable) {
//				select.addFromClause(dummyTable);
//			}

			if (null == retVal) {
				/* this select is the first select */
				retVal = select;
			}
			else {
				retVal = _factory.createBinOpUnion(Constants.UNION_ALL, retVal, select);
			}
		}

		Trace.exit(this,"visit(ConstantTempTable arg)");
		return retVal;
	}

	public SqlExpression visit(TempTable arg) {
		Trace.enter(this,"visit(TempTable arg)");
		Trace.exit(this,"visit(TempTable arg)");

		SqlExpression retVal = null;
		
		// TODO: create a select into (create table ... as) statement

		return retVal;
	}
	public SqlExpression visit(TempValue arg)
	{
		Trace.enter(this,"visit(TempValue arg)");

		SqlExpression   retVal = null;
		if (null != arg.getValue()) {
			retVal = arg.getValue().accept(this);
		}
		else {
			retVal = arg.getOperand().accept(this);
		}

		Trace.exit(this,"visit(TempValue arg)");
		return retVal;
	}

	/**
	@param arg
	@return org.xquark.extractor.sql.SqlExpression
	@roseuid 3BD177C90034
	 */
	public SqlExpression visit(UnOpAggregate arg)
	{
		Trace.enter(this,"visit(UnOpAggregate arg)");

		SqlSelect   retVal = _factory.createSelect(arg);
		_nestStack.push(retVal);
		retVal = selectAddSelectClause(retVal,arg);
		_nestStack.pop();

		Trace.exit(this,"visit(UnOpAggregate arg)");
		return retVal;
	}

	/**
	@param arg
	@return org.xquark.extractor.sql.SqlExpression
	@roseuid 3BD177C901EC
	 */
//    public SqlExpression visit(UnOpGroup arg)
//    {
//        Trace.enter(this,"visit(UnOpGroup arg)");
//
//        SqlSelect   retVal = _factory.createSelect();
//        retVal = selectAddGroupbyClause(retVal,arg);
//
//        Trace.exit(this,"visit(UnOpGroup arg)");
//        return retVal;
//    }

	/**
	@param arg
	@return org.xquark.extractor.sql.SqlExpression
	@roseuid 3BD177C90391
	 */
	public SqlExpression visit(UnOpMinus arg)
	{
		Trace.enter(this,"visit(UnOpMinus arg)");

		SqlExpression   sExpr = arg.getOperand().accept(this);
		SqlUnOpMinus    retVal = _factory.createUnOpMinus(sExpr);

		Trace.exit(this,"visit(UnOpMinus arg)");
		return retVal;
	}

	/**
	@param arg
	@return org.xquark.extractor.sql.SqlExpression
	@roseuid 3BD177CA016C
	 */
	public SqlExpression visit(UnOpNot arg)
	{
		Trace.enter(this,"visit(UnOpNot arg)");

		SqlExpression   sExpr = arg.getOperand().accept(this);
		SqlUnOpNot      retVal = _factory.createUnOpNot(sExpr);

		Trace.exit(this,"visit(UnOpNot arg)");
		return retVal;
	}

	/**
	@param arg
	@return org.xquark.extractor.sql.SqlExpression
	@roseuid 3BD177CA031A
	 */
	public SqlExpression visit(UnOpProject arg)
	{
		Trace.enter(this,"visit(UnOpProject arg)");

		SqlSelect   retVal = _factory.createSelect(arg);
		_nestStack.push(retVal);
		retVal = selectAddSelectClause(retVal,arg);
		_nestStack.pop();

		Trace.exit(this,"visit(UnOpProject arg)");
		return retVal;
	}

	/**
	@param arg
	@return org.xquark.extractor.sql.SqlExpression
	@roseuid 3BD177CB00F5
	 */
	public SqlExpression visit(UnOpRestrict arg)
	{
		Trace.enter(this,"visit(UnOpRestrict arg)");

		SqlSelect   retVal = _factory.createSelect(arg);
		_nestStack.push(retVal);
		if (arg.isPostResult())
			retVal = selectAddHavingClause(retVal, arg);
		else
			retVal = selectAddWhereClause(retVal, arg);
		_nestStack.pop();

		Trace.exit(this,"visit(UnOpRestrict arg)");
		return retVal;
	}

	/**
	@param arg
	@return org.xquark.extractor.sql.SqlExpression
	@roseuid 3BD177CB02AE
	 */
	public SqlExpression visit(UnOpSort arg)
	{
		Trace.enter(this,"visit(UnOpSort arg)");

		SqlSelect   retVal = _factory.createSelect(arg);
		_nestStack.push(retVal);
		selectAddSortClause(retVal,arg);
		_nestStack.pop();

		Trace.exit(this,"visit(UnOpSort arg)");
		return retVal;
	}


	/**
	 * Main switch for select statement building.
     * @param select
     * @param operand
     * @return
     */
    protected SqlSelect selectAddNextClause(SqlSelect select, Expression operand)
	{
		Trace.message(this,"selectAddFromClause (SqlSelect select, Expression operand)",operand);

		if (operand instanceof UnOpProject) {
			if (false == select.getSelectDistinct() && null == select.getSelectList()) {
				return selectAddSelectClause(select,(UnOpProject)operand);
			}
			else {
				SqlExpression  newSelect = operand.accept(this);
				select.addFromClause(newSelect);
				return select;
			}
		}
        if (operand instanceof UnOpRestrict) {
            UnOpRestrict restrict = (UnOpRestrict) operand;
            if (restrict.isPostResult())
                return selectAddHavingClause(select, (UnOpRestrict) operand);
            else
                return selectAddWhereClause(select, (UnOpRestrict) operand);
        }
		if (operand instanceof UnOpAggregate)   return selectAddSelectClause(select,(UnOpAggregate)operand);
		if (operand instanceof UnOpGroup)       return selectAddGroupbyClause(select,(UnOpGroup)operand);
		if (operand instanceof UnOpSort)        return selectAddSortClause(select,(UnOpSort)operand);
		if (operand instanceof Join)            return selectAddFromClause(select,(Join)operand);
		if (operand instanceof Table)           return selectAddFromClause(select,(Table)operand);
		if (operand instanceof DummyTable)      return selectAddFromClause(select,(DummyTable)operand);
		if (operand instanceof BinOpOuterJoin)  return selectAddFromClause(select,(BinOpOuterJoin)operand);
		if (operand instanceof BinaryAlgebra)   return selectAddFromClause(select,(BinaryAlgebra)operand);
		if (operand instanceof RenameRelation)  return selectAddFromClause(select,(RenameRelation)operand);

		Debug.assertTrue(false,"NYI!"+operand);
		return null;
	}

	protected SqlSelect selectAddSortClause(SqlSelect select, UnOpSort sort)
	{
		Trace.enter(this,"selectAddSortClause(SqlSelect select, UnOpSort sort)");

		List aSortList  = sort.getSortSpecificationList();
		if (null!=aSortList && null == select.getOrderbyClause()) {
			List sSortList  = new ArrayList();
			Iterator iter = aSortList.iterator();
			while (iter.hasNext()) {
				SortSpecification aSortSpecification = (SortSpecification)iter.next();
				SqlSortSpecification sSortSpecification = (SqlSortSpecification)aSortSpecification.accept(this);
				sSortList.add(sSortSpecification);
			}
			select.setOrderbyClause(sSortList);
		}
		select = selectAddNextClause(select,sort.getOperand());
		Trace.exit(this,"selectAddSortClause(SqlSelect select, UnOpSort sort)");
		return select;
	}


	/**
	 * Sets the attributes of the select clause and calls the method to add the
	 * from clause corresponding to the operand. 
     * @param select
     * @param project
     * @return
     */
    protected SqlSelect selectAddSelectClause(SqlSelect select, UnOpProject project)
	{
		Trace.enter(this,"selectAddSelectClause(SqlSelect select, UnOpProject project)",project);

		List aExprList = project.getItemList();
		ArrayList sExprList = new ArrayList();

		if (null != aExprList)
		{
			// translate item list in UnOpProject into SqlExpression
			Iterator  itr = aExprList.iterator () ;
			while ( itr.hasNext())
			{
				SqlExpression sExpr = ((Expression)itr.next()).accept (this);
				sExprList.add(sExpr) ;
			}
			select.setSelectClause ( project.getDistinct(),sExprList);
		}

		select = selectAddNextClause(select,project.getOperand());
		Trace.exit(this,"selectAddSelectClause(SqlSelect select, UnOpProject project)",project);
		return select;
	}


	protected SqlSelect selectAddFromClause(SqlSelect select, RenameRelation arg)
	{
		Trace.enter(this,"selectAddFromClause(SqlSelect select, RenameRelation arg)", arg);

		SqlRenameRelation sRR = null;

		String relationName = arg.getUniqueName();
		Expression aRelation = arg.getOperand();
		SqlExpression sRelation = null;
		if (aRelation instanceof  Table)
		{
			Table table = (Table)aRelation;
			sRelation = table.accept(this);
			sRR = _factory.createRenameRelation ( sRelation ,relationName);

		}
		else if  ( aRelation instanceof UnaryAlgebra)
		{
			//case : Project,Restrict,Sort,Group,...
			sRelation = aRelation.accept(this);
			sRR = _factory.createRenameRelation ( sRelation,relationName);
		}
		else if ( aRelation instanceof BinaryAlgebra)
		{
			// case : UNION, DIFFERENCE, EXCEPT
			sRelation = aRelation.accept(this);
			sRR = _factory.createRenameRelation ( sRelation,relationName);
		}
		else
		{
			throw new SqlWrapperException( MessageLibrary.getMessage("INT_LOG_ERR"),null) ;
		}

		select.addFromClause(sRR);

		Trace.exit(this,"selectAddFromClause(SqlSelect select, RenameRelation arg)", arg);
		return select;
	}


	protected SqlSelect selectAddFromClause (SqlSelect select, Join join)
	{
		Trace.enter(this,"selectAddFromClause (SqlSelect select, Join join)", join);

		/* Add it to the select from clause */
		select.addFromClause(join.accept(this));

		/*  add join predicate to WHERE clause */
		List aPredicateList = join.getPredicateList();
		if (null !=  aPredicateList) {
			Iterator iter = aPredicateList.iterator();
			while (iter.hasNext()) {
				Expression aPredicate = (Expression)iter.next();
				SqlExpression sPredicate = aPredicate.accept(this);
				SqlExpression newWhereClause = whereClauseAddPredicate(select.getWhereClause(), sPredicate);
				select.setWhereClause(newWhereClause);
			}
		}
		else {
			Trace.warn( this , "visit" , "join operation without predicate");
		}

		Trace.exit(this,"selectAddFromClause (SqlSelect select, Join join)", join);
		return select;
	}

	// TODO: this default implementation is for Oracle (7 because outer joins are
	// used). Move if mysql and progress use the same standard syntax ?
	protected SqlSelect selectAddFromClause (SqlSelect select, BinOpOuterJoin arg) {
		Trace.enter(this,"selectAddFromClause (SqlSelect select, BinOpOuterJoin arg)", arg);
		Expression [] relations = {arg.getLeftOperand(), arg.getRightOperand()};

		RenameRelation aRR;
		Expression      aRelation;
		SqlExpression   sRelation;
		SqlRenameRelation    sInnerTable = null;

		for (int i = 0; i < relations.length; i++) {
			/* generate FROM clause for SELECT */
			aRelation = relations[i];
			if (aRelation instanceof RenameRelation) {
				aRR = (RenameRelation)aRelation;
				aRelation = aRR.getOperand();
				
				// NOTE: a list of if (instanceof) used to check algebra integrity have
				// been removed because these tests make visitor automatic type switch worthless...
				sRelation = aRelation.accept(this);
				SqlRenameRelation sRR = _factory.createRenameRelation (sRelation, aRR.getUniqueName());
				select.addFromClause(sRR);
				
				if (LEFT_OUTER_JOIN == arg.getJoinType() && i == 1){
					sInnerTable = sRR;
				}
				else if (RIGHT_OUTER_JOIN == arg.getJoinType() && i == 0){
					sInnerTable = sRR;
				}
			}
			else {
				selectAddNextClause(select ,aRelation);
			}
		}

		/*  add join predicate to WHERE clause */
		List aPredicateList = arg.getPredicateList();
		if  (null !=  aPredicateList)
		{
			Iterator iter = aPredicateList.iterator();
			while (iter.hasNext()) {
				Expression aPredicate = (Expression)iter.next();
				SqlExpression sPredicate = aPredicate.accept(this);
				sPredicate = new SqlOuterJoinPredicate(sPredicate ,sInnerTable);
				SqlExpression newWhereClause = whereClauseAddPredicate(select.getWhereClause(), sPredicate);
				select.setWhereClause(newWhereClause);
			}
		}
		else
		{
			Trace.warn( this , "visit" , "outer join operation without predicate");
		}

		Trace.exit(this,"selectAddFromClause (SqlSelect select, BinOpOuterJoin arg)", arg);
		return select;
	}


	protected SqlSelect selectAddWhereClause (SqlSelect select, UnOpRestrict restrict)
	{
		Trace.enter(this,"selectAddWhereClause (SqlSelect select, UnOpRestrict restrict)", restrict);
		List aPredicates = restrict.getPredicateList();
		SqlExpression sPredicate ;

		if ( null != aPredicates ) {
			Iterator iter = aPredicates.iterator();
			while (iter.hasNext()) {
				Expression aPredicate = (Expression)iter.next();
				sPredicate = aPredicate.accept(this) ;
				SqlExpression newWhereClause = whereClauseAddPredicate(select.getWhereClause(), sPredicate);
				select.setWhereClause(newWhereClause);
			}
		}

		select = selectAddNextClause(select, restrict.getOperand());
		Trace.exit(this,"selectAddWhereClause (SqlSelect select, UnOpRestrict restrict)", restrict);
		return select;
	}

	protected SqlSelect selectAddHavingClause (SqlSelect select, UnOpRestrict restrict)
	{
		Trace.enter(this,"selectAddHavingClause (SqlSelect select, UnOpRestrict restrict)", restrict);
		List aPredicates = restrict.getPredicateList();
		SqlExpression sPredicate ;

		if ( null != aPredicates ) {
			Iterator iter = aPredicates.iterator();
			while (iter.hasNext()) {
				Expression aPredicate = (Expression)iter.next();
				sPredicate = aPredicate.accept(this) ;
				SqlExpression newWhereClause = whereClauseAddPredicate(select.getHavingClause(), sPredicate);
				select.setHavingClause(newWhereClause);
			}
		}

		select = selectAddNextClause(select, restrict.getOperand());
		Trace.exit(this,"selectAddHavingClause (SqlSelect select, UnOpRestrict restrict)", restrict);
		return select;
	}

	public SqlSelect selectAddGroupbyClause (SqlSelect select, UnOpGroup group)
	{
		Trace.enter(this,"selectAddGroupbyClause (SqlSelect select, UnOpGroup group)");

		List aExprList = group.getGroupExpressionList();
		ArrayList sExprList = new ArrayList();
		if (null != aExprList)
		{
			// translate group expression list in UnOpGroup into SqlExpression
			Iterator  itr = aExprList.iterator () ;
			while ( itr.hasNext())
			{
				SqlExpression sExpr = ((Expression)itr.next()).accept (this);
				sExprList.add(sExpr) ;
			}
			select.setGroupbyClause (sExprList);
			select = selectAddNextClause(select,group.getOperand());
		}

		Trace.exit(this,"selectAddGroupbyClause (SqlSelect select, UnOpGroup group)");
		return select;
	}

	protected SqlSelect selectAddFromClause (SqlSelect select, Table operand)
	{
		Trace.enter(this,"selectAddFromClause", operand);

		SqlTable newFrom = _factory.createTable (operand.getCatalogName(), operand.getSchemaName(), operand.getName());
		select.addFromClause(newFrom);

		Trace.exit(this,"selectAddFromClause", operand);
		return select;
	}

	protected SqlSelect selectAddFromClause (SqlSelect select, DummyTable operand)
	{
		Trace.enter(this,"selectAddFromClause (SqlSelect select, DummyTable operand)");

		Trace.exit(this,"selectAddFromClause (SqlSelect select, DummyTable operand)");
		return select;
	}

	protected SqlSelect selectAddFromClause (SqlSelect select, BinaryAlgebra operand)
	{
		Trace.enter(this,"selectAddFromClause (SqlSelect select, BinAlgebra operand)", operand);

		SqlExpression sRelation = operand.accept(this);
		SqlRenameRelation sRR = _factory.createRenameRelation ( sRelation,null);

		select.addFromClause(sRR);

		Trace.exit(this,"selectAddFromClause (SqlSelect select, BinAlgebra operand)", operand);
		return select;
	}

	public SqlExpression visit(AfTrim arg) throws SqlWrapperException
	{
		Trace.enter(this,"visit(AfTrim arg)");
		Expression arg0 = (Expression)arg.getArgument(0);
		SqlExpression retVal = _factory.createSfTrim(arg0.accept(this));
		Trace.exit(this,"visit(AfTrim arg)");
		return retVal;
	}

	public SqlExpression visit(XfCast arg) throws SqlWrapperException
	{
		Trace.enter(this,"visit(XfCast arg)");
		Expression expr = (Expression)arg.getExpression();
		SqlExpression sqlExpr = expr.accept(this);
		DataType type0= ((SqlTypeAtom)expr.getType()).getType();
		if (type0 instanceof DbUdt) {
			type0 = ((DbUdt)type0).getPrimitiveType();
		}
		DbPrimitiveType type1= (DbPrimitiveType)arg.getTargetType();
		SqlExpression retVal = _factory.createConvert(sqlExpr, (DbPrimitiveType)type0, type1);

		Trace.exit(this,"visit(XfCast arg)");
		return retVal;
	}

	public SqlExpression visit(AfCast arg) throws SqlWrapperException
	{
		Trace.enter(this,"visit(AfCast arg)");
		SqlExpression retVal = null;

		Expression expr = (Expression)arg.getExpression();


		DataType type0= ((SqlTypeAtom)expr.getType()).getType();
		if (type0 instanceof DbUdt) {
			type0 = ((DbUdt)type0).getPrimitiveType();
		}
		DbPrimitiveType type1= arg.getTargetType();

		String name = expr.getName();
		if (null != name){
			if (expr instanceof RenameItem) {
				expr = ((RenameItem)expr).getOperand();;
			}
			SqlExpression sqlExpr = expr.accept(this);
			retVal = _factory.createConvert(sqlExpr, (DbPrimitiveType)type0, type1);
			retVal = _factory.createRenameItem(retVal,name);
		}
		else {
			SqlExpression sqlExpr = expr.accept(this);
			retVal = _factory.createConvert(sqlExpr, (DbPrimitiveType)type0, type1);
		}

		Trace.exit(this,"visit(AfCast arg)");
		return retVal;
	}

	public SqlExpression visit(XfAbs arg) throws SqlWrapperException
	{
		Trace.enter(this,"visit(XfAbs arg)");
		Expression arg0 = (Expression)arg.getArgument(0);
		SqlExpression retVal = _factory.createSfAbs(arg0.accept(this));
		Trace.exit(this,"visit(XfAbs arg)");
		return retVal;
	}

	public SqlExpression visit(XfCeiling arg) throws SqlWrapperException
	{
		Trace.enter(this,"visit(XfCeiling arg)");
		Expression arg0 = (Expression)arg.getArgument(0);
		SqlExpression retVal = _factory.createSfCeiling(arg0.accept(this));
		Trace.exit(this,"visit(XfCeiling arg)");
		return retVal;
	}

	public SqlExpression visit(XfConcat arg)
	{
		Trace.enter(this,"visit(XfConcat arg)");

		SfConcat  retVal = _factory.createSfConcat();

		List  sqlArguments = new ArrayList();

		List  algArguments = arg.getArguments();
		Iterator iter = algArguments.iterator();
		while (iter.hasNext()) {
			  Expression item = (Expression)iter.next();
			  sqlArguments.add(item.accept(this));
		}

		retVal.setArguments (sqlArguments);
		Trace.exit(this,"visit(SfConcat arg)");
		return retVal;
	}


	public SqlExpression visit(XfContains arg) throws SqlWrapperException
	{
		Trace.enter(this,"visit(XfContains arg)");

		Debug.assertTrue(2 == arg.getArgumentNumber(),"2 == arg.getArgumentNumber()");
		SqlBinOpLike retVal = _factory.createBinOpLike();

		Expression aArg0 = (Expression)arg.getArgument(0);
		SqlExpression sArg0 = aArg0.accept(this);

		Expression aArg1 = (Expression)arg.getArgument(1);
		SqlExpression sArg1 = aArg1.accept(this);

		retVal.setLeftOperand(sArg0);

		SfConcat pattern = _factory.createSfConcat();
		pattern.setArgument(0, _factory.createLitString("%"));
		pattern.setArgument(1,sArg1);
		pattern.setArgument(2, _factory.createLitString("%"));

		retVal.setRightOperand(pattern);

		Trace.exit(this,"visit(XfContains arg)");
		return retVal;
	}

	public SqlExpression visit(XfCurrentDateTime arg) throws SqlWrapperException
	{
		Trace.enter(this,"visit(XfCurrentDateTime arg)");

		SfCurrentDateTime retVal = _factory.createSfCurrentDateTime();

		Trace.exit(this,"visit(XfCurrentDateTime arg)");
		return retVal;
	}

	public SqlExpression visit(XfDate arg) throws SqlWrapperException
	{
		Trace.enter(this,"visit(XfDate arg)");

		Expression arg0 = (Expression)arg.getArgument(0);
		DbPrimitiveType arg0Type = (DbPrimitiveType)((SqlTypeAtom)arg0.getType()).getType();

		SqlExpression aArg = arg0.accept(this);
		SqlConvert retVal = _factory.createConvert(aArg, arg0Type, new DbPrimitiveType(Types.DATE));

		Trace.exit(this,"visit(XfDate arg)");
		return retVal;
	}

	public SqlExpression visit(XfDateTime arg) throws SqlWrapperException
	{
		Trace.enter(this,"visit(XfDateTime arg)");

		Expression arg0 = (Expression)arg.getArgument(0);
		DbPrimitiveType arg0Type = (DbPrimitiveType)((SqlTypeAtom)arg0.getType()).getType();

		SqlExpression aArg = arg0.accept(this);
		SqlConvert retVal = _factory.createConvert(aArg, arg0Type, new DbPrimitiveType(Types.TIMESTAMP));

		Trace.exit(this,"visit(XfDateTime arg)");
		return retVal;
	}

	public SqlExpression visit(XfEndsWith arg) throws SqlWrapperException
	{
		Trace.enter(this,"visit(XfEndsWith arg)");

		Debug.assertTrue(2 == arg.getArgumentNumber(),"2 == arg.getArgumentNumber()");
		SqlBinOpLike retVal = _factory.createBinOpLike();

		Expression aArg0 = ((Expression)arg.getArgument(0));
		SqlExpression sArg0 = aArg0.accept(this);

		Expression aArg1 = (Expression)arg.getArgument(1);
		SqlExpression sArg1 = aArg1.accept(this);

		retVal.setLeftOperand(sArg0);

		SfConcat pattern = _factory.createSfConcat();
		pattern.setArgument(0, _factory.createLitString("%"));
		pattern.setArgument(1,sArg1);

		retVal.setRightOperand(pattern);

		Trace.exit(this,"visit(XfEndsWith arg)");
		return retVal;
	}


	public SqlExpression visit(XfFloor arg) throws SqlWrapperException
	{
		Trace.enter(this,"visit(XfFloor arg)");
		Expression arg0 = (Expression)arg.getArgument(0);
		SqlExpression retVal = _factory.createSfFloor(arg0.accept(this));
		Trace.exit(this,"visit(XfFloor arg)");
		return retVal;
	}


	public SqlExpression visit(XfLowerCase arg) throws SqlWrapperException
	{
		Trace.enter(this,"visit(XfLowerCase arg)");

		Debug.assertTrue(1 == arg.getArgumentNumber(),"1 == arg.getArgumentNumber()");
		SfLowerCase retVal = _factory.createSfLowerCase();

		Expression aArg = ((Expression)arg.getArgument(0));
		SqlExpression sArg = aArg.accept(this);
		retVal.setArgument(0,sArg);

		Trace.exit(this,"visit(XfLowerCase arg)");
		return retVal;
	}


	public SqlExpression visit(XfRound arg) throws SqlWrapperException
	{
		Trace.enter(this,"visit(XfRound arg)");
		Expression arg0 = (Expression)arg.getArgument(0);
		SqlExpression retVal = _factory.createBinOpArithmetic(PLUS_ARITHMETICS,
			arg0.accept(this), _factory.createLitFloat(new Float(0.5)));
		_factory.createSfFloor(retVal);
		Trace.exit(this,"visit(XfRound arg)");
		return retVal;
	}


	public SqlExpression visit(XfStartsWith arg) throws SqlWrapperException
	{
		Trace.enter(this,"visit(XfStartsWith arg)");

		Debug.assertTrue(2 == arg.getArgumentNumber(),"2 == arg.getArgumentNumber()");
		SqlBinOpLike retVal = _factory.createBinOpLike();

		Expression aArg0 = ((Expression)arg.getArgument(0));
		SqlExpression sArg0 = aArg0.accept(this);

		Expression aArg1 = (Expression)arg.getArgument(1);
		SqlExpression sArg1 = aArg1.accept(this);

		retVal.setLeftOperand(sArg0);

		SfConcat pattern = _factory.createSfConcat();
		pattern.setArgument(0,sArg1);
		pattern.setArgument(1, _factory.createLitString("%"));

		retVal.setRightOperand(pattern);

		Trace.exit(this,"visit(XfStartsWith arg)");
		return retVal;
	}


	public SqlExpression visit(XfStringLength arg) throws SqlWrapperException
	{
		Trace.enter(this,"visit(XfStringLength arg)");

		Debug.assertTrue(1 == arg.getArgumentNumber(),"1 == arg.getArgumentNumber()");
		SfStringLength retVal = _factory.createSfStringLength();

		Expression aArg = ((Expression)arg.getArgument(0));
		SqlExpression sArg = aArg.accept(this);
		retVal.setArgument(0,sArg);

		Trace.exit(this,"visit(XfStringLength arg)");
		return retVal;
	}


	public SqlExpression visit(XfSubString arg) throws SqlWrapperException
	{
		Trace.enter(this,"visit(XfSubString arg)");

		SfSubString retVal = _factory.createSfSubString();

		Expression aArg = ((Expression)arg.getArgument(0));
		SqlExpression sArg = aArg.accept(this);
		retVal.setArgument(0,sArg);

		aArg = (Expression)arg.getArgument(1);
		sArg = aArg.accept(this);
		retVal.setArgument(1,sArg);

		if (2 < arg.getArgumentNumber()) {
			aArg = (Expression)arg.getArgument(2);
			sArg = aArg.accept(this);
			retVal.setArgument(2,sArg);
		}

		Trace.exit(this,"visit(XfSubString arg)");
		return retVal;
	}

	public SqlExpression visit(XfTime arg) throws SqlWrapperException
	{
		Trace.enter(this,"visit(XfTime arg)");

		Expression arg0 = (Expression)arg.getArgument(0);
		DbPrimitiveType arg0Type = (DbPrimitiveType)((SqlTypeAtom)arg0.getType()).getType();

		SqlExpression aArg = arg0.accept(this);
		SqlConvert retVal = _factory.createConvert(aArg, arg0Type, new DbPrimitiveType(Types.TIME));

		Trace.exit(this,"visit(XfTime arg)");
		return retVal;
	}

	public SqlExpression visit(XfUpperCase arg) throws SqlWrapperException
	{
		Trace.enter(this,"visit(XfUpperCase arg)");

		Debug.assertTrue(1 == arg.getArgumentNumber(),"1 == arg.getArgumentNumber()");
		SfUpperCase retVal = _factory.createSfUpperCase();

		Expression aArg = ((Expression)arg.getArgument(0));
		SqlExpression sArg = aArg.accept(this);
		retVal.setArgument(0,sArg);

		Trace.exit(this,"visit(XfUpperCase arg)");
		return retVal;
	}

	protected SqlExpression whereClauseAddPredicate(SqlExpression whereClause, SqlExpression predicate)
	{
		Trace.enter(this,"whereClauseAddPredicate(SqlExpression whereClause, SqlExpression predicate)");
		SqlExpression retVal = null;
		if (whereClause != null) {
			retVal = _factory.createBinOpBoolean(AND, whereClause, predicate);
		}
		else {
			retVal = predicate;
		}
		Trace.exit(this,"whereClauseAddPredicate(SqlExpression whereClause, SqlExpression predicate)");
		return retVal;
	}


    public org.xquark.extractor.sql.SqlExpression visit(BinOpOuterJoin arg) {
    	Trace.enter(this, "visit(BinOpOuterJoin arg)");
    	Trace.enter(this,"visit(Join arg)");
    
    	SqlSelect   retVal = _factory.createSelect(arg);
    	_nestStack.push(retVal);
    	selectAddFromClause(retVal,arg);
    	_nestStack.pop();
    
    	Trace.enter(this, "visit(BinOpOuterJoin arg)");
    	return retVal;
    }

}
