/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */

package org.xquark.extractor.sql;

import org.xquark.extractor.common.Trace;

public class SqlBinOpArithmetic extends SqlBinaryOperator
{

	private static final String RCSRevision = "$Revision: 1.3 $";
	private static final String RCSName     = "$Name:  $";

    private int _operator;

    /**
    @param operator
    @param leftOperand
    @param rightOperand
    @roseuid 3BD177D1014E
     */
    public SqlBinOpArithmetic(int operator, SqlExpression leftOperand, SqlExpression rightOperand)
    {
        super (  leftOperand,  rightOperand );
        setOperator ( operator ) ;
    }

    /**
    @param operator
    @param leftOperator
    @param rightOperator
    @roseuid 3BD177D1011C
     */
    public SqlBinOpArithmetic()
    {

    }

    /**
    Access method for the _operator property.
    @return   the current value of the _operator property
    @roseuid 3BD177D102E8
     */
    public int getOperator()
    {
        return _operator;
    }

    /**
    Sets the value of the _operator property.
    @param aOperator the new value of the _operator property
    @roseuid 3BD177D1031A
     */
    public void setOperator(int aOperator)
    {
        _operator = aOperator;
    }

    /**
    @return String
    @roseuid 3BD177D1039D
     */
    public String toSql(Context context)
    {
        Trace.enter(this,"toSql(Context context)");
        StringBuffer retVal = new StringBuffer();

        String      lOprnd , rOprnd ;

        lOprnd = getLeftOperand().toSql(context) ;
        rOprnd = getRightOperand().toSql(context) ;

        int     optr = getOperator () ;
        if (optr == Constants.MODULO_ARITHMETICS)
		{
			retVal.append(ARITHMETICSSTRINGS [optr]);
			retVal.append('(');
			retVal.append(lOprnd);
			retVal.append(',');
			retVal.append(rOprnd);
			retVal.append(')');
		}
		else
        {
            retVal.append("(");
            retVal.append(lOprnd);
            retVal.append(") ");
            retVal.append(ARITHMETICSSTRINGS [optr]);
            retVal.append(" (");
            retVal.append(rOprnd);
            retVal.append(")");
        }

        Trace.exit(this,"toSql(Context context)", retVal);
        return retVal.toString();
    }
}
