/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */

package org.xquark.extractor.sql;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.xquark.extractor.common.Trace;


public class SqlExpression implements Constants
{

	private static final String RCSRevision = "$Revision: 1.3 $";
	private static final String RCSName     = "$Name:  $";


    /* the ExtractorStatement who creates this SqlExpression */
//    private ExtractorStatement _extractorStatement = null;
    private List _precedents = null;;

    private String _sql = null; // TODO: check that this field is not set somewhere before generation (2times)



    /**
    @roseuid 3BD177DE0247
     */
    public SqlExpression() {}

    public String getName()
    {
        /* this is a default implemetaion for most of subclasses of SqlExpression
        * SqlTable, SqlAttributeExpression, SqlRenameItem, SqlRenameRelation
        *re-implemente this function */
        return null;
    }

    /**
    Access method for the _precedents property.

    @return   the current value of the _precedents property
     */
    public List getPrecedents()
    {
        return _precedents;
    }

    /**
    Sets the value of the _precedents property.

    @param aPrecedents the new value of the _precedents property
     */
    public void setPrecedents(List aPrecedents)
    {
        //_precedents = precedents ;
        _precedents = aPrecedents;
    }


    /**
    @param precedent
    @roseuid 3BD177DE0387
     */
    public void appendPrecedent(SqlExpression precedent)
    {
        if (null == _precedents) {
            _precedents = new ArrayList();
        }
        _precedents.add(precedent);
    }

//
//    public void setExtractorStatement(ExtractorStatement extractorStatement) {
//        _extractorStatement = extractorStatement;
//        if ( null != _precedents )
//        {
//            Iterator iter = _precedents.iterator();
//            while (iter.hasNext()) {
//                ((SqlExpression)iter.next()).setExtractorStatement(extractorStatement);
//            }
//        }
//    }
//
//    public ExtractorStatement getExtractorStatement() {
//        return _extractorStatement;
//    }

    /**
    @param statement
    @return int
    @throws java.sql.SQLException
    @roseuid 3BD177DF00E0
     */
    public ResultSet Execute(Statement statement) throws SQLException
    {
        ResultSet retResultSet ;
        if ( null != _precedents )
        {
            Iterator iter = _precedents.iterator();
            while (iter.hasNext()) {
                ((SqlExpression)iter.next()).Execute( statement );
            }
        }
        String query = this.toSql();
        Trace.message(this,"Execute",query);

        retResultSet = statement.executeQuery(query) ;

        return  retResultSet;
    }

    /**
    @param statement
    @throws java.sql.SQLException
    @roseuid 3BD177DF02B6
     */
    public void cleanAll(Statement statement) throws SQLException
    {
        if ( null != _precedents )
        {
            Iterator iter = _precedents.iterator();
            while (iter.hasNext()) {
                ((SqlExpression)iter.next()).cleanAll(statement);
            }
        }
        clean (statement);
    }

    /**
    @param statement
    @throws java.sql.SQLException
    @roseuid 3BD177E00055
     */
    protected void clean(Statement statement) throws SQLException
    {

    }


   public String toSql() {
        if (null == _sql) {
            _sql = toSql(new Context());
        }
        return _sql;
   }

    /**
    @return String
    @roseuid 3BD177E001F0
     */
    public String toSql(Context context)
    {
        return toString () ;
    }


    public List generateRequestList() {
        List retVal = new ArrayList();
        generateRequestList(retVal);
        return retVal;
    }

    void generateRequestList(List requestList)
    {
        if (null != _precedents && ! _precedents.isEmpty()) {
            SqlExpression precedent = null;
            for (int i = 0; i < _precedents.size(); i++) {
                precedent = (SqlExpression) _precedents.get(i);
                precedent.generateRequestList(requestList);
            }
        }
        requestList.add(toSql());
    }

    public String pprint() {
        StringBuffer retVal = new StringBuffer();
        List requestList = generateRequestList();

        String request = null;
        for (int i = 0; i < requestList.size(); i++) {
            request = (String) requestList.get(i);
            retVal.append(request);
            retVal.append('\n');
        }

        return retVal.toString();
    }

	public String toString() {
		return toSql();
	}
}
