/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */

package org.xquark.extractor.sql;

import java.util.List;

import org.xquark.extractor.algebra.Expression;
import org.xquark.extractor.metadata.DbPrimitiveType;

abstract public interface SqlFactory {



	public SqlAttributeExpression createAttributeExpression();

	public SqlAttributeExpression createAttributeExpression(String tableInstance, String attribute);

	public SqlBinOpArithmetic createBinOpArithmetic();

	public SqlBinOpArithmetic createBinOpArithmetic(int operator, SqlExpression leftOperand, SqlExpression rightOperand);

	public SqlBinOpBoolean createBinOpBoolean();

	public SqlBinOpBoolean createBinOpBoolean(int operator, SqlExpression leftOperand, SqlExpression rightOperand);

	public SqlBinOpCompare createBinOpCompare();

	public SqlBinOpCompare createBinOpCompare(int operator, SqlExpression leftOperand, SqlExpression rightOperand);

	public SqlBinOpCompareAny createBinOpCompareAny() ;

	public SqlBinOpCompareAny createBinOpCompareAny(int operator, SqlExpression leftOperand, SqlExpression rightOperand);

	public SqlBinOpLike createBinOpLike();

	public SqlBinOpLike createBinOpLike(SqlExpression leftOperand, SqlExpression rightOperand, boolean not);

	public SqlBinOpUnion createBinOpUnion();

	public SqlBinOpUnion createBinOpUnion(int operator, SqlExpression leftOperand, SqlExpression rightOperand);

	public SqlConvert createConvert();

	public SqlConvert createConvert(SqlExpression expr, DbPrimitiveType originalType, DbPrimitiveType targetType);

	public SqlIfThenElse createIfThenElse();

	public SqlIfThenElse createIfThenElse(SqlExpression iif, SqlExpression tthen, SqlExpression eslse);

//    public SqlOuterJoinPredicate createOuterJoinPredicate();
//
//    public SqlOuterJoinPredicate createOuterJoinPredicate(SqlExpression predicate,org.xquark.extractor.sql.SqlRenameRelation innerTable);

	public SqlJoin createJoin();

	public SqlJoin createJoin(List operandList);

	public SqlUnOpExists createUnOpExists();

	public SqlUnOpExists createUnOpExists(SqlExpression query, boolean notExists);

	public SqlUnOpIsNull createUnOpIsNull();

	public SqlUnOpIsNull createUnOpIsNull(SqlExpression expr, boolean notIsNull);

	public SqlUnOpMinus createUnOpMinus();

	public SqlUnOpMinus createUnOpMinus(SqlExpression expr);

	public SqlUnOpNot createUnOpNot();

	public SqlUnOpNot createUnOpNot(SqlExpression expr);

	public SqlFunAggregate createFunAggregate();

	public SqlFunAggregate createFunAggregate(int operator, SqlExpression operand, boolean distinct);

	public SqlBinOpIn createBinOpIn();

	public SqlBinOpIn createBinOpIn(SqlExpression leftOperand, SqlExpression rightOperand);

	public SqlLitBoolean createLitBoolean();

	public SqlLitBoolean createLitBoolean(Boolean value);

	public SqlLitDate createLitDate();

	public SqlLitDate createLitDate(java.sql.Date value);

	public SqlLitDecimal createLitDecimal();

	public SqlLitDecimal createLitDecimal(java.math.BigDecimal value);

	public SqlLitDouble createLitDouble();

	public SqlLitDouble createLitDouble(Double value);

	public SqlLitFloat createLitFloat();

	public SqlLitFloat createLitFloat(Float value);

	public SqlLitInteger createLitInteger();

	public SqlLitInteger createLitInteger(Integer value);

	public SqlLitList createLitList();

	public SqlLitList createLitList(List litList);

	public SqlLitString createLitString();

	public SqlLitString createLitString(String value);

	public SqlLitNull createLitNull();

	public SqlSelect createSelect(Expression expr); // Added SR 12/08/03 because this is needed for
	// renaming attribute expressions when creating temporary tables

	public SqlSortSpecification createSortSpecification();

	public SqlSortSpecification createSortSpecification(SqlExpression sortExpression, int sortDirection);

	public SqlTable createTable();

	public SqlTable createTable(String name);

	public SqlTable createTable(String catalogName, String schemaName, String tableName);


	public SqlRenameItem createRenameItem();

	public SqlRenameItem createRenameItem(SqlExpression expr, String name);

	public SqlRenameRelation createRenameRelation();

	public SqlRenameRelation createRenameRelation(SqlExpression relation,String alias);

	/* create SQL Functions*/
	public SfAbs createSfAbs();

	public SfAbs createSfAbs(SqlExpression arg0);

	public SfCeiling createSfCeiling();

	public SfCeiling createSfCeiling(SqlExpression arg0);

	public SfConcat createSfConcat();

	public SfConcat createSfConcat(List list);

	public SfContains createSfContains();

	public SfContains createSfContains(SqlExpression arg0, SqlExpression arg1);

	public SfCurrentDateTime createSfCurrentDateTime();


	public SfFloor createSfFloor();

	public SfFloor createSfFloor(SqlExpression arg0);

	public SfLowerCase createSfLowerCase();

	public SfLowerCase createSfLowerCase(SqlExpression arg0);

	public SfRound createSfRound();

	public SfRound createSfRound(SqlExpression arg0);

	public SfSign createSfSign();

	public SfSign createSfSign(SqlExpression arg0);

	public SfStringLength createSfStringLength();

	public SfStringLength createSfStringLength(SqlExpression arg0);

	public SfSubString createSfSubString();

	public SfSubString createSfSubString(SqlExpression arg0, SqlExpression arg1);

//    public SfToChar createSfToChar();
//
//    public SfToChar createSfToChar(SqlExpression arg0);
//
//    public SfToDate createSfToDate();
//
//    public SfToDate createSfToDate(SqlExpression arg0, SqlExpression arg1);
//
//    public SfToNumber createSfToNumber();
//
//    public SfToNumber createSfToNumber(SqlExpression arg0);

	public SfTrim createSfTrim();

	public SfTrim createSfTrim(SqlExpression arg0);

	public SfUpperCase createSfUpperCase();

	public SfUpperCase createSfUpperCase(SqlExpression arg0);
}
