/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */

package org.xquark.extractor.sybase;

import java.sql.Timestamp;
import java.sql.Types;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.xquark.extractor.algebra.*;
import org.xquark.extractor.common.*;
import org.xquark.extractor.metadata.DbPrimitiveType;
import org.xquark.extractor.sql.*;
import org.xquark.extractor.sybase.sql.SqlCreateView;
import org.xquark.extractor.xfunctions.XfDate;
import org.xquark.extractor.xfunctions.XfDateTime;
import org.xquark.extractor.xfunctions.XfTime;
import org.xquark.schema.datatypes.PrimitiveType;

final public class SybaseGenSqlVisitor extends AbstractGenSqlVisitor {
	private static final String RCSRevision = "$Revision: 1.3 $";
	private static final String RCSName     = "$Name:  $";



	private int _nameCounter = 0;

	public SybaseGenSqlVisitor() {
		super((SqlFactory)SybaseFactory.getInstance());
	}

	public void reinit() {
		super.reinit();
	}

	public SqlExpression visit(BinOpArithmetic arg)
	{
		Trace.enter(this,"visit(BinOpArithmetic arg)");

		SqlExpression  retVal = null;

		Expression      aLOprnd = arg.getLeftOperand();
		SqlExpression   sLOprnd = aLOprnd.accept(this);

		Expression      aROprnd = arg.getRightOperand();
		SqlExpression   sROprnd = aROprnd.accept(this);

		int oprtr = arg.getOperator();

		if (MODULO_ARITHMETICS == oprtr ) {
			retVal = visitModulo(sLOprnd, (SqlTypeAtom)aLOprnd.getType(), sROprnd, (SqlTypeAtom)aROprnd.getType());
		}
		else {
			if ( DIVIDE_ARITHMETICS == oprtr &&
				aLOprnd.getType().isInteger() &&
				aROprnd.getType().isInteger() ) {
				/* Sybase will execute integer division while XQuery expect decimal result.
				   So we cast left operand to decimal type before the division */
				sLOprnd = _factory.createConvert(sLOprnd,
							new DbPrimitiveType(Types.INTEGER),
							new DbPrimitiveType(Types.DECIMAL));
			}
			retVal = _factory.createBinOpArithmetic(oprtr,sLOprnd, sROprnd);
		}

		Trace.exit(this,"visit(BinOpArithmetic arg)");
		return retVal;
	}

	protected SqlExpression	visitModulo(SqlExpression lOprnd, SqlTypeAtom lOprndType,
										SqlExpression rOprnd, SqlTypeAtom rOprndType) {
		SqlBinOpArithmetic  retVal = null;
		if (lOprndType.isInteger() && rOprndType.isInteger()) {
			retVal = _factory.createBinOpArithmetic(MODULO_ARITHMETICS,lOprnd,rOprnd);
		}
		else {
			/* sign(lOprnd) *  ( abs(lOprnd)- floor(abs(lOprnd/rOprnd))*abs(rOprnd) */

			SfSign  signL = _factory.createSfSign(lOprnd);

			SfAbs absL = _factory.createSfAbs(lOprnd);

			SqlBinOpArithmetic LdivR = _factory.createBinOpArithmetic(DIVIDE_ARITHMETICS,lOprnd, rOprnd);
			SfAbs absLdivR = _factory.createSfAbs(LdivR);
			SfFloor floorAbs = _factory.createSfFloor(absLdivR);
			SfAbs absR = _factory.createSfAbs(rOprnd);

			SqlBinOpArithmetic floorMultiR = _factory.createBinOpArithmetic(MULTIPLY_ARITHMETICS,floorAbs, absR);

			SqlBinOpArithmetic AbsMinusFloor = _factory.createBinOpArithmetic(MINUS_ARITHMETICS,absL, floorMultiR);

			retVal = _factory.createBinOpArithmetic(MULTIPLY_ARITHMETICS,signL, AbsMinusFloor);
		}
		return retVal;
	}

	public SqlExpression visit(XfTime arg) throws SqlWrapperException
	{
		Trace.enter(this,"visit(XfTime arg)");
		PrimitiveType pt = PrimitiveType.createType("time");
		Expression arg0 = arg.getArgument(0);
		String dateTimeString = ((LitString)arg0).getValue();
		Timestamp ts = null;
		try {
			ts = (Timestamp)pt.convert(dateTimeString, false);
		}
		catch (Exception ex) {
			new SqlWrapperException(ex.getMessage(), ex);
		}

		SimpleDateFormat formatter = new SimpleDateFormat ("hh:mm:ss.S");
		dateTimeString = formatter.format(ts);


		DbPrimitiveType arg0Type = (DbPrimitiveType)((SqlTypeAtom)arg0.getType()).getType();

		SqlExpression aArg = _factory.createLitString(dateTimeString);
		SqlConvert retVal = _factory.createConvert(aArg, arg0Type, new DbPrimitiveType(Types.DATE));

		Trace.exit(this,"visit(XfTime arg)");
		return retVal;
	}


//    public SqlExpression visit(UnOpAggregate arg)
//    {
//        Trace.enter(this,"visit(UnOpAggregate arg)");
//
//        SqlSelect   retVal = _factory.createSelect();
//        retVal = selectAddSelectClause(retVal,arg);
//
//        Trace.exit(this,"visit(UnOpAggregate arg)");
//        return retVal;
//    }
//
//    protected SqlSelect selectAddSelectClause(SqlSelect select, UnOpProject project)
//    {
//        Trace.enter(this,"selectAddSelectClause(SqlSelect select, UnOpProject project)",project);
//
//        List aExprList = project.getItemList();
//        ArrayList sExprList = new ArrayList();
//
//        if (null != aExprList)
//        {
//            // translate item list in UnOpProject into SqlExpression
//            Iterator  itr = aExprList.iterator () ;
//            while ( itr.hasNext())
//            {
//                SqlExpression sExpr = ((Expression)itr.next()).accept (this);
//                sExprList.add(sExpr) ;
//            }
//            select.setSelectClause ( project.getDistinct(),sExprList);
//        }
//
//        select = selectAddFromClause(select,project.getOperand());
//        Trace.exit(this,"selectAddSelectClause(SqlSelect select, UnOpProject project)",project);
//        return select;
//    }


	public SqlExpression visit(XfDate arg) throws SqlWrapperException
	{
		Trace.enter(this,"visit(XfDate arg)");
		PrimitiveType pt = PrimitiveType.createType("date");
		Expression arg0 = arg.getArgument(0);
		String dateTimeString = ((LitString)arg0).getValue();

		Timestamp ts = null;
		try {
			ts = (Timestamp)pt.convert(dateTimeString, false);
		}
		catch (Exception ex) {
			new SqlWrapperException(ex.getMessage(), ex);
		}

		SimpleDateFormat formatter = new SimpleDateFormat ("yyyy.MM.dd");
		dateTimeString = formatter.format(ts);


		DbPrimitiveType arg0Type = (DbPrimitiveType)((SqlTypeAtom)arg0.getType()).getType();

		SqlExpression aArg = _factory.createLitString(dateTimeString);
		SqlConvert retVal = _factory.createConvert(aArg, arg0Type, new DbPrimitiveType(Types.DATE));

		Trace.exit(this,"visit(XfDate arg)");
		return retVal;
	}

	public SqlExpression visit(XfDateTime arg) throws SqlWrapperException
	{
		Trace.enter(this,"visit(XfDateTime arg)");
		PrimitiveType pt = PrimitiveType.createType("dateTime");
		Expression arg0 = arg.getArgument(0);
		String dateTimeString = ((LitString)arg0).getValue();

		Timestamp ts = null;
		try {
			ts = (Timestamp)pt.convert(dateTimeString, false);
		}
		catch (Exception ex) {
			new SqlWrapperException(ex.getMessage(), ex);
		}

		SimpleDateFormat formatter = new SimpleDateFormat ("yyyy.MM.dd hh:mm:ss.S");
		dateTimeString = formatter.format(ts);


		DbPrimitiveType arg0Type = (DbPrimitiveType)((SqlTypeAtom)arg0.getType()).getType();

		SqlExpression aArg = _factory.createLitString(dateTimeString);
		SqlConvert retVal = _factory.createConvert(aArg, arg0Type, new DbPrimitiveType(Types.TIMESTAMP));

		Trace.exit(this,"visit(XfDateTime arg)");
		return retVal;
	}

	protected SqlSelect selectAddFromClause (SqlSelect select, DummyTable operand) {
		Trace.enter(this,"selectAddFromClause (SqlSelect select, DummyTable operand)");

		Trace.exit(this,"selectAddFromClause (SqlSelect select, DummyTable operand)");
		return select;
	}



	public SqlExpression visit(SortSpecification arg) {
		Trace.enter(this,"visit(SortSpecification arg)");
		SqlSortSpecification   retVal = null;
		Expression sortOperand = ((UnOpSort)arg.getFather()).getOperand();
		if (sortOperand instanceof BinaryAlgebra && !(sortOperand instanceof BinOpOuterJoin)) {
			Expression sortExpr = arg.getSortExpression();
	//        if (sortExpr instanceof AttributeExpression) {
	//            sortExpr = ((AttributeExpression)sortExpr).getUnderlyinExpr();
	//        }
			SqlExpression   sqlSortExpr = sortExpr.accept(this);

			retVal = _factory.createSortSpecification(sqlSortExpr,arg.getSortDirection());
		}
		else {
			Expression sortExpr = arg.getSortExpression();
			Expression underlyingExpr = null;
			if (sortExpr instanceof AttributeExpression) {
				underlyingExpr = ((AttributeExpression)sortExpr).getUnderlyinExpr();
				if (underlyingExpr instanceof AttributeExpression) {
					sortExpr = underlyingExpr;
				}
			}
			SqlExpression   sqlSortExpr = sortExpr.accept(this);

			retVal = _factory.createSortSpecification(sqlSortExpr,arg.getSortDirection());
		}
		Trace.exit(this,"visit(SortSpecification arg)");
		return retVal;
	}

	public SqlExpression visit(ConstantTempTable arg) {
		Trace.enter(this,"visit(ConstantTempTable arg)");
		SqlExpression retVal = null;

		/* 1. create a temporay table */
		String tableName = arg.getName();
		SqlTable table = _factory.createTable("tempdb", null, tableName);
		org.xquark.extractor.sybase.sql.SqlCreateTable createTable = new org.xquark.extractor.sybase.sql.SqlCreateTable();
		createTable.setTableName(table);

		List attrList  = arg.getAttributes();
		List attrDefList = new ArrayList();
		Attribute  attribute = null;
		org.xquark.extractor.sybase.sql.SqlAttributeDefination attrDef = null;
		DbPrimitiveType pType = null;
		for (int i = 0; i < attrList.size(); i++) {
			attribute = (Attribute)attrList.get(i);
			attrDef = new org.xquark.extractor.sybase.sql.SqlAttributeDefination();

			attrDef.setName(attribute.getName());
			pType = (DbPrimitiveType)((SqlTypeAtom)attribute.getType()).getType();
			attrDef.setType(TypeMapper.mapType(pType));
			if (0 == i) {
				attrDef.setPrimaryKey(true);
			}
			attrDefList.add(attrDef);
		}
		createTable.setAttrDefList(attrDefList);

		SqlExpression currentStatement = createTable;
		/* 2. Insert data to the new created table */
		List tupleList = arg.getValues();
		List tuple = null;
		org.xquark.extractor.sybase.sql.SqlInsert insert = null;
		org.xquark.extractor.sybase.sql.SqlValueList sqlValueList = null;
		List valueList = null;
		Expression aValue = null;
		SqlExpression sValue = null;
		for (int i = 0; i < tupleList.size(); i++) {
			tuple = (List)tupleList.get(i);
			/* create one INSERT statement. */
			valueList = new ArrayList();
			for (int j = 0; j < tuple.size(); j++) {
				aValue = (Expression)tuple.get(j);
				sValue = aValue.accept(this);
				valueList.add(sValue);
			}
			sqlValueList = new org.xquark.extractor.sybase.sql.SqlValueList(valueList);
			insert = new org.xquark.extractor.sybase.sql.SqlInsert(table,sqlValueList);
			insert.appendPrecedent(currentStatement);
			currentStatement = insert;
		}

		retVal = currentStatement;
		Trace.exit(this,"visit(ConstantTempTable arg)");
		return retVal;
	}


	/* Do nothing: desactivate the inherited rename relation creation for
	 * relation binary operations.
     * @see org.xquark.extractor.sql.AbstractGenSqlVisitor#selectAddFromClause(org.xquark.extractor.sql.SqlSelect, org.xquark.extractor.algebra.BinaryAlgebra)
     */
    protected SqlSelect selectAddFromClause (SqlSelect select, BinaryAlgebra operand)
	{
		Trace.enter(this,"selectAddFromClause (SqlSelect select, BinAlgebra operand)",operand.toString());

		Debug.nyi("SqlSelect selectAddFromClause (SqlSelect select, BinaryAlgebra operand)");

		Trace.exit(this,"selectAddFromClause (SqlSelect select, BinAlgebra operand)",operand.toString());
		return select;
	}


	/* Do nothing: desactivate the inherited process.
	 * @see org.xquark.extractor.sql.AbstractGenSqlVisitor#selectAddFromClause(org.xquark.extractor.sql.SqlSelect, org.xquark.extractor.algebra.BinaryAlgebra)
	 */
	public SqlSelect selectAddFromClause (SqlSelect select, BinOpOuterJoin arg)
	{
		Trace.enter(this,"selectAddFromClause (SqlSelect select, BinOpOuterJoin arg)", arg.pprint());

		Debug.nyi("SqlSelect selectAddFromClause (SqlSelect select, BinaryAlgebra operand)");

		Trace.exit(this,"selectAddFromClause (SqlSelect select, BinOpOuterJoin arg)", arg.pprint());
		return select;
	}


	protected SqlSelect selectAddFromClause(SqlSelect select, RenameRelation arg)
	{
		Trace.enter(this,"selectAddFromClause(SqlSelect select, RenameRelation arg)",arg.pprint());

		SqlRenameRelation sRR = null;

		String relationName = arg.getUniqueName();
		Expression aRelation = arg.getOperand();
		SqlExpression sRelation = null;
		if  (aRelation instanceof ConstantTempTable) {
			ConstantTempTable tempTable = (ConstantTempTable)aRelation;
			SqlExpression sTempTable = tempTable.accept(this);
			select.appendPrecedent(sTempTable);
			sRelation = _factory.createTable ("tempdb", null, tempTable.getName());
			sRR = _factory.createRenameRelation ( sRelation ,relationName);
		}
		else if(aRelation instanceof Table) {
			Table table = (Table)aRelation;
			sRelation = _factory.createTable (table.getCatalogName(), table.getSchemaName(), table.getName());
			sRR = _factory.createRenameRelation ( sRelation ,relationName);

		}
		else if  ( aRelation instanceof UnOpProject)  {
			sRelation = aRelation.accept(this);
			String tempViewName = getTemporaryViewName();
			SqlCreateView sCreateView = new SqlCreateView(tempViewName,sRelation);
			select.appendPrecedent(sCreateView);
			SqlTable tempView = _factory.createTable(tempViewName);
			sRR = _factory.createRenameRelation(tempView,relationName);
		}

		else if  ( aRelation instanceof UnaryAlgebra)  {
			//case : ,Restrict,Sort,Group,...
			sRelation = aRelation.accept(this);
			sRR = _factory.createRenameRelation ( sRelation,relationName);
		}
		else if ( aRelation instanceof BinaryAlgebra)   {
			// case : UNION, DIFFERENCE, EXCEPT
			sRelation = aRelation.accept(this);
			sRR = _factory.createRenameRelation ( sRelation,relationName);
		}
		else  {
			throw new SqlWrapperException( MessageLibrary.getMessage("INT_LOG_ERR"),null) ;
		}

		select.addFromClause(sRR);

		Trace.exit(this,"selectAddFromClause(SqlSelect select, RenameRelation arg)",arg.pprint());
		return select;
	}

	protected SqlSelect selectAddSelectClause(SqlSelect select, UnOpProject project)
	{
		Trace.enter(this,"selectAddSelectClause(SqlSelect select, UnOpProject project)",project);

		List aExprList = project.getItemList();
		ArrayList sExprList = new ArrayList();

		boolean isNestedSelect = _nestStack.size()>1;
		boolean isAtomicValue = project.getType().isAtom();

		if (null != aExprList)
		{
			// translate item list in UnOpProject into SqlExpression
			Iterator  itr = aExprList.iterator () ;
			while ( itr.hasNext())
			{

				Expression aExpr = (Expression)itr.next();

				if (isNestedSelect && isAtomicValue& aExpr instanceof RenameItem) {
					aExpr = ((RenameItem)aExpr).getOperand();
				}
				SqlExpression sExpr = aExpr.accept (this);
				sExprList.add(sExpr) ;
			}
			select.setSelectClause ( project.getDistinct(),sExprList);
		}

		select = selectAddNextClause(select,project.getOperand());
		Trace.exit(this,"selectAddSelectClause(SqlSelect select, UnOpProject project)",project);
		return select;
	}

	 protected String getTemporaryViewName() {
		return "extractorTempView" + Integer.toString(_nameCounter++) ;
	}




}
