/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */

package org.xquark.extractor.xfunctions;

import java.util.List;

import org.xquark.extractor.algebra.Expression;
import org.xquark.extractor.common.Trace;
import org.xquark.extractor.metadata.DbPrimitiveType;
import org.xquark.extractor.sql.SqlExpression;

public final class XfCast extends Expression {

	private static final String RCSRevision = "$Revision: 1.3 $";
	private static final String RCSName     = "$Name:  $";


    Expression _expression = null;
    DbPrimitiveType _targetType = null;

    public XfCast()
    {
    }

    public XfCast(Expression arg, DbPrimitiveType targetType)
    {
        setExpression(arg);
        setTargetType(targetType);
    }

    public Expression getExpression()
    {
        return _expression;
    }

    public void setExpression(Expression expression)
    {
        _expression = expression;
        _expression.setFather(this);
    }

    public DbPrimitiveType getTargetType()
    {
        return _targetType;
    }

    public void setTargetType(DbPrimitiveType targetType)
    {
        _targetType = targetType;
    }

    public String getName()
    {
        return null;
    }

    public List getOperands()
    {
        return null;
    }

    public boolean replaceChild(Expression oldChild, Expression newChild)
    {
        Trace.enter(this, "replaceChild(Expression oldChild, Expression newChild)");
        boolean retVal = false;

        if (getExpression().equals(oldChild)) {
            setExpression(newChild);
            retVal = true;
        }

        Trace.exit(this, "replaceChild(Expression oldChild, Expression newChild)");
        return retVal;
    }

    public String pprint()
    {
        StringBuffer retVal = new StringBuffer();
        retVal.append("XfCast(");
        retVal.append(getExpression().pprint());
        retVal.append(", ");
        retVal.append(getTargetType().pprint());
        retVal.append(")");
        return retVal.toString();
    }

    public SqlExpression accept(org.xquark.extractor.algebra.GenSqlVisitor visitor)
    {
        return visitor.visit(this);
    }

    public void accept(org.xquark.extractor.algebra.AlgebraVisitor visitor)
    {
        visitor.visit(this);
    }

	/**
	 * @see Expression#deepEquals(Object)
	 */
	public boolean deepEquals(Object o)
	{
		if (o instanceof XfCast)
		{
			XfCast cast = (XfCast)o;
			return	 _expression.deepEquals(cast.getExpression())
					&& _targetType.equals(cast.getTargetType());
		}
		return false;
	}
}

