/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */

package org.xquark.extractor.xfunctions;

import java.util.List;

import org.xquark.extractor.common.Debug;
import org.xquark.extractor.sql.SqlExpression;
/**
3.5.2 xf:concat
3.5.2.1 Signature
xf:concat(string $headString, string* $otherStrings) => string
3.5.2.2 Parameters
$headString  static type is string
$otherStrings  static type is string*
3.5.2.3 Semantics
Returns the string that is the concatenation of the values of its arguments. XPath allows a
variable number of arguments. The resulting string might not be normalized in any Unicode or W3C
normalization.
NOTE:
The concat() function is specified to allow an arbitrary number of string arguments that are
concatenated together. This capability is retained for compatibility with [XPath 1.0] and is the
only function specified in this document that has that characteristic.
3.5.2.4 Examples
xf:concat('abc', 'def') returns "abcdef".
xf:concat('abc') returns abc.
xf:concat('abc', 'def', 'ghi', 'jkl', 'mno') returns "abcdefghijklmno".
 */

public final class XfConcat extends Xfunction_String
{
	private static final String RCSRevision = "$Revision: 1.3 $";
	private static final String RCSName     = "$Name:  $";


	/**
	@roseuid 3BD14D1E02B2
	 */
	public XfConcat()
	{
		super("XfConcat");
	}

	/**
	@param arguments
	@roseuid 3BD1465C017E
	 */
	public XfConcat(List arguments)
	{
		super("XfConcat");
		setArguments(arguments);
		Debug.assertTrue(1<arguments.size(),"1<arguments.size()");
	}

	public SqlExpression accept(org.xquark.extractor.algebra.GenSqlVisitor visitor)
	{
		return visitor.visit(this);
	}

	public void accept(org.xquark.extractor.algebra.AlgebraVisitor visitor)
	{
		visitor.visit(this);
	}
}
