/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */

package org.xquark.extractor.xfunctions;

import org.xquark.extractor.algebra.Expression;
import org.xquark.extractor.sql.SqlExpression;

/**
6.2.2 xf:dateTime
xf:dateTime(string $srcval) => dateTime
If the value of $srcval conforms to the lexical representation of a dateTime as defined in [XML
Schema Part 2: Datatypes], the constructor returns the dateTime corresponding to that
representation. Otherwise, the constructor returns the error value as defined in [XQuery 1.0 and
XPath 2.0 Data Model].

6.2.2.1 Examples
xf:dateTime("1999-05-31T05:00:00") returns a dateTime value corresponding to the 31st. of May, 1999
at 5:00 AM in an unspecified timezone.

xf:dateTime("1999-05-31T13:20:00-05:00") returns a dateTime value corresponding to 1:20 pm on May
the 31st, 1999 for a timezone which is 5 hours behind Coordinated Universal Time (UTC).
 */
public final class XfDateTime extends XFunction
{

	private static final String RCSRevision = "$Revision: 1.3 $";
	private static final String RCSName     = "$Name:  $";


    /**
    @roseuid 3BD1922C00FF
     */
    public XfDateTime()
    {
        super("XfDate");
    }

    /**
    @param dateTimeString
    @roseuid 3BD1914B02F0
     */
    public XfDateTime(Expression arg0)
    {
        super("XfDate");
        setArgument(0, arg0);
    }

    /**
    @param visitor
    @return org.xquark.extractor.sql.SqlExpression
    @throws org.xquark.extractor.common.SqlWrapperException
    @roseuid 3BD1922C011D
     */
    public SqlExpression accept(org.xquark.extractor.algebra.GenSqlVisitor visitor)
    {
        return visitor.visit(this);
    }

    public void accept(org.xquark.extractor.algebra.AlgebraVisitor visitor)
    {
        visitor.visit(this);
    }
}
