/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */

package org.xquark.extractor.xfunctions;

import org.xquark.extractor.algebra.Expression;
import org.xquark.extractor.sql.SqlExpression;

/**
2.6.1 xf:floor
2.6.1.1 Signature
xf:floor(double $srcval) => integer

2.6.1.2 Parameters
$srcval  static type is double

2.6.1.3 Semantics
Returns the largest (closest to positive infinity) number that is not greater than the value of
$srcvaland that is an integer.

2.6.1.4 Examples
xf:floor(10.5) returns 10.

xf:floor(-10.5) returns -11.
 */
public final class XfFloor extends Xfunction_Integer
{

	private static final String RCSRevision = "$Revision: 1.3 $";
	private static final String RCSName     = "$Name:  $";

    public XfFloor()
    {
      super("XfFloor");
    }

    public XfFloor(Expression arg )
    {
      super("XfFloor");
      setArgument(0,arg);
    }

    public SqlExpression accept(org.xquark.extractor.algebra.GenSqlVisitor visitor)
    {
        return visitor.visit(this);
    }

    public void accept(org.xquark.extractor.algebra.AlgebraVisitor visitor)
    {
        visitor.visit(this);
    }
}
