#!/bin/sh
# ----------------------------------------------------------------
# 	xqfusion - Shell-script for running XQuark Fusion
# ----------------------------------------------------------------

# Platform check

case "`uname`" in
	CYGWIN | Darwin)
		echo "Windows environment is not supported by this script."
		echo "Please use xqfusion.bat instead."
		exit 1
		;;
	*)
		if [ -z "$JAVA_HOME" ]
		then
			JAVA_HOME=${JAVA_HOME}
		fi
		;;
esac

# XQuark Fusion home setting

if [ -z "$XQFUSION_HOME" ]
then
	scriptdir=`pwd`
	XQFUSION_HOME=`dirname "$0"`/..

	# make XQFUSION_HOME a fully-qualified, straight path
	XQFUSION_HOME=`cd "$XQFUSION_HOME" && pwd`

	cd $scriptdir
fi

# Locate JVM

if [ -n "$JAVA_HOME"  ]
then
	JAVA="$JAVA_HOME/bin/java"
else
    JAVA=java
	echo "WARNING: JAVA_HOME not set. Trying with java found in the execution path."
fi
 
if [ ! -x "$JAVA" ]
then
  echo "ERROR: ${JAVA} cannot be executed. Please check JAVA_HOME value."
  exit 1
fi

# Set classpath

CP=$XQFUSION_HOME/bin/:$XQFUSION_HOME/lib/xquark-fusion.jar:.
if [ -n "$XQ_CLASSPATH" ]
then
	CP=$CP:$XQ_CLASSPATH
fi
if [ -n "$CLASSPATH" ]
then
	CP=$CP:$CLASSPATH
fi

# Third-party libraries
CP=$CP:$XQFUSION_HOME/lib/crimson.jar
CP=$CP:$XQFUSION_HOME/lib/mysql-connector-java-3.0.6-stable-bin.jar

# Run XQuark Fusion

"$JAVA" -classpath "$CP" org.xquark.studio.fusion.FusionFrame "$@"
