/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */

package org.xquark.mediator.DOMUtils;

import java.util.ArrayList;
import java.util.HashMap;

import org.xquark.xpath.datamodel.TypedNode;
import org.xquark.xquery.parser.*;

public class BufferTuple extends ArrayList {
	// **********************************************************************
	// * VERSIONING
	// **********************************************************************
	private static final String RCSRevision = "$Revision: 1.4 $";
	private static final String RCSName = "$Name:  $";
	// **********************************************************************
	// * CLASS VARIABLES
	// **********************************************************************
	/**
	 * List of List of String.
	 * A list of String represent a path.
	 */
	private ArrayList stepslist = null;
	private ArrayList pathlist = null;
	private HashMap indexpaths = null;
	private long identifier = 0;
	private ArrayList indexes = null; // list of SortedKeyMap
	private ArrayList indexesSpec = null; // list of ArrayList of items to be indexed
	private int idsize = 0;

	// ***********************************************************************
	// * INITIALIZATION
	// ***********************************************************************
	/**
	 * Allocate a new BufferTuple.
	 * The String paths are decomposed in arraylist of strings representing
	 * steps
	 */
	public BufferTuple(ArrayList list, int idsize) {
		if (list == null)
			return;
		this.idsize = idsize;
		stepslist = new ArrayList(list.size());
		pathlist = new ArrayList(list.size());
		indexpaths = new HashMap(list.size());

		for (int i = 0; i < list.size(); i++) {
			XQueryExpression tmpExpr = (XQueryExpression) list.get(i);
			String tmpExprStr = tmpExpr.toSimpleString(); // to avoid braces !!!
			ArrayList tmpList = new ArrayList();
			if (tmpExpr instanceof LocatedExpression) {
				LocatedExpression tmpLocExpr = (LocatedExpression) tmpExpr;
				for (int j = 0; j < tmpLocExpr.getSteps().size(); j++) {
					tmpList.add(((Step) tmpLocExpr.getStep(j)).getExpression().toString());
				}
			} else if (tmpExpr instanceof Variable)
				tmpList.add(tmpExprStr);
			stepslist.add(tmpList);
			pathlist.add(tmpExprStr);
			indexpaths.put(tmpExprStr, new Integer(i));
		}
	}

	// ***********************************************************************
	// * METHODS
	// ***********************************************************************
	public long getIdentifier() {
		identifier++;
		return identifier - 1;
	}
	public int getPathIndex(String path) {
		if (indexpaths == null)
			return -1;
		Integer idx = (Integer) indexpaths.get(path);
		if (idx == null)
			return -1;
		return idx.intValue();
	}
	public HashMap getIndexPath() {
		return indexpaths;
	}
	public ArrayList getPathList() {
		return pathlist;
	}

	// modified methods to accomodate with indexes if any
	public void add(Tuple tuple) {
		super.add(tuple);
		tuple.setParent(this);
		// calculate indexes if any
		fillIndexes(tuple);
	}
	public void add(int index, Tuple tuple) {
		super.add(index, tuple);
		tuple.setParent(this);
		// calculate indexes if any
		fillIndexes(tuple);
	}
	private void fillIndexes(Tuple tuple) {
		if (this.indexesSpec != null && !this.indexesSpec.isEmpty()) {
			//org.xquark.xml.misc.Debug.print("tuple " + tuple.toCompleteString());
			if (this.indexes == null)
				this.indexes = new ArrayList(indexesSpec.size());
			for (int i = 0; i < this.indexesSpec.size(); i++) {
				ArrayList indexspeci = (ArrayList) this.indexesSpec.get(i);
				TupleKey valuelist = new TupleKey(indexspeci.size(), ((XQueryExpression) indexspeci.get(indexspeci.size() - 1)).getOrder()[0]);
				for (int j = 0; j < indexspeci.size(); j++) {
					XQueryExpression indexspecistrj = (XQueryExpression) indexspeci.get(j);
					ArrayList tmpnodes = tuple.getPath(indexspecistrj.toString());
					if (tmpnodes != null)
						valuelist.add(((TypedNode) tmpnodes.get(0)).getExtendedTypedValue());
				}
				if (valuelist.isEmpty()) 
					continue;
				if (this.getIndex(i) == null)
					this.addIndex(new SortedKeyMap());
				SortedKeyMap sortedmap = this.getIndex(i);
				ArrayList tuplelist = (ArrayList) sortedmap.get(valuelist);
				if (tuplelist == null) {
					tuplelist = new ArrayList(1);
					sortedmap.put(valuelist, tuplelist);
				}
				tuplelist.add(tuple);
			}
		}
	}

	public void removeLast() {
		super.remove(size() - 1);
	}
	//----------------------------------------------------
	public int getPos(String path) {
		for (int i = 0; i < pathlist.size(); i++) {
			String pathi = (String) pathlist.get(i);
			if (pathi.equals(path))
				return i;
		}
		return -1;
	}
	public Tuple newTuple() {
		Tuple newTuple = new Tuple(stepslist.size(), idsize);
		newTuple.setParent(this);
		return newTuple;
	}
	//	public Tuple newNullTuple() {
	//		return new Tuple(stepslist.size(),idsize);
	//	}

	// methods for indexes specification handling
	public void setIndexesSpec(ArrayList indexesSpec) {
		this.indexesSpec = indexesSpec;
	}
	public ArrayList getIndexesSpec() {
		return this.indexesSpec;
	}
	public ArrayList getIndexSpec(int index) {
		return (ArrayList) this.indexesSpec.get(index);
	}
	public int addIndexSpec(ArrayList list) {
		if (this.indexesSpec == null)
			this.indexesSpec = new ArrayList();
		this.indexesSpec.add(list);
		return this.indexesSpec.size();
	}
	// methods for indexes handling
	public ArrayList getIndexes() {
		return this.indexes;
	}
	public SortedKeyMap getIndex(int index) {
		if (this.indexes == null)
			return null;
		if (this.indexes.size() <= index)
			return null;
		return (SortedKeyMap) this.indexes.get(index);
	}
	public int addIndex(SortedKeyMap map) {
		if (this.indexes == null)
			this.indexes = new ArrayList();
		this.indexes.add(map);
		return this.indexes.size();
	}
	// ***********************************************************************
	// * DEBUG
	// ***********************************************************************
	/**
	 *
	 */
	public String toString() {
		StringBuffer sb = new StringBuffer();
		for (int i = 0; i < stepslist.size(); i++) {
			sb.append(stepslist.get(i));
			sb.append("\t");
		}
		sb.append("\n---------------------------------------------------\n");
		for (int i = 0; i < size(); i++) {
			sb.append(((Tuple) get(i)).toCompleteString());
			sb.append("\n-----------------------------------------------\n");
		}
		return sb.toString();
	}
}
