/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * Copyright (C) 2003 XQuark Group.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */

package org.xquark.mediator.DOMUtils;

import java.math.BigDecimal;
import java.sql.Timestamp;
import java.util.*;

import org.w3c.dom.Attr;
import org.w3c.dom.Node;
import org.xquark.schema.SchemaException;
import org.xquark.schema.SchemaManager;
import org.xquark.schema.SimpleType;
import org.xquark.xpath.datamodel.*;
import org.xquark.xquery.parser.*;
import org.xquark.xquery.parser.primitivefunctions.fnfunctions.*;
import org.xquark.xquery.parser.primitivefunctions.myfunctions.FunctionABS;
import org.xquark.xquery.parser.primitivefunctions.xsfunctions.*;
import org.xquark.xquery.parser.util.Constants;

/**
 * A simple class for evaluating en attendant the real evaluator.
 * evaluate the expression and return true or false
 *
 */
public class EvaluationVisitor extends DefaultParserVisitor {
	// **********************************************************************
	// * VERSIONING
	// **********************************************************************
	private static final String RCSRevision = "$Revision: 1.6 $";
	private static final String RCSName = "$Name:  $";
	// **********************************************************************
	// * CLASS VARIABLES
	// **********************************************************************
	private BufferTuple buffertuple = null;
	private ArrayList tuplelist = null;
	private Tuple tuple = null;
	//private XDBCResultSetInterface resultSet = null;
	
	private boolean verdict = false;

	//private BufferTuple resultbuffertuple = null;
	private Tuple resulttuple = null;
	private ArrayList resnodes = null;
	private Comparable resvalue = null;

	protected static TypedDocumentImpl docimpl = null;
	static {
		docimpl = new TypedDocumentImpl();
	}
	private SchemaManager schemamanager = null;

	public static Integer UNKNOWN_TYPE = new Integer(-1);
	public static Integer BOOLEAN_TYPE = new Integer(0);
	public static Integer VALUE_TYPE = new Integer(1);
	public static Integer NODE_TYPE = new Integer(2);
	public static Integer TUPLE_TYPE = new Integer(3);
	public static Integer BUFFER_TYPE = new Integer(4);
	private Stack returnType = new Stack();

	/**
	 *
	 */
	public EvaluationVisitor(SchemaManager schemamanager) {
		this.schemamanager = schemamanager;
	}

	/**
	 *
	 */
	public EvaluationVisitor(SchemaManager schemamanager, Tuple tuple) {
		this.schemamanager = schemamanager;
		this.tuple = tuple;
	}

	/**
	 *
	 */
	public EvaluationVisitor(SchemaManager schemamanager, ArrayList tuplelist) {
		this.schemamanager = schemamanager;
		this.tuplelist = tuplelist;
	}

	/**
	 *
	 */
//	public EvaluationVisitor(SchemaManager schemamanager, BufferTuple buffertuple) {
//		this.schemamanager = schemamanager;
//		this.tuple = null;
//		this.buffertuple = buffertuple;
//	}

	public void setReturnType(Integer setType) {
		if (!returnType.isEmpty())
			this.returnType.pop();
		returnType.push(setType);
	}

	private void clear() {
		//resultbuffertuple = null;
		resulttuple = null;
		resnodes = null;
		resvalue = null;
		verdict = false;
	}
	private void setResultTuple(Tuple tuple) {
		//resultbuffertuple = null;
		resulttuple = tuple;
		resnodes = null;
		resvalue = null;
		verdict = false;
	}
	private void setResultBufferTuple(BufferTuple buffertuple) {
		//resultbuffertuple = buffertuple;
		resulttuple = null;
		resnodes = null;
		resvalue = null;
		verdict = false;
	}
	private void setResnodes(ArrayList nodes) {
		//resultbuffertuple = null;
		resulttuple = null;
		resnodes = nodes;
		resvalue = null;
		verdict = false;
	}
	private void setValue(Comparable comp) {
		//resultbuffertuple = null;
		resulttuple = null;
		resnodes = null;
		resvalue = comp;
		verdict = false;
	}
	private void setVerdict(boolean bool) {
		//resultbuffertuple = null;
		resulttuple = null;
		resnodes = null;
		resvalue = null;
		verdict = bool;
	}
	public void setSchemaManager(SchemaManager schemamanager) {
		this.schemamanager = schemamanager;
	}
	public void reset() {
		clear();
		this.tuple = null;
		this.buffertuple = null;
		this.tuplelist = null;
		returnType.clear();
	}
	public void reset(Tuple tuple) {
		clear();
		this.tuple = tuple;
		this.buffertuple = null;
		this.tuplelist = null;
		returnType.clear();
	}
	public void reset(BufferTuple buffertuple) {
		clear();
		this.tuple = null;
		this.buffertuple = buffertuple;
		this.tuplelist = null;
		returnType.clear();
	}
	public void reset(ArrayList tuplelist) {
		clear();
		if (tuplelist.size() > 1) {
			this.tuple = null;
			this.buffertuple = null;
			this.tuplelist = tuplelist;
		} else if (tuplelist.size() == 1) {
			this.tuple = (Tuple) tuplelist.get(0);
			this.buffertuple = null;
			this.tuplelist = null;
		} else {
			this.tuple = null;
			this.buffertuple = null;
			this.tuplelist = null;
		}
		returnType.clear();
	}
	public boolean getVerdict() {
		return verdict;
	}
	//	public BufferTuple getResultBufferTuple() {
	//		return resultbuffertuple;
	//	}
	public Tuple getResultTuple() {
		return resulttuple;
	}
	public ArrayList getResNodes() {
		return resnodes;
	}

	public void visit(AttributeValuePair arg) throws XQueryException {
		if (!(arg.getExpression1() instanceof QName)) {
			throw new XQueryException("EvaluationVisitor : name of AttributeValuePair is not a QName : " + arg.getExpression1());
		}
		QName attName = (QName) arg.getExpression1();
		Object attValue = null;
		if (!(arg.getExpression2() instanceof Value)) {
			returnType.push(VALUE_TYPE);
			arg.getExpression2().accept(this);
			returnType.pop();
			if (resvalue == null) {
				throw new XQueryException("EvaluationVisitor : value of AttributeValuePair is null : " + arg.getExpression2());
			}
			attValue = resvalue;
		} else
			attValue = ((Value) arg.getExpression2()).getObjectValue();
		if (attValue != null) {
			TypedAttributeImpl att = new TypedAttributeImpl(docimpl, ((QName) attName).getNameSpace(), ((QName) attName).getLocalName());
			att.setNodeValue(attValue.toString());
			att.setTypedValue(attValue);
			ArrayList tmplist = new ArrayList(1);
			tmplist.add(att);
			setResnodes(tmplist);
			return;
		}
		setResnodes(null);
	}

	public void visit(BinOpANDExpression arg) throws XQueryException {
		//verdict = true;
		if (tuple != null) {
			XQueryExpression exp1 = arg.getExpression1();
			EvaluationVisitor visitor = new EvaluationVisitor(this.schemamanager, tuple);
			visitor.setReturnType(EvaluationVisitor.BOOLEAN_TYPE);
			exp1.accept(visitor);
			verdict = visitor.getVerdict();
			if (verdict == false) {
				setVerdict(false);
				return;
			}
			XQueryExpression exp2 = arg.getExpression2();
			//visitor = new EvaluationVisitor(this.schemamanager, tuple);
			visitor.reset(tuple);
			visitor.setReturnType(EvaluationVisitor.BOOLEAN_TYPE);
			exp2.accept(visitor);
			verdict &= visitor.getVerdict();
		} else if (buffertuple != null) {
			verdict = false;
		} else if (tuplelist != null) {
			XQueryExpression exp1 = arg.getExpression1();
			EvaluationVisitor visitor = new EvaluationVisitor(this.schemamanager, tuplelist);
			visitor.setReturnType(EvaluationVisitor.BOOLEAN_TYPE);
			exp1.accept(visitor);
			verdict = visitor.getVerdict();
			if (verdict == false) {
				setVerdict(false);
				return;
			}
			XQueryExpression exp2 = arg.getExpression2();
			//visitor = new EvaluationVisitor(this.schemamanager, tuplelist);
			visitor.reset(tuplelist);
			visitor.setReturnType(EvaluationVisitor.BOOLEAN_TYPE);
			exp2.accept(visitor);
			verdict &= visitor.getVerdict();
		} else {
			XQueryExpression exp1 = arg.getExpression1();
			EvaluationVisitor visitor = new EvaluationVisitor(this.schemamanager);
			visitor.setReturnType(EvaluationVisitor.BOOLEAN_TYPE);
			exp1.accept(visitor);
			verdict = visitor.getVerdict();
			if (verdict == false) {
				setVerdict(false);
				return;
			}
			XQueryExpression exp2 = arg.getExpression2();
			//visitor = new EvaluationVisitor(this.schemamanager);
			visitor.reset();
			visitor.setReturnType(EvaluationVisitor.BOOLEAN_TYPE);
			exp2.accept(visitor);
			verdict &= visitor.getVerdict();
		}
		setVerdict(verdict);
	}

	public void visit(BinOpORExpression arg) throws XQueryException {
		//verdict = false;
		if (tuple != null) {
			XQueryExpression exp1 = arg.getExpression1();
			EvaluationVisitor visitor = new EvaluationVisitor(this.schemamanager, tuple);
			exp1.accept(visitor);
			verdict = visitor.getVerdict();
			XQueryExpression exp2 = arg.getExpression2();
			//visitor = new EvaluationVisitor(this.schemamanager, tuple);
			visitor.reset(tuple);
			exp2.accept(visitor);
			verdict = verdict || visitor.getVerdict();
		} else if (buffertuple != null) {
			verdict = false;
		} else if (tuplelist != null) {
			XQueryExpression exp1 = arg.getExpression1();
			EvaluationVisitor visitor = new EvaluationVisitor(this.schemamanager, tuplelist);
			exp1.accept(visitor);
			verdict = visitor.getVerdict();
			XQueryExpression exp2 = arg.getExpression2();
			//visitor = new EvaluationVisitor(this.schemamanager, tuplelist);
			visitor.reset(tuplelist);
			exp2.accept(visitor);
			verdict = verdict || visitor.getVerdict();
		} else {
			XQueryExpression exp1 = arg.getExpression1();
			EvaluationVisitor visitor = new EvaluationVisitor(this.schemamanager);
			exp1.accept(visitor);
			verdict = visitor.getVerdict();
			XQueryExpression exp2 = arg.getExpression2();
			//visitor = new EvaluationVisitor(this.schemamanager);
			visitor.reset();
			exp2.accept(visitor);
			verdict = verdict || visitor.getVerdict();
		}
		setVerdict(verdict);
	}

	// public void visit(CastTreatExpression arg) throws XQueryException
	// public void visit(ContextDeclaration arg) throws XQueryException
	// public void visit(Dereference arg) throws XQueryException

	public void visit(Element arg) throws XQueryException {
		XQueryExpression startTag = arg.getStartTag();
		if (!(startTag instanceof QName))
			throw new XQueryException("Tag of Element expression is not a QName : " + startTag.toString());
		QName startTagQName = (QName) startTag;
		TypedElement element = new TypedElementImpl(docimpl, startTagQName.getNameSpace(), startTagQName.getLocalName());
		ArrayList tmplist = arg.getAttributes();
		if (tmplist != null && !tmplist.isEmpty()) {
			for (int i = 0; i < tmplist.size(); i++) {
				AttributeValuePair attValPair = (AttributeValuePair) tmplist.get(i);
				returnType.push(NODE_TYPE);
				attValPair.accept(this);
				returnType.pop();
				if (resnodes != null) {
					TypedAttribute att = (TypedAttribute) resnodes.get(0);
					element.setAttributeNodeNS((Attr) att);
				}
			}
		}
		tmplist = arg.getSubExpressions();
		if (tmplist != null && !tmplist.isEmpty()) {
			for (int i = 0; i < tmplist.size(); i++) {
				XQueryExpression subExpr = (XQueryExpression) tmplist.get(i);
				returnType.push(NODE_TYPE);
				subExpr.accept(this);
				returnType.pop();
				if (resnodes != null) {
					for (int j = 0; j < resnodes.size(); j++) {
						TypedNode resnode = (TypedNode) resnodes.get(j);
						if (docimpl != resnode.getOwnerDocument())
							resnode = (TypedNode) element.getOwnerDocument().importNode(resnode, true);
						if (resnode instanceof TypedAttributeImpl) {
							element.setAttributeNodeNS((Attr) resnode);
						} else if (resnode instanceof TypedElementImpl) {
							element.appendChild((Node) resnode);
						} else if (resnode instanceof TypedValueImpl) {
							element.appendChild((Node) resnode);
						} else {
							throw new XQueryException("Could not add child : " + resnode + " to parent.");
							// do nothing for now, warning?
						}
					}
				}
			}
		}
		tmplist = new ArrayList(1);
		tmplist.add(element);
		setResnodes(tmplist);
	}

	// public void visit (FLWRExpression arg) throws XQueryException
	// public void visit (FunctionCall arg) throws XQueryException
	// public void visit(FunctionDefinition arg) throws XQueryException
	// public void visit(InstanceOfExpression arg) throws XQueryException
	// public void visit(InternalFunctionCall arg) throws XQueryException
	// public void visit(ITEExpression arg) throws XQueryException
	// public void visit(LibraryFunctionCall arg) throws XQueryException
	// public void visit(ListOpAFTERExpression arg) throws XQueryException

	public void visit(ListOpArithExpression arg) throws XQueryException {
		XQueryExpression exp1 = arg.getExpression1();
		XQueryExpression exp2 = arg.getExpression2();
		int operator = arg.getOperator();

		returnType.push(UNKNOWN_TYPE);
		exp1.accept(this);
		ArrayList trees1 = resnodes;
		Number value1 = (resvalue instanceof Number) ? (Number) resvalue : null;
		exp2.accept(this);
		ArrayList trees2 = resnodes;
		Number value2 = (resvalue instanceof Number) ? (Number) resvalue : null;
		returnType.pop();

		//TypedDocument docimpl = new TypedDocument() ;
		//Integer retType = (Integer)returnType.pop();
		if ((trees1 == null || trees1.isEmpty()) && (trees2 == null || trees2.isEmpty())) {
			Number res = processOperation(exp1, exp2, null, null, value1, value2, operator);
			//if (retType.equals(NODE_TYPE)) {
			SimpleType st = getSimpleType((Comparable) res);
			TypedNode text = (TypedNode) docimpl.createTypedValue(res, st);
			text.setTypedValue(res);
			text.setType(st);
			ArrayList tmplist = new ArrayList(1);
			tmplist.add(text);
			setResnodes(tmplist);
			//			}
			//			else if (retType.equals(VALUE_TYPE)) {
			//				setValue(new Double(res));
			//			}
			//			else
			//				clear();
			return;
		}

		if ((trees1 == null || trees1.isEmpty()) && (trees2 != null) && (!trees2.isEmpty())) {
			ArrayList tmplist = new ArrayList(trees2.size());
			for (int j = 0; j < trees2.size(); j++) {
				TypedNode tree2 = (TypedNode) trees2.get(j);
				Number res = processOperation(exp1, exp2, null, tree2, value1, value2, operator);
				SimpleType st = getSimpleType((Comparable) res);
				TypedNode text = (TypedNode) docimpl.createTypedValue(res, st);
				text.setTypedValue(res);
				text.setType(st);
				tmplist.add(text);
			}
			//			if (retType.equals(NODE_TYPE)) {
			setResnodes(tmplist);
			//			}
			//			else
			//				clear();
			return;
		}

		if ((trees1 != null) && (!trees1.isEmpty()) && (trees2 == null) || (trees2.isEmpty())) {
			ArrayList tmplist = new ArrayList(trees1.size());
			for (int i = 0; i < trees1.size(); i++) {
				TypedNode tree1 = (TypedNode) trees1.get(i);
				Number res = processOperation(exp1, exp2, tree1, null, value1, value2, operator);
				SimpleType st = getSimpleType((Comparable) res);
				TypedNode text = (TypedNode) docimpl.createTypedValue(res, st);
				text.setTypedValue(res);
				text.setType(st);
				tmplist.add(text);
			}
			//			if (retType.equals(NODE_TYPE)) {
			setResnodes(tmplist);
			//			}
			//			else
			//				clear();
			return;
		}

		ArrayList tmplist = new ArrayList(trees1.size() * trees2.size());
		for (int i = 0; i < trees1.size(); i++) {
			TypedNode tree1 = (TypedNode) trees1.get(i);
			for (int j = 0; j < trees2.size(); j++) {
				TypedNode tree2 = (TypedNode) trees2.get(j);
				Number res = processOperation(exp1, exp2, tree1, tree2, value1, value2, operator);
				SimpleType st = getSimpleType((Comparable) res);
				TypedNode text = (TypedNode) docimpl.createTypedValue(res, st);
				text.setTypedValue(res);
				text.setType(st);
				tmplist.add(text);
			}
		}
		//		if (retType.equals(NODE_TYPE)) {
		setResnodes(tmplist);
		//		}
		//		else
		//			clear();
	}

	// public void visit(ListOpBEFOREExpression arg) throws XQueryException
	public void visit(ListOpCompExpression arg) throws XQueryException {
		XQueryExpression exp1 = arg.getExpression1();
		XQueryExpression exp2 = arg.getExpression2();
		int operator = arg.getOperator();

		returnType.push(EvaluationVisitor.NODE_TYPE);
		resnodes = null;
		resvalue = null;
		exp1.accept(this);
		//returnType.pop();
		ArrayList trees1 = resnodes;
		if (trees1 != null && trees1.isEmpty())
			trees1 = null;
		Comparable value1 = resvalue;
		//returnType.push(EvaluationVisitor.NODE_TYPE);
		resnodes = null;
		resvalue = null;
		exp2.accept(this);
		returnType.pop();
		ArrayList trees2 = resnodes;
		if (trees2 != null && trees2.isEmpty())
			trees2 = null;
		Comparable value2 = resvalue;

		if (value1 != null && value2 != null) {
			processComparison(null, null, null, null, value1, value2, operator);
			setVerdict(verdict);
			return;
		}

		if (trees1 == null && trees2 == null) {
			processComparison(exp1, exp2, null, null, value1, value2, operator);
			return;
		}

		if (trees1 == null && trees2 != null) {
			for (int j = 0; j < trees2.size(); j++) {
				TypedNode tree2 = (TypedNode) trees2.get(j);
				processComparison(exp1, exp2, null, tree2, value1, value2, operator);
				if (verdict == true) {
					setVerdict(true);
					return;
				}
			}
			setVerdict(false);
			return;
		}

		if (trees1 != null && trees2 == null) {
			for (int i = 0; i < trees1.size(); i++) {
				TypedNode tree1 = (TypedNode) trees1.get(i);
				processComparison(exp1, exp2, tree1, null, value1, value2, operator);
				if (verdict == true) {
					setVerdict(true);
					return;
				}
			}
			setVerdict(false);
			return;
		}

		for (int i = 0; i < trees1.size(); i++) {
			TypedNode tree1 = (TypedNode) trees1.get(i);
			for (int j = 0; j < trees2.size(); j++) {
				TypedNode tree2 = (TypedNode) trees2.get(j);
				processComparison(exp1, exp2, tree1, tree2, value1, value2, operator);
				if (verdict == true) {
					setVerdict(true);
					return;
				}
			}
			setVerdict(false);
		}
	}

	//	public void visit(ListOpEXCEPTExpression arg) throws XQueryException
	//	public void visit(ListOpINTERSECTExpression arg) throws XQueryException

	public void visit(ListOpUNIONExpression arg) throws XQueryException {
		XQueryExpression exp1 = arg.getExpression1();
		XQueryExpression exp2 = arg.getExpression2();

		returnType.push(NODE_TYPE);
		resnodes = null;
		exp1.accept(this);
		ArrayList trees = resnodes;
		resnodes = null;
		exp2.accept(this);
		returnType.pop();
		if (resnodes != null) {
			if (trees == null)
				trees = resnodes;
			else
				trees.addAll(resnodes);
		}
		setResnodes(trees);
	}

	public void visit(LocatedExpression arg) throws XQueryException {
		clear();
		if (!(arg.getStepNum(0).getExpression() instanceof Variable)) {
			returnType.push(NODE_TYPE);
			arg.getStepNum(0).getExpression().accept(this);
			returnType.pop();
			ArrayList tmplist = resnodes;
			for (int j = 1; j < arg.getSteps().size(); j++) {
				tmplist = DOMUtils.getStep(tmplist, arg.getStepNum(j));
				if (tmplist == null || tmplist.isEmpty())
					break;
			}
			if (tmplist != null)
				setResnodes(tmplist);
			return;
		}
		if (this.tuple == null && this.tuplelist == null) {
			return;
		}
		if (tuple != null) {
			if ((resnodes = tuple.getPath(arg.print())) != null) {
				setResnodes(resnodes);
				return;
			}
		} else {
			for (int i = 0; i < tuplelist.size(); i++) {
				if ((resnodes = ((Tuple) tuplelist.get(i)).getPath(arg.print())) != null) {
					setResnodes(resnodes);
					return;
				}
			}
		}
		resnodes = null;
		ArrayList steps = arg.getSteps();
		ArrayList tmplist = null;
		if (tuple != null) {
			StringBuffer buf = new StringBuffer(arg.print());
			int i;
			for (i = steps.size() - 1; i > 0; i--) {
				buf.delete(buf.length() - ((Step) steps.get(i)).toString().length(), buf.length());
				tmplist = tuple.getPath(buf.toString());
				if (tmplist != null)
					break;
			}
			if (tmplist == null)
				return;
			for (int j = i; j < steps.size(); j++) {
				tmplist = DOMUtils.getStep(tmplist, arg.getStepNum(j));
				if (tmplist == null || tmplist.isEmpty())
					break;
			}
		} else { // tuplelist not null
			for (int k = 0; k < tuplelist.size(); k++) {
				StringBuffer buf = new StringBuffer(arg.print());
				Tuple tuplek = (Tuple) tuplelist.get(k);
				int i;
				for (i = steps.size() - 1; i > 0; i--) {
					buf.delete(buf.length() - ((Step) steps.get(i)).toString().length(), buf.length());
					tmplist = tuplek.getPath(buf.toString());
					if (tmplist != null)
						break;
				}
				if (tmplist == null)
					continue;
				for (int j = i; j < steps.size(); j++) {
					tmplist = DOMUtils.getStep(tmplist, arg.getStepNum(j));
					if (tmplist == null || tmplist.isEmpty())
						break;
				}
				break;
			}
		}
		if (tmplist != null)
			setResnodes(tmplist);
	}
	//	public void visit(NodeType arg) throws XQueryException
	//	public void visit(Parameter arg) throws XQueryException
	//	public void visit(PrimitiveFunctionCall arg) throws XQueryException
	//  public void visit(QName arg) throws XQueryException
	//	public void visit(QuantifiedExpression arg) throws XQueryException
	//	public void visit(RangeExpression arg) throws XQueryException
	//	public void visit(RootNodeFunctionCall arg) throws XQueryException
	//	public void visit(SortedExpression arg) throws XQueryException
	//	public void visit(Step arg) throws XQueryException
	//	public void visit(TypeSwitchExpression arg) throws XQueryException

	private Comparable getMinus(Comparable obj) {
		Comparable retObj = null;
		if (obj instanceof Number) {
			//BigDecimal, BigInteger, Byte, Double, Float, Integer, Long, Short 
			if (obj instanceof Double) {
				retObj = new Double(- ((Double) obj).doubleValue());
			} else if (obj instanceof Float) {
				retObj = new Float(- ((Float) obj).floatValue());
			} else if (obj instanceof BigDecimal) {
				retObj = new BigDecimal(((BigDecimal) obj).unscaledValue().negate(), ((BigDecimal) obj).scale());
			} else if (obj instanceof Integer) {
				retObj = new Integer(- ((Integer) obj).intValue());
			} else if (obj instanceof Long) {
				retObj = new Long(- ((Long) obj).longValue());
			}
		} else if (obj instanceof String) {
			try {
				retObj = new Double(-Double.parseDouble((String) obj));
			} catch (Exception e) {
			}
		}
		return retObj;
	}

	public void visit(UnOpMinusExpression arg) throws XQueryException {
		returnType.push(UNKNOWN_TYPE);
		arg.getExpression().accept(this);
		returnType.pop();
		Integer rettype = (Integer) returnType.peek();
		if (resvalue != null) {
			if (!arg.getMinus()) {
				if (rettype == NODE_TYPE) {
					ArrayList nodes = new ArrayList(1);
					TypedValue text = null;
					if (resvalue instanceof Long)
						text = docimpl.createTypedValue(resvalue, this.schemamanager.getIntegerType());
					else if (resvalue instanceof BigDecimal)
						text = docimpl.createTypedValue(resvalue, this.schemamanager.getDecimalType());
					else if (resvalue instanceof Double)
						text = docimpl.createTypedValue(resvalue, this.schemamanager.getDoubleType());
					else
						throw new XQueryException("UnOpMinusExpression could not cast resvalue");
					nodes.add(text);
					setResnodes(nodes);
					return;
				} else { //(rettype == VALUE_TYPE) {
					setValue(resvalue);
					return;
				}
			}
			Comparable comp = getMinus(resvalue);
			if (comp != null) {
				if (rettype == NODE_TYPE) {
					ArrayList nodes = new ArrayList(1);
					TypedValue text = null;
					if (comp instanceof Long)
						text = docimpl.createTypedValue(comp, this.schemamanager.getIntegerType());
					else if (resvalue instanceof BigDecimal)
						text = docimpl.createTypedValue(comp, this.schemamanager.getDecimalType());
					else if (comp instanceof Double)
						text = docimpl.createTypedValue(comp, this.schemamanager.getDoubleType());
					else
						throw new XQueryException("UnOpMinusExpression could not cast comp");
					nodes.add(text);
					setResnodes(nodes);
					return;
				} else { //(rettype == VALUE_TYPE) {
					setValue(comp);
					return;
				}
			} else
				throw new XQueryException("UnOpMinusExpression could not cast operator to Number");
		} else if (resnodes != null) {
			if (rettype == VALUE_TYPE && resnodes.size() != 1) {
				throw new XQueryException("UnOpMinusExpression could not get value from node list");
			}
			if (!arg.getMinus()) {
				if (rettype == NODE_TYPE) {
					ArrayList tmplist = new ArrayList(resnodes.size());
					for (int i = 0; i < resnodes.size(); i++) {
						TypedNode tmpnode = (TypedNode) ((TypedNode) resnodes.get(0)).getFirstChild();
						if (tmpnode == null)
							tmpnode = (TypedNode) resnodes.get(0);
						tmplist.add(docimpl.createTypedValue((Comparable) tmpnode.getTypedValue(), (SimpleType) tmpnode.getType()));
					}
					setResnodes(tmplist);
					return;
				} else { // (rettype == VALUE_TYPE) {
					TypedNode tmpnode = (TypedNode) ((TypedNode) resnodes.get(0)).getFirstChild();
					if (tmpnode == null)
						tmpnode = (TypedNode) resnodes.get(0);
					setValue((Comparable) tmpnode.getTypedValue());
					return;
				}
			}
			ArrayList tmplist = new ArrayList(resnodes.size());
			for (int i = 0; i < resnodes.size(); i++) {
				TypedNode tmpnode = (TypedNode) ((TypedNode) resnodes.get(0)).getFirstChild();
				if (tmpnode == null)
					tmpnode = (TypedNode) resnodes.get(0);
				Comparable comp = getMinus((Comparable) tmpnode.getTypedValue());
				if (comp != null) {
					TypedValue text = docimpl.createTypedValue(comp, (SimpleType) tmpnode.getType());
					tmplist.add(text);
				}
			}
			if (rettype == NODE_TYPE) {
				setResnodes(tmplist);
				return;
			} else { // (rettype == VALUE_TYPE) {
				if (tmplist.size() == 1) {
					setValue((Comparable) ((TypedValue) tmplist.get(0)).getTypedValue());
					return;
				}
			}
		}
		clear();
	}

	public void visit(Value arg) throws XQueryException {
		clear();
	}
	//	public void visit(ValueBoolean arg) throws XQueryException {
	public void visit(ValueDecimal arg) throws XQueryException {
		Integer rettype = returnType.isEmpty() ? UNKNOWN_TYPE : (Integer) returnType.peek();
		if (rettype == NODE_TYPE) {
			ArrayList nodes = new ArrayList(1);
			TypedValue text = docimpl.createTypedValue(arg.getDecimalValue(), this.schemamanager.getDecimalType());
			nodes.add(text);
			setResnodes(nodes);
			return;
		} else { // rettype == VALUE_TYPE
			setValue(arg.getDecimalValue());
			return;
		}
	}
	public void visit(ValueDouble arg) throws XQueryException {
		Integer rettype = returnType.isEmpty() ? UNKNOWN_TYPE : (Integer) returnType.peek();
		if (rettype == NODE_TYPE) {
			ArrayList nodes = new ArrayList(1);
			TypedValue text = docimpl.createTypedValue(arg.getDoubleValue(), this.schemamanager.getDoubleType());
			nodes.add(text);
			setResnodes(nodes);
			return;
		} else { // rettype == VALUE_TYPE
			setValue(arg.getDoubleValue());
			return;
		}
	}
	public void visit(ValueFloat arg) throws XQueryException {
		Integer rettype = returnType.isEmpty() ? UNKNOWN_TYPE : (Integer) returnType.peek();
		if (rettype == NODE_TYPE) {
			ArrayList nodes = new ArrayList(1);
			TypedValue text = docimpl.createTypedValue(arg.getFloatValue(), this.schemamanager.getFloatType());
			nodes.add(text);
			setResnodes(nodes);
			return;
		} else { // rettype == VALUE_TYPE
			setValue(arg.getFloatValue());
			return;
		}
	}
	public void visit(ValueInteger arg) throws XQueryException {
		Integer rettype = returnType.isEmpty() ? UNKNOWN_TYPE : (Integer) returnType.peek();
		if (rettype == NODE_TYPE) {
			ArrayList nodes = new ArrayList(1);
			TypedValue text = docimpl.createTypedValue(new Long(arg.getIntValue().longValue()), this.schemamanager.getIntegerType());
			nodes.add(text);
			setResnodes(nodes);
			return;
		} else { // rettype == VALUE_TYPE
			setValue(new Long(arg.getIntValue().longValue()));
			return;
		}
	}
	public void visit(ValueString arg) throws XQueryException {
		Integer rettype = returnType.isEmpty() ? UNKNOWN_TYPE : (Integer) returnType.peek();
		if (rettype == NODE_TYPE) {
			ArrayList nodes = new ArrayList(1);
			TypedValue text = docimpl.createTypedValue(arg.getStringValue(), this.schemamanager.getStringType());
			nodes.add(text);
			setResnodes(nodes);
			return;
		} else { // rettype == VALUE_TYPE
			setValue(arg.getStringValue());
			return;
		}
	}
	public void visit(ValueText arg) throws XQueryException {
		Integer rettype = returnType.isEmpty() ? UNKNOWN_TYPE : (Integer) returnType.peek();
		if (rettype == NODE_TYPE) {
			ArrayList nodes = new ArrayList(1);
			TypedValue text = docimpl.createTypedValue(arg.getTextValue(), this.schemamanager.getStringType());
			nodes.add(text);
			setResnodes(nodes);
			return;
		} else { // rettype == VALUE_TYPE
			setValue(arg.getTextValue());
			return;
		}
	}
	public void visit(Variable arg) throws XQueryException {
		if (this.tuple != null) {
			setResnodes(tuple.getPath(arg.print()));
			return;
		} else if (tuplelist != null) {
			for (int i = 0; i < tuplelist.size(); i++) {
				if ((resnodes = ((Tuple) tuplelist.get(i)).getPath(arg.print())) != null) {
					this.setResnodes(resnodes);
					return;
				}
			}
		}
		this.clear();
	}
	//	public void visit (XQueryBinaryOperatorExpression arg) throws XQueryException
	//	public void visit(XQueryBooleanOperatorExpression arg) throws XQueryException
	public void visit(XQueryExpression arg) throws XQueryException {
		throw new XQueryException("Class " + arg.getClass().getName() + " not supported in EvaluationVisitor");
	}
	public void visit(XQueryExpressionSequence arg) throws XQueryException {
		if (arg.getSubExpressions().size() == 1)
			 ((XQueryExpression) arg.getSubExpressions().get(0)).accept(this);
		else {
			ArrayList expressions = arg.getSubExpressions();
			ArrayList tmpnodes = new ArrayList();
			returnType.push(NODE_TYPE);
			for (int i = 0; i < expressions.size(); i++) {
				XQueryExpression expri = (XQueryExpression) expressions.get(i);
				expri.accept(this);
				if (resnodes != null)
					tmpnodes.addAll(resnodes);
			}
			setResnodes(tmpnodes);
			returnType.pop();
			//throw new XQueryException("Class " + arg.getClass().getName() + " with more than one expression not supported in EvaluationVisitor");
		}
	}
	// public void visit(XQueryListBinaryOperatorExpression arg) throws XQueryException
	// public void visit(XQueryListUnaryOperatorExpression arg) throws XQueryException
	//	public void visit (XQueryUnaryOperatorExpression arg) throws XQueryException

	//-----------------------------------------------------------
	// BOOLEAN FUNCTIONS
	//-----------------------------------------------------------
	public void visit(FunctionFALSE arg) throws XQueryException {
		setVerdict(false);
	}

	public void visit(FunctionTRUE arg) throws XQueryException {
		setVerdict(true);
	}

	//-----------------------------------------------------------
	// AGGREGATE FUNCTIONS
	//-----------------------------------------------------------

	public void visit(FunctionAVG arg) throws XQueryException {
		boolean isLongSum = false;
		double doubleSum = 0;
		long longSum = 0;
		int count = 0;
		// treat special case first
		if (buffertuple != null) {
			int index = buffertuple.getPathIndex(arg.getArgument(0).toString());
			if (index == -1)
				index = 0;
			for (int i = 0; i < buffertuple.size(); i++) {
				Tuple tuplei = (Tuple) buffertuple.get(i);
				//				if (tuplei.size() != 1)
				//					return;
				Number valuei = null;
				TypedNode treei = (TypedNode) tuplei.getNodesAtIndex(index).get(0);
				try {
					if (treei.getTypedValue() != null)
						valuei = (Number) treei.getTypedValue();
					else
						valuei = (Number) DOMUtils.getTextLeaf(treei);
				} catch (Exception ClassCastException) {
				}
				if (valuei == null)
					return;
				if (isLongSum && (valuei instanceof Integer || valuei instanceof Long)) {
					longSum += valuei.longValue();
				} else {
					if (isLongSum) {
						doubleSum = longSum;
						isLongSum = false;
					}
					doubleSum += valuei.doubleValue();
				}
			}
			count = buffertuple.size();
		} else {
			returnType.push(NODE_TYPE);
			arg.getArgument(0).accept(this);
			returnType.pop();
			if (resnodes == null || resnodes.isEmpty())
				return;

			for (int i = 0; i < resnodes.size(); i++) {
				Number valuei = null;
				TypedNode treei = (TypedNode) resnodes.get(i);
				try {
					if (treei.getTypedValue() != null)
						valuei = (Number) treei.getTypedValue();
					else
						valuei = (Number) DOMUtils.getTextLeaf(treei);
				} catch (Exception ClassCastException) {
				}
				if (valuei == null)
					return;
				if (isLongSum && (valuei instanceof Integer || valuei instanceof Long)) {
					longSum += valuei.longValue();
				} else {
					if (isLongSum) {
						doubleSum = longSum;
						isLongSum = false;
					}
					doubleSum += valuei.doubleValue();
				}
			}
			count = resnodes.size();
		}
		Integer retType = (Integer) this.returnType.peek();
		if (retType == TUPLE_TYPE) {
			Tuple tuple = new Tuple(1);
			TypedValue text = null;
			if (isLongSum)
				text = docimpl.createTypedValue(new BigDecimal(longSum / count), this.schemamanager.getDecimalType());
			else
				text = docimpl.createTypedValue(new Double(doubleSum / count), this.schemamanager.getDoubleType());
			tuple.addNodeAtIndex(0, text);
			this.setResultTuple(tuple);
			return;
		} else if (retType == NODE_TYPE) {
			ArrayList nodes = new ArrayList(1);
			TypedValue text = null;
			if (isLongSum)
				text = docimpl.createTypedValue(new BigDecimal(longSum / count), this.schemamanager.getDecimalType());
			else
				text = docimpl.createTypedValue(new Double(doubleSum / count), this.schemamanager.getDoubleType());
			nodes.add(text);
			setResnodes(nodes);
			return;
		} else /*if (retType == VALUE_TYPE)*/ {
			if (isLongSum)
				setValue(new BigDecimal(longSum / count));
			else
				setValue(new Double(doubleSum / count));
			return;
		}
		// clear();

	}

	public void visit(FunctionSUM arg) throws XQueryException {
		boolean isLongSum = false;
		double doubleSum = 0;
		long longSum = 0;
		// treat special case first
		if (buffertuple != null) {
			int index = buffertuple.getPathIndex(arg.getArgument(0).toString());
			if (index == -1)
				index = 0;
			for (int i = 0; i < buffertuple.size(); i++) {
				Tuple tuplei = (Tuple) buffertuple.get(i);
				//				if (tuplei.size() != 1)
				//					return;
				Number valuei = null;
				TypedNode treei = (TypedNode) tuplei.getNodesAtIndex(index).get(0);
				try {
					if (treei.getTypedValue() != null)
						valuei = (Number) treei.getTypedValue();
					else
						valuei = (Number) DOMUtils.getTextLeaf(treei);
				} catch (Exception ClassCastException) {
				}
				if (valuei == null)
					return;
				if (isLongSum && (valuei instanceof Integer || valuei instanceof Long)) {
					longSum += valuei.longValue();
				} else {
					if (isLongSum) {
						doubleSum = longSum;
						isLongSum = false;
					}
					doubleSum += valuei.doubleValue();
				}
			}
		} else {
			returnType.push(NODE_TYPE);
			arg.getArgument(0).accept(this);
			returnType.pop();
			if (resnodes == null || resnodes.isEmpty())
				return;

			for (int i = 0; i < resnodes.size(); i++) {
				Number valuei = null;
				TypedNode treei = (TypedNode) resnodes.get(i);
				try {
					if (treei.getTypedValue() != null)
						valuei = (Number) treei.getTypedValue();
					else
						valuei = (Number) DOMUtils.getTextLeaf(treei);
				} catch (Exception ClassCastException) {
				}
				if (valuei == null)
					return;
				if (isLongSum && (valuei instanceof Integer || valuei instanceof Long)) {
					longSum += valuei.longValue();
				} else {
					if (isLongSum) {
						doubleSum = longSum;
						isLongSum = false;
					}
					doubleSum += valuei.doubleValue();
				}
			}
		}
		Integer retType = (Integer) this.returnType.peek();
		if (retType == TUPLE_TYPE) {
			Tuple tuple = new Tuple(1);
			TypedValue text = null;
			if (isLongSum)
				text = docimpl.createTypedValue(new Long(longSum), this.schemamanager.getIntegerType());
			else
				text = docimpl.createTypedValue(new Double(doubleSum), this.schemamanager.getDoubleType());
			tuple.addNodeAtIndex(0, text);
			setResultTuple(tuple);
			return;
		} else if (retType == NODE_TYPE) {
			ArrayList nodes = new ArrayList(1);
			TypedValue text = null;
			if (isLongSum)
				text = docimpl.createTypedValue(new Long(longSum), this.schemamanager.getIntegerType());
			else
				text = docimpl.createTypedValue(new Double(doubleSum), this.schemamanager.getDoubleType());
			nodes.add(text);
			setResnodes(nodes);
			return;
		} else /*if (retType == VALUE_TYPE)*/ {
			if (isLongSum)
				setValue(new Long(longSum));
			else
				setValue(new Double(doubleSum));
			return;
		}
		// clear();

	}

	public void visit(FunctionMAX arg) throws XQueryException {
		Comparable max = null;
		TypedNode result = null;
		// treat special case first
		if (buffertuple != null) {
			int index = buffertuple.getPathIndex(arg.getArgument(0).toString());
			if (index == -1)
				index = 0;
			for (int i = 0; i < buffertuple.size(); i++) {
				Tuple tuplei = (Tuple) buffertuple.get(i);
				//				if (tuplei.size() != 1)
				//					return;
				Comparable valuei = null;
				TypedNode treei = (TypedNode) tuplei.getNodesAtIndex(index).get(0);
				try {
					if (treei.getTypedValue() != null)
						valuei = (Comparable) treei.getTypedValue();
					else
						valuei = (Comparable) DOMUtils.getTextLeaf(treei);
				} catch (Exception ClassCastException) {
				}
				if (valuei == null)
					return;
				if (max == null) {
					max = valuei;
					result = treei;
				} else if (valuei.compareTo(max) > 0) {
					max = valuei;
					result = treei;
				}
			}
		} else {
			returnType.push(NODE_TYPE);
			arg.getArgument(0).accept(this);
			returnType.pop();
			if (resnodes == null || resnodes.isEmpty())
				return;

			for (int i = 0; i < resnodes.size(); i++) {
				Comparable valuei = null;
				TypedNode treei = (TypedNode) resnodes.get(i);
				try {
					if (treei.getTypedValue() != null)
						valuei = (Comparable) treei.getTypedValue();
					else
						valuei = (Comparable) DOMUtils.getTextLeaf(treei);
				} catch (Exception ClassCastException) {
				}
				if (valuei == null)
					return;
				if (max == null) {
					max = valuei;
					result = treei;
				} else if (valuei.compareTo(max) > 0) {
					max = valuei;
					result = treei;
				}
			}
		}

		Integer retType = (Integer) this.returnType.peek();
		if (retType == TUPLE_TYPE) {
			Tuple tuple = new Tuple(1);
			TypedValue text = docimpl.createTypedValue(max, (SimpleType) result.getType());
			tuple.addNodeAtIndex(0, text);
			setResultTuple(tuple);
			return;
		} else if (retType == NODE_TYPE) {
			ArrayList nodes = new ArrayList(1);
			TypedValue text = docimpl.createTypedValue(max, (SimpleType) result.getType());
			nodes.add(text);
			setResnodes(nodes);
			return;
		} else /*if (retType == VALUE_TYPE)*/ {
			setValue(max);
			return;
		}
		// clear();

	}

	public void visit(FunctionMIN arg) throws XQueryException {
		Comparable min = null;
		TypedNode result = null;
		// treat special case first
		if (buffertuple != null) {
			int index = buffertuple.getPathIndex(arg.getArgument(0).toString());
			if (index == -1)
				index = 0;
			for (int i = 0; i < buffertuple.size(); i++) {
				Tuple tuplei = (Tuple) buffertuple.get(i);
				//				if (tuplei.size() != 1)
				//					return;
				Comparable valuei = null;
				TypedNode treei = (TypedNode) tuplei.getNodesAtIndex(index).get(0);
				try {
					if (treei.getTypedValue() != null)
						valuei = (Comparable) treei.getTypedValue();
					else
						valuei = (Comparable) DOMUtils.getTextLeaf(treei);
				} catch (Exception ClassCastException) {
				}
				if (valuei == null)
					return;
				if (min == null) {
					min = valuei;
					result = treei;
				} else if (valuei.compareTo(min) < 0) {
					min = valuei;
					result = treei;
				}
			}
		} else {
			returnType.push(NODE_TYPE);
			arg.getArgument(0).accept(this);
			returnType.pop();
			if (resnodes == null || resnodes.isEmpty())
				return;

			for (int i = 0; i < resnodes.size(); i++) {
				Comparable valuei = null;
				TypedNode treei = (TypedNode) resnodes.get(i);
				try {
					if (treei.getTypedValue() != null)
						valuei = (Comparable) treei.getTypedValue();
					else
						valuei = (Comparable) DOMUtils.getTextLeaf(treei);
				} catch (Exception ClassCastException) {
				}
				if (valuei == null)
					return;
				if (min == null) {
					min = valuei;
					result = treei;
				} else if (valuei.compareTo(min) < 0) {
					min = valuei;
					result = treei;
				}
			}
		}

		Integer retType = (Integer) this.returnType.peek();
		if (retType == TUPLE_TYPE) {
			Tuple tuple = new Tuple(1);
			TypedValue text = docimpl.createTypedValue(min, (SimpleType) result.getType());
			tuple.addNodeAtIndex(0, text);
			setResultTuple(tuple);
			return;
		} else if (retType == NODE_TYPE) {
			ArrayList nodes = new ArrayList(1);
			TypedValue text = docimpl.createTypedValue(min, (SimpleType) result.getType());
			nodes.add(text);
			setResnodes(nodes);
			return;
		} else /*if (retType == VALUE_TYPE)*/ {
			setValue(min);
			return;
		}
		// clear();

	}

	public void visit(FunctionCOUNT arg) throws XQueryException {
		long strvalue = 0;
		// treat special case first
		if (buffertuple != null) {
			strvalue = buffertuple.size();
		} else {
			returnType.push(NODE_TYPE);
			arg.getArgument(0).accept(this);
			returnType.pop();
			if (resnodes != null)
				strvalue = resnodes.size();
		}

		Integer retType = (Integer) this.returnType.peek();
		if (retType == TUPLE_TYPE) {
			Tuple tuple = new Tuple(1);
			TypedValue text = docimpl.createTypedValue(new Long(strvalue), this.schemamanager.getIntegerType());
			tuple.addNodeAtIndex(0, text);
			setResultTuple(tuple);
			return;
		} else if (retType == NODE_TYPE) {
			ArrayList nodes = new ArrayList(1);
			TypedValue text = docimpl.createTypedValue(new Long(strvalue), this.schemamanager.getIntegerType());
			nodes.add(text);
			setResnodes(nodes);
			return;
		} else /*if (retType == VALUE_TYPE)*/ {
			setValue(new Long(strvalue));
			return;
		}
		//clear();
	}

	//-----------------------------------------------------------
	// DATE/TIME FUNCTIONS
	//-----------------------------------------------------------
	//	private int[] parseDate(String date) {
	//		if (date == null)
	//			return null;
	//		int[] retval = new int[3];
	//		retval[0] = Integer.parseInt(date.substring(0, 4));
	//		retval[1] = Integer.parseInt(date.substring(5, 7));
	//		retval[2] = Integer.parseInt(date.substring(8, 10));
	//		return retval;
	//	}
	Calendar cal = Calendar.getInstance();

	public void visit(FunctionCURRENT_DATETIME arg) throws XQueryException {
		resvalue = Calendar.getInstance().getTime();
		Integer rettype = (Integer) returnType.peek();
		if (rettype == NODE_TYPE) {
			ArrayList nodes = new ArrayList(1);
			nodes.add(docimpl.createTypedValue(resvalue, this.schemamanager.getDateTimeType()));
			setResnodes(nodes);
			return;
		} else { //if (rettype == VALUE_TYPE) {
			setValue(resvalue);
			return;
		}
		// clear();
	}

	public void visit(FunctionDATE arg) throws XQueryException {
		returnType.push(VALUE_TYPE);
		arg.getArgument(0).accept(this);
		returnType.pop();
		if (resvalue instanceof Date) { // do nothing
			cal.setTime((Date) resvalue);
			cal.set(Calendar.HOUR, 0);
			cal.set(Calendar.MINUTE, 0);
			cal.set(Calendar.SECOND, 0);
			resvalue = cal.getTime();
		} else if (resvalue instanceof String) {
			try {
				resvalue = (Comparable) this.schemamanager.getDateType().convert((String) resvalue, true, null);
			} catch (SchemaException se) {
				throw new XQueryException("Argument of function date is not a valid date");
			}
		} else
			throw new XQueryException("Argument of function date cannot be converted");
		Integer rettype = (Integer) returnType.peek();
		if (rettype == NODE_TYPE) {
			ArrayList nodes = new ArrayList(1);
			nodes.add(docimpl.createTypedValue(resvalue, this.schemamanager.getDateType()));
			setResnodes(nodes);
			return;
		} else { //if (rettype == VALUE_TYPE) {
			setValue(resvalue);
			return;
		}
		//clear();
	}

	public void visit(FunctionDATETIME arg) throws XQueryException {
		returnType.push(VALUE_TYPE);
		arg.getArgument(0).accept(this);
		returnType.pop();
		if (resvalue instanceof Date) { // do nothing
		} else if (resvalue instanceof String) {
			try {
				resvalue = (Comparable) this.schemamanager.getDateTimeType().convert((String) resvalue, true, null);
			} catch (SchemaException se) {
				throw new XQueryException("Argument of function datetime is not a valid date");
			}
		} else
			throw new XQueryException("Argument of function datetime cannot be converted");
		Integer rettype = (Integer) returnType.peek();
		if (rettype == NODE_TYPE) {
			ArrayList nodes = new ArrayList(1);
			nodes.add(docimpl.createTypedValue(resvalue, this.schemamanager.getDateTimeType()));
			setResnodes(nodes);
			return;
		} else { //if (rettype == VALUE_TYPE) {
			setValue(resvalue);
			return;
		}
		//clear();
	}

	public void visit(FunctionTIME arg) throws XQueryException {
		returnType.push(VALUE_TYPE);
		arg.getArgument(0).accept(this);
		returnType.pop();
		if (resvalue instanceof Date) { // do nothing
		} else if (resvalue instanceof String) {
			try {
				resvalue = (Comparable) this.schemamanager.getTimeType().convert((String) resvalue, true, null);
			} catch (SchemaException se) {
				throw new XQueryException("Argument of function time is not a valid date");
			}
		} else
			throw new XQueryException("Argument of function time cannot be converted");
		/*
		StringBuffer buf = new StringBuffer();
		buf.append(cal.get(Calendar.HOUR));
		buf.append(":");
		buf.append(cal.get(Calendar.MINUTE));
		buf.append(":");
		buf.append(cal.get(Calendar.SECOND));
		*/
		cal.setTime((Date) resvalue);
		cal.set(Calendar.YEAR, 0);
		cal.set(Calendar.MONTH, 0);
		cal.set(Calendar.DAY_OF_MONTH, 0);
		resvalue = cal.getTime();
		Integer rettype = (Integer) returnType.peek();
		if (rettype == NODE_TYPE) {
			ArrayList nodes = new ArrayList(1);
			nodes.add(docimpl.createTypedValue(resvalue, this.schemamanager.getTimeType()));
			setResnodes(nodes);
			return;
		} else { //if (rettype == VALUE_TYPE) {
			setValue(resvalue);
			return;
		}
		//clear();
	}

	public void visit(FunctionGET_YEAR_FROM_DATE arg) throws XQueryException {
		returnType.push(VALUE_TYPE);
		arg.getArgument(0).accept(this);
		returnType.pop();
		if (resvalue == null) {
			clear();
			return;
		}
		if (resvalue instanceof Date)
			throw new XQueryException("Argument of function get-year-from-date is not a date");
		cal.setTime((Date) resvalue);
		resvalue = new Long(cal.get(Calendar.YEAR));
		Integer rettype = (Integer) returnType.peek();
		if (rettype == NODE_TYPE) {
			ArrayList nodes = new ArrayList(1);
			nodes.add(docimpl.createTypedValue(resvalue, this.getSimpleType(resvalue)));
			setResnodes(nodes);
			return;
		} else { //if (rettype == VALUE_TYPE) {
			setValue(resvalue);
			return;
		}
		//clear();
	}

	public void visit(FunctionGET_MONTH_FROM_DATE arg) throws XQueryException {
		returnType.push(VALUE_TYPE);
		arg.getArgument(0).accept(this);
		returnType.pop();
		if (resvalue == null) {
			clear();
			return;
		}
		if (resvalue instanceof Date)
			throw new XQueryException("Argument of function get-month-from-date is not a date");
		cal.setTime((Date) resvalue);
		resvalue = new Long(cal.get(Calendar.MONTH));
		Integer rettype = (Integer) returnType.peek();
		if (rettype == NODE_TYPE) {
			ArrayList nodes = new ArrayList(1);
			nodes.add(docimpl.createTypedValue(resvalue, this.getSimpleType(resvalue)));
			setResnodes(nodes);
			return;
		} else { //if (rettype == VALUE_TYPE) {
			setValue(resvalue);
			return;
		}
		//clear();
	}

	//-----------------------------------------------------------
	// STRING FUNCTIONS
	//-----------------------------------------------------------

	private Comparable c0 = null;
	private Comparable c1 = null;

	private void getComparables(XQueryExpression arg0, XQueryExpression arg1) throws XQueryException {
		resulttuple = null;
		resnodes = null;
		c0 = null;
		c1 = null;
		if (arg0 instanceof Value)
			c0 = (((Value) arg0).getValue());
		if (arg1 instanceof Value)
			c1 = (((Value) arg1).getValue());
		if (c0 == null) {
			returnType.push(VALUE_TYPE);
			arg0.accept(this);
			returnType.pop();
			c0 = this.resvalue;
			if (c0 == null && resnodes != null && resnodes.size() == 1) {
				TypedNode node = (TypedNode) resnodes.get(0);
				if (node instanceof TypedValueImpl) {
					c0 = (Comparable) node.getTypedValue();
				} else if (node instanceof TypedElementImpl) {
					StringBuffer sb = new StringBuffer();
					DOMUtils.getStringValue(node, sb);
					c0 = sb.toString();
				}
			}
		}
		if (c1 == null) {
			returnType.push(VALUE_TYPE);
			arg1.accept(this);
			returnType.pop();
			c1 = this.resvalue;
			if (c1 == null && resnodes != null && resnodes.size() == 1) {
				TypedNode node = (TypedNode) resnodes.get(0);
				if (node instanceof TypedValueImpl) {
					c1 = (Comparable) node.getTypedValue();
				} else if (node instanceof TypedElementImpl) {
					StringBuffer sb = new StringBuffer();
					DOMUtils.getStringValue(node, sb);
					c1 = sb.toString();
				}
			}
		}

	}

	public void visit(FunctionLOWER_CASE arg) throws XQueryException {
		returnType.push(UNKNOWN_TYPE);
		arg.getArgument(0).accept(this);
		returnType.pop();
		if (resvalue != null) { // do nothing
		} else if (resnodes != null && resnodes.size() == 1) {
			TypedNode tmpnode = (TypedNode) ((TypedNode) resnodes.get(0)).getFirstChild();
			if (tmpnode == null)
				tmpnode = (TypedNode) resnodes.get(0);
			resvalue = (Comparable) tmpnode.getTypedValue();
		}
		if (resvalue instanceof String) {
			resvalue = ((String) resvalue).toLowerCase();
		} else {
			throw new XQueryException("Argument of function lower-case is not a string");
		}
		Integer rettype = (Integer) returnType.peek();
		if (rettype == VALUE_TYPE) {
			setValue(resvalue);
			return;
		} else if (rettype == NODE_TYPE) {
			ArrayList nodes = new ArrayList(1);
			nodes.add(docimpl.createTypedValue(resvalue, this.getSimpleType(resvalue)));
			setResnodes(nodes);
			return;
		}
	}

	public void visit(FunctionUPPER_CASE arg) throws XQueryException {
		returnType.push(UNKNOWN_TYPE);
		arg.getArgument(0).accept(this);
		returnType.pop();
		if (resvalue != null) { // do nothing
		} else if (resnodes != null && resnodes.size() == 1) {
			TypedNode tmpnode = (TypedNode) ((TypedNode) resnodes.get(0)).getFirstChild();
			if (tmpnode == null)
				tmpnode = (TypedNode) resnodes.get(0);
			resvalue = (Comparable) tmpnode.getTypedValue();
		}
		if (resvalue instanceof String) {
			resvalue = ((String) resvalue).toUpperCase();
		} else {
			throw new XQueryException("Argument of function upper-case is not a string");
		}
		Integer rettype = (Integer) returnType.peek();
		if (rettype == VALUE_TYPE) {
			setValue(resvalue);
			return;
		} else if (rettype == NODE_TYPE) {
			ArrayList nodes = new ArrayList(1);
			nodes.add(docimpl.createTypedValue(resvalue, this.getSimpleType(resvalue)));
			setResnodes(nodes);
			return;
		}
	}

	public void visit(FunctionCONCAT arg) throws XQueryException {
		StringBuffer buf = new StringBuffer();
		for (int i = 0; i < arg.getArguments().size(); i++) {
			returnType.push(UNKNOWN_TYPE);
			arg.getArgument(i).accept(this);
			returnType.pop();
			if (resvalue != null) { // do nothing
				if (resvalue instanceof String)
					buf.append(resvalue);
				else
					buf.append(resvalue.toString());
			} else if (resnodes == null || resnodes.isEmpty()) { // do nothing
			} else if (resnodes != null && resnodes.size() == 1) {
				StringBuffer sb = new StringBuffer();
				DOMUtils.getStringValue((TypedNode) resnodes.get(0), sb);
				buf.append(sb.toString());
			} else if (resnodes != null && resnodes.size() > 1) {
				throw new XQueryException("Evaluation : incorrect argument for function concat");
			}
		}
		Integer rettype = (Integer) returnType.peek();
		if (rettype == NODE_TYPE) {
			ArrayList nodes = new ArrayList(1);
			nodes.add(docimpl.createTypedValue(buf.toString(), this.schemamanager.getStringType()));
			setResnodes(nodes);
			return;
		} else { // if (rettype == VALUE_TYPE) {
			setValue(buf.toString());
			return;
		}
		// clear();
	}

	public void visit(FunctionCONTAINS arg) throws XQueryException {
		getComparables(arg.getArgument(0), arg.getArgument(1));
		if (c0 != null && c1 != null) {
			setVerdict(((String) c0).indexOf((String) c1) != -1);
			return;
		}
		clear();
	}

	public void visit(FunctionENDS_WITH arg) throws XQueryException {
		getComparables(arg.getArgument(0), arg.getArgument(1));
		if (c0 != null && c1 != null) {
			setVerdict(((String) c0).endsWith((String) c1));
			return;
		}
		clear();
	}

	public void visit(FunctionSTARTS_WITH arg) throws XQueryException {
		getComparables(arg.getArgument(0), arg.getArgument(1));
		if (c0 != null && c1 != null) {
			setVerdict(((String) c0).startsWith((String) c1));
			return;
		}
		clear();
	}

	public void visit(FunctionLOCAL_NAME arg) throws XQueryException {
		returnType.push(NODE_TYPE);
		arg.getArgument(0).accept(this);
		returnType.pop();
		Integer rettype = (Integer) returnType.peek();
		//if (rettype == VALUE_TYPE) {
		if (this.resnodes != null && resnodes.size() == 1) {
			this.setValue(((TypedNode) resnodes.get(0)).getLocalName());
			return;
		}
		//}
		clear();
	}

	public void visit(FunctionNAMESPACE_URI arg) throws XQueryException {
		returnType.push(NODE_TYPE);
		arg.getArgument(0).accept(this);
		returnType.pop();
		Integer rettype = (Integer) returnType.peek();
		//if (rettype == VALUE_TYPE) {
		if (this.resnodes != null && resnodes.size() == 1) {
			this.setValue(((TypedNode) resnodes.get(0)).getNamespaceURI());
			return;
		}
		//}
		clear();
	}

	/*
	2.3 fn:string
	fn:string() as xs:string 
	fn:string($srcval as item?) as xs:string 
	Returns the value of $srcval represented as a xs:string. If no argument is supplied, $srcval defaults to the context item (.).
	
	If $srcval is the empty sequence, the zero-length string is returned.
	
	If $srcval is a node, the function returns the string-value of the node, as obtained using the string-value accessor defined in the [XQuery 1.0 and XPath 2.0 Data Model].
	
	If $srcval is an atomic value, then the function returns the same string as is returned by the expression cast as xs:string ($srcval), except in the cases listed below:
	
	If the type of $srcval is xs:anyURI, the URI is converted to a string without any escaping of special characters.
	
	Note:
	
	The reason for the special rule for xs:anyURI is that, although XML Schema strongly discourages the use of spaces within URI values, 
	the escaping of spaces can cause problems with legacy applications (for example, this applies to spaces within fragment identifiers 
	in many HTML browsers), and should therefore be under user control.
	
	Note:
	
	The string representation of xs:double values is not backwards-compatible with the representation of number values in [XPath 1.0]. 
	Ordinary xs:double values are now represented using scientific notation; the representations of positive and negative infinity are 
	now 'INF' and '-INF' rather than 'Infinity' and '-Infinity'. (It should be observed that '+INF' is not supported as a lexical form 
	of infinity in [XML Schema Part 2: Datatypes] and is thus not supported by this specification; if that lexical form is added in a 
	future version of [XML Schema Part 2: Datatypes], then it will be supported by a future version of this specification that aligns 
	with that future version of [XML Schema Part 2: Datatypes].) 
	However, most expressions that would have produced a number in [XPath 1.0] will produce a decimal (or integer) in [XPath 2.0], 
	so unless there is a loss of precision caused by numeric approximation, the result of the expression will in most simple cases be 
	the same after conversion to a string.
	*/
	public void visit(FunctionSTRING arg) throws XQueryException {
		returnType.push(UNKNOWN_TYPE);
		arg.getArgument(0).accept(this);
		returnType.pop();
		if (resvalue != null) { // do nothing
			if (!(resvalue instanceof String))
				resvalue = resvalue.toString();
		} else if (resnodes == null || resnodes.isEmpty()) {
			resvalue = new String("");
		} else if (resnodes != null && resnodes.size() == 1) {
			StringBuffer sb = new StringBuffer();
			DOMUtils.getStringValue((TypedNode) resnodes.get(0), sb);
			resvalue = sb.toString();
		} else if (resnodes != null && resnodes.size() > 1) {
			throw new XQueryException("Evaluation : incorrect argument for function string");
		}
		Integer rettype = (Integer) returnType.peek();
		if (rettype == NODE_TYPE) {
			ArrayList nodes = new ArrayList(1);
			TypedValue text = docimpl.createTypedValue(resvalue, this.schemamanager.getStringType());
			nodes.add(text);
			setResnodes(nodes);
			return;
		} else { //if (rettype == VALUE_TYPE) {
			setValue(resvalue);
			return;
		}
		//clear();
	}

	/*
	7.4.7 fn:string-length
	fn:string-length() as xs:integer? 
	fn:string-length($srcval as xs:string?) as xs:integer? 
	Returns an xs:integer equal to the length in characters of the value of $srcval. If the value of $srcval is the empty sequence, 
	the empty sequence is returned. If no argument is supplied, $srcval defaults to the string value (calculated using fn:string()) of the context item (.).
	
	Note:
	
	The value returned is the number of XML characters in the string (or equivalently, the number of Unicode code points). 
	Some implementations may represent a code point above xFFFF using two 16-bit values known as a surrogate pair. 
	A surrogate pair counts as one character, not two.
	*/
	public void visit(FunctionSTRING_LENGTH arg) throws XQueryException {
		returnType.push(UNKNOWN_TYPE);
		arg.getArgument(0).accept(this);
		returnType.pop();
		if (resvalue != null) { // do nothing
			if (resvalue instanceof String)
				resvalue = new Long(((String) resvalue).length());
			else
				resvalue = new Long(resvalue.toString().length());
		} else if (resnodes == null || resnodes.isEmpty()) {
			resvalue = new Long(0);
		} else if (resnodes != null && resnodes.size() == 1) {
			StringBuffer sb = new StringBuffer();
			DOMUtils.getStringValue((TypedNode) resnodes.get(0), sb);
			resvalue = new Long(sb.length());
		} else if (resnodes != null && resnodes.size() > 1) {
			throw new XQueryException("Evaluation : incorrect argument for function string-length");
		}
		Integer rettype = (Integer) returnType.peek();
		if (rettype == NODE_TYPE) {
			ArrayList nodes = new ArrayList(1);
			TypedValue text = docimpl.createTypedValue(resvalue, this.schemamanager.getIntegerType());
			nodes.add(text);
			setResnodes(nodes);
			return;
		} else { //if (rettype == VALUE_TYPE) {
			setValue(resvalue);
			return;
		}
		//clear();
	}

	/*
	7.4.6 fn:substring
	fn:substring( $sourceString  as xs:string?, 
	$startingLoc  as xs:double) as xs:string? 
	
	fn:substring( $sourceString  as xs:string?, 
	$startingLoc  as xs:double, 
	$length  as xs:double) as xs:string? 
	
	If the value of $sourceString is the empty sequence, the empty sequence is returned.
	
	Otherwise, returns the portion of the value of $sourceString beginning at the position indicated by the value of $startingLoc and continuing 
	for the number of characters indicated by the value of $length. More specifically, returns the characters in $sourceString whose position $p obeys:
	
	fn:round($startingLoc) <= $p < fn:round($startingLoc) + fn:round($length)
	
	In the above computation, the rules for op:numeric-less-than() and op:numeric-greater-than() apply.
	
	If $startingLoc is zero or negative, the substring includes characters from the beginning of the $sourceString.
	
	If $length is not specified, the substring includes characters to the end of $sourceString.
	
	If $length is greater than the number of characters in the value of $sourceString following $startingLoc, the substring includes characters to the end of $sourceString.
	
	The first character of a string is located at position 1, not position 0.
	
	Note:
	
	The position and length given in the second and (optional) third argument relate to the number of XML characters in the string 
	(or equivalently, the number of Unicode code points). Some implementations may represent a code point above xFFFF using two 16-bit 
	values known as a surrogate pair. A surrogate pair counts as one character, not two.
	*/
	public void visit(FunctionSUBSTRING arg) throws XQueryException {
		boolean hasLength = (arg.getArguments().size() == 3) ? true : false;
		String sourceString = null;
		returnType.push(UNKNOWN_TYPE);
		arg.getArgument(0).accept(this);
		if (resvalue != null) {
			if (resvalue instanceof String)
				sourceString = (String) resvalue;
			else
				sourceString = resvalue.toString();
		} else if (resnodes != null && resnodes.size() == 1) {
			StringBuffer sb = new StringBuffer();
			DOMUtils.getStringValue((TypedNode) resnodes.get(0), sb);
			sourceString = sb.toString();
		} else if (resnodes != null && resnodes.size() > 1) {
			throw new XQueryException("Evaluation : incorrect first argument for function substring");
		}
		if (sourceString == null) {
			clear();
			return;
		}
		double startingLoc = 0;
		arg.getArgument(1).accept(this);
		if (resvalue != null) { // do nothing
		} else if (resnodes != null && resnodes.size() == 1) {
			TypedNode tmpnode = (TypedNode) ((TypedNode) resnodes.get(0)).getFirstChild();
			if (tmpnode == null)
				tmpnode = (TypedNode) resnodes.get(0);
			resvalue = (Comparable) tmpnode.getTypedValue();
		} else
			throw new XQueryException("Evaluation : incorrect second argument for function substring");
		if (resvalue instanceof Number) {
			startingLoc = ((Number) resvalue).doubleValue();
		} else if (resvalue instanceof String) {
			try {
				startingLoc = Double.parseDouble((String) resvalue);
			} catch (Exception e) {
			}
		} else
			throw new XQueryException("Evaluation : incorrect second argument for function substring");
		if (startingLoc < 1)
			startingLoc = 1;
		double length = sourceString.length();
		if (hasLength) {
			arg.getArgument(2).accept(this);
			if (resvalue != null) { // do nothing
			} else if (resnodes != null && resnodes.size() == 1) {
				TypedNode tmpnode = (TypedNode) ((TypedNode) resnodes.get(0)).getFirstChild();
				if (tmpnode == null)
					tmpnode = (TypedNode) resnodes.get(0);
				resvalue = (Comparable) tmpnode.getTypedValue();
			} else
				throw new XQueryException("Evaluation : incorrect third argument for function substring");
			if (resvalue instanceof Number) {
				length = ((Number) resvalue).doubleValue();
			} else if (resvalue instanceof String) {
				try {
					length = Double.parseDouble((String) resvalue);
				} catch (Exception e) {
				}
			} else
				throw new XQueryException("Evaluation : incorrect third argument for function substring");
			if (startingLoc + length > sourceString.length())
				hasLength = false;
		}
		returnType.pop();
		if (hasLength)
			resvalue = sourceString.substring((int) Math.round(startingLoc) - 1, (int) (Math.round(startingLoc) + Math.round(length)) - 1);
		else
			resvalue = sourceString.substring((int) Math.round(startingLoc) - 1);
		Integer rettype = (Integer) returnType.peek();
		if (rettype == NODE_TYPE) {
			ArrayList nodes = new ArrayList(1);
			TypedValue text = docimpl.createTypedValue(resvalue, this.schemamanager.getStringType());
			nodes.add(text);
			setResnodes(nodes);
			return;
		} else { //if (rettype == VALUE_TYPE) {
			setValue(resvalue);
			return;
		}
		//clear();
	}

	//-----------------------------------------------------------
	// MATHEMATICAL FUNCTIONS
	//-----------------------------------------------------------
	public void visit(FunctionABS arg) throws XQueryException {
		returnType.push(UNKNOWN_TYPE);
		arg.getArgument(0).accept(this);
		returnType.pop();
		if (resvalue != null) { // do nothing
		} else if (resnodes != null && resnodes.size() == 1) {
			TypedNode tmpnode = (TypedNode) ((TypedNode) resnodes.get(0)).getFirstChild();
			if (tmpnode == null)
				tmpnode = (TypedNode) resnodes.get(0);
			resvalue = (Comparable) tmpnode.getTypedValue();
			if (!(resvalue instanceof Number))
				throw new XQueryException("Could not apply function abs");
		} else
			throw new XQueryException("Could not apply function abs");

		if (resvalue instanceof Long)
			resvalue = new Long(Math.abs(((Long) resvalue).longValue()));
		else if (resvalue instanceof BigDecimal)
			resvalue = ((BigDecimal) resvalue).abs();
		else if (resvalue instanceof Float)
			resvalue = new Float(Math.abs(((Float) resvalue).floatValue()));
		else if (resvalue instanceof Double)
			resvalue = new Double(Math.abs(((Double) resvalue).doubleValue()));

		Integer rettype = (Integer) returnType.peek();
		if (rettype == VALUE_TYPE) {
			setValue(resvalue);
			return;
		} else { // (rettype == NODE_TYPE) {
			ArrayList nodes = new ArrayList(1);
			TypedValue text = docimpl.createTypedValue(resvalue, this.getSimpleType(resvalue));
			nodes.add(text);
			setResnodes(nodes);
			return;
		}
		// clear();
	}
	public void visit(FunctionCEILING arg) throws XQueryException {
		returnType.push(UNKNOWN_TYPE);
		arg.getArgument(0).accept(this);
		returnType.pop();
		if (resvalue != null) { // do nothing
		} else if (resnodes != null && resnodes.size() == 1) {
			TypedNode tmpnode = (TypedNode) ((TypedNode) resnodes.get(0)).getFirstChild();
			if (tmpnode == null)
				tmpnode = (TypedNode) resnodes.get(0);
			resvalue = (Comparable) tmpnode.getTypedValue();
			if (!(resvalue instanceof Number))
				throw new XQueryException("Could not apply function ceiling");
		} else
			throw new XQueryException("Could not apply function ceiling");

		if (resvalue instanceof Long) { // do nothing
		} else if (resvalue instanceof BigDecimal)
			resvalue = ((BigDecimal) resvalue).setScale(0, BigDecimal.ROUND_CEILING);
		else if (resvalue instanceof Float)
			resvalue = new Float(Math.ceil(((Float) resvalue).floatValue()));
		else if (resvalue instanceof Double)
			resvalue = new Double(Math.ceil(((Double) resvalue).doubleValue()));

		Integer rettype = (Integer) returnType.peek();
		if (rettype == VALUE_TYPE) {
			setValue(resvalue);
			return;
		} else { // (rettype == NODE_TYPE) {
			ArrayList nodes = new ArrayList(1);
			TypedValue text = docimpl.createTypedValue(resvalue, this.getSimpleType(resvalue));
			nodes.add(text);
			setResnodes(nodes);
			return;
		}
		// clear();
	}
	public void visit(FunctionFLOOR arg) throws XQueryException {
		returnType.push(UNKNOWN_TYPE);
		arg.getArgument(0).accept(this);
		returnType.pop();
		if (resvalue != null) { // do nothing
		} else if (resnodes != null && resnodes.size() == 1) {
			TypedNode tmpnode = (TypedNode) ((TypedNode) resnodes.get(0)).getFirstChild();
			if (tmpnode == null)
				tmpnode = (TypedNode) resnodes.get(0);
			resvalue = (Comparable) tmpnode.getTypedValue();
			if (!(resvalue instanceof Number))
				throw new XQueryException("Could not apply function floor");
		} else
			throw new XQueryException("Could not apply function floor");

		if (resvalue instanceof Long) { // do nothing
		} else if (resvalue instanceof BigDecimal)
			resvalue = ((BigDecimal) resvalue).setScale(0, BigDecimal.ROUND_FLOOR);
		else if (resvalue instanceof Float)
			resvalue = new Float(Math.floor(((Float) resvalue).floatValue()));
		else if (resvalue instanceof Double)
			resvalue = new Double(Math.floor(((Double) resvalue).doubleValue()));

		Integer rettype = (Integer) returnType.peek();
		if (rettype == VALUE_TYPE) {
			setValue(resvalue);
			return;
		} else { // (rettype == NODE_TYPE) {
			ArrayList nodes = new ArrayList(1);
			TypedValue text = docimpl.createTypedValue(resvalue, this.getSimpleType(resvalue));
			nodes.add(text);
			setResnodes(nodes);
			return;
		}
		// clear();
	}
	public void visit(FunctionROUND arg) throws XQueryException {
		returnType.push(UNKNOWN_TYPE);
		arg.getArgument(0).accept(this);
		returnType.pop();
		if (resvalue != null) { // do nothing
		} else if (resnodes != null && resnodes.size() == 1) {
			TypedNode tmpnode = (TypedNode) ((TypedNode) resnodes.get(0)).getFirstChild();
			if (tmpnode == null)
				tmpnode = (TypedNode) resnodes.get(0);
			resvalue = (Comparable) tmpnode.getTypedValue();
			if (!(resvalue instanceof Number))
				throw new XQueryException("Could not apply function round");
		} else
			throw new XQueryException("Could not apply function round");

		if (resvalue instanceof Long) { // do nothing
		} else if (resvalue instanceof BigDecimal)
			resvalue = ((BigDecimal) resvalue).setScale(0, BigDecimal.ROUND_UP);
		else if (resvalue instanceof Float)
			resvalue = new BigDecimal(Math.round(((Float) resvalue).floatValue()));
		else if (resvalue instanceof Double)
			resvalue = new BigDecimal(Math.round(((Double) resvalue).doubleValue()));

		Integer rettype = (Integer) returnType.peek();
		if (rettype == VALUE_TYPE) {
			setValue(resvalue);
			return;
		} else { // (rettype == NODE_TYPE) {
			ArrayList nodes = new ArrayList(1);
			TypedValue text = docimpl.createTypedValue(resvalue, this.getSimpleType(resvalue));
			nodes.add(text);
			setResnodes(nodes);
			return;
		}
		// clear();
	}

	/*
	14.1.4 fn:number
	fn:number() as xs:double 
	fn:number($srcval as item?) as xs:double 
	Returns the value indicated by $srcval or, if $srcval is not specified, the context node, converted to an xs:double. If there is no context node (that is, if there is no context item, or if the context item is not a node), NaN is returned.
	
	If $srcval is the empty sequence, returns the xs:double value NaN.
	
	If $srcval is atomic, returns the value obtained by converting it to xs:double following the rules of 17.8 Casting to numeric types.
	
	If $srcval is a node with an atomic type, returns that value converted to xs:double following the rules of 17.8 Casting to numeric types.
	
	Otherwise, converts $srcval to an xs:string as if by a call to the fn:string() function and then converts the result to an xs:double following the rules of 17.8 Casting to numeric types.
	
	If the conversion to xs:double fails because the lexical representation is not a valid lexical representation of a numeric simple type as defined in [XML Schema Part 2: Datatypes], returns the xs:double value NaN.
	*/
	public void visit(FunctionNUMBER arg) throws XQueryException {
		returnType.push(UNKNOWN_TYPE);
		arg.getArgument(0).accept(this);
		returnType.pop();
		if (resvalue != null) { // do nothing
		} else if (resnodes != null && resnodes.size() == 1) {
			TypedNode tmpnode = (TypedNode) ((TypedNode) resnodes.get(0)).getFirstChild();
			if (tmpnode == null)
				tmpnode = (TypedNode) resnodes.get(0);
			resvalue = (Comparable) tmpnode.getTypedValue();
			if (resvalue instanceof Number) {
				resvalue = new Double(((Number) resvalue).doubleValue());
			} else if (resvalue instanceof String) {
				try {
					resvalue = new Double(Double.parseDouble((String) resvalue));
				} catch (Exception e) { // do nothing
				}
			}
		} else if (resnodes != null && resnodes.size() > 1) {
			throw new XQueryException("Evaluation : incorrect argument for function number");
		}
		if (resvalue == null) {
			resvalue = new Double(Double.NaN);
		}
		Integer rettype = (Integer) returnType.peek();
		if (rettype == NODE_TYPE) {
			ArrayList nodes = new ArrayList(1);
			TypedValue text = docimpl.createTypedValue(resvalue, this.schemamanager.getDoubleType());
			nodes.add(text);
			setResnodes(nodes);
			return;
		} else { // (rettype == VALUE_TYPE) {
			setValue(resvalue);
			return;
		}
		// clear();
	}

	//-----------------------------------------------------------
	// CAST/CONSTRUCTOR FUNCTIONS
	//-----------------------------------------------------------
	public void visit(FunctionDOUBLE arg) throws XQueryException {
		returnType.push(UNKNOWN_TYPE);
		arg.getArgument(0).accept(this);
		returnType.pop();
		if (resvalue != null) { // do nothing
		} else if (resnodes != null && resnodes.size() == 1) {
			TypedNode tmpnode = (TypedNode) ((TypedNode) resnodes.get(0)).getFirstChild();
			if (tmpnode == null)
				tmpnode = (TypedNode) resnodes.get(0);
			resvalue = (Comparable) tmpnode.getTypedValue();
		} else
			throw new XQueryException("Could not create double");

		if (resvalue instanceof Double) { // do nothing
		} else if (resvalue instanceof Number) {
			resvalue = new Double(((Number) resvalue).doubleValue());
		} else if (resvalue instanceof String) {
			try {
				resvalue = new Double(Double.parseDouble((String) resvalue));
			} catch (Exception e) {
			}
		} else
			throw new XQueryException("Could not create double");
		Integer rettype = (Integer) returnType.peek();
		if (rettype == VALUE_TYPE) {
			setValue(resvalue);
			return;
		} else { // (rettype == NODE_TYPE) {
			ArrayList nodes = new ArrayList(1);
			TypedValue text = docimpl.createTypedValue(resvalue, this.schemamanager.getDoubleType());
			nodes.add(text);
			setResnodes(nodes);
			return;
		}
		//clear();
	}

	public void visit(FunctionINTEGER arg) throws XQueryException {
		returnType.push(UNKNOWN_TYPE);
		arg.getArgument(0).accept(this);
		returnType.pop();
		if (resvalue != null) { // do nothing
		} else if (resnodes != null && resnodes.size() == 1) {
			TypedNode tmpnode = (TypedNode) ((TypedNode) resnodes.get(0)).getFirstChild();
			if (tmpnode == null)
				tmpnode = (TypedNode) resnodes.get(0);
			resvalue = (Comparable) tmpnode.getTypedValue();
		} else
			throw new XQueryException("Could not create integer");

		if (resvalue instanceof Long) { // do nothing
		} else if (resvalue instanceof Number) {
			resvalue = new Long(((Number) resvalue).longValue());
		} else if (resvalue instanceof String) {
			try {
				resvalue = new Long((String) resvalue);
			} catch (Exception e) {
			}
		} else
			throw new XQueryException("Could not create integer");
		Integer rettype = (Integer) returnType.peek();
		if (rettype == VALUE_TYPE) {
			setValue(resvalue);
			return;
		} else { // (rettype == NODE_TYPE) {
			ArrayList nodes = new ArrayList(1);
			TypedValue text = docimpl.createTypedValue(resvalue, this.schemamanager.getIntegerType());
			nodes.add(text);
			setResnodes(nodes);
			return;
		}
		//clear();
	}

	public void visit(FunctionDECIMAL arg) throws XQueryException {
		returnType.push(UNKNOWN_TYPE);
		arg.getArgument(0).accept(this);
		returnType.pop();
		if (resvalue != null) { // do nothing
		} else if (resnodes != null && resnodes.size() == 1) {
			TypedNode tmpnode = (TypedNode) ((TypedNode) resnodes.get(0)).getFirstChild();
			if (tmpnode == null)
				tmpnode = (TypedNode) resnodes.get(0);
			resvalue = (Comparable) tmpnode.getTypedValue();
		} else
			throw new XQueryException("Could not create decimal");

		if (resvalue instanceof BigDecimal) { // do nothing
		} else if (resvalue instanceof Number) {
			resvalue = new BigDecimal(((Number) resvalue).doubleValue());
		} else if (resvalue instanceof String) {
			try {
				resvalue = new BigDecimal((String) resvalue);
			} catch (Exception e) {
			}
		} else
			throw new XQueryException("Could not create decimal");
		Integer rettype = (Integer) returnType.peek();
		if (rettype == VALUE_TYPE) {
			setValue(resvalue);
			return;
		} else { // (rettype == NODE_TYPE) {
			ArrayList nodes = new ArrayList(1);
			TypedValue text = docimpl.createTypedValue(resvalue, this.schemamanager.getDecimalType());
			nodes.add(text);
			setResnodes(nodes);
			return;
		}
		//clear();
	}

	//-----------------------------------------------------------
	// OTHER FUNCTIONS
	//-----------------------------------------------------------

	public void visit(FunctionDATA arg) throws XQueryException {
		returnType.push(UNKNOWN_TYPE);
		arg.getArgument(0).accept(this);
		returnType.pop();
		Integer rettype = (Integer) returnType.peek();
		if (resvalue != null) {
			if (rettype == VALUE_TYPE) {
				setValue(resvalue);
				return;
			} else { // (rettype == NODE_TYPE) {
				ArrayList nodes = new ArrayList(1);
				TypedValue text = docimpl.createTypedValue(resvalue, this.getSimpleType(resvalue));
				nodes.add(text);
				setResnodes(nodes);
				return;
			}
		} else if (resnodes != null) {
			ArrayList tmplist = new ArrayList(resnodes.size());
			for (int i = 0; i < resnodes.size(); i++) {
				TypedNode tmpnode = (TypedNode) ((TypedNode) resnodes.get(i)).getFirstChild();
				// change   02/09/2003
				if (tmpnode == null || tmpnode.getTypedValue() == null)
					tmpnode = (TypedNode) resnodes.get(i);
				Comparable comp = (Comparable) tmpnode.getTypedValue();
				if (comp != null)
					tmplist.add(docimpl.createTypedValue(comp, this.getSimpleType(comp)));
			}
			if (rettype == VALUE_TYPE) {
				if (tmplist.size() == 1) {
					TypedNode tmpnode = (TypedNode) ((TypedNode) tmplist.get(0)).getFirstChild();
					if (tmpnode == null)
						tmpnode = (TypedNode) tmplist.get(0);
					setValue((Comparable) tmpnode.getTypedValue());
					return;
				}
			} else { // (rettype == NODE_TYPE) {
				setResnodes(tmplist);
				return;
			}
		} else
			throw new XQueryException("Could not apply function data");
		// clear();
	}

	public void visit(FunctionEXISTS arg) throws XQueryException {
		returnType.push(NODE_TYPE);
		arg.getArgument(0).accept(this);
		returnType.pop();
		setVerdict(resnodes != null && !resnodes.isEmpty());
	}

	public void visit(FunctionEMPTY arg) throws XQueryException {
		if (buffertuple != null && !buffertuple.isEmpty()) {
			setVerdict(false);
			return;
		}
		returnType.push(NODE_TYPE);
		arg.getArgument(0).accept(this);
		returnType.pop();
		setVerdict(resnodes == null || resnodes.isEmpty());
	}

	public void visit(FunctionNOT arg) throws XQueryException {
		arg.getArgument(0).accept(this);
		setVerdict(!verdict);
	}

	public void visit(FunctionDISTINCT_VALUES arg) throws XQueryException {
		returnType.push(NODE_TYPE);
		arg.getArgument(0).accept(this);
		returnType.pop();
		if (resnodes != null) {
			// cloning must be done otherwise context tuple could be changed
			resnodes = (ArrayList) resnodes.clone();
			for (int i = 0; i < resnodes.size(); i++) {
				TypedNode nodei = (TypedNode) resnodes.get(i);
				for (int j = 0; j < i; j++) {
					TypedNode nodej = (TypedNode) resnodes.get(j);
					if (DOMUtils.compareTree(nodei, nodej)) {
						resnodes.remove(i);
						i--;
						break;
					}
				}
			}
		}
		setResnodes(resnodes);
	}

	public void visit(FunctionDEEP_EQUALS arg) throws XQueryException {
		resulttuple = null;
		resnodes = null;
		XQueryExpression arg0 = arg.getArgument(0);
		XQueryExpression arg1 = arg.getArgument(1);
		if (arg0 instanceof Value && arg1 instanceof Value)
			processComparison(arg0, arg1, null, null, null, null, Constants.EQ_COMPOP);
		else {
			if (arg0 instanceof Value) {
				if (tuple != null) {
					ArrayList nodes1 = tuple.getNodesAtIndex(tuple.getParent().getPathIndex(arg1.toString()));
					if (nodes1 != null && nodes1.size() == 1) {
						processComparison(arg0, arg1, (TypedNode) null, (TypedNode) nodes1.get(0), null, null, Constants.EQ_COMPOP);
					}
				} else if (tuplelist != null) {
					for (int i = 0; i < tuplelist.size(); i++) {
						Tuple tuplei = (Tuple) tuplelist.get(i);
						ArrayList nodes1 = tuplei.getNodesAtIndex(tuplei.getParent().getPathIndex(arg1.toString()));
						if (nodes1 != null && nodes1.size() == 1) {
							processComparison(arg0, arg1, (TypedNode) null, (TypedNode) nodes1.get(0), null, null, Constants.EQ_COMPOP);
							break;
						}
					}
				}
			} else if (arg1 instanceof Value) {
				if (tuple != null) {
					ArrayList nodes0 = tuple.getNodesAtIndex(tuple.getParent().getPathIndex(arg0.toString()));
					if (nodes0 != null && nodes0.size() == 1) {
						processComparison(arg0, arg1, (TypedNode) nodes0.get(0), (TypedNode) null, null, null, Constants.EQ_COMPOP);
					}
				} else if (tuplelist != null) {
					for (int i = 0; i < tuplelist.size(); i++) {
						Tuple tuplei = (Tuple) tuplelist.get(i);
						ArrayList nodes0 = tuplei.getNodesAtIndex(tuplei.getParent().getPathIndex(arg0.toString()));
						if (nodes0 != null && nodes0.size() == 1) {
							processComparison(arg0, arg1, (TypedNode) nodes0.get(0), (TypedNode) null, null, null, Constants.EQ_COMPOP);
							break;
						}
					}
				}
			} else {
				if (this.tuple != null) {
					//org.xquark.xml.misc.Debug.print("tuple = " + tuple.toCompleteString());
					ArrayList nodes0 = tuple.getNodesAtIndex(tuple.getParent().getPathIndex(arg0.toString()));
					ArrayList nodes1 = tuple.getNodesAtIndex(tuple.getParent().getPathIndex(arg1.toString()));
					verdict = DOMUtils.compareTree(nodes0, nodes1);
				} else if (tuplelist != null) {
					ArrayList nodes0 = null;
					ArrayList nodes1 = null;
					for (int i = 0; i < tuplelist.size(); i++) {
						Tuple tmptuple = (Tuple) tuplelist.get(i);
						if (nodes0 == null)
							nodes0 = tmptuple.getNodesAtIndex(tmptuple.getParent().getPathIndex(arg0.toString()));
						if (nodes1 == null)
							nodes1 = tmptuple.getNodesAtIndex(tmptuple.getParent().getPathIndex(arg1.toString()));
						if (nodes0 != null && nodes1 != null)
							break;
					}
					verdict = DOMUtils.compareTree(nodes0, nodes1);
				} else if (buffertuple != null) {
					throw new XQueryException("FunctionVALUE_EQUALS on BufferTuple not yet implemented");
				}
			}
		}
	}

	//-----------------------------------------------------------
	// PRIVATE METHODS
	//-----------------------------------------------------------

	private SimpleType getSimpleType(Comparable obj) {
		if (obj instanceof Long)
			return this.schemamanager.getIntegerType();
		else if (obj instanceof BigDecimal)
			return this.schemamanager.getDecimalType();
		else if (obj instanceof Float)
			return this.schemamanager.getFloatType();
		else if (obj instanceof Double)
			return this.schemamanager.getDoubleType();
		else if (obj instanceof String)
			return this.schemamanager.getStringType();
		else if (obj instanceof Date)
			return this.schemamanager.getDateTimeType();
		return null;
	}

	private void processComparison(XQueryExpression exp1, XQueryExpression exp2, TypedNode tree1, TypedNode tree2, Comparable value1, Comparable value2, int operator) {
		//Comparable value1 = null;
		boolean doublecompare = false;
		double value1double = 0;
		if (value1 == null) {
			if (exp1 instanceof ValueInteger) {
				value1 = new Long(((ValueInteger) exp1).getIntValue().longValue());
				//				value1double = ((Number) value1).doubleValue();
			} else if (exp1 instanceof ValueDecimal) {
				value1 = ((ValueDecimal) exp1).getDecimalValue();
				//				value1double = ((Number) value1).doubleValue();
			} else if (exp1 instanceof ValueFloat) {
				value1 = ((ValueFloat) exp1).getFloatValue();
				//				value1double = ((Number) value1).doubleValue();
			} else if (exp1 instanceof ValueDouble) {
				value1 = ((ValueDouble) exp1).getDoubleValue();
				//				value1double = ((Number) value1).doubleValue();
			} else if (exp1 instanceof ValueString) {
				value1 = ((ValueString) exp1).getValue();
			} else if (tree1 != null) {
				if (tree1.getTypedValue() != null)
					value1 = (Comparable) tree1.getTypedValue();
				else
					value1 = (Comparable) DOMUtils.getTextLeaf(tree1);
				//				if (value1 != null && !(value1 instanceof String) && !(value1 instanceof Timestamp)) {
				//					value1double = ((Number) value1).doubleValue();
				//					doublecompare = true;
				//				}
			}
		}
		if (value1 != null && !(value1 instanceof String) && !(value1 instanceof Timestamp) && !(value1 instanceof Date)) {
			value1double = ((Number) value1).doubleValue();
			doublecompare = true;
		}
		//Comparable value2 = null;
		double value2double = 0;
		if (value2 == null) {
			if (exp2 instanceof ValueInteger) {
				value2 = new Long(((ValueInteger) exp2).getIntValue().longValue());
				//				value2double = ((Number) value2).doubleValue();
			} else if (exp2 instanceof ValueDecimal) {
				value2 = ((ValueDecimal) exp2).getDecimalValue();
				//				value2double = ((Number) value2).doubleValue();
			} else if (exp2 instanceof ValueFloat) {
				value2 = ((ValueFloat) exp2).getFloatValue();
				//				value2double = ((Number) value2).doubleValue();
			} else if (exp2 instanceof ValueDouble) {
				value2 = ((ValueDouble) exp2).getDoubleValue();
				//				value2double = ((Number) value2).doubleValue();
			} else if (exp2 instanceof ValueString)
				value2 = ((ValueString) exp2).getValue();
			else if (tree2 != null) {
				if (tree2.getTypedValue() != null)
					value2 = (Comparable) tree2.getTypedValue();
				else
					value2 = (Comparable) DOMUtils.getTextLeaf(tree2);
				//				if (value2 != null && !(value2 instanceof String) && !(value2 instanceof Timestamp)) {
				//					value2double = ((Number) value2).doubleValue();
				//				}
				//				else
				//					doublecompare = false;
			}
		}
		if (value2 != null && !(value2 instanceof String) && !(value2 instanceof Timestamp) && !(value2 instanceof Date)) {
			value2double = ((Number) value2).doubleValue();
		} else
			doublecompare = false;
		//		if (value1 == null && value2 == null /*&& tree1 == null && tree2 == null*/)
		//			if ((operator == Constants.GEQ_COMPOP) || (operator == Constants.LEQ_COMPOP) || (operator == Constants.EQ_COMPOP)) { verdict = true ; return ; }
		//			else { verdict = false ; return ; }
		/*
		if (value1 == null && value2 != null) { verdict = false ; return ; }
		if (value2 == null && value1 != null) { verdict = false ; return ; }
		if (value1 != null && value2 != null) {
			if (value1 instanceof String && !(value2 instanceof String)) { verdict = false ; return ; }
			if (value2 instanceof String && !(value1 instanceof String)) { verdict = false ; return ; }
		}
		*/
		boolean stringcomp = false;
		if (value1 == null) {
			if (value2 != null) {
				setVerdict(false);
				return;
			}
		} else {
			if (value2 == null) {
				setVerdict(false);
				return;
			} else {
				if (value1 instanceof String) {
					if (!(value2 instanceof String)) {
						value2 = value2.toString();
						if (value2 == null) {
							setVerdict(false);
							return;
						}
						stringcomp = true;
					}
				} else if (value2 instanceof String) {
					value1 = value1.toString();
					if (value1 == null) {
						setVerdict(false);
						return;
					}
					stringcomp = true;
				}
			}
		}
		//if ((tree1 == null) && (tree2 != null)) { verdict = false ; return ; }
		//if ((tree2 == null) && (tree1 != null)) { verdict = false ; return ; }
		switch (operator) {
			case Constants.LT_COMPOP :
				if ((value1 == null) && (value2 == null))
					setVerdict(false);
				else if (stringcomp || !doublecompare)
					setVerdict((value1.compareTo(value2) < 0));
				else
					setVerdict((value1double < value2double));
				return;
			case Constants.GT_COMPOP :
				if ((value1 == null) && (value2 == null))
					setVerdict(false);
				else if (stringcomp || !doublecompare)
					setVerdict((value1.compareTo(value2) > 0));
				else
					setVerdict((value1double > value2double));
				return;
			case Constants.GEQ_COMPOP :
				if ((value1 == null) && (value2 == null))
					setVerdict(true);
				else if (stringcomp || !doublecompare)
					setVerdict((value1.compareTo(value2) >= 0));
				else
					setVerdict((value1double >= value2double));
				return;
			case Constants.LEQ_COMPOP :
				if ((value1 == null) && (value2 == null))
					setVerdict(true);
				else if (stringcomp || !doublecompare)
					setVerdict((value1.compareTo(value2) <= 0));
				else
					setVerdict((value1double <= value2double));
				return;
			case Constants.EQ_COMPOP :
				if ((value1 == null) && (value2 == null))
					setVerdict(true);
				else if (stringcomp || !doublecompare)
					setVerdict((value1.compareTo(value2) == 0));
				else
					setVerdict((value1double == value2double));
				return;
			case Constants.NEQ_COMPOP :
				if ((value1 == null) && (value2 == null))
					setVerdict(false);
				else if (stringcomp || !doublecompare)
					setVerdict((value1.compareTo(value2) != 0));
				else
					setVerdict((value1double != value2double));
				return;
				/*
				case Constants.EQQ_COMPOP :
					setVerdict(Tuple.compareTree(tree1, tree2));
					return;
				 */
		}
		clear();
	}

	private Number processOperation(XQueryExpression exp1, XQueryExpression exp2, TypedNode tree1, TypedNode tree2, Number value1, Number value2, int operator) throws XQueryException {
		if (value1 == null) {
			if (exp1 instanceof ValueInteger)
				value1 = new Long(((ValueInteger) exp1).getIntValue().longValue());
			else if (exp1 instanceof ValueDecimal)
				value1 = ((ValueDecimal) exp1).getDecimalValue();
			else if (exp1 instanceof ValueFloat)
				value1 = ((ValueFloat) exp1).getFloatValue();
			else if (exp1 instanceof ValueDouble)
				value1 = ((ValueDouble) exp1).getDoubleValue();
			else if (tree1 != null)
				if (tree1.getTypedValue() != null)
					value1 = (Number) tree1.getTypedValue();
				else
					value1 = (Number) DOMUtils.getTextLeaf(tree1);
		}
		if (value2 == null) {
			if (exp2 instanceof ValueInteger)
				value2 = new Long(((ValueInteger) exp2).getIntValue().longValue());
			else if (exp2 instanceof ValueDecimal)
				value2 = ((ValueDecimal) exp2).getDecimalValue();
			else if (exp2 instanceof ValueFloat)
				value2 = ((ValueFloat) exp2).getFloatValue();
			else if (exp2 instanceof ValueDouble)
				value2 = ((ValueDouble) exp2).getDoubleValue();
			else if (tree2 != null)
				if (tree2.getTypedValue() != null)
					value2 = (Number) tree2.getTypedValue();
				else
					value2 = (Number) DOMUtils.getTextLeaf(tree2);
		}

		boolean integerOp = false;
		boolean decimalOp = false;
		boolean floatOp = false;
		if (value1 instanceof Long) {
			if (value2 instanceof Long)
				integerOp = true;
			else if (value2 instanceof BigDecimal) {
				value1 = new BigDecimal(value1.longValue());
				decimalOp = true;
			}
		} else if (value1 instanceof BigDecimal) {
			if (value2 instanceof BigDecimal)
				decimalOp = true;
			else if (value2 instanceof Long) {
				value2 = new BigDecimal(value2.longValue());
				decimalOp = true;
			}
		} else if (value1 instanceof Float) {
			if (value2 instanceof Float)
				floatOp = true;
			if (!(value2 instanceof Double)) {
				value2 = new Float(value2.doubleValue());
				floatOp = true;
			}
		}

		switch (operator) {
			case Constants.DIVIDE_ARITHMETICS :
				//if (integerOp) {
				//					try {
				//						return value1.intValue() / value2.intValue();
				//					} catch (ArithmeticException ae) {
				//						throw new XQueryException("Division by zero");
				//					}					
				//				} else 
				if (decimalOp) {
					try {
						return ((BigDecimal) value1).divide((BigDecimal) value2, 38, BigDecimal.ROUND_UP);
					} catch (ArithmeticException ae) {
						throw new XQueryException("Division by zero");
					}
				} else if (floatOp)
					return new Float(value1.floatValue() / value2.floatValue());
				return new Double(value1.doubleValue() / value2.doubleValue());
			case Constants.MINUS_ARITHMETICS :
				if (integerOp)
					return new Long(value1.longValue() - value2.longValue());
				else if (decimalOp)
					return ((BigDecimal) value1).subtract((BigDecimal) value2);
				else if (floatOp)
					return new Float(value1.floatValue() - value2.floatValue());
				return new Double(value1.doubleValue() - value2.doubleValue());
			case Constants.MODULO_ARITHMETICS :
				if (integerOp)
					return new Long(value1.longValue() % value2.longValue());
				else if (floatOp)
					return new Float(value1.floatValue() % value2.floatValue());
				return new Double(value1.doubleValue() % value2.doubleValue());
			case Constants.MULTIPLY_ARITHMETICS :
				if (integerOp)
					return new Long(value1.longValue() * value2.longValue());
				else if (decimalOp)
					return ((BigDecimal) value1).multiply((BigDecimal) value2);
				else if (floatOp)
					return new Float(value1.floatValue() * value2.floatValue());
				return new Double(value1.doubleValue() * value2.doubleValue());
			case Constants.PLUS_ARITHMETICS :
				if (integerOp)
					return new Long(value1.longValue() + value2.longValue());
				else if (decimalOp)
					return ((BigDecimal) value1).add((BigDecimal) value2);
				else if (floatOp)
					return new Float(value1.floatValue() + value2.floatValue());
				return new Double(value1.doubleValue() + value2.doubleValue());
		}
		throw new XQueryException("Could not process operation");
	}

}
