/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */

package org.xquark.mediator.DOMUtils;

import java.util.ArrayList;

import org.xquark.mediator.algebra.AlgFLWR;
import org.xquark.mediator.algebra.Algebra;
import org.xquark.schema.SchemaManager;
import org.xquark.xpath.datamodel.TypedNode;
import org.xquark.xquery.parser.*;


public class FillExprWithTupleVisitor extends DefaultParserVisitor {
	// **********************************************************************
	// * VERSIONING
	// **********************************************************************
	private static final String RCSRevision = "$Revision: 1.2 $";
	private static final String RCSName = "$Name:  $";
	// **********************************************************************
	// * CLASS VARIABLES
	// **********************************************************************
	//private Tuple tuple = null;
	private ArrayList tuples = null;
	private XQueryExpression resultExpr = null;
	private SchemaManager schemamanager = null;
	private TypeVisitor typevisitor = null;
	// **********************************************************************
	// * INITIALIZATION
	// **********************************************************************
	/**
	 *
	 */
	public FillExprWithTupleVisitor(SchemaManager schemamanager, TypeVisitor typevisitor, Tuple tuple) {
		this.tuples = new ArrayList(1);
		tuples.add(tuple);
		this.schemamanager = schemamanager;
		this.typevisitor = typevisitor;
	}

	public FillExprWithTupleVisitor(SchemaManager schemamanager, TypeVisitor typevisitor, ArrayList tuples) {
		this.tuples = tuples;
		this.schemamanager = schemamanager;
		this.typevisitor = typevisitor;
	}

	//public XQueryExpression getResultExpr() { return resultExpr; }

	// **********************************************************************
	// * METHODS
	// **********************************************************************
	// special

	public void visit(Algebra arg) throws XQueryException {
		//org.xquark.xml.misc.Debug.print("Call FillExprWithTupleVisitor -> XQueryExpression ( " + arg.getClass().getName() + " ) : " + arg );
		arg.getExpression().accept(this);
		if (resultExpr != null)
			arg.setExpression(resultExpr);
		ArrayList children = arg.getChildren();
		if (children != null)
			for (int i = 0; i < children.size(); i++)
				visit((Algebra) children.get(i));
		if (arg.isQDB()) {
			AlgFLWR tmpdepmem = (AlgFLWR) arg;
			if (tmpdepmem.getDependencyList() != null)
				for (int i = 0; i < tmpdepmem.getDependencyList().size(); i++)
					visit((AlgFLWR) tmpdepmem.getDependencyList().get(i));
		}
		arg.getVarsDependingOn().clear();
		resultExpr = null;
	}

	// **********************************************************************
	// * VISITATION
	// **********************************************************************
	public void visit(XQueryExpression arg) throws XQueryException {
		resultExpr = null;
		//org.xquark.xml.misc.Debug.print("Call FillExprWithTupleVisitor -> XQueryExpression ( " + arg.getClass().getName() + " ) : " + arg );
	}

	public void visit(FunctionCall arg) throws XQueryException {
		//org.xquark.xml.misc.Debug.print("Call FillExprWithTupleVisitor -> FunctionCall : " + arg );
		ArrayList arguments = arg.getArguments();
		if (arguments != null) {
			for (int i = 0; i < arguments.size(); i++) {
				XQueryExpression argi = (XQueryExpression) arguments.get(i);
				argi.accept(this);
				if (resultExpr != null)
					arguments.set(i, resultExpr);
			}
		}
		resultExpr = null;
	}

	public void visit(FLWRExpression arg) throws XQueryException {
		//org.xquark.xml.misc.Debug.print("Call FillExprWithTupleVisitor -> FLWRExpression : " + arg );
		for (int i = 0; i < arg.getVariables().size(); i++) {
			Variable vari = (Variable) arg.getVariables().get(i);
			vari.getExpression().accept(this);
			if (resultExpr != null)
				vari.setExpression(resultExpr,typevisitor);
		}
		if (arg.getWhereClause() != null) {
			arg.getWhereClause().accept(this);
			if (resultExpr != null)
				arg.setWhereClause(resultExpr);
		}
		arg.getReturnClause().accept(this);
		if (resultExpr != null)
			arg.setReturnClause(resultExpr);
		resultExpr = null;
	}

	public void visit(XQueryBinaryOperatorExpression arg) throws XQueryException {
		//org.xquark.xml.misc.Debug.print("Call FillExprWithTupleVisitor -> XQueryBinaryOperatorExpression : " + arg );
		arg.getExpression1().accept(this);
		if (resultExpr != null)
			arg.setExpression1(resultExpr);
		arg.getExpression2().accept(this);
		if (resultExpr != null)
			arg.setExpression2(resultExpr);
		resultExpr = null;
	}

	public void visit(LocatedExpression arg) throws XQueryException {
		//org.xquark.xml.misc.Debug.print("Call FillExprWithTupleVisitor -> LocatedExpression : " + arg );
		resultExpr = null;
		XQueryExpression step0expr = arg.getExpression();
		if (step0expr instanceof Variable) {

			EvaluationVisitor visitor = null;
			for (int j = 0; j < tuples.size(); j++) {

				if (j == 0)
					visitor = new EvaluationVisitor(this.schemamanager, (Tuple) tuples.get(j));
				else
					visitor.reset((Tuple) tuples.get(j));

				arg.accept(visitor);
				ArrayList tmplist = visitor.getResNodes();
				//org.xquark.xml.misc.Debug.print("tmplist : " + tmplist);
				if (tmplist != null && !tmplist.isEmpty()) {
					if (tmplist.size() == 1) {
						TypedNode tmpnode = (TypedNode) tmplist.get(0);
						Object strvalue = tmpnode.getTypedValue();
						if (strvalue == null) {
							strvalue = DOMUtils.getTextLeaf(tmpnode);
						}
						if (strvalue == null)
							throw new XQueryException("Unexpected value encountered while replacing variable");
						resultExpr = DOMUtils.getXQueryExpression(strvalue);
						if (resultExpr == null)
							throw new XQueryException("Unexpected type encountered while replacing variable");
					} else {
						ArrayList seqlist = new ArrayList();
						for (int i = 0; i < tmplist.size(); i++) {
							TypedNode tmpnode = (TypedNode) tmplist.get(i);
							Object strvalue = tmpnode.getTypedValue();
							if (strvalue == null) {
								strvalue = DOMUtils.getTextLeaf(tmpnode);
							}
							if (strvalue == null)
								throw new XQueryException("Unexpected value encountered while replacing variable");
							XQueryExpression tmpExpr = DOMUtils.getXQueryExpression(strvalue);
							if (tmpExpr == null)
								throw new XQueryException("Unexpected type encountered while replacing variable");
							seqlist.add(tmpExpr);
						}
						resultExpr = new XQueryExpressionSequence(seqlist, typevisitor);
						resultExpr.setParenthesis(true);
					}
					break;
				}
			}
		}
	}

	public void visit(Variable arg) throws XQueryException {
		//org.xquark.xml.misc.Debug.print("Call FillExprWithTupleVisitor -> Variable : " + arg );
		for (int j = 0; j < tuples.size(); j++) {
			ArrayList tmplist = ((Tuple) tuples.get(j)).getPath(arg.toString());
			//org.xquark.xml.misc.Debug.print("tmplist : " + tmplist);
			if (tmplist != null) {
				if (tmplist.size() == 1) {
					TypedNode tmpnode = (TypedNode) tmplist.get(0);
					
					Object strvalue = tmpnode.getTypedValue();
					if (strvalue == null) {
						strvalue = DOMUtils.getTextLeaf(tmpnode);
					}
					if (strvalue == null)
						throw new XQueryException("Unexpected value encountered while replacing variable");
					resultExpr = DOMUtils.getXQueryExpression(strvalue);					
					
					/*
					if (arg.getQType().isNumeric()) resultExpr = new ValueDecimal(strvalue,null);
					else if (arg.getQType().isString()) resultExpr = new ValueString(strvalue,null);
					else if (arg.getQType().isDate()) resultExpr = new ValueString(strvalue,null);
					else throw new XQueryException("Unexpected type encountered while replacing variable" );
					*/
					if (resultExpr == null)
						throw new XQueryException("Unexpected type encountered while replacing variable");
				} else {
					throw new XQueryException("Replacing variable by more than one value -> not implemented yet");
				}
				break;
			}
		}
	}

}
