/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */

package org.xquark.mediator.DOMUtils ;

import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.DefaultHandler;
import org.xquark.schema.Type;
import org.xquark.schema.validation.ElementPSVInfoset;
import org.xquark.schema.validation.PSVInfoset;
import org.xquark.schema.validation.SchemaValidationContext;
import org.xquark.xpath.datamodel.*;

/**
 *
 * This class computes consumes SAX events to produce a DOM tree.
 * input: document, top-level element
 * output: top-level element, with some new nodes underneath
 *
 * document is needed to grab DOM methods
 * element is needed to insert the nodes
 */

public class SAX2DOM extends DefaultHandler {
	// **********************************************************************
	// * VERSIONING
	// **********************************************************************
	private static final String RCSRevision = "$Revision: 1.2 $";
	private static final String RCSName = "$Name:  $";	
	// **********************************************************************
	// * CLASS VARIABLES
	// **********************************************************************
	private TypedDocumentImpl doc = null ;
	// Element top;
	private TypedNode currentNode = null ;
	//private Node root = null ;
	//private int toKeep = -1 ;
	
	//private ArrayList steps = null ;
	private Tuple tuple ;
	//private PathStack pathstack = null ;
	private int index = 0;
	private int depth = 0;
	private SchemaValidationContext svc = null;
	private StringBuffer ongoingchars = new StringBuffer();
	// ************************************************************************
	// * INITIALIZATION
	// ************************************************************************
	/**
	 *
	 */
	//new ValidatingSchemaHandler(
	public SAX2DOM(SchemaValidationContext svc) { this.svc = svc; doc = new TypedDocumentImpl() ; }
	
	public void init(Tuple tuple) { this.tuple = tuple ; index = 0;	}
	
	// ************************************************************************
	// * SAX Implementation
	// ************************************************************************
	/**
	 *
	 */
	public void characters(char[] ch, int start, int length) throws SAXException {
		// we create a TEXT node and add it to the current node
		/*
		Node n = doc.createTextNode(new String(ch, start, length));
		if (depth == 2) tuple.addNodeAtIndex(index, n) ;
		else currentNode.appendChild(n);
		*/
		ongoingchars.append(new String(ch, start, length));
	}
	
	//public void endDocument() throws SAXException { return; }
	
	public void endElement(String uri, String local, String raw) throws SAXException {
//org.xquark.xml.misc.Debug.print("uri = " + uri + " local = " + local + " raw = " + raw);
		boolean onlytext = false;
		if (depth >= 2) {
			ElementPSVInfoset epsvis = svc.getCurrentInfoset();
			Type type = epsvis.getType();
//org.xquark.xml.misc.Debug.print("type = " + type);
			if (ongoingchars.length() > 0) {
				TypedValueImpl tv = (TypedValueImpl)doc.createTextNode(ongoingchars.toString());
				if (type != null) {
					if (type.getValueType() != null) {
						tv.setType(type.getValueType());
						tv.setTypedValue(epsvis.getActualValue());
						tv.setNodeValue(epsvis.getNormalizedValue());
					}
				}
				else {
					tv.setType(this.svc.getSchemaManager().getAnySimpleType());
					tv.setTypedValue(ongoingchars.toString());
				}
				if (currentNode == null) { currentNode = tv; depth++; onlytext = true;}
				else currentNode.appendChild(tv);
				ongoingchars.setLength(0);
			}
//org.xquark.xml.misc.Debug.print("depth = " + depth);
			if (currentNode != null && !onlytext) {
				currentNode.setType(epsvis.getType());
				currentNode.setDeclaration(epsvis.getDeclaration());
			}
		}			
		depth--;
		switch (depth) {
			case 0 : index = 0; break;
			case 1 : index++; break;
			case 2 : 
				// setting type of element
				//currentNode.setType
				// add node to current tuple				
				tuple.addNodeAtIndex(index, currentNode) ; 
				if (onlytext) {	depth--; index++; }
				currentNode = null;
				break;
			default : currentNode = (TypedNode)currentNode.getParentNode();
		}
	}
	
	//public void ignorableWhitespace(char[] ch, int start, int length) throws SAXException { return; }
	
//	public void processingInstruction(String target, String data) throws SAXException {
//		// we create a PI node and add it to the current node
//		Node n = doc.createProcessingInstruction(target, data);
//		/*
//		if (currentNode == null) {
//			tuple.addNodeAtIndex(index, n) ;
//		}
//		else currentNode.appendChild(n);
//		return;
//		 */
//		if (depth == 2) tuple.addNodeAtIndex(index, n) ;
//		else currentNode.appendChild(n);
//	}
	
	//public void setDocumentLocator(Locator locator) { return; }
	
	public void startDocument() throws SAXException {
		/*pathstack = new PathStack() ;*/
		depth = 0;
		index = 0;
	}
	
	public void startElement(String uri, String local, String raw, Attributes atts) throws SAXException {
		if (depth >= 2) {
			TypedElement el = (TypedElement)doc.createElementNS(uri,local);
			ElementPSVInfoset epsvis = svc.getCurrentInfoset();
			for (int i=0; i<epsvis.getAttributeCount();i++) {
				PSVInfoset infoset = epsvis.getAttributePSVInfoset(i);
				TypedAttribute attr = (TypedAttribute)doc.createAttributeNS(infoset.getNamespaceURI(),infoset.getLocalName());
				attr.setNodeValue(infoset.getNormalizedValue());
				attr.setTypedValue(infoset.getActualValue());
				attr.setType(infoset.getType());
				attr.setDeclaration(infoset.getDeclaration());
				el.setAttributeNode(attr);
			}
			if (ongoingchars.length() > 0) {
				TypedValueImpl tv = (TypedValueImpl)doc.createTextNode(ongoingchars.toString());
				tv.setType(this.svc.getSchemaManager().getAnySimpleType());
				tv.setTypedValue(ongoingchars.toString());
				if (currentNode == null) throw new SAXException("Incorrect result!");
				currentNode.appendChild(tv);
				ongoingchars.setLength(0);
			}
			if (depth > 2) currentNode.appendChild(el);
			currentNode = el;
		}
		depth++;
		
	}
}

