/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */

package org.xquark.mediator.DOMUtils;

import java.io.IOException;
import java.util.ArrayList;

import org.w3c.dom.Node;
import org.xml.sax.ContentHandler;
import org.xml.sax.SAXException;
import org.xml.sax.ext.LexicalHandler;
import org.xquark.mediator.plan.Operator;
import org.xquark.mediator.plan.ResultSet;
import org.xquark.util.DOMReader;
import org.xquark.util.SAXConstants;
import org.xquark.xpath.datamodel.TypedNode;
import org.xquark.xquery.parser.XQueryException;

/**
 * This is the implementation of a "semi-structured" tuple.
 * As a semi-structured data is tree-based, we can't just store as an
 * arraylist of Strings.
 * A tuple is a tree (DOM representation for now) indexed with paths.
 *
 */
public class Tuple {
	// **********************************************************************
	// * VERSIONING
	// **********************************************************************
	private static final String RCSRevision = "$Revision: 1.3 $";
	private static final String RCSName = "$Name:  $";
	// **********************************************************************
	// * CLASS VARIABLES
	// **********************************************************************
	//protected static DOM2SAX dom2sax = null ;
	//static { dom2sax = new DOM2SAX() ; }

	//private ArrayList roots = null ;
	private int size = 0;
	private int idsize = 0;

	/**
	 * TypedNode are referenced by paths, and must be classified in the *same order*
	 * that is given in paths of BufferTuple.
	 * As there can be multivaluated node, this is an arraylist of arraylist
	 * of references on node.
	 */
	private ArrayList indexnodes = null;
	private ArrayList identifiers = null;
	private BufferTuple parent = null;

	// ***********************************************************************
	// * INITIALIZATION
	// ***********************************************************************
	/**
	 * Allocate a new Tuple.
	 */

	public Tuple(Operator algebra) {
		this.size = algebra.getSize();
		this.idsize = algebra.getIdSize();
		if (size > 0) {
			indexnodes = new ArrayList(size);
			for (int i = 0; i < size; i++)
				indexnodes.add(null);
		}
		if (idsize > 0) {
			identifiers = new ArrayList(idsize);
			for (int i = 0; i < idsize; i++)
				identifiers.add(null);
		}
		//indexnodes = new ArrayList[algebra.getSize()];		
		//identifiers = new Object[algebra.getIdSize()];
		//roots = new ArrayList() ;
	}

	public Tuple(int size) {
		this.size = size;
		if (size > 0) {
			indexnodes = new ArrayList(size);
			for (int i = 0; i < size; i++)
				indexnodes.add(null);
		}
		//indexnodes = new ArrayList[size];
		//roots = new ArrayList() ;
	}

	public Tuple(int size, int idsize) {
		this.size = size;
		this.idsize = idsize;
		if (size > 0) {
			indexnodes = new ArrayList(size);
			for (int i = 0; i < size; i++)
				indexnodes.add(null);
		}
		if (idsize > 0) {
			identifiers = new ArrayList(idsize);
			for (int i = 0; i < idsize; i++)
				identifiers.add(null);
		}
		//indexnodes = new ArrayList[size];
		//identifiers = new Object[idsize];
		//roots = new ArrayList() ;
	}

	public Tuple(ResultSet rs) {
		this.size = rs.getAlgebra().getSize();
		this.idsize = rs.getAlgebra().getIdSize();
		if (size > 0) {
			indexnodes = new ArrayList(size);
			for (int i = 0; i < size; i++)
				indexnodes.add(null);
		}
		if (idsize > 0) {
			identifiers = new ArrayList(idsize);
			for (int i = 0; i < idsize; i++)
				identifiers.add(null);
		}
		//indexnodes = new ArrayList[size];
		//identifiers = new Object[rs.getAlgebra().getIdSize()];
		//roots = new ArrayList() ;
	}

	public Tuple(Tuple tuple) {
		this.size = tuple.size();
		this.idsize = tuple.idsize();
		if (size > 0) {
			indexnodes = new ArrayList(size);
			for (int i = 0; i < size; i++)
				indexnodes.add(null);
		}
		if (idsize > 0) {
			identifiers = new ArrayList(idsize);
			for (int i = 0; i < idsize; i++)
				identifiers.add(null);
		}
		this.fillIdentifiers(0, tuple);
		this.appendTupleAtIndex(0, tuple);
		//indexnodes = new ArrayList[size];
		//identifiers = new Object[rs.getAlgebra().getIdSize()];
		//roots = new ArrayList() ;
	}

	public Object clone() throws CloneNotSupportedException {
		return super.clone();
	}
	// ***********************************************************************
	// * GET/SET METHODS
	// ***********************************************************************
	public void setParent(BufferTuple parent) {
		this.parent = parent;
	}
	public BufferTuple getParent() {
		return parent;
	}

	// ***********************************************************************
	// * TOOLS METHODS
	// ***********************************************************************
	/**
	 *
	 */
	public void addNodeAtIndex(int idx, TypedNode element) /*throws XQueryException*/ {
		//if ((idx < 0) || (idx >= indexnodes.length)) throw new XQueryException("Can't index " + element + " at " + idx) ;		
		if (indexnodes.get(idx) == null)
			indexnodes.set(idx, new ArrayList());
		((ArrayList) indexnodes.get(idx)).add(element);
		/*
		if (indexnodes[idx] == null)
			indexnodes[idx] = new ArrayList();
		indexnodes[idx].add(element);
		*/
	}

	/**
	 *
	 */
	public void addNodesAtIndex(int idx, ArrayList elements) {
		if (elements == null)
			return;
		if (indexnodes.get(idx) == null)
			indexnodes.set(idx, new ArrayList());
		((ArrayList) indexnodes.get(idx)).addAll(elements);
		/*
		if (indexnodes[idx] == null)
			indexnodes[idx] = new ArrayList();
		indexnodes[idx].addAll(elements);
		*/
	}

	/**
	 *
	 */
	public ArrayList getPath(String path) {
		int pos = parent.getPathIndex(path);
		//HashMap hm = parent.getIndexPath() ;
		if (pos != -1)
			return getNodesAtIndex(pos);
		return null;
	}

	public ArrayList getNodesAtIndex(int idx) {
		if (idx >= 0 && idx < indexnodes.size())
			return (ArrayList) indexnodes.get(idx);
		else
			return null;
		/*
		if (idx < indexnodes.length)
			return indexnodes[idx];
		else
			return null;
		*/
	}

	public void appendTuple(Tuple tuple) {
		appendTupleAtIndex(0, tuple);
	}

	public void appendTupleAtIndex(int idx, Tuple tuple) {
		for (int i = 0; i < tuple.size(); i++)
			addNodesAtIndex(idx + i, tuple.getNodesAtIndex(i));
	}

	/**
	 *
	 */
	// private void add(int idx, ArrayList nodes) throws XMLDBCException { addNodesAtIndex(idx, nodes) ; }

	/**
	 *
	 */
	public int size() {
		return size;
	}

	/**
	 *
	 */
	public int idsize() {
		return idsize;
		/*
		if (identifiers == null)
			return 0;
		else
			return identifiers.length;
		*/
	}

	/**
	 *
	 */
	public void eraseIdSize() {
		identifiers = null;
		idsize = 0;
	}

	// ***********************************************************************
	// * TOOLS METHODS
	// ***********************************************************************

	public Tuple getPart(int index, int part, int idsiz) {
		if (index >= size)
			return null;
		ArrayList tmplist = this.getNodesAtIndex(index);
		if (tmplist == null || part >= tmplist.size())
			return null;
		ArrayList reslist = new ArrayList(1);
		reslist.add(tmplist.get(part));
		Tuple newtuple = new Tuple(1, idsiz);
		newtuple.addNodesAtIndex(0, reslist);
		newtuple.fillIdentifiers();
		return newtuple;
	}

	public Tuple getPart(int index, int idsiz) {
		if (index < 0 || index >= size)
			return null;
		ArrayList tmplist = this.getNodesAtIndex(index);
		Tuple newtuple = new Tuple(1, idsiz);
		newtuple.fillIdentifiers(0,newtuple);
		newtuple.addNodesAtIndex(0, tmplist);
		return newtuple;
	}

	public Tuple getPart(int index) {
		return getPart(index,0);
	}

	public Tuple getPart(String str, int idsiz) {
		if (parent == null)
			return null;
		return getPart(parent.getPathIndex(str),idsiz);
	}
	
	public Tuple getPart(String str) {
		return getPart(str,0);
	}

	/**
	 *
	 */
	public Tuple project(ArrayList steps, ArrayList idxsteps) {
		return project(steps, idxsteps, false);
	}
	public Tuple project(ArrayList steps, ArrayList idxsteps, boolean erase) {
		//System.out.println ("Projection sur " + steps + " de " + this) ;

		// if it is the first time this method is called, idxsteps is empty
		// and we must calculate which column must be kept
		if (idxsteps.isEmpty()) {
			for (int i = 0; i < steps.size(); i++) {
				/*
				ArrayList stepi = (ArrayList) steps.get(i) ;
				StringBuffer sb =  new StringBuffer() ;
				for (int j = 0 ; j < stepi.size() ; j ++) {
				    if (j!=0) sb.append("/") ;
				    sb.append((String) stepi.get(j)) ;
				
				}
				 */
				String stepsi = (String) steps.get(i);
				// CHANGE   (add condition on index)
				int tmpInt = parent.getPathIndex(stepsi);
				if (tmpInt >= 0)
					idxsteps.add(new Integer(tmpInt));
			}
		}

		Tuple newtuple = null;
		if (erase) { // not done yet
			newtuple = this;
		} else {
			newtuple = new Tuple(idxsteps.size(), idsize);
			newtuple.fillIdentifiers(0, this);
			int cpt = 0;
			for (int i = 0; i < idxsteps.size(); i++) {
				int idx = ((Integer) idxsteps.get(i)).intValue();
				newtuple.addNodesAtIndex(cpt++, getNodesAtIndex(idx));
			}
		}
		//System.out.println ("RESULT " + newtuple) ;
		return newtuple;
	}

	/**
	 *
	 */
	public Tuple merge(Tuple addtuple, Operator algebra) {
		//System.out.println ("MERGING " + tuple2  + " TO " +this + " with " + equiv) ;
		Tuple newtuple = new Tuple(algebra);
		newtuple.appendTuple(this);
		newtuple.fillIdentifiers(0, this);
		int cpt = size();
		for (int k = 0; k < addtuple.size(); k++) {
//			int equivi = -1;
//			if ((equivi = getEquiv(algebra.getEquivalences(), k)) != -1) {
//				ArrayList trees1 = (ArrayList) getNodesAtIndex(equivi);
//				ArrayList trees2 = (ArrayList) addtuple.getNodesAtIndex(equivi);
//				if (DOMUtils.compareTree(trees1, trees2)) {
//					// it's OK, do nothing
//				} else
//					return null;
//			} else {
				// System.out.println (cpt + " -> " + tuple2.get (k) + " in " + newtuple) ;
				newtuple.addNodesAtIndex(cpt++, addtuple.getNodesAtIndex(k));
//			}
		}
		cpt = idsize;

		for (int k = 0; k < addtuple.idsize(); k++)
//			if (getEquiv(algebra.getIdEquivalences(), k) == -1)
				newtuple.fillIdentifiers(cpt++, addtuple.getIdentifiers(k));

		// System.out.println ("RESULTS " + newtuple) ;
		return newtuple;
	}

	/**
	 *
	 */
	public void mergeInPlace(ArrayList nodes, Operator algebra) {
		//System.out.println ("MERGING " + tuple2  + " TO " +this + " with " + equiv) ;
		int oldsize = size;
		if (size < algebra.getSize()) {
			for (int i = size; i < algebra.getSize(); i++)
				indexnodes.add(null);
			size = algebra.getSize();
		}
		if (idsize < algebra.getIdSize()) {
			if (identifiers == null) 
				identifiers = new ArrayList();
			for (int i = idsize; i < algebra.getIdSize(); i++)
				identifiers.add(this);
			idsize = algebra.getIdSize();
		}
		if (nodes != null)
			addNodesAtIndex(oldsize, nodes);
	}

	/**
	 *
	 */
	public void mergeInPlace(TypedNode node, Operator algebra) {
		//System.out.println ("MERGING " + tuple2  + " TO " +this + " with " + equiv) ;
		int oldsize = size;
		if (size < algebra.getSize()) {
			for (int i = size; i < algebra.getSize(); i++)
				indexnodes.add(null);
			size = algebra.getSize();
		}
		if (idsize < algebra.getIdSize()) {
			if (identifiers == null) 
				identifiers = new ArrayList();
			for (int i = idsize; i < algebra.getIdSize(); i++)
				identifiers.add(this);
			idsize = algebra.getIdSize();
		}
		if (node != null)
			addNodeAtIndex(oldsize, node);
	}

	/**
	 *
	 */
	//    public Tuple mergeWithNull(Tuple addtuple, int size, int location, int idsize, int idlocation)  {
	//        //System.out.println ("MERGING WITH NULL " + tuple2  + " TO " +this + " with " + equiv + " and " + location) ;
	//
	//        Tuple newtuple = new Tuple(size,idsize) ;
	//        newtuple.add(this) ;
	//        newtuple.add(location,addtuple);
	//        newtuple.fillIdentifiers(0,this);
	//        newtuple.fillIdentifiers(idlocation,addtuple);
	//
	//        for (int k = 0 ; k < tuple2.size() ; k ++) {
	//            int equivi = -1 ;
	//            if ((equivi = getEquiv(equiv, k)) != -1) {
	//                ArrayList trees1 = (ArrayList) get(equivi) ;
	//                ArrayList trees2 = (ArrayList) tuple2.get(equivi) ;
	//                if (compareTree(trees1, trees2)) {
	//                    // it's OK, do nothing
	//                }
	//                else
	//                    return null ;
	//            }
	//            else {
	//                //System.out.println (cpt + " -> " + tuple2.get (k) + " in " + newtuple) ;
	//                newtuple.index(location ++, tuple2.get(k)) ;
	//            }
	//        }
	//
	//        // System.out.println ("RESULTS " + newtuple) ;
	//        return newtuple ;
	//    }

	/**
	 *
	 */
//	private int getEquiv(ArrayList equivalences, int j) {
//		if (equivalences != null)
//			for (int k = 0; k < equivalences.size(); k++) {
//				int[] couple = (int[]) equivalences.get(k);
//				if (couple[1] == j)
//					return couple[0];
//			}
//		return -1;
//	}

	/**
	 *
	 */
	public Tuple completeTupleForLeft(int size) {
		//System.out.println ("COMPLETE LEFT") ;
		Tuple tuple = new Tuple(size);
		tuple.appendTuple(this);
		//for (int i = size() ; i < size ; i ++) tuple.appendTuple(null) ;
		return tuple;
	}

	/**
	 *
	 */
	public Tuple completeTupleForRight(int size, ArrayList equivalences) {
		//System.out.println ("COMPLETE RIGHT") ;
		Tuple tuple = new Tuple(size);
		int cpt = 0;

		// initialize all to null
		//for (int i = 0 ; i < size ; i ++) tuple.appendTuple(null) ;

		int limit = size - size();

		// complete the common
		for (int i = 0; i < size(); i++) {
//			int idx1 = getEquiv(equivalences, i);
//			if (idx1 != -1)
//				tuple.addNodesAtIndex(idx1, getNodesAtIndex(i));
//			else {
				tuple.addNodesAtIndex(limit + cpt, getNodesAtIndex(i));
				cpt++;
//			}
		}
		return tuple;
	}

	// pour fetch
	/**
	 *
	 */
	public String makeStringPath(int index) {
		//System.out.println ("Trying to fetch  of " + toString () + " at " + index) ;
		ArrayList nodes = getNodesAtIndex(index);
		//System.out.println ("got " + nodes) ;
		if (nodes == null)
			return null;
		if (nodes.size() != 1)
			return null;

		TypedNode node = (TypedNode) nodes.get(0);
		if (node == null)
			return null;
		String res = null;
		if ((res = node.getNormalizedStringValue()) != null)
			return res;
		org.w3c.dom.NodeList nl = node.getChildNodes() ;		
		if (nl.getLength() != 1) return null ;
		TypedNode n = (TypedNode)nl.item(0) ;
		return n.getNormalizedStringValue();
	}
	// pour generate
	/**
	 *
	 */
	public void makeEventPath(DOMReader domreader, ContentHandler handler, int index) throws XQueryException {
		if (domreader == null)
			return;
		//System.out.println ("Trying to fetch  of " + toString () + " at " + index) ;
		ArrayList nodes = getNodesAtIndex(index);
		//System.out.println ("got " + nodes) ;
		if (nodes == null)
			return;
		try {
			for (int i = 0; i < nodes.size(); i++) {
				TypedNode nodei = (TypedNode) nodes.get(i);
				if (nodei == null)
					continue;
				domreader.setContentHandler(handler);
				if (handler instanceof LexicalHandler)
					domreader.setProperty("http://xml.org/sax/properties/lexical-handler", handler);
				domreader.setRoot(nodei);
				domreader.setFeature(SAXConstants.FRAGMENT_FEATURE, true);
				domreader.parse("");
			}
		} catch (IOException ioe) {
			throw new XQueryException("Tuple.makeEventPath: " + ioe.getMessage());
		} catch (SAXException saxe) {
			throw new XQueryException("Tuple.makeEventPath: " + saxe.getMessage());
		}
	}

	/**
	 *
	 */
	//public void fillIdentifiers(long identifier) {
	public void fillIdentifiers() {
		if (identifiers == null)
			return;
		//Long id = new Long(identifier);
		for (int i = 0; i < idsize; i++)
			identifiers.set(i, this);
		/*
		if (identifiers == null)
			return;
		Long id = new Long(identifier);
		for (int i = 0; i < idsize; i++)
			identifiers[i] = id;
		*/
	}

	/**
	 *
	 */
	public void fillIdentifiers(int startindex, Tuple tuple) {
		for (int i = startindex, j = 0; i < startindex + tuple.idsize(); i++, j++)
			identifiers.set(i, tuple.getIdentifiers(j));
		/*
		for (int i = startindex, j = 0; i < startindex + tuple.idsize(); i++, j++)
			identifiers[i] = tuple.getIdentifiers(j);
		*/
	}

	/**
	 *
	 */
	public void fillIdentifiers(int startindex, Object obj) {
		identifiers.set(startindex, obj);
		//identifiers[startindex] = obj;
	}

	/**
	 *
	 */
	public ArrayList retIdentifiers() {
		return identifiers;
	}
	//	public Object[] retIdentifiers() {
	//		return identifiers;
	//	}

	/**
	 *
	 */
	public Object getIdentifiers(int index) {
		if (identifiers == null || idsize <= index)
			return null;
		return identifiers.get(index);
	}
	//	public Object getIdentifiers(int index) {
	//		if (identifiers == null || idsize <= index)
	//			return null;
	//		return identifiers[index];
	//	}

	/**
	 *
	 */
	public void getIdentifiers(Object[] identifiers) {
		if (this.identifiers == null) {
			for (int i = 0; i < identifiers.length; i++)
				identifiers[i] = null;
			return;
		}
		for (int i = 0; i < identifiers.length; i++)
			if (idsize > i)
				identifiers[i] = this.identifiers.get(i);
			else
				identifiers[i] = null;
	}
	//	public void getIdentifiers(Object[] identifiers) {
	//		if (this.identifiers == null) {
	//			for (int i = 0; i < identifiers.length; i++)
	//				identifiers[i] = null;
	//			return;
	//		}
	//		for (int i = 0; i < this.identifiers.length; i++)
	//			identifiers[i] = this.identifiers[i];
	//	}

	public boolean equals(Tuple tuple) {
		if (tuple == null)
			return false;
		if (tuple.size() != this.size)
			return false;
		if (tuple.idsize() != this.idsize)
			return false;
		for (int i = 0; i < size; i++) {
			ArrayList listi = tuple.getNodesAtIndex(i);
			if (((ArrayList) indexnodes.get(i)).size() != listi.size())
				return false;
			for (int j = 0; j < listi.size(); j++) {
				TypedNode nodeij1 = (TypedNode) listi.get(j);
				TypedNode nodeij2 = (TypedNode) ((ArrayList) indexnodes.get(i)).get(j);
				if (!DOMUtils.compareTree(nodeij1, nodeij2))
					return false;
			}
		}
		return true;
	}
	//	public boolean equals(Tuple tuple) {
	//		if (tuple == null)
	//			return false;
	//		if (tuple.size() != this.size)
	//			return false;
	//		if (tuple.idsize() != this.idsize)
	//			return false;
	//		for (int i = 0; i < size; i++) {
	//			ArrayList listi = tuple.getNodesAtIndex(i);
	//			if (indexnodes[i].size() != listi.size())
	//				return false;
	//			for (int j = 0; j < listi.size(); j++) {
	//				TypedNode nodeij1 = (TypedNode) listi.get(j);
	//				TypedNode nodeij2 = (TypedNode) indexnodes[i].get(j);
	//				if (!DOMUtils.compareTree(nodeij1, nodeij2))
	//					return false;
	//			}
	//		}
	//		return true;
	//	}

	// ***********************************************************************
	// * DEBUG
	// ***********************************************************************
	/*
	public String toString() {
		return toCompleteString();
	}
	*/
	public String toCompleteString() {
		StringBuffer sb = new StringBuffer();
		try {
			sb.append("\n----------- <TUPLE (" + size() + ")> -----------------\n");
			sb.append("INDEX \n");
			for (int i = 0; i < indexnodes.size(); i++)
				sb.append(i + " - " + indexnodes.get(i) + "\n");

			if (identifiers == null)
				sb.append("\nidentifiers is empty\n");
			else {
				sb.append("\nidentifiers \n");
				for (int i = 0; i < identifiers.size(); i++)
					sb.append(i + " - " + identifiers.get(i) + "\n");
			}

			for (int i = 0; i < size; i++) {
				sb.append("\n-------------------------------\n");
				ArrayList nodes = getNodesAtIndex(i);
				if (nodes == null) {
					sb.append("null nodes " + i + "\n");
					continue;
				}
				for (int j = 0; j < nodes.size(); j++) {
					//TypedNode nodej = (TypedNode) nodes.get(j);
					if (nodes.get(j) == null)
						sb.append("null node " + i + "-" + j + "\n");
					else {
						Object obj = nodes.get(j);
						if (obj instanceof TypedNode)
							sb.append(stringRepresentationOf((TypedNode)obj));
						else if (obj instanceof Node)
							sb.append(stringRepresentationOf((Node)obj));
						else
							sb.append("unknown node type " + obj.getClass().getName());
					}
				}
			}
			sb.append("\n----------------------------------------\n");
			return sb.toString();
		} catch (Exception e) {
			//e.printStackTrace () ;
			return "Error in displaying : " + e.getMessage();
		}
	}

	/**
	 * Make a programmer-friendly representation of a TypedNode (useful for
	 * debugging).
	 *
	 * @param root the node to display recursively.
	 * @return a String represention of the node.
	 */
	public String stringRepresentationOf(TypedNode root) {
		return stringRepresentationOf("", root);
	}
	public String stringRepresentationOf(Node root) {
		return stringRepresentationOf("", root);
	}

	/**
	 * Make a programmer-friendly representation of a TypedNode (useful for
	 * debugging).
	 *
	 * @param root the node to display recursively.
	 * @return a String representation of the node.
	 */
	private String stringRepresentationOf(String before, TypedNode root) {
		if (root == null)
			return before + "<null>";

		StringBuffer str = new StringBuffer();
		String newline = System.getProperty("line.separator");

		// display Attributes first
		org.w3c.dom.NamedNodeMap nnm = root.getAttributes();
		if (nnm != null) {
			for (int i = 0; i < nnm.getLength(); i++)
				str.append(newline + before + "*---- [A] " + nnm.item(i).getNodeName() + " : " + nnm.item(i).getNodeValue());
		}

		// display TypedNode first
		org.w3c.dom.NodeList nl = root.getChildNodes();
		if ((nl == null) || (nl.getLength() <= 0))
			str.append(newline + before + "+----" + root.getNodeName() + " : " + root.getNodeValue() + " [" + root.getNodeType() + "]");
		else {
			str.append(newline + before + "+----" + root.getNodeName() + " [" + root.getNodeType() + "]");
			for (int i = 0; i < nl.getLength(); i++)
				str.append(stringRepresentationOf(before + "  ", nl.item(i)));
		}
		return str.toString();
	}

	private String stringRepresentationOf(String before, Node root) {
		if (root == null)
			return before + "<null>";

		StringBuffer str = new StringBuffer();
		String newline = System.getProperty("line.separator");

		// display Attributes first
		org.w3c.dom.NamedNodeMap nnm = root.getAttributes();
		if (nnm != null) {
			for (int i = 0; i < nnm.getLength(); i++)
				str.append(newline + before + "*---- [A] " + nnm.item(i).getNodeName() + " : " + nnm.item(i).getNodeValue());
		}

		// display TypedNode first
		org.w3c.dom.NodeList nl = root.getChildNodes();
		if ((nl == null) || (nl.getLength() <= 0))
			str.append(newline + before + "+----" + root.getNodeName() + " : " + root.getNodeValue() + " [" + root.getNodeType() + "]");
		else {
			str.append(newline + before + "+----" + root.getNodeName() + " [" + root.getNodeType() + "]");
			for (int i = 0; i < nl.getLength(); i++)
				str.append(stringRepresentationOf(before + "  ", nl.item(i)));
		}
		return str.toString();
	}
	
}
