/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */

package org.xquark.mediator;

import org.xquark.xml.xdbc.*;

public class MediatorDriver implements XMLDriver {
	// **********************************************************************
	// * VERSIONING
	// **********************************************************************
	private static final String RCSRevision = "$Revision: 1.4 $";
	private static final String RCSName = "$Name:  $";
	// **********************************************************************
	// * CLASS VARIABLE
	// **********************************************************************
	static MediatorDriver instance = null;

	static {
		try {
			createInstance();
		} catch (Exception ex) {
			System.err.println("load MediatorDriver failed.");
		}
	}

	// ************************************************************************
	// * INITIALISATION
	// ************************************************************************
	private MediatorDriver() throws XMLDBCException {
		XMLDriverManager.registerDriver(this);
	}

	public static MediatorDriver createInstance() throws XMLDBCException {
		if (null == instance) {
			instance = new MediatorDriver();
		}
		return instance;
	}

	// ************************************************************************
	// * XMLDriver IMPLEMENTATION
	// ************************************************************************
	/**
	 * To obtain a handler to a data source with the specified URI.
	 * @param uri an URI corresponding to a data source.
	 * @return an instance of the specified data source,
	 *         or null if the driver does not know this type of data source
	 * @throws XMLDBCException if a data source access error occurs.
	 */
	public XMLDataSource getDataSource(String uri) throws XMLDBCException {
		try {
			URIParser urip = new URIParser(uri);
			//System.out.println("MEDIATOR GETSOURCE proto=" + urip.getProtocol() + " vendor=" + urip.getVendor() + " source=" + urip.getDataSourceType() + " file=" + urip.getConfFileName());
			if (urip.getProtocol().equalsIgnoreCase("xdbc") && urip.getVendor().equalsIgnoreCase("xquark") && urip.getDataSourceType().equalsIgnoreCase("mediator")) {
				return new Mediator(urip);
			} else {
				return null;
			}
		} catch (XMLDBCException e) {
			e.printStackTrace();
			return null;
		}
	}
}
