/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */

package  org.xquark.mediator ;

import java.io.File;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.Properties;

import org.xml.sax.*;
import org.xml.sax.helpers.DefaultHandler;
import org.xquark.xml.xdbc.XMLDBCException;


/**
 * This class is used to parse a mediator accessor file.
 * 
 */

public class MediatorHandler extends DefaultHandler
{
	// **********************************************************************
	// * VERSIONING
	// **********************************************************************
	private static final String RCSRevision = "$Revision: 1.3 $";
	private static final String RCSName = "$Name:  $";
	// **********************************************************************
	// * CLASS VARIABLE
	// **********************************************************************
	private final static String LABEL_ACCESSOR = "accessor" ;
	private final static String LABEL_NAME = "name" ;
	private final static String LABEL_DRIVER = "driver" ;
	private final static String LABEL_CONNECTION = "connection" ;
	private final static String LABEL_SUBACCESSOR = "subaccessor" ;

	private final static String LABEL_LOGIN = "login" ;
	private final static String LABEL_PASSWD = "passwd" ;

	private final static String KEY_NAME = "key_name" ;
	private final static String KEY_DRIVER = "key_driver" ;
	private final static String KEY_CONNECTION = "key_connection" ;

	private final static String KEY_LOGIN = "key_login" ;
	private final static String KEY_PASSWD = "key_passwd" ;

	protected Properties xmlproperties;
	protected String cur_car;
	protected Mediator accessor ;
	private String fileLocation = null;

	// **********************************************************************
	// * CONSTRUCTOR
	// **********************************************************************
	/**
	 *
	 */
	public MediatorHandler (Mediator accessor) 
	{
		this.accessor = accessor ;
		xmlproperties = new Properties () ;
	}


	public void setDocumentLocator(Locator loc) {
		try {
			File confFile = new File((new URL(loc.getSystemId())).getFile());
			fileLocation = confFile.getParent();
		} catch (MalformedURLException mue) {}
	}
	
	// **********************************************************************
	// * TOOLS METHODS
	// **********************************************************************
    /**
     * Return  a property of the accessor.
     * 
     * @return a property of the accessor.
     */
    protected String getProperty (String key)
    {
         return (String) xmlproperties.getProperty (key);
    }

	/**
     * Set a property of the accessor.
     *
     * @param a property of the accessor.
     */
    private void setProperty (String key, String value)
     {
        if (key == null)
        {
            System.err.println ("Key " +  key + " is null.") ;
            return ;
        }
        if (value == null)
        {
            System.err.println ("Value " + value + " of " + key + " is null.") ;
            return ;
        }
        xmlproperties.setProperty (key, value);
    }


	// **********************************************************************
	// * SAX IMPLEMENTATION
	// **********************************************************************
	/**
	 *
	 */
	public void startElement (String namespaceURI, String localName, String qualifiedName, Attributes atts) throws SAXException
	{
		if (localName.equalsIgnoreCase(LABEL_SUBACCESSOR)) 
		{
			setProperty(KEY_NAME, atts.getValue (LABEL_NAME));
			return ;
		}
	}

	/**
	 *
	 */
    public void characters (char ch[], int start, int length)
        throws SAXException
    {
		///*DEBUG*/System.out.println ("*** characters path : " + new String (ch, start, length));
        cur_car = new String(ch, start, length) ;
        cur_car.trim () ;
    }


	/**
	 *
	 */
	public void endElement (String namespaceURI, String localName, String qualifiedName) throws SAXException
	{
		if (localName.equalsIgnoreCase(LABEL_DRIVER)) 
		{
			setProperty(KEY_DRIVER, cur_car);
			return ;
		}

		if (localName.equalsIgnoreCase(LABEL_CONNECTION)) 
		{
			try {
				URIParser urip  = new URIParser(cur_car);
				String parentDir = (new File(urip.getConfFileName())).getParent();
				if (parentDir == null || parentDir.length() <= 0) {
					int ind = cur_car.indexOf(urip.getConfFileName());
					cur_car = cur_car.substring(0,ind) + fileLocation +  "\\" + cur_car.substring(ind);
				}
			} catch (XMLDBCException xmlde) {}
			setProperty(KEY_CONNECTION, cur_car);
			return ;
		}

		if (localName.equalsIgnoreCase(LABEL_LOGIN)) 
		{
			setProperty(KEY_LOGIN, cur_car);
			return ;
		}

		if (localName.equalsIgnoreCase(LABEL_PASSWD)) 
		{
			setProperty(KEY_PASSWD, cur_car);
			return ;
		}

		if (localName.equalsIgnoreCase(LABEL_SUBACCESSOR)) 
		{
			try
			{				
				if ((getProperty (KEY_LOGIN) != null) && (getProperty (KEY_PASSWD) != null))
					accessor.addSubAccessor (getProperty (KEY_NAME), getProperty (KEY_DRIVER), getProperty (KEY_CONNECTION), getProperty (KEY_LOGIN), getProperty (KEY_PASSWD)) ;
				else
					accessor.addSubAccessor (getProperty (KEY_NAME), getProperty (KEY_DRIVER), getProperty (KEY_CONNECTION)) ;
			}
			catch (MediatorException me)
			{
				throw new SAXException ("MediatorHandler.endElement: couldn't add subaccessor " + getProperty (KEY_NAME)) ;
			}
			xmlproperties.clear() ;
		}
	}
	
	public void error(SAXParseException spee) throws SAXException {
		System.err.println("Warning : " + spee.toString());
	}
}
