/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */

package org.xquark.mediator;

import org.xquark.xml.xdbc.XMLDBCException;


/**
 * This class is used as a structure to handle characteristics of
 * a xdbc uri string. It also parse an URI strings and extract these 
 * characteristics.
 * 
 */
public class URIParser 
{
	// **********************************************************************
	// * VERSIONING
	// **********************************************************************
	private static final String RCSRevision = "$Revision: 1.2 $";
	private static final String RCSName = "$Name:  $";
	
	// **********************************************************************
	// * CLASS VARIABLES
	// **********************************************************************
	private String protocol = null ;
	private String vendor = null ;
	private String datasourcetype = null ;
	private String conffilename = null ;
	private String uri = null ;

	// ************************************************************************
	// * INITIALIZATION
	// ************************************************************************
	/** 
	 *
	 */
	public URIParser (String uri) throws XMLDBCException
	{
		this.uri = uri ;
        int index = uri.indexOf(":");
        if (-1 == index) 
            throw new XMLDBCException("URI string is not valid.");
        protocol = uri.substring(0,index);

        int newIndex = uri.indexOf(":", index+1);
        if (-1 == newIndex) 
            throw new XMLDBCException("URI string is not valid.");
        vendor = uri.substring(index+1, newIndex);

        index = newIndex;
        newIndex = uri.indexOf(":", index+1);
        if (-1 == newIndex) 
            throw new XMLDBCException("URI string is not valid.");
        datasourcetype = uri.substring(index+1, newIndex);

        index = newIndex;
        newIndex = uri.indexOf(":", index+1);

        if (-1 == newIndex) 
            throw new XMLDBCException("URI string is not valid.");
        String token = uri.substring(index+1, newIndex);

        if (token.equalsIgnoreCase("file")) 
            conffilename = uri.substring(newIndex+1, uri.length());
        else 
            throw new XMLDBCException("URI string is not valid.");
	}

	// ************************************************************************
	// * GET/SET METHODS
	// ************************************************************************
	/**
	 *
	 */
	public String getProtocol ()
	{
		return protocol ;
	}

	/**
	 *
	 */
	public String getVendor ()
	{
		return vendor ;
	}
	
	/**
	 *
	 */
	public String getDataSourceType ()
	{
		return datasourcetype ;
	}

	/**
	 *
	 */
	public String getConfFileName ()
	{
		return conffilename ;
	}

	public String getURI ()
	{
		return uri ;
	}
}
