/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */

package org.xquark.mediator.algebra;

import java.net.URL;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.xquark.mediator.MediatorException;
import org.xquark.mediator.decomposer.GetVariablePathsVisitor;
import org.xquark.mediator.decomposer.IsolateWhereVisitor;
import org.xquark.mediator.decomposer.Utils;
import org.xquark.mediator.plan.*;
import org.xquark.xquery.metadata.MetaDataImpl;
import org.xquark.xquery.parser.*;
import org.xquark.xquery.parser.primitivefunctions.fnfunctions.FunctionCOLLECTION;
import org.xquark.xquery.parser.primitivefunctions.fnfunctions.FunctionDOC;
import org.xquark.xquery.typing.QTypeAtom;
import org.xquark.xquery.typing.QTypeElement;

/**
 * This class represent the terminal leaf of the dependancy tree.
 * It is generated after atomization (and is resulted from the transformation
 * of DepSource that disappear).
 *
 */
public class AlgSource extends Algebra {
	// **********************************************************************
	// * VERSIONING
	// **********************************************************************
	private static final String RCSRevision = "$Revision: 1.4 $";
	private static final String RCSName = "$Name:  $";
	// **********************************************************************
	// * CLASS VARIABLES
	// **********************************************************************
	private ArrayList sources;
	//	private boolean hasIdentifier = false;
	//	private boolean flwrcreated = false;
	//	private boolean returncreated = false;
	// ************************************************************************
	// * INITIALIZATION
	// ************************************************************************
	/**
	 * Create a node from a variable and an expression.
	 *
	 * To create the node, we take the original expression, and we
	 * extract everything relative to variable given by the leaf.
	 * from that, we create a new flower expression that will be associated
	 * to this DepNodeSource, and the old XQueryExpression that is
	 * referenced by it's DepNode is simplified.
	 * (bon, en gros, a sert a faire une optimisation en faisant
	 * redescendre les restrictions (et plus tard les projection) vers les
	 * sources.
	 *
	 * @param typevisitor
	 * @param origexp
	 * @param varleaf
	 * @param depmanager
	 */
	public AlgSource(XQueryExpression origexp, ArrayList variables, AlgebraManager depmanager, boolean islet) throws MediatorException {
		super(origexp, variables, depmanager, islet);
	}

	public AlgSource(XQueryExpression origexp, Variable varleaf, AlgebraManager depmanager, boolean islet) throws MediatorException {
		// to satisfy the super constructor, but expression and variable will be changed in following step.
		super(origexp, varleaf, depmanager, islet);
	}

	// ******************************************************************
	// * METHODS
	// ******************************************************************

	public void ParentHasIdentifier() {
		hasIdentifier = true;
	}

	/**
	 * Create the FLWRExpression like this :
	 * - F : is the variable varleaf that we consider with no changes
	 * (even the expression associated to it is kept ;
	 * - L : there is no Let at this point ;
	 * - W : is the major part, we must scan the origexp to see what
	 *       to keep
	 * - R : for now, we return just the variable, but later, when
	 *       we will care about prjection, we will make good
	 *       LocatedExpression with it.
	 * @param origexp
	 * @param varleaf
	 */

	//	private void createFLWRExpression() throws MediatorException {
	//		this.createFLWRExpression(null, true);
	//	}
	//	public void createFLWRExpression(boolean doReturn) throws MediatorException {
	//		this.createFLWRExpression(null, doReturn);
	//	}
	//	public void createFLWRExpression(ArrayList vars) throws MediatorException {
	//		this.createFLWRExpression(null, true);
	//	}
	private void createFLWRExpression(boolean doReturn) throws MediatorException {
		if (!(expression instanceof FLWRExpression))
			throw new MediatorException("DepNodeSource : " + expression + " is not a FLWR.");

		try {
			FLWRExpression origflwr = (FLWRExpression) expression;

			// Variables
			ArrayList newvariables = (ArrayList) variables.clone();
			newvariables.addAll(this.parent.getVarsDependingOn());
			newvariables.addAll(this.getVarsDependingOn());
			//newvariables.addAll(this.parent.getVarsWhereOn());
			newvariables.addAll(this.getVarsWhereOn());

			// Where
			XQueryExpression newWhereClause = origflwr.getWhereClause();

			if (!doReturn) {
				// update paths because where clause is not calculate in flwr
				if (newWhereClause != null) {
					GetVariablePathsVisitor gvpv = new GetVariablePathsVisitor(true);
					newWhereClause.accept(gvpv);
					org.xquark.util.misc.Debug.print("createFLWRExpression -> paths = " + paths);
					ArrayList tmplist = gvpv.getPaths(newvariables);
					if (tmplist != null)
						for (int i = 0; i < tmplist.size(); i++)
							if (!this.algManager.getPathsList().contains(tmplist.get(i).toString()))
								paths.add(tmplist.get(i));
					org.xquark.util.misc.Debug.print("createFLWRExpression -> paths = " + paths);
				}
			}
			if (newWhereClause != null) {
				newWhereClause = (XQueryExpression) origflwr.getWhereClause().clone();
				IsolateWhereVisitor visitor = new IsolateWhereVisitor(newvariables);
				newWhereClause.accept(visitor);
				newWhereClause = visitor.getIsolatedExpression();
				// erase isolated where clause from original where clause
				 ((FLWRExpression) expression).setWhereClause(visitor.getRemainingExpression());
			}
			//			if (!doReturn) {
			//				// update paths because where clause is not calculate in flwr
			//				if (newWhereClause != null) {
			//					GetVariablePathsVisitor gvpv = new GetVariablePathsVisitor(true);
			//					newWhereClause.accept(gvpv);
			//					org.xquark.xml.misc.Debug.print("createFLWRExpression -> paths = " + paths);
			//					ArrayList tmplist = gvpv.getPaths();
			//					if (tmplist != null)
			//						for (int i = 0; i < tmplist.size(); i++)
			//							if (!this.depmanager.getPathsList().contains(tmplist.get(i).toString()))
			//								paths.add(tmplist.get(i));
			//					org.xquark.xml.misc.Debug.print("createFLWRExpression -> paths = " + paths);
			//				}
			//			}

			// OrderBy
			ArrayList newOrderbys = null;
			if (origflwr.getOrderBy() != null) {
				newOrderbys = (ArrayList) origflwr.getOrderBy().clone();
				for (int i = 0, j = 0; i < newOrderbys.size(); i++, j++) {
					XQueryExpression exp = (XQueryExpression) newOrderbys.get(i);
					IsolateWhereVisitor visitor = new IsolateWhereVisitor(variables);
					exp.accept(visitor);
					exp = visitor.getIsolatedExpression();
					if (exp != null) {
						newOrderbys.set(i, exp);
					} else {
						newOrderbys.remove(i);
						i--;
					}
				}
				if (newOrderbys.isEmpty())
					newOrderbys = null;
			}
			if (doReturn) {
				// create new FLWRExpression
				// keep only single variable
				// origflwr.setVariables(vars);
				// recalculate sources if necessary
				//if (vars.size() != ((FLWRExpression)expression).getVariables().size()) {
				sources = expression.getSourceNames();
				if (sources != null) {
					sources.clear();
					for (int i = 0; i < variables.size(); i++) {
						Variable vari = (Variable) variables.get(i);
						if (vari.getSourceNames() != null) {
							for (int j = 0; j < vari.getSourceNames().size(); j++)
								if (!sources.contains(vari.getSourceNames().get(j)))
									sources.add(vari.getSourceNames().get(j));
						}
					}
				}
			}
			//}
			// make return expression
			FLWRExpression newFlwr = null;
			if (doReturn) {
				if (newWhereClause != null && newWhereClause.getQType() == null)
					newWhereClause.setQType(new QTypeAtom(this.algManager.getPlan().getSchemaManager().getBooleanType()));
				newFlwr = new FLWRExpression(variables, newOrderbys, newWhereClause, createReturnExpression(), this.algManager.getTypeVisitor());
			} else
				newFlwr = new FLWRExpression(variables, newOrderbys, newWhereClause, new XQueryVoid(this.algManager.getTypeVisitor()), null);
			newFlwr.setParentUnit(origflwr.getParentUnit());
			newFlwr.setSourceNames(sources);
			expression = newFlwr;
		} catch (TypeException te) {
			throw new MediatorException("DepNodeSource.createFLWRExpression : " + te.getMessage(), te);
		} catch (Exception e) {
			throw new MediatorException("DepNodeSource.createFLWRExpression : " + e.getMessage(), e);
		}
	}

	private XQueryExpression createReturnExpression() throws XQueryException, TypeException {
		TypeVisitor typevisitor = this.algManager.getTypeVisitor();
		ArrayList subexpressions = null;
		if (this.paths != null) {
			subexpressions = new ArrayList(this.paths.size());
			for (int i = 0; i < paths.size(); i++) {
				//ArrayList tmplist = new ArrayList(1);
				//tmplist.add((XQueryExpression) paths.get(i));
				//XQueryExpressionSequence tmpseq = new XQueryExpressionSequence(tmplist, typevisitor);
				//tmpseq.setBrace(true);
				XQueryExpression expr = null;
				try {
					expr = (XQueryExpression) ((XQueryExpression) paths.get(i)).clone();
					expr.setBrace(true);
				} catch (Exception e) {
				}
				ArrayList tmpeltexprs = new ArrayList(1);
				//tmpeltexprs.add(tmpseq);
				tmpeltexprs.add(expr);
				subexpressions.add(new org.xquark.xquery.parser.Element(new QName(null, null, "R" + Integer.toString(i + 1), null), null, null, tmpeltexprs, typevisitor));
			}
		}
		return new Element(new QName(null, null, "R", null), null, null, subexpressions, typevisitor);
	}

	// ******************************************************************
	// * CREATE EXECUTION PLAN
	// ******************************************************************
	/**
	 *
	 * @param plan
	 * @return
	 */
	public Operator createOperator(ExecutionPlan plan) throws MediatorException {
		//if (true) throw new MediatorException("createAlgebra not supported in DepNodeSource"); 
		createFLWRExpression(!plan.getQAOnly());

		FLWRExpression newFlwr = (FLWRExpression) expression;
		if (!plan.getQAOnly()) {
			//org.xquark.xml.misc.Debug.print("createFLWRExpression -> newFlwr = " + newFlwr) ;
			QTypeElement flwrqtype = null;
			//			if (newFlwr.getQType() instanceof QTypeCollection)
			//				flwrqtype = (QTypeElement) ((QTypeCollection) newFlwr.getQType()).getQType();
			//			else
			//				flwrqtype = (QTypeElement) newFlwr.getQType();
			flwrqtype = (QTypeElement) newFlwr.getQType();
			//org.xquark.xml.misc.Debug.print("createFLWRExpression -> flwrqtype = " + flwrqtype) ;
			try {
				this.getAlgebraManager().getPlan().setTypeName(flwrqtype.getType());
				//org.xquark.xml.misc.Debug.print("createFLWRExpression -> flwrqtype = " + flwrqtype) ;

				ArrayList attributes = new ArrayList(1);
				attributes.add(new AttributeValuePair(new ValueText("xsi:type", null), new ValueString("ns:" + flwrqtype.getType().getName(), null), null));
				((Element) newFlwr.getReturnClause()).setAttributes(attributes);
				//org.xquark.xml.misc.Debug.print("createFLWRExpression -> newFlwr = " + newFlwr) ;
			} catch (TypeException te) {
				throw new MediatorException("DepNodeSource.createAlgebra : " + te.getMessage(), te);
			} catch (Exception e) {
				throw new MediatorException("DepNodeSource.createAlgebra : " + e.getMessage(), e);
			}

			sources = expression.getSourceNames();
			if (sources == null || sources.isEmpty()) {
				try {
					Utils.fillSources(expression, algManager.getMetaData(), false);
				} catch (Exception e) {
					throw new MediatorException(e.getMessage(), e);
				}
				sources = expression.getSourceNames();
			}
		} else
			sources = null;
		if (sources == null || sources.isEmpty()) {
			if (!plan.getQAOnly())
				throw new MediatorException("DepNodeSource.createAlgebra: " + toString() + " has no sources.");
			//****************************************************************************************************
			// start special case for QA
			// get all documents in collection and make a union
			// test if only one variable
			if (newFlwr.getVariables().size() != 1)
				throw new MediatorException("Collection for QA, flwr should have only one variable");
			Variable var = (Variable) newFlwr.getVariables().get(0);
			XQueryExpression expr = var.getExpression();
			// test expression is located expression starting with collection
			// first get the collection name
			FunctionCOLLECTION func = null;
			boolean onlyFunction = false;
			if (expr instanceof LocatedExpression && ((LocatedExpression) expr).getExpression() instanceof FunctionCOLLECTION)
				func = (FunctionCOLLECTION) ((LocatedExpression) expr).getExpression();
			else if (expr instanceof FunctionCOLLECTION) {
				func = (FunctionCOLLECTION) expr;
				onlyFunction = true;
			} else
				throw new MediatorException("Collection for QA, variable should be bound to located expression starting with collection");
			expr = func.getArgument(0);
			if (!(expr instanceof ValueString))
				throw new MediatorException("Collection for QA, first argument of collection should be value string");
			String colName = ((ValueString) expr).getStringValue();
			int index = colName.indexOf(":");
			if (index != -1)
				colName = colName.substring(index + 1);
			// second, get all documents in the collection
			Operator operator = null;
			try {
				MetaDataImpl metadata = (MetaDataImpl) plan.getConnection().getMetaData();
				// test special case *
				ArrayList documents = null;
				if (colName.equals("*")) {
					List list = metadata.getMetaWrapper("CONFFILES").getCollectionsName();
					if (!list.isEmpty()) {
						for (Iterator it = list.iterator(); it.hasNext();) {
							colName = (String) it.next();
							if (documents == null)
								documents = (ArrayList) metadata.getMetaWrapper("CONFFILES").getMetaCollection(colName).getDocuments();
							else
								documents.addAll(metadata.getMetaWrapper("CONFFILES").getMetaCollection(colName).getDocuments());
						}
					}
				} else {
					documents = metadata.getMetaWrapper("CONFFILES").getMetaCollection(colName).getDocuments();
				}
				if (documents == null || documents.isEmpty())
					throw new MediatorException("Collection for QA, collection has no documents");
				if (documents.size() == 1) {
					String docname = (String) documents.get(0);
					if (plan.getBaseURI() != null)
						docname = (new URL(new URL(plan.getBaseURI()), docname)).toString();
					ArrayList args = new ArrayList(1);
					args.add(new ValueString(docname, null));
					if (onlyFunction)
						 ((Variable) newFlwr.getVariables().get(0)).setExpression(new FunctionDOC(args, this.algManager.getTypeVisitor()));
					else
						 ((LocatedExpression) ((Variable) newFlwr.getVariables().get(0)).getExpression()).getStepNum(0).setExpression(new FunctionDOC(args, this.algManager.getTypeVisitor()));
					operator = new OpSourceEval(plan, newFlwr, paths, hasIdentifier);
					// finally if some sort clauses are present sort the resultset
					if (newFlwr.getOrderBy() != null && !newFlwr.getOrderBy().isEmpty())
						operator = new OpSort(plan, operator, newFlwr.getOrderBy());
				} else {
					//throw new MediatorException("Collection for QA, collection with more than one document");
					ArrayList algebralist = new ArrayList(documents.size());
					for (int i = 0; i < documents.size(); i++) {
						FLWRExpression cloneflwr = (FLWRExpression) newFlwr.clone();
						String docname = (String) documents.get(i);
						ArrayList args = new ArrayList(1);
						args.add(new ValueString(docname, null));
						if (onlyFunction)
							 ((Variable) cloneflwr.getVariables().get(0)).setExpression(new FunctionDOC(args, this.algManager.getTypeVisitor()));
						else
							 ((LocatedExpression) ((Variable) cloneflwr.getVariables().get(0)).getExpression()).getStepNum(0).setExpression(new FunctionDOC(args, this.algManager.getTypeVisitor()));
						algebralist.add(new OpSourceEval(plan, cloneflwr, paths, hasIdentifier));
					}
					// finally if some sort clauses are present sort the resultset
					if (newFlwr.getOrderBy() != null && !newFlwr.getOrderBy().isEmpty())
						operator = new OpUnionSort(plan, algebralist, newFlwr.getOrderBy());
					else
						operator = new OpUnion(plan, algebralist);

				}
			} catch (Exception e) {
				throw new MediatorException("Collection for QA, could not access metadata");
			}
			operator.isLet(islet);
			return operator;
			// end special case for QA
			//****************************************************************************************************
		}

		// there is at least 1 source
		String source = (String) sources.get(0);
		Operator algebra = new OpSource(plan, expression, paths, source, hasIdentifier);
		if (sources.size() > 1) {
			// there is at least 2 leaves
			ArrayList orderbys = ((FLWRExpression) expression).getOrderBy();
			ArrayList algebralist = new ArrayList(sources.size());
			algebralist.add(algebra);
			for (int i = 1; i < sources.size(); i++)
				algebralist.add(new OpSource(plan, expression, paths, (String) sources.get(i), hasIdentifier));
			if (orderbys != null && !orderbys.isEmpty())
				algebra = new OpUnionSort(plan, algebralist, orderbys);
			else
				algebra = new OpUnion(plan, algebralist);
		}
		algebra.isLet(islet);
		return algebra;
	}

	public Object clone() throws CloneNotSupportedException {
		AlgSource newobj = (AlgSource) super.clone();
		newobj.hasIdentifier = this.hasIdentifier;
		newobj.sources = this.sources;
		return newobj;
	}

	// ******************************************************************
	// * GET/SET METHODS
	// ******************************************************************
	/**
	 *
	 * @return
	 */
	//public ArrayList getPaths() { return paths ; }

	/**
	 *
	 * @return
	 */
	// public ArrayList getVarsThatDepend() throws MediatorException { return varsthatdepend ; }

	// ******************************************************************
	// * DEBUGGING
	// ******************************************************************
	/**
	 *
	 * @return
	 */
	public String toCompleteString(int indent) {
		StringBuffer buf = new StringBuffer();

		buf.append(Utils.makeIndent(indent) + "<" + this.getClass().getName() + ">\n");
		buf.append(Utils.makeIndent(indent + 1) + "<variables> " + variables + " </variables>\n");
		buf.append(Utils.makeIndent(indent + 1) + "<sources> " + getSources() + " </sources>\n");
		buf.append(Utils.makeIndent(indent + 1) + "<urls> " + getUrls() + " </urls>\n");
		buf.append(Utils.makeIndent(indent + 1) + "<isLet> " + islet + " </isLet>\n");
		buf.append(Utils.makeIndent(indent + 1) + "<monosource> " + isMonoSource() + " </monosource>\n");
		buf.append(Utils.makeIndent(indent + 1) + "<hasIdentifier> " + hasIdentifier + " </hasIdentifier>\n");
		buf.append(Utils.makeIndent(indent + 1) + "<Expression>\n");
		buf.append(Utils.makeIndent(indent + 2) + expression + "\n");
		buf.append(Utils.makeIndent(indent + 1) + "</Expression>\n");
		//buf.append("\t\t<Function>" + funcToString() + "</Function>\n") ;
		if (sources != null) {
			buf.append(Utils.makeIndent(indent + 1) + "<Sources>\n");
			for (int i = 0; i < sources.size(); i++)
				buf.append(Utils.makeIndent(indent + 2) + "<Source>" + sources.get(i) + "</Sources>\n");
			buf.append(Utils.makeIndent(indent + 1) + "</Sources>\n");
		}
		if (algChildren != null && !algChildren.isEmpty()) {
			buf.append(Utils.makeIndent(indent + 1) + "<Leaves>\n");
			for (int i = 0; i < algChildren.size(); i++) {
				buf.append(Utils.makeIndent(indent + 2) + "<Leave>\n");
				Algebra depchild = (Algebra) algChildren.get(i);
				buf.append(depchild.toCompleteString(indent + 3));
				buf.append(Utils.makeIndent(indent + 2) + "</Leave>\n");
			}
			buf.append(Utils.makeIndent(indent + 1) + "</Leaves>\n");
		}
		if (referenceDefinitionVars != null && !referenceDefinitionVars.isEmpty()) {
			buf.append(Utils.makeIndent(indent + 1) + "<referenceDefinitionVars>\n");
			for (int i = 0; i < referenceDefinitionVars.size(); i++) {
				Variable var = (Variable) referenceDefinitionVars.get(i);
				buf.append(Utils.makeIndent(indent + 2) + "<Var>" + var + "</Var>\n");
			}
			buf.append(Utils.makeIndent(indent + 1) + "</referenceDefinitionVars>\n");
		}
		if (dependDefinitionVars != null && !dependDefinitionVars.isEmpty()) {
			buf.append(Utils.makeIndent(indent + 1) + "<DependingOn>\n");
			for (int i = 0; i < dependDefinitionVars.size(); i++) {
				Variable var = (Variable) dependDefinitionVars.get(i);
				buf.append(Utils.makeIndent(indent + 2) + "<Var>" + var + "</Var>\n");
			}
			buf.append(Utils.makeIndent(indent + 1) + "</dependDefinitionVars>\n");
		}
		if (paths != null) {
			buf.append(Utils.makeIndent(indent + 1) + "<Paths>\n");
			for (int i = 0; i < paths.size(); i++)
				buf.append(Utils.makeIndent(indent + 2) + "<Path>" + paths.get(i) + "</Path>\n");
			buf.append(Utils.makeIndent(indent + 1) + "</Paths>\n");
		}
		buf.append(Utils.makeIndent(indent) + "</" + this.getClass().getName() + ">\n");
		return buf.toString();
	}

	public void execute(ExecutionPlan plan) throws MediatorException {
	}

}
