/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */

package org.xquark.mediator.algebra;

import java.util.ArrayList;

import org.xquark.mediator.MediatorException;
import org.xquark.mediator.plan.*;
import org.xquark.xquery.parser.Variable;
import org.xquark.xquery.parser.XQueryExpression;

/**
 * This class represent a terminal leaf of the dependancy tree on first step
 * of creation (before atomization).
 *
 */
public class AlgUnion extends Algebra {
	// **********************************************************************
	// * VERSIONING
	// **********************************************************************
	private static final String RCSRevision = "$Revision: 1.3 $";
	private static final String RCSName = "$Name:  $";
	// **********************************************************************
	// * CLASS VARIABLES
	// **********************************************************************

	// ************************************************************************
	// * INITIALIZATION
	// ************************************************************************
	/**
	 *
	 * @param typevisitor
	 * @param expression
	 * @param var
	 * @param parent
	 */
	public AlgUnion(XQueryExpression expression, Variable var, AlgebraManager depmanager, boolean islet) {
		super(expression, var, depmanager, islet);
		fillVarLists();
	}

	// ******************************************************************
	// * METHODS
	// ******************************************************************

	// ******************************************************************
	// * CREATE EXECUTION PLAN
	// ******************************************************************
	/**
	 *
	 * @param plan
	 * @return
	 */
	public Operator createOperator(ExecutionPlan plan) throws MediatorException {

		//if (true) throw new MediatorException("createAlgebra not supported in DepArithFunction");
		Operator operator = null;

		if (algChildren != null) {
			ArrayList algebraList = new ArrayList();
			for (int i = 0; i < algChildren.size(); i++) {
				algebraList.add(((Algebra) algChildren.get(i)).createOperator(plan));
			}
			operator = new OpUnion(plan, algebraList);
		} else
			operator = new OpEval(plan, this, null, hasIdentifier);
		operator.isLet(islet);
		return operator;
	}

	public Object clone() throws CloneNotSupportedException {
		AlgUnion newobj = (AlgUnion) super.clone();
		return newobj;
	}

	public void execute(ExecutionPlan plan) throws MediatorException {
	}

	// ******************************************************************
	// * DEBUGGING
	// ******************************************************************
}
