/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * Copyright (C) 2003 XQuark Group.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */

package org.xquark.mediator.decomposer;

import java.util.ArrayList;

import org.xquark.mediator.algebra.AlgFLWR;
import org.xquark.mediator.algebra.Algebra;
import org.xquark.xquery.parser.*;

public class GetUsedVariablesVisitor extends DefaultParserVisitor {
    
    private static final String RCSRevision = "$Revision: 1.3 $";
    private static final String RCSName = "Name";
    
	//private ArrayList wherevariables = null;
    private ArrayList variables = null;
    private ArrayList declaredvariables = null;
    //private boolean stopWhenFound = false;
    //private boolean hasFound = false;
    //private boolean isflwr = false;
    
    public GetUsedVariablesVisitor() {
		variables = new ArrayList() ;
        declaredvariables = new ArrayList() ;
    }
        
    public ArrayList getVariables() { return variables; }
	//public ArrayList getWhereVariables() { return wherevariables; }
    //public boolean hasFound() { return hasFound; }
    
//    public void reset(boolean isflwr) {
//    	this.isflwr = isflwr;
//        if (variables != null) variables.clear();
//        declaredvariables.clear();
//    }
	public void reset() {
		if (variables != null) variables.clear();
		declaredvariables.clear();
	}
    
    // special
    
    public void visit(Algebra arg) throws XQueryException {
        //org.xquark.xml.misc.Debug.print("Call FillExprWithTupleVisitor -> XQueryExpression ( " + arg.getClass().getName() + " ) : " + arg );
        arg.getExpression().accept(this);
        ArrayList children = arg.getChildren();
        if (children != null)
            for (int i=0;i<children.size();i++) {
                //if (stopWhenFound && hasFound) return;
                visit((Algebra)children.get(i));
            }
        if (arg.isQDB()) {
            AlgFLWR tmpdepmem = (AlgFLWR)arg;
            if (tmpdepmem.getDependencyList() != null)
                for (int i=0;i<tmpdepmem.getDependencyList().size();i++) {
                    //if (stopWhenFound && hasFound) return;
                    visit((AlgFLWR)tmpdepmem.getDependencyList().get(i));
                }
        }
    }

    //	public void visit(AggregateFunctionCall arg) throws XQueryException
    //	public void visit(AttributeValuePair arg) throws XQueryException
    //	public void visit(BinOpANDExpression arg) throws XQueryException
    //	public void visit(BinOpORExpression arg) throws XQueryException
    //	public void visit(CastTreatExpression arg) throws XQueryException
    //	public void visit(ContextDeclaration arg) throws XQueryException
    //	public void visit(Dereference arg) throws XQueryException
    public void visit(Element arg) throws XQueryException {
        //if (stopWhenFound && hasFound) return;
        if (arg.getStartTag() != null) {
            arg.getStartTag().accept(this);
        }
        if (arg.getAttributes() != null) {
            for (int i = 0; i < arg.getAttributes().size(); i++) {
                //if (stopWhenFound && hasFound) return;
                ((AttributeValuePair)arg.getAttributes().get(i)).accept(this);
            }
        }
        if (arg.getSubExpressions() != null) {
            for (int j = 0; j < arg.getSubExpressions().size(); j++) {
                //if (stopWhenFound && hasFound) return;
                ((XQueryExpression)arg.getSubExpressions().get(j)).accept(this);
            }
        }
        //if (stopWhenFound && hasFound) return;
    }
    
    public void visit(FLWRExpression arg) throws XQueryException {
        for (int i = 0; i < arg.getVariables().size(); i++) {
            //if (stopWhenFound && hasFound) return;
            Variable vari = (Variable)arg.getVariables().get(i);
            declaredvariables.add(vari);
            vari.getExpression().accept(this);
        }
        //if (stopWhenFound && hasFound) return;
        if (arg.getWhereClause() != null) 
//        	if (isflwr) {
//        		GetUsedVariablesVisitor visitor = new GetUsedVariablesVisitor();
//        		visitor.reset();
//        		arg.getWhereClause().accept(visitor);
//        		wherevariables = visitor.getVariables();
//        	}
//        	else
				arg.getWhereClause().accept(this); 
        //arg.getReturnClause().accept(this);
    }
    
    public void visit(FunctionCall arg) throws XQueryException {        
        if (arg.getArguments() != null) {
            for (int i = 0; i < arg.getArguments().size(); i++) {
                //if (stopWhenFound && hasFound) return;
                ((XQueryExpression)arg.getArguments().get(i)).accept(this);
            }
        }        
    }
    
    //	public void visit(FunctionDefinition arg) throws XQueryException
    //	public void visit(InstanceOfExpression arg) throws XQueryException
    //	public void visit(InternalFunctionCall arg) throws XQueryException
    
    public void visit(ITEExpression arg) throws XQueryException {
        //if (stopWhenFound && hasFound) return;
        arg.getIfExpression().accept(this);
        //if (stopWhenFound && hasFound) return;
        arg.getElseExpression().accept(this);
        //if (stopWhenFound && hasFound) return;
        arg.getThenExpression().accept(this);
    }
    
    //	public void visit(LibraryFunctionCall arg) throws XQueryException
    //	public void visit(ListOpAFTERExpression arg) throws XQueryException
    //	public void visit(ListOpArithExpression arg) throws XQueryException
    //	public void visit(ListOpBEFOREExpression arg) throws XQueryException
    //	public void visit(ListOpCompExpression arg) throws XQueryException
    //	public void visit(ListOpEXCEPTExpression arg) throws XQueryException
    //	public void visit(ListOpINTERSECTExpression arg) throws XQueryException
    //	public void visit(ListOpUNIONExpression arg) throws XQueryException
    public void visit(LocatedExpression arg) throws XQueryException {
        for (int i = 0; i < arg.getSteps().size(); i++) {
            //if (stopWhenFound && hasFound) return;
            ((Step)arg.getSteps().get(i)).accept(this);
        }
    }
    
    //	public void visit(NodeType arg) throws XQueryException
    //	public void visit(Parameter arg) throws XQueryException
    //	public void visit(PrimitiveFunctionCall arg) throws XQueryException
    //	public void visit(QName arg) throws XQueryException
    public void visit(QuantifiedExpression arg) throws XQueryException {
        /*
        declaredvariables.add(arg.getVariable());
        arg.getVariable().getExpression().accept(this);
        arg.getConstraintExpresson().accept(this);
        */
    }
    //	public void visit(RangeExpression arg) throws XQueryException
    //	public void visit(RootNodeFunctionCall arg) throws XQueryException
    //	public void visit(SortedExpression arg) throws XQueryException
    //	public void visit(Step arg) throws XQueryException
    //	public void visit(TypeSwitchExpression arg) throws XQueryException
    //	public void visit(UnOpMinusExpression arg) throws XQueryException
    //	public void visit(Value arg) throws XQueryException
    //	public void visit(ValueBoolean arg) throws XQueryException
    //	public void visit(ValueFloat arg) throws XQueryException
    //	public void visit(ValueInteger arg) throws XQueryException
    //	public void visit(ValueString arg) throws XQueryException
    public void visit(Variable arg) throws XQueryException {
        if (!declaredvariables.contains(arg)) {
            //if (stopWhenFound) hasFound = true;
            if (!variables.contains(arg)) variables.add(arg);
        }
    }
    
    public void visit(XQueryBinaryOperatorExpression arg) throws XQueryException {
        if (arg.getExpression1() != null) {
            //if (stopWhenFound && hasFound) return;
            arg.getExpression1().accept(this);
        }
        if (arg.getExpression2() != null) {
            //if (stopWhenFound && hasFound) return;
            arg.getExpression2().accept(this);
        }
    }
    
    //	public void visit(XQueryBooleanOperatorExpression arg) throws XQueryException
    
    public void visit(XQueryExpression arg) throws XQueryException {
        //System.out.println("Call GetUsedVariables -> XQueryExpression ( " + arg.getClass().getName() + " ) : " + arg.pprint () );
    }
    
    
    public void visit(XQueryExpressionSequence arg) throws XQueryException {
        for (int i = 0; i < arg.getSubExpressions().size(); i++) {
            //if (stopWhenFound && hasFound) return;
            ((XQueryExpression)arg.getSubExpressions().get(i)).accept(this);
        }
    }
    
    //	public void visit(XQueryListBinaryOperatorExpression arg) throws XQueryException
    //	public void visit(XQueryListUnaryOperatorExpression arg) throws XQueryException
    public void visit(XQueryUnaryOperatorExpression arg) throws XQueryException {
        if (arg.getExpression() != null) {
            //if (stopWhenFound && hasFound) return;
            arg.getExpression().accept(this);
        }
    }
    
    public void visit(XQueryFile arg) throws XQueryException {
        for (int i = 0 ; i < arg.getUnits().size() ; i ++) {
            //if (stopWhenFound && hasFound) return;
            XQueryUnit uniti = (XQueryUnit) arg.getUnits().get(i) ;
            uniti.accept(this) ;
        }
    }
    
    public void visit(XQueryUnit arg) throws XQueryException {
        for (int i = 0 ; i < arg.getExpressions().size() ; i ++) {
            //if (stopWhenFound && hasFound) return;
            XQueryExpression expressioni = (XQueryExpression) arg.getExpressions().get(i) ;
            expressioni.accept(this) ;
        }
    }
    
    //	public void visit(XQueryVoid arg) throws XQueryException;
    
}
