/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * Copyright (C) 2003 XQuark Group.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */

package org.xquark.mediator.decomposer;

import java.util.ArrayList;

import org.xquark.xquery.metadata.MetaDataImpl;
import org.xquark.xquery.parser.*;
import org.xquark.xquery.parser.util.ArrayListUtil;

public class MakeStepsListVisitor extends DefaultParserVisitor {
	// **********************************************************************
	// * VERSIONING
	// **********************************************************************
	private static final String RCSRevision = "$Revision: 1.3 $";
	private static final String RCSName = "$Name:  $";

	// **********************************************************************
	// * CLASS VARIABLES
	// **********************************************************************
	private StepLists steplists = null;
	private boolean noOrderBy = false;
	// **********************************************************************
	// * INITIALIZATION
	// **********************************************************************
	/**
	 *
	 */
	public MakeStepsListVisitor(MetaDataImpl metadata, boolean noOrderBy) {
		this.steplists = new StepLists(metadata);
		this.noOrderBy = noOrderBy;
	}
	//	public MakeStepsListVisitor(StepLists steplists, boolean noOrderBy) {
	//		this.steplists = steplists;
	//		this.noOrderBy = noOrderBy;
	//	}
	//	public MakeStepsListVisitor(MetaDataImpl metadata) {
	//		this.steplists = new StepLists(metadata);
	//	}
//	public MakeStepsListVisitor(StepLists steplists) {
//		this.steplists = steplists;
//	}
	public MakeStepsListVisitor() {
	}

	// **********************************************************************
	// * METHODS
	// **********************************************************************
	public StepLists getStepsLists() {
		return steplists;
	}
	public void setStepsLists(StepLists steplists) {
		this.steplists = steplists;
	}

	// **********************************************************************
	// * VISITATION
	// **********************************************************************
	//	public void visit(AttributeValuePair arg) throws XQueryException
	// public void visit(BinOpANDExpression arg) throws XQueryException
	// public void visit(BinOpORExpression arg) throws XQueryException
	//	public void visit(CastTreatExpression arg) throws XQueryException
	//	public void visit(ContextDeclaration arg) throws XQueryException
	//	public void visit(Dereference arg) throws XQueryException
	public void visit(Element arg) throws XQueryException {
		///*DEBUG*/System.out.println ("Visiting " + arg.getClass () + arg.pprint ()) ;
		ArrayList al = arg.getSubExpressions();
		if (al == null)
			return;
		for (int i = 0; i < al.size(); i++) {
			((XQueryExpression) al.get(i)).accept(this);
		}
	}

	public void visit(FLWRExpression arg) throws XQueryException {
		///*DEBUG*/System.out.println ("Visiting " + arg + " -> " + arg.getClass ()) ;
		ArrayList vars = arg.getVariables();
		ArrayList sources = null;
		ArrayList urls = null;
		for (int i = 0; i < vars.size(); i++) {
			Variable vari = (Variable) vars.get(i);
			// At this point, in a QDB case, there should be only
			// a LocatedExpression of type Collection
			//if ((vari.getSourceNames() == null || vari.getSourceNames().isEmpty()) && (vari.getUrls() == null || vari.getUrls().isEmpty()))
			steplists.registerVar(vari);
			if (vari.getSourceNames() != null && !vari.getSourceNames().isEmpty()) {
				if (sources == null)
					sources = (ArrayList) vari.getSourceNames().clone();
				else
					ArrayListUtil.add(sources, vari.getSourceNames());
			}
			if (vari.getUrls() != null && !vari.getUrls().isEmpty()) {
				if (urls == null)
					urls = (ArrayList) vari.getUrls().clone();
				else
					ArrayListUtil.add(urls, vari.getUrls());
			}
		}

		XQueryExpression expr = arg.getWhereClause();
		if (expr != null) {
			steplists.newStepslist();
			expr.accept(this);
			if (expr.getSourceNames() != null && !expr.getSourceNames().isEmpty()) {
				if (sources == null)
					sources = (ArrayList) expr.getSourceNames().clone();
				else
					ArrayListUtil.add(sources, expr.getSourceNames());
			}
			if (expr.getUrls() != null && !expr.getUrls().isEmpty()) {
				if (urls == null)
					urls = (ArrayList) expr.getUrls().clone();
				else
					ArrayListUtil.add(urls, expr.getUrls());
			}

		}
		expr = arg.getReturnClause();
		expr.accept(this);
		if (expr.getSourceNames() != null && !expr.getSourceNames().isEmpty()) {
			if (sources == null)
				sources = (ArrayList) expr.getSourceNames().clone();
			else
				ArrayListUtil.add(sources, expr.getSourceNames());
		}
		if (expr.getUrls() != null && !expr.getUrls().isEmpty()) {
			if (urls == null)
				urls = (ArrayList) expr.getUrls().clone();
			else
				ArrayListUtil.add(urls, expr.getUrls());
		}

		//
		//		if (!noOrderBy && arg.getOrderBy() != null) {
		//			ArrayList al = arg.getOrderBy();
		//			for (int i = 0; i < al.size(); i++)
		//				 ((XQueryExpression) al.get(i)).accept(this);
		//		}

		//		Set set = this.steplists.getRegisteredVariables();
		//		if (set == null) {
		//			arg.setSourceNames(null);
		//			arg.setUrls(null);
		//		}
		//		else {
		//			ArrayList sourceNames = null;
		//			ArrayList urls = null;
		//			for (Iterator it=set.iterator();it.hasNext();) {
		//				Variable var = (Variable)it.next();
		//				if (var.getSourceNames() != null && !var.getSourceNames().isEmpty()) {
		//					if (sourceNames == null)
		//					sourceNames = (ArrayList)var.getSourceNames().clone();
		//					else
		//						ArrayListUtil.add(sourceNames,var.getSourceNames());
		//				}
		//				if (var.getUrls() != null && !var.getUrls().isEmpty()) {
		//					if (urls == null)
		//					urls = (ArrayList)var.getUrls().clone();
		//					else
		//						ArrayListUtil.add(urls,var.getUrls());
		//				}				
		//			}
		//			arg.setSourceNames(sourceNames);
		//			arg.setUrls(urls);
		//		}
		arg.setSourceNames(sources);
		arg.setUrls(urls);
	}

	public void visit(QuantifiedExpression arg) throws XQueryException {
		///*DEBUG*/System.out.println ("Visiting " + arg.getClass ()) ;
		ArrayList variables = arg.getVariables();
		// At this point, in a QDB case, there should be only
		// a LocatedExpression of type Collection
		steplists.registerVars(variables);

		if (arg.getExpression() != null) {
			steplists.newStepslist();
			arg.getExpression().accept(this);
		}
	}

	public void visit(FunctionCall arg) throws XQueryException {
		///*DEBUG*/System.out.println ("Visiting " + arg.getClass ()) ;
		ArrayList al = arg.getArguments();
		if (al == null)
			return;
		for (int i = 0; i < al.size(); i++)
			 ((XQueryExpression) al.get(i)).accept(this);
	}

	//	public void visit(FunctionDefinition arg) throws XQueryException
	//	public void visit(InstanceOfExpression arg) throws XQueryException
	//	public void visit(InternalFunctionCall arg) throws XQueryException
	//	public void visit(ITEExpression arg) throws XQueryException
	//	public void visit(LibraryFunctionCall arg) throws XQueryException
	//	public void visit(ListOpAFTERExpression arg) throws XQueryException
	//	public void visit(ListOpArithExpression arg) throws XQueryException
	//	public void visit(ListOpBEFOREExpression arg) throws XQueryException
	//  public void visit(ListOpCompExpression arg) throws XQueryException
	//	public void visit(ListOpEXCEPTExpression arg) throws XQueryException
	//	public void visit(ListOpINTERSECTExpression arg) throws XQueryException
	//	public void visit(ListOpUNIONExpression arg) throws XQueryException

	public void visit(LocatedExpression arg) throws XQueryException {
		///*DEBUG*/System.out.println ("Visiting " + arg.getClass () + " : " + arg) ;
		steplists.newStepslist();
		ArrayList steps = arg.getSteps();
		for (int i = 0; i < steps.size(); i++)
			 ((Step) steps.get(i)).accept(this);
	}

	//	public void visit(NodeType arg) throws XQueryException
	//	public void visit(Parameter arg) throws XQueryException
	//	public void visit(PrimitiveFunctionCall arg) throws XQueryException
	//  public void visit (QName arg) throws XQueryException
	//	public void visit(QuantifiedExpression arg) throws XQueryException
	//	public void visit(RangeExpression arg) throws XQueryException
	//	public void visit(RootNodeFunctionCall arg) throws XQueryException
	public void visit(SortedExpression arg) throws XQueryException {
		///*DEBUG*/System.out.println ("Visiting " + arg.getClass () + " : " + arg) ;
		arg.getExpression().accept(this);
		arg.setSourceNames(arg.getExpression().getSourceNames());
		arg.setUrls(arg.getExpression().getUrls());
	}

	public void visit(Step arg) throws XQueryException {
		///*DEBUG*/System.out.println ("Visiting " + arg.getClass () + " : " + arg) ;
		steplists.addStep(arg);
	}

	//	public void visit(TypeSwitchExpression arg) throws XQueryException
	//	public void visit(UnOpMinusExpression arg) throws XQueryException
	//    public void visit(Value arg) throws XQueryException {
	//        ///*DEBUG*/System.out.println ("Visiting " + arg.getClass ()) ;
	//        steplists.addStep(new Step(-1,-1,arg,null,null)) ;
	//    }
	//	public void visit(ValueBoolean arg) throws XQueryException
	//	public void visit(ValueFloat arg) throws XQueryException
	//	public void visit(ValueInteger arg) throws XQueryException
	//	public void visit(ValueString arg) throws XQueryException
	//	public void visit(ValueText arg) throws XQueryException
	public void visit(Variable arg) throws XQueryException {
		///*DEBUG*/System.out.println ("Visiting " + arg.getClass ()) ;
		steplists.addStep(arg);
	}

	public void visit(XQueryBinaryOperatorExpression arg) throws XQueryException {
		///*DEBUG*/System.out.println ("Visiting " + arg.getClass ()) ;
		arg.getExpression1().accept(this);
		arg.getExpression2().accept(this);
	}

	//	public void visit(XQueryBooleanOperatorExpression arg) throws XQueryException
	// public void visit (XQueryExpression arg) throws XQueryException

	public void visit(XQueryExpressionSequence arg) throws XQueryException {
		///*DEBUG*/System.out.println ("Visiting " + arg.getClass ()) ;
		ArrayList al = arg.getSubExpressions();
		if (al == null)
			return;
		for (int i = 0; i < al.size(); i++)
			 ((XQueryExpression) al.get(i)).accept(this);
	}

	// public void visit(XQueryListBinaryOperatorExpression arg) throws XQueryException
	// public void visit(XQueryListUnaryOperatorExpression arg) throws XQueryException

	public void visit(XQueryUnaryOperatorExpression arg) throws XQueryException {
		///*DEBUG*/System.out.println ("Visiting " + arg.getClass ()) ;
		arg.getExpression().accept(this);

	}
}
