/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */

package org.xquark.mediator.decomposer;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.Set;

import org.xquark.xquery.metadata.MetaDataImpl;
import org.xquark.xquery.parser.Variable;
import org.xquark.xquery.parser.XQueryException;
import org.xquark.xquery.parser.XQueryExpression;
import org.xquark.xquery.parser.util.ArrayListUtil;

public class Utils {
	// **********************************************************************
	// * VERSIONING
	// **********************************************************************
	private static final String RCSRevision = "$Revision: 1.2 $";
	private static final String RCSName = "$Name:  $";

	// **********************************************************************
	// * CLASS VARIABLES
	// **********************************************************************

	// **********************************************************************
	// * INITIALIZATION
	// **********************************************************************

	/*
	static public void FillSources(XQueryFile arg, UnitEnvironment environment, MetaData metadata) throws XQueryException {
	        for (int i = 0; i < arg.getUnits().size(); i++) FillSources((XQueryUnit)arg.getUnits().get(i),environment,metadata);
	}
	
	static public void FillSources(XQueryUnit arg, UnitEnvironment environment, MetaData metadata) throws XQueryException {
	        for (int i = 0; i < arg.getExpressions().size(); i++) FillSources((XQueryExpression)arg.getExpressions().get(i),environment,metadata);
	}
	 */

	/**
	 * Finds out on which sources this sources a <code>XQueryExpression</code> is defined, and
	 * attach this information (a List of String presenting source names)to this <code>XQueryExpression</code>.
	 * @param arg
	 * @param metadata
	 * @throws XQueryException
	 * note : this is not recursive, only this <code>arg</code>, none of its child traited.
	 */
	static public void fillSources(XQueryExpression arg, MetaDataImpl metadata, boolean force) throws XQueryException {

		// if it already has been generated
		if (arg.getSourceNames() != null) {
			return;
		}
		//arg.setSources(new ArrayList(1));

		MakeStepsListVisitor stepvisitor = new MakeStepsListVisitor(metadata, true);
		arg.accept(stepvisitor);
		
		if (force || arg.getSourceNames() == null || arg.getSourceNames().isEmpty()) {
			
			// CHANGE   11/04/2003
			Set set = stepvisitor.getStepsLists().getRegisteredVariables();
			if (set == null) {
				arg.setSourceNames(null);
				arg.setUrls(null);
				return;
			}
			ArrayList sourceNames = null;
			ArrayList urls = null;
			for (Iterator it=set.iterator();it.hasNext();) {
				Variable var = (Variable)it.next();
				if (var.getSourceNames() != null && !var.getSourceNames().isEmpty()) {
					if (sourceNames == null)
					sourceNames = (ArrayList)var.getSourceNames().clone();
					else
						ArrayListUtil.add(sourceNames,var.getSourceNames());
				}
				if (var.getUrls() != null && !var.getUrls().isEmpty()) {
					if (urls == null)
					urls = (ArrayList)var.getUrls().clone();
					else
						ArrayListUtil.add(urls,var.getUrls());
				}				
			}
			arg.setSourceNames(sourceNames);
			arg.setUrls(urls);
			
//			ArrayList paths = stepvisitor.getStepsLists().getStepsLists();
//
//			if (paths != null && !paths.isEmpty()) {
//				PathResolver pathresolver = new PathResolver(metadata);
//				pathresolver.resolveStepsList(paths);
//				ArrayList sourceNames = pathresolver.getSourceNames();
//				if (arg.getSourceNames() == null)
//					arg.setSourceNames(sourceNames);
//				else {
//					for (int i=0;i<sourceNames.size();i++) {
//						if (!arg.getSourceNames().contains(sourceNames.get(i)))
//							arg.getSourceNames().add(sourceNames.get(i));
//					}
//				}
//				ArrayList urls = pathresolver.getUrls();
//				if (urls != null && !urls.isEmpty()) {
//					if (arg.getUrls() == null)
//						arg.setUrls(urls);
//					else 
//					arg.getUrls().addAll(urls);
//				}
//			}
		}
		//else arg.setSources(new ArrayList(1));

		//		GenerateSourceForSonsVisitor visitor = new GenerateSourceForSonsVisitor(environment, metadata);
		//        arg.accept(visitor);
	}

	/**
	 *
	 */
	public static String makeIndent(int indent) {
		char[] buf = new char[indent];
		for (int i = 0; i < buf.length; i++)
			buf[i] = '\t';
		return new String(buf);
	}
}
