/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */

package org.xquark.mediator.plan ;

import java.util.*;

import org.xquark.mediator.MediatorConnection;
import org.xquark.mediator.MediatorException;
import org.xquark.mediator.DOMUtils.Tuple;
import org.xquark.schema.*;
import org.xquark.util.NamespaceContextStack;
import org.xquark.xml.xdbc.XMLConnection;
import org.xquark.xml.xdbc.XMLDBCException;
import org.xquark.xquery.parser.TypeVisitor;
import org.xquark.xquery.parser.Variable;
import org.xquark.xquery.parser.XQueryExpression;
import org.xquark.xquery.parser.hinttree.HintTree;
import org.xquark.xquery.xdbc.XDBCResultSetInterface;

public class ExecutionPlan {
    // **********************************************************************
    // * VERSIONING
    // **********************************************************************
    private static final String RCSRevision = "$Revision: 1.3 $";
    private static final String RCSName = "$Name:  $";

    // **********************************************************************
    // * CLASS VARIABLES
    // **********************************************************************

    private MediatorConnection connection = null ;
    private Operator root= null ;
    private ResultSet resultset = null ;
    private TypeVisitor typevisitor ;
	private String declarations = null;
	private ArrayList idvars = null;
    private HashMap aliases = null;
    private ArrayList currentTuples = new ArrayList();
    private String baseURI = null;
    private boolean qaOnly = false;
    
    private final int defaultsubqueryhandling = HintTree.MERGE_TYPE;
	//private final int defaultsubqueryhandling = HintTree.NESTED_TYPE;
    
	// to handle types generated by depnodesource
	SchemaManager schemamanager = null;
	Schema schema = null;
    // ***********************************************************************
    // * INITIALIZATION
    // ***********************************************************************
    public ExecutionPlan(MediatorConnection connection, TypeVisitor typevisitor, ArrayList idvars, NamespaceContextStack declarations, String baseURI, boolean qaOnly) throws MediatorException {
        if (typevisitor == null) throw new MediatorException("TypeVisitor in plan constructor cannot be null");
		this.connection = connection ;
        this.typevisitor = typevisitor ;
		this.idvars = idvars;
		this.baseURI = baseURI;
		this.qaOnly = qaOnly;
		// compute declaration string
        if (declarations == null)  this.declarations = null;
		else {
			StringBuffer buf = new StringBuffer();			
			Collection declPrefixes =  declarations.getPrefixes();			
//			Iterator itValues = declPrefixes.iterator();
//			while (itValues.hasNext()) { buf.append(" "); buf.append((()itValues.next()).pprint()); }
//			this.declarations = buf.toString();
			Iterator itValues = declPrefixes.iterator();
			String prefix = null;
			while (itValues.hasNext()) {
				prefix = (String)itValues.next();
				buf.append("namespace ");
				if (prefix != null) buf.append(prefix);
				else buf.append("default");
				buf.append(" = \"");
				buf.append(declarations.getNamespaceURI(prefix));
				buf.append("\"\n");
				}
			this.declarations = buf.toString();
		}
		// create schemamanager and schema
		schemamanager = typevisitor.getTypingSchemaManager();
		schema = schemamanager.getSchema("http://www.xquark.org/EXECUTIONPLAN");
    }

    // ***********************************************************************
    // * GET/SET METHODS
    // ***********************************************************************
    public XDBCResultSetInterface getResultSet() throws MediatorException {
        try {
            PlanResultSet presultset = new PlanResultSet(resultset) ;
            return presultset ;
        }
        catch (XMLDBCException xmldbce) {
            throw new MediatorException("ExecutionPlan.getResultSet: " + xmldbce.getMessage(), xmldbce) ;
        }
    }

    public SchemaManager getSchemaManager() { return schemamanager ; }

	public String getDeclarations() {return  declarations; }
	public TypeVisitor getTypeVisitor() { return typevisitor ; }
    public Operator getRoot() { return root ; }
    public void setRoot(Operator algebra) throws MediatorException { root = algebra ; }
    public ArrayList getIdVars() { return idvars ; }
    public void setIdVars(ArrayList idvars) { this.idvars = idvars ; }

	public MediatorConnection getConnection() throws MediatorException {
		return connection ;
	}
    public XMLConnection getConnection(String name) throws MediatorException {
        XMLConnection con = connection.getSubConnectionByName(name) ;
        return con ;
    }

    public String getAlias(String varstr ) {
		if (aliases == null) return null;
        for (Iterator it = aliases.keySet().iterator();it.hasNext();) {
            String stri = (String)it.next();
            if (varstr.startsWith(stri)) return (String)aliases.get(stri) + varstr.substring(stri.length());
        }
        return null;
    }
    
    public int getDefaultSubHandling() {
    	return defaultsubqueryhandling;
    }
    // ***********************************************************************
    // * METHODS
    // ***********************************************************************
    public void makeAlias(Variable var, XQueryExpression path) {
		if (aliases == null) aliases = new HashMap();
        aliases.put(var.toString(), path.toString()) ;
    }
    
    /*
    public void addReplaceValues(Tuple tuple) {    	
    	BufferTuple bt = tuple.getParent();
    	for (int i=0;i<tuple.size();i++) {
    		ArrayList tmplist = tuple.getNodesAtIndex(i);
    		if (tmplist.size() != 1) continue;
    		this.replaceValues.put(bt.getPathList().get(i),tmplist.get(0));
    	}
    }
    
    public void deleteReplaceValues(Tuple tuple) {
    	BufferTuple bt = tuple.getParent();
    	for (int i=0;i<tuple.size();i++) {
    		this.replaceValues.remove(bt.getPathList().get(i));
    	}
    }
    
    public HashMap getCurrentValues() {
    	return this.replaceValues;
    }
    */
    public void addCurrentTuple(Tuple tuple) {
    	this.currentTuples.add(0,tuple);
    }
    
    public void deleteCurrentTuple(Tuple tuple) {
    	this.currentTuples.remove(tuple);
    }
    
    public ArrayList getCurrentTuples() {
    	return this.currentTuples;
    }
    
    public String getBaseURI() {
    	return baseURI;
    }
    
    public boolean getQAOnly() {
    	return qaOnly;
    }

    // ***********************************************************************
    // * EXECUTE QUERY
    // ***********************************************************************
    /**
     *
     */
    public void executeQuery() throws MediatorException, XMLDBCException {
//		try {
//			this.schema.initialize();
//		}
//		catch(SAXException saxe) { throw new MediatorException("Could not initialize schema"); }
        if (root != null) {
            root.executeQuery() ;
            resultset = root.getResultSet() ;
        }
        else resultset = new FakeResultSet(null);
    }
    // **********************************************************************
    // * OPTIMIZE
    // **********************************************************************
    /**
     *
     */
    public void optimize() throws MediatorException {
        //root = root.optimize() ;
    }

    // type name handling
	private final String TYPE_NAME = "PlanType";
	private int typeIndex = 0;
	public void setTypeName(Type type) throws SchemaException {
		type.setName(TYPE_NAME+(++typeIndex));
		schema.register(type);
		this.schema.initialize();
//System.out.println(this.schemamanager.getType("http://www.xquark.org/EXECUTIONPLAN", TYPE_NAME+typeIndex));
	}
    // **********************************************************************
    // * DEBUG
    // **********************************************************************
    /**
     *
     */
    public String toString() {
        StringBuffer buf = new StringBuffer() ;
        buf.append("<Plan>\n") ;
        if (root != null) buf.append(root.toCompleteString(1)) ;
        buf.append("</Plan>\n") ;
        return buf.toString() ;
    }
}
