/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */

package org.xquark.mediator.plan;

import org.xquark.mediator.MediatorException;
import org.xquark.mediator.DOMUtils.BufferTuple;
import org.xquark.mediator.DOMUtils.EvaluationVisitor;
import org.xquark.mediator.DOMUtils.Tuple;
import org.xquark.xml.xdbc.XMLDBCException;
import org.xquark.xquery.parser.XQueryExpression;

public class OpAggregateFunc extends UnOp implements Runnable {
    // **********************************************************************
    // * VERSIONING
    // **********************************************************************
    private static final String RCSRevision = "$Revision: 1.1 $";
    private static final String RCSName = "$Name:  $";    // * INITIALIZATION
    // ***********************************************************************
    /**
     *
     */
    public OpAggregateFunc(ExecutionPlan plan, XQueryExpression expression, Operator childexp) throws MediatorException {
        super(plan, expression, childexp) ;
        size = 1;
        idsize = 0;
    }

	// #############################################################################
	// VISITOR STUFF
	// #############################################################################

	public void accept(OperatorVisitor visitor) throws MediatorException {
		visitor.visit(this);
	}

    // ***********************************************************************
    // * EXECUTE QUERY
    // ***********************************************************************
    /**
     *
     */
    public void executeQuery() throws MediatorException {
        super.executeQuery() ;
        try {
            resultset = new AggregateFuncResultSet(this) ;
        }
        catch (XMLDBCException e) {
            throw new MediatorException("Can't construct " + getClass() + ": " + e.getMessage(), e) ;
        }
    }

    // **********************************************************************
    // * OPTIMIZE
    // **********************************************************************
    /**
     * FuncMAX *this* node with the subnode(s). As it should be called
     * only from source, the returned node must be of type AlgSource.
     * The return node is a AlgSource where the XQueryExpression is the merging
     * of all child(ren) nodes' expression.
     */
    // protected Algebra mergeWithSub() throws MediatorException { return this ; }

    // **********************************************************************
    // * DEBUG
    // **********************************************************************
    /**
     *
     */
}


class AggregateFuncResultSet extends UnResultSet {
    // **********************************************************************
    // * VERSIONING
    // **********************************************************************
    private static final String RCSRevision = "$Revision: 1.1 $";
    private static final String RCSName = "$Name:  $";    // * CLASS VARIABLES
    // ***********************************************************************

    // ***********************************************************************
    // * INITIALIZATION
    // ***********************************************************************
    public AggregateFuncResultSet(OpAggregateFunc algebra) throws XMLDBCException {
        super(algebra) ;
    }

    // ***********************************************************************
    // * EVALUATE IMPLEMENTATION
    // ***********************************************************************
    /**
     * In a merge product, we can all do in parallel, so we
     * must generate all results of both source and then make the merge
     * product.
     *
     * @param non_blocking if the evaluation is blocking, we must
     *        generate all the results before passing.
     */
    protected void evaluate(boolean non_blocking) throws XMLDBCException {
        //ArrayList childpaths = null ;
        //Algebra child = ((AlgUnOp) algebra).getChildAlgebra() ;
        //childpaths = child.getPaths() ;

        if (! resultset.hasNext()) {
            resultset.close() ;
            setFinishedWhenEmpty() ;
            return ;
        }

        BufferTuple tmpBufferTuple = new BufferTuple(null,0);
        if (resultset.hasNext()) {
            while (resultset.hasNext()) {
                Tuple tuple = resultset.next() ;
//org.xquark.xml.misc.Debug.print("AlgAggregateFunction : tuple = " + tuple.toCompleteString());
                tuple.eraseIdSize();
                tmpBufferTuple.add(tuple);
            }
        }

        resultset.close() ;
        try {
            EvaluationVisitor visitor = new EvaluationVisitor(this.algebra.getPlan().getSchemaManager()) ;
            visitor.reset(tmpBufferTuple);
            visitor.setReturnType(EvaluationVisitor.TUPLE_TYPE);
            this.getAlgebra().getExpression().accept(visitor) ;
            Tuple tuple = visitor.getResultTuple();
            if (tuple != null) this.buftuples.add(tuple);
        }
        catch (Exception e) {
			e.printStackTrace();
			throw new XMLDBCException("Could not evaluate aggregate function"); 
        }
        
//org.xquark.xml.misc.Debug.print("buftuples = " + buftuples.toString());

        setFinishedWhenEmpty() ;
    }
}
