/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */

package org.xquark.mediator.plan;

import java.util.ArrayList;

import org.xquark.mediator.MediatorException;
import org.xquark.mediator.DOMUtils.EvaluationVisitor;
import org.xquark.mediator.DOMUtils.Tuple;
import org.xquark.xml.xdbc.XMLDBCException;
import org.xquark.xquery.parser.XQueryExpression;

public class OpFunc extends UnOp implements Runnable {
	// **********************************************************************
	// * VERSIONING
	// **********************************************************************
	private static final String RCSRevision = "$Revision: 1.1 $";
	private static final String RCSName = "$Name:  $"; // * INITIALIZATION
	// ***********************************************************************
	/**
	 *
	 */
	public OpFunc(ExecutionPlan plan, XQueryExpression expression, Operator childexp) throws MediatorException {
		super(plan, expression, childexp);
		size = 1;
		idsize = 0;
	}

	// #############################################################################
	// VISITOR STUFF
	// #############################################################################

	public void accept(OperatorVisitor visitor) throws MediatorException {
		visitor.visit(this);
	}

	// ***********************************************************************
	// * EXECUTE QUERY
	// ***********************************************************************
	/**
	 *
	 */
	public void executeQuery() throws MediatorException {
		super.executeQuery();
		try {
			resultset = new FuncResultSet(this);
		} catch (XMLDBCException e) {
			throw new MediatorException("Can't construct " + getClass() + ": " + e.getMessage(), e);
		}
	}

	// **********************************************************************
	// * OPTIMIZE
	// **********************************************************************
	/**
	 * FuncMAX *this* node with the subnode(s). As it should be called
	 * only from source, the returned node must be of type AlgSource.
	 * The return node is a AlgSource where the XQueryExpression is the merging
	 * of all child(ren) nodes' expression.
	 */
	// protected Algebra mergeWithSub() throws MediatorException { return this ; }

	// **********************************************************************
	// * DEBUG
	// **********************************************************************
	/**
	 *
	 */
}

class FuncResultSet extends UnResultSet {
	// **********************************************************************
	// * VERSIONING
	// **********************************************************************
	private static final String RCSRevision = "$Revision: 1.1 $";
	private static final String RCSName = "$Name:  $"; // * CLASS VARIABLES
	// ***********************************************************************
	EvaluationVisitor visitor = null;
	// ***********************************************************************
	// * INITIALIZATION
	// ***********************************************************************
	public FuncResultSet(OpFunc algebra) throws XMLDBCException {
		super(algebra);
		visitor = new EvaluationVisitor(this.algebra.getPlan().getSchemaManager());
	}

	// ***********************************************************************
	// * EVALUATE IMPLEMENTATION
	// ***********************************************************************
	/**
	 * In a merge product, we can all do in parallel, so we
	 * must generate all results of both source and then make the merge
	 * product.
	 *
	 * @param non_blocking if the evaluation is blocking, we must
	 *        generate all the results before passing.
	 */
	protected void evaluate(boolean non_blocking) throws XMLDBCException {
		//ArrayList childpaths = null ;
		//Algebra child = ((AlgUnOp) algebra).getChildAlgebra() ;
		//childpaths = child.getPaths() ;

		//boolean islet = this.algebra.isLet();
		Tuple newtuple = null;
		if (resultset == null) {
			try {
				ArrayList tuples = this.algebra.getPlan().getCurrentTuples();
				//if (tuples != null && !tuples.isEmpty()) {
					visitor = new EvaluationVisitor(this.algebra.getPlan().getSchemaManager());
					visitor.reset(tuples);
					visitor.setReturnType(EvaluationVisitor.NODE_TYPE);
					if (expression != null) {
						expression.accept(visitor);
						ArrayList resnodes = visitor.getResNodes();						
						if (resnodes != null && !resnodes.isEmpty()) {
							newtuple = buftuples.newTuple();
							newtuple.addNodesAtIndex(0,resnodes);
							buftuples.add(newtuple);
						}
					}
				//}
			} catch (Exception e) {
				//e.printStackTrace () ;
				throw new XMLDBCException("AlgNotSource.evaluate: " + e.getMessage());
			}
			setFinishedWhenEmpty();
		}
		else {
			if (!resultset.hasNext()) {
				resultset.close();
				setFinishedWhenEmpty();
				return;
			}
			while (true) {
				try {
					Tuple tuple = resultset.next();
					//org.xquark.xml.misc.Debug.print("AlgAggregateFunction : tuple = " + tuple.toCompleteString());
					tuple.eraseIdSize();
					visitor.reset(tuple);
					visitor.setReturnType(EvaluationVisitor.NODE_TYPE);
					this.getAlgebra().getExpression().accept(visitor);
					ArrayList resnodes = visitor.getResNodes();
					if (resnodes != null && !resnodes.isEmpty()) {
						newtuple = buftuples.newTuple();
						newtuple.addNodesAtIndex(0,resnodes);
						buftuples.add(newtuple);
						if (non_blocking)
							break;
					}
				}
				catch (Exception e) {
					e.printStackTrace();
					throw new XMLDBCException("Could not evaluate function");
				}
			}
			//org.xquark.xml.misc.Debug.print("buftuples = " + buftuples.toString());
			
			if (!resultset.hasNext()) {
				resultset.close();
				setFinishedWhenEmpty();
			}
		}
	}
}
