/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */

package org.xquark.mediator.plan;

import java.util.ArrayList;

import org.xquark.mediator.MediatorException;
import org.xquark.mediator.DOMUtils.Tuple;
import org.xquark.xml.xdbc.XMLDBCException;
import org.xquark.xquery.parser.XQueryExpression;

public class OpMerge extends BinOp implements Runnable {
	// **********************************************************************
	// * VERSIONING
	// **********************************************************************
	private static final String RCSRevision = "$Revision: 1.2 $";
	private static final String RCSName = "$Name:  $";

	// **********************************************************************
	// * CLASS VARIABLES
	// **********************************************************************
	// ***********************************************************************
	// * INITIALIZATION
	// ***********************************************************************
	/**
	 *
	 */
	public OpMerge(ExecutionPlan plan, XQueryExpression expression, Operator left, Operator right) throws MediatorException {
		super(plan, expression, left, right);
		init();
	}

//	public OpMerge(ExecutionPlan plan, XQueryExpression expression, ArrayList algebralist) throws MediatorException {
//		super(plan, expression, algebralist);
//		init();
//	}

	private void init() throws MediatorException {
		// Sometimes, it can happens that the two members of merge product have common attributes. In that case, we must mark them
		// as common. So when we will generate the tuples, we can access them efficiently.
		ArrayList childpaths1 = getLeftAlgebra().getPathsList();
		ArrayList childpaths2 = getRightAlgebra().getPathsList();
		size = getLeftAlgebra().getSize() + getRightAlgebra().getSize();

//		equivalences = new ArrayList();
//		int index = -1;
//		for (int i = 0; i < childpaths2.size(); i++) {
//			if ((index = childpaths1.indexOf(childpaths2.get(i))) != -1) {
//				addEquivalence(equivalences, index, i);
//				size--;
//				break;
//			}
//		}

//		idequivalences = new ArrayList();
//		ArrayList childvars1 = getLeftAlgebra().getVariables();
//		ArrayList childvars2 = getRightAlgebra().getVariables();
		idsize = getLeftAlgebra().getIdSize() + getRightAlgebra().getIdSize();
//		for (int i = 0; i < childvars2.size(); i++) {
//			Variable var2i = (Variable) childvars2.get(i);
//			if ((index = childvars1.indexOf(var2i)) != -1) {
//				addEquivalence(idequivalences, index, i);
//				idsize--;
//			}
//		}

		ordertype = ORDER_PARALLELIZATION;
	}

	// #############################################################################
	// VISITOR STUFF
	// #############################################################################

	public void accept(OperatorVisitor visitor) throws MediatorException {
		visitor.visit(this);
	}

	// ***********************************************************************
	// * EXECUTE QUERY
	// ***********************************************************************
	/**
	 *
	 */
	public void executeQuery() throws MediatorException {
		super.executeQuery();
		try {
			resultset = new MergeResultSet(this);
		} catch (XMLDBCException e) {
			throw new MediatorException("Can't construct " + getClass() + ": " + e.getMessage(), e);
		}
	}

	// **********************************************************************
	// * OPTIMIZE
	// **********************************************************************
	/**
	 * Merge *this* node with the subnode(s). As it should be called
	 * only from source, the returned node must be of type AlgSource.
	 * The return node is a AlgSource where the XQueryExpression is the merging
	 * of all child(ren) nodes' expression.
	 */
	// protected Algebra mergeWithSub() throws MediatorException { return this ; }

	// **********************************************************************
	// * DEBUG
	// **********************************************************************
	/**
	 *
	 */
}

class MergeResultSet extends BinResultSet {
	// **********************************************************************
	// * VERSIONING
	// **********************************************************************
	private static final String RCSRevision = "$Revision: 1.2 $";
	private static final String RCSName = "$Name:  $"; // * CLASS VARIABLES
	// ***********************************************************************
	
	private Tuple halftuple2 = null;
	// ***********************************************************************
	// * INITIALIZATION
	// ***********************************************************************
	public MergeResultSet(OpMerge algebra) throws XMLDBCException {
		super(algebra);
	}

	// ***********************************************************************
	// * EVALUATE IMPLEMENTATION
	// ***********************************************************************
	/**
	 * In a merge product, we can all do in parallel, so we
	 * must generate all results of both source and then make the merge
	 * product.
	 * @param non_blocking if the evaluation is blocking, we must
	 *        generate all the results before passing.
	 *			A merge is necessarly blocking.
	 */
	protected void evaluate(boolean non_blocking) throws XMLDBCException {
		
		//non_blocking = false;

		if (!this.getSubElement(0).hasNext()) {
			this.getSubElement(0).close();
			if (halftuple2 == null) 
				this.getSubElement(1).close();
			setFinishedWhenEmpty();
			return;
		}
		
		if (halftuple2 == null) {
			if (this.getSubElement(1).hasNext())
				halftuple2 = this.getSubElement(1).next() ;
			this.getSubElement(1).close();
		}		

//		if (!this.getSubElement(0).hasNext()) {
//			this.getSubElement(0).close();
//			if (halftuple2 == null) 
//				this.getSubElement(1).close();
//			setFinishedWhenEmpty();
//			return;
//		}
//		
//		if (halftuple2 == null) {
//			if (!this.getSubElement(1).hasNext()) {
//				this.getSubElement(0).close();
//				this.getSubElement(1).close();
//				setFinishedWhenEmpty();
//				return;
//			}
//			else {
//				halftuple2 = this.getSubElement(1).next() ;
//				this.getSubElement(1).close();
//			}
//		}		
			
        while (true) {
            if (this.getSubElement(0).hasNext()) {
                Tuple halftuple1 = this.getSubElement(0).next() ;
                if (halftuple2 != null)
					buftuples.add(halftuple1.merge(halftuple2, algebra));
				else
					buftuples.add(halftuple1.completeTupleForLeft(algebra.getSize()));
                if (non_blocking) {
                    // can make higher buffering by modifying this test
                    if ((buftuples != null) && (! buftuples.isEmpty()))
                        break ;
                }
            }
            else {
//org.xquark.xml.misc.Debug.print("BUF RESULT" + buftuples) ;
        		this.getSubElement(0).close();
                setFinishedWhenEmpty() ;
                return ;
            }
        }
        if (! this.getSubElement(0).hasNext()) {
        	this.getSubElement(0).close();
            setFinishedWhenEmpty() ;
//org.xquark.xml.misc.Debug.print("BUF RESULT" + buftuples) ;
        }
	}

	//    protected void evaluate(boolean non_blocking) throws XMLDBCException {
	//
	//        Algebra child1 = ((AlgBinOp) algebra).getLeftAlgebra() ;
	//        Algebra child2 = ((AlgBinOp) algebra).getRightAlgebra() ;
	//        ArrayList childpaths1 = child1.getPaths() ;
	//        ArrayList childpaths2 = child2.getPaths() ;
	//
	//        if ((! this.getSubElement(0).hasNext()) && (! this.getSubElement(1).hasNext())) {
	//            this.getSubElement(0).close() ;
	//            this.getSubElement(1).close() ;
	//            setFinishedWhenEmpty() ;
	//            return ;
	//        }
	//
	//        // fetch all tuples of rs1
	//        BufferTuple al1 = new BufferTuple(childpaths1) ;
	//        if (this.getSubElement(0).hasNext()) {
	//            while (this.getSubElement(0).hasNext()) {
	//                Tuple halftuple1 = this.getSubElement(0).next() ;
	////org.xquark.xml.misc.Debug.print("halftuple1 = " + halftuple1.toCompleteString());
	//                al1.add(halftuple1) ;
	//            }
	//        }
	//
	//        // fetch all tuples of rs2
	//        BufferTuple al2 = new BufferTuple(childpaths2) ;
	//        if (this.getSubElement(1).hasNext()) {
	//            while (this.getSubElement(1).hasNext()) {
	//                Tuple halftuple2 = this.getSubElement(1).next() ;
	////org.xquark.xml.misc.Debug.print("halftuple2 = " + halftuple2.toCompleteString());
	//                al2.add(halftuple2) ;
	//            }
	//        }
	//
	//        // make the merge product
	//        boolean [] has_been_used2 = new boolean [al2.size()] ;
	//        for (int i = 0 ; i < has_been_used2.length ; i ++) has_been_used2 [i] = false ;
	//
	//        //System.out.println ("BUFFER 1") ;
	//        //System.out.println (al1) ;
	//        //System.out.println ("BUFFER 2") ;
	//        //System.out.println (al2) ;
	//
	//        for (int i = 0 ; i < al1.size() ; i ++) {
	//            Tuple halftuple1 = (Tuple) al1.get(i) ;
	//            boolean has_been_used1 = false ;
	//            for (int j = 0 ; j < al2.size() ; j ++) {
	//                Tuple halftuple2 = (Tuple) al2.get(j) ;
	//                Tuple newtuple = halftuple1.merge(halftuple2, algebra) ;
	//                if (newtuple != null) {
	//                    buftuples.add(newtuple) ;
	//                    has_been_used1 = true ;
	//                    has_been_used2 [j] = true ;
	//                }
	//            }
	//            if (!has_been_used1) {
	//                Tuple tuple = halftuple1.completeTupleForLeft(algebra.getSize()) ;
	//                buftuples.add(tuple) ;
	//            }
	//        }
	//
	//        for (int i = 0 ; i < al2.size() ; i ++) {
	//            Tuple halftuple2 = (Tuple) al2.get(i) ;
	//            if (! has_been_used2 [i]) {
	//                Tuple tuple = halftuple2.completeTupleForRight(algebra.getSize(), algebra.getEquivalences()) ;
	//                buftuples.add(tuple) ;
	//            }
	//
	//        }
	//        this.getSubElement(0).close() ;
	//        this.getSubElement(1).close() ;
	//        setFinishedWhenEmpty() ;
	//    }

}
