/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */

package org.xquark.mediator.plan;

import java.util.ArrayList;

import org.xquark.mediator.MediatorException;
import org.xquark.mediator.DOMUtils.DOMUtils;
import org.xquark.mediator.DOMUtils.EvaluationVisitor;
import org.xquark.mediator.DOMUtils.Tuple;
import org.xquark.mediator.algebra.Algebra;
import org.xquark.xml.xdbc.XMLDBCException;
import org.xquark.xpath.datamodel.TypedNode;
import org.xquark.xquery.parser.Variable;
import org.xquark.xquery.parser.XQueryException;
import org.xquark.xquery.parser.XQueryExpression;

public class OpSimpleMerge extends UnOp implements Runnable {
	// **********************************************************************
	// * VERSIONING
	// **********************************************************************
	private static final String RCSRevision = "$Revision: 1.3 $";
	private static final String RCSName = "$Name:  $"; // * INITIALIZATION
	// ***********************************************************************
	private boolean onlyVar = false;

	public OpSimpleMerge(ExecutionPlan plan, XQueryExpression expression, Variable var, Operator childexp, boolean isLet) throws MediatorException {
		super(plan, expression, childexp);
		init(var);
		this.isLet(isLet);
	}

	public OpSimpleMerge(ExecutionPlan plan, Algebra algebra, Operator childexp) throws MediatorException {
		super(plan, algebra.getExpression(), childexp);
		Variable tmpVar = (Variable) algebra.getVariables().get(0);
		if (algebra.getPaths().isEmpty())
			init(tmpVar);
		else {
			ArrayList tmpPaths = algebra.getPaths();
			addPaths(tmpPaths);
			size = childalgebra.getSize() + tmpPaths.size();
			idsize = childalgebra.getIdSize();
			if (plan.getIdVars().contains(tmpVar))
				idsize++;
			if (tmpPaths.size() == 1 && tmpPaths.get(0).toString().equals(tmpVar.toString()))
				onlyVar = true;
		}
		this.isLet(algebra.isLet());
	}

	private void init(Variable var) {
		//resetPaths();
		addPath(var);
		size = childalgebra.getSize() + 1;
		idsize = childalgebra.getIdSize();
		if (plan.getIdVars().contains(var))
			idsize++;
		onlyVar = true;
	}

	public boolean isOnlyVar() {
		return onlyVar;
	}

	// #############################################################################
	// VISITOR STUFF
	// #############################################################################

	public void accept(OperatorVisitor visitor) throws MediatorException {
		visitor.visit(this);
	}

	// ***********************************************************************
	// * EXECUTE QUERY
	// ***********************************************************************
	/**
	 *
	 */
	public void executeQuery() throws MediatorException {
		super.executeQuery();
		try {
			resultset = new SimpleMergeResultSet(this);
		} catch (XMLDBCException e) {
			throw new MediatorException("Can't construct " + getClass() + ": " + e.getMessage(), e);
		}
	}

	// **********************************************************************
	// * OPTIMIZE
	// **********************************************************************
	/**
	 *
	 */

	// **********************************************************************
	// * DEBUG
	// **********************************************************************
	/**
	 *
	 */
}

class SimpleMergeResultSet extends UnResultSet {
	// **********************************************************************
	// * VERSIONING
	// **********************************************************************
	private static final String RCSRevision = "$Revision: 1.3 $";
	private static final String RCSName = "$Name:  $"; // * INITIALIZATION
	// ***********************************************************************
	EvaluationVisitor visitor = null;
	/**
	 *
	 */
	public SimpleMergeResultSet(OpSimpleMerge algebra) throws XMLDBCException {
		super(algebra);
	}

	// ***********************************************************************
	// * EVALUATE IMPLEMENTATION
	// ***********************************************************************
	/*
	 * @param non_blocking if the evaluation is blocking, we must
	 *        generate all the results before passing.
	 */
	protected void evaluate(boolean non_blocking) throws XMLDBCException {

		//non_blocking = false;

		//org.xquark.xml.misc.Debug.print("OpSimplMerge : expression = " + expression);
		///*DEBUG*/System.out.println (getClass () + ".evaluate") ;
		ArrayList idxsteps = new ArrayList();
		idxsteps.add(new Integer(0));
		boolean islet = this.algebra.isLet();
		XQueryExpression expression = this.algebra.getExpression();
		//org.xquark.xml.misc.Debug.print("algebra = " + algebra.toCompleteString());
		Tuple newtuple = null;
		while (true) {
			if (resultset.hasNext()) {
				Tuple tuple = resultset.next();
				//org.xquark.xml.misc.Debug.print("OpSimplMerge : path list = " + tuple.getParent().getPathList());
				//org.xquark.xml.misc.Debug.print("OpSimplMerge : tuple = " + tuple.toCompleteString());
				//org.xquark.xml.misc.Debug.print("OpSimplMerge : expression = " + expression);
				ArrayList nodes = tuple.getPath(expression.toString());
				if (nodes == null) {
					try {
						if (visitor == null)
							visitor = new EvaluationVisitor(this.algebra.getPlan().getSchemaManager());
						visitor.reset(tuple);
						visitor.setReturnType(EvaluationVisitor.NODE_TYPE);
						expression.accept(visitor);
						nodes = visitor.getResNodes();
					} catch (Exception e) {
						//e.printStackTrace();
						throw new XMLDBCException(e.getMessage());
					}
				}
				//if (nodes == null) throw new XMLDBCException("Could not evaluate expression : " + expression);
				if (islet) {
					if (((OpSimpleMerge) algebra).isOnlyVar()) {
						tuple.mergeInPlace(nodes, this.algebra);
						buftuples.add(tuple);
					} else {
						newtuple = buftuples.newTuple();
						for (int j = 0; j < paths.size(); j++) {
							String pathStrj = paths.get(j).toString();
							// test if paths already in tuple
							int index = tuple.getParent().getPathIndex(pathStrj);
							if (index != -1) {
								newtuple.addNodesAtIndex(j, tuple.getNodesAtIndex(index));
							} else if (pathStrj.equals(expression.toString())) {
								newtuple.addNodesAtIndex(j, nodes);
							} else {
								for (int i = 0; i < nodes.size(); i++) {
									ArrayList reslist = null;
									try {
										reslist = DOMUtils.getSteps((TypedNode) nodes.get(i), (XQueryExpression) paths.get(j));
									} catch (XQueryException xqe) {
										throw new XMLDBCException(xqe.getMessage());
									}
									if (reslist != null)
										newtuple.addNodesAtIndex(j, reslist);
								}
							}
						}
						//						for (int i = 0; i < nodes.size(); i++) {
						//							ArrayList nodelist = new ArrayList(1);
						//							nodelist.add(nodes.get(i));
						//							// get paths but test if path is expression !!!
						//							for (int j = 0; j < paths.size(); j++) {
						//								if (paths.get(j).toString().equals(expression.toString()))
						//									newtuple.addNodeAtIndex(j, (TypedNode) nodes.get(i));
						//								else {
						//									ArrayList reslist = null;
						//									try {
						//										reslist = DOMUtils.getSteps(nodelist, (XQueryExpression) paths.get(j));
						//									} catch (XQueryException xqe) {
						//										throw new XMLDBCException(xqe.getMessage());
						//									}
						//									if (reslist != null)
						//										newtuple.addNodesAtIndex(j, reslist);
						//								}
						//							}
						//						}
						newtuple.fillIdentifiers();
						buftuples.add(newtuple);
					}
					//org.xquark.xml.misc.Debug.print("OpSimplMerge : tuple = " + tuple.toCompleteString());
				} else {
					if (nodes != null) {
						if (((OpSimpleMerge) algebra).isOnlyVar()) {
							for (int i = 0; i < nodes.size(); i++) {
								newtuple = new Tuple(tuple);
								newtuple.mergeInPlace((TypedNode) nodes.get(i), this.algebra);
								//org.xquark.xml.misc.Debug.print("OpSimplMerge : newtuple = " + newtuple.toCompleteString());
								buftuples.add(newtuple);
							}
						} else {
							for (int i = 0; i < nodes.size(); i++) {
								ArrayList nodelist = new ArrayList(1);
								nodelist.add(nodes.get(i));
								newtuple = buftuples.newTuple();
								// get paths but test if path is expression !!!
								for (int j = 0; j < paths.size(); j++) {
									String pathStrj = paths.get(j).toString();
									// test if paths already in tuple
									int index = tuple.getParent().getPathIndex(pathStrj);
									if (index != -1) {
										newtuple.addNodesAtIndex(j, tuple.getNodesAtIndex(index));
									} else if (pathStrj.equals(expression.toString())) {
										newtuple.addNodesAtIndex(j, nodes);
									} else {
										ArrayList reslist = null;
										try {
											reslist = DOMUtils.getSteps(nodelist, (XQueryExpression) paths.get(j));
										} catch (XQueryException xqe) {
											throw new XMLDBCException(xqe.getMessage());
										}
										if (reslist != null)
											newtuple.addNodesAtIndex(j, reslist);
									}
								}
//								// get paths but test if path is expression !!!
//								for (int j = 0; j < paths.size(); j++) {
//									if (paths.get(j).toString().equals(expression.toString()))
//										newtuple.addNodeAtIndex(j, (TypedNode) nodes.get(i));
//									else {
//										ArrayList reslist = null;
//										try {
//											reslist = DOMUtils.getSteps(nodelist, (XQueryExpression) paths.get(j));
//										} catch (XQueryException xqe) {
//											throw new XMLDBCException(xqe.getMessage());
//										}
//										if (reslist != null)
//											newtuple.addNodesAtIndex(j, reslist);
//									}
//								}
								newtuple.fillIdentifiers();
								buftuples.add(newtuple);
							}
						}

					}
				}
				if (non_blocking) {
					// can make higher buffering by modifying this test
					if ((buftuples != null) && (!buftuples.isEmpty())) {
						//Debug.print("OpSimplMerge : buftuples = " + buftuples);
						break;
					}
				}
			} else {
				setFinishedWhenEmpty();
				resultset.close();
				//Debug.print("OpSimplMerge : " + algebra.toCompleteString());
				//Debug.print("OpSimplMerge : buftuples = " + buftuples);
				return;
			}
		}
//		if (!resultset.hasNext()) {
//			setFinishedWhenEmpty();
//			resultset.close();
//			//Debug.print("OpSimplMerge : " + algebra.toCompleteString());
//			//Debug.print("OpSimplMerge : buftuples = " + buftuples);
//		}
	}
}
