/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */

package org.xquark.mediator.plan;

import java.util.ArrayList;

import org.xquark.mediator.MediatorException;
import org.xquark.mediator.DOMUtils.DOMUtils;
import org.xquark.mediator.DOMUtils.SortedKeyMap;
import org.xquark.mediator.DOMUtils.Tuple;
import org.xquark.mediator.decomposer.Utils;
import org.xquark.xml.xdbc.XMLDBCException;
import org.xquark.xpath.datamodel.TypedNode;
import org.xquark.xquery.parser.ListOpCompExpression;
import org.xquark.xquery.parser.util.Constants;


public class OpSortMerge extends BinOp implements Runnable {
	// **********************************************************************
	// * VERSIONING
	// **********************************************************************
	private static final String RCSRevision = "$Revision: 1.3 $";
	private static final String RCSName = "$Name:  $";
	private ArrayList restrictions = null; // in fact ListOpComp
	// ***********************************************************************
	// * INITIALIZATION
	// ***********************************************************************
	/**
	 *
	 */

	public OpSortMerge(ExecutionPlan plan, ArrayList algebras, ArrayList restrictions) throws MediatorException {
		super(plan, null, algebras);
		this.restrictions = restrictions;
		init();
	}

	public OpSortMerge(ExecutionPlan plan, Operator left, Operator right, ArrayList restrictions) throws MediatorException {
		super(plan, null, left, right);
		this.restrictions = restrictions;
		init();
	}

	private void init() throws MediatorException {
		// Sometimes, it can happens that the two members of merge product have common attributes. In that case, we must mark them
		// as common. So when we will generate the tuples, we can access them efficiently.
		ArrayList childpaths1 = getLeftAlgebra().getPathsList();
		ArrayList childpaths2 = getRightAlgebra().getPathsList();
		size = getLeftAlgebra().getSize() + getRightAlgebra().getSize();

//		equivalences = new ArrayList();
//		int index = -1;
//		for (int i = 0; i < childpaths2.size(); i++) {
//			if ((index = childpaths1.indexOf(childpaths2.get(i))) != -1) {
//				addEquivalence(equivalences, index, i);
//				size--;
//				break;
//			}
//		}

//		idequivalences = new ArrayList();
//		ArrayList childvars1 = getLeftAlgebra().getVariables();
//		ArrayList childvars2 = getRightAlgebra().getVariables();
		idsize = getLeftAlgebra().getIdSize() + getRightAlgebra().getIdSize();
//		for (int i = 0; i < childvars2.size(); i++) {
//			Variable var2i = (Variable) childvars2.get(i);
//			if ((index = childvars1.indexOf(var2i)) != -1) {
//				addEquivalence(idequivalences, index, i);
//				idsize--;
//			}
//		}

		ordertype = ORDER_PARALLELIZATION;
	}

	public ArrayList getRestrictions() {
		return this.restrictions;
	}

	// #############################################################################
	// VISITOR STUFF
	// #############################################################################

	public void accept(OperatorVisitor visitor) throws MediatorException {
		visitor.visit(this);
	}

	// ***********************************************************************
	// * EXECUTE QUERY
	// ***********************************************************************
	/**
	 *
	 */
	public void executeQuery() throws MediatorException {
		/*
		if (restrictions == null || restrictions.size() != 1)
			throw new MediatorException("Restrictions number is not 1, not yet supported");
		*/
		if (restrictions != null) {
			for (int i = 0; i < restrictions.size(); i++) {
				int typ = ((ListOpCompExpression) restrictions.get(i)).getOperator();
				if (typ != Constants.EQ_COMPOP) {
					if (i != restrictions.size() - 1)
						throw new MediatorException("Incorrect restriction type, not yet supported");
					if (typ != Constants.LEQ_COMPOP && typ != Constants.LT_COMPOP && typ != Constants.GEQ_COMPOP && typ != Constants.GT_COMPOP)
						throw new MediatorException("Incorrect restriction type, not yet supported");
				}
			}
		}
		super.executeQuery();
		try {
			resultset = new SortMergeResultSet(this);
		} catch (XMLDBCException e) {
			//throw new MediatorException("Can't construct " + getClass() + ": " + e.getMessage(), e) ;
			throw new MediatorException("Can't construct " + getClass() + ": " + e.getMessage());
		}
	}

	// **********************************************************************
	// * OPTIMIZE
	// **********************************************************************
	/**
	 * Merge *this* node with the subnode(s). As it should be called
	 * only from source, the returned node must be of type AlgSource.
	 * The return node is a AlgSource where the XQueryExpression is the merging
	 * of all child(ren) nodes' expression.
	 */
	// protected Algebra mergeWithSub() throws MediatorException { return this ; }

	// **********************************************************************
	// * DEBUG
	// **********************************************************************

	public String toCompleteString(int indent) {
		StringBuffer buf = new StringBuffer();
		buf.append(Utils.makeIndent(indent) + "<" + getClass().getName() + " isLet=\"" + islet + "\" valueDepends=\"" + valueDepends + "\" whereDepends=\"" + whereDepends + "\">\n") ;
		buf.append(Utils.makeIndent(indent + 1) + "<Expression>\n");
		buf.append(Utils.makeIndent(indent + 2) + expression + "\n");
		buf.append(Utils.makeIndent(indent + 1) + "</Expression>\n");
//		if (variables != null) {
//			buf.append(Utils.makeIndent(indent + 1) + "<Variables>\n");
//			for (int i = 0; i < variables.size(); i++) {
//				buf.append(Utils.makeIndent(indent + 2) + "<Variable>" + variables.get(i) + "</Variable>\n");
//			}
//			buf.append(Utils.makeIndent(indent + 1) + "</Variables>\n");
//		}
		if (paths != null) {
			buf.append(Utils.makeIndent(indent + 1) + "<Paths>\n");
			for (int i = 0; i < paths.size(); i++) {
				buf.append(Utils.makeIndent(indent + 2) + "<Path>" + paths.get(i) + "</Path>\n");
			}
			buf.append(Utils.makeIndent(indent + 1) + "</Paths>\n");
		}
		if (restrictions != null) {
			buf.append(Utils.makeIndent(indent + 1) + "<Restrictions>\n");
			for (int i = 0; i < restrictions.size(); i++) {
				buf.append(Utils.makeIndent(indent + 2) + "<Restriction>" + restrictions.get(i) + "</Restriction>\n");
			}
			buf.append(Utils.makeIndent(indent + 1) + "</Restrictions>\n");
		}
		buf.append(Utils.makeIndent(indent + 1) + "<Elements>\n");
		for (int i = 0; i < algebralist.size(); i++) {
			buf.append(Utils.makeIndent(indent + 2) + "<Element" + (i + 1) + ">\n");
			buf.append(((Operator) algebralist.get(i)).toCompleteString(indent + 3));
			buf.append(Utils.makeIndent(indent + 2) + "</Element" + (i + 1) + ">\n");
		}
		buf.append(Utils.makeIndent(indent + 1) + "</Elements>\n");
		buf.append(Utils.makeIndent(indent) + "</" + getClass().getName() + ">\n");
		return buf.toString();
	}

}

class SortMergeResultSet extends BinResultSet {
	// **********************************************************************
	// * VERSIONING
	// **********************************************************************
	private static final String RCSRevision = "$Revision: 1.3 $";
	private static final String RCSName = "$Name:  $";
	private int size = 0;
	private Tuple left = null;
	private Tuple right = null;
	private Tuple nextright = null;
	private ArrayList leftstrings = null;
	private ArrayList rightstrings = null;
	private ArrayList leftbuffer = null;
	private ArrayList rightbuffer = null;
	private int operator = 0; // tells whether the last restriction is not an equality
	boolean hasIndex = false;
	private SortedKeyMap tm0 = null;
	private SortedKeyMap tm1 = null;

	// ***********************************************************************
	// * INITIALIZATION
	// ***********************************************************************
	/**
	 *
	 */
	public SortMergeResultSet(OpSortMerge algebra) throws XMLDBCException {
		super(algebra);
		size = algebra.getRestrictions().size();
		leftstrings = new ArrayList(size);
		rightstrings = new ArrayList(size);
		for (int i = 0; i < size; i++) {
			leftstrings.add(((ListOpCompExpression) algebra.getRestrictions().get(i)).getExpression1().toString());
			rightstrings.add(((ListOpCompExpression) algebra.getRestrictions().get(i)).getExpression2().toString());
		}
		if (algebra.getRestrictions() != null && !algebra.getRestrictions().isEmpty())
			operator = ((ListOpCompExpression) algebra.getRestrictions().get(size - 1)).getOperator();
		leftbuffer = new ArrayList();
		rightbuffer = new ArrayList();

	}

	// ***********************************************************************
	// * EVALUATE IMPLEMENTATION
	// ***********************************************************************
	/**
	 *
	 * @param non_blocking if the evaluation is blocking, we must
	 *        generate all the results before passing.
	 *		  A cartesian product is necessary blocking.
	 */
	protected void evaluate(boolean non_blocking) throws XMLDBCException {
		Tuple leftfront = null;
		Tuple rightfront = null;
		// look for indexes
		if (hasIndex || (this.getSubElement(0).getAlgebra().getIndexSpec(0) != null && this.getSubElement(1).getAlgebra().getIndexSpec(0) != null)) {
			//non_blocking = false;
			//org.xquark.xml.misc.Debug.print("indexspec 0 = " + this.getSubElement(0).getAlgebra().getIndexSpec(0));
			//org.xquark.xml.misc.Debug.print("indexspec 1 = " + this.getSubElement(1).getAlgebra().getIndexSpec(0));
			if (!hasIndex) {
				if (!this.getSubElement(0).hasNext() || !this.getSubElement(1).hasNext()) {
					setFinishedWhenEmpty();
					this.getSubElement(0).close();
					this.getSubElement(1).close();
					//org.xquark.xml.misc.Debug.print("buftuples = " + buftuples);
					return;
				}
				while (this.getSubElement(0).hasNext())
					this.getSubElement(0).next();
				tm0 = this.getSubElement(0).getIndex(0);
				tm0.makeIterator();
				this.getSubElement(0).close();
				while (this.getSubElement(1).hasNext())
					this.getSubElement(1).next();
				tm1 = this.getSubElement(1).getIndex(0);
				tm1.makeIterator();
				this.getSubElement(1).close();
				hasIndex = true;
			}
			//org.xquark.xml.misc.Debug.print("tm0 = " + tm0);
			//org.xquark.xml.misc.Debug.print("tm1 = " + tm1);
			if (operator == 0) {
				while (true) {
					if (left == null || right == null)
						if (!tm0.hasNext() || !tm1.hasNext()) {
							setFinishedWhenEmpty();
							this.getSubElement(0).close();
							this.getSubElement(1).close();
							//org.xquark.xml.misc.Debug.print("buftuples = " + buftuples);
							return;
						}
					// find first equality
					if (left == null) {
						left = tm0.next();
						//org.xquark.xml.misc.Debug.print("left = " + left.toCompleteString());
					}
					if (right == null) {
						right = tm1.next();
						//org.xquark.xml.misc.Debug.print("right = " + right.toCompleteString());
					}
					while (left != null && right != null) {
						int num = 0;
						while (num < size) {
							//org.xquark.xml.misc.Debug.print("left = " + left.toCompleteString());
							//org.xquark.xml.misc.Debug.print("right = " + right.toCompleteString());
							int res = DOMUtils.simpleCompareNodeValue((TypedNode) left.getPath((String) leftstrings.get(num)).get(0), (TypedNode) right.getPath((String) rightstrings.get(num)).get(0));
							if (res == 0)
								num++;
							else {
								if (res < 0) {
									if (tm0.hasNext()) {
										left = tm0.next();
										//org.xquark.xml.misc.Debug.print("left = " + left.toCompleteString());
									} else
										left = null;
								} else if (res > 0) {
									if (tm1.hasNext()) {
										right = tm1.next();
										//org.xquark.xml.misc.Debug.print("right = " + right.toCompleteString());
									} else
										right = null;
								}
								break;
							}
						}
						if (num == size)
							break;
					}
					if (left == null || right == null) {
						setFinishedWhenEmpty();
						this.getSubElement(0).close();
						this.getSubElement(1).close();
						//org.xquark.xml.misc.Debug.print("buftuples = " + buftuples);
						return;
					}
					// clean buffers
					leftbuffer.clear();
					leftbuffer.add(left);
					rightbuffer.clear();
					rightbuffer.add(right);
					// get all left elements equal to another and bufferize them
					while (true) {
						leftfront = null;
						if (tm0.hasNext()) {
							leftfront = tm0.next();
							//org.xquark.xml.misc.Debug.print("leftfront = " + leftfront.toCompleteString());
							int num = 0;
							while (num < size) {
								int res = DOMUtils.simpleCompareNodeValue((TypedNode) left.getPath((String) leftstrings.get(num)).get(0), (TypedNode) leftfront.getPath((String) leftstrings.get(num)).get(0));
								if (res == 0)
									num++;
								else
									break;
							}
							if (num == size) {
								leftbuffer.add(leftfront);
								left = leftfront;
							} else
								break;
						} else
							break;

					}
					// get all right elements equal to another and bufferize them
					while (true) {
						rightfront = null;
						if (tm1.hasNext()) {
							rightfront = tm1.next();
							//org.xquark.xml.misc.Debug.print("rightfront = " + rightfront.toCompleteString());
							int num = 0;
							while (num < size) {
								int res = DOMUtils.simpleCompareNodeValue((TypedNode) right.getPath((String) rightstrings.get(num)).get(0), (TypedNode) rightfront.getPath((String) rightstrings.get(num)).get(0));
								if (res == 0)
									num++;
								else
									break;
							}
							if (num == size) {
								rightbuffer.add(rightfront);
								right = rightfront;
							} else {
								break;
							}
						} else
							break;
					}
					// add all necessary tuples to buffertuple result
					for (int i = 0; i < leftbuffer.size(); i++) {
						for (int j = 0; j < rightbuffer.size(); j++) {
							this.buftuples.add(((Tuple) leftbuffer.get(i)).merge((Tuple) rightbuffer.get(j), this.algebra));
						}
					}
					left = leftfront;
					right = rightfront;
					if (non_blocking) {
						if ((buftuples != null) && (!buftuples.isEmpty()))
							break;
					}
				}
			} else { // with inequality

				while (true) {
					if (!tm0.hasNext()) {
						setFinishedWhenEmpty();
						this.getSubElement(0).close();
						this.getSubElement(1).close();
						//org.xquark.xml.misc.Debug.print("buftuples = " + buftuples);
						return;
					} else if (right == null && nextright == null && !tm1.hasNext() && rightbuffer.isEmpty()) {
						setFinishedWhenEmpty();
						this.getSubElement(0).close();
						this.getSubElement(1).close();
						//org.xquark.xml.misc.Debug.print("buftuples = " + buftuples);
						return;
					}
					int num;
					// find first equality
					if (left == null) {
						left = tm0.next();
						//org.xquark.xml.misc.Debug.print("left = " + left.toCompleteString());
					} else {
						leftfront = tm0.next();
						//org.xquark.xml.misc.Debug.print("leftfront = " + leftfront.toCompleteString());
						num = 0;
						while (num < size - 1) {
							int res = DOMUtils.simpleCompareNodeValue((TypedNode) left.getPath((String) leftstrings.get(num)).get(0), (TypedNode) leftfront.getPath((String) leftstrings.get(num)).get(0));
							if (res == 0)
								num++;
							else
								break;
						}
						if (num != size - 1) {
							rightbuffer.clear();
						}
						left = leftfront;
					}
					if (right == null) {
						if (nextright != null) {
							right = nextright;
							nextright = null;
						} else if (tm1.hasNext()) {
							right = tm1.next();
							//org.xquark.xml.misc.Debug.print("right = " + right.toCompleteString());
						}
					}
					// find first match
					boolean added = false;
					while (right != null) {
						num = 0;
						//org.xquark.xml.misc.Debug.print("left = " + left.toCompleteString());
						//org.xquark.xml.misc.Debug.print("right = " + right.toCompleteString());
						while (num < size) {
							int res = DOMUtils.simpleCompareNodeValue((TypedNode) left.getPath((String) leftstrings.get(num)).get(0), (TypedNode) right.getPath((String) rightstrings.get(num)).get(0));
							if (num != size - 1) {
								if (res == 0)
									num++;
								else {
									if (res < 0) {
										num = size + 1;
										break;
									} else if (res > 0) {
										if (tm1.hasNext()) {
											right = tm1.next();
											//org.xquark.xml.misc.Debug.print("right = " + right.toCompleteString());
										} else
											right = null;
									}
									break;
								}
							} else {
								if ((res == 0 && (operator == Constants.GEQ_COMPOP || operator == Constants.LEQ_COMPOP)) || (res < 0 && (operator == Constants.LT_COMPOP || operator == Constants.LEQ_COMPOP)) || (res > 0 && (operator == Constants.GEQ_COMPOP || operator == Constants.GT_COMPOP))) {
									num++;
								} else {
									num = size + 1;
									break;
								}
							}
						}
						if (num == size) {
							added = true;
							rightbuffer.add(right);
							break;
						}
						if (num > size)
							break;
					}

					if (rightbuffer.isEmpty()) {
						continue;
					}
					// get all right elements equal to another and bufferize them
					if (added) {
						rightfront = nextright;
						while (true) {
							if (rightfront == null)
								if (tm1.hasNext()) {
									rightfront = tm1.next();
									//org.xquark.xml.misc.Debug.print("rightfront = " + rightfront.toCompleteString());
								}
							if (rightfront != null) {
								//org.xquark.xml.misc.Debug.print("rightfront = " + rightfront.toCompleteString());
								num = 0;
								while (num < size) {
									if (num != size - 1) {
										int res = DOMUtils.simpleCompareNodeValue((TypedNode) right.getPath((String) rightstrings.get(num)).get(0), (TypedNode) rightfront.getPath((String) rightstrings.get(num)).get(0));
										if (res == 0)
											num++;
										else {
											nextright = rightfront;
											rightfront = null;
											break;
										}
									} else {
										int res = DOMUtils.simpleCompareNodeValue((TypedNode) left.getPath((String) leftstrings.get(num)).get(0), (TypedNode) rightfront.getPath((String) rightstrings.get(num)).get(0));
										if ((res == 0 && (operator == Constants.GEQ_COMPOP || operator == Constants.LEQ_COMPOP)) || (res < 0 && (operator == Constants.LT_COMPOP || operator == Constants.LEQ_COMPOP)) || (res > 0 && (operator == Constants.GEQ_COMPOP || operator == Constants.GT_COMPOP)))
											num++;
										else {
											nextright = rightfront;
											rightfront = null;
											break;
										}
									}
								}
								if (num == size) {
									rightbuffer.add(rightfront);
									right = rightfront;
									rightfront = null;
								} else {
									break;
								}
							} else
								break;
						}
						right = rightfront;
					}
					// add all necessary tuples to buffertuple result
					for (int j = 0; j < rightbuffer.size(); j++) {
						this.buftuples.add(left.merge((Tuple) rightbuffer.get(j), this.algebra));
					}
					if (non_blocking) {
						if ((buftuples != null) && (!buftuples.isEmpty()))
							break;
					}
				}
			}
			if (left == null || right == null)
				if (!tm0.hasNext() || (!tm1.hasNext()) && operator == 0) {
					setFinishedWhenEmpty();
					this.getSubElement(0).close();
					this.getSubElement(1).close();
					//org.xquark.xml.misc.Debug.print("buftuples = " + buftuples);
					return;
				}
			//org.xquark.xml.misc.Debug.print("buftuples = " + buftuples);
			return;
		}
		// NO INDEX FROM HERE ON !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
		if (operator == 0) {
			while (true) {
				if (left == null || right == null)
					if (!this.getSubElement(0).hasNext() || !this.getSubElement(1).hasNext()) {
						setFinishedWhenEmpty();
						this.getSubElement(0).close();
						this.getSubElement(1).close();
						//org.xquark.xml.misc.Debug.print("buftuples = " + buftuples);
						return;
					}
				// find first equality
				if (left == null) {
					left = this.getSubElement(0).next();
					org.xquark.util.misc.Debug.print("left = " + left.toCompleteString());
				}
				if (right == null) {
					right = this.getSubElement(1).next();
					org.xquark.util.misc.Debug.print("right = " + right.toCompleteString());
				}
				while (left != null && right != null) {
					int num = 0;
					while (num < size) {
						//org.xquark.xml.misc.Debug.print("left = " + left.toCompleteString());
						//org.xquark.xml.misc.Debug.print("right = " + right.toCompleteString());
						int res = DOMUtils.simpleCompareNodeValue((TypedNode) left.getPath((String) leftstrings.get(num)).get(0), (TypedNode) right.getPath((String) rightstrings.get(num)).get(0));
						if (res == 0)
							num++;
						else {
							if (res < 0) {
								if (this.getSubElement(0).hasNext()) {
									left = this.getSubElement(0).next();
									org.xquark.util.misc.Debug.print("left = " + left.toCompleteString());
								} else
									left = null;
							} else if (res > 0) {
								if (this.getSubElement(1).hasNext()) {
									right = this.getSubElement(1).next();
									org.xquark.util.misc.Debug.print("right = " + right.toCompleteString());
								} else
									right = null;
							}
							break;
						}
					}
					if (num == size)
						break;
				}
				if (left == null || right == null) {
					setFinishedWhenEmpty();
					this.getSubElement(0).close();
					this.getSubElement(1).close();
					//org.xquark.xml.misc.Debug.print("buftuples = " + buftuples);
					return;
				}
				// clean buffers
				leftbuffer.clear();
				leftbuffer.add(left);
				rightbuffer.clear();
				rightbuffer.add(right);
				// get all left elements equal to another and bufferize them
				while (true) {
					leftfront = null;
					if (this.getSubElement(0).hasNext()) {
						leftfront = this.getSubElement(0).next();
						org.xquark.util.misc.Debug.print("leftfront = " + leftfront.toCompleteString());
						int num = 0;
						while (num < size) {
							int res = DOMUtils.simpleCompareNodeValue((TypedNode) left.getPath((String) leftstrings.get(num)).get(0), (TypedNode) leftfront.getPath((String) leftstrings.get(num)).get(0));
							if (res == 0)
								num++;
							else
								break;
						}
						if (num == size) {
							leftbuffer.add(leftfront);
							left = leftfront;
						} else
							break;
					} else
						break;

				}
				// get all right elements equal to another and bufferize them
				while (true) {
					rightfront = null;
					if (this.getSubElement(1).hasNext()) {
						rightfront = this.getSubElement(1).next();
						org.xquark.util.misc.Debug.print("rightfront = " + rightfront.toCompleteString());
						int num = 0;
						while (num < size) {
							int res = DOMUtils.simpleCompareNodeValue((TypedNode) right.getPath((String) rightstrings.get(num)).get(0), (TypedNode) rightfront.getPath((String) rightstrings.get(num)).get(0));
							if (res == 0)
								num++;
							else
								break;
						}
						if (num == size) {
							rightbuffer.add(rightfront);
							right = rightfront;
						} else {
							break;
						}
					} else
						break;
				}
				// add all necessary tuples to buffertuple result
				for (int i = 0; i < leftbuffer.size(); i++) {
					for (int j = 0; j < rightbuffer.size(); j++) {
						this.buftuples.add(((Tuple) leftbuffer.get(i)).merge((Tuple) rightbuffer.get(j), this.algebra));
					}
				}
				left = leftfront;
				right = rightfront;
				if (non_blocking) {
					if ((buftuples != null) && (!buftuples.isEmpty()))
						break;
				}
			}
		} else { // with inequality

			while (true) {
				if (!this.getSubElement(0).hasNext()) {
					setFinishedWhenEmpty();
					this.getSubElement(0).close();
					this.getSubElement(1).close();
					//org.xquark.xml.misc.Debug.print("buftuples = " + buftuples);
					return;
				} else if (right == null && nextright == null && !this.getSubElement(1).hasNext() && rightbuffer.isEmpty()) {
					setFinishedWhenEmpty();
					this.getSubElement(0).close();
					this.getSubElement(1).close();
					//org.xquark.xml.misc.Debug.print("buftuples = " + buftuples);
					return;
				}
				int num;
				// find first equality
				if (left == null) {
					left = this.getSubElement(0).next();
					org.xquark.util.misc.Debug.print("left = " + left.toCompleteString());
				} else {
					leftfront = this.getSubElement(0).next();
					org.xquark.util.misc.Debug.print("leftfront = " + leftfront.toCompleteString());
					num = 0;
					while (num < size - 1) {
						int res = DOMUtils.simpleCompareNodeValue((TypedNode) left.getPath((String) leftstrings.get(num)).get(0), (TypedNode) leftfront.getPath((String) leftstrings.get(num)).get(0));
						if (res == 0)
							num++;
						else
							break;
					}
					if (num != size - 1) {
						rightbuffer.clear();
					}
					left = leftfront;
				}
				if (right == null) {
					if (nextright != null) {
						right = nextright;
						nextright = null;
					} else if (this.getSubElement(1).hasNext()) {
						right = this.getSubElement(1).next();
						org.xquark.util.misc.Debug.print("right = " + right.toCompleteString());
					}
				}
				// find first match
				boolean added = false;
				while (right != null) {
					num = 0;
					//org.xquark.xml.misc.Debug.print("left = " + left.toCompleteString());
					//org.xquark.xml.misc.Debug.print("right = " + right.toCompleteString());
					while (num < size) {
						int res = DOMUtils.simpleCompareNodeValue((TypedNode) left.getPath((String) leftstrings.get(num)).get(0), (TypedNode) right.getPath((String) rightstrings.get(num)).get(0));
						if (num != size - 1) {
							if (res == 0)
								num++;
							else {
								if (res < 0) {
									num = size + 1;
									break;
								} else if (res > 0) {
									if (this.getSubElement(1).hasNext()) {
										right = this.getSubElement(1).next();
										org.xquark.util.misc.Debug.print("right = " + right.toCompleteString());
									} else
										right = null;
								}
								break;
							}
						} else {
							if ((res == 0 && (operator == Constants.GEQ_COMPOP || operator == Constants.LEQ_COMPOP)) || (res < 0 && (operator == Constants.LT_COMPOP || operator == Constants.LEQ_COMPOP)) || (res > 0 && (operator == Constants.GEQ_COMPOP || operator == Constants.GT_COMPOP))) {
								num++;
							} else {
								num = size + 1;
								break;
							}
						}
					}
					if (num == size) {
						added = true;
						rightbuffer.add(right);
						break;
					}
					if (num > size)
						break;
				}

				if (rightbuffer.isEmpty()) {
					continue;
				}
				// get all right elements equal to another and bufferize them
				if (added) {
					rightfront = nextright;
					while (true) {
						if (rightfront == null)
							if (this.getSubElement(1).hasNext()) {
								rightfront = this.getSubElement(1).next();
								org.xquark.util.misc.Debug.print("rightfront = " + rightfront.toCompleteString());
							}
						if (rightfront != null) {
							//org.xquark.xml.misc.Debug.print("rightfront = " + rightfront.toCompleteString());
							num = 0;
							while (num < size) {
								if (num != size - 1) {
									int res = DOMUtils.simpleCompareNodeValue((TypedNode) right.getPath((String) rightstrings.get(num)).get(0), (TypedNode) rightfront.getPath((String) rightstrings.get(num)).get(0));
									if (res == 0)
										num++;
									else {
										nextright = rightfront;
										rightfront = null;
										break;
									}
								} else {
									int res = DOMUtils.simpleCompareNodeValue((TypedNode) left.getPath((String) leftstrings.get(num)).get(0), (TypedNode) rightfront.getPath((String) rightstrings.get(num)).get(0));
									if ((res == 0 && (operator == Constants.GEQ_COMPOP || operator == Constants.LEQ_COMPOP)) || (res < 0 && (operator == Constants.LT_COMPOP || operator == Constants.LEQ_COMPOP)) || (res > 0 && (operator == Constants.GEQ_COMPOP || operator == Constants.GT_COMPOP)))
										num++;
									else {
										nextright = rightfront;
										rightfront = null;
										break;
									}
								}
							}
							if (num == size) {
								rightbuffer.add(rightfront);
								right = rightfront;
								rightfront = null;
							} else {
								break;
							}
						} else
							break;
					}
					right = rightfront;
				}
				// add all necessary tuples to buffertuple result
				for (int j = 0; j < rightbuffer.size(); j++) {
					this.buftuples.add(left.merge((Tuple) rightbuffer.get(j), this.algebra));
				}
				if (non_blocking) {
					if ((buftuples != null) && (!buftuples.isEmpty()))
						break;
				}
			}
		}
		if (left == null || right == null)
			if (!this.getSubElement(0).hasNext() || (!this.getSubElement(1).hasNext()) && operator == 0) {
				setFinishedWhenEmpty();
				this.getSubElement(0).close();
				this.getSubElement(1).close();
				//org.xquark.xml.misc.Debug.print("buftuples = " + buftuples);
				return;
			}
		//org.xquark.xml.misc.Debug.print("buftuples = " + buftuples);
	}
}
