/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 XQuark Group.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */

package org.xquark.mediator.plan;

import java.util.ArrayList;

import org.xquark.mediator.MediatorException;
import org.xquark.mediator.decomposer.Utils;
import org.xquark.xml.xdbc.XMLDBCException;
import org.xquark.xml.xdbc.XMLResultSet;
import org.xquark.xml.xqueryevaluator.eval.EvalFilter;
import org.xquark.xquery.parser.FLWRExpression;
import org.xquark.xquery.parser.XQueryException;
import org.xquark.xquery.parser.XQueryExpression;

public class OpSourceEval extends ZeroOp implements Runnable {
	// **********************************************************************
	// * VERSIONING
	// **********************************************************************
	private static final String RCSRevision = "$Revision: 1.2 $";
	private static final String RCSName = "$Name:  $";

	// **********************************************************************
	// * CLASS VARIABLES
	// **********************************************************************
	//private XMLConnection connection = null;
	//private String source = null;

	// ***********************************************************************
	// * INITIALIZATION
	// ***********************************************************************
	/**
	 *
	 */
	public OpSourceEval(ExecutionPlan plan, XQueryExpression expression, ArrayList origpaths, boolean hasIdentifier) throws MediatorException {
		super(plan, expression);

		addPaths(origpaths);
		this.size = this.getPaths().size();
		if (hasIdentifier)
			this.idsize = 1;
	}

	// #############################################################################
	// VISITOR STUFF
	// #############################################################################

	public void accept(OperatorVisitor visitor) throws MediatorException {
		visitor.visit(this);
	}

	// ***********************************************************************
	// * EXECUTE QUERY
	// ***********************************************************************
	/**
	 *
	 */
	public void executeQuery() throws MediatorException {
		super.executeQuery();

		try {
			resultset = new SourceEvalResultSet(this);
		} catch (XMLDBCException e) {
			//e.printStackTrace();
			throw new MediatorException("Can't construct " + getClass() + ": " + e.getMessage() + " with expression " + expression, e);
		}
	}

	// **********************************************************************
	// * OPTIMIZE
	// **********************************************************************
	/**
	 * Return all the sources depending of this node.
	 */
	public ArrayList getSources() {
		return null;
	}

	public String toCompleteString(int indent) {
		StringBuffer buf = new StringBuffer();
		buf.append(Utils.makeIndent(indent) + "<" + getClass().getName() + " size=" + size + " id size=" + idsize + " isLet=\"" + islet + "\" valueDepends=\"" + valueDepends + "\" whereDepends=\"" + whereDepends + "\">\n") ;
		buf.append(Utils.makeIndent(indent + 1) + "<Expression>\n");
		buf.append(Utils.makeIndent(indent + 2) + expression + "\n");
		buf.append(Utils.makeIndent(indent + 1) + "</Expression>\n");
//		buf.append(Utils.makeIndent(indent + 1) + "<Source>" + source + "</Source>\n");
//		if (variables != null) {
//			buf.append(Utils.makeIndent(indent + 1) + "<Variables>\n");
//			for (int i = 0; i < variables.size(); i++) {
//				buf.append(Utils.makeIndent(indent + 2) + "<Variable>" + variables.get(i) + "</Variable>\n");
//			}
//			buf.append(Utils.makeIndent(indent + 1) + "</Variables>\n");
//		}
		if (paths != null) {
			buf.append(Utils.makeIndent(indent + 1) + "<Paths>\n");
			for (int i = 0; i < paths.size(); i++) {
				buf.append(Utils.makeIndent(indent + 2) + "<Path>" + paths.get(i) + "</Path>\n");
			}
			buf.append(Utils.makeIndent(indent + 1) + "</Paths>\n");
		}
		buf.append(Utils.makeIndent(indent) + "</" + getClass().getName() + ">\n");
		return buf.toString();
	}

}

class SourceEvalResultSet extends ZeroResultSet {
	// **********************************************************************
	// * VERSIONING
	// **********************************************************************
	private static final String RCSRevision = "$Revision: 1.2 $";
	private static final String RCSName = "$Name:  $";

	// **********************************************************************
	// * CLASS VARIABLES
	// **********************************************************************
	private XMLResultSet resultset = null;
	private EvalFilter evalFilter;

	// ***********************************************************************
	// * INITIALIZATION
	// ***********************************************************************
	/**
	 *
	 */
	public SourceEvalResultSet(OpSourceEval algebra) throws XMLDBCException {
		super(algebra);
		evalFilter = new EvalFilter(this.buftuples,this.algebra.getPlan(),this.paths,(FLWRExpression)this.expression);
	}

	// ***********************************************************************
	// * EVALUATE IMPLEMENTATION
	// ***********************************************************************
	/**
	 *
	 * @param non_blocking if the evaluation is blocking, we must
	 *        generate all the results before passing.
	 */
	//int tupleindex = 0;
	protected void evaluate(boolean non_blocking) throws XMLDBCException {
		boolean isDone = false;
		try {
			org.xquark.util.misc.Debug.print("OpSourceEval : expression = " + expression);
			isDone = evalFilter.evaluate();
			org.xquark.util.misc.Debug.print("OpSourceEval : buftuples = " + buftuples);
		} catch (XQueryException e) {
			throw new XMLDBCException("OpSourceEval.evaluate: " + e.getMessage());
		}
		if (isDone) setFinishedWhenEmpty();		
	}

	protected void finalize() {
//		try {
//			if (null != resultset) {
//				resultset.close();
//			}
//		} catch (Exception ex) {};
//		try {
//			if (null != this.stat) {
//				stat.close();
//			}
//		} catch (Exception ex) {}
	}
}
