/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */

package org.xquark.mediator.plan;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

import org.xquark.mediator.MediatorException;
import org.xquark.mediator.DOMUtils.*;
import org.xquark.mediator.decomposer.Utils;
import org.xquark.xml.xdbc.XMLDBCException;
import org.xquark.xpath.datamodel.TypedNode;
import org.xquark.xquery.parser.ListOpCompExpression;
import org.xquark.xquery.parser.XQueryExpression;
import org.xquark.xquery.parser.hinttree.HintTree;
import org.xquark.xquery.parser.util.Constants;

public class OpSubQuery extends Operator {
	// **********************************************************************
	// * VERSIONING
	// **********************************************************************
	private static final String RCSRevision = "$Revision: 1.2 $";
	private static final String RCSName = "$Name:  $";
	// ***********************************************************************
	// * VARIABLES
	// ***********************************************************************
	private int[] locations = null;
	private int[] idlocations = null;
	//private HashMap subelementsrestrict = new HashMap() ;
	private boolean pivot = false;
	private boolean imbricated = false;
	protected ArrayList algebralist = null;
	protected int ordertype = ORDER_SERIALIZATION;
	protected int outertype = HintTree.NESTED_TYPE;
	protected ArrayList mergeRestrictions = null;
	protected ArrayList outsideRestrictions = null;

	// ***********************************************************************
	// * INITIALIZATION
	// ***********************************************************************
	/**
	 *
	 */

	public OpSubQuery(ExecutionPlan plan, XQueryExpression expression, ArrayList algebralist, boolean pivot, boolean imbricated, ArrayList mergeRestrictions, ArrayList outsideRestrictions) throws MediatorException {
		//public AlgSubQuery(ExecutionPlan plan, XQueryExpression expression,ArrayList algebralist) throws MediatorException {
		super(plan, expression);
		ordertype = ORDER_PARALLELIZATION;
		this.algebralist = algebralist;
		this.pivot = pivot;
		this.imbricated = imbricated;
		this.outertype = HintTree.MERGE_TYPE;
		this.mergeRestrictions = mergeRestrictions;
		this.outsideRestrictions = outsideRestrictions;
		init();
	}

	public OpSubQuery(ExecutionPlan plan, XQueryExpression expression, ArrayList algebralist, boolean pivot, boolean imbricated) throws MediatorException {
		//public AlgSubQuery(ExecutionPlan plan, XQueryExpression expression,ArrayList algebralist) throws MediatorException {
		super(plan, expression);
		ordertype = ORDER_PARALLELIZATION;
		this.algebralist = algebralist;
		this.pivot = pivot;
		this.imbricated = imbricated;
		init();
	}

//	public AlgSubQuery(ExecutionPlan plan, XQueryExpression expression, ArrayList algebralist, boolean pivot, boolean imbricated, boolean depends) throws MediatorException {
//		//public AlgSubQuery(ExecutionPlan plan, XQueryExpression expression,ArrayList algebralist) throws MediatorException {
//		super(plan, expression);
//		ordertype = ORDER_PARALLELIZATION;
//		this.algebralist = algebralist;
//		this.pivot = pivot;
//		this.imbricated = imbricated;
//		this.depends = depends;
//		init();
//	}

	public boolean getPivot() {
		return this.pivot;
	}
	//public void setPivot(boolean pivot) { this.pivot = pivot; }
	public boolean getImbricated() {
		return this.imbricated;
	}
	//public void setImbricated(boolean imbricated) { this.imbricated = imbricated; }
	public int getOuterType() {
		return this.outertype;
	}

	public ArrayList getMergeRestrictions() {
		return this.mergeRestrictions;
	}
	public ArrayList getOutsideRestrictions() {
		return this.outsideRestrictions;
	}

	public void init() throws MediatorException {
		// Sometimes, it can happens that the two members of merge
		// product have common attributes. In that case, we must mark them
		// as common. So when we will generate the tuples, we can access
		// them efficiently.
		if (this.algebralist == null)
			return;
		this.resetPaths();
		if (this.paths == null || this.paths.isEmpty()) {
			for (int i = 0; i < algebralist.size(); i++) {
				addPaths(((Operator) algebralist.get(i)).getPaths());
			}
			/*
			if (subelementsrestrict != null && !subelementsrestrict.isEmpty()) {
				try {
					GetVariablePathVisitor visitor = new GetVariablePathVisitor(true) ;
					for (Iterator it = subelementsrestrict.keySet().iterator();it.hasNext();) {
						Integer index = (Integer)it.next();
						ArrayList tmplist = (ArrayList)subelementsrestrict.get(index);
						if (tmplist == null) continue;
						for (int i = 0 ; i < tmplist.size() ; i ++) {
							visitor.reset();
							((XQueryExpression)tmplist.get(i)).accept(visitor) ;
							ArrayList al = visitor.getPaths() ;
							addPaths(al);
						}
					}
				}
				catch (Exception e) { throw new MediatorException("Could not initialize AlgSubQuery", e); }
			}
			 */
		}
		size = ((Operator) this.getSubElement(0)).getSize(); //.getPaths().size() ;
		idsize = ((Operator) this.getSubElement(0)).getIdSize();
		locations = new int[this.getSubElementsSize() - 1];
		idlocations = new int[this.getSubElementsSize() - 1];
		for (int k = 1; k < this.getSubElementsSize(); k++) {
			locations[k - 1] = size;
			idlocations[k - 1] = idsize;
			size += ((Operator) getSubElement(k)).getSize(); //.getPaths().size();
			idsize += ((Operator) getSubElement(k)).getIdSize();
		}
	}

	/**
	 * Return a reference on the child at given position expression.
	 *
	 * @return a reference on the child at given position expression.
	 */
	public ArrayList getSubElements() {
		return algebralist;
	}
	public void setSubElements(ArrayList algebralist) {
		this.algebralist = algebralist;
	}

	public Operator getSubElement(int pos) {
		if (pos < 0 || pos >= algebralist.size())
			return null;
		return (Operator) algebralist.get(pos);
	}

	public int getSubElementsSize() {
		return this.algebralist.size();
	}

	public int[] getIdLocations() {
		return idlocations;
	}
	public int[] getLocations() {
		return locations;
	}

	/**
	 * Return true if the node is of type source.
	 * AlgBinOp is not a source.
	 *
	 * @return false ;
	 */
	public boolean isSource() {
		return false;
	}

	/**
	 * Return all the sources depending of this node.
	 * @return
	 */
	public ArrayList getSources() {
		/*
		if (sources == null) {
			sources = new ArrayList() ;
			for (int i=0;i<algebralist.size();i++) {
				ArrayList tmplist = ((Algebra)algebralist.get(i)).getSources();
				for (int j=0;j<tmplist.size();j++) {
					Object sourcej = tmplist.get(j) ;
					if (! sources.contains(sourcej)) sources.add(sourcej) ;
				}
			}
		}
		return sources ;
		 */
		return null;
	}

	protected boolean shareSameSources() {
		/*
		for (int i=0;i<algebralist.size();i++)
			if (((Algebra)algebralist.get(i)).shareSameSources() == false) return false;
		if (compareSource()) return true ;
		 */
		return false;
	}

	// #############################################################################
	// VISITOR STUFF
	// #############################################################################

	public void accept(OperatorVisitor visitor) throws MediatorException {
		visitor.visit(this);
	}

	// ***********************************************************************
	// * EXECUTE QUERY
	// ***********************************************************************
	/**
	 *
	 */
	public void executeQuery() throws MediatorException {
		///*DEBUG*/System.out.println ("****** " + getClass () + ".executeQuery ()") ;
		Thread[] threads = null;
		try {
			if (ordertype == ORDER_SERIALIZATION) {
				for (int i = 0; i < algebralist.size(); i++) {
					Operator algebrai = (Operator) algebralist.get(i);
					if (!algebrai.getDepends() || i == 0 || this.outertype == HintTree.MERGE_TYPE)
						 ((Operator) algebralist.get(i)).run();
				}
			} else if (ordertype == ORDER_PARALLELIZATION) {
				threads = new Thread[algebralist.size()];
				for (int i = 0; i < algebralist.size(); i++) {
					Operator algebrai = (Operator) algebralist.get(i);
					if (!algebrai.getDepends() || i == 0 || this.outertype == HintTree.MERGE_TYPE) {
						threads[i] = new Thread((Runnable) algebrai);
						threads[i].start();
					}
				}
			}

		} catch (Exception e) {
			//throw new MediatorException("AlgMultOp.executeQuery: " +  e.getMessage(),e) ;
			throw new MediatorException("AlgMultOp.executeQuery: " + e.getMessage());
		}

		try {
			// wait for all threads to exit
			for (int i = 0; i < threads.length; i++) {
				Thread thread = threads[i];
				if (thread != null && thread.isAlive())
					thread.join();
			}
		} catch (Exception e) {
			//throw new MediatorException("AlgSubQuery.executeQuery: " +  e.getMessage(),e) ;
			throw new MediatorException("AlgSubQuery.executeQuery: " + e.getMessage());
		}

		try {
			/*
			if (pivot) {
				if (imbricated) resultset = new SubQueryPivotImbResultSet(this) ;
				else resultset = new SubQueryPivotNoImbResultSet(this) ;
			}
			else resultset = new SubQueryNoPivotResultSet(this) ;
			*/
			resultset = new SubQueryNoPivotResultSet(this, pivot, imbricated);
		} catch (XMLDBCException e) {
			//throw new MediatorException("Can't construct " + getClass() + ": " + e.getMessage(), e) ;
			throw new MediatorException("Can't construct " + getClass() + ": " + e.getMessage());
		}
	}

	// **********************************************************************
	// * OPTIMIZE
	// **********************************************************************

	// **********************************************************************
	// * DEBUG
	// **********************************************************************
	public String toCompleteString(int indent) {
		StringBuffer buf = new StringBuffer();
		buf.append(Utils.makeIndent(indent) + "<" + getClass().getName() + " hasPivot=\"" + pivot + "\" isImbricated=\"" + this.imbricated + " size=" + size + " id size=" + idsize + " isLet=\"" + islet + "\" valueDepends=\"" + valueDepends + "\" whereDepends=\"" + whereDepends + "\">\n");
		buf.append(Utils.makeIndent(indent + 1) + "<mergeRestrictions>\n") ;
		buf.append(Utils.makeIndent(indent + 2) + mergeRestrictions + "\n") ;
		buf.append(Utils.makeIndent(indent + 1) + "</mergeRestrictions>\n") ;
		buf.append(Utils.makeIndent(indent + 1) + "<outsideRestrictions>\n") ;
		buf.append(Utils.makeIndent(indent + 2) + outsideRestrictions + "\n") ;
		buf.append(Utils.makeIndent(indent + 1) + "</outsideRestrictions>\n") ;
		/*
		buf.append(Utils.makeIndent(indent + 1) + "<Expression>\n") ;
		buf.append(Utils.makeIndent(indent + 2) + expression + "\n") ;
		buf.append(Utils.makeIndent(indent + 1) + "</Expression>\n") ;
		if (variables != null) {
			buf.append(Utils.makeIndent(indent + 1) + "<Variables>\n") ;
			for (int i = 0 ; i < variables.size() ; i ++) {
				buf.append(Utils.makeIndent(indent + 2) + "<Variable>" + variables.get(i) + "</Variable>\n") ;
			}
			buf.append(Utils.makeIndent(indent + 1) + "</Variables>\n") ;
		}
		 */
		if (paths != null) {
			buf.append(Utils.makeIndent(indent + 1) + "<Paths>\n");
			for (int i = 0; i < paths.size(); i++) {
				buf.append(Utils.makeIndent(indent + 2) + "<Path>" + paths.get(i) + "</Path>\n");
			}
			buf.append(Utils.makeIndent(indent + 1) + "</Paths>\n");
		}
		/*
		if (subelementsrestrict != null && !subelementsrestrict.isEmpty()) {
			buf.append(Utils.makeIndent(indent + 1) + "<Restrictions>\n") ;
			for (Iterator it = subelementsrestrict.keySet().iterator();it.hasNext();) {
				Integer index = (Integer)it.next();
				buf.append(Utils.makeIndent(indent + 2) + "<Index>" + index + "</Index>\n") ;
				ArrayList tmplist = (ArrayList)subelementsrestrict.get(index);
				if (tmplist == null) continue;
				for (int i = 0 ; i < tmplist.size() ; i ++)
					buf.append(Utils.makeIndent(indent + 2) + "<Restriction>" + tmplist.get(i) + "</Restriction>\n") ;
			}
			buf.append(Utils.makeIndent(indent + 1) + "</Restrictions>\n") ;
		}
		 */
		for (int i = 0; i < algebralist.size(); i++) {
			buf.append(Utils.makeIndent(indent + 1) + "<Element" + (i + 1) + ">\n");
			buf.append(((Operator) algebralist.get(i)).toCompleteString(indent + 2));
			buf.append(Utils.makeIndent(indent + 1) + "</Element" + (i + 1) + ">\n");
		}
		buf.append(Utils.makeIndent(indent) + "</" + getClass().getName() + ">\n");
		return buf.toString();
	}
}

abstract class SubQueryResultSet extends ResultSet {
	// **********************************************************************
	// * VERSIONING
	// **********************************************************************
	private static final String RCSRevision = "$Revision: 1.2 $";
	private static final String RCSName = "$Name:  $";
	// ***********************************************************************
	// * VARIABLES
	// ***********************************************************************
	protected int[] locations = null;
	protected int[] idlocations = null;
	protected ArrayList algResultSetList = null;
	protected boolean pivot;
	protected boolean imbricated;
	// ***********************************************************************
	// * INITIALIZATION
	// ***********************************************************************
	/**
	 *
	 */
	//public SubQueryResultSet(AlgSubQuery algebra, HashMap subelementsrestrict, boolean pivot) throws XMLDBCException {
	public SubQueryResultSet(OpSubQuery algebra, boolean pivot, boolean imbricated) throws XMLDBCException {
		super(algebra);
		algResultSetList = new ArrayList(algebra.getSubElementsSize());
		for (int i = 0; i < algebra.getSubElementsSize(); i++) {
			Operator algebrai = algebra.getSubElement(i);
			if (!algebrai.getDepends() || i==0 || algebra.getOuterType() == HintTree.MERGE_TYPE)
				algResultSetList.add(algebrai.getResultSet());
		}
		this.pivot = pivot;
		this.imbricated = imbricated;
		// initLocations(algebra);
		this.locations = algebra.getLocations();
		this.idlocations = algebra.getIdLocations();
	}

	//	public int[] getLocations() {
	//		return locations;
	//	}
	//
	//	public int[] getIdLocations() {
	//		return idlocations;
	//	}

	public ResultSet getSubElement(int pos) {
		return (ResultSet) algResultSetList.get(pos);
	}
	public int getSubElementsSize() {
		return algResultSetList.size();
	}
	public List getSubElements() {
		return algResultSetList;
	}
}

class SubQueryNoPivotResultSet extends SubQueryResultSet {
	// **********************************************************************
	// * VERSIONING
	// **********************************************************************
	private static final String RCSRevision = "$Revision: 1.2 $";
	private static final String RCSName = "$Name:  $";

	private int index = 0;
	//private boolean hasinformation = false;
	private boolean allLets = true;

	private ResultSet pivotResultSet = null;
	private ArrayList allTuples = null;
	private int[] indexes = null;
	private Tuple tuplepivot = null;
	private boolean is_used = false;
	private ArrayList currentTuples = null;
	private int size = -1;
	private boolean endindex = false;
	private int outertype = HintTree.NESTED_TYPE;
	private SortedKeyMap tm0 = null;
	private SortedKeyMap tm1 = null;
	private HashMap pivotindex = null;
	private int indexindex = 0;
	
	public SubQueryNoPivotResultSet(OpSubQuery algebra) throws XMLDBCException {
		super(algebra, false, false);
		init();
	}
	public SubQueryNoPivotResultSet(OpSubQuery algebra, boolean pivot, boolean imbricated) throws XMLDBCException {
		super(algebra, pivot, imbricated);
		init();
	}

	protected void init() throws XMLDBCException {
		size = ((OpSubQuery) algebra).getSubElementsSize();
		if (pivot) {
			pivotResultSet = this.getSubElement(0);
			if (!pivotResultSet.hasNext()) {
				pivotResultSet.close();
				setFinishedWhenEmpty();
				for (int i = 1; i < size; i++)
				if (!((OpSubQuery) algebra).getSubElement(i).getDepends())
					this.getSubElement(i).close();
				//this.hasinformation = true;
				return;
			}
			outertype = ((OpSubQuery) algebra).getOuterType();
			if (outertype == HintTree.NESTED_TYPE) {
				allTuples = new ArrayList(size - 1);
				indexes = new int[size-1];
				for (int i = 1; i < size; i++) {
					indexes[i-1] = -1;
					if (allLets && (!((OpSubQuery) algebra).getSubElement(i).isLet() || ((OpSubQuery) algebra).getSubElement(i).getWhereDepends()))
						allLets = false;
				}
			}
			else {
				allTuples = new ArrayList(size);
				indexes = new int[size];
				indexes[0] = -1;
				for (int i = 1; i < size; i++) {
					indexes[i] = -1;
					if (allLets && (!((OpSubQuery) algebra).getSubElement(i).isLet() || ((OpSubQuery) algebra).getSubElement(i).getWhereDepends()))
						allLets = false;
				}
			}
		}
	}

	// ***********************************************************************
	// * EVALUATE IMPLEMENTATION
	// ***********************************************************************
	/**
	 * 	
	 * @param non_blocking if the evaluation is blocking, we must
	 *        generate all the results before passing.
	 */
	protected void evaluate(boolean non_blocking) throws XMLDBCException {
		
		//non_blocking = false;
		
// CODE only for repository or wrappers not supporting TUPLE and PART functions (EVERYBODY!!!)
//		VERSION OK avant le 17/03/2003		
//				if (pivot) {
//					AlgResultSet pivotResultSet = this.getSubElement(0);
//					ArrayList allTuples = new ArrayList(((AlgSubQuery) algebra).getSubElementsSize() - 1);
//					for (int i = 1; i < ((AlgSubQuery) algebra).getSubElementsSize(); i++) {
//						Algebra algebrai = ((AlgSubQuery) algebra).getSubElement(i);
//						if (allLets && !algebrai.isLet())
//							allLets = false;
//						ArrayList al = new ArrayList();
//						if (!algebrai.getDepends()) {
//							AlgResultSet subalgi = this.getSubElement(i);
//							while (subalgi.hasNext()) {
//								Tuple tuplei = subalgi.next();
//								al.add(tuplei);
//							}
//						}
//						allTuples.add(al);
//					}
//		
//					//boolean removeValues = false;
//					Tuple tuplepivot = null;
//					while (pivotResultSet.hasNext()) {
//						tuplepivot = this.getSubElement(0).next();
//						//org.xquark.xml.misc.Debug.print("tuplepivot = " + tuplepivot.toCompleteString());
//		
//						// calculate all DepNodes if any
//		
//						//ArrayList tmplist = (ArrayList) ((AlgSubQuery) algebra).getSubElements().clone();
//						for (int i = 1; i < ((AlgSubQuery) algebra).getSubElementsSize(); i++) {
//							Algebra algebrai = ((AlgSubQuery) algebra).getSubElement(i);
//							if (algebrai.getDepends()) {
//								ArrayList al = (ArrayList) allTuples.get(i - 1);
//								al.clear();
//								//this.getAlgebra().getPlan().addReplaceValues(tuplepivot);
//								//if (!removeValues) removeValues = true;
//								this.getAlgebra().getPlan().addCurrentTuple(tuplepivot);
//		
//								try {
//									//org.xquark.xml.misc.Debug.print("algebrai = \n" + algebrai.toCompleteString());
//									algebrai.executeQuery();
//									AlgResultSet subresultset = algebrai.getResultSet();
//									while (subresultset.hasNext()) {
//										Tuple tuple2 = subresultset.next();
//										//org.xquark.xml.misc.Debug.print("tuple2 = " + tuple2.toCompleteString());
//										al.add(tuple2);
//									}
//								} catch (Exception e) {
//									//e.printStackTrace();
//									throw new XMLDBCException(e.getMessage());
//								}
//								this.getAlgebra().getPlan().deleteCurrentTuple(tuplepivot);
//							}
//						}
//		
//						boolean is_used = false;
//						for (int i = 1; i < ((AlgSubQuery) algebra).getSubElementsSize(); i++) {
//							ArrayList al = (ArrayList) allTuples.get(i - 1);
//		
//							for (int j = 0; j < al.size(); j++) {
//								Tuple tuplej = (Tuple) al.get(j);
//								Tuple newtuple = new Tuple(algebra);
//								//org.xquark.xml.misc.Debug.print("newtuple = " + newtuple.toCompleteString());
//								//org.xquark.xml.misc.Debug.print("tuplepivot = " + tuplepivot.toCompleteString());
//								//org.xquark.xml.misc.Debug.print("tuplej = " + tuplej.toCompleteString());
//								newtuple.appendTuple(tuplepivot);
//								newtuple.appendTupleAtIndex(locations[i - 1], tuplej);
//								newtuple.fillIdentifiers(0, tuplepivot);
//								newtuple.fillIdentifiers(idlocations[i - 1], tuplej);
//								if (newtuple != null) {
//									buftuples.add(newtuple);
//									is_used = true;
//								}
//							}
//						}
//		
//						if (is_used == false && (imbricated || allLets)) {
//							Tuple newtuple = new Tuple(algebra);
//							newtuple.appendTuple(tuplepivot);
//							newtuple.fillIdentifiers(0, tuplepivot);
//							buftuples.add(newtuple);
//						}
//					}
//					//if (removeValues) this.algebra.getPlan().deleteReplaceValues(tuplepivot);
//		
//					for (int i = 0; i < ((AlgSubQuery) algebra).getSubElementsSize(); i++) {
//						Algebra algebrai = ((AlgSubQuery) algebra).getSubElement(i);
//						if (i == 0 || !algebrai.getDepends())
//							this.getSubElement(i).close();
//					}
//					//org.xquark.xml.misc.Debug.print("buftuples = " + buftuples.toString());
//					setFinishedWhenEmpty();
		
		
//		VERSION OK le 17/03/2003		
//		if (pivot) {
//			if (!hasinformation) {
//				allTuples = new ArrayList(size - 1);
//				for (int i = 1; i < size; i++) {
//					Algebra algebrai = ((AlgSubQuery) algebra).getSubElement(i);
//					ArrayList al = new ArrayList();
//					if (!algebrai.getDepends()) {
//						AlgResultSet subalgi = this.getSubElement(i);
//						while (subalgi.hasNext()) {
//							Tuple tuplei = subalgi.next();
//							al.add(tuplei);
//						}
//						subalgi.close();
//					}
//					allTuples.add(al);
//				}
//				hasinformation = true;
//			}
//
//			//Tuple tuplepivot = null;
//			boolean added = false;
//			while (!added) {
//				if (tuplepivot == null) {
//					if (pivotResultSet.hasNext()) {
//						tuplepivot = pivotResultSet.next();
//						is_used = false;
//					} else {
//						pivotResultSet.close();
//						setFinishedWhenEmpty();
//						return;
//					}
//				}
//				//org.xquark.xml.misc.Debug.print("tuplepivot = " + tuplepivot.toCompleteString());
//				for (int i = 1; i < size; i++) {
//					Algebra algebrai = ((AlgSubQuery) algebra).getSubElement(i);
//					if (algebrai.getDepends()) {
//						ArrayList al = (ArrayList) allTuples.get(i - 1);
//						al.clear();
//						this.getAlgebra().getPlan().addCurrentTuple(tuplepivot);
//
//						try {
//							//org.xquark.xml.misc.Debug.print("algebrai = \n" + algebrai.toCompleteString());
//							algebrai.executeQuery();
//							AlgResultSet subresultset = algebrai.getResultSet();
//							while (subresultset.hasNext()) {
//								Tuple tuple2 = subresultset.next();
//								//org.xquark.xml.misc.Debug.print("tuple2 = " + tuple2.toCompleteString());
//								al.add(tuple2);
//							}
//							subresultset.close();
//						} catch (Exception e) {
//							//e.printStackTrace();
//							throw new XMLDBCException(e.getMessage());
//						}
//						this.getAlgebra().getPlan().deleteCurrentTuple(tuplepivot);
//					}
//				}
//
//				for (int i = 1; i < size; i++) {
//					ArrayList al = (ArrayList) allTuples.get(i - 1);
//
//					for (int j = 0; j < al.size(); j++) {
//						Tuple tuplej = (Tuple) al.get(j);
//						Tuple newtuple = new Tuple(algebra);
//						//org.xquark.xml.misc.Debug.print("newtuple = " + newtuple.toCompleteString());
//						//org.xquark.xml.misc.Debug.print("tuplepivot = " + tuplepivot.toCompleteString());
//						//org.xquark.xml.misc.Debug.print("tuplej = " + tuplej.toCompleteString());
//						newtuple.appendTuple(tuplepivot);
//						newtuple.appendTupleAtIndex(locations[i - 1], tuplej);
//						newtuple.fillIdentifiers(0, tuplepivot);
//						newtuple.fillIdentifiers(idlocations[i - 1], tuplej);
//						if (newtuple != null) {
//							buftuples.add(newtuple);
//							if (!added && non_blocking) added = true;
//							is_used = true;
//						}
//					}
//				}
//
//				if (is_used == false && (imbricated || allLets)) {
//					Tuple newtuple = new Tuple(algebra);
//					newtuple.appendTuple(tuplepivot);
//					newtuple.fillIdentifiers(0, tuplepivot);
//					buftuples.add(newtuple);
//					if (!added && non_blocking) added = true;
//				}
//				tuplepivot = null;
//			}
//			//org.xquark.xml.misc.Debug.print("buftuples = " + buftuples.toString());
//			//setFinishedWhenEmpty();
//		} 
		
		
		
		// DERNIERE VERSION teste		

		if (pivot) {
			if (outertype == HintTree.NESTED_TYPE) {
				boolean added = false;
				while (!added) {
					if (endindex) {
						endindex = false;
						if (is_used == false && (imbricated || allLets)) {
							Tuple newtuple = new Tuple(algebra);
							newtuple.appendTuple(tuplepivot);
							newtuple.fillIdentifiers(0, tuplepivot);
							buftuples.add(newtuple);
							if (!added && non_blocking) added = true;
							tuplepivot = null;
							continue;
						}
						tuplepivot = null;
					}
					if (tuplepivot == null) {
						if (pivotResultSet.hasNext()) {
							tuplepivot = pivotResultSet.next();
							is_used = false;
							index = 0;
						} else {
							pivotResultSet.close();
							setFinishedWhenEmpty();
							//Debug.print("OpSubQuery : buftuples = " + buftuples.toString());
							return;
						}
					}
					Tuple tuple = null;
					//org.xquark.xml.misc.Debug.print("tuplepivot = " + tuplepivot.toCompleteString());
					Operator algebrai = ((OpSubQuery) algebra).getSubElement(index+1);
					if (algebrai.getDepends()) {
						this.getAlgebra().getPlan().addCurrentTuple(tuplepivot);
						ResultSet subresultset = algebrai.getResultSet();
						if (subresultset == null) {	
							try {
								//org.xquark.xml.misc.Debug.print("algebrai = \n" + algebrai.toCompleteString());
								algebrai.executeQuery();
							} catch (Exception e) {
								//e.printStackTrace();
								throw new XMLDBCException(e.getMessage());
							}
							subresultset = algebrai.getResultSet();
						}
						if (subresultset.hasNext()) { // more tuples
							tuple = subresultset.next();
						}
						else { // no more tuples
							this.getAlgebra().getPlan().deleteCurrentTuple(tuplepivot);
							subresultset.close();
							allTuples.add(null);
							algebrai.setResultSet(null);
							if (index == size-2) 
								endindex = true;
							else
								index++;
							continue;						
						}	
						this.getAlgebra().getPlan().deleteCurrentTuple(tuplepivot);
					}
					else { // no dependency					
						if (indexes[index] == -1) { // first time round
							ResultSet subresultset = algebrai.getResultSet();
							if (subresultset.hasNext()) { // more tuples
								tuple = subresultset.next();
								if (currentTuples == null)
									currentTuples = new ArrayList();
								currentTuples.add(tuple);
							}
							else { // no more tuples
								subresultset.close();
								allTuples.add(currentTuples);
								currentTuples = null;
								indexes[index] = 0;
								if (index == size-2) 
									endindex = true;
								else
									index++;
								continue;
							}
						}
						else { // not first time round
							if (indexes[index] == 0)
								currentTuples = (ArrayList)allTuples.get(index);
							if (indexes[index] == ((currentTuples == null)?0:currentTuples.size())) {
								indexes[index] = 0;
								if (index == size-2) 
									endindex = true;
								else
									index++;
								continue;
							}
							if (currentTuples != null)
								tuple = (Tuple)currentTuples.get(indexes[index]);
							else
								tuple = null;
							indexes[index]++;
						}
					}
					
					Tuple newtuple = new Tuple(algebra);
					//org.xquark.xml.misc.Debug.print("tuplepivot = " + tuplepivot.toCompleteString());
					//org.xquark.xml.misc.Debug.print("tuple = " + tuple.toCompleteString());
					newtuple.appendTuple(tuplepivot);
					newtuple.appendTupleAtIndex(locations[index], tuple);
					newtuple.fillIdentifiers(0, tuplepivot);
					newtuple.fillIdentifiers(idlocations[index], tuple);
					//org.xquark.xml.misc.Debug.print("newtuple = " + newtuple.toCompleteString());
					if (newtuple != null) {
						buftuples.add(newtuple);
						if (!added && non_blocking) added = true;
						is_used = true;
					}
				}
				//Debug.print("OpSubQuery : buftuples = " + buftuples.toString());
			}
			
			
			
			
			else if (outertype == HintTree.MERGE_TYPE) {
				//this is blocking!
				// get all pivot elements and index them with row id
				if (allTuples.isEmpty()) {
					pivotindex = new HashMap();					
					ArrayList tmplist = new ArrayList();
					int tmpint = 0;
					while (pivotResultSet.hasNext()) {
						Tuple tmptuple = pivotResultSet.next();
						tmplist.add(tmptuple);
						pivotindex.put(tmptuple,new Integer(tmpint++));
					}
					// do not close (it puts the buftuples to null)
					// pivotResultSet.close();
					allTuples.add(tmplist);
					indexes[0] = 0;
					index = 1;
					indexindex = 0;
				}
				// make sort merge if necessary
				boolean added = false;
				while (!added) {
					if (endindex) {
						endindex = false;
						indexes[0]++;
						if (is_used == false && (imbricated || allLets)) {
							Tuple newtuple = new Tuple(algebra);
							newtuple.appendTuple(tuplepivot);
							newtuple.fillIdentifiers(0, tuplepivot);
							buftuples.add(newtuple);
							if (!added && non_blocking) added = true;
							tuplepivot = null;
							continue;
						}
						tuplepivot = null;
					}
					if (tuplepivot == null) {
						if (indexes[0] < ((ArrayList)allTuples.get(0)).size()) {
							tuplepivot = (Tuple)((ArrayList)allTuples.get(0)).get(indexes[0]);
							is_used = false;
							index = 1;
						} else {
							setFinishedWhenEmpty();
							pivotResultSet.close();
							return;
						}
					}
					//Tuple tuple = null;
					Operator algebrai = ((OpSubQuery) algebra).getSubElement(index);
					if (algebrai.getDepends()) {
						if (indexes[index] == -1) { // merge not calculate yet
							// make sort merge
							// fill alltuples with hashMap						
							ResultSet subalgi = this.getSubElement(index);
							if (!subalgi.hasNext()) {
								subalgi.close();
								// add empty resultset
								allTuples.add(null);
								// set index for no recalculation
								indexes[index] = 0;
								continue;
							}
							while (subalgi.hasNext()) {
								Tuple tuplei = subalgi.next();
							}
							// do not close (it puts the buftuples to null)
							tm0 = this.getSubElement(0).getIndex(indexindex);
							tm0.makeIterator();
							tm1 = this.getSubElement(index).getIndex(0);
							tm1.makeIterator();
							subalgi.close();
							//org.xquark.xml.misc.Debug.print("tm0 = " + tm0);
							//org.xquark.xml.misc.Debug.print("tm1 = " + tm1);
							
							HashMap tmpmap = new HashMap();
							
							// initialize variables
							ArrayList mergeRestrictions = (ArrayList)((ArrayList)((OpSubQuery)algebra).getMergeRestrictions()).get(indexindex);
							XQueryExpression outsideRestrictions = (XQueryExpression)((ArrayList)((OpSubQuery)algebra).getOutsideRestrictions()).get(indexindex);
							// increment index of index
							indexindex++;
							int size = mergeRestrictions.size();
							ArrayList leftstrings = new ArrayList(size);
							ArrayList rightstrings = new ArrayList(size);
							for (int i = 0; i < size; i++) {
								leftstrings.add(((ListOpCompExpression) mergeRestrictions.get(i)).getExpression1().toString());
								rightstrings.add(((ListOpCompExpression) mergeRestrictions.get(i)).getExpression2().toString());
							}
							int operator = ((ListOpCompExpression) mergeRestrictions.get(size - 1)).getOperator();
							ArrayList leftbuffer = new ArrayList();
							ArrayList rightbuffer = new ArrayList();
							Tuple left = null;
							Tuple right = null;
							int leftindex = -1;
							Tuple leftfront = null;
							Tuple rightfront = null;
							Tuple nextright = null;
														
							if (operator == 0) {
								while (true) {
									if (left == null || right == null)
										if (!tm0.hasNext() || !tm1.hasNext()) {
											//org.xquark.xml.misc.Debug.print("buftuples = " + buftuples);
											break;
										}
									// find first equality
									if (left == null) {
										left = tm0.next();
										//org.xquark.xml.misc.Debug.print("left = " + left.toCompleteString());
									}
									if (right == null) {
										right = tm1.next();
										//org.xquark.xml.misc.Debug.print("right = " + right.toCompleteString());
									}
									while (left != null && right != null) {
										int num = 0;
										while (num < size) {
											//org.xquark.xml.misc.Debug.print("left = " + left.toCompleteString());
											//org.xquark.xml.misc.Debug.print("right = " + right.toCompleteString());
											int res = DOMUtils.simpleCompareNodeValue((TypedNode) left.getPath((String) leftstrings.get(num)).get(0), (TypedNode) right.getPath((String) rightstrings.get(num)).get(0));
											if (res == 0)
												num++;
											else {
												if (res < 0) {
													if (tm0.hasNext()) {
														left = tm0.next();
														//org.xquark.xml.misc.Debug.print("left = " + left.toCompleteString());
													} else
														left = null;
												} else if (res > 0) {
													if (tm1.hasNext()) {
														right = tm1.next();
														//org.xquark.xml.misc.Debug.print("right = " + right.toCompleteString());
													} else
														right = null;
												}
												break;
											}
										}
										if (num == size)
											break;
									}
									if (left == null || right == null) {
										//setFinishedWhenEmpty();
										//this.getSubElement(0).close();
										//this.getSubElement(1).close();
										//org.xquark.xml.misc.Debug.print("buftuples = " + buftuples);
										//return;
										continue;
									}
									// clean buffers
									leftbuffer.clear();
									leftbuffer.add(left);
									rightbuffer.clear();
									rightbuffer.add(right);
									// get all left elements equal to another and bufferize them
									while (true) {
										leftfront = null;
										if (tm0.hasNext()) {
											leftfront = tm0.next();
											//org.xquark.xml.misc.Debug.print("leftfront = " + leftfront.toCompleteString());
											int num = 0;
											while (num < size) {
												int res = DOMUtils.simpleCompareNodeValue((TypedNode) left.getPath((String) leftstrings.get(num)).get(0), (TypedNode) leftfront.getPath((String) leftstrings.get(num)).get(0));
												if (res == 0)
													num++;
												else
													break;
											}
											if (num == size) {
												leftbuffer.add(leftfront);
												left = leftfront;
											} else
												break;
										} else
											break;
				
									}
									// get all right elements equal to another and bufferize them
									while (true) {
										rightfront = null;
										if (tm1.hasNext()) {
											rightfront = tm1.next();
											//org.xquark.xml.misc.Debug.print("rightfront = " + rightfront.toCompleteString());
											int num = 0;
											while (num < size) {
												int res = DOMUtils.simpleCompareNodeValue((TypedNode) right.getPath((String) rightstrings.get(num)).get(0), (TypedNode) rightfront.getPath((String) rightstrings.get(num)).get(0));
												if (res == 0)
													num++;
												else
													break;
											}
											if (num == size) {
												rightbuffer.add(rightfront);
												right = rightfront;
											} else {
												break;
											}
										} else
											break;
									}
									// add all necessary tuples to buffertuple result
									//org.xquark.xml.misc.Debug.print("pivotindex = " + pivotindex);
									for (int i = 0; i < leftbuffer.size(); i++) {
										for (int j = 0; j < rightbuffer.size(); j++) {
											//org.xquark.xml.misc.Debug.print("Adding tuple ... ");
											//org.xquark.xml.misc.Debug.print("first  = " + leftbuffer.get(i));
											//org.xquark.xml.misc.Debug.print("second = " + rightbuffer.get(j));
											// TEST outside restrictions
											if (outsideRestrictions != null) {
												//org.xquark.xml.misc.Debug.print("outsideRestrictions = " + outsideRestrictions);
												ArrayList tmpList = new ArrayList();
												tmpList.addAll(algebra.getPlan().getCurrentTuples());
												tmpList.add(leftbuffer.get(i));
												tmpList.add(rightbuffer.get(j));
												//org.xquark.xml.misc.Debug.print("tmplist = " + tmpList);
												try {
						                        	EvaluationVisitor visitor = new EvaluationVisitor(this.algebra.getPlan().getSchemaManager(),tmpList) ;
						                        	outsideRestrictions.accept(visitor) ;
						                        	if (!visitor.getVerdict())
						                        		continue;
												} catch (Exception e) {
													 throw new XMLDBCException(e.getMessage());
												}
											}
											Integer tmpint = (Integer)pivotindex.get(leftbuffer.get(i));
											ArrayList tmplist = (ArrayList) tmpmap.get(tmpint);
											if (tmplist == null) {
												tmplist = new ArrayList();
												tmpmap.put(tmpint,tmplist);
											}
											tmplist.add(rightbuffer.get(j));
											//this.buftuples.add(((Tuple) leftbuffer.get(i)).merge((Tuple) rightbuffer.get(j), this.algebra));
										}
									}
									left = leftfront;
									right = rightfront;
								}
							} else { // with inequality
				
								while (true) {
									if (!tm0.hasNext()) {
										//org.xquark.xml.misc.Debug.print("buftuples = " + buftuples);
										break;
									} else if (right == null && nextright == null && !tm1.hasNext() && rightbuffer.isEmpty()) {
										//org.xquark.xml.misc.Debug.print("buftuples = " + buftuples);
										break;
									}
									int num;
									// find first equality
									if (left == null) {
										left = tm0.next();
										//org.xquark.xml.misc.Debug.print("left = " + left.toCompleteString());
									} else {
										leftfront = tm0.next();
										//org.xquark.xml.misc.Debug.print("leftfront = " + leftfront.toCompleteString());
										num = 0;
										while (num < size - 1) {
											int res = DOMUtils.simpleCompareNodeValue((TypedNode) left.getPath((String) leftstrings.get(num)).get(0), (TypedNode) leftfront.getPath((String) leftstrings.get(num)).get(0));
											if (res == 0)
												num++;
											else
												break;
										}
										if (num != size - 1) {
											rightbuffer.clear();
										}
										left = leftfront;
									}
									if (right == null) {
										if (nextright != null) {
											right = nextright;
											nextright = null;
										} else if (tm1.hasNext()) {
											right = tm1.next();
											//org.xquark.xml.misc.Debug.print("right = " + right.toCompleteString());
										}
									}
									// find first match
									boolean add = false;
									while (right != null) {
										num = 0;
										//org.xquark.xml.misc.Debug.print("left = " + left.toCompleteString());
										//org.xquark.xml.misc.Debug.print("right = " + right.toCompleteString());
										while (num < size) {
											int res = DOMUtils.simpleCompareNodeValue((TypedNode) left.getPath((String) leftstrings.get(num)).get(0), (TypedNode) right.getPath((String) rightstrings.get(num)).get(0));
											if (num != size - 1) {
												if (res == 0)
													num++;
												else {
													if (res < 0) {
														num = size + 1;
														break;
													} else if (res > 0) {
														if (tm1.hasNext()) {
															right = tm1.next();
															//org.xquark.xml.misc.Debug.print("right = " + right.toCompleteString());
														} else
															right = null;
													}
													break;
												}
											} else {
												if ((res == 0 && (operator == Constants.GEQ_COMPOP || operator == Constants.LEQ_COMPOP)) || (res < 0 && (operator == Constants.LT_COMPOP || operator == Constants.LEQ_COMPOP)) || (res > 0 && (operator == Constants.GEQ_COMPOP || operator == Constants.GT_COMPOP))) {
													num++;
												} else {
													num = size + 1;
													break;
												}
											}
										}
										if (num == size) {
											add = true;
											rightbuffer.add(right);
											break;
										}
										if (num > size)
											break;
									}
				
									if (rightbuffer.isEmpty()) {
										continue;
									}
									// get all right elements equal to another and bufferize them
									if (add) {
										rightfront = nextright;
										while (true) {
											if (rightfront == null)
												if (tm1.hasNext()) {
													rightfront = tm1.next();
													//org.xquark.xml.misc.Debug.print("rightfront = " + rightfront.toCompleteString());
												}
											if (rightfront != null) {
												//org.xquark.xml.misc.Debug.print("rightfront = " + rightfront.toCompleteString());
												num = 0;
												while (num < size) {
													if (num != size - 1) {
														int res = DOMUtils.simpleCompareNodeValue((TypedNode) right.getPath((String) rightstrings.get(num)).get(0), (TypedNode) rightfront.getPath((String) rightstrings.get(num)).get(0));
														if (res == 0)
															num++;
														else {
															nextright = rightfront;
															rightfront = null;
															break;
														}
													} else {
														int res = DOMUtils.simpleCompareNodeValue((TypedNode) left.getPath((String) leftstrings.get(num)).get(0), (TypedNode) rightfront.getPath((String) rightstrings.get(num)).get(0));
														if ((res == 0 && (operator == Constants.GEQ_COMPOP || operator == Constants.LEQ_COMPOP)) || (res < 0 && (operator == Constants.LT_COMPOP || operator == Constants.LEQ_COMPOP)) || (res > 0 && (operator == Constants.GEQ_COMPOP || operator == Constants.GT_COMPOP)))
															num++;
														else {
															nextright = rightfront;
															rightfront = null;
															break;
														}
													}
												}
												if (num == size) {
													rightbuffer.add(rightfront);
													right = rightfront;
													rightfront = null;
												} else {
													break;
												}
											} else
												break;
										}
										right = rightfront;
									}
									// add all necessary tuples to buffertuple result
									for (int j = 0; j < rightbuffer.size(); j++) {
										//org.xquark.xml.misc.Debug.print("Adding tuple ... ");
										//org.xquark.xml.misc.Debug.print("first  = " + left);
										//org.xquark.xml.misc.Debug.print("second = " + rightbuffer.get(j));
										// TEST outside restrictions
										if (outsideRestrictions != null) {
											//org.xquark.xml.misc.Debug.print("outsideRestrictions = " + outsideRestrictions);
											ArrayList tmpList = new ArrayList();
											tmpList.addAll(algebra.getPlan().getCurrentTuples());
											tmpList.add(left);
											tmpList.add(rightbuffer.get(j));
											//org.xquark.xml.misc.Debug.print("tmplist = " + tmpList);
											try {
					                        	EvaluationVisitor visitor = new EvaluationVisitor(this.algebra.getPlan().getSchemaManager(),tmpList) ;
					                        	outsideRestrictions.accept(visitor) ;
					                        	if (!visitor.getVerdict())
					                        		continue;
											} catch (Exception e) {
												 throw new XMLDBCException(e.getMessage());
											}
										}
										Integer tmpint = (Integer)pivotindex.get(left);
										ArrayList tmplist = (ArrayList) tmpmap.get(tmpint);
										if (tmplist == null) {
											tmplist = new ArrayList();
											tmpmap.put(tmpint,tmplist);
										}
										tmplist.add(rightbuffer.get(j));
										//this.buftuples.add(left.merge((Tuple) rightbuffer.get(j), this.algebra));
									}
								}
							}
//							if (left == null || right == null)
//								if (!tm0.hasNext() || (!tm1.hasNext()) && operator == 0) {
//									setFinishedWhenEmpty();
//									this.getSubElement(0).close();
//									this.getSubElement(1).close();
//									org.xquark.xml.misc.Debug.print("buftuples = " + buftuples);
//									return;
//								}
							//org.xquark.xml.misc.Debug.print("buftuples = " + buftuples);
							// add hashmap to alltuples
							//org.xquark.xml.misc.Debug.print("tmpmap = " + tmpmap);
							if (tmpmap.isEmpty())
								allTuples.add(null);
							else
								allTuples.add(tmpmap);
							// set index for no recalculation
							indexes[index] = 0;
						}
						if (indexes[index] == 0) { // merge already calculated
							HashMap map = (HashMap)allTuples.get(index);
							if (map != null) {
								ArrayList tmplist = (ArrayList)map.get(new Integer(indexes[0]));
								if (tmplist != null) {
									for (int i=0;i<tmplist.size();i++) {
										Tuple newtuple = new Tuple(algebra);
										newtuple.appendTuple(tuplepivot);
										newtuple.appendTupleAtIndex(locations[index-1], (Tuple)tmplist.get(i));
										newtuple.fillIdentifiers(0, tuplepivot);
										newtuple.fillIdentifiers(idlocations[index-1], (Tuple)tmplist.get(i));
										if (newtuple != null) {
											buftuples.add(newtuple);
											if (!added && non_blocking) added = true;
											is_used = true;
										}
									}
								}
							}
							if (index == size-1) 
								endindex = true;
							else
								index++;
						}						
						continue;
					}
					else { // no dependency
						Tuple tuple = null;					
						if (indexes[index] == -1) { // first time round
							ResultSet subresultset = algebrai.getResultSet();
							if (subresultset.hasNext()) { // more tuples
								tuple = subresultset.next();
								if (currentTuples == null)
									currentTuples = new ArrayList();
								currentTuples.add(tuple);
							}
							else { // no more tuples
								subresultset.close();
								allTuples.add(currentTuples);
								currentTuples = null;
								indexes[index] = 0;
								if (index == size-1) 
									endindex = true;
								else
									index++;
								continue;
							}
						}
						else { // not first time round
							if (indexes[index] == 0)
								currentTuples = (ArrayList)allTuples.get(index);
							if (indexes[index] == currentTuples.size()) {
								indexes[index] = 0;
								if (index == size-1) 
									endindex = true;
								else
									index++;
								continue;
							}
							tuple = (Tuple)currentTuples.get(indexes[index]);
							indexes[index]++;
						}
						Tuple newtuple = new Tuple(algebra);
						newtuple.appendTuple(tuplepivot);
						newtuple.appendTupleAtIndex(locations[index-1], tuple);
						newtuple.fillIdentifiers(0, tuplepivot);
						newtuple.fillIdentifiers(idlocations[index-1], tuple);
						if (newtuple != null) {
							buftuples.add(newtuple);
							if (!added && non_blocking) added = true;
							is_used = true;
						}
					}
				}
				//Debug.print("OpSubQuery : buftuples = " + buftuples.toString());
				
			}
		}
		
		
		
		
		
		else { // NORMAL CODE for this class!!!	
			while (index < this.getSubElementsSize() && !this.getSubElement(index).hasNext())
				index++;
			while (index < this.getSubElementsSize()) {
				if (this.getSubElement(index).hasNext()) {
					Tuple tuple = this.getSubElement(index).next();

					//org.xquark.xml.misc.Debug.print("tuple = ");
					//org.xquark.xml.misc.Debug.print(tuple.toCompleteString());

					Tuple newtuple = new Tuple(algebra);
					if (index == 0)
						newtuple.appendTupleAtIndex(0, tuple);
					else
						newtuple.appendTupleAtIndex(locations[index - 1], tuple);
					if (index == 0)
						newtuple.fillIdentifiers(0, tuple);
					else
						newtuple.fillIdentifiers(idlocations[index - 1], tuple);

					//org.xquark.xml.misc.Debug.print("newtuple = ");
					//org.xquark.xml.misc.Debug.print(newtuple.toCompleteString());

					buftuples.add(newtuple);

					if (non_blocking) {
						if ((buftuples != null) && (!buftuples.isEmpty()))
							break;
					}
				} else {
					this.getSubElement(index).close();
					index++;
				}
			}
			if (index == this.getSubElementsSize())
				setFinishedWhenEmpty();
		}
	}
}

class SubQueryPivotNoImbResultSet extends SubQueryResultSet {

	public SubQueryPivotNoImbResultSet(OpSubQuery algebra) throws XMLDBCException {
		super(algebra, true, false);
	}

	// ***********************************************************************
	// * EVALUATE IMPLEMENTATION
	// ***********************************************************************
	/**
	 *
	 * @param non_blocking if the evaluation is blocking, we must
	 *        generate all the results before passing.
	 */
	protected void evaluate(boolean non_blocking) throws XMLDBCException {
		//		try {
		//			AlgResultSet pivotResultSet = this.getSubElement(0);
		//
		//			if (pivotResultSet.hasNext()) {
		//				/* 10. Find out paths that derived from variables defined in pivot, and  used in join predicate. */
		//
		//				DepNode depnode = ((DepNode) ((AlgSubQuery) algebra).getSubElement(1));
		//				ArrayList vars = ((Algebra) ((AlgSubQuery) algebra).getSubElement(0)).getVariables();
		//
		//				DepNodeSource firstLeafDepNode = depnode.getFirstSourceLeafDepNode();
		//				//org.xquark.xml.misc.Debug.print("firstLeafDepNode = ");
		//				//org.xquark.xml.misc.Debug.print(firstLeafDepNode.toCompleteString());
		//				firstLeafDepNode.createFLWRExpression(vars, false);
		//				//org.xquark.xml.misc.Debug.print("firstLeafDepNode = ");
		//				//org.xquark.xml.misc.Debug.print(firstLeafDepNode.toCompleteString());
		//
		//				GetVariablePathsVisitor vpv = new GetVariablePathsVisitor(true);
		//				vpv.setVariables(vars);
		//				vpv.setVarList(vars);
		//				((FLWRExpression) firstLeafDepNode.getExpression()).accept(vpv);
		//				List pivotJoinPathStringList = vpv.getPathStrList();
		//				//org.xquark.xml.misc.Debug.print("pivotJoinPathStringList = " + pivotJoinPathStringList.toString());
		//				/* 15. Create a XQueryExpressionSequence containing data to be passed to the other subquery
		//				 * from the pivot ResultSet, and all data in the pivot ResulSet is to be saved to a BufferTuple
		//				 * for later join.
		//				 */
		//
		//				BufferTuple pivotBufferTuple = new BufferTuple(this.getSubElement(0).getPaths());
		//				TypeVisitor typeVisitor = algebra.getPlan().getTypeVisitor();
		//				XQueryExpressionSequence sequence = createSequence(pivotResultSet, pivotJoinPathStringList, pivotBufferTuple, typeVisitor);
		//
		//				//org.xquark.xml.misc.Debug.print("sequence = " + sequence.toString());
		//				/* 20. Modify DepNode */
		//				Variable varOnSequence = createVariable(org.xquark.xml.xqueryparser.util.Constants.FOR_BINDINGTYPE, sequence, typeVisitor);
		//
		//				transformDepNode(firstLeafDepNode, pivotJoinPathStringList, varOnSequence, typeVisitor);
		//				//org.xquark.xml.misc.Debug.print("firstLeafDepNode = ");
		//				//org.xquark.xml.misc.Debug.print(firstLeafDepNode.toCompleteString());
		//
		//				/* 40. Create and execute the a Algebra and get a new AlgResultSet. */
		//				Algebra subAlg = (Algebra) depnode.createAlgebra(algebra.getPlan());
		//				//org.xquark.xml.misc.Debug.print("subAlg = ");
		//				//org.xquark.xml.misc.Debug.print(subAlg.toCompleteString());
		//
		//				subAlg.executeQuery();
		//				AlgResultSet subresultset = subAlg.getResultSet();
		//
		//				/* 50. join the pivot BufferTuple and the new one. */
		//				//Tuple tupleProto = new Tuple(algebra);
		//				join(pivotBufferTuple, subresultset, buftuples);
		//			}
		//			setFinishedWhenEmpty();
		//		} catch (Exception ex) {
		//			throw new XMLDBCException(ex.getMessage(), ex);
		//		}
	}

	//	protected void transformDepNode(DepNodeSource depNode, List pathList, Variable varOnSequence, TypeVisitor typeVisitor) throws TypeException, XQueryException {
	//		/* 10. Transform FLWRExpression in the depNode*/
	//
	//		/* 10.10 Replace LocatedExpression by FunctionPART.*/
	//		Map replaceMap = createReplaceMap(varOnSequence, pathList, typeVisitor);
	//		FLWRExpression flwr = (FLWRExpression) depNode.getExpression();
	//		replace(flwr, replaceMap);
	//
	//		/* 10.20. Create a temporay Variable --varOnPart0 binded to  "part(varOnSequence, 0)"--id of tuple tuple */
	//		ArrayList args = new ArrayList();
	//		args.add(varOnSequence);
	//		args.add(new ValueInteger("0", typeVisitor));
	//		FunctionPART part0 = new FunctionPART(args, typeVisitor);
	//
	//		Variable varOnPart0 = this.createVariable(org.xquark.xml.xqueryparser.util.Constants.LET_BINDINGTYPE, part0, typeVisitor);
	//
	//		/* 10.30. Add varOnSequence and varOnPart0 to FLWRExpression */
	//		flwr.insertVariable(varOnSequence);
	//		flwr.addVariable(varOnPart0);
	//		ArrayList orderBy = flwr.getOrderBy();
	//		//Debug.assert(null == orderBy || orderBy.isEmpty(),"null == orderBy && orderBy.isEmpty()");
	//		if (orderBy == null) {
	//			orderBy = new ArrayList(1);
	//			orderBy.add(varOnPart0);
	//			flwr.setOrderBy(orderBy);
	//		} else
	//			orderBy.add(0, varOnPart0);
	//
	//		/* 20. Modify depNode. */
	//		depNode.getPaths().add(0, varOnPart0);
	//		//depNode.getVariables().add(0,varOnSequence);
	//		//depNode.getVariables().add(varOnPart0);
	//		depNode.setDepends(false);
	//	}

	/**
	 * Join the <code>leftOperand</code> and the <code>rightOperand</code> into target.
	 */
	//	protected void join(BufferTuple leftOperand, AlgResultSet rightOperand, BufferTuple target) throws XMLDBCException {
	//		Tuple newTuple = null;
	//
	//		Tuple pivotTuple = null;
	//		int cursoOnPivotResultSet = 0;
	//		Tuple subTuple = null;
	//		Number strIndex = null;
	//		int index;
	//		while (rightOperand.hasNext()) {
	//			subTuple = rightOperand.next();
	//			org.xquark.xml.misc.Debug.print("subTuple = " + subTuple.toCompleteString());
	//			subTuple.getNodesAtIndex(0);
	//			strIndex = (Number) ((TypedNode) subTuple.getNodesAtIndex(0).get(0)).getTypedValue();
	//			index = strIndex.intValue();
	//
	//			pivotTuple = (Tuple) leftOperand.get(index);
	//			newTuple = new Tuple(algebra);
	//			newTuple.appendTuple(pivotTuple);
	//			org.xquark.xml.misc.Debug.print("newTuple = " + newTuple.toCompleteString());
	//			for (int i = 1; i < subTuple.size(); i++) {
	//				newTuple.addNodesAtIndex(locations[0] + i - 1, subTuple.getNodesAtIndex(i));
	//			}
	//			org.xquark.xml.misc.Debug.print("newTuple = " + newTuple.toCompleteString());
	//			newTuple.fillIdentifiers(0, pivotTuple);
	//			newTuple.fillIdentifiers(idlocations[0], subTuple);
	//			target.add(newTuple);
	//		}
	//	}

	/** Replace <code>LocatedExpression</code> in <code>expr</code> according to <code>replaceMap</code>.
	 */
	//	protected void replace(XQueryExpression expr, Map replaceMap) throws XQueryException {
	//		XQueryReplaceVisitor replaceVisitor = new XQueryReplaceVisitor(replaceMap) {
	//			public void visit(LocatedExpression arg) throws XQueryException {
	//				String expr = arg.toString();
	//				XQueryExpression replacer = (XQueryExpression) _replaceMap.get(expr);
	//				if (null != replacer) {
	//					XQueryExpression father = (XQueryExpression) getScanPath().get(getScanPath().size() - 1);
	//					boolean done = replaceChild(father, arg, replacer);
	//					if (done) {} else {
	//						String message = "Failed in replacing " + expr.toString() + " with " + replacer.toString();
	//						throw new XQueryException(message);
	//					}
	//				}
	//			}
	//		};
	//		expr.accept(replaceVisitor);
	//	}

	/**
	 *  Create a map between a path and a FunctionPART(var, index).
	 *  @param var: the Variable that will be the first argument of FunctionPART.
	 *  @param pathList: a list of path(in form of String);
	 */
	//	protected Map createReplaceMap(Variable var, List pathList, TypeVisitor typeVisitor) throws XQueryException, TypeException {
	//		Map retVal = new HashMap();
	//
	//		String path = null;
	//		Integer index = null;
	//		ValueInteger intValue = null;
	//		FunctionPART replacer = null;
	//		ArrayList args = null;
	//		for (int i = 0; i < pathList.size(); i++) {
	//			path = (String) pathList.get(i);
	//
	//			intValue = new ValueInteger(Integer.toString(i + 1), typeVisitor);
	//
	//			args = new ArrayList();
	//			args.add(var);
	//			args.add(intValue);
	//			replacer = new FunctionPART(args, typeVisitor);
	//
	//			retVal.put(path, replacer);
	//		}
	//
	//		return retVal;
	//	}

	/** Save all the <code>resultSet</code>(<code>Tuple</code>)in the
	 * <code>targetBufferTuple</code>(<code>BufferTuple</code>), and create
	 * a <code>XQueryExpressionSequence</code> represent all data of of all
	 * attributes corresponding to <code>pathList</code>.
	 *
	 */
	//	protected XQueryExpressionSequence createSequence(AlgResultSet resultSet, List pathList, BufferTuple targetBufferTuple, TypeVisitor typeVisitor) throws XMLDBCException, TypeException, XQueryException {
	//		XQueryExpressionSequence retVal = null;
	//
	//		int id = 0;
	//		Tuple tuple = null;
	//		FunctionTUPLE funTuple = null;
	//		ArrayList funTupleList = new ArrayList();
	//
	//		while (resultSet.hasNext()) {
	//			tuple = resultSet.next();
	//			targetBufferTuple.add(tuple);
	//
	//			funTuple = createTupleFunction(id++, tuple, typeVisitor, pathList);
	//			funTupleList.add(funTuple);
	//		}
	//		retVal = new XQueryExpressionSequence(funTupleList, typeVisitor);
	//		retVal.setParenthesis(true);
	//
	//		return retVal;
	//	}
	//	protected XQueryExpressionSequence createSequence(List pathList, BufferTuple contentBufferTuple, TypeVisitor typeVisitor) throws XMLDBCException, TypeException, XQueryException {
	//		XQueryExpressionSequence retVal = null;
	//
	//		int id = 0;
	//		Tuple tuple = null;
	//		FunctionTUPLE funTuple = null;
	//		ArrayList funTupleList = new ArrayList();
	//
	//		for (int i = 0; i < contentBufferTuple.size(); i++) {
	//			tuple = (Tuple) contentBufferTuple.get(i);
	//			funTuple = createTupleFunction(id++, tuple, typeVisitor, pathList);
	//			funTupleList.add(funTuple);
	//		}
	//		retVal = new XQueryExpressionSequence(funTupleList, typeVisitor);
	//		retVal.setParenthesis(true);
	//
	//		return retVal;
	//	}
	//
	//	protected FunctionTUPLE createTupleFunction(int id, Tuple tuple, TypeVisitor typeVisitor, List pathList) throws XQueryException, TypeException {
	//		FunctionTUPLE retVal = null;
	//
	//		/* translate nodes(org.w3c.dom.Node) in tuple into values(org.xquark.xml.xqueryparser.Value)*/
	//		ArrayList attrList = new ArrayList();
	//
	//		ValueInteger idValue = new ValueInteger(Integer.toString(id), typeVisitor);
	//		attrList.add(idValue);
	//
	//		List nodesAtI = null;
	//		TypedNode node = null;
	//		Object stringValue = null;
	//		XQueryExpression value = null;
	//		int index;
	//
	//		for (int i = 0; i < pathList.size(); i++) {
	//			index = tuple.getParent().getPathIndex((String) pathList.get(i));
	//			nodesAtI = (List) tuple.getNodesAtIndex(index);
	//			//Debug.assert(1 == nodesAtI.size(), "1 == nodesAtI.size()");
	//			node = (TypedNode) nodesAtI.get(0);
	//			stringValue = node.getTypedValue();
	//			if (null == stringValue)
	//				stringValue = DOMUtils.getTextLeaf(node);
	//			//Debug.assert(null != stringValue, "null != stringValue");
	//			value = DOMUtils.getXQueryExpression(stringValue);
	//			//Debug.assert(null != value, "null != value");
	//			attrList.add(value);
	//		}
	//
	//		/* constract a FunctionTUPLE.*/
	//		retVal = new FunctionTUPLE(attrList, typeVisitor);
	//		return retVal;
	//	}

	//	private final String VAR_NAME = "tmp_var_Seq";
	//	private int varIndex = 0;
	//	String getTempVarName() {
	//		return VAR_NAME + (++varIndex);
	//	}
	//
	//	protected Variable createVariable(int bindingType, XQueryExpression expr, TypeVisitor typeVisitor) throws XQueryException {
	//		if (typeVisitor != null && expr != null)
	//			expr.accept(typeVisitor);
	//		Variable var = new Variable(new QName(null, null, getTempVarName(), typeVisitor), typeVisitor);
	//		var.setBindingType(bindingType);
	//		var.setExpression(expr);
	//		if (typeVisitor != null && expr != null)
	//			var.accept(typeVisitor);
	//		return var;
	//	}

}

class SubQueryPivotImbResultSet extends SubQueryPivotNoImbResultSet {

	public SubQueryPivotImbResultSet(OpSubQuery algebra) throws XMLDBCException {
		super(algebra);
		this.imbricated = false;
	}

	protected void evaluate(boolean non_blocking) throws XMLDBCException {
		//		try {
		//			AlgResultSet pivotResultSet = this.getSubElement(0);
		//			if (pivotResultSet.hasNext()) {
		//
		//				/* 10. Find out paths that derived from variables defined in pivot, and  used in join predicate. */
		//				//DepNode depnode = ((DepNode)((AlgSubQuery)algebra).getSubElement(1));
		//				ArrayList vars = ((Algebra) ((AlgSubQuery) algebra).getSubElement(0)).getVariables();
		//
		//				BufferTuple pivotBufferTuple = new BufferTuple(this.getSubElement(0).getPaths());
		//				while (pivotResultSet.hasNext())
		//					pivotBufferTuple.add(pivotResultSet.next());
		//
		//				TypeVisitor typeVisitor = algebra.getPlan().getTypeVisitor();
		//				//XQueryExpressionSequence sequence = createSequence(pivotResultSet,pivotJoinPathStringList, pivotBufferTuple, typeVisitor);
		//				//Variable varOnSequence = createVariable(org.xquark.xml.xqueryparser.util.Constants.FOR_BINDINGTYPE, sequence, typeVisitor);
		//				XQueryExpressionSequence sequence = null;
		//				Variable varOnSequence = null;
		//
		//				/* Evaluate DepNodes and add their resultSet in subElement list.*/
		//				DepNodeSource firstLeafDepNode = null;
		//				Algebra subAlgebra = null;
		//				AlgResultSet subResultSet = null;
		//
		//				for (int i = 1; i < ((AlgSubQuery) algebra).getSubElementsSize(); i++) {
		//					Object subElement = ((AlgSubQuery) algebra).getSubElement(i);
		//					if (subElement instanceof DepNode) {
		//
		//						org.xquark.xml.misc.Debug.print("subElement = ");
		//						org.xquark.xml.misc.Debug.print(((DepNode) subElement).toCompleteString());
		//						firstLeafDepNode = ((DepNode) subElement).getFirstSourceLeafDepNode();
		//						org.xquark.xml.misc.Debug.print("firstLeafDepNode = ");
		//						org.xquark.xml.misc.Debug.print(firstLeafDepNode.toCompleteString());
		//						firstLeafDepNode.createFLWRExpression(vars, false);
		//						org.xquark.xml.misc.Debug.print("firstLeafDepNode = ");
		//						org.xquark.xml.misc.Debug.print(firstLeafDepNode.toCompleteString());
		//
		//						GetVariablePathsVisitor vpv = new GetVariablePathsVisitor(true);
		//						vpv.setVariables(vars);
		//						vpv.setVarList(vars);
		//						//((FLWRExpression)firstLeafDepNode.getExpression()).accept(vpv);
		//						FLWRExpression tmpflwr = (FLWRExpression) ((DepNode) subElement).getExpression();
		//						tmpflwr.setReturnClause(new XQueryVoid(null));
		//						tmpflwr.accept(vpv);
		//						List pivotJoinPathStringList = vpv.getPathStrList();
		//						org.xquark.xml.misc.Debug.print("pivotJoinPathStringList = " + pivotJoinPathStringList.toString());
		//
		//						sequence = createSequence(pivotJoinPathStringList, pivotBufferTuple, typeVisitor);
		//						org.xquark.xml.misc.Debug.print("sequence = " + sequence.toString());
		//						varOnSequence = createVariable(org.xquark.xml.xqueryparser.util.Constants.FOR_BINDINGTYPE, sequence, typeVisitor);
		//
		//						transformDepNode(firstLeafDepNode, pivotJoinPathStringList, varOnSequence, typeVisitor);
		//						org.xquark.xml.misc.Debug.print("firstLeafDepNode = ");
		//						org.xquark.xml.misc.Debug.print(firstLeafDepNode.toCompleteString());
		//
		//						subAlgebra = (Algebra) ((DepNode) subElement).createAlgebra(algebra.getPlan());
		//					} else
		//						subAlgebra = (Algebra) subElement;
		//					org.xquark.xml.misc.Debug.print("subAlgebra = ");
		//					org.xquark.xml.misc.Debug.print(subAlgebra.toCompleteString());
		//					subAlgebra.executeQuery();
		//					subResultSet = subAlgebra.getResultSet();
		//					this.getSubElements().add(i, subResultSet);
		//				}
		//
		//				/* 50. join the pivot BufferTuple and the new one. */
		//				List operandList = new ArrayList(this.getSubElements());
		//				operandList.set(0, pivotBufferTuple);
		//				outerJoin(operandList, ((AlgSubQuery) algebra).getSubElements(), buftuples);
		//			}
		//			setFinishedWhenEmpty();
		//		} catch (Exception ex) {
		//			throw new XMLDBCException(ex.getMessage(), ex);
		//		}
	}

	//	protected void outerJoin(List operandList, List operandTypeList, BufferTuple target) throws XMLDBCException {
	//
	//		List allTuples = new ArrayList();
	//		int[] cursors = new int[operandList.size()];
	//
	//		allTuples.add(null);
	//		for (int i = 1; i < operandList.size(); i++) {
	//			allTuples.add(bufferResultset((AlgResultSet) operandList.get(i), 100));
	//			cursors[i] = 0;
	//		}
	//
	//		int algebraSize = algebra.getSize();
	//		int algebraIdSize = algebra.getIdSize();
	//
	//		BufferTuple pivot = (BufferTuple) operandList.get(0);
	//		boolean hasChild = false;
	//		boolean isIndependentSubQuery = false;
	//		Object subElementOfAlgebra = null;
	//		Tuple newTuple = null;
	//		Tuple pivotTuple = null;
	//
	//		Iterator iter = pivot.iterator();
	//		while (iter.hasNext()) {
	//			pivotTuple = (Tuple) iter.next();
	//			hasChild = false;
	//			for (int j = 1; j < operandTypeList.size(); j++) {
	//				subElementOfAlgebra = operandTypeList.get(j);
	//				isIndependentSubQuery = (subElementOfAlgebra instanceof Algebra);
	//				if (isIndependentSubQuery) {
	//					hasChild |= appendProduct(pivotTuple, (List) allTuples.get(j), target, j);
	//				} else {
	//					hasChild |= appendJoin(pivotTuple, (List) allTuples.get(j), target, j, cursors);
	//				}
	//			}
	//			if (!hasChild) {
	//				newTuple = new Tuple(algebraSize, algebraIdSize);
	//				newTuple.appendTuple(pivotTuple);
	//				newTuple.fillIdentifiers(0, pivotTuple);
	//				target.add(newTuple);
	//			}
	//			cursors[0]++;
	//		}
	//
	//	}
	//
	//	protected boolean appendProduct(Tuple pivotTuple, List list, BufferTuple target, int index) {
	//		boolean retVal = false;
	//
	//		Tuple newTuple = null;
	//		Tuple tuple = null;
	//		int algebraSize = algebra.getSize();
	//		int algebraIdSize = algebra.getIdSize();
	//		for (int i = 0; i < list.size(); i++) {
	//			newTuple = new Tuple(algebraSize, algebraIdSize);
	//			tuple = (Tuple) list.get(i);
	//			newTuple.appendTuple(pivotTuple);
	//			newTuple.appendTupleAtIndex(locations[index - 1], tuple);
	//			newTuple.fillIdentifiers(0, pivotTuple);
	//			newTuple.fillIdentifiers(idlocations[index - 1], tuple);
	//			target.add(newTuple);
	//			retVal = true;
	//		}
	//
	//		return retVal;
	//	}
	//
	//	protected boolean appendJoin(Tuple pivotTuple, List list, BufferTuple target, int index, int[] cursors) {
	//		boolean retVal = false;
	//
	//		//org.xquark.xml.misc.Debug.print("pivotTuple = ");
	//		//org.xquark.xml.misc.Debug.print(pivotTuple.toCompleteString());
	//		Tuple newTuple = null;
	//		Tuple tuple = null;
	//		int algebraSize = algebra.getSize();
	//		int algebraIdSize = algebra.getIdSize();
	//		int i;
	//		for (i = cursors[index]; i < list.size(); i++) {
	//			tuple = (Tuple) list.get(i);
	//			//org.xquark.xml.misc.Debug.print("tuple = ");
	//			//org.xquark.xml.misc.Debug.print(tuple.toCompleteString());
	//			int id = ((Number) ((TypedNode) tuple.getNodesAtIndex(0).get(0)).getTypedValue()).intValue();
	//			if (id == cursors[0]) {
	//				newTuple = new Tuple(algebraSize, algebraIdSize);
	//
	//				newTuple.appendTuple(pivotTuple);
	//
	//				//org.xquark.xml.misc.Debug.print("newTuple = ");
	//				//org.xquark.xml.misc.Debug.print(newTuple.toCompleteString());
	//
	//				for (int j = 1; j < tuple.size(); j++) {
	//					newTuple.addNodesAtIndex(locations[index - 1] + j - 1, tuple.getNodesAtIndex(j));
	//				}
	//
	//				newTuple.fillIdentifiers(0, pivotTuple);
	//				newTuple.fillIdentifiers(idlocations[index - 1], tuple);
	//				target.add(newTuple);
	//				//org.xquark.xml.misc.Debug.print("newTuple = ");
	//				//org.xquark.xml.misc.Debug.print(newTuple.toCompleteString());
	//				retVal = true;
	//			} else if (id > cursors[0]) {
	//				cursors[index] = i;
	//				break;
	//			} else {
	//				//Debug.assert(false, "Internal error.");
	//			}
	//		}
	//		if (i == list.size())
	//			cursors[index] = i;
	//		return retVal;
	//	}
	//
	//	protected List bufferResultset(AlgResultSet resultSet, int hintSize) throws XMLDBCException {
	//
	//		List retVal = new ArrayList(hintSize);
	//		while (resultSet.hasNext()) {
	//			Tuple tuplei = resultSet.next();
	//			retVal.add(tuplei);
	//		}
	//		return retVal;
	//	}
}
