/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */

package org.xquark.mediator.plan ;

import java.util.ArrayList;

import org.xquark.mediator.MediatorException;
import org.xquark.mediator.DOMUtils.DOMUtils;
import org.xquark.mediator.DOMUtils.Tuple;
import org.xquark.xml.xdbc.XMLDBCException;

public class OpUnionSort extends MultOp implements Runnable {
    // **********************************************************************
    // * VERSIONING
    // **********************************************************************
    private static final String RCSRevision = "$Revision: 1.1 $";
    private static final String RCSName = "$Name:  $";

    private ArrayList orderbys = null;
    // ***********************************************************************
    // * INITIALIZATION
    // ***********************************************************************
    /**
     *
     */

    public OpUnionSort(ExecutionPlan plan, ArrayList algebralist, ArrayList orderbys) throws MediatorException {
        super(plan, null, algebralist) ;
        this.orderbys = orderbys;
        init();
    }

    private void init() {
        size = this.getSubElement(0).getSize();
        idsize = this.getSubElement(0).getIdSize();
        //ordertype = ORDER_PARALLELIZATION ;
        //ordertype = this.ORDER_SERIALIZATION ;
    }

    public ArrayList getOrderBys() { return this.orderbys; }

	// #############################################################################
	// VISITOR STUFF
	// #############################################################################

	public void accept(OperatorVisitor visitor) throws MediatorException {
		visitor.visit(this);
	}

    // ***********************************************************************
    // * EXECUTE QUERY
    // ***********************************************************************
    /**
     *
     */
    public void executeQuery() throws MediatorException {
        super.executeQuery() ;
        try {
            resultset = new UnionSortResultSet(this) ;
        }
        catch (XMLDBCException e) {
            //throw new MediatorException("Can't construct " + getClass() + ": " + e.getMessage(), e) ;
            throw new MediatorException("Can't construct " + getClass() + ": " + e.getMessage()) ;
        }
    }

    // **********************************************************************
    // * OPTIMIZE
    // **********************************************************************
    /**
     * Merge *this* node with the subnode(s). As it should be called
     * only from source, the returned node must be of type AlgSource.
     * The return node is a AlgSource where the XQueryExpression is the merging
     * of all child(ren) nodes' expression.
     */
    // protected Algebra mergeWithSub() throws MediatorException { return this ; }

    // **********************************************************************
    // * DEBUG
    // **********************************************************************
}

class UnionSortResultSet extends MultResultSet {
    // **********************************************************************
    // * VERSIONING
    // **********************************************************************
    private static final String RCSRevision = "$Revision: 1.1 $";
    private static final String RCSName = "$Name:  $";

    private int index = 0;
    // ***********************************************************************
    // * INITIALIZATION
    // ***********************************************************************
    /**
     *
     */
    public UnionSortResultSet(OpUnionSort algebra) throws XMLDBCException {
        super(algebra) ;
    }

    // ***********************************************************************
    // * EVALUATE IMPLEMENTATION
    // ***********************************************************************
    /**
     *
     * @param non_blocking if the evaluation is blocking, we must
     *        generate all the results before passing.
     *		  A cartesian product is necessary blocking.
     */
    protected void evaluate(boolean non_blocking) throws XMLDBCException {
    	
    	//non_blocking = false;
    	
        boolean islet = this.algebra.isLet();
        ArrayList allTuples = new ArrayList() ;
        int[] indexes = new int[this.getSubElementsSize()];
        Tuple restuple = null;
        for (int i = 0 ; i < this.getSubElementsSize() ; i ++) {
            ArrayList al = new ArrayList() ;
            ResultSet subalgi = this.getSubElement(i) ;
            while (subalgi.hasNext()) {
                Tuple tuplei = subalgi.next() ;
                if (islet) {
                    if (restuple == null) restuple = tuplei;
                    else restuple.appendTuple(tuplei);
                }
                else al.add(tuplei) ;
            }
            if (islet && restuple != null) al.add(restuple);
            allTuples.add(al) ;
            if (al.isEmpty()) indexes[i] = -1;
            else indexes[i] = 0;
        }
        ArrayList orderbys = ((OpUnionSort)algebra).getOrderBys();
        int index = 0;
        int size = this.getSubElementsSize();
        restuple = null;
        // get first element to be compared with
        while (index < size) {
            if (indexes[index] != -1) { restuple = (Tuple)((ArrayList)allTuples.get(index)).get(indexes[index]); break; }
            else this.getSubElement(index).close();
            index++;
        }
        if (restuple == null) {
            this.setFinishedWhenEmpty();
            return;
        }
        // it is supposed that all different lists are already ordered
        int tmpindex = index+1;
        if (tmpindex == size) tmpindex = 0;
        while (true) {
            int lowindex = index;
            while (tmpindex != index) {
                if (indexes[tmpindex] != -1) {
                    Tuple tmptuple = (Tuple)((ArrayList)allTuples.get(tmpindex)).get(indexes[tmpindex]);
                    if (!DOMUtils.orderTuple(restuple,tmptuple,orderbys)) {
                        restuple = tmptuple;
                        lowindex = tmpindex;
                    }
                }
                tmpindex++;
                if (tmpindex == size) tmpindex = 0;
            }
            // add to buftuples
            this.buftuples.add(restuple);
            indexes[lowindex]++;
            if (indexes[lowindex] == ((ArrayList)allTuples.get(lowindex)).size()) {
                indexes[lowindex] = -1;
                if (lowindex == index) {
                    index++;
                    restuple = null;
                    // find new index
                    while (index < size) {
                        if (indexes[index] != -1) { restuple = (Tuple)((ArrayList)allTuples.get(index)).get(indexes[index]); break; }
                        else this.getSubElement(index).close();
                        index++;
                    }
                    if (restuple == null) break;
                    tmpindex = index+1;
                    if (tmpindex == size) tmpindex = 0;
                }
                else {
                    restuple = null;
                    // find new index
                    while (index < size) {
                        if (indexes[index] != -1) { restuple = (Tuple)((ArrayList)allTuples.get(index)).get(indexes[index]); break; }
                        else this.getSubElement(index).close();
                        index++;
                    }
                    if (restuple == null) break;
                    tmpindex = index+1;
                    if (tmpindex == size) tmpindex = 0;
                }
            }
            else {
                if (lowindex == index) {
                    tmpindex = index+1;
                    if (tmpindex == size) tmpindex = 0;
                }
                else tmpindex = lowindex;
                restuple = (Tuple)((ArrayList)allTuples.get(index)).get(indexes[index]);
            }
        }
		//org.xquark.xml.misc.Debug.print("AlgUnionSort : " + algebra.toCompleteString());
		//org.xquark.xml.misc.Debug.print("AlgUnionSort : buftuples = " + buftuples);
        this.setFinishedWhenEmpty();
    }
}
