/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */

package org.xquark.mediator.plan ;

import org.xquark.mediator.MediatorException;
import org.xquark.mediator.decomposer.Utils;
import org.xquark.xquery.parser.XQueryExpression;

public abstract class ZeroOp extends Operator implements Runnable {
    // **********************************************************************
    // * VERSIONING
    // **********************************************************************
    private static final String RCSRevision = "$Revision: 1.1 $";
    private static final String RCSName = "$Name:  $";
    // **********************************************************************
    // * CLASS VARIABLES
    // **********************************************************************
    
    // ***********************************************************************
    // * INITIALIZATION
    // ***********************************************************************
    /**
     *
     */
    public ZeroOp(ExecutionPlan plan, XQueryExpression expression) throws MediatorException {
        super(plan, expression) ;
    }
	// #############################################################################
	// VISITOR STUFF
	// #############################################################################

	public void accept(OperatorVisitor visitor) throws MediatorException {
		visitor.visit(this);
	}

    // ***********************************************************************
    // * EXECUTE QUERY
    // ***********************************************************************
    /**
     *
     */
    public void executeQuery() throws MediatorException {
    }
    
    // ***********************************************************************
    // * OPTIMIZE
    // ***********************************************************************
    /**
     * If the set of sources handled by this node are exactly the same (order
     * not important) then return true.
     * An AlgZeroOp is alway true.
     * @return true
     */
    protected boolean shareSameSources() {
        return true ;
    }
    
    /**
     * Return true if the node is of type source.
     * An AlgZeroOp is alway a source.
     * @return true
     */
    public boolean isSource() {
        return true ;
    }
    
    /**
     * Merge *this* node with the subnode(s). As it should be called
     * only from source, the returned node must be of type AlgZeroOp.
     * For AlgZeroOp it is the node itself.
     */
    // protected Algebra mergeWithSub() throws MediatorException { return this ; }
    
    public String toCompleteString(int indent) {
        StringBuffer buf = new StringBuffer() ;
        buf.append(Utils.makeIndent(indent) + "<" + getClass().getName() + " isLet=\"" + isLet() + "\">\n") ;
        buf.append(Utils.makeIndent(indent + 1) + "<Expression>\n") ;
        buf.append(Utils.makeIndent(indent + 2) + expression + "\n") ;
        buf.append(Utils.makeIndent(indent + 1) + "</Expression>\n") ;
//        if (variables != null) {
//            buf.append(Utils.makeIndent(indent + 1) + "<Variables>\n") ;
//            for (int i = 0 ; i < variables.size() ; i ++) {
//                buf.append(Utils.makeIndent(indent + 2) + "<Variable>" + variables.get(i) + "</Variable>\n") ;
//            }
//            buf.append(Utils.makeIndent(indent + 1) + "</Variables>\n") ;
//        }
        if (paths != null) {
            buf.append(Utils.makeIndent(indent + 1) + "<Paths>\n") ;
            for (int i = 0 ; i < paths.size() ; i ++) {
                buf.append(Utils.makeIndent(indent + 2) + "<Path>" + paths.get(i) + "</Path>\n") ;
            }
            buf.append(Utils.makeIndent(indent + 1) + "</Paths>\n") ;
        }
        buf.append(Utils.makeIndent(indent) + "</" + getClass().getName() + ">\n") ;
        return buf.toString() ;
    }
}
