/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */

package org.xquark.mediator.plan.primitivefunctions ;

import java.util.ArrayList;

import org.xquark.mediator.MediatorException;
import org.xquark.mediator.DOMUtils.EvaluationVisitor;
import org.xquark.mediator.DOMUtils.Tuple;
import org.xquark.mediator.plan.*;
import org.xquark.xml.xdbc.XMLDBCException;
import org.xquark.xpath.datamodel.TypedNode;
import org.xquark.xquery.parser.*;
import org.xquark.xquery.parser.primitivefunctions.fnfunctions.FunctionDISTINCT_VALUES;

public class OpFuncDISTINCT_VALUES extends UnOp implements Runnable {
    // **********************************************************************
    // * VERSIONING
    // **********************************************************************
    private static final String RCSRevision = "$Revision: 1.1 $";
    private static final String RCSName = "$Name:  $";
    // ***********************************************************************
    // * INITIALIZATION
    // ***********************************************************************
    /**
     *
     */
    public OpFuncDISTINCT_VALUES(ExecutionPlan plan, XQueryExpression expression,Operator childexp) throws MediatorException {
        super(plan, expression, childexp) ;
        // the path number can be only one (otherwise distinct values might be incorrect)
        if (paths != null) 
        	paths.clear();
       	// calculate paths
		if (paths == null) 
			paths = new ArrayList();
       	getRetPath(((FunctionDISTINCT_VALUES)expression).getArgument(0),paths);
       	if (paths.isEmpty() || paths.size() > 1)
       		throw new MediatorException("Could not find paths for distinct-values function");
		//idsize = childalgebra.getIdSize();
		size = 1;
		idsize = 0;
    }
    
    // TODO better!!!
    private void getRetPath(XQueryExpression expr, ArrayList reslist) {
		if (expr instanceof XQueryExpressionSequence) {
			for (int i=0;i<((XQueryExpressionSequence)expr).getSubExpressions().size();i++) {
				getRetPath((XQueryExpression)((XQueryExpressionSequence)expr).getSubExpressions().get(i),reslist);
			}
		}
		if (expr instanceof LocatedExpression) {
			reslist.add(expr);
		}
		else if (expr instanceof Variable) {
			reslist.add(expr);
		}
		else if (expr instanceof FLWRExpression) {
			getRetPath(((FLWRExpression)expr).getReturnClause(),reslist);        	
		}
    }
    // ***********************************************************************
    // * EXECUTE QUERY
    // ***********************************************************************
    /**
     *
     */
    public void executeQuery() throws MediatorException {
        super.executeQuery() ;
        try {
            resultset = new FuncDISTINCT_VALUESResultSet(this) ;
        }
        catch (XMLDBCException e) {
            throw new MediatorException("Can't construct " + getClass() + ": " + e.getMessage()) ;
        }
    }
    
    // **********************************************************************
    // * OPTIMIZE
    // **********************************************************************
    /**
     * FuncDISTINCT_VALUES *this* node with the subnode(s). As it should be called
     * only from source, the returned node must be of type AlgSource.
     * The return node is a AlgSource where the XQueryExpression is the merging
     * of all child(ren) nodes' expression.
     */
    // protected Algebra mergeWithSub() throws MediatorException { return this ; }
    
    // **********************************************************************
    // * DEBUG
    // **********************************************************************
    /**
     *
     */
}




class FuncDISTINCT_VALUESResultSet extends UnResultSet {
    // **********************************************************************
    // * VERSIONING
    // **********************************************************************
    private static final String RCSRevision = "$Revision: 1.1 $";
    private static final String RCSName = "$Name:  $";
    // ***********************************************************************
    // * CLASS VARIABLES
    // ***********************************************************************
    
    // ***********************************************************************
    // * INITIALIZATION
    // ***********************************************************************
    public FuncDISTINCT_VALUESResultSet(OpFuncDISTINCT_VALUES algebra) throws XMLDBCException {
        super(algebra) ;
    }
    
    // ***********************************************************************
    // * EVALUATE IMPLEMENTATION
    // ***********************************************************************
    /**
     * In a merge product, we can all do in parallel, so we
     * must generate all results of both source and then make the merge
     * product.
     *
     * @param non_blocking if the evaluation is blocking, we must
     *        generate all the results before passing.
     */
    protected void evaluate(boolean non_blocking) throws XMLDBCException {
    	
    	//non_blocking = false;
    	
    	// if no resultset do evaluation visitor
    	if (resultset == null) {
    		try {
				ArrayList tuples = this.algebra.getPlan().getCurrentTuples();
				if (tuples != null && !tuples.isEmpty()) {
					EvaluationVisitor visitor = new EvaluationVisitor(this.algebra.getPlan().getSchemaManager());
					visitor.reset(tuples);
					visitor.setReturnType(EvaluationVisitor.NODE_TYPE);
					if (expression != null) {
						expression.accept(visitor);
						ArrayList nodes = visitor.getResNodes();
						if (nodes != null) {
							for (int i=0;i<nodes.size();i++) {
								Tuple newtuple = buftuples.newTuple();
								newtuple.addNodeAtIndex(0,(TypedNode)nodes.get(i));
								//newtuple.fillIdentifiers(buftuples.getIdentifier());
								newtuple.fillIdentifiers();
								buftuples.add(newtuple);
							}
						}
						else {
							//org.xquark.xml.misc.Debug.print("AlgFuncDISTINCT_VALUES : expression = " + expression);
							//org.xquark.xml.misc.Debug.print("AlgFuncDISTINCT_VALUES : tuples = " + tuples);
						}
					}
				}
			} catch (Exception e) {
				//e.printStackTrace () ;
				throw new XMLDBCException("AlgFuncDISTINCT_VALUES.evaluate: " + e.getMessage());
			}
			setFinishedWhenEmpty();
			return;
    	}
    	
    	// resultset is not null
        if (! resultset.hasNext()) {
            resultset.close() ;
            setFinishedWhenEmpty() ;
            return ;
        }
        
        if (! resultset.hasNext()) {
        }
        else {
			String path = algebra.getPaths().get(0).toString();
        	ArrayList tmplist = new ArrayList();
            //al = new BufferTuple(childpaths,this.algebra.getIdSize()) ;
            while (resultset.hasNext()) {
                Tuple tuple = resultset.next() ;
				//org.xquark.xml.misc.Debug.print("tuple = " + tuple.toCompleteString());
				Tuple restuple = tuple.getPart(path);
				if (restuple == null)
					restuple = tuple.getPart(0);
				if (restuple != null) {
					tmplist.add(restuple) ;
				}
            }
        
            for (int i = 0 ; i < tmplist.size() ; i ++) {
                Tuple tuplei = (Tuple) tmplist.get(i) ;
				//org.xquark.xml.misc.Debug.print("tuplei = " + tuplei.toCompleteString());
                boolean found = false;
                for (int j=0;j<i;j++) {
                    Tuple tuplej = (Tuple)tmplist.get(j);
					//org.xquark.xml.misc.Debug.print("tuplej = " + tuplej.toCompleteString());
                    if (tuplej.equals(tuplei)) { found = true; break; }
                }
                if (!found) buftuples.add(tuplei) ;
            }
        }
        
		//org.xquark.xml.misc.Debug.print("AlgFuncDISTINCT_VALUES : " + algebra.toCompleteString());
		//org.xquark.xml.misc.Debug.print("AlgFuncDISTINCT_VALUES : buftuples = " + buftuples);
        resultset.close() ;
        setFinishedWhenEmpty() ;
    }
}
