/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */

 package org.xquark.schema;
/**
 * class for Attribute Declaration.
 *
 * <p>This is the implementation of the Attribute Declaration.
 * The following properties are defined:</p>
 * <p> 1. name : a NCName, defined in SchemaComponent.</p>
 * <p> 2. target namespace : null or a namespace name, defined in SchemaComponent.</p>
 * <p> 3. type defintion : a simple type defintion, defined in Declaration.</p>
 * <p> 4. scope : either null (means gloabal) or a complex type defintion, defined in Declaration. </p>
 * <p> 5. value constraint : optional. Either a defaultValue or a fixedValue, defined in Declaration. </p>
 *
 * <p>See XML Schema Part 1: Structures 3.2
 * W3C Recommendation 2 May 2001</p>
 *
 * @see org.xquark.xml.schema.SchemaComponent
 * @see org.xquark.xml.schema.Declaration
 */
public class AttributeDeclaration extends Declaration implements SchemaConstants {
private static final String RCSRevision = "$Revision: 1.1 $"; 
private static final String RCSName     = "$Name:  $";
  
  // schema 2001-05
  // use = optional, prohibited, required : optional  
  private int use = OPTIONAL;
  
  /**
   * Create a new AttributeDeclaration.
   *
   * @param schema The schema which contains this attribute declaration.
   * @param name The attribute declaration name, a NCName.
   * @param scope The scope, null if the attribute declaration is in a group, 
   * or a complex type which contains this attribute declaration.
   */
  public AttributeDeclaration(Schema schema, String name, SchemaScope scope) {
    super(schema, name, scope);
  }

  public void accept(SchemaVisitor visitor) throws SchemaException {
    visitor.visit(this);
  }

  public int getUse() { return this.use; }
  
  public String getUseString() { 
    switch( use ) {
      case OPTIONAL: return OPTIONAL_VALUE;
      case PROHIBITED: return PROHIBITED_VALUE;
      case REQUIRED: return REQUIRED_VALUE;
    }
    return OPTIONAL_VALUE;
  }
  
  public void setUse(int use) {
    this.use = use; 
  }
  
}


