/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */

package org.xquark.schema;

import java.util.HashSet;

public final class Facet implements Cloneable {
private static final String RCSRevision = "$Revision: 1.1 $"; 
private static final String RCSName     = "$Name:  $";
	String name;
	String value;
	boolean fixed = false;
	
	// enumeration facet : special
	HashSet enumFacets = new HashSet();
	
  // contructor for facet other than enumeation
  public Facet(String name, String value, String fixed) {
		this.name = name;
		this.value = value;
		if ( "true".equals(fixed) ) this.fixed = true;
  }
	
  // contructor only for facet enumeation
  Facet(String name) {
		this.name = name;
  }
  
  public Object clone() throws CloneNotSupportedException {
    Facet result = null;
    result = (Facet)super.clone();
    result.enumFacets = new HashSet();
    java.util.Iterator it = enumFacets.iterator();
    while ( it.hasNext() ) {
      String value = new String((String)it.next());
      result.enumFacets.add(value);
    }
    return result;
  }	
  
	// add a new value to enumeation hashset
  public void addFacet(String value) {
		enumFacets.add(value);
  }

  public void setFacet(String name, String value, String fixed) {
		this.name = name;
		this.value = value;
		if ( "true".equals(fixed) ) this.fixed = true;
  }
	
  public void setValue(String value) {
		this.value = value;
  }	
	
  public String getName() {
		return this.name;
  }
	
  public String getValue() {
		return this.value;
  }
	
  public boolean isFixed() {
		return this.fixed;
  }
	
  public HashSet getEnumFacets() {
		return enumFacets;
  }
  
  public String toString() {
		return this.name;
  }

}
