/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */

package org.xquark.schema;

public interface SchemaConstants {
 

    
  public static final String XMLSCHEMA_URI = 
    "http://www.w3.org/2001/XMLSchema";
  public static final String XSI_URI = 
    "http://www.w3.org/2001/XMLSchema-instance";
  public static final String XML_URI = 
    "http://www.w3.org/XML/1998/namespace";
  public static final String XMLNS_URI = 
    "http://www.w3.org/2000/xmlns/";
  public static final String XML_PREFIX = "xml";
  public static final String XMLNS_PREFIX = "xmlns";
  
  // schema tags
  public static final String SCHEMA_TAG = "schema";
  public static final String INCLUDE_TAG = "include";
  public static final String IMPORT_TAG = "import";
  public static final String REDEFINE_TAG = "redefine";
  public static final String SIMPLE_UR_TYPE = "anySimpleType";
  public static final String SIMPLE_TYPE_TAG = "simpleType";
  public static final String COMPLEX_TYPE_TAG = "complexType";
  public static final String ELEMENT_TAG = "element";
  public static final String ATTRIBUTE_TAG = "attribute";
  public static final String GROUP_TAG = "group";
  public static final String SEQUENCE_TAG = "sequence";
  public static final String CHOICE_TAG = "choice";
  public static final String ALL_TAG = "all";
  public static final String ATTRIBUTE_GROUP_TAG = "attributeGroup";
  public static final String ANY_TAG = "any";
  public static final String ANY_ATTRIBUTE_TAG = "anyAttribute";
  public static final String UNIQUE_TAG = "unique";
  public static final String KEY_TAG = "key";
  public static final String KEYREF_TAG = "keyref";
  public static final String SELECTOR_TAG = "selector";
  public static final String FIELD_TAG = "field";
  public static final String ANNOTATION_TAG = "annotation";
  public static final String APPINFO_TAG = "appinfo";
  public static final String DOCUMENTATION_TAG = "documentation";
  public static final String NOTATION_TAG = "notation";

  public static final String SIMPLECONTENT_TAG = "simpleContent";
	public static final String COMPLEXCONTENT_TAG = "complexContent";	
  public static final String LIST_TAG = "list";
  public static final String UNION_TAG = "union";
  public static final String RESTRICTION_TAG = "restriction";
  public static final String EXTENSION_TAG = "extension";

  // facets tags
  public static final String LENGTH_TAG = "length";
  public static final String MIN_LENGTH_TAG = "minLength";
  public static final String MAX_LENGTH_TAG = "maxLength";
  public static final String PATTERN_TAG = "pattern";
  public static final String ENUMERATION_TAG = "enumeration";
  public static final String WHITESPACE_TAG = "whiteSpace";
  public static final String MIN_EXCLUSIVE_TAG = "minExclusive";
  public static final String MIN_INCLUSIVE_TAG = "minInclusive";
  public static final String MAX_EXCLUSIVE_TAG = "maxExclusive";
  public static final String MAX_INCLUSIVE_TAG = "maxInclusive";  
  public static final String TOTALDIGITS_TAG = "totalDigits";
  public static final String FRACTIONDIGITS_TAG = "fractionDigits";
	
  public static final String[] FACETS = {
    LENGTH_TAG, MIN_LENGTH_TAG, MAX_LENGTH_TAG,
    PATTERN_TAG, ENUMERATION_TAG, WHITESPACE_TAG,
    MIN_EXCLUSIVE_TAG, MIN_INCLUSIVE_TAG, MAX_EXCLUSIVE_TAG, MAX_INCLUSIVE_TAG,
    TOTALDIGITS_TAG, FRACTIONDIGITS_TAG,
  };

  // generic attribute
  public static final String ID_ATTR = "id";

  // schema attributes
  public static final String ATTRIBUTE_FORM_DEFAULT_ATTR = "attributeFormDefault";
  public static final String BLOCK_DEFAULT_ATTR = "blockDefault";
  public static final String ELEMENT_FORM_DEFAULT_ATTR = "elementFormDefault";
  public static final String FINAL_DEFAULT_ATTR = "finalDefault";
  public static final String TARGET_NAMESPACE_ATTR = "targetNamespace";
  public static final String VERSION_ATTR = "version";

  // include attributes
  public static final String SCHEMA_LOCATION_ATTR = "schemaLocation";
  public static final String NAMESPACE_ATTR = "namespace";
  
  // simpleType attributes
  public static final String ABSTRACT_ATTR = "abstract";
  public static final String BASE_ATTR = "base";
  public static final String NAME_ATTR = "name";
  public static final String ITEM_TYPE_ATTR = "itemType";
  public static final String MEMBER_TYPES_ATTR = "memberTypes";
  
  // complexType attributes
  public static final String BLOCK_ATTR = "block";
  public static final String CONTENT_ATTR = "content";
  public static final String FINAL_ATTR = "final";
  public static final String MIXED_ATTR = "mixed";

  // element attributes
  public static final String DEFAULT_ATTR = "default";
  public static final String SUBSTITUTION_GROUP_ATTR = "substitutionGroup";
  public static final String FIXED_ATTR = "fixed";
  public static final String FORM_ATTR = "form";
  public static final String MAX_OCCURS_ATTR = "maxOccurs";
  public static final String MIN_OCCURS_ATTR = "minOccurs";
  public static final String NILLABLE_ATTR = "nillable";
  public static final String REF_ATTR = "ref";
  public static final String TYPE_ATTR = "type";
  
  // attribute attributes
  public static final String USE_ATTR = "use";
  public static final String VALUE_ATTR = "value";

  public static final String PROCESS_CONTENTS_ATTR = "processContents";
  
  public static final String PUBLIC_ATTR = "public";
  public static final String SYSTEM_ATTR = "system";

  public static final String REFER_ATTR = "refer";
  public static final String XPATH_ATTR = "xpath";
  
  // annotation attributes
  public static final String SOURCE_ATTR = "source";
  
  // XSI attributes
  public static final String XSI_NIL_ATTR = "nil";
  public static final String XSI_TYPE_ATTR = "type";
  public static final String XSI_SCHEMA_LOCATION_ATTR = "schemaLocation";
  public static final String XSI_NO_NAMESPACE_SCHEMA_LOCATION_ATTR =
    "noNamespaceSchemaLocation";
  
  // values
  public static final String TRUE_VALUE = "true";
  public static final String FALSE_VALUE = "false";
  public static final String ONE_VALUE = "1";
  public static final String ZERO_VALUE = "0";
  
  public static final String QUALIFIED_VALUE = "qualified";
  public static final String UNQUALIFIED_VALUE = "unqualified";

  public static final String ALL_VALUE = "#all";
  public static final String LIST_VALUE = "list";
  public static final String UNION_VALUE = "union";
  public static final String RESTRICTION_VALUE = "restriction";
  public static final String EXTENSION_VALUE = "extension";
  public static final String SUBSTITUTION_VALUE = "substitution";
  
  public static final int LIST = 1;
  public static final int UNION = 2;
  public static final int RESTRICTION = 4;
  public static final int EXTENSION = 8;
  public static final int SUBSTITUTION = 16;
  
  public static final String DEFAULT_VALUE = "default";
  public static final String FIXED_VALUE = "fixed";
  public static final String OPTIONAL_VALUE = "optional";
  public static final String PROHIBITED_VALUE = "prohibited";
  public static final String REQUIRED_VALUE = "required";

  public static final int DEFAULT = 0;
  public static final int FIXED = 1;
  public static final int OPTIONAL = 2;
  public static final int PROHIBITED = 3;
  public static final int REQUIRED = 4;

  public static final String UNBOUNDED_VALUE = "unbounded";

  public static final String EMPTY_VALUE = "empty";
  public static final String TEXT_ONLY_VALUE = "textOnly";
  public static final String ELEMENT_ONLY_VALUE = "elementOnly";
  public static final String MIXED_VALUE = "mixed";

  public static final int UNKNOWN = -1;
  public static final int EMPTY = 0;
  public static final int TEXT_ONLY = 1;
  public static final int ELEMENT_ONLY = 2;
  public static final int MIXED = 3;

  public static final String ANY_VALUE = "##any";
  public static final String LOCAL_VALUE = "##local";
  public static final String OTHER_VALUE = "##other";
  public static final String TARGET_NAMESPACE_VALUE = "##targetNamespace";
  
  public static final String LAX_VALUE = "lax";
  public static final String SKIP_VALUE = "skip";
  public static final String STRICT_VALUE = "strict";
  
  public static final int LAX = 0;
  public static final int SKIP = 1;
  public static final int STRICT = 2;
  	 	
  // any type, used in element initialisation
  public static final String ANY_TYPE = "anyType";
 
}

