/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */

package org.xquark.schema;

import java.util.Iterator;
import java.util.LinkedList;

public class SchemaContext {
private static final String RCSRevision = "$Revision: 1.1 $"; 
private static final String RCSName     = "$Name:  $";

  private LinkedList contextStack = new LinkedList();
  private SchemaManager manager;
  private SchemaLocator locator;
  
  public SchemaContext(SchemaManager manager) {
    this(manager, manager);
  }

  public SchemaContext(SchemaManager manager, SchemaLocator locator) {
    this.manager = manager;
    this.locator = locator;
  }

  public ElementDeclaration getElementDeclaration (String namespace, String localName) 
  {
      if (namespace != null && namespace.length() == 0) namespace = null;
      ElementDeclaration decl = null;
      
      if (!contextStack.isEmpty() && contextStack.getFirst() != null)
      {
          decl = (ElementDeclaration)contextStack.getFirst();
          decl = decl.getType().getElementDeclaration(namespace, localName);
      }
      // search in top level declaration if at the root or substitution group or wildcard
      // TO DO : check that this is a substitution group or wildcard
      if (decl == null)
      {
          Schema schema = null;
          try 
          {
              schema = manager.loadSchema(locator, namespace);
          }
          catch ( Exception e )
          {
      //e.printStackTrace(System.out);
              //throw new RuntimeException(e.getMessage());
          }
          if (schema == null)
              return null;
          decl = schema.getElementDeclaration(localName);
      }
      return decl;
  }
  
  public AttributeDeclaration getAttributeDeclaration(String namespace, String localName)
  {
    if (namespace != null && namespace.length() == 0) namespace = null;
    if (contextStack.isEmpty() || contextStack.getFirst() == null) {
      Schema schema = null;
      try {
        schema = manager.loadSchema(locator, namespace);
      }
      catch ( Exception e ) {
				throw new RuntimeException(e.getMessage());
      }
      if (schema == null) return null;
      AttributeDeclaration decl = schema.getAttributeDeclaration(localName);
      return decl;
    } else {
      ElementDeclaration decl = (ElementDeclaration)contextStack.getFirst();
      return decl.getType().getAttributeDeclaration(namespace, localName);
    }
  }

  public boolean isEmpty() {
    return contextStack.isEmpty();
  }
  
  public Iterator iterator() {
    return contextStack.iterator();
  }
  
  public void push(ElementDeclaration decl) {
    contextStack.addFirst(decl);
  }

  public ElementDeclaration top() {
    if (contextStack.isEmpty()) return null;
    return (ElementDeclaration)contextStack.getFirst();
  }
  
  public ElementDeclaration pop() {
    if (contextStack.isEmpty()) return null;
    return (ElementDeclaration)contextStack.removeFirst();
  }
}
