/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */

package org.xquark.schema.datatypes;

import org.apache.xerces.impl.xpath.regex.RegularExpression;
import org.xquark.schema.SchemaException;
import org.xquark.schema.validation.ValidationContextProvider;


abstract class AbstractQNameType extends MesureableType {
    private static final String RCSRevision = "$Revision: 1.1 $";
    private static final String RCSName = "$Name:  $";

    private RegularExpression ncNameFormat = null;

    AbstractQNameType(String name, int type) {
        super(name, type);
		ncNameFormat = new RegularExpression("[\\i-[:]][\\c-[:]]*", "X");
    }

	protected abstract QName createQName(String namespaceURI, String localName);
	
    protected QName toQName(String value, ValidationContextProvider context) throws SchemaException {
		String prefix = null;
		String localName = null;
		int index = value.indexOf(':');
		if (index == 0) {
			super.invalidValue(value);
		} else if (index == -1) {
			prefix = "";
			localName = value;
		} else {
			prefix = value.substring(0, index);
			if (!ncNameFormat.matches(prefix))
				super.invalidValue(value);
			localName = value.substring(index + 1);
		}
        if (!ncNameFormat.matches(localName))
            super.invalidValue(value);
        // Avoids errors when context is null. This is lax
        if (context == null) super.invalidValue(value);
        String uri = context.getNamespaceURI(prefix);
        if (uri == null && prefix.length() > 0)
        	super.invalidValue(value);
        return createQName(uri, localName);
    }

    public void checkFacets(Object valueSpace) throws SchemaException {
        super.checkFacets(valueSpace);
        // length-related facets are deprecated for QName and NOTATION datatypes
        // super.checkLength(((String) valueSpace).length(), valueSpace);
    }

    protected Object toValueSpace(String value, ValidationContextProvider context) throws SchemaException {
        return toQName(value, context);
    }

    public String toXMLString(Object data, ValidationContextProvider context) {
        if (data == null)
            return null;
        QName value = (QName) data;
        if (context == null)
            return value.toString();
        if (value.getNamespaceURI() == null)
        	return value.getLocalName();
        String prefix = context.getPrefix(value.getNamespaceURI());
        if (prefix == null)
            return value.toString();
        if (prefix.length() == 0)
            return value.getLocalName();
        else
            return prefix + ":" + value.getLocalName();
    }

}
