/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */

package org.xquark.schema.datatypes;

import java.util.*;

import org.xquark.schema.SchemaException;
import org.xquark.schema.validation.ValidationContextProvider;


abstract class EnumerableType extends PrimitiveType {
    private static final String RCSRevision = "$Revision: 1.1 $";
    private static final String RCSName = "$Name:  $";

    private HashSet enumerations = null;
    private List values = null;

    EnumerableType(String name, int type) {
        super(name, type);
    }

    // this method is used in NOTATIONType
    // enumerations for types derived from NOTATION are mandatory
    protected java.util.Collection getEnumerations() {
        return enumerations;
    }

    protected void setEnumeration(HashSet value, ValidationContextProvider vcProvider) throws SchemaException {
        HashSet result = new HashSet();
        Iterator it = value.iterator();
        while (it.hasNext()) {
            String enum = normalizedValue((String) it.next());
            Object valueSpace = this.toValueSpace(enum, vcProvider);
            if (enumerations != null) checkFacets(valueSpace);
            result.add(valueSpace);
        }
        enumerations = result;
        values = null;
    }

    public void checkFacets(Object valueSpace) throws SchemaException {
        if (enumerations != null && !enumerations.contains(valueSpace))
            super.invalidFacet("cvc-enumeration-valid", enumerations, valueSpace.toString());
    }

    public List getValues() {
        if (enumerations != null && values == null) {
            values = new ArrayList();
            Iterator it = enumerations.iterator();
            while (it.hasNext()) {
                values.add(toXMLString(it.next(), null));
            }
        }
        return values;
    }
}
