/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */

package org.xquark.schema.datatypes;

import java.util.ArrayList;

import org.xquark.schema.SchemaException;
import org.xquark.schema.validation.ValidationContextProvider;


class PrimitiveUnionType extends EnumerableType {
    private static final String RCSRevision = "$Revision: 1.1 $";
    private static final String RCSName = "$Name:  $";

    ArrayList primitives = new ArrayList();

    PrimitiveUnionType(ArrayList primitives, String unionName) {
        super(unionName, PrimitiveType.UNION);
        // No direct whitespace handling for union
        nWhiteSpace = PrimitiveType.PRESERVE;
        this.primitives = primitives;
    }

    protected void setWhiteSpace(String value) throws SchemaException {
        //    "Facet whiteSpace not supported in " + this;
        throw new SchemaException("cos-applicable-facets");
    }

    protected void invalidValue(String value, SchemaException se) throws SchemaException {
        throw new SchemaException("cvc-datatype-valid.1.2.3", this, se);
    }

    protected Object toValueSpace(String value, ValidationContextProvider context) throws SchemaException {
        return checkUnion(value, context);
    }

    protected Object checkUnion(String value, ValidationContextProvider context) throws SchemaException {
        Object result = null;
        ArrayList exceptions = null;
        for (int i = 0; i < primitives.size(); i++) {
            try {
                PrimitiveType pType = (PrimitiveType) primitives.get(i);
                result = pType.convert(value, true, context);
                break;
            } catch (SchemaException ex) {
                if (exceptions == null)
                    exceptions = new ArrayList();
                exceptions.add(ex);
            }
        }
        if (result != null) {
            return result;
        } else {
            throw new SchemaException("cvc-datatype-valid.1.2.3", this, exceptions);
        }
    }
}
