/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */

package org.xquark.schema.validation;

import java.net.URL;
import java.util.*;

import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.xml.sax.XMLReader;
import org.xquark.schema.*;
import org.xquark.util.NamespaceContext;

public class SchemaValidationContext
	extends DefaultSchemaLocator
	implements NamespaceContext {
	private static final String RCSRevision = "$Revision: 1.1 $";
	private static final String RCSName = "$Name:  $";

	private SchemaManager manager;
	private NamespaceContext nsContext = null;
	private ArrayList infoset = new ArrayList(32);
	private int level = 0;
	private int validationContext = -1;
	private HashSet IDSet = new HashSet();
	private HashSet pendingRefs = new HashSet();

	// load schema from namespace ? true, yes; false, no
	private boolean bLoadSchemaFromNamespace = false;

	/** 
	 * Creates new SchemaValidationContext
	 */

	public SchemaValidationContext() {
		this(new SchemaManager());
	}

	public SchemaValidationContext(SchemaManager manager) {
		this.manager = manager;
		reset(null);
	}

	public void setLoadSchemaFromNamespace(boolean flag) {
		bLoadSchemaFromNamespace = flag;
	}

	public boolean isLoadSchemaFromNamespace() {
		return bLoadSchemaFromNamespace;
	}

	public InputSource resolveLocation(String namespace, URL location) {
		if (!bLoadSchemaFromNamespace && location.toString().equals(namespace))
			return null;
		else
			return super.resolveLocation(namespace, location);
	}

	public XMLReader getSchemaReader() throws SAXException {
		// before gerSchemaReader, validate source
		// source is a schmema input source
		SchemaValidationContext context = new SchemaValidationContext();
		ValidatingSchemaFilter filter =
			new ValidatingSchemaFilter(super.getSchemaReader(), context, true);
		return filter;
		//    return super.getSchemaReader();
	}

	public ElementPSVInfoset getCurrentInfoset() {
		return getInfoset(level);
	}

	public ElementPSVInfoset getParentInfoset() {
		return getInfoset(level - 1);
	}

	public ElementPSVInfoset getInfoset(ElementDeclaration decl) {
		for (int i = level; i >= 0; i--) {
			ElementPSVInfoset result = (ElementPSVInfoset) infoset.get(i);
			if (result.getDeclaration() == decl)
				return result;
		}
		return null;
	}

	public ElementPSVInfoset getInfoset(int index) {
		if (index < 0 || index > level)
			return null;
		return (ElementPSVInfoset) infoset.get(index);
	}

	public SchemaManager getSchemaManager() {
		return manager;
	}

	/* (non-Javadoc)
	 * @see org.xquark.xml.util.NamespaceContext#getNamespaceURI(java.lang.String)
	 */
	public String getNamespaceURI(String prefix) {
		return nsContext != null ? nsContext.getNamespaceURI(prefix) : null;
	}

	/* (non-Javadoc)
	 * @see org.xquark.xml.util.NamespaceContext#getNamespaceURI(java.lang.String)
	 */
	public Collection getNamespaceURIs() {
		return nsContext != null ? nsContext.getNamespaceURIs() : null;
	}

	/* (non-Javadoc)
	 * @see org.xquark.xml.util.NamespaceContext#getPrefix(java.lang.String)
	 */
	public String getPrefix(String uri) {
		return nsContext != null ? nsContext.getPrefix(uri) : null;
	}

	/* (non-Javadoc)
	 * @see org.xquark.xml.util.NamespaceContext#getPrefixes()
	 */
	public Collection getPrefixes() {
		return nsContext != null ? nsContext.getPrefixes() : null;
	}

	/* (non-Javadoc)
	 * @see org.xquark.xml.util.NamespaceContext#getPrefixes(java.lang.String)
	 */
	public Collection getPrefixes(String uri) {
		return nsContext != null ? nsContext.getPrefixes(uri) : null;
	}

	/* (non-Javadoc)
	 * @see org.xquark.xml.util.NamespaceContext#getDeclaredPrefixes()
	 */
	public List getDeclaredPrefixes() {
		return nsContext != null ? nsContext.getDeclaredPrefixes() : null;
	}

	// This section contains (non-public) methods used by ValidatingSchemaFilter

	void reset(NamespaceContext nc) {
		nsContext = nc;
		level = 0;
		// validationContext = -1;
		validationContext = 1;
		IDSet.clear();
		pendingRefs.clear();
		infoset.clear();
		infoset.add(new ElementPSVInfosetImpl());
	}

	void pushElementInfoset(String namespace, String localName) {
		level++;
		if (level == infoset.size())
			infoset.add(new ElementPSVInfosetImpl());
		((ElementPSVInfosetImpl) getCurrentInfoset()).initialize(
			namespace,
			localName);
	}

	void popElementInfoset() {
		((ElementPSVInfosetImpl) getCurrentInfoset()).cleanup();
		level--;
		/* This is commented out as validationContext is the level at which assessment
		 * began, not the one at which it was successful
		if (level < validationContext) 
		  validationContext = -1;
		*/
	}

	ElementDeclaration getElementDeclaration() {
		return (ElementDeclaration) getCurrentInfoset().getDeclaration();
	}

	Type getElementType() {
		return getCurrentInfoset().getType();
	}

	void initElementValidationInfo(
		ElementDeclaration matchedDecl,
		Type type,
		boolean isNil,
		int process) {
		ElementPSVInfosetImpl psvi =
			(ElementPSVInfosetImpl) getCurrentInfoset();
		psvi.setField(-1, PSVInfosetImpl.DECLARATION, matchedDecl);
		psvi.setType(type);
		psvi.setNil(isNil);
		//@@ Normally, we should use temporary variables to store VALIDITY and VALIDATION_ATTEMPTED,
		// which should only be available at the end of the element processing.
		if (matchedDecl == null && type == null) {
			psvi.setField(
				-1,
				PSVInfosetImpl.VALIDATION_ATTEMPTED,
				PSVInfoset.NONE);
			psvi.setField(-1, PSVInfosetImpl.VALIDITY, PSVInfoset.NOTKNOWN);
		} else {
			psvi.setField(
				-1,
				PSVInfosetImpl.VALIDATION_ATTEMPTED,
				PSVInfoset.FULL);
			psvi.setField(-1, PSVInfosetImpl.VALIDITY, PSVInfoset.VALID);
		}
		psvi.setProcess(process);

		/* This is commented out as validationContext is the level at which assessment
		 * began, not the one at which it was successful
		if (validationContext == -1 && matchedDecl != null) {
		  validationContext = level;
		  IDSet.clear();
		  pendingRefs.clear();
		}
		*/
		if (matchedDecl != null
			&& matchedDecl.getIdentityConstraints() != null) {
			psvi.addIdentityConstraints(matchedDecl.getIdentityConstraints());
		}
	}

	void computeValidationInfo() {
		ElementPSVInfosetImpl psvi =
			(ElementPSVInfosetImpl) getCurrentInfoset();
		ElementPSVInfosetImpl parentPsvi =
			(ElementPSVInfosetImpl) getParentInfoset();
		if (psvi.getErrorCodes() != null) {
			psvi.setField(-1, PSVInfosetImpl.VALIDITY, PSVInfoset.INVALID);
		}
		if (psvi.getValidity() == PSVInfoset.INVALID
			|| (psvi.getValidity() == PSVInfoset.NOTKNOWN
				&& psvi.getProcess() == SchemaConstants.STRICT))
			parentPsvi.setField(
				-1,
				PSVInfosetImpl.VALIDITY,
				PSVInfoset.INVALID);
		if (psvi.getValidationAttempted()
			!= parentPsvi.getValidationAttempted())
			parentPsvi.setField(
				-1,
				PSVInfosetImpl.VALIDATION_ATTEMPTED,
				PSVInfoset.PARTIAL);
	}

	int addAttributePSVI(
		String namespace,
		String localName,
		AttributeDeclaration attrDecl,
		boolean strict) {
		ElementPSVInfosetImpl psvi =
			(ElementPSVInfosetImpl) getCurrentInfoset();
		int index = psvi.addAttribute(namespace, localName, attrDecl);
		if (attrDecl == null) {
			psvi.setField(
				index,
				PSVInfosetImpl.VALIDATION_ATTEMPTED,
				PSVInfoset.NONE);
			psvi.setField(index, PSVInfosetImpl.VALIDITY, PSVInfoset.NOTKNOWN);
			if (psvi.getValidationAttempted() != PSVInfoset.NONE)
				psvi.setField(
					-1,
					PSVInfosetImpl.VALIDATION_ATTEMPTED,
					PSVInfoset.PARTIAL);
			if (strict)
				psvi.setField(-1, PSVInfosetImpl.VALIDITY, PSVInfoset.INVALID);
		} else {
			psvi.setField(
				index,
				PSVInfosetImpl.VALIDATION_ATTEMPTED,
				PSVInfoset.FULL);
			psvi.setField(index, PSVInfosetImpl.VALIDITY, PSVInfoset.VALID);
			if (psvi.getValidationAttempted() != PSVInfoset.FULL)
				psvi.setField(
					-1,
					PSVInfosetImpl.VALIDATION_ATTEMPTED,
					PSVInfoset.PARTIAL);
		}
		return index;
	}

	void computeAttributeValidationInfo(int index) {
		ElementPSVInfosetImpl psvi =
			(ElementPSVInfosetImpl) getCurrentInfoset();
		if (psvi.getField(index, PSVInfosetImpl.SCHEMA_ERROR_CODES) != null) {
			psvi.setField(index, PSVInfosetImpl.VALIDITY, PSVInfoset.INVALID);
			psvi.setField(-1, PSVInfosetImpl.VALIDITY, PSVInfoset.INVALID);
		}
	}

	void addErrorCodes(int attrIndex, SchemaException ex) {
		ElementPSVInfosetImpl psvi =
			(ElementPSVInfosetImpl) getCurrentInfoset();
		psvi.addErrorCodes(attrIndex, ex);
	}

	void setMemberType(int attrIndex, Type memberType) {
		((ElementPSVInfosetImpl) getCurrentInfoset()).setField(
			attrIndex,
			PSVInfosetImpl.MEMBER_TYPE,
			memberType);
	}

	void setNormalizedValue(int attrIndex, String normalizedValue) {
		((ElementPSVInfosetImpl) getCurrentInfoset()).setField(
			attrIndex,
			PSVInfosetImpl.SCHEMA_NORMALIZED_VALUE,
			normalizedValue);
	}

	void setActualValue(int attrIndex, Object actualValue) {
		((ElementPSVInfosetImpl) getCurrentInfoset()).setField(
			attrIndex,
			PSVInfosetImpl.SCHEMA_ACTUAL_VALUE,
			actualValue);
	}

	void setNotation(NotationDeclaration notation) {
		((ElementPSVInfosetImpl) getCurrentInfoset()).setNotation(notation);
	}

	void setSchemaSpecified(int attrIndex, boolean schema) {
		if (schema)
			((ElementPSVInfosetImpl) getCurrentInfoset()).setField(
				attrIndex,
				PSVInfosetImpl.SCHEMA_SPECIFIED,
				PSVInfoset.SCHEMA);
		else
			((ElementPSVInfosetImpl) getCurrentInfoset()).setField(
				attrIndex,
				PSVInfosetImpl.SCHEMA_SPECIFIED,
				PSVInfoset.INFOSET);
	}

	void checkKeyValue(Object actualValue) throws SchemaException {
		ElementPSVInfosetImpl psvi =
			(ElementPSVInfosetImpl) getCurrentInfoset();
		psvi.checkKeyValue(actualValue);
		checkIDValue(psvi.getType(), actualValue);
	}

	void checkKeyValue(AttributeDeclaration decl, Object actualValue)
		throws SchemaException {
		((ElementPSVInfosetImpl) getCurrentInfoset()).checkKeyValue(
			decl,
			actualValue);
		checkIDValue(decl.getType(), actualValue);
	}

	// check IdentityConstraint
	void checkIdentityConstraints() throws SchemaException {
		((ElementPSVInfosetImpl) getCurrentInfoset())
			.checkIdentityConstraints();
		checkPendingIDRefs();
	}

	private void checkIDValue(Type type, Object value) throws SchemaException {
		if (type != null) {
			if (type.isDerivedFrom(manager.getIDType())) {
				pendingRefs.remove(value);
				if (!IDSet.add(value)) {
					throw idException(
						new SchemaException("cvc-id.2", (Object) value));
				}
			} else if (type.isDerivedFrom(manager.getIDREFType())) {
				if (!IDSet.contains(value))
					pendingRefs.add(value);
			} else if (type.isDerivedFrom(manager.getIDREFSType())) {
				List refList = (List)value;
				for (int i = 0; i < refList.size(); i++) {
					Object ref = refList.get(i);
					if (!IDSet.contains(ref))
						pendingRefs.add(ref);
				}
			}
		}
	}

	private void checkPendingIDRefs() throws SchemaException {
		if (level == validationContext && pendingRefs.size() > 0)
			throw idException(new SchemaException("cvc-id.1", pendingRefs));
	}

	private SchemaException idException(SchemaException se) {
		SchemaException result =
			new SchemaException("cvc-elt.7", getValidationRoot(), se);
		ElementPSVInfosetImpl rootPsvi =
			(ElementPSVInfosetImpl) getInfoset(validationContext);
		rootPsvi.addErrorCodes(-1, result);
		return result;
	}

	private SchemaException idException(List exceptions) {
		SchemaException result =
			new SchemaException("cvc-elt.7", getValidationRoot(), exceptions);
		ElementPSVInfosetImpl rootPsvi =
			(ElementPSVInfosetImpl) getInfoset(validationContext);
		rootPsvi.addErrorCodes(-1, result);
		return result;
	}

	private ElementDeclaration getValidationRoot() {
		if (validationContext == -1)
			return null;
		return (ElementDeclaration)
			((ElementPSVInfosetImpl) getInfoset(validationContext))
			.getDeclaration();
	}

	static abstract class PSVInfosetImpl implements PSVInfoset {
		protected static final int NAMESPACE = 0;
		protected static final int LOCAL_NAME = 1;
		protected static final int DECLARATION = 2;
		protected static final int SCHEMA_ERROR_CODES = 3;
		protected static final int SCHEMA_SPECIFIED = 4;
		protected static final int SCHEMA_NORMALIZED_VALUE = 5;
		protected static final int MEMBER_TYPE = 6;
		protected static final int VALIDITY = 7;
		protected static final int VALIDATION_ATTEMPTED = 8;
		protected static final int SCHEMA_ACTUAL_VALUE = 9;
		protected static final int NFIELDS = 10;
		protected static final ArrayList EMPTY;
		static {
			EMPTY = new ArrayList(NFIELDS);
			for (int i = 0; i < NFIELDS; i++)
				EMPTY.add(null);
		}

		public String getNamespaceURI() {
			return (String) getField(NAMESPACE);
		}

		public String getLocalName() {
			return (String) getField(LOCAL_NAME);
		}

		public Declaration getDeclaration() {
			return (Declaration) getField(DECLARATION);
		}

		public Type getType() {
			Declaration decl = getDeclaration();
			if (decl == null)
				return null;
			return decl.getType();
		}

		public String getNormalizedValue() {
			return (String) getField(SCHEMA_NORMALIZED_VALUE);
		}

		public Object getActualValue() {
			return getField(SCHEMA_ACTUAL_VALUE);
		}

		public List getErrorCodes() {
			return (List) getField(SCHEMA_ERROR_CODES);
		}

		public String getSpecified() {
			return (String) getField(SCHEMA_SPECIFIED);
		}

		public String getSchemaDefault() {
			Declaration decl = getDeclaration();
			if (decl == null)
				return null;
			return decl.getDefaultValue();
		}

		public SimpleType getValidatingMemberType() {
			return (SimpleType) getField(MEMBER_TYPE);
		}

		public String getValidity() {
			return (String) getField(VALIDITY);
		}

		public String getValidationAttempted() {
			return (String) getField(VALIDATION_ATTEMPTED);
		}

		public abstract ElementPSVInfoset getValidationContext();

		abstract Object getField(int field);
	}

	class ElementPSVInfosetImpl
		extends PSVInfosetImpl
		implements ElementPSVInfoset {
		private static final String RCSRevision = "$Revision: 1.1 $";
		private static final String RCSName = "$Name:  $";
		private Type elementType = null;
		private boolean nilElement = false;
		private NotationDeclaration notation = null;
		private ArrayList fields;
		private int process = SchemaConstants.STRICT;
		private HashMap bindingsTable = null;
		private List references = null;
		private List filters = null;
		private List selectors = null;

		ElementPSVInfosetImpl() {
			fields = new ArrayList(5 * NFIELDS);
			fields.addAll(EMPTY);
		}

		public Type getType() {
			return elementType;
		}

		public boolean isNil() {
			return nilElement;
		}

		public int getAttributeCount() {
			return (fields.size() / NFIELDS) - 1;
		}

		public int getAttributeIndex(String namespace, String localName) {
			if (namespace != null && namespace.length() == 0)
				namespace = null;
			for (int i = 0; i < getAttributeCount(); i++) {
				String ns = (String) getField(i, NAMESPACE);
				String name = (String) getField(i, LOCAL_NAME);
				if (localName.equals(name)
					&& ((namespace == null && ns == null)
						|| (namespace != null && namespace.equals(ns))))
					return i;
			}
			return -1;
		}

		public ElementPSVInfoset getValidationContext() {
			return getInfoset(validationContext);
		}

		public PSVInfoset getAttributePSVInfoset(int index) {
			if (index < 0)
				return null;
			return new ElementPSVInfosetImpl.AttributePSVInfosetImpl(index);
		}

		public Collection getIdentityConstraintBindings() {
			return bindingsTable.values();
		}

		public NotationDeclaration getNotation() {
			return notation;
		}

		public Collection getSchemas() {
			ElementPSVInfoset root = getValidationContext();
			if (this != root)
				return null;
			return manager.getSchemas();
		}

		/* (non-Javadoc)
		 * @see org.xquark.xml.util.NamespaceContext#getNamespaceURI(java.lang.String)
		 */
		public String getNamespaceURI(String prefix) {
			return nsContext != null ? nsContext.getNamespaceURI(prefix) : null;
		}

		/* (non-Javadoc)
		 * @see org.xquark.xml.util.NamespaceContext#getNamespaceURI(java.lang.String)
		 */
		public Collection getNamespaceURIs() {
			return nsContext != null ? nsContext.getNamespaceURIs() : null;
		}

		/* (non-Javadoc)
		 * @see org.xquark.xml.util.NamespaceContext#getPrefix(java.lang.String)
		 */
		public String getPrefix(String uri) {
			return nsContext != null ? nsContext.getPrefix(uri) : null;
		}

		/* (non-Javadoc)
		 * @see org.xquark.xml.util.NamespaceContext#getPrefixes()
		 */
		public Collection getPrefixes() {
			return nsContext != null ? nsContext.getPrefixes() : null;
		}

		/* (non-Javadoc)
		 * @see org.xquark.xml.util.NamespaceContext#getPrefixes(java.lang.String)
		 */
		public Collection getPrefixes(String uri) {
			return nsContext != null ? nsContext.getPrefixes(uri) : null;
		}

		/* (non-Javadoc)
		 * @see org.xquark.xml.util.NamespaceContext#getDeclaredPrefixes()
		 */
		public List getDeclaredPrefixes() {
			return nsContext != null ? nsContext.getDeclaredPrefixes() : null;
		}

		Object getField(int field) {
			return getField(-1, field);
		}

		void initialize(String namespace, String localName) {
			elementType = null;
			nilElement = false;
			fields.clear();
			fields.addAll(EMPTY);
			setField(-1, NAMESPACE, namespace);
			setField(-1, LOCAL_NAME, localName);
			HashMap prevBindings =
				((ElementPSVInfosetImpl) getParentInfoset()).getBindingsTable();
			if (prevBindings != null)
				bindingsTable = (HashMap) prevBindings.clone();
			List prevFilters =
				((ElementPSVInfosetImpl) getParentInfoset()).getFilters();
			if (prevFilters != null) {
				Iterator it = prevFilters.iterator();
				while (it.hasNext()) {
					XPathFilter xf = (XPathFilter) it.next();
					if (xf.nextElement(namespace, localName)) {
						addFilter(xf);
						if (xf.isDone() && xf instanceof SelectorFilter) {
							IdentityConstraintBinding binding =
								((SelectorFilter) xf).getBinding();
							addSelector(
								new ElementPSVInfosetImpl.Selector(binding));
						}
					}
				}
			}
		}

		void cleanup() {
			if (filters != null) {
				Iterator it = filters.iterator();
				while (it.hasNext()) {
					XPathFilter xf = (XPathFilter) it.next();
					xf.decrementIndex();
				}
			}
			HashMap prevBindings =
				((ElementPSVInfosetImpl) getParentInfoset()).getBindingsTable();
			if (prevBindings != null) {
				Iterator it = bindingsTable.entrySet().iterator();
				while (it.hasNext()) {
					Map.Entry entry = (Map.Entry) it.next();
					IdentityConstraintBinding prev =
						(IdentityConstraintBinding) prevBindings.get(
							entry.getKey());
					if (prev != null && entry.getValue() != prev) {
						prev.propagateEntries(
							(IdentityConstraintBinding) entry.getValue());
					}
				}
			}
			filters = null;
			selectors = null;
			bindingsTable = null;
			references = null;
			fields.clear();
		}

		int addAttribute(
			String namespace,
			String localName,
			AttributeDeclaration attrDecl) {
			int index = getAttributeCount();
			fields.addAll(EMPTY);
			setField(index, NAMESPACE, namespace);
			setField(index, LOCAL_NAME, localName);
			setField(index, DECLARATION, attrDecl);
			return index;
		}

		void setType(Type type) {
			elementType = type;
		}

		void setNil(boolean isNil) {
			nilElement = isNil;
		}

		void setNotation(NotationDeclaration notation) {
			this.notation = notation;
		}

		void setField(int index, int field, Object value) {
			int i = (index + 1) * NFIELDS + field;
			try {
				fields.set(i, value);
			} catch (IndexOutOfBoundsException e) {
				throw new IndexOutOfBoundsException("index = " + index);
			}
		}

		Object getField(int index, int field) {
			int i = (index + 1) * NFIELDS + field;
			try {
				return fields.get(i);
			} catch (IndexOutOfBoundsException e) {
				throw new IndexOutOfBoundsException("index = " + index);
			}
		}

		int getProcess() {
			return process;
		}

		void setProcess(int process) {
			this.process = process;
		}

		List getSelectors() {
			return selectors;
		}

		void addSelector(ElementPSVInfosetImpl.Selector selector) {
			if (selectors == null)
				selectors = new ArrayList();
			selectors.add(selector);
			Iterator it = selector.getFields().iterator();
			int index = 0;
			while (it.hasNext()) {
				Iterator it2 = ((List) it.next()).iterator();
				while (it2.hasNext()) {
					XPathExpr xpath = (XPathExpr) it2.next();
					addFilter(new FieldFilter(xpath, selector, index));
				}
				index++;
			}
		}

		List getFilters() {
			return filters;
		}

		void addFilter(XPathFilter xi) {
			if (filters == null)
				filters = new ArrayList();
			filters.add(xi);
		}

		HashMap getBindingsTable() {
			return bindingsTable;
		}

		void addIdentityConstraints(Collection constraints) {
			Iterator it = constraints.iterator();
			while (it.hasNext()) {
				IdentityConstraint constraint = (IdentityConstraint) it.next();
				if (constraint.getReferencedKey() == null) {
					IdentityConstraintBinding binding =
						addIdentityConstraintBinding(constraint);
					addBindingFilters(binding);
				}
			}
			it = constraints.iterator();
			while (it.hasNext()) {
				IdentityConstraint constraint = (IdentityConstraint) it.next();
				IdentityConstraint ref = constraint.getReferencedKey();
				if (ref != null) {
					IdentityConstraintBinding binding =
						addReferenceBinding(constraint);
					addBindingFilters(binding);
					IdentityConstraintBinding refBinding =
						getIdentityConstraintBinding(ref);
					if (refBinding == null) {
						refBinding = addIdentityConstraintBinding(ref);
					}
					binding.setReference(refBinding);
				}
			}
		}

		IdentityConstraintBinding addIdentityConstraintBinding(IdentityConstraint constraint) {
			if (bindingsTable == null)
				bindingsTable = new HashMap();
			IdentityConstraintBinding binding =
				new IdentityConstraintBinding(constraint);
			bindingsTable.put(constraint, binding);
			return binding;
		}

		IdentityConstraintBinding addReferenceBinding(IdentityConstraint constraint) {
			if (references == null)
				references = new ArrayList();
			IdentityConstraintBinding binding =
				new IdentityConstraintBinding(constraint);
			references.add(binding);
			return binding;
		}

		void addBindingFilters(IdentityConstraintBinding binding) {
			IdentityConstraint constraint = binding.getDefinition();
			List xpaths = constraint.getSelector();
			Iterator it = xpaths.iterator();
			while (it.hasNext()) {
				SelectorFilter xf =
					new SelectorFilter((XPathExpr) it.next(), binding);
				addFilter(xf);
				if (xf.isDone())
					addSelector(new ElementPSVInfosetImpl.Selector(binding));
			}
		}

		IdentityConstraintBinding getIdentityConstraintBinding(IdentityConstraint constraint) {
			if (bindingsTable == null)
				return null;
			return (IdentityConstraintBinding) bindingsTable.get(constraint);
		}

		void checkKeyValue(Object value) throws SchemaException {
			if (filters != null) {
				Iterator it = filters.iterator();
				ArrayList exceptions = null;
				while (it.hasNext()) {
					try {
						XPathFilter xf = (XPathFilter) it.next();
						if (xf.isDone())
							xf.setKeyValue(value);
					} catch (SchemaException ex) {
						if (exceptions == null)
							exceptions = new ArrayList();
						exceptions.add(ex);
					}
				}
				if (exceptions != null)
					throw new SchemaException(exceptions);
			}
		}

		void checkKeyValue(AttributeDeclaration decl, Object value)
			throws SchemaException {
			if (filters != null) {
				Iterator it = filters.iterator();
				ArrayList exceptions = null;
				String ns = decl.getNamespace();
				String localName = decl.getName();
				while (it.hasNext()) {
					try {
						XPathFilter xf = (XPathFilter) it.next();
						if (xf.nextAttribute(ns, localName))
							xf.setKeyValue(value);
					} catch (SchemaException ex) {
						if (exceptions == null)
							exceptions = new ArrayList();
						exceptions.add(ex);
					}
				}
				if (exceptions != null)
					throw new SchemaException(exceptions);
			}
		}

		void checkIdentityConstraints() throws SchemaException {
			ArrayList exceptions = null;
			// Check that element value used as field is of simple type and not nillable
			boolean nillable =
				getDeclaration() != null
					&& ((ElementDeclaration) getDeclaration()).isNillable();
			boolean complex =
				getType() != null && getType().getValueType() == null;
			if (filters != null && (nillable || complex)) {
				Iterator it = filters.iterator();
				while (it.hasNext()) {
					XPathFilter xf = (XPathFilter) it.next();
					if (xf.isDone() && xf instanceof FieldFilter) {
						IdentityConstraint constraint =
							((FieldFilter) xf)
								.getSelector()
								.getIdentityConstraint();
						if (complex) {
							if (exceptions == null)
								exceptions = new ArrayList();
							SchemaException se =
								new SchemaException(
									"cvc-identity-constraint.3",
									constraint);
							exceptions.add(
								identityConstraintException(constraint, se));
						}
						if (nillable
							&& constraint.getCategory()
								== IdentityConstraint.CATEGORY_KEY) {
							if (exceptions == null)
								exceptions = new ArrayList();
							SchemaException se =
								new SchemaException(
									"cvc-identity-constraint.4.2.3",
									constraint);
							exceptions.add(
								identityConstraintException(constraint, se));
						}
					}
				}
			}
			// Check that key sequence is complete for key constraints
			if (selectors != null) {
				Iterator it = selectors.iterator();
				while (it.hasNext()) {
					ElementPSVInfosetImpl.Selector s =
						(ElementPSVInfosetImpl.Selector) it.next();
					IdentityConstraint constraint = s.getIdentityConstraint();
					if (s.isSequenceComplete()) {
						// add new entry in identity constraint binding
						try {
							s.getBinding().registerEntry(s.getSequence());
						} catch (SchemaException se) {
							if (exceptions == null)
								exceptions = new ArrayList();
							exceptions.add(
								identityConstraintException(constraint, se));
						}
					} else if (
						constraint.getCategory()
							== IdentityConstraint.CATEGORY_KEY) {
						// All selected nodes must have a key sequence for a key constraint
						if (exceptions == null)
							exceptions = new ArrayList();
						SchemaException ke = null;
						if (s.isValid())
							ke =
								new SchemaException(
									null,
									"key sequence: " + s.getSequence());
						SchemaException se =
							new SchemaException(
								"cvc-identity-constraint.4.2.1",
								constraint,
								ke);
						exceptions.add(
							identityConstraintException(constraint, se));
					}
				}
			}
			if (references != null) {
				Iterator it = references.iterator();
				while (it.hasNext()) {
					IdentityConstraintBinding binding =
						(IdentityConstraintBinding) it.next();
					try {
						binding.checkPendingRefs();
					} catch (SchemaException se) {
						if (exceptions == null)
							exceptions = new ArrayList();
						exceptions.add(
							identityConstraintException(
								binding.getDefinition(),
								se));
					}
				}
			}
			if (exceptions != null)
				throw new SchemaException(exceptions);
		}

		void addErrorCodes(int attrIndex, SchemaException ex) {
			ArrayList list =
				(ArrayList) getField(attrIndex, SCHEMA_ERROR_CODES);
			if (list == null) {
				list = new ArrayList();
				setField(attrIndex, SCHEMA_ERROR_CODES, list);
			}
			addErrorCodes(list, ex);
		}

		private void addErrorCodes(ArrayList list, SchemaException ex) {
			list.add(ex.getErrorCode());
			List exceptions = ex.getExceptions();
			if (exceptions != null) {
				Iterator it = exceptions.iterator();
				while (it.hasNext()) {
					Exception e = (Exception) it.next();
					if (e instanceof SchemaException)
						addErrorCodes(list, (SchemaException) e);
				}
			}
		}

		SchemaException identityConstraintException(
			IdentityConstraint constraint,
			SchemaException se) {
			ElementDeclaration context = constraint.getContextDeclaration();
			SchemaException result =
				new SchemaException("cvc-elt.6", context, se);
			ElementPSVInfosetImpl contextPsvi =
				(ElementPSVInfosetImpl) getInfoset(context);
			contextPsvi.addErrorCodes(-1, result);
			return result;
		}

		SchemaException identityConstraintException(
			IdentityConstraint constraint,
			List exceptions) {
			ElementDeclaration context = constraint.getContextDeclaration();
			SchemaException result =
				new SchemaException("cvc-elt.6", context, exceptions);
			ElementPSVInfosetImpl contextPsvi =
				(ElementPSVInfosetImpl) getInfoset(context);
			contextPsvi.addErrorCodes(-1, result);
			return result;
		}

		public PSVInfoset getAttributePSVInfoset(
			String namespace,
			String localName) {
			return getAttributePSVInfoset(
				getAttributeIndex(namespace, localName));
		}

		class AttributePSVInfosetImpl extends PSVInfosetImpl {
			private static final String RCSRevision = "$Revision: 1.1 $";
			private static final String RCSName = "$Name:  $";

			private int index;

			AttributePSVInfosetImpl(int index) {
				this.index = index;
			}

			public ElementPSVInfoset getValidationContext() {
				return getInfoset(validationContext);
			}

			Object getField(int field) {
				return ElementPSVInfosetImpl.this.getField(index, field);
			}
		}

		class Selector {
			private static final String RCSRevision = "$Revision: 1.1 $";
			private static final String RCSName = "$Name:  $";
			IdentityConstraintBinding binding;
			ArrayList keySequence = null;
			boolean valid = true;

			Selector(IdentityConstraintBinding binding) {
				this.binding = binding;
			}

			IdentityConstraintBinding getBinding() {
				return binding;
			}

			IdentityConstraint getIdentityConstraint() {
				return binding.getDefinition();
			}

			void setKeyValue(Object value, int index) throws SchemaException {
				if (keySequence == null) {
					int size = getFields().size();
					keySequence = new ArrayList(size);
					for (int i = 0; i < size; i++)
						keySequence.add(null);
				}
				if (keySequence.get(index) != null) {
					valid = false;
					SchemaException se =
						new SchemaException(
							"cvc-identity-constraint.3",
							getIdentityConstraint(),
							new SchemaException(
								null,
								"key sequence: " + keySequence));
					throw identityConstraintException(
						getIdentityConstraint(),
						se);
				}
				keySequence.set(index, value);
			}

			boolean isValid() {
				return valid;
			}

			List getFields() {
				return getIdentityConstraint().getFields();
			}

			boolean isSequenceComplete() {
				if (!valid)
					return false;
				if (keySequence == null)
					return false;
				return !keySequence.contains(null);
			}

			List getSequence() {
				return keySequence;
			}
		}
	}

	static class XPathFilter {
		private static final String RCSRevision = "$Revision: 1.1 $";
		private static final String RCSName = "$Name:  $";
		int index = 0;
		XPathExpr xpath;

		XPathFilter(XPathExpr xpath) {
			this.xpath = xpath;
		}

		boolean isDone() {
			return index == xpath.getStepCount();
		}

		boolean nextElement(String namespaceURI, String localName) {
			if (isDone())
				return false;
			if (xpath.matches(namespaceURI, localName, index, false)) {
				index++;
				return true;
			}
			if (index == 0 && xpath.isAnyLevel())
				return true;
			return false;
		}

		boolean nextAttribute(String namespaceURI, String localName) {
			if (isDone())
				return false;
			if (xpath.matches(namespaceURI, localName, index, true)) {
				return true;
			}
			return false;
		}

		void decrementIndex() {
			if (index > 0)
				index--;
		}

		void setKeyValue(Object value) throws SchemaException {
		}
	}

	static class SelectorFilter extends XPathFilter {
		private static final String RCSRevision = "$Revision: 1.1 $";
		private static final String RCSName = "$Name:  $";
		IdentityConstraintBinding binding;

		SelectorFilter(XPathExpr xpath, IdentityConstraintBinding binding) {
			super(xpath);
			this.binding = binding;
		}

		IdentityConstraintBinding getBinding() {
			return binding;
		}
	}

	static class FieldFilter extends XPathFilter {
		private static final String RCSRevision = "$Revision: 1.1 $";
		private static final String RCSName = "$Name:  $";
		ElementPSVInfosetImpl.Selector selector;
		int index;

		FieldFilter(
			XPathExpr xpath,
			ElementPSVInfosetImpl.Selector selector,
			int index) {
			super(xpath);
			this.selector = selector;
			this.index = index;
		}

		ElementPSVInfosetImpl.Selector getSelector() {
			return selector;
		}

		void setKeyValue(Object value) throws SchemaException {
			selector.setKeyValue(value, index);
		}
	}

}
