/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */

package org.xquark.util;

import org.xml.sax.*;
import org.xml.sax.helpers.AttributesImpl;

/**
 * An XMLFilter implementation used to perform qualified SAX names (rawnames)
 * reconstruction from prefix mapping events, intended repository only support
 * the default values. It also performs xmlns attributes reconstruction.
 * IMPORTANT: SAX2 XMLReader
 * <code>http://xml.org/sax/features/namespaces</code> feature is
 * supposed to be set to true which is the default.
 */
public class NSPrefixFilter extends DecoratorFilter implements SAXConstants
{
    private static final String RCSRevision = "$Revision: 1.2 $";
    private static final String RCSName     = "$Name:  $";
    
    private boolean prefixFeature = false;
    private AttributesImpl attributes = null;
    
    public final static String NS_PREFIX = "xmlns";
    public final static String XML_PREFIX = "xml";
    
    public NSPrefixFilter()
    {
		super(new NSPrefixDecorator());    
    }
    
    public NSPrefixFilter(XMLReader reader)
    {
        super(reader, new NSPrefixDecorator());
    }
    
    public boolean getFeature(String name)
    throws SAXNotRecognizedException, SAXNotSupportedException
    {
        if (name.equals(SAX_PREFIX_FEATURE))
            return ((NSPrefixDecorator)decorator).isEnabled();
            // Note : try not to set this feature for repository cannot be set other to false
        else if (getParent() != null)
            return getParent().getFeature(name);
        else
        	return false;
    }
    
    public void setFeature(String name,boolean value)
    throws SAXNotRecognizedException, SAXNotSupportedException
    {
        if (name.equals(SAX_PREFIX_FEATURE))
			((NSPrefixDecorator)decorator).setEnabled(value);
        else if (getParent() != null)
            getParent().setFeature(name, value);
    }
}
