/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */

package org.xquark.util;

import java.util.*;

import org.xml.sax.*;

public class SAXLoader implements ContentHandler {
private static final String RCSRevision = "$Revision: 1.1 $"; 
private static final String RCSName     = "$Name:  $";
  
  private NamespaceContextStack contexts = new NamespaceContextStack();
  private ElementHandler currentHandler = null;
  private LinkedList handlers = new LinkedList();
  private HashMap currentMap = new HashMap();
  private Locator locator = null;
  
  public SAXLoader(ElementHandler handler) {
    this.currentHandler = handler;
  }
  
  public void setElementHandler(ElementHandler handler) {
    this.currentHandler = handler;
  }
  
  public String getPrefixMapping(String prefix) {
    return getNamespaceURI(prefix);
  }
  
  public HashMap getElementPrefixMap() {
    if (currentMap.isEmpty()) return null;
    return (HashMap)currentMap.clone();
  }
  
  public Locator getDocumentLocator() {
    return locator;
  }
  
  public final void characters(char[] ch, int start, int length)
  throws SAXException {
    currentHandler.characters(ch, start, length);
  }
  
  public void endDocument() throws SAXException {
    currentHandler = null;
  }
  
  public final void endElement(String namespaceURI, String localName, String qName)
  throws SAXException {
    contexts.popContext();
    //System.err.println("<"+namespaceURI+" "+localName+" "+qName);
    currentHandler = (ElementHandler)handlers.removeFirst();
    // localName sometimes has colon: Bug Xerces?
    int index = localName.indexOf(':');
    if (index != -1) localName = localName.substring(index+1);
    currentHandler.endElement(namespaceURI, localName);
  }
  
  public void endPrefixMapping(String prefix) throws SAXException {
  }
  
  public final void ignorableWhitespace(char[] ch, int start, int length)
  throws SAXException {
    currentHandler.ignorableWhitespace(ch, start, length);
  }
  
  public final void processingInstruction(String target, String data)
  throws SAXException {
    currentHandler.processingInstruction(target, data);
  }
  
  public void setDocumentLocator(Locator locator) {
    this.locator = locator;
  }
  
  public final void skippedEntity(String name)
  throws SAXException {
    currentHandler.skippedEntity(name);
  }
  
  public void startDocument() throws SAXException {
    contexts.reset();
    currentMap.put("xml", "http://www.w3.org/XML/1998/namespace");
  }
  
  // History :
  //    02/01/01  SAXParseException is added  
  //
  public final void startElement(String namespaceURI, String localName,
  String qName, Attributes atts)
  throws SAXException {
    handlers.addFirst(currentHandler);
    //System.err.println(">"+namespaceURI+" "+localName+" "+qName);
    currentHandler = currentHandler.startElement(namespaceURI, localName, atts);
    currentMap.clear();
    contexts.pushContext();
  }
  
  public void startPrefixMapping(String prefix, String uri) throws SAXException {
    contexts.declarePrefix(prefix, uri);
    currentMap.put(prefix, uri);
  }
  
  public NamespaceContext getNamespaceContext() {
    return contexts.getNamespaceContext();
  }
  
  public String getNamespaceURI(String prefix) {
    return contexts.getNamespaceURI(prefix);
  }
  
  public String getPrefix(String uri) {
    return contexts.getPrefix(uri);
  }
  
  public List getDeclaredPrefixes() {
    return contexts.getDeclaredPrefixes();
  }
  
  public Collection getPrefixes() {
    return contexts.getPrefixes();
  }
  
  public Collection getPrefixes(String uri) {
    return contexts.getPrefixes(uri);
  }
}
