/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */

package org.xquark.util.misc;

import java.util.HashSet;
import java.util.Iterator;

/**
 * For package debugging only, will not be included in distribution.
 */
public final class Debug
{


	private static final String RCSRevision = "$Revision: 1.1 $"; 
	private static final String RCSName     = "$Name:  $";

        public static boolean ACTIVATE = true;

	/**
	  * Set this variable to true for debuging.
	  */
	public static boolean ON = false;


	/**
	 * List of classes running debug mode.
	 */
	public static HashSet debugClassSet = new HashSet();


	/**
	 * Set of packages to exclude
	 */
	public static HashSet excludedPackageSet = new HashSet();

	/**
	 * Put a class in the debug list.
	 *
	 * @deprecated
	 *
	 */
	public static void debug(String classname)
	{
		Debug.debugClassSet.add(classname);
	}


	/**
	 * Put a class in the debug list.
	 */
	public static void addDebugClass(String classname)
	{
		Debug.debugClassSet.add(classname);
	}


	/**
	 * Adds a package to the exclusion list.
	 *
	 */
	public static void addExcludedPackage(String packagename)
	{
		Debug.excludedPackageSet.add(packagename);
	}


	/**
	 * Remove a class from the debug list.
	 *
	 * @deprecated
	 */
	public static void undebug(String classname)
	{
		Debug.debugClassSet.remove(classname);
	}


	/**
	 * Remove a class from the debug list.
	 *
	 */
	public static void removeDebugClass(String classname)
	{
		Debug.debugClassSet.remove(classname);
	}


	/**
	 * Remove a package from the exclusion list.
	 *
	 */
	public static void removeIgnoredPackage(String packagename)
	{
		Debug.excludedPackageSet.remove(packagename);
	}

	/**
	 * Print the debug message when <code>Debug.ON</code> is set to true.
	 */
	public static void print(String msg)
	{
		if (Debug.ON)
			System.out.println(msg);
	}


	/**
	 * Print the debug message when <code>Debug.ON</code> is set to true.
	 */
	public static void print(String classname, String msg)
	{
	    if  (Debug.ON) {
		if (isDebugging(classname))
		{
			System.out.println(classname+"\n"+msg + "\n");
		}
	    }
	}


	/**
	 * Print the debug message when <code>Debug.ON</code> is set to true or
	 * when the class is in the debug list.
	 */
	public static void print(Object object, String method)
	{
		if (isDebugging(object))
		{
			System.out.println(getClassName(object) + "." + method + "\n");
		}
	}

	/**
	 * Print the debug message when <code>Debug.ON</code> is set to true or
	 * when the class is in the debug list.
	 */
	public static void print(Object object, String method, String msg)
	{
		/*

		if(object.getClass().getName().startsWith("org.xquark.repository.metadata"))
			return;

		if(object.getClass().getName().startsWith("org.xquark.repository.storage"))
			return;

		*/
		if (isDebugging(object))
		{
			System.out.println(getClassName(object) + "." + method + "\n" + msg + "\n");
		}
	}


	public static void print(Object object, String method, Object obj)
	{
		/*

		if(object.getClass().getName().startsWith("org.xquark.repository.metadata"))
			return;

		if(object.getClass().getName().startsWith("org.xquark.repository.storage"))
			return;
		*/

		if (isDebugging(object))
		{
			System.out.println(getClassName(object) + "." + method + "\n" + obj.toString() + "\n");
		}
	}



	/**
	 * In the static method, since one can not pass "this" as parameter, the string
	 * containing the classname is used instead.
	 */
	private static String getClassName(Object o)
	{
		if(o instanceof String)
			return (String) o;
		else
			return o.getClass().getName();
	}


	public static void log(String msg)
	{

	}


	static StringBuffer indent = new StringBuffer();

	public static void begin(String s)
	{
		if(ON)
		{
			System.out.println(indent+">> "+s);
		}
		indent.append("  ");
	}


	public static void end(String s)
	{
		indent.setLength(indent.length()-2);
		if(ON)
		{
			System.out.println(indent+"<< "+s);
		}
	}


	private static boolean isDebugging(Object object)
	{
		return isDebugging((String)getClassName(object));
	}

	private static boolean isDebugging(String classname)
	{
	    if  (Debug.ACTIVATE) {

		// #1. Check ingored package set
		Iterator it = excludedPackageSet.iterator();
		while(it.hasNext())
		{
			String packagename = (String) it.next();
			if(classname.startsWith(packagename))
			{
				return false;
			}
		}

		// #2. Check global debug flag
		if(Debug.ON == true)
		{
			return true;
		}

		// #3. Check individual debug class set
		else
		{
			if(Debug.debugClassSet.contains(classname))
			{
				return true;
			}
			else
			{
				return false;
			}
		}
	    } else {
		return false;
	    }
	}





}

