/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */

package org.xquark.util.misc;

import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.PrintStream;

public class Log extends PrintStream
{	
	public static final byte LEVEL_0 = 0x00;
	public static final byte LEVEL_1 = 0x01;
	public static final byte LEVEL_2 = 0x02;
	public static final byte LEVEL_3 = 0x03;
	
	
	public static final byte NO_LOG = 0x00;
	public static final byte NORMAL = 0x01;
	public static final byte VERBOSE = 0x02;
	public static final byte DEBUG = 0x03;
	
	
	public static final byte LOWEST = LEVEL_0;
	public static final byte HIGHEST = LEVEL_3;
	
	private String filename;
	
	private byte _log_level = LEVEL_1;
	
	/**
	 * Create a loger that logs to System.out
	 */
	public Log()
	{
		super(System.out, true);
	}
	
	
	/**
	 * Create a loger that logs to the file
	 */
	public Log(String filename)
		throws FileNotFoundException
	{
		super(new FileOutputStream(filename), true);
		this.filename = filename;
	}
	
	
	public void setLogLevel(byte level)
	{
		if(level < LOWEST || level > HIGHEST)
		{
			return;	
		}
		this._log_level = level;
	}
	
	/**
	 * 
	 */
	public void print(byte level, Object o)
	{
		if(level >= _log_level)
		{
			super.print(o);
		}
	}
	
	
	public void println(byte level, Object o)
	{
		if(level >= _log_level)
		{
			super.println(o);
		}
	}
	
	
	/**
	 * 
	 */
	public void print(byte level, String s)
	{
		if(level >= _log_level)
		{
			super.print(s);
		}
	}
	
	public void println(byte level, String s)
	{
		if(level >= _log_level)
		{
			super.println(s);
		}
	}
}
