/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */


package  org.xquark.xquery.metadata;

import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;

import org.xml.sax.SAXException;
import org.xml.sax.helpers.AttributesImpl;
import org.xquark.schema.Schema;
import org.xquark.schema.SchemaReader;
import org.xquark.util.DefaultXMLReader;
import org.xquark.xpath.XTreeReader;


public class MetaHandler
extends DefaultXMLReader {
	private static final String RCSRevision = "$Revision: 1.1 $";
	private static final String RCSName     = "$Name:  $";
	
	private MetaDataImpl metadata = null;
	private MetaWrapper metawrapper = null;
	private MetaCollection metacollection = null;
	private AttributesImpl atts = new AttributesImpl();
	private String label = "ipo";
	
	public MetaHandler(MetaDataImpl metadata) { this.metadata = metadata; }
	
	public MetaHandler(MetaWrapper metawrapper) { this.metawrapper = metawrapper; }
	
	public MetaHandler(MetaCollection metacollection) { this.metacollection = metacollection; }
	
	/**
	 * SystemId is ignored and the XTree passed at construction time is parsed
	 */
	public void parse(String systemId) throws SAXException, java.io.IOException {
		if (metacollection != null) { browseMetaCollection(metacollection); }
		else {
			if (metawrapper != null) { browseMetaWrapper(metawrapper); }
			else {
				if (metadata != null) { browseMetaData(metadata); }
			}
		}
	}
	
	public void browseMetaCollection(MetaCollection metacollection) throws SAXException, java.io.IOException {
		atts.addAttribute("", "name", "", "CDATA", metacollection.getCollectionName());
		contentHandler.startElement("", "collection", "", atts);
		atts.clear() ;
		XTreeReader reader = new XTreeReader(metacollection.getXTree());
		reader.setFeature(XTreeReader.FRAGMENT_FEATURE, true) ;
		reader.setContentHandler(contentHandler) ;
		reader.parse("");
		contentHandler.endElement("", "collection", "");
	}
	
	public void startPrint(String sourcename, String label) throws SAXException, java.io.IOException {
		contentHandler.startDocument() ;
		
		atts.addAttribute("xmlns", "xsi", "", "CDATA", "http://www.w3.org/2001/XMLSchema-instance");
		atts.addAttribute("xmlns", label, "", "CDATA", "http://www.xquark.org/WRAPPERMETADATA");
		atts.addAttribute("xsi", "schemaLocation", "", "CDATA", "http://www.xquark.org/XMLDBC/Metadata XMLDBCMetaData.xsd");
		atts.addAttribute("", "source", "", "CDATA", sourcename);
		contentHandler.startElement(label, "metadata", "", atts);
		
		atts.clear() ;
		
		contentHandler.startElement("", "schemas", "", atts);
		Collection schemas;
		
		if (metawrapper != null) {
			schemas = metawrapper.getMetaData().getSchemaManager().getSchemas();
		}
		else {
			schemas = metadata.getSchemaManager().getSchemas();
		}
		
		if (schemas != null) {
			for (Iterator iterator = schemas.iterator(); iterator.hasNext();) {
				Schema schema = (Schema)iterator.next();
				if (schema == null) {
					continue;
				}
				String namespace = schema.getNamespace();
				if (namespace.startsWith("http://www.w3.org")) {
					continue;
				}
				//contentHandler.startElement("", "schemadefinition", "", atts);
				SchemaReader schemaReader = new SchemaReader(null);
				schemaReader.setContentHandler(contentHandler) ;
				schemaReader.parse(schema, false);
				//contentHandler.endElement("", "schemadefinition", "");
			}
		}
		
		contentHandler.endElement("", "schemas", "");
	}
	
	private void endPrint(String label) throws SAXException, java.io.IOException {
		contentHandler.endElement(label, "metadata", "");
	}
	
	public void browseMetaWrapper(MetaWrapper metawrapper) throws SAXException, java.io.IOException {
		startPrint(metawrapper.getSourceName(), label);
		atts.clear() ;
		contentHandler.startElement("", "collections", "", atts);
		Iterator values = metawrapper.getMetaCollections().values().iterator();
		
		while (values.hasNext()) {
			MetaCollection metacollection = (MetaCollection)values.next();
			browseMetaCollection(metacollection);
		}
		
		contentHandler.endElement("", "collections", "");
		endPrint(label);
	}
	
	public void browseMetaData(MetaDataImpl metadata) throws SAXException, java.io.IOException {
		atts.clear() ;
		startPrint("ACCESSOR-SOURCE", label);
		contentHandler.startElement("", "collections", "", atts);
		
		for (Iterator e = metadata.getMetaWrappers().values().iterator(); e.hasNext();) {
			MetaWrapper wrapperi = (MetaWrapper)e.next();
			HashMap metacollections = wrapperi.getMetaCollections();
			Iterator values = metacollections.values().iterator();
			
			while (values.hasNext()) {
				MetaCollection metacollection = (MetaCollection)values.next();
				browseMetaCollection(metacollection);
			}
		}
		
		contentHandler.endElement("", "collections", "");
		endPrint(label);
	}
	
}



