/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */


package  org.xquark.xquery.metadata;

import java.util.*;

import org.xml.sax.SAXException;
import org.xquark.xquery.metadata.resolver.CollectionMetadata;
import org.xquark.xquery.metadata.resolver.SourceMetadata;

// ***************************************************************************
// * Class 'MetaWrapper'
// ***************************************************************************

public class MetaWrapper implements SourceMetadata{
    
    // **********************************************************************
    // * Constant
    // **********************************************************************
	private static final String RCSRevision = "$Revision: 1.1 $"; 
	private static final String RCSName     = "$Name:  $";
    // **********************************************************************
    // * Fields
    // **********************************************************************
    private HashMap metacollections = null;
    private String sourcename = null;
    private MetaDataImpl metadata = null ;

    
    
    // **********************************************************************
    // * Constructor
    // **********************************************************************
    /**
     * Construct the class MetaWrapper
     *
     * @param
     */
    public MetaWrapper(MetaDataImpl metadata) {
        initMetaCollections();
        this.metadata = metadata;
    }

	// use for importing this with new MetaData reference 
    public MetaWrapper(MetaDataImpl metadata, MetaWrapper oldwrapper) {
        this.metadata = metadata;
        initMetaCollections();
		HashMap hm = oldwrapper.getMetaCollections () ;
		for (Iterator it = hm.values ().iterator () ; it.hasNext () ; )
		{
			MetaCollection col = (MetaCollection) it.next () ;
			MetaCollection newmetacol = new MetaCollection (metadata, col) ;
			addMetaCollection (newmetacol) ;
		}
    }
    
    
    // **********************************************************************
    // * Methods
    // **********************************************************************
    /**
     *
     */
    public void addMetaCollection(MetaCollection metacollection) {
        // incorrect argument
        if (metacollection == null) {
            System.err.println("metacollection null") ;
            return;
        }
        // get name of collection
        String key = metacollection.getCollectionName();
        // incorrect name
        if (key == null) {
            System.err.println("incorrect key name") ;
            return;
        }
        // initialize collections if necessary
        if (metacollections == null) {
            initMetaCollections();
        }
        // verify collection of same name not present
        if (metacollections.get(key) != null) {
            return;
        }
        // add collection
        metacollections.put(key, metacollection);
    }
    
    
    /**
     *
     */
    public void initMetaCollections() {
        metacollections = new HashMap();
    }
    
    
    /**
     *
     */
    public String getSourceName() {
        return  sourcename;
    }
    
    
    /**
     *
     */
    public void setSourceName(String sourcename) {
        this.sourcename = sourcename;
    }
    
    

    
    public Collection getCollectionsMetadata() {
        if (metacollections == null) return null;
        return  metacollections.values();
    }
    
    
    /**
     * Return the schematrees associated with this wrapper
     */
    public HashMap getSchemaTrees() throws SAXException {
        HashMap collections = new HashMap();
        Iterator it = metacollections.entrySet().iterator();
        while (it.hasNext()) {
            Map.Entry entry = (Map.Entry)it.next();
            MetaCollection col = (MetaCollection)entry.getValue();
            if (col.getXTree() != null) collections.put((String)entry.getKey(),col.getXTree());
        }
        return  collections;
    }
    
	public MetaCollection getMetaCollection(String colName) {
		if (metacollections == null) return null;
		return (MetaCollection)metacollections.get(colName);
//		Collection metacols = metacollections.values();
//		for (Iterator it = metacols.iterator();it.hasNext();) {
//			MetaCollection metacol = (MetaCollection)it.next();
//			if (metacol.getCollectionName().equals(colName))
//				return metacol;
//		}
//		return null;
	}

    public HashMap getMetaCollections() {
        return metacollections ;
    }
    
    public MetaDataImpl getMetaData() {
        return metadata ;
    }
    
    public CollectionMetadata getCollectionMetadata(String colname) {
        if (metacollections == null) return null;
        return (CollectionMetadata)metacollections.get(colname.toUpperCase());
    }
    
    public boolean isCaseSensitive() { return false; }
    
	public boolean isStrict() { return false; }

	public List getCollectionsName ()
	{
		if (metacollections == null) return null;
		List al = new ArrayList () ;
		al.addAll(metacollections.keySet());		
/*
		for (Iterator it = metacollections.values ().iterator () ; it.hasNext () ; )
		{
//			MetaCollection col = (MetaCollection) it.next () ;
//			String colname = col.getCollectionName () ;
//			if ((colname != null) && (! al.contains (colname)))
//				al.add (colname) ;
			al.add(((MetaCollection) it.next()).getCollectionName());
		}
*/
		return al ;

	}
    // **********************************************************************
    // * Destructor
    // **********************************************************************
    /**
     * Dereference allocated object which will be further be destroy
     * by the garbage collector.
     */
    protected void finalize()
    throws Throwable {
        
    }
}
