/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */


package  org.xquark.xquery.parser;

import org.xquark.xquery.parser.util.Constants;

public class BinOpANDExpression extends XQueryBooleanOperatorExpression implements Cloneable {
    
    private static final String RCSRevision = "$Revision: 1.1 $";
    private static final String RCSName     = "$Name:  $";
    
    // #############################################################################
    // VISITOR STUFF
    // #############################################################################
    
    public void accept(ParserVisitor visitor) throws XQueryException {
        visitor.visit(this);
    }
    
    // #############################################################################
    // CONSTRUCTOR STUFF
    // #############################################################################
    
    public BinOpANDExpression(XQueryExpression expression1, XQueryExpression expression2, TypeVisitor typevisitor) throws TypeException, XQueryException {
        super(expression1,expression2);
        if (typevisitor != null) accept(typevisitor);
    }
    
    // #############################################################################
    // OTHER STUFF
    // #############################################################################
    // only one level for now
	public XQueryExpression erasePart(XQueryExpression expr) throws XQueryException {
		if (this == expr)
			return null;
		if (expression1 == expr) 
			return expression2;
		if (expression2 == expr) 
			return expression1;
		XQueryExpression resexpr = expression1.erasePart(expr);
		if (resexpr != expression1)
			this.setExpression1(resexpr);
		else {
			resexpr = expression2.erasePart(expr);
			if (resexpr != expression2) 
				this.setExpression2(resexpr);
		}
		return this;
	}

    // #############################################################################
    // PRINT STUFF
    // #############################################################################
    
    public String print() {
        return expression1.pprint() + " " + Constants.NODELISTOPERATORSSTRINGS[Constants.AND_NODELISTOPERATOR] + " " + expression2.pprint();
    }
    
    // #############################################################################
    // CLONE STUFF
    // #############################################################################
    
//    public synchronized Object clone() throws CloneNotSupportedException {
//        try {
//            Object newobj = super.clone();
//            return  newobj;
//        }
//        catch (Exception e) {
//            //System.err.println(e.getMessage()) ;
//            throw new CloneNotSupportedException("Error in cloning BinOpANDExpression");
//        }
//    }
//    
//    public synchronized Object clone(HashMap mapvars) throws CloneNotSupportedException {
//        try {
//            Object newobj = super.clone(mapvars) ;
//            if (newobj == null) return null ;
//            return  newobj;
//        }
//        catch (Exception e) {
//            throw new CloneNotSupportedException("Error in cloning BinOpANDExpression");
//        }
//    }
}
