/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * Copyright (C) 2003 XQuark Group.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */

package org.xquark.xquery.parser;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Stack;

import org.xquark.xquery.parser.util.Constants;

public class CanonizeVisitor extends DefaultParserVisitor {

	private static final String RCSRevision = "$Revision: 1.2 $";
	private static final String RCSName = "$Name:  $";

	private Stack parents = null;
	private Stack FLWRParents = null;
	private XQueryExpression resultExpr = null;

	private ArrayList QDB = null;
	private XQueryExpression QMEM = null;
	private int inReturn = 0;
	private int inWhere = 0;
	private Variable inVarDef = null;

	protected TypeVisitor typeVisitor = null;

	//	public CanonizeVisitor()
	public CanonizeVisitor(TypeVisitor typeVisitor) {
		parents = new Stack();
		FLWRParents = new Stack();
		this.typeVisitor = typeVisitor;
	}

	public XQueryExpression getResult() {
		return resultExpr;
	}
	public ArrayList getQDB() {
		return QDB;
	}
	public XQueryExpression getQMEM() {
		return QMEM;
	}

	// visitors

	public void visit(Element arg) throws XQueryException {
		resultExpr = null;
		//org.xquark.xml.misc.Debug.print(this,"START CanonizeVisitor:Element ( "+arg+" )");
		parents.push(arg);
		ArrayList subExpressions = arg.getSubExpressions();
		ArrayList newSubExpressions = new ArrayList();
		XQueryExpression rexExpr = null;
		if (subExpressions != null) {
			for (int i = 0; i < subExpressions.size(); i++) {
				XQueryExpression expri = (XQueryExpression) subExpressions.get(i);
				expri.accept(this);
				while (resultExpr != null) {
					subExpressions.set(i, resultExpr);
					expri = (XQueryExpression) subExpressions.get(i);
					resultExpr = null;
					expri.accept(this);
				}
			}
		}
		parents.pop();
		//org.xquark.xml.misc.Debug.print(this,"STOP  CanonizeVisitor:Element ( "+arg+" )");
	}

	public void visit(FLWRExpression arg) throws XQueryException {
		resultExpr = null;
		//org.xquark.xml.misc.Debug.print(this,"START CanonizeVisitor:FLWRExpression ( "+arg+" )");
		int i, j, k;
		// test if it has a parent FLWR
		if (inReturn > 0 && !FLWRParents.isEmpty()) {
			FLWRExpression parentFLWRExpr = (FLWRExpression) FLWRParents.peek();
			//org.xquark.xml.misc.Debug.print(this,"parentFLWRExpr = " + parentFLWRExpr);
			// look at dependencies between variables! (LATER!!!)
			// create new variable
			Variable var = createVariable(Constants.OUTERLET_BINDINGTYPE, arg);
			parentFLWRExpr.addVariable(var);
			//org.xquark.xml.misc.Debug.print(this,"var = " + var);
			XQueryExpression retClause = parentFLWRExpr.getReturnClause();
			if (retClause == arg)
				parentFLWRExpr.setReturnClause(var);
			else {
				// substiture FLWR by var in it's parent
				HashMap map = new HashMap();
				map.put(arg, var);
				//org.xquark.xml.misc.Debug.print(this,"map = " + map);
				retClause.substitute(map, typeVisitor);
			}
			var.accept(typeVisitor);
			var.accept(this);
			//org.xquark.xml.misc.Debug.print(this,"STOP  CanonizeVisitor:FLWRExpression ( "+arg+" )");
			//org.xquark.xml.misc.Debug.print(this,"parentFLWRExpr = " + parentFLWRExpr);
			return;
		}

		parents.push(arg);
		FLWRParents.push(arg);

		XQueryExpression whereExpr = arg.getWhereClause();
		if (whereExpr != null) {
			inWhere++;
			whereExpr.accept(this);
			inWhere--;
			whereExpr = arg.getWhereClause();
		}

		XQueryExpression returnExpr = arg.getReturnClause();
		inReturn++;
		returnExpr.accept(this);
		inReturn--;
		returnExpr = arg.getReturnClause();

		//HintTree hinttree = arg.getHintTree();
		//ArrayList orderby = arg.getOrderBy();
		/*
		for (i=0;i<orderby.size();i++) {
		    XQueryExpression orderbyi = (XQueryExpression)orderby.get(i);
		    inReturn++;
		    orderbyi.accept(this);
		    inReturn--;
		}
		*/
		ArrayList variables = arg.getVariables();
		Variable tmpVar = null;
		int varsize = variables.size();
		for (i = 0; i < varsize; i++) {
			tmpVar = (Variable) variables.get(i);
			Variable tmpVarDef = inVarDef;
			inVarDef = tmpVar;
			tmpVar.accept(this);
			inVarDef = tmpVarDef;
		}

		ArrayList qdb = new ArrayList();
		ArrayList forvarlist = new ArrayList();
		ArrayList letvarlist = new ArrayList();
		// ArrayList usedvarlist = new ArrayList();
		// get variables used in return clause
		// put variables in different lists
		for (i = 0; i < variables.size(); i++) {
			tmpVar = (Variable) variables.get(i);
			if (tmpVar.getBindingType() == Constants.OUTERLET_BINDINGTYPE)
				letvarlist.add(tmpVar);
			else
				forvarlist.add(tmpVar);
		}

		// XQueryExpressionSequence tmpSeq = new XQueryExpressionSequence(usedvarlist,typeVisitor);
		XQueryExpressionSequence tmpSeq = new XQueryExpressionSequence(forvarlist, typeVisitor);
		tmpSeq.setParenthesis(true);
		//org.xquark.xml.misc.Debug.print(this,"forvarlist = " + forvarlist);
		//org.xquark.xml.misc.Debug.print(this,"orderby = " + orderby);
		//org.xquark.xml.misc.Debug.print(this,"whereExpr = " + whereExpr);
		//org.xquark.xml.misc.Debug.print(this,"tmpSeq = " + tmpSeq);
		//org.xquark.xml.misc.Debug.print(this,"typeVisitor = " + typeVisitor);
		// change   23/01/2003
		//FLWRExpression tmpFLWR = new FLWRExpression(forvarlist, orderby, whereExpr, tmpSeq, typeVisitor);
		FLWRExpression tmpFLWR = new FLWRExpression(forvarlist, arg.getHintTree(), arg.getOrderBy(), whereExpr, tmpSeq, typeVisitor);
		tmpFLWR.setParentUnit(arg.getParentUnit());
		tmpVar = createVariable(Constants.LET_BINDINGTYPE, tmpFLWR);
		//org.xquark.xml.misc.Debug.print(this,"tmpVar.getExpression() = " + tmpVar.getExpression());
		qdb.add(tmpVar);
		//org.xquark.xml.misc.Debug.print(this,"qdb = " + qdb);
		XQueryExpression qmem = returnExpr;
		//org.xquark.xml.misc.Debug.print(this,"qmem = " + qmem);
		//org.xquark.xml.misc.Debug.print(this,"forvarlist = " + forvarlist);
		//org.xquark.xml.misc.Debug.print(this,"letvarlist = " + letvarlist);

		ArrayList allinnerqdb = new ArrayList();
		for (i = 0; i < letvarlist.size(); i++) {
			Variable letVarListi = (Variable) letvarlist.get(i);
			//org.xquark.xml.misc.Debug.print(this,"letVarListi = " + letVarListi);
			//org.xquark.xml.misc.Debug.print(this,"letVarListi.getExpression() = " + letVarListi.getExpression());

			FLWRExpression letVarListiExpr = (FLWRExpression) letVarListi.getExpression();
			//org.xquark.xml.misc.Debug.print(this,"letVarListiExpr = " + letVarListiExpr);

			letVarListiExpr.accept(this);

			ArrayList innerqdb = QDB;
			XQueryExpression innerqmem = QMEM;
			//org.xquark.xml.misc.Debug.print(this,"innerqdb = " + innerqdb);
			//org.xquark.xml.misc.Debug.print(this,"innerqmem = " + innerqmem);

			// put depends
			ArrayList vars = letVarListiExpr.getVariables();
			if (!vars.isEmpty()) {
				ArrayList dependids = new ArrayList();
				for (j = 0; j < vars.size(); j++) {
					Variable varj = (Variable) vars.get(j);
					if (varj.getBindingType() == Constants.FOR_BINDINGTYPE)
						dependids.add(varj);
				}
				if (letVarListiExpr.getReturnClause().getDependIDs() == null && !dependids.isEmpty())
					letVarListiExpr.getReturnClause().setDependIDs(dependids);
			}

			// add for variables from qdb into innerqmem
			for (j = 0; j < innerqdb.size(); j++) {
				Variable innerQdbVari = (Variable) innerqdb.get(j);
				//org.xquark.xml.misc.Debug.print(this,"innerQdbVari = " + innerQdbVari);
				FLWRExpression innerQdbVariExpr = (FLWRExpression) innerQdbVari.getExpression();
				//org.xquark.xml.misc.Debug.print(this,"innerQdbVariExpr = " + innerQdbVariExpr);
				for (k = 0; k < qdb.size(); k++) {
					Variable qdbVari = (Variable) qdb.get(k);
					//org.xquark.xml.misc.Debug.print(this,"qdbVari = " + qdbVari);
					XQueryExpression qdbVariExpr = qdbVari.getExpression();
					//org.xquark.xml.misc.Debug.print(this,"qdbVariExpr = " + qdbVariExpr);
					tmpVar = createVariable(Constants.FOR_BINDINGTYPE, qdbVari);
					//org.xquark.xml.misc.Debug.print(this,"tmpVar = " + tmpVar);
					//org.xquark.xml.misc.Debug.print(this,"tmpVar.getExpression() = " + tmpVar.getExpression());
					innerQdbVariExpr.insertVariable(tmpVar);
					//org.xquark.xml.misc.Debug.print(this,"innerQdbVariExpr = " + innerQdbVariExpr);
					//org.xquark.xml.misc.Debug.print(this,"innerQdbVariExpr.getVariables() = " + innerQdbVariExpr.getVariables());
					//org.xquark.xml.misc.Debug.print(this,"innerQdbVariExpr.getWhereClause() = " + innerQdbVariExpr.getWhereClause());
					//org.xquark.xml.misc.Debug.print(this,"innerQdbVariExpr.getReturnClause() = " + innerQdbVariExpr.getReturnClause());
				}
			}
			// add innerqdb to allinnerqdb
			allinnerqdb.addAll(innerqdb);
			//org.xquark.xml.misc.Debug.print(this,"qdb = " + qdb);
			// replace in qmem variable by innerqmem
			//org.xquark.xml.misc.Debug.print(this,"qmem = " + qmem);
			//org.xquark.xml.misc.Debug.print(this,"letVarListi = " + letVarListi);
			//org.xquark.xml.misc.Debug.print(this,"innerqmem = " + innerqmem);
			// handle special case when entire return expression must be replaced
			if (qmem == letVarListi) {
				arg.setReturnClause(innerqmem);
				qmem = arg.getReturnClause();
			} else {
				ArrayList parentList = letVarListi.getParentExpression();
				//org.xquark.xml.misc.Debug.print(this,"parentList = " + parentList);
				HashMap map = new HashMap();
				map.put(letVarListi, innerqmem);
				//org.xquark.xml.misc.Debug.print(this,"map = " + map);
				for (j = 0; j < parentList.size(); j++) {
					XQueryExpression parenti = (XQueryExpression) parentList.get(j);
					parenti.substitute(map, typeVisitor);
				}
			}
		}

		// add allinnerqdb to qdb
		qdb.addAll(allinnerqdb);
		// put skolems
		//org.xquark.xml.misc.Debug.print(this,"PUT SKOLEMS");
		//org.xquark.xml.misc.Debug.print(this,"letvarlist = " + letvarlist);
		//org.xquark.xml.misc.Debug.print(this,"forvarlist = " + forvarlist);
		//org.xquark.xml.misc.Debug.print(this,"qmem = " + qmem);
		//        qmem = arg.getReturnClause();
		//        //org.xquark.xml.misc.Debug.print(this,"qmem = " + qmem);
		if (letvarlist.size() > 0) {
			//org.xquark.xml.misc.Debug.print(this,"qmem.getSkolemIDs() = " + qmem.getSkolemIDs());
			ArrayList tmpList = new ArrayList(forvarlist.size());
			for (i = 0; i < forvarlist.size(); i++) {
				Variable var = (Variable) forvarlist.get(i);
				// change   27/03/2002
				if (var.getBindingType() != Constants.LET_BINDINGTYPE)
					tmpList.add(var);
			}
			qmem.putSkolemIDs(tmpList);
			//org.xquark.xml.misc.Debug.print(this,"qmem.getSkolemIDs() = " + qmem.getSkolemIDs());
		}

		// set return values
		QDB = qdb;
		QMEM = qmem;

		// returnExpr.accept(this);
		// XQueryExpression newReturnExpr = resultExpr;

		FLWRParents.pop();
		parents.pop();

		//org.xquark.xml.misc.Debug.print(this,"STOP  CanonizeVisitor:FLWRExpression ( "+arg+" )");
	}

	public void visit(Variable arg) throws XQueryException {
		resultExpr = null;
		//org.xquark.xml.misc.Debug.print(this,"START CanonizeVisitor:Variable ( "+arg+" )");
		parents.push(arg);
		if (inReturn > 0) {
			if (inVarDef != null) {
				XQueryExpression expr = arg.getExpression();
				expr.accept(this);
			}
		}
		parents.pop();
		//org.xquark.xml.misc.Debug.print(this,"STOP  CanonizeVisitor:Variable ( "+arg+" )");
	}

	public void visit(XQueryBinaryOperatorExpression arg) throws XQueryException {
		//org.xquark.xml.misc.Debug.print(this,"START CanonizeVisitor:XQueryBinaryOperatorExpression ( "+arg+" )");
		//context.push(IN_FUNCTION);
		resultExpr = null;
		boolean change = false;
		XQueryExpression tmpExpr = arg.getExpression1();
		tmpExpr.accept(this);
		if (resultExpr != null) {
			change = true;
			arg.setExpression1(resultExpr);
		}
		tmpExpr = arg.getExpression2();
		tmpExpr.accept(this);
		if (resultExpr != null) {
			change = true;
			arg.setExpression2(resultExpr);
		}
		if (change) {
			resultExpr = arg;
		}
		//context.pop();
		//org.xquark.xml.misc.Debug.print(this,"STOP  CanonizeVisitor:XQueryBinaryOperatorExpression ( "+arg+" ) --> result = " + resultExpr);
	}
	//	public void visit(XQueryBooleanOperatorExpression arg) throws XQueryException;

	public void visit(XQueryExpression arg) throws XQueryException {
		resultExpr = null;
		//org.xquark.xml.misc.Debug.print(this,"HANDLE CanonizeVisitor:XQueryExpression ( ["+arg.getClass().getName()+"]"+arg+" )");
	}

	public void visit(XQueryExpressionSequence arg) throws XQueryException {
		resultExpr = null;
		//org.xquark.xml.misc.Debug.print(this,"START CanonizeVisitor:XQueryExpressionSequence ( "+arg+" )");
		parents.push(arg);
		ArrayList subExpressions = arg.getSubExpressions();
		ArrayList newSubExpressions = new ArrayList();
		XQueryExpression rexExpr = null;
		if (subExpressions != null) {
			for (int i = 0; i < subExpressions.size(); i++) {
				XQueryExpression expri = (XQueryExpression) subExpressions.get(i);
				expri.accept(this);
				while (resultExpr != null) {
					subExpressions.set(i, resultExpr);
					expri = (XQueryExpression) subExpressions.get(i);
					resultExpr = null;
					expri.accept(this);
				}
			}
		}
		//org.xquark.xml.misc.Debug.print(this,"subExpressions = " + subExpressions);
		//        //org.xquark.xml.misc.Debug.print(this,"newSubExpressions = " + newSubExpressions);

		parents.pop();
		//org.xquark.xml.misc.Debug.print(this,"STOP  CanonizeVisitor:XQueryExpressionSequence ( "+arg+" )");
	}

	public void visit(XQueryUnaryOperatorExpression arg) throws XQueryException {
		//org.xquark.xml.misc.Debug.print(this,"START CanonizeVisitor:XQueryUnaryOperatorExpression ( "+arg+" )");
		//context.push(IN_FUNCTION);
		resultExpr = null;
		boolean change = false;
		XQueryExpression tmpExpr = arg.getExpression();
		tmpExpr.accept(this);
		if (resultExpr != null) {
			change = true;
			arg.setExpression(resultExpr);
		}
		if (change) {
			resultExpr = arg;
		}
		//context.pop();
		//org.xquark.xml.misc.Debug.print(this,"STOP  CanonizeVisitor:XQueryUnaryOperatorExpression ( "+arg+" ) --> result = " + resultExpr);
	}

	// private methods

	private final String VAR_NAME = "tmp_var_can";
	private int varIndex = 0;

	private Variable createVariable(int bindingType, XQueryExpression expr) throws XQueryException {
		Variable var = new Variable(new QName(null, null, VAR_NAME + (++varIndex), typeVisitor), typeVisitor);
		var.setBindingType(bindingType);
		var.setExpression(expr, typeVisitor);
		return var;
	}

}
