/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */

/*
 * CaseClauseExpression.java
 *
 * Created on 26 septembre 2002, 14:10
 */

package org.xquark.xquery.parser;


public class CaseClauseExpression extends XQueryExpression implements Cloneable {
	
	private static final String RCSRevision = "$Revision: 1.1 $";
	private static final String RCSName     = "$Name:  $";
	
	protected SequenceTypeExpression sequenceType = null;
	protected Variable variable = null;
	// cannot be null
	protected XQueryExpression returnExpression = null;
	
	// #############################################################################
	// VISITOR STUFF
	// #############################################################################
	
	public void accept(ParserVisitor visitor) throws XQueryException {
		visitor.visit(this);
	}
	
	// #############################################################################
	// CONSTRUCTOR STUFF
	// #############################################################################
	
	public CaseClauseExpression(SequenceTypeExpression sequenceType, Variable variable, XQueryExpression returnExpression, TypeVisitor typeVisitor) throws TypeException, XQueryException {
		setSequenceType(sequenceType);
		setVariable(variable);
		setReturnExpression(returnExpression);
		if (typeVisitor != null) accept(typeVisitor);
	}
	
	// #############################################################################
	// ATTRIBUTE ACCESS STUFF
	// #############################################################################
	
	public void setParentUnit(XQueryUnit parentUnit) {
		if (parentUnit == null) return;
		this.parentUnit = parentUnit;
		super.setParentUnit(parentUnit);
		sequenceType.setParentUnit(parentUnit);
		if (variable != null) variable.setParentUnit(parentUnit);
		returnExpression.setParentUnit(parentUnit);
	}
	
	public void setSequenceType(SequenceTypeExpression sequenceType) throws XQueryException {
		this.sequenceType = sequenceType;
		if (sequenceType != null) {
			this.sequenceType.setParentUnit(parentUnit);
			this.sequenceType.setParentExpression(this);
		}
	}
	
	public SequenceTypeExpression getSequenceType() { return sequenceType; }
	
	public void setVariable(Variable variable) {
		this.variable = variable;
		if (variable != null) {
			this.variable.setParentUnit(parentUnit);
			this.variable.setParentExpression(this);
		}
	}
	
	public Variable getVariable() { return variable; }
	
	public void setReturnExpression(XQueryExpression returnExpression) throws XQueryException {
		// cannot be null
		if (returnExpression == null) throw new XQueryException("ReturnExpression of a caseClause cannot be null.");
		this.returnExpression = returnExpression;
		this.returnExpression.setParentUnit(parentUnit);
		this.returnExpression.setParentExpression(this);
	}
	
	public XQueryExpression getReturnExpression() { return returnExpression; }
	
	// #############################################################################
	// PRINT STUFF
	// #############################################################################
	
	public String print() {
		StringBuffer buf = new StringBuffer();
		buf.append("case ");
		if (sequenceType != null) buf.append(sequenceType);
		if (variable != null) buf.append(" "+variable);
		buf.append(" return ");
		buf.append(returnExpression);
		return buf.toString();
	}
	
	// #############################################################################
	// CLONE STUFF
	// #############################################################################
	
//	public synchronized Object clone() throws CloneNotSupportedException {
//		try {
//			CaseClauseExpression newobj = (CaseClauseExpression)super.clone();
//			newobj.setSequenceType(getSequenceType());
//			newobj.setVariable(getVariable());
//			newobj.setReturnExpression(getReturnExpression());
//			return newobj;
//		}
//		catch (XQueryException e) {
//			throw new CloneNotSupportedException("Error in cloning Element");
//		}
//	}
//	
//	public synchronized Object clone(HashMap mapvars) throws CloneNotSupportedException {
//		try {
//			CaseClauseExpression newobj = (CaseClauseExpression)super.clone(mapvars);
//			if (newobj == null) return null;
//			newobj.setSequenceType(getSequenceType());
//			newobj.setVariable(getVariable());
//			newobj.setReturnExpression(getReturnExpression());
//			return newobj;
//		}
//		catch (XQueryException e) {
//			throw new CloneNotSupportedException("Error in cloning Element");
//		}
//	}
}
