/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */

package org.xquark.xquery.parser;

import org.xquark.xquery.parser.util.Constants;

public class CastTreatExpression extends XQueryUnaryOperatorExpression implements Cloneable {

	private static final String RCSRevision = "$Revision: 1.1 $";
	private static final String RCSName = "$Name:  $";

	// cannot be null
	// OLIVIER - W3C WD 16 August 2002
	// change QName in SequenceTypeExpression
	protected SequenceTypeExpression sequenceType = null;
	// cannot be -1
	protected int operator = -1;

	// #############################################################################
	// VISITOR STUFF
	// #############################################################################

	public void accept(ParserVisitor visitor) throws XQueryException {
		visitor.visit(this);
	}

	// #############################################################################
	// CONSTRUCTOR STUFF
	// #############################################################################

	public CastTreatExpression(int operator, SequenceTypeExpression sequenceType, XQueryExpression expression, TypeVisitor typevisitor) throws TypeException, XQueryException {
		super(expression);
		setSequenceType(sequenceType);
		setOperator(operator);
		if (typevisitor != null)
			accept(typevisitor);
	}

	// #############################################################################
	// ATTRIBUTE ACCESS STUFF
	// #############################################################################

	// get
	public SequenceTypeExpression getSequenceType() {
		return sequenceType;
	}
	public int getOperator() {
		return operator;
	}

	// set
	public void setParentUnit(XQueryUnit parentUnit) {
		if (parentUnit == null)
			return;
		this.parentUnit = parentUnit;
		sequenceType.setParentUnit(parentUnit);
		super.setParentUnit(parentUnit);
	}

	public void setSequenceType(SequenceTypeExpression sequenceType) throws XQueryException {
		// cannot be null
		if (sequenceType == null)
			throw new XQueryException("sequenceType of CastTreatExpression cannot be null");
		this.sequenceType = sequenceType;
		// set parent
		this.sequenceType.setParentExpression(this);
		this.sequenceType.setParentUnit(parentUnit);
	}

	public void setOperator(int operator) throws XQueryException {
		// cannot be -1
		if (operator == -1)
			throw new XQueryException("operator of CastTreatExpression cannot be -1");
		this.operator = operator;
	}

	// #############################################################################
	// PRINT STUFF
	// #############################################################################

	public String print() {
		StringBuffer buf = new StringBuffer();
		buf.append(Constants.TYPINGSTRINGS[operator]);
		buf.append(" as ");
		buf.append(sequenceType.pprint());
		buf.append("(");
		buf.append(expression.pprint());
		buf.append(")");
		return buf.toString();
	}

	// #############################################################################
	// CLONE STUFF
	// #############################################################################

	public void addParentExpression(XQueryExpression parentExpression) {
		addParentExpression(parentExpression);
		sequenceType.addParentExpression(parentExpression);
		expression.addParentExpression(parentExpression);
	}

//	public synchronized Object clone() throws CloneNotSupportedException {
//		try {
//			CastTreatExpression newobj = (CastTreatExpression) super.clone();
//			newobj.setSequenceType((getSequenceType() == null) ? null : (SequenceTypeExpression) getSequenceType().clone());
//			newobj.setOperator(operator);
//			return newobj;
//		}
//		catch (Exception e) {
//			//System.err.println (e.getMessage ()) ;
//			throw new CloneNotSupportedException("Error in cloning CastTreatExpression");
//		}
//	}
//
//	public synchronized Object clone(HashMap mapvars) throws CloneNotSupportedException {
//		try {
//			CastTreatExpression newobj = (CastTreatExpression) super.clone(mapvars);
//			if (newobj == null)
//				return null;
//			newobj.setSequenceType((getSequenceType() == null) ? null : (SequenceTypeExpression) getSequenceType().clone(mapvars));
//			newobj.setOperator(operator);
//			return newobj;
//		}
//		catch (Exception e) {
//			//System.err.println (e.getMessage ()) ;
//			throw new CloneNotSupportedException("Error in cloning CastTreatExpression");
//		}
//	}

}
