/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */


package  org.xquark.xquery.parser;
import org.xquark.xquery.parser.util.Constants;

public class ContextDeclaration /* extends XQueryExpression */ implements Cloneable {
    
    private static final String RCSRevision = "$Revision: 1.1 $";
    private static final String RCSName     = "$Name:  $";
    
    protected String identifier = null;
    protected String literal = null;
    int declType = -1;
    
    // #############################################################################
    // VISITOR STUFF
    // #############################################################################
    
    public void accept(ParserVisitor visitor) throws XQueryException {
        visitor.visit(this);
    }
    
    // #############################################################################
    // CONSTRUCTOR STUFF
    // #############################################################################
    
    public ContextDeclaration(String identifier, String literal, int declType, TypeVisitor typevisitor) throws XQueryException {
        setIdentifier(identifier);
        setLiteral(literal);
        setDeclType(declType);
        if (typevisitor != null) accept(typevisitor);
    }
    
    // #############################################################################
    // ATTRIBUTE ACCESS STUFF
    // #############################################################################
    
    public int getDeclTypeIdentifier() {	return declType; }
    public void setDeclType(int declType) throws XQueryException {
        if (declType != 0 && declType != 1) throw new XQueryException("declType of ContextDeclaration has incorrect value");
        this.declType = declType;
    }
    
    public String getIdentifier() { if (identifier == null) return ""; return identifier; }
    public void setIdentifier(String identifier) { this.identifier = identifier; }
    
    public String getLiteral() { return literal; }
    public void setLiteral(String literal) throws XQueryException {
        // cannot be null
        if (literal == null) throw new XQueryException("literal of ContextDeclaration cannot be null");
        this.literal = literal;
    }
    
    // #############################################################################
    // PRINT STUFF
    // #############################################################################
    
    public String toString() { return pprint(); }
    public String pprint() {
        if (declType == Constants.NAMESPACE_DECL)
            if (identifier != null) return "namespace " + identifier + " = \"" + literal + "\"";
            else return "namespace default = \"" + literal + "\"";
        else return "schema " + identifier + " \"" + literal + "\"";
    }
    
    // #############################################################################
    // CLONE STUFF
    // #############################################################################
    
    public synchronized Object clone()
    throws CloneNotSupportedException {
        try {
            ContextDeclaration newobj = (ContextDeclaration)super.clone();
            newobj.setIdentifier((getIdentifier() == null) ? null : new String(getIdentifier()));
            newobj.setLiteral((getLiteral() == null) ? null : new String(getLiteral()));
            newobj.setDeclType(getDeclTypeIdentifier());
            return  newobj;
        }
        catch (Exception e) {
            throw new CloneNotSupportedException("Error in cloning ContextDeclaration");
        }
    }

//    public synchronized Object clone(HashMap mapvars)
//    throws CloneNotSupportedException {
//        try {
//            ContextDeclaration newobj = (ContextDeclaration)super.clone();
//            if (newobj == null) return null ;
//            newobj.setIdentifier((getIdentifier() == null) ? null : new String(getIdentifier()));
//            newobj.setLiteral((getLiteral() == null) ? null : new String(getLiteral()));
//            newobj.setDeclType(getDeclTypeIdentifier());
//            return  newobj;
//        }
//        catch (Exception e) {
//            System.err.println(e.getMessage()) ;
//            throw new CloneNotSupportedException("Error in cloning ContextDeclaration");
//        }
//    }
    
}



