/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */


package org.xquark.xquery.parser;

import org.xquark.xquery.parser.primitivefunctions.fnfunctions.*;
import org.xquark.xquery.parser.primitivefunctions.myfunctions.FunctionABS;
import org.xquark.xquery.parser.primitivefunctions.xsfunctions.*;

public class DefaultParserVisitor implements ParserVisitor {
    
    private static final String RCSRevision = "$Revision: 1.3 $";
    private static final String RCSName     = "$Name:  $";
    
    public void visit(AggregateFunctionCall arg) throws XQueryException {
        visit((PrimitiveFunctionCall)arg); }
    
    public void visit(AttributeValuePair arg) throws XQueryException {
        visit((XQueryBinaryOperatorExpression)arg); }
    
    public void visit(BinOpANDExpression arg) throws XQueryException {
        visit((XQueryBooleanOperatorExpression)arg); }
    
    public void visit(BinOpORExpression arg) throws XQueryException {
        visit((XQueryBooleanOperatorExpression)arg); }
    
    public void visit(CaseClauseExpression arg) throws XQueryException {
        visit((XQueryExpression)arg); }

    public void visit(CastTreatExpression arg) throws XQueryException {
        visit((XQueryUnaryOperatorExpression)arg); }
    
    public void visit(CData arg) throws XQueryException {
        visit((XQueryExpression)arg); }
    
    public void visit(ContextDeclaration arg) throws XQueryException { }
    
    public void visit(Document arg) throws XQueryException {
        visit((XQueryExpression)arg); }

    public void visit(Element arg) throws XQueryException {
        visit((XQueryExpression)arg); }
    
    public void visit(FLWRExpression arg) throws XQueryException {
        visit((XQueryExpression)arg); }
    
    public void visit(FunctionCall arg) throws XQueryException {
        visit((XQueryExpression)arg); }
    
    public void visit(FunctionDefinition arg) throws XQueryException {
        visit((XQueryExpression)arg); }
	    
    public void visit(InstanceOfExpression arg) throws XQueryException {
        visit((XQueryUnaryOperatorExpression)arg); }
    
    public void visit(InternalFunctionCall arg) throws XQueryException {
        visit((FunctionCall)arg); }
    
    public void visit(ITEExpression arg) throws XQueryException {
        visit((XQueryExpression)arg); }
    
    public void visit(ItemType arg) throws XQueryException {
        visit((XQueryExpression)arg); }
    
    public void visit(LibraryFunctionCall arg) throws XQueryException {
        visit((FunctionCall)arg); }
    
    public void visit(ListOpAFTERExpression arg) throws XQueryException {
        visit((XQueryListBinaryOperatorExpression)arg); }
    
    public void visit(ListOpArithExpression arg) throws XQueryException {
        visit((XQueryListBinaryOperatorExpression)arg); }
    
    public void visit(ListOpBEFOREExpression arg) throws XQueryException {
        visit((XQueryListBinaryOperatorExpression)arg); }
    
    public void visit(ListOpCompExpression arg) throws XQueryException {
        visit((XQueryListBinaryOperatorExpression)arg); }
    
    public void visit(ListOpEXCEPTExpression arg) throws XQueryException {
        visit((XQueryListBinaryOperatorExpression)arg); }
    
    public void visit(ListOpINTERSECTExpression arg) throws XQueryException {
        visit((XQueryListBinaryOperatorExpression)arg); }
    
    public void visit(ListOpUNIONExpression arg) throws XQueryException {
        visit((XQueryListBinaryOperatorExpression)arg); }
    
    public void visit(LocatedExpression arg) throws XQueryException {
        visit((XQueryExpression)arg); }
    
    public void visit(NodeType arg) throws XQueryException {
        visit((XQueryExpression)arg); }
    
    public void visit(Parameter arg) throws XQueryException {
        visit((XQueryExpression)arg); }
    
    public void visit(PrimitiveFunctionCall arg) throws XQueryException {
        visit((FunctionCall)arg); }
    
    public void visit(QName arg) throws XQueryException {
        visit((XQueryExpression)arg); }
    
    public void visit(QuantifiedExpression arg) throws XQueryException {
        visit((XQueryListUnaryOperatorExpression)arg); }
    
    public void visit(RangeExpression arg) throws XQueryException {
        visit((XQueryListBinaryOperatorExpression)arg); }
    
    public void visit(RootNodeFunctionCall arg) throws XQueryException {
        visit((PrimitiveFunctionCall)arg); }
    
    public void visit(SequenceTypeExpression arg) throws XQueryException {
        visit((XQueryExpression)arg); }

    public void visit(SortedExpression arg) throws XQueryException {
        visit((XQueryListUnaryOperatorExpression)arg); }
    
    public void visit(Step arg) throws XQueryException {
        visit((XQueryUnaryOperatorExpression)arg); }
    
    public void visit(TypeSwitchExpression arg) throws XQueryException {
        visit((XQueryBinaryOperatorExpression)arg); }
    
    public void visit(UnOpMinusExpression arg) throws XQueryException {
        visit((XQueryUnaryOperatorExpression)arg); }
    
    public void visit(Value arg) throws XQueryException {
        visit((XQueryExpression)arg); }
    
    public void visit(ValueBoolean arg) throws XQueryException {
        visit((Value)arg); }
    
    public void visit(ValueDecimal arg) throws XQueryException {
        visit((Value)arg); }
    
    public void visit(ValueDouble arg) throws XQueryException {
        visit((Value)arg); }
    
    public void visit(ValueFloat arg) throws XQueryException {
            visit((Value)arg); }
    
    public void visit(ValueInteger arg) throws XQueryException {
        visit((Value)arg); }
    
    public void visit(ValueString arg) throws XQueryException {
        visit((Value)arg); }
    
    public void visit(ValueParameter arg) throws XQueryException {
        visit((Value)arg); }

    public void visit(ValueText arg) throws XQueryException {
        visit((Value)arg); }
    
    public void visit(Variable arg) throws XQueryException {
        visit((XQueryExpression)arg); }
    
    public void visit(XQueryBinaryOperatorExpression arg) throws XQueryException {
        visit((XQueryExpression)arg); }
    
    public void visit(XMLComment arg) throws XQueryException {
        visit((XQueryExpression)arg); }
    
    public void visit(XMLProcessingInstruction arg) throws XQueryException {
        visit((XQueryExpression)arg); }
    
    public void visit(XQueryBooleanOperatorExpression arg) throws XQueryException {
        visit((XQueryBinaryOperatorExpression)arg); }
    
    public void visit(XQueryExpression arg) throws XQueryException { }
    
    public void visit(XQueryExpressionSequence arg) throws XQueryException {
        visit((XQueryExpression)arg); }
    
    public void visit(XQueryFile arg) throws XQueryException {
        /*visit((XQueryFile)arg);*/ }
    
    public void visit(XQueryListBinaryOperatorExpression arg) throws XQueryException {
        visit((XQueryBinaryOperatorExpression)arg); }
    
    public void visit(XQueryListUnaryOperatorExpression arg) throws XQueryException {
        visit((XQueryUnaryOperatorExpression)arg); }
    
    public void visit(XQueryUnaryOperatorExpression arg) throws XQueryException {
        visit((XQueryExpression)arg); }
    
    public void visit(XQueryUnit arg) throws XQueryException {
        /*visit((XQueryUnit)arg);*/ }
    
    public void visit(XQueryVoid arg) throws XQueryException {
        visit((XQueryExpression)arg); }
    
    // primitive functions
    
    public void visit(FunctionABS arg) throws XQueryException {
        visit((PrimitiveFunctionCall)arg); }
    
    public void visit(FunctionAVG arg) throws XQueryException {
        visit((AggregateFunctionCall)arg); }
    
    public void visit(FunctionCEILING arg) throws XQueryException {
        visit((PrimitiveFunctionCall)arg); }
    
    public void visit(FunctionCOLLECTION arg) throws XQueryException {
        visit((RootNodeFunctionCall)arg); }
    
    public void visit(FunctionCONCAT arg) throws XQueryException {
        visit((PrimitiveFunctionCall)arg); }
    
    public void visit(FunctionCONTAINS arg) throws XQueryException {
        visit((PrimitiveFunctionCall)arg); }
    
    public void visit(FunctionCOUNT arg) throws XQueryException {
        visit((AggregateFunctionCall)arg); }
    
	public void visit(FunctionCURRENT_DATE arg) throws XQueryException {
		visit((PrimitiveFunctionCall)arg); }

	public void visit(FunctionCURRENT_DATETIME arg) throws XQueryException {
		visit((PrimitiveFunctionCall)arg); }

    public void visit(FunctionCURRENT_TIME arg) throws XQueryException {
        visit((PrimitiveFunctionCall)arg); }
    
    public void visit(FunctionDATA arg) throws XQueryException {
        visit((PrimitiveFunctionCall)arg); }
    
    public void visit(FunctionDATE arg) throws XQueryException {
        visit((PrimitiveFunctionCall)arg); }
    
    public void visit(FunctionDATETIME arg) throws XQueryException {
        visit((PrimitiveFunctionCall)arg); }
    
    public void visit(FunctionDECIMAL arg) throws XQueryException {
        visit((PrimitiveFunctionCall)arg); }
    
    public void visit(FunctionDISTINCT_VALUES arg) throws XQueryException {
        visit((PrimitiveFunctionCall)arg); }
    
    public void visit(FunctionDOC arg) throws XQueryException {
        visit((RootNodeFunctionCall)arg); }

    public void visit(FunctionDOUBLE arg) throws XQueryException {
        visit((PrimitiveFunctionCall)arg); }
    
    public void visit(FunctionEMPTY arg) throws XQueryException {
        visit((PrimitiveFunctionCall)arg); }
    
    public void visit(FunctionENDS_WITH arg) throws XQueryException {
        visit((PrimitiveFunctionCall)arg); }
    
    public void visit(FunctionEXISTS arg) throws XQueryException {
        visit((PrimitiveFunctionCall)arg); }
    
    public void visit(FunctionFALSE arg) throws XQueryException {
        visit((PrimitiveFunctionCall)arg); }
    
    public void visit(FunctionFLOOR arg) throws XQueryException {
        visit((PrimitiveFunctionCall)arg); }
    
	public void visit(FunctionGET_MONTH_FROM_DATE arg) throws XQueryException {
		visit((PrimitiveFunctionCall)arg); }

	public void visit(FunctionGET_YEAR_FROM_DATE arg) throws XQueryException {
		visit((PrimitiveFunctionCall)arg); }

    public void visit(FunctionINPUT arg) throws XQueryException {
        visit((FunctionCOLLECTION)arg); }
    
    public void visit(FunctionINTEGER arg) throws XQueryException {
        visit((PrimitiveFunctionCall)arg); }
    
	public void visit(FunctionLOCAL_NAME arg) throws XQueryException {
		visit((PrimitiveFunctionCall)arg); }

    public void visit(FunctionLOWER_CASE arg) throws XQueryException {
        visit((PrimitiveFunctionCall)arg); }
    
    public void visit(FunctionMATCHES arg) throws XQueryException {
        visit((PrimitiveFunctionCall)arg); }
    
    public void visit(FunctionMAX arg) throws XQueryException {
        visit((AggregateFunctionCall)arg); }
    
    public void visit(FunctionMIN arg) throws XQueryException {
        visit((AggregateFunctionCall)arg); }
    
	public void visit(FunctionNAMESPACE_URI arg) throws XQueryException {
		visit((PrimitiveFunctionCall)arg); }

    public void visit(FunctionNOT arg) throws XQueryException {
        visit((PrimitiveFunctionCall)arg); }
    
    public void visit(FunctionNUMBER arg) throws XQueryException {
        visit((PrimitiveFunctionCall)arg); }
        
    public void visit(FunctionSTARTS_WITH arg) throws XQueryException {
        visit((PrimitiveFunctionCall)arg); }
    
    public void visit(FunctionSTRING arg) throws XQueryException {
        visit((PrimitiveFunctionCall)arg); }
    
    public void visit(FunctionSTRING_LENGTH arg) throws XQueryException {
        visit((PrimitiveFunctionCall)arg); }
    
    public void visit(FunctionROUND arg) throws XQueryException {
        visit((PrimitiveFunctionCall)arg); }
    
    public void visit(FunctionSUBSTRING arg) throws XQueryException {
        visit((PrimitiveFunctionCall)arg); }
    
    public void visit(FunctionSUM arg) throws XQueryException {
        visit((AggregateFunctionCall)arg); }
    
    public void visit(FunctionTIME arg) throws XQueryException {
        visit((PrimitiveFunctionCall)arg); }
    
    public void visit(FunctionTRUE arg) throws XQueryException {
        visit((PrimitiveFunctionCall)arg); }
    
    public void visit(FunctionUPPER_CASE arg) throws XQueryException {
        visit((PrimitiveFunctionCall)arg); }

	public void visit(FunctionDEEP_EQUALS arg) throws XQueryException {
		visit((PrimitiveFunctionCall)arg); }
}
