/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */

package org.xquark.xquery.parser;

import java.util.ArrayList;
import java.util.HashMap;

import org.xquark.xquery.parser.util.ArrayListUtil;

public class Element extends XQueryExpression implements Cloneable {

	private static final String RCSRevision = "$Revision: 1.2 $";
	private static final String RCSName = "$Name:  $";

	// cannot be null
	protected XQueryExpression startTag = null;
	
	protected HashMap localPrefixes = null;

	// false by default
	protected boolean computed = false;

	ArrayList attributes = null; // ArrayList of AttributeValuePair
	ArrayList subExpressions = null; // ArrayList of XQueryExpression

	// #############################################################################
	// VISITOR STUFF
	// #############################################################################

	public void accept(ParserVisitor visitor) throws XQueryException {
		visitor.visit(this);
	}

	// #############################################################################
	// CONSTRUCTOR STUFF
	// #############################################################################

	public Element(XQueryExpression startTag, HashMap localPrefixes, ArrayList attributes, ArrayList subExpressions, TypeVisitor typevisitor) throws TypeException, XQueryException {
		setStartTag(startTag);
		setLocalPrefixes(localPrefixes);
		setAttributes(attributes);
		setSubExpressions(subExpressions);
		setIsComputed(false);
		if (typevisitor != null)
			accept(typevisitor);
	}

	public Element(QName startTag, HashMap localPrefixes, ArrayList attributes, ArrayList subExpressions, TypeVisitor typevisitor) throws TypeException, XQueryException {
		setStartTag(startTag);
		setLocalPrefixes(localPrefixes);
		setAttributes(attributes);
		setSubExpressions(subExpressions);
		setIsComputed(false);
		if (typevisitor != null)
			accept(typevisitor);
	}

	// #############################################################################
	// ATTRIBUTE ACCESS STUFF
	// #############################################################################

	public void setParentUnit(XQueryUnit parentUnit) {
		if (parentUnit == null)
			return;
		this.parentUnit = parentUnit;
		startTag.setParentUnit(parentUnit);
		if (attributes != null)
			for (int j = 0; j < this.attributes.size(); j++)
				 ((AttributeValuePair) this.attributes.get(j)).setParentUnit(parentUnit);
		if (subExpressions != null)
			for (int j = 0; j < subExpressions.size(); j++)
				 ((XQueryExpression) subExpressions.get(j)).setParentUnit(parentUnit);
	}

	public XQueryExpression getStartTag() {
		return startTag;
	}
	public void setStartTag(XQueryExpression startTag) throws XQueryException {
		if (startTag == null)
			throw new XQueryException("startTag of Element cannot be null");
		this.startTag = startTag;
		this.startTag.setParentExpression(this);
		this.startTag.setParentUnit(parentUnit);
	}

	public HashMap getLocalPrefixes() {
		return localPrefixes;
	}
	public void setLocalPrefixes(HashMap localPrefixes) {
		this.localPrefixes = localPrefixes;
	}

	public ArrayList getAttributes() {
		return attributes;
	}
	public void setAttributes(ArrayList attributes) {
		this.attributes = attributes;
		if (this.attributes != null)
			for (int j = 0; j < this.attributes.size(); j++) {
				AttributeValuePair attributeValuePair = (AttributeValuePair) this.attributes.get(j);
				attributeValuePair.setParentExpression(this);
				attributeValuePair.setParentUnit(parentUnit);
			}
	}

	public ArrayList getSubExpressions() {
		return subExpressions;
	}
	public void setSubExpressions(ArrayList subExpressions) {
		this.subExpressions = subExpressions;
		if (this.subExpressions != null)
			for (int j = 0; j < this.subExpressions.size(); j++) {
				XQueryExpression expression = (XQueryExpression) this.subExpressions.get(j);
				expression.setParentExpression(this);
				expression.setParentUnit(parentUnit);
			}
	}

	public boolean isComputed() {
		return computed;
	}

	public void setIsComputed(boolean computed) {
		this.computed = computed;
	}

	public void addSubExpressions(XQueryExpression subExpression) throws XQueryException {
		subExpressions.add(subExpression);
		subExpression.setParentExpression(this);
		subExpression.setParentUnit(parentUnit);
	}

	public void addSubExpressions(int index, XQueryExpression subExpression) throws XQueryException {
		subExpressions.add(index, subExpression);
		subExpression.setParentExpression(this);
		subExpression.setParentUnit(parentUnit);
	}

	public void addAttributes(AttributeValuePair attribute) throws XQueryException {
		attributes.add(attribute);
		attribute.setParentExpression(this);
		attribute.setParentUnit(parentUnit);
	}

	public void addSubExpressions(int index, AttributeValuePair attribute) throws XQueryException {
		attributes.add(index, attribute);
		attribute.setParentExpression(this);
		attribute.setParentUnit(parentUnit);
	}

	public ArrayList getSourceNames() {
		if (sourceNames != null) return sourceNames;
		if (subExpressions == null) return null;
		ArrayList tmplist = null;
		for (int i = 0; i < subExpressions.size(); i++) {
			XQueryExpression expressioni = (XQueryExpression) subExpressions.get(i);
			if (expressioni.getSourceNames() != null && !expressioni.getSourceNames().isEmpty()) {
				if (tmplist == null) 
					tmplist = (ArrayList)expressioni.getSourceNames().clone();
				else 
					ArrayListUtil.add(tmplist, expressioni.getSourceNames());
			}
		}
		return tmplist;
	}

	public ArrayList getUrls() {
		if (urls != null) return urls;
		if (subExpressions == null) return null;
		ArrayList tmplist = null;
		for (int i = 0; i < subExpressions.size(); i++) {
			XQueryExpression expressioni = (XQueryExpression) subExpressions.get(i);
			if (expressioni.getUrls() != null && !expressioni.getUrls().isEmpty()) {
				if (tmplist == null) 
					tmplist = (ArrayList)expressioni.getUrls().clone();
				else 
					ArrayListUtil.add(tmplist, expressioni.getUrls());
			}
		}
		return tmplist;
	}

	// #############################################################################
	// PRINT STUFF
	// #############################################################################

	public String print() {
		StringBuffer buf = new StringBuffer();
		if (computed) {
			buf.append("element ");
			if (!(startTag instanceof QName))
				buf.append("{ ");
			buf.append(startTag);
			if (!(startTag instanceof QName))
				buf.append(" }");
			buf.append(" {");
			if (attributes != null)
				for (int i = 0; i < attributes.size(); i++) {
					buf.append(" ");
					buf.append(((AttributeValuePair) attributes.get(i)).pprint());
					if (i < attributes.size() - 1)
						buf.append(",");
				}
			if (subExpressions != null)
				for (int j = 0; j < subExpressions.size(); j++) {
					if (j == 0 && attributes != null)
						buf.append(",");
					buf.append(" ");
					buf.append(((XQueryExpression) subExpressions.get(j)).pprint());
					if (j < subExpressions.size() - 1)
						buf.append(",");
				}
			buf.append(" }");
		} else {
			buf.append("<" + startTag.pprint());
			if (attributes != null)
				for (int i = 0; i < attributes.size(); i++)
					buf.append(" " + ((AttributeValuePair) attributes.get(i)).pprint());
			buf.append(">");
			if (subExpressions != null)
				for (int j = 0; j < subExpressions.size(); j++)
					buf.append(((XQueryExpression) subExpressions.get(j)).pprint());
			buf.append("</");
			buf.append(startTag.pprint());
			buf.append(">");
		}
		return buf.toString();
	}

	// #############################################################################
	// CLONE STUFF
	// #############################################################################
	public void addParentExpression(XQueryExpression parentExpression) {
		addParentExpression(parentExpression);

		startTag.addParentExpression(parentExpression);
		if (this.attributes != null)
			for (int j = 0; j < this.attributes.size(); j++)
				 ((AttributeValuePair) this.attributes.get(j)).addParentExpression(parentExpression);
		if (subExpressions != null)
			for (int j = 0; j < subExpressions.size(); j++)
				 ((XQueryExpression) subExpressions.get(j)).addParentExpression(parentExpression);
	}

//	public synchronized Object clone() throws CloneNotSupportedException {
//		try {
//			Element newobj = (Element) super.clone();
//			newobj.setStartTag((getStartTag() == null) ? null : (XQueryExpression) getStartTag().clone());
//			newobj.setEndTag((getEndTag() == null) ? null : (QName) getEndTag().clone());
//			newobj.setNamespaceContextStack(getNamespaceContextStack());
//			newobj.setAttributes((getAttributes() == null) ? null : XQueryExpressionTools.totalClone(getAttributes()));
//			newobj.setSubExpressions((getSubExpressions() == null) ? null : XQueryExpressionTools.totalClone(getSubExpressions()));
//			return newobj;
//		} catch (Exception e) {
//			//System.err.println(e.getMessage()) ;
//			throw new CloneNotSupportedException("Error in cloning Element");
//		}
//	}
//
//	public synchronized Object clone(HashMap mapvars) throws CloneNotSupportedException {
//		try {
//			Element newobj = (Element) super.clone(mapvars);
//			if (newobj == null)
//				return null;
//			newobj.setStartTag((getStartTag() == null) ? null : (XQueryExpression) getStartTag().clone(mapvars));
//			newobj.setEndTag((getEndTag() == null) ? null : (QName) getEndTag().clone(mapvars));
//			newobj.setNamespaceContextStack(getNamespaceContextStack());
//			newobj.setAttributes((getAttributes() == null) ? null : XQueryExpressionTools.totalClone(getAttributes(), mapvars));
//			newobj.setSubExpressions((getSubExpressions() == null) ? null : XQueryExpressionTools.totalClone(getSubExpressions(), mapvars));
//			return newobj;
//		} catch (Exception e) {
//			//System.err.println(e.getMessage()) ;
//			throw new CloneNotSupportedException("Error in cloning Element");
//		}
//	}

}
