/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */

package org.xquark.xquery.parser;

import java.util.ArrayList;

public class FunctionDefinition extends XQueryExpression implements Cloneable {

	private static final String RCSRevision = "$Revision: 1.1 $";
	private static final String RCSName = "$Name:  $";

	// cannot be null
	protected QName funcName = null;
	protected ArrayList parameters = null;
	// cannot be null
	protected SequenceTypeExpression returnType = null;
	// cannot be null or empty
	private ArrayList expressions = null;

	// #############################################################################
	// VISITOR STUFF
	// #############################################################################

	public void accept(ParserVisitor visitor) throws XQueryException {
		visitor.visit(this);
	}

	// #############################################################################
	// CONSTRUCTOR STUFF
	// #############################################################################

	public FunctionDefinition(QName name, ArrayList parameters, SequenceTypeExpression returnType, ArrayList expressions, TypeVisitor typevisitor) throws TypeException, XQueryException {
		setFuncName(name);
		setParameters(parameters);
		setReturnType(returnType);
		setExpressions(expressions);
		if (typevisitor != null)
			accept(typevisitor);
	}

	// #############################################################################
	// ATTRIBUTE ACCESS STUFF
	// #############################################################################

	public void setParentUnit(XQueryUnit parentUnit) {
		if (parentUnit == null)
			return;
		this.parentUnit = parentUnit;
		funcName.setParentUnit(parentUnit);
		if (parameters != null)
			for (int i = 0; i < parameters.size(); i++)
				 ((Parameter) parameters.get(i)).setParentUnit(parentUnit);
		returnType.setParentUnit(parentUnit);
		for (int i = 0; i < this.expressions.size(); i++)
			 ((XQueryExpression) this.expressions.get(i)).setParentUnit(parentUnit);
	}

	public QName getFuncName() {
		return funcName;
	}
	public void setFuncName(QName funcName) throws XQueryException {
		if (funcName == null || funcName.getName().equals(""))
			throw new XQueryException("funcName of FunctionDefinition cannot be null or empty string");
		this.funcName = funcName;
		funcName.setParentUnit(parentUnit);
	}

	public SequenceTypeExpression getReturnType() {
		return returnType;
	}
	public void setReturnType(SequenceTypeExpression returnType) {
		this.returnType = returnType;
		this.returnType.setParentExpression(this);
		this.returnType.setParentUnit(parentUnit);
	}

	public ArrayList getParameters() {
		return parameters;
	}

	public void setParameters(ArrayList parameters) {
		this.parameters = parameters;
		if (this.parameters != null)
			for (int i = 0; i < this.parameters.size(); i++) {
				Parameter param = (Parameter) this.parameters.get(i);
				param.setParentExpression(this);
				param.setParentUnit(parentUnit);
			}
	}

	public ArrayList getExpressions() {
		return expressions;
	}
	public void setExpressions(ArrayList expressions) throws XQueryException {
		if (expressions == null || expressions.isEmpty())
			throw new XQueryException("expressions of FunctionDefinition cannot be null or empty");
		this.expressions = expressions;
		for (int i = 0; i < this.expressions.size(); i++)
			 ((XQueryExpression) this.expressions.get(i)).setParentUnit(parentUnit);
	}

	// #############################################################################
	// PRINT STUFF
	// #############################################################################
	public void addParentExpression(XQueryExpression parentExpression) {
		addParentExpression(parentExpression);
		funcName.addParentExpression(parentExpression);
		if (parameters != null)
			for (int i = 0; i < parameters.size(); i++)
				 ((Parameter) parameters.get(i)).addParentExpression(parentExpression);
		returnType.addParentExpression(parentExpression);
		for (int i = 0; i < this.expressions.size(); i++)
			 ((XQueryExpression) this.expressions.get(i)).addParentExpression(parentExpression);
	}

	public String print() {
		StringBuffer buf = new StringBuffer();
		buf.append("define function " + funcName.print() + " (");
		if (parameters != null)
			for (int i = 0; i < parameters.size(); i++) {
				if (i != 0)
					buf.append(",");
				buf.append(((Parameter) parameters.get(i)).pprint());
			}
		buf.append(") returns ");
		buf.append(returnType.pprint());
		buf.append(" { ");
		for (int i = 0; i < this.expressions.size(); i++) {
			if (i != 0)
				buf.append(" , ");
			buf.append(((XQueryExpression) expressions.get(i)).print());
		}
		buf.append(" } ");
		return buf.toString();
	}

	// #############################################################################
	// CLONE STUFF
	// #############################################################################

//	public synchronized Object clone() throws CloneNotSupportedException {
//		try {
//			FunctionDefinition newobj = (FunctionDefinition) super.clone();
//			newobj.setFuncName((getFuncName() == null) ? null : (QName) (getFuncName()));
//			newobj.setParameters((getParameters() == null) ? null : XQueryExpressionTools.totalClone(getParameters()));
//			newobj.setReturnType((getReturnType() == null) ? null : (SequenceTypeExpression) getReturnType().clone());
//			newobj.setExpressions((getExpressions() == null) ? null : XQueryExpressionTools.totalClone(getExpressions()));
//			return newobj;
//		}
//		catch (Exception e) {
//			throw new CloneNotSupportedException("Error in cloning FunctionDefinition");
//		}
//	}
//
//	public synchronized Object clone(HashMap mapvars) throws CloneNotSupportedException {
//		try {
//			FunctionDefinition newobj = (FunctionDefinition) super.clone(mapvars);
//			if (newobj == null)
//				return null;
//			newobj.setFuncName((getFuncName() == null) ? null : (QName) (getFuncName()));
//			newobj.setParameters((getParameters() == null) ? null : XQueryExpressionTools.totalClone(getParameters(), mapvars));
//			newobj.setReturnType((getReturnType() == null) ? null : (SequenceTypeExpression) getReturnType().clone(mapvars));
//			newobj.setExpressions((getExpressions() == null) ? null : XQueryExpressionTools.totalClone(getExpressions(), mapvars));
//			return newobj;
//		}
//		catch (Exception e) {
//			//System.err.println (e.getMessage ()) ;
//			throw new CloneNotSupportedException("Error in cloning FunctionDefinition");
//		}
//	}

}
