/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */

package org.xquark.xquery.parser;

import java.util.ArrayList;

import org.xquark.xquery.parser.util.ArrayListUtil;


public class GetBoundVariablesVisitor extends DefaultParserVisitor {
    
    private static final String RCSRevision = "$Revision: 1.2 $";
    private static final String RCSName     = "$Name:  $";
    
    private ArrayList variables = null;
    private ArrayList IDs = null;
    private boolean doVariables = true;
    private boolean doIDs = true;
	    
//    public GetBoundVariablesVisitor() {
//        variables = new ArrayList() ;
//        IDs = new ArrayList() ;
//    }
    
    public GetBoundVariablesVisitor(boolean doVariables, boolean doIDs) {
        this.doVariables = doVariables;
        if (doVariables) variables = new ArrayList() ;
        this.doIDs = doIDs;
        if (doIDs) IDs = new ArrayList();
    }
    
    public ArrayList getVariables() {
        ArrayList all = new ArrayList();
        if (doVariables) all.addAll(variables);
        if (doIDs) all.addAll(IDs);
        return all ;
    }
    
    public ArrayList getUniqueVariables() {
        ArrayList all = new ArrayList();
        if (doVariables) for (int i=0;i<variables.size();i++) if (!all.contains(variables.get(i))) all.add(variables.get(i));
        if (doIDs) for (int i=0;i<IDs.size();i++) if (!all.contains(IDs.get(i))) all.add(IDs.get(i));
        return all ;
    }
    
    public ArrayList getOnlyVariables() { return variables ; }
    public ArrayList getIDsVariables() { return IDs ; }
    
    public void reset() {
        if (doVariables) variables.clear();
        if (doIDs) IDs.clear();
    }
    
    //	public void visit(AggregateFunctionCall arg) throws XQueryException
    //	public void visit(AttributeValuePair arg) throws XQueryException
    //	public void visit(BinOpANDExpression arg) throws XQueryException
    //	public void visit(BinOpORExpression arg) throws XQueryException
    //	public void visit(CastTreatExpression arg) throws XQueryException
    //	public void visit(ContextDeclaration arg) throws XQueryException
    //	public void visit(Dereference arg) throws XQueryException
    public void visit(Element arg) throws XQueryException {
        if (doIDs) {
            ArrayListUtil.add(IDs, arg.getSkolemIDs());
            ArrayListUtil.add(IDs, arg.getDependIDs());
        }
        if (arg.getStartTag() != null) {
            arg.getStartTag().accept(this);
        }
        if (arg.getAttributes() != null) {
            for (int i = 0; i < arg.getAttributes().size(); i++) {
                ((AttributeValuePair)arg.getAttributes().get(i)).accept(this);
            }
        }
        if (arg.getSubExpressions() != null) {
            for (int j = 0; j < arg.getSubExpressions().size(); j++) {
                ((XQueryExpression)arg.getSubExpressions().get(j)).accept(this);
            }
        }
    }
    
    public void visit(FLWRExpression arg) throws XQueryException {
        if (doIDs) {
            ArrayListUtil.add(IDs, arg.getSkolemIDs());
            ArrayListUtil.add(IDs, arg.getDependIDs());
        }
        arg.getReturnClause().accept(this);
    }
    
    public void visit(FunctionCall arg) throws XQueryException {
        if (doIDs) {
            ArrayListUtil.add(IDs, arg.getSkolemIDs());
            ArrayListUtil.add(IDs, arg.getDependIDs());
        }
        if (arg.getArguments() != null) {
            for (int i = 0; i < arg.getArguments().size(); i++) {
                ((XQueryExpression)arg.getArguments().get(i)).accept(this);
            }
        }
    }
    
    //	public void visit(FunctionDefinition arg) throws XQueryException
    //	public void visit(InstanceOfExpression arg) throws XQueryException
    //	public void visit(InternalFunctionCall arg) throws XQueryException
    
    public void visit(ITEExpression arg) throws XQueryException {
        if (doIDs) {
            ArrayListUtil.add(IDs, arg.getSkolemIDs());
            ArrayListUtil.add(IDs, arg.getDependIDs());
        }
        arg.getIfExpression().accept(this);
        arg.getElseExpression().accept(this);
        arg.getThenExpression().accept(this);
    }
    
    //	public void visit(LibraryFunctionCall arg) throws XQueryException
    //	public void visit(ListOpAFTERExpression arg) throws XQueryException
    //	public void visit(ListOpArithExpression arg) throws XQueryException
    //	public void visit(ListOpBEFOREExpression arg) throws XQueryException
    //	public void visit(ListOpCompExpression arg) throws XQueryException
    //	public void visit(ListOpEXCEPTExpression arg) throws XQueryException
    //	public void visit(ListOpINTERSECTExpression arg) throws XQueryException
    //	public void visit(ListOpUNIONExpression arg) throws XQueryException
    public void visit(LocatedExpression arg) throws XQueryException {
        if (doIDs) {
            ArrayListUtil.add(IDs, arg.getSkolemIDs());
            ArrayListUtil.add(IDs, arg.getDependIDs());
        }
        for (int i = 0; i < arg.getSteps().size(); i++) {
            ((Step)arg.getSteps().get(i)).accept(this);
        }
    }
    
    //	public void visit(NodeType arg) throws XQueryException
    //	public void visit(Parameter arg) throws XQueryException
    //	public void visit(PrimitiveFunctionCall arg) throws XQueryException
    //	public void visit(QName arg) throws XQueryException
    //	public void visit(QuantifiedExpression arg) throws XQueryException
    //	public void visit(RangeExpression arg) throws XQueryException
    //	public void visit(RootNodeFunctionCall arg) throws XQueryException
    //	public void visit(SortedExpression arg) throws XQueryException
    //	public void visit(TypeSwitchExpression arg) throws XQueryException
    //	public void visit(UnOpMinusExpression arg) throws XQueryException
    //	public void visit(Value arg) throws XQueryException
    //	public void visit(ValueBoolean arg) throws XQueryException
    //	public void visit(ValueFloat arg) throws XQueryException
    //	public void visit(ValueInteger arg) throws XQueryException
    //	public void visit(ValueString arg) throws XQueryException
    public void visit(Variable arg) throws XQueryException {
        if (doIDs) {
            ArrayListUtil.add(IDs, arg.getSkolemIDs());
            ArrayListUtil.add(IDs, arg.getDependIDs());
        }
        if (doVariables) ArrayListUtil.add(variables, arg);
    }
    
    public void visit(XQueryBinaryOperatorExpression arg) throws XQueryException {
        if (doIDs) {
            ArrayListUtil.add(IDs, arg.getSkolemIDs());
            ArrayListUtil.add(IDs, arg.getDependIDs());
        }
        if (arg.getExpression1() != null) {
            arg.getExpression1().accept(this);
        }
        if (arg.getExpression2() != null) {
            arg.getExpression2().accept(this);
        }
    }
    
    //	public void visit(XQueryBooleanOperatorExpression arg) throws XQueryException
    
    public void visit(XQueryExpression arg) throws XQueryException {
        if (doIDs) {
            ArrayListUtil.add(IDs, arg.getSkolemIDs());
            ArrayListUtil.add(IDs, arg.getDependIDs());
        }
        //System.out.println("Call getBoundVariables -> XQueryExpression ( " + arg.getClass().getName() + " ) : " + arg.pprint () );
    }
    
    
    public void visit(XQueryExpressionSequence arg) throws XQueryException {
        if (doIDs) {
            ArrayListUtil.add(IDs, arg.getSkolemIDs());
            ArrayListUtil.add(IDs, arg.getDependIDs());
        }
        for (int i = 0; i < arg.getSubExpressions().size(); i++) {
            ((XQueryExpression)arg.getSubExpressions().get(i)).accept(this);
        }
    }
    
    //	public void visit(XQueryListBinaryOperatorExpression arg) throws XQueryException
    //	public void visit(XQueryListUnaryOperatorExpression arg) throws XQueryException
    public void visit(XQueryUnaryOperatorExpression arg) throws XQueryException {
        if (doIDs) {
            ArrayListUtil.add(IDs, arg.getSkolemIDs());
            ArrayListUtil.add(IDs, arg.getDependIDs());
        }
        if (arg.getExpression() != null) {
            arg.getExpression().accept(this);
        }
    }
    
    public void visit(XQueryFile arg) throws XQueryException {
        for (int i = 0 ; i < arg.getUnits().size() ; i ++) {
            XQueryUnit uniti = (XQueryUnit) arg.getUnits().get(i) ;
            uniti.accept(this) ;
        }
    }
    
    public void visit(XQueryUnit arg) throws XQueryException {
        for (int i = 0 ; i < arg.getExpressions().size() ; i ++) {
            XQueryExpression expressioni = (XQueryExpression) arg.getExpressions().get(i) ;
            expressioni.accept(this) ;
        }
    }
    
    //	public void visit(XQueryVoid arg) throws XQueryException;
    
}
