/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */


package  org.xquark.xquery.parser;



public class ITEExpression extends XQueryExpression implements Cloneable {
    
    private static final String RCSRevision = "$Revision: 1.4 $";
    private static final String RCSName     = "$Name:  $";
    
    // cannot be null
    protected XQueryExpression ifExpression = null;
    // cannot be null
    protected XQueryExpression thenExpression = null;
    // cannot be null
    protected XQueryExpression elseExpression = null;
    
    // #############################################################################
    // VISITOR STUFF
    // #############################################################################
    
    public void accept(ParserVisitor visitor) throws XQueryException {
        visitor.visit(this);
    }
    
    // #############################################################################
    // CONSTRUCTOR STUFF
    // #############################################################################
    
    public ITEExpression(XQueryExpression ifExpression, XQueryExpression thenExpression, XQueryExpression elseExpression, TypeVisitor typevisitor) throws TypeException, XQueryException {
        setIfExpression(ifExpression);
        setThenExpression(thenExpression);
        setElseExpression(elseExpression);
        if (typevisitor != null) accept(typevisitor);
    }
    
    // #############################################################################
    // ATTRIBUTE ACCESS STUFF
    // #############################################################################
    
    public void setParentUnit(XQueryUnit parentUnit) {
        if (parentUnit == null) return;
        this.parentUnit = parentUnit;
        ifExpression.setParentUnit(parentUnit);
        thenExpression.setParentUnit(parentUnit);
        elseExpression.setParentUnit(parentUnit);
    }
    
    public XQueryExpression getIfExpression() { return ifExpression; }
    public void setIfExpression(XQueryExpression expression) throws XQueryException {
        // cannot be null
        if (expression == null) throw new XQueryException("ifExpression of ITEExpression cannot be null");
        this.ifExpression = expression;
        this.ifExpression.setParentUnit(parentUnit);
        this.ifExpression.setParentExpression(this);
    }
    
    public XQueryExpression getThenExpression() { return thenExpression; }
    public void setThenExpression(XQueryExpression expression) throws XQueryException {
        // cannot be null
        if (expression == null) throw new XQueryException("thenExpression of ITEExpression cannot be null");
        this.thenExpression = expression;
        this.thenExpression.setParentUnit(parentUnit);
        this.thenExpression.setParentExpression(this);
    }
    
    public XQueryExpression getElseExpression() { return elseExpression; }
    public void setElseExpression(XQueryExpression expression) throws XQueryException {
        // cannot be null
        if (expression == null) throw new XQueryException("elseExpression of ITEExpression cannot be null");
        this.elseExpression = expression;
        this.elseExpression.setParentUnit(parentUnit);
        this.elseExpression.setParentExpression(this);
    }
    
    // #############################################################################
    // PRINT STUFF
    // #############################################################################
    
    public String print() {
    	StringBuffer buf = new StringBuffer();
    	buf.append(" if ( ");
    	buf.append(ifExpression.pprint());
    	buf.append(" ) then ");
    	buf.append(thenExpression.pprint());
    	buf.append(" else ");
    	buf.append(elseExpression.pprint());
    	return buf.toString();
    }
    
    // #############################################################################
    // CLONE STUFF
    // #############################################################################
    public void addParentExpression(XQueryExpression parentExpression) {
        addParentExpression(parentExpression) ;
        ifExpression.addParentExpression(parentExpression);
        thenExpression.addParentExpression(parentExpression);
        elseExpression.addParentExpression(parentExpression);
    }
   
//    public synchronized Object clone()
//    throws CloneNotSupportedException {
//        try {
//            ITEExpression newobj = (ITEExpression)super.clone();
//            newobj.setIfExpression((getIfExpression() == null) ? null : (XQueryExpression) getIfExpression().clone());
//            newobj.setThenExpression((getThenExpression() == null) ? null : (XQueryExpression) getThenExpression().clone());
//            newobj.setElseExpression((getElseExpression() == null) ? null : (XQueryExpression) getElseExpression().clone());
//            return  newobj;
//        }
//        catch (Exception e) {
//            //System.err.println (e.getMessage ()) ;
//            throw new CloneNotSupportedException("Error in cloning ITEExpression");
//        }
//    }
//    
//    public synchronized Object clone(HashMap mapvars)
//    throws CloneNotSupportedException {
//        try {
//            ITEExpression newobj = (ITEExpression)super.clone(mapvars);
//            if (newobj == null) return null ;
//            newobj.setIfExpression((getIfExpression() == null) ? null : (XQueryExpression) getIfExpression().clone(mapvars));
//            newobj.setThenExpression((getThenExpression() == null) ? null : (XQueryExpression) getThenExpression().clone(mapvars));
//            newobj.setElseExpression((getElseExpression() == null) ? null : (XQueryExpression) getElseExpression().clone(mapvars));
//            return  newobj;
//        }
//        catch (Exception e) {
//            //System.err.println (e.getMessage ()) ;
//            throw new CloneNotSupportedException("Error in cloning ITEExpression");
//        }
//    }
    
    
}



