/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */

package org.xquark.xquery.parser;


public class InstanceOfExpression extends XQueryUnaryOperatorExpression implements Cloneable {

	private static final String RCSRevision = "$Revision: 1.1 $";
	private static final String RCSName = "$Name:  $";

	// cannot be null
	// OLIVIER - W3C WD 16 August 2002
	// change QName in SequenceTypeExpression
	protected SequenceTypeExpression sequenceType = null;

	// #############################################################################
	// VISITOR STUFF
	// #############################################################################

	public void accept(ParserVisitor visitor) throws XQueryException {
		visitor.visit(this);
	}

	// #############################################################################
	// CONSTRUCTOR STUFF
	// #############################################################################

	public InstanceOfExpression(XQueryExpression expression, SequenceTypeExpression sequenceType, TypeVisitor typevisitor) throws TypeException, XQueryException {
		super(expression);
		setSequenceType(sequenceType);
		if (typevisitor != null)
			accept(typevisitor);
	}

	// #############################################################################
	// ATTRIBUTE ACCESS STUFF
	// #############################################################################

	public void setParentUnit(XQueryUnit parentUnit) {
		if (parentUnit == null)
			return;
		this.parentUnit = parentUnit;
		sequenceType.setParentUnit(parentUnit);
		if (expression != null)
			expression.setParentUnit(parentUnit);
	}

	public SequenceTypeExpression getSequenceType() {
		return sequenceType;
	}
	public void setSequenceType(SequenceTypeExpression sequenceType) throws XQueryException {
		// cannot be null
		if (sequenceType == null)
			throw new XQueryException("sequenceType of InstanceOfExpression cannot be null");
		this.sequenceType = sequenceType;
		this.sequenceType.setParentUnit(parentUnit);
		this.sequenceType.setParentExpression(this);
	}

	// #############################################################################
	// PRINT STUFF
	// #############################################################################

	public String print() {
		return expression.pprint() + " instanceof " + sequenceType.pprint();
	}

	// #############################################################################
	// CLONE STUFF
	// #############################################################################
	public void addParentExpression(XQueryExpression parentExpression) {
		addParentExpression(parentExpression);
		sequenceType.addParentExpression(parentExpression);
		if (expression != null)
			expression.addParentExpression(parentExpression);
	}

//	public synchronized Object clone() throws CloneNotSupportedException {
//		try {
//			InstanceOfExpression newobj = (InstanceOfExpression) super.clone();
//			newobj.setSequenceType((getSequenceType() == null) ? null : (SequenceTypeExpression) getSequenceType().clone());
//			return newobj;
//		}
//		catch (Exception e) {
//			System.err.println(e.getMessage());
//			throw new CloneNotSupportedException("Error in cloning InstanceOfExpression");
//		}
//	}
//
//	public synchronized Object clone(HashMap mapvars) throws CloneNotSupportedException {
//		try {
//			InstanceOfExpression newobj = (InstanceOfExpression) super.clone(mapvars);
//			if (newobj == null)
//				return null;
//			newobj.setSequenceType((getSequenceType() == null) ? null : (SequenceTypeExpression) getSequenceType().clone(mapvars));
//			return newobj;
//		}
//		catch (Exception e) {
//			System.err.println(e.getMessage());
//			throw new CloneNotSupportedException("Error in cloning InstanceOfExpression");
//		}
//	}

}
