/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */

/*
 * ItemType.java
 *
 * Created on 27 septembre 2002, 17:58
 */

package org.xquark.xquery.parser;

import org.xquark.xquery.parser.util.Constants;

public class ItemType extends XQueryExpression implements Cloneable {
	
	private static final String RCSRevision = "$Revision: 1.1 $";
	private static final String RCSName     = "$Name:  $";
	
	protected QName typeName = null;
	protected int typeDependency = -1;
	protected XQueryExpression dependency = null;
	
	// #############################################################################
	// VISITOR STUFF
	// #############################################################################
	
	public void accept(ParserVisitor visitor) throws XQueryException {
		visitor.visit(this);
	}
	
	// #############################################################################
	// CONSTRUCTOR STUFF
	// #############################################################################
	
	public ItemType(QName typeName, int typeDependency, XQueryExpression dependency, TypeVisitor typevisitor) throws TypeException, XQueryException {
		setTypeName(typeName);
		setTypeDependency(typeDependency);
		setDependency(dependency);
		if (typevisitor != null) accept(typevisitor);
	}
	
	// #############################################################################
	// ATTRIBUTE ACCESS STUFF
	// #############################################################################
	
	public void setParentUnit(XQueryUnit parentUnit) {
		if (parentUnit == null) return;
		this.parentUnit = parentUnit;
		super.setParentUnit(parentUnit);
		if (typeName != null) typeName.setParentUnit(parentUnit);
		if (dependency != null) dependency.setParentUnit(parentUnit);
	}
	
	public void setTypeName(QName typeName) { this.typeName = typeName; }
	
	public QName getTypeName() { return typeName; }
	
	public void setTypeDependency(int typeDependency) { this.typeDependency = typeDependency; }
	
	public int getTypeDependency() { return typeDependency; }
	
	public void setDependency(XQueryExpression dependency) { this.dependency = dependency; }
	
	public XQueryExpression getDependency() { return dependency; }
	
	// #############################################################################
	// PRINT STUFF
	// #############################################################################
	
	public String print() {
		StringBuffer buf = new StringBuffer();
		if (typeName != null) {
			buf.append(typeName);
			buf.append(" ");
		}
		switch (typeDependency) {
			case Constants.IT_OF_TYPE:
				buf.append("of type ");
				break;
			case Constants.IT_IN:
				buf.append("in ");
				break;
			case Constants.IT_IN_TYPE:
				buf.append("in type ");
				break;
			default:
				break;
		}
		if (dependency != null) buf.append(dependency);
		return buf.toString();
	}
	
	// #############################################################################
	// CLONE STUFF
	// #############################################################################
	
//	public synchronized Object clone() throws CloneNotSupportedException {
//		ItemType newobj = (ItemType)super.clone();
//		newobj.setTypeName(getTypeName());
//		newobj.setTypeDependency(getTypeDependency());
//		newobj.setDependency(getDependency());
//		return newobj;
//	}
//	
//	public synchronized Object clone(HashMap mapvars) throws CloneNotSupportedException {
//		ItemType newobj = (ItemType)super.clone(mapvars);
//		if (newobj == null) return null;
//		newobj.setTypeName(getTypeName());
//		newobj.setTypeDependency(getTypeDependency());
//		newobj.setDependency(getDependency());
//		return newobj;
//	}
}
