/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */


package  org.xquark.xquery.parser;

import org.xquark.xquery.parser.util.Constants;

public class ListOpArithExpression extends XQueryListBinaryOperatorExpression implements Cloneable {
    
    private static final String RCSRevision = "$Revision: 1.1 $";
    private static final String RCSName     = "$Name:  $";
    
    // cannot be -1
    protected int operator = -1;
    
    // #############################################################################
    // VISITOR STUFF
    // #############################################################################
    
    public void accept(ParserVisitor visitor) throws XQueryException {
        visitor.visit(this);
    }
    
    // #############################################################################
    // CONSTRUCTOR STUFF
    // #############################################################################
    
    public ListOpArithExpression(XQueryExpression expression1, XQueryExpression expression2, int operator, TypeVisitor typevisitor) throws TypeException, XQueryException {
        super(expression1,expression2);
        this.operator = operator;
        if (typevisitor != null) accept(typevisitor);
    }
    
    // #############################################################################
    // ATTRIBUTE ACCESS STUFF
    // #############################################################################
    
    public int getOperator() { return operator; }
    public void setOperator(int operator) throws XQueryException {
        if (operator == -1 || operator > Constants.MAX_ARITHMETICS)
            throw new XQueryException("operator of ListOpArithExpression has incorrect value " + operator);
        this.operator = operator ;
    }
    
    // #############################################################################
    // PRINT STUFF
    // #############################################################################
    
    public String print() {
        return expression1.pprint() + " " + Constants.ARITHMETICSSTRINGS[operator] + " " + expression2.pprint();
    }
    
    // #############################################################################
    // CLONE STUFF
    // #############################################################################
    
//    public synchronized Object clone()
//    throws CloneNotSupportedException {
//        try {
//            ListOpArithExpression newobj = (ListOpArithExpression)super.clone();
//            newobj.setOperator(getOperator()) ;
//            return  newobj;
//        }
//        catch (XQueryException e) {
//            //System.err.println(e.getMessage()) ;
//            throw new CloneNotSupportedException("Error in cloning ListOpArithExpression");
//        }
//    }
//    
//    public synchronized Object clone(HashMap mapvars)
//    throws CloneNotSupportedException {
//        try {
//            ListOpArithExpression newobj = (ListOpArithExpression)super.clone(mapvars);
//            if (newobj == null) return null ;
//            newobj.setOperator(getOperator()) ;
//            return  newobj;
//        }
//        catch (XQueryException e) {
//            //System.err.println(e.getMessage()) ;
//            throw new CloneNotSupportedException("Error in cloning ListOpArithExpression");
//        }
//    }
    
    
}



