/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * Copyright (C) 2003 XQuark Group.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */

package org.xquark.xquery.parser;

import java.util.ArrayList;

import org.xquark.xpath.Axis;
import org.xquark.xpath.PathExpr;
import org.xquark.xpath.StepExpr;
import org.xquark.xquery.parser.normalize.LocatedExpressionReducer;

public class LocatedExpression extends XQueryExpression implements PathExpr, Cloneable {

	private static final String RCSRevision = "$Revision: 1.4 $";
	private static final String RCSName = "$Name:  $";

	// cannot be null or empty
	protected ArrayList steps = null;

	protected boolean inPredicate = false;

	// #############################################################################
	// VISITOR STUFF
	// #############################################################################

	public void accept(ParserVisitor visitor) throws XQueryException {
		visitor.visit(this);
	}

	// #############################################################################
	// CONSTRUCTOR STUFF
	// #############################################################################

	public LocatedExpression(ArrayList steps, TypeVisitor typevisitor) throws TypeException, XQueryException {
		this(steps, false, typevisitor);
	}

	public LocatedExpression(ArrayList steps, boolean inPredicate, TypeVisitor typevisitor) throws TypeException, XQueryException {
		init(steps,inPredicate,typevisitor,true);
	}
	public LocatedExpression(ArrayList steps, boolean inPredicate, TypeVisitor typevisitor, boolean simplify) throws TypeException, XQueryException {
		init(steps,inPredicate,typevisitor,simplify);
	}
	
	private void init(ArrayList steps, boolean inPredicate, TypeVisitor typevisitor, boolean simplify) throws TypeException, XQueryException {
		if (steps == null || steps.isEmpty())
			throw new XQueryException("steps of LocatedExpression cannot be null or empty");
		/*
		boolean hasRootVar = false;
		Step step0 = (Step) steps.get(0);
		if (step0.getExpression() instanceof Variable)
			hasRootVar = true;
		if (step0.getAxis() != -1 && step0.hasSeparator()) {
			ArrayList argVect = new ArrayList();
			argVect.add(new ValueString("*:*", typevisitor));
			FunctionCOLLECTION funcColExpr = new FunctionCOLLECTION(argVect, typevisitor);
			Step step = new Step(false, Axis.NONE, funcColExpr, null, typevisitor);
			steps.add(0, step);
		}
		else if (inPredicate && !step0.hasSeparator()) {
			if (step0.getExpression() instanceof QName || step0.getExpression() instanceof NodeType) {
				hasRootVar = true;
				if (step0.getAxis() == -1 && step0.getExpression() instanceof QName)
					step0.setAxis(Axis.CHILD);
			}
		}
		*/
		if (simplify) {
			int retValue = LocatedExpressionReducer.simplifySteps(steps,inPredicate, typevisitor) ;
			if (retValue == 0)
				throw new XQueryException("steps of LocatedExpression are not valid");
		}
		setSteps(steps);
		setInPredicate(inPredicate);
		if (typevisitor != null)
			accept(typevisitor);		
	}

	// #############################################################################
	// ATTRIBUTE ACCESS STUFF
	// #############################################################################

	public StepExpr getStep(int num) {
		if (num >= steps.size())
			return null;
		return (Step) steps.get(num);
	}
	public Step getStepNum(int num) {
		if (num >= steps.size())
			return null;
		return (Step) steps.get(num);
	}
	public XQueryExpression getExpression() {
		return ((Step)steps.get(0)).getExpression();
	}
	public ArrayList getSteps() {
		return steps;
	}
	public void setSteps(ArrayList steps) throws XQueryException {
		// cannot be null or empty
		if (steps == null || steps.isEmpty())
			throw new XQueryException("steps of LocatedExpression cannot be null or empty");
		this.steps = steps;
		for (int i = 0; i < this.steps.size(); i++) {
			Step step = (Step) this.steps.get(i);
			step.setParentExpression(this);
			step.setParentUnit(parentUnit);
		}
	}

	public void addStep(Step step) throws XQueryException {
		if (step == null)
			return;
		if (steps == null)
			steps = new ArrayList(1);
		steps.add(step);
		setSteps(steps);
	}

	public void addSteps(ArrayList steps) throws XQueryException {
		if (steps == null || steps.isEmpty())
			return;
		if (this.steps == null || this.steps.isEmpty()) {
			setSteps(steps);
			return;
		}
		this.steps.addAll(steps);
		setSteps(this.steps);
	}

	public void addStepsAt(int index, ArrayList steps) throws XQueryException {
		if (steps == null || steps.isEmpty())
			return;
		if (this.steps == null || this.steps.isEmpty()) {
			setSteps(steps);
			return;
		}
		this.steps.addAll(index, steps);
		setSteps(this.steps);
	}

	public void setParentUnit(XQueryUnit parentUnit) {
		if (parentUnit == null)
			return;
		this.parentUnit = parentUnit;
		for (int i = 0; i < steps.size(); i++)
			 ((Step) steps.get(i)).setParentUnit(parentUnit);
	}

	public boolean isRelativeExpression() {
		Step step0 = (Step)steps.get(0);
		if (step0.getAxis() == Axis.NONE && !step0.hasSeparator())
			return false;
		return true;
	}

	public boolean isRelative() {
		if (((Step) steps.get(0)).getExpression() instanceof QName || ((Step) steps.get(0)).getExpression() instanceof NodeType)
			return true;
		return false;
	}

	public void setInPredicate(boolean inPredicate) {
		this.inPredicate = inPredicate;
	}

	public boolean getInPredicate() {
		return inPredicate;
	}
	
	public boolean startsWith(Variable var) {
		return ((Step)steps.get(0)).getExpression() == var;
	}


	// #############################################################################
	// PRINT STUFF
	// #############################################################################
	
	public String print() {
		StringBuffer buf = new StringBuffer();
		//		if (expression != null)	buf.append(expression.print());
		Step step = null;
		for (int i = 0; i < steps.size(); i++) {
			step = (Step) steps.get(i);
			buf.append(step.toString());
		}
		return buf.toString();
	}

	// #############################################################################
	// CLONE STUFF
	// #############################################################################
	public void addParentExpression(XQueryExpression parentExpression) {
		addParentExpression(parentExpression);
		for (int i = 0; i < steps.size(); i++)
			 ((Step) steps.get(i)).addParentExpression(parentExpression);
	}

	// #############################################################################
	// XPATH STUFF
	// #############################################################################

	/**
	 * Returns the PathNode list of the last step.
	 *
	 */
	public ArrayList getXTrees() {
		return ((Step) this.steps.get(this.steps.size() - 1)).getXTrees();
	}

	// #############################################################################
	// RESTRUCTURATION STUFF
	// #############################################################################

	// tells whether the located expression starts with a variable
	public boolean startsWithVariable() {
		return ((Step)steps.get(0)).getExpression() instanceof Variable;
	}

	// tells whether the LocatedExpression has a parent step
	public int hasParentStep() {
		if (steps != null)
			for (int i = 0; i < steps.size(); i++)
				if (((Step) steps.get(i)).getAxis() == Axis.PARENT)
					return i;
		return -1;
	}

	// tells whether the LocatedExpression has a parent step
	public int hasUnionStep() {
		if (steps != null)
			for (int i = 0; i < steps.size(); i++)
				if (((Step) steps.get(i)).getExpression() instanceof ListOpUNIONExpression)
					return i;
		return -1;
	}

	// tells whether the LocatedExpression has predicates
	public boolean hasPredicates() {
		if (steps != null)
			for (int i = 0; i < steps.size(); i++)
				if (((Step) steps.get(i)).hasPredicates())
					return true;
		return false;
	}

	// tells whether the LocatedExpression has position predicates
	public boolean hasPositionPredicates() {
		if (steps != null)
			for (int i = 0; i < steps.size(); i++)
				if (((Step) steps.get(i)).hasPositionPredicates())
					return true;
		return false;
	}

}
