/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */


package  org.xquark.xquery.parser;

import org.xquark.xpath.NodeKind;

public class NodeType extends XQueryExpression implements Cloneable {
    
    private static final String RCSRevision = "$Revision: 1.1 $";
    private static final String RCSName     = "$Name:  $";
    
    // cannot be -1
    protected byte kind = NodeKind.NONE;
    // can be null
    String PIArg = null;
    
    // #############################################################################
    // VISITOR STUFF
    // #############################################################################
    
    public void accept(ParserVisitor visitor) throws XQueryException {
        visitor.visit(this);
    }
    
    // #############################################################################
    // CONSTRUCTOR STUFF
    // #############################################################################
    
    public NodeType(byte kind,String PIArg, TypeVisitor typevisitor) throws TypeException, XQueryException {
        setKind(kind);
        setPIArg(PIArg);
        if (typevisitor != null) accept(typevisitor);
    }
    
    // #############################################################################
    // ATTRIBUTE ACCESS STUFF
    // #############################################################################
    
    public byte getKind() { return kind; }
    public void setKind(byte kind) throws XQueryException {
        this.kind = kind  ;
    }
    
    public String getPIArg() { return PIArg; }
    public void setPIArg(String PIArg) {
        this.PIArg = PIArg  ;
    }
    
    // #############################################################################
    // PRINT STUFF
    // #############################################################################
    
    public String print() {
        if (PIArg == null) return NodeKind.NODEKINDSTRINGS[kind] + "()";
        return NodeKind.NODEKINDSTRINGS[kind] + "(" + PIArg + ")";
    }
    
    // #############################################################################
    // CLONE STUFF
    // #############################################################################
    
//    public synchronized Object clone()
//    throws CloneNotSupportedException {
//        try {
//            NodeType newobj = (NodeType)super.clone();
//            newobj.setKind(getKind()) ;
//            newobj.setPIArg((getPIArg() == null) ? null : new String(getPIArg()));
//            return  newobj;
//        }
//        catch (Exception e) {
//            //System.err.println (e.getMessage ()) ;
//            throw new CloneNotSupportedException("Error in cloning NodeType");
//        }
//    }
//    
//    public synchronized Object clone(HashMap mapvars)
//    throws CloneNotSupportedException {
//        try {
//            NodeType newobj = (NodeType)super.clone(mapvars);
//            if (newobj == null) return null ;
//            newobj.setKind(getKind()) ;
//            newobj.setPIArg((getPIArg() == null) ? null : new String(getPIArg()));
//            return  newobj;
//        }
//        catch (Exception e) {
//            //System.err.println (e.getMessage ()) ;
//            throw new CloneNotSupportedException("Error in cloning NodeType");
//        }
//    }
    
}



