/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */

package org.xquark.xquery.parser;


public class Parameter extends XQueryExpression implements Cloneable {

	private static final String RCSRevision = "$Revision: 1.1 $";
	private static final String RCSName = "$Name:  $";

	// cannot be null
	// OLIVIER - W3C WD 16 August 2002
	// change QName in SequenceType
	protected SequenceTypeExpression sequenceType;
	// cannot be null
	protected Variable variable;

	// #############################################################################
	// VISITOR STUFF
	// #############################################################################

	public void accept(ParserVisitor visitor) throws XQueryException {
		visitor.visit(this);
	}

	// #############################################################################
	// CONSTRUCTOR STUFF
	// #############################################################################

	public Parameter(SequenceTypeExpression sequenceType, Variable variable, TypeVisitor typevisitor) throws TypeException, XQueryException {
		setSequenceType(sequenceType);
		setVariable(variable);
		if (typevisitor != null)
			accept(typevisitor);
	}

	// #############################################################################
	// ATTRIBUTE ACCESS STUFF
	// #############################################################################

	public void setParentUnit(XQueryUnit parentUnit) {
		if (parentUnit == null)
			return;
		this.parentUnit = parentUnit;
		if (sequenceType != null)
			sequenceType.setParentUnit(parentUnit);
		variable.setParentUnit(parentUnit);
	}

	public SequenceTypeExpression getSequenceType() {
		return sequenceType;
	}
	public void setSequenceType(SequenceTypeExpression sequenceType) throws XQueryException {
		this.sequenceType = sequenceType;
		if (sequenceType != null) {
			this.sequenceType.setParentUnit(parentUnit);
			this.sequenceType.setParentExpression(this);
		}
	}

	public Variable getVariable() {
		return variable;
	}
	public void setVariable(Variable variable) throws XQueryException {
		if (variable == null)
			throw new XQueryException("sequenceType of Parameter cannot be null");
		this.variable = variable;
		this.variable.setParentUnit(parentUnit);
		this.variable.setParentExpression(this);
	}

	// #############################################################################
	// PRINT STUFF
	// #############################################################################

	public String print() {
		if (sequenceType != null)
			return sequenceType.pprint() + " " + variable.pprint();
		else
			return variable.pprint();
	}

	// #############################################################################
	// CLONE STUFF
	// #############################################################################
	public void addParentExpression(XQueryExpression parentExpression) {
		addParentExpression(parentExpression);
		if (sequenceType != null)
			sequenceType.addParentExpression(parentExpression);
		variable.addParentExpression(parentExpression);
	}

//	public synchronized Object clone() throws CloneNotSupportedException {
//		try {
//			Parameter newobj = (Parameter) super.clone();
//			newobj.setSequenceType((getSequenceType() == null) ? null : (SequenceTypeExpression) getSequenceType().clone());
//			newobj.setVariable((getVariable() == null) ? null : (Variable) getVariable().clone());
//			return newobj;
//		}
//		catch (Exception e) {
//			//System.err.println(e.getMessage()) ;
//			throw new CloneNotSupportedException("Error in cloning Parameter");
//		}
//	}
//
//	public synchronized Object clone(HashMap mapvars) throws CloneNotSupportedException {
//		try {
//			Parameter newobj = (Parameter) super.clone(mapvars);
//			if (newobj == null)
//				return null;
//			newobj.setSequenceType((getSequenceType() == null) ? null : (SequenceTypeExpression) getSequenceType().clone(mapvars));
//			newobj.setVariable((getVariable() == null) ? null : (Variable) getVariable().clone(mapvars));
//			return newobj;
//		}
//		catch (Exception e) {
//			//System.err.println(e.getMessage()) ;
//			throw new CloneNotSupportedException("Error in cloning Parameter");
//		}
//	}

}
