/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */

package org.xquark.xquery.parser;

import java.util.ArrayList;
import java.util.HashMap;

import org.xquark.util.NamespaceContextStack;
import org.xquark.xquery.parser.hinttree.HintTree;

public interface ParserFactoryInterfaceMediator {

	AttributeValuePair createAttributeValuePair(XQueryExpression attributeName, XQueryExpression attributeValue, TypeVisitor typevisitor) throws TypeException, XQueryException;

	BinOpANDExpression createBinOpANDExpression(XQueryExpression mainExpression, XQueryExpression expression, TypeVisitor typevisitor) throws TypeException, XQueryException;

	BinOpORExpression createBinOpORExpression(XQueryExpression mainExpression, XQueryExpression expression, TypeVisitor typevisitor) throws TypeException, XQueryException;

	CData createCData(String data, TypeVisitor typevisitor) throws TypeException, XQueryException;

	CaseClauseExpression createCaseClauseExpression(SequenceTypeExpression sequenceType, Variable variable, XQueryExpression returnExpression, TypeVisitor typeVisitor) throws TypeException, XQueryException;

	CastTreatExpression createCastTreatExpression(int kind, SequenceTypeExpression sequenceType, XQueryExpression expression, TypeVisitor typevisitor) throws TypeException, XQueryException;

	ContextDeclaration createContextDeclaration(String identifier, String value, int declType, TypeVisitor typevisitor) throws TypeException, XQueryException;

	Document createDocument(XQueryExpression element, TypeVisitor typevisitor) throws TypeException, XQueryException;

	Element createElement(XQueryExpression startTag, HashMap localPrefixes, ArrayList vectAttributes, ArrayList vectExpressions, TypeVisitor typevisitor) throws TypeException, XQueryException;

	FLWRExpression createFLWRExpression(ArrayList bindings, HintTree hintClause, XQueryExpression whereClause, ArrayList orderbys, XQueryExpression returnClause, TypeVisitor typevisitor) throws TypeException, XQueryException;

	FLWRExpression createFLWRExpression(ArrayList bindings, XQueryExpression whereClause, ArrayList orderbys, XQueryExpression returnClause, TypeVisitor typevisitor) throws TypeException, XQueryException;

	FLWRExpression createFLWRExpression(ArrayList bindings, XQueryExpression whereClause, XQueryExpression returnClause, TypeVisitor typevisitor) throws TypeException, XQueryException;

	FunctionDefinition createFunctionDefinition(QName funcName, ArrayList tmpVect, SequenceTypeExpression sequenceType, ArrayList vectExpression, TypeVisitor typevisitor) throws TypeException, XQueryException;
	
	HintTree createHintTree(int type, int side, Object left, Object right);

	ITEExpression createITEExpression(XQueryExpression expressionIf, XQueryExpression expressionThen, XQueryExpression expressionElse, TypeVisitor typevisitor) throws TypeException, XQueryException;

	InstanceOfExpression createInstanceOfExpression(XQueryExpression expression, SequenceTypeExpression sequenceType, TypeVisitor typevisitor) throws TypeException, XQueryException;

	InternalFunctionCall createInternalFunctionCall(QName funcName, ArrayList argsVect, TypeVisitor typevisitor) throws TypeException, XQueryException;

	ItemType createItemType(QName typeName, int typeDependency, XQueryExpression dependency, TypeVisitor typevisitor) throws TypeException, XQueryException;

	LibraryFunctionCall createLibraryFunctionCall(QName funcName, ArrayList argsVect, FunctionDefinition functionDefinition, TypeVisitor typevisitor) throws TypeException, XQueryException;

	ListOpAFTERExpression createListOpAFTERExpression(XQueryExpression mainExpression, XQueryExpression expression, TypeVisitor typevisitor) throws TypeException, XQueryException;

	ListOpArithExpression createListOpArithExpression(XQueryExpression mainExpression, XQueryExpression expression, int kind, TypeVisitor typevisitor) throws TypeException, XQueryException;

	ListOpBEFOREExpression createListOpBEFOREExpression(XQueryExpression mainExpression, XQueryExpression expression, TypeVisitor typevisitor) throws TypeException, XQueryException;

	ListOpCompExpression createListOpCompExpression(XQueryExpression mainExpression, XQueryExpression expression, int kind, TypeVisitor typevisitor) throws TypeException, XQueryException;

	ListOpEXCEPTExpression createListOpEXCEPTExpression(XQueryExpression mainExpression, XQueryExpression expression, TypeVisitor typevisitor) throws TypeException, XQueryException;

	ListOpINTERSECTExpression createListOpINTERSECTExpression(XQueryExpression mainExpression, XQueryExpression expression, TypeVisitor typevisitor) throws TypeException, XQueryException;

	ListOpUNIONExpression createListOpUNIONExpression(XQueryExpression mainExpression, XQueryExpression expression, TypeVisitor typevisitor) throws TypeException, XQueryException;

	LocatedExpression createLocatedExpression(ArrayList pathVect, boolean inPredicate, TypeVisitor typevisitor) throws TypeException, XQueryException;

	NodeType createNodeType(byte kind, String arg, TypeVisitor typevisitor) throws TypeException, XQueryException;

	Parameter createParameter(SequenceTypeExpression sequenceType, Variable var, TypeVisitor typevisitor) throws TypeException, XQueryException;

	QName createQName(String namespace, String prefixname, String typeName, TypeVisitor typevisitor) throws TypeException, XQueryException;

	QuantifiedExpression createQuantifiedExpression(int kind, ArrayList variables, XQueryExpression expressionSat, TypeVisitor typevisitor) throws TypeException, XQueryException;

	RangeExpression createRangeExpression(XQueryExpression expression1, XQueryExpression expression2, TypeVisitor typevisitor) throws TypeException, XQueryException;

	SequenceTypeExpression createSequenceTypeExpression(int kind, ItemType itemType, int occurrence, TypeVisitor typevisitor) throws TypeException, XQueryException;

	SortedExpression createSortedExpression(XQueryExpression expression, ArrayList tmpVect, boolean stable, TypeVisitor typevisitor) throws TypeException, XQueryException;

	Step createStep(boolean separator, byte axis, XQueryExpression expression, ArrayList predicates, TypeVisitor typevisitor) throws TypeException, XQueryException;

	TypeSwitchExpression createTypeSwitchExpression(XQueryExpression switchExpression, HashMap cases, Variable defaultVariable, XQueryExpression defaultExpression, TypeVisitor typevisitor) throws TypeException, XQueryException;

	UnOpMinusExpression createUnOpMinusExpression(XQueryExpression expression, boolean minus, TypeVisitor typevisitor) throws TypeException, XQueryException;

	ValidateExpression createValidateExpression(XQueryExpression schemaContext, XQueryExpression validateExpression, TypeVisitor typeVisitor) throws TypeException, XQueryException;

	ValueBoolean createValueBoolean(String str, TypeVisitor typevisitor) throws TypeException, XQueryException;

	ValueDecimal createValueDecimal(String str, TypeVisitor typevisitor) throws TypeException, XQueryException;

	ValueDouble createValueDouble(String str, TypeVisitor typevisitor) throws TypeException, XQueryException;

	ValueInteger createValueInteger(String str, TypeVisitor typevisitor) throws TypeException, XQueryException;

	ValueString createValueString(String str, TypeVisitor typevisitor) throws TypeException, XQueryException;

	ValueParameter createValueParameter(String str, QName typeName, TypeVisitor typevisitor) throws TypeException, XQueryException;

	ValueText createValueText(String str, TypeVisitor typevisitor) throws TypeException, XQueryException;

	Variable createVariable(QName var, TypeVisitor typevisitor) throws TypeException, XQueryException;

	XMLComment createXMLComment(String comment, TypeVisitor typevisitor) throws TypeException, XQueryException;

	XMLProcessingInstruction createXMLProcessingInstruction(String PITarget, String data, TypeVisitor typevisitor) throws TypeException, XQueryException;

	XQueryExpressionSequence createXQueryExpressionSequence(ArrayList subExpressions, TypeVisitor typevisitor) throws TypeException, XQueryException;

	XQueryFile createXQueryFile(ArrayList vect, TypeVisitor typevisitor) throws TypeException, XQueryException;

	XQueryUnit createXQueryUnit(NamespaceContextStack hashDeclList, HashMap hashFuncList, ArrayList vectExpression, String defaultCollation, TypeVisitor typevisitor) throws TypeException, XQueryException;

	XQueryVoid createXQueryVoid(TypeVisitor typevisitor) throws TypeException, XQueryException;
}
