/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */

package org.xquark.xquery.parser;

import java.util.ArrayList;
import java.util.HashMap;

import org.xquark.util.NamespaceContextStack;
import org.xquark.xquery.parser.hinttree.HintTree;

public class ParserFactoryMediator implements ParserFactoryInterfaceMediator {

	private static final String RCSRevision = "$Revision: 1.3 $";
	private static final String RCSName = "$Name:  $";

	public AttributeValuePair createAttributeValuePair(XQueryExpression attributeName, XQueryExpression attributeValue, TypeVisitor typevisitor) throws TypeException, XQueryException {
		return new AttributeValuePair(attributeName, attributeValue, typevisitor);
	}

	public BinOpANDExpression createBinOpANDExpression(XQueryExpression mainExpression, XQueryExpression expression, TypeVisitor typevisitor) throws TypeException, XQueryException {
		return new BinOpANDExpression(mainExpression, expression, typevisitor);
	}

	public BinOpORExpression createBinOpORExpression(XQueryExpression mainExpression, XQueryExpression expression, TypeVisitor typevisitor) throws TypeException, XQueryException {
		return new BinOpORExpression(mainExpression, expression, typevisitor);
	}

	public CData createCData(String data, TypeVisitor typevisitor) throws TypeException, XQueryException {
		return new CData(data, typevisitor);
	}

	public CaseClauseExpression createCaseClauseExpression(SequenceTypeExpression sequenceType, Variable variable, XQueryExpression returnExpression, TypeVisitor typeVisitor) throws TypeException, XQueryException {
		return new CaseClauseExpression(sequenceType, variable, returnExpression, typeVisitor);
	}

	public CastTreatExpression createCastTreatExpression(int kind, SequenceTypeExpression sequenceType, XQueryExpression expression, TypeVisitor typevisitor) throws TypeException, XQueryException {
		return new CastTreatExpression(kind, sequenceType, expression, typevisitor);
	}

	public ContextDeclaration createContextDeclaration(String identifier, String value, int declType, TypeVisitor typevisitor) throws TypeException, XQueryException {
		return new ContextDeclaration(identifier, value, declType, typevisitor);
	}

	public Document createDocument(XQueryExpression element, TypeVisitor typevisitor) throws TypeException, XQueryException {
		return new Document(element, typevisitor);
	}

	public Element createElement(XQueryExpression startTag, HashMap localPrefixes, ArrayList vectAttributes, ArrayList vectExpressions, TypeVisitor typevisitor) throws TypeException, XQueryException {
		return new Element(startTag, localPrefixes, vectAttributes, vectExpressions, typevisitor);
	}

	public FLWRExpression createFLWRExpression(ArrayList bindings, HintTree hintClause, XQueryExpression whereClause, ArrayList orderbys, XQueryExpression returnClause, TypeVisitor typevisitor) throws TypeException, XQueryException {
		return new FLWRExpression(bindings, hintClause, whereClause, orderbys, returnClause, typevisitor);
	}

	public FLWRExpression createFLWRExpression(ArrayList bindings, XQueryExpression whereClause, ArrayList orderbys, XQueryExpression returnClause, TypeVisitor typevisitor) throws TypeException, XQueryException {
		return new FLWRExpression(bindings, orderbys, whereClause, returnClause, typevisitor);
	}

	public FLWRExpression createFLWRExpression(ArrayList bindings, XQueryExpression whereClause, XQueryExpression returnClause, TypeVisitor typevisitor) throws TypeException, XQueryException {
		return new FLWRExpression(bindings, whereClause, returnClause, typevisitor);
	}

	public FunctionDefinition createFunctionDefinition(QName funcName, ArrayList tmpVect, SequenceTypeExpression sequenceType, ArrayList vectExpression, TypeVisitor typevisitor) throws TypeException, XQueryException {
		return new FunctionDefinition(funcName, tmpVect, sequenceType, vectExpression, typevisitor);
	}

	public ITEExpression createITEExpression(XQueryExpression expressionIf, XQueryExpression expressionThen, XQueryExpression expressionElse, TypeVisitor typevisitor) throws TypeException, XQueryException {
		return new ITEExpression(expressionIf, expressionThen, expressionElse, typevisitor);
	}

	public InstanceOfExpression createInstanceOfExpression(XQueryExpression expression, SequenceTypeExpression sequenceType, TypeVisitor typevisitor) throws TypeException, XQueryException {
		return new InstanceOfExpression(expression, sequenceType, typevisitor);
	}

	public InternalFunctionCall createInternalFunctionCall(QName funcName, ArrayList argsVect, TypeVisitor typevisitor) throws TypeException, XQueryException {
		return new InternalFunctionCall(funcName, argsVect, typevisitor);
	}

	public ItemType createItemType(QName typeName, int typeDependency, XQueryExpression dependency, TypeVisitor typevisitor) throws TypeException, XQueryException {
		return new ItemType(typeName, typeDependency, dependency, typevisitor);
	}
	
	public HintTree createHintTree(int type, int side, Object left, Object right) {
		return new HintTree(type, side, left, right);
	}

	public LibraryFunctionCall createLibraryFunctionCall(QName funcName, ArrayList argsVect, FunctionDefinition funcdef, TypeVisitor typevisitor) throws TypeException, XQueryException {
		return new LibraryFunctionCall(funcName, argsVect, funcdef, typevisitor);
	}

	public ListOpAFTERExpression createListOpAFTERExpression(XQueryExpression mainExpression, XQueryExpression expression, TypeVisitor typevisitor) throws TypeException, XQueryException {
		return new ListOpAFTERExpression(mainExpression, expression, typevisitor);
	}

	public ListOpArithExpression createListOpArithExpression(XQueryExpression mainExpression, XQueryExpression expression, int kind, TypeVisitor typevisitor) throws TypeException, XQueryException {
		return new ListOpArithExpression(mainExpression, expression, kind, typevisitor);
	}

	public ListOpBEFOREExpression createListOpBEFOREExpression(XQueryExpression mainExpression, XQueryExpression expression, TypeVisitor typevisitor) throws TypeException, XQueryException {
		return new ListOpBEFOREExpression(mainExpression, expression, typevisitor);
	}

	public ListOpCompExpression createListOpCompExpression(XQueryExpression mainExpression, XQueryExpression expression, int kind, TypeVisitor typevisitor) throws TypeException, XQueryException {
		return new ListOpCompExpression(mainExpression, expression, kind, typevisitor);
	}

	public ListOpEXCEPTExpression createListOpEXCEPTExpression(XQueryExpression mainExpression, XQueryExpression expression, TypeVisitor typevisitor) throws TypeException, XQueryException {
		return new ListOpEXCEPTExpression(mainExpression, expression, typevisitor);
	}

	public ListOpINTERSECTExpression createListOpINTERSECTExpression(XQueryExpression mainExpression, XQueryExpression expression, TypeVisitor typevisitor) throws TypeException, XQueryException {
		return new ListOpINTERSECTExpression(mainExpression, expression, typevisitor);
	}

	public ListOpUNIONExpression createListOpUNIONExpression(XQueryExpression mainExpression, XQueryExpression expression, TypeVisitor typevisitor) throws TypeException, XQueryException {
		return new ListOpUNIONExpression(mainExpression, expression, typevisitor);
	}

	public LocatedExpression createLocatedExpression(ArrayList pathVect, boolean inPredicate, TypeVisitor typevisitor) throws TypeException, XQueryException {
		return new LocatedExpression(pathVect, inPredicate, typevisitor, false);
	}

	public NodeType createNodeType(byte kind, String arg, TypeVisitor typevisitor) throws TypeException, XQueryException {
		return new NodeType(kind, arg, typevisitor);
	}

	public Parameter createParameter(SequenceTypeExpression sequenceType, Variable var, TypeVisitor typevisitor) throws TypeException, XQueryException {
		return new Parameter(sequenceType, var, typevisitor);
	}

	public QName createQName(String namespace, String prefixname, String typeName, TypeVisitor typevisitor) throws TypeException, XQueryException {
		return new QName(namespace, prefixname, typeName, typevisitor);
	}

	public QuantifiedExpression createQuantifiedExpression(int kind, ArrayList variables, XQueryExpression expressionSat, TypeVisitor typevisitor) throws TypeException, XQueryException {
		return new QuantifiedExpression(kind, variables, expressionSat, typevisitor);
	}

	public RangeExpression createRangeExpression(XQueryExpression expression1, XQueryExpression expression2, TypeVisitor typevisitor) throws TypeException, XQueryException {
		return new RangeExpression(expression1, expression2, typevisitor);
	}

	public SequenceTypeExpression createSequenceTypeExpression(int kind, ItemType itemType, int occurrence, TypeVisitor typevisitor) throws TypeException, XQueryException {
		return new SequenceTypeExpression(kind, itemType, occurrence, typevisitor);
	}

	public SortedExpression createSortedExpression(XQueryExpression expression, ArrayList tmpVect, boolean stable, TypeVisitor typevisitor) throws TypeException, XQueryException {
		return new SortedExpression(expression, tmpVect, stable, typevisitor);
	}

	public Step createStep(boolean separator, byte axis, XQueryExpression expression, ArrayList predicates, TypeVisitor typevisitor) throws TypeException, XQueryException {
		return new Step(separator, axis, expression, predicates, typevisitor);
	}

	public TypeSwitchExpression createTypeSwitchExpression(XQueryExpression switchExpression, HashMap cases, Variable defaultVariable, XQueryExpression defaultExpression, TypeVisitor typevisitor) throws TypeException, XQueryException {
		return new TypeSwitchExpression(switchExpression, cases, defaultVariable, defaultExpression, typevisitor);
	}

	public UnOpMinusExpression createUnOpMinusExpression(XQueryExpression expression, boolean minus, TypeVisitor typevisitor) throws TypeException, XQueryException {
		return new UnOpMinusExpression(expression, minus, typevisitor);
	}

	public ValidateExpression createValidateExpression(XQueryExpression schemaContext, XQueryExpression validateExpression, TypeVisitor typeVisitor) throws TypeException, XQueryException {
		return new ValidateExpression(schemaContext, validateExpression, typeVisitor);
	}

	public ValueBoolean createValueBoolean(String str, TypeVisitor typevisitor) throws TypeException, XQueryException {
		return new ValueBoolean(str, typevisitor);
	}

	public ValueDecimal createValueDecimal(String str, TypeVisitor typevisitor) throws TypeException, XQueryException {
		return new ValueDecimal(str, typevisitor);
	}

	public ValueDouble createValueDouble(String str, TypeVisitor typevisitor) throws TypeException, XQueryException {
		return new ValueDouble(str, typevisitor);
	}

	public ValueInteger createValueInteger(String str, TypeVisitor typevisitor) throws TypeException, XQueryException {
		return new ValueInteger(str, typevisitor);
	}

	public ValueString createValueString(String str, TypeVisitor typevisitor) throws TypeException, XQueryException {
		return new ValueString(str, typevisitor);
	}

	public ValueParameter createValueParameter(String str, QName typeName, TypeVisitor typevisitor) throws TypeException, XQueryException {
		return new ValueParameter(str, typeName, typevisitor);
	}

	public ValueText createValueText(String str, TypeVisitor typevisitor) throws TypeException, XQueryException {
		return new ValueText(str, typevisitor);
	}

	public Variable createVariable(QName var, TypeVisitor typevisitor) throws TypeException, XQueryException {
		return new Variable(var, typevisitor);
	}

	public XMLComment createXMLComment(String comment, TypeVisitor typevisitor) throws TypeException, XQueryException {
		return new XMLComment(comment, typevisitor);
	}

	public XMLProcessingInstruction createXMLProcessingInstruction(String PITarget, String data, TypeVisitor typevisitor) throws TypeException, XQueryException {
		return new XMLProcessingInstruction(PITarget, data, typevisitor);
	}

	public XQueryExpressionSequence createXQueryExpressionSequence(ArrayList subExpressions, TypeVisitor typevisitor) throws TypeException, XQueryException {
		return new XQueryExpressionSequence(subExpressions, typevisitor);
	}

	public XQueryFile createXQueryFile(ArrayList vect, TypeVisitor typevisitor) throws TypeException, XQueryException {
		return new XQueryFile(vect, typevisitor);
	}

	public XQueryUnit createXQueryUnit(NamespaceContextStack hashDeclList, HashMap hashFuncList, ArrayList vectExpression, String defaultCollation, TypeVisitor typevisitor) throws TypeException, XQueryException {
		return new XQueryUnit(hashDeclList, hashFuncList, vectExpression, defaultCollation);
	}

	public XQueryVoid createXQueryVoid(TypeVisitor typevisitor) throws TypeException, XQueryException {
		return new XQueryVoid(typevisitor);
	}
}
