/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */

package org.xquark.xquery.parser;

import java.util.ArrayList;

import org.xquark.xquery.parser.util.Constants;

public class PrintVisitor extends DefaultParserVisitor {

	private boolean ids = false;
	private boolean arounds = false;
	private boolean order = false;
	private StringBuffer buf = new StringBuffer();
	private PrintVisitor pvisitor = null;

	PrintVisitor() {
	}

	PrintVisitor(boolean ids) {
		this.ids = ids;
	}

	public void reset(boolean ids, boolean arounds) {
		this.ids = ids;
		this.arounds = arounds;
		buf.setLength(0);
	}
	public String getString() {
		return buf.toString();
	}

	private String foreSee(XQueryExpression arg) throws XQueryException {
		if (pvisitor == null)
			pvisitor = new PrintVisitor();
		pvisitor.reset(ids, arounds);
		arg.accept(pvisitor);
		return pvisitor.getString();
	}

	private void doBefore(XQueryExpression arg) throws XQueryException {
		if (ids) {
			ArrayList tmplist = arg.getSkolemIDs();
			if (tmplist != null && !tmplist.isEmpty()) {
				buf.append(" ID(");
				for (int i = 0; i < tmplist.size(); i++) {
					if (i != 0)
						buf.append(",");
					((Variable) tmplist.get(i)).accept(this);
				}
				buf.append(")");
			}
			tmplist = arg.getDependIDs();
			if (tmplist != null && !tmplist.isEmpty()) {
				buf.append(" DEPEND(");
				for (int i = 0; i < tmplist.size(); i++) {
					if (i != 0)
						buf.append(",");
					((Variable) tmplist.get(i)).accept(this);
				}
				buf.append(")");
			}
		}
		if (arounds) {
			if (arg.getQuotes())
				buf.append("\"");
			if (arg.getSingleQuotes())
				buf.append("'");
			if (arg.getParenthesis())
				buf.append("(");
			if (arg.getBrace())
				buf.append("{");
			if (arg.getBracket())
				buf.append("[");
		}
		if (order) {
			if (arg.getUnordered())
				buf.append("unordered ");
		}
	}

	private void doAfter(XQueryExpression arg) {
		if (order) {
			if (arg.getOrder()[0] != -1) {
				buf.append(" ");
				buf.append(Constants.ORDERSSTRINGS[arg.getOrder()[0]]);
			}
			if (arg.getOrder()[1] != -1) {
				buf.append(" empty ");
				buf.append(Constants.ORDERSSTRINGS[arg.getOrder()[1]]);
			}
		}
		if (arounds) {
			if (arg.getBracket())
				buf.append("]");
			if (arg.getBrace())
				buf.append("}");
			if (arg.getParenthesis())
				buf.append(")");
			if (arg.getSingleQuotes())
				buf.append("'");
			if (arg.getQuotes())
				buf.append("\"");
		}
	}

	//	public void visit(AggregateFunctionCall arg) throws XQueryException;
	public void visit(AttributeValuePair arg) throws XQueryException {
		if (ids || arounds || order)
			doBefore(arg);
		if (arg.isComputed()) {
			buf.append("attribute ");
			if (!(arg.getExpression1() instanceof QName))
				buf.append("{ ");
			arg.getExpression1().accept(this);
			if (!(arg.getExpression1() instanceof QName))
				buf.append(" }");
			buf.append(" { ");
			arg.getExpression2().accept(this);
			buf.append(" }");
		} else {
			arg.getExpression1().accept(this);
			String expr2 = foreSee(arg.getExpression2());
			if (!(expr2.charAt(0) == '\"')) {
				buf.append("=\"");
				buf.append(expr2);
				buf.append("\"");
			} else
				buf.append(expr2);
		}
		if (arounds || order)
			doAfter(arg);
	}
	public void visit(BinOpANDExpression arg) throws XQueryException {
		if (ids || arounds || order)
			doBefore(arg);
		arg.getExpression1().accept(this);
		buf.append(Constants.NODELISTOPERATORSSTRINGS[Constants.AND_NODELISTOPERATOR]);
		arg.getExpression2().accept(this);
		doAfter(arg);
	}
	public void visit(BinOpORExpression arg) throws XQueryException {
		if (ids || arounds || order)
			doBefore(arg);
		arg.getExpression1().accept(this);
		buf.append(Constants.NODELISTOPERATORSSTRINGS[Constants.OR_NODELISTOPERATOR]);
		arg.getExpression2().accept(this);
		if (arounds || order)
			doAfter(arg);
	}
	//	public void visit(CaseClauseExpression arg) throws XQueryException;
	public void visit(CastTreatExpression arg) throws XQueryException {
		if (ids || arounds || order)
			doBefore(arg);
		buf.append(Constants.TYPINGSTRINGS[arg.getOperator()]);
		buf.append(" as ");
		arg.getSequenceType().accept(this);
		buf.append("(");
		arg.getExpression().accept(this);
		buf.append(")");
		if (arounds || order)
			doAfter(arg);
	}
	//	public void visit(CData arg) throws XQueryException;
	//	public void visit(ContextDeclaration arg) throws XQueryException;
	public void visit(Document arg) throws XQueryException {
		if (ids || arounds || order)
			doBefore(arg);
		buf.append("document { ");
		arg.getExprSequence().accept(this);
		buf.append(" }");
		if (arounds || order)
			doAfter(arg);
	}
	public void visit(Element arg) throws XQueryException {
		if (ids || arounds || order)
			doBefore(arg);
		if (arg.isComputed()) {
			buf.append("element ");
			if (!(arg.getStartTag() instanceof QName))
				buf.append("{ ");
			arg.getStartTag().accept(this);
			if (!(arg.getStartTag() instanceof QName))
				buf.append(" }");
			buf.append(" {");
			ArrayList attributes = arg.getAttributes();
			if (attributes != null)
				for (int i = 0; i < attributes.size(); i++) {
					buf.append(" ");
					((AttributeValuePair) attributes.get(i)).accept(this);
					if (i < attributes.size() - 1)
						buf.append(",");
				}
			ArrayList subExpressions = arg.getSubExpressions();
			if (subExpressions != null)
				for (int j = 0; j < subExpressions.size(); j++) {
					if (j == 0 && attributes != null)
						buf.append(",");
					buf.append(" ");
					((XQueryExpression) subExpressions.get(j)).accept(this);
					if (j < subExpressions.size() - 1)
						buf.append(",");
				}
			buf.append(" }");
		} else {
			buf.append("<");
			arg.getStartTag().accept(this);
			ArrayList attributes = arg.getAttributes();
			if (attributes != null)
				for (int i = 0; i < attributes.size(); i++) {
					buf.append(" ");
					((AttributeValuePair) attributes.get(i)).accept(this);
				}
			buf.append(">");
			ArrayList subExpressions = arg.getSubExpressions();
			if (subExpressions != null)
				for (int j = 0; j < subExpressions.size(); j++)
					 ((XQueryExpression) subExpressions.get(j)).accept(this);
			buf.append("</");
			arg.getStartTag().accept(this);
			buf.append(">");
		}
		if (arounds || order)
			doAfter(arg);
	}
	public void visit(FLWRExpression arg) throws XQueryException {
		if (ids || arounds || order)
			doBefore(arg);
		ArrayList variables = arg.getVariables();
		if (variables != null)
			for (int i = 0; i < variables.size(); i++)
				buf.append(((Variable) variables.get(i)).printBinding());
		if (arg.getWhereClause() != null) {
			buf.append(" where ");
			arg.getWhereClause().accept(this);
		}
		if (arg.getOrderBy() != null) {
			buf.append(" order by ");
			for (int i = 0; i < arg.getOrderBy().size(); i++) {
				if (i != 0)
					buf.append(",");
				boolean prevOrder = order;
				order = true;
				((XQueryExpression) arg.getOrderBy().get(i)).accept(this);
				order = prevOrder;
			}
		}
		if (arg.getReturnClause() != null) {
			buf.append(" return ");
			arg.getReturnClause().accept(this);
		}
		if (arounds || order)
			doAfter(arg);
	}
	public void visit(FunctionCall arg) throws XQueryException {
		if (ids || arounds || order)
			doBefore(arg);
		arg.getFuncName().accept(this);
		buf.append("(");
		ArrayList arguments = arg.getArguments();
		if (arguments != null)
			for (int i = 0; i < arguments.size(); i++) {
				if (i != 0)
					buf.append(",");
				((XQueryExpression) arguments.get(i)).accept(this);
			}
		buf.append(")");
		if (arounds || order)
			doAfter(arg);
	}
	//	public void visit(FunctionDefinition arg) throws XQueryException;
	//	public void visit(InstanceOfExpression arg) throws XQueryException;
	//	public void visit(InternalFunctionCall arg) throws XQueryException;
	//	public void visit(ITEExpression arg) throws XQueryException;
	//	public void visit(ItemType arg) throws XQueryException;
	//	public void visit(LibraryFunctionCall arg) throws XQueryException;
	//	public void visit(ListOpAFTERExpression arg) throws XQueryException;
	//	public void visit(ListOpArithExpression arg) throws XQueryException;
	//	public void visit(ListOpBEFOREExpression arg) throws XQueryException;
	//	public void visit(ListOpCompExpression arg) throws XQueryException;
	//	public void visit(ListOpEXCEPTExpression arg) throws XQueryException;
	//	public void visit(ListOpINTERSECTExpression arg) throws XQueryException;
	//	public void visit(ListOpUNIONExpression arg) throws XQueryException;
	//	public void visit(LocatedExpression arg) throws XQueryException;
	//	public void visit(NodeType arg) throws XQueryException;
	//	public void visit(Parameter arg) throws XQueryException;
	//	public void visit(PrimitiveFunctionCall arg) throws XQueryException;
	//	public void visit(QName arg) throws XQueryException;
	//	public void visit(QuantifiedExpression arg) throws XQueryException;
	//	public void visit(RangeExpression arg) throws XQueryException;
	//	public void visit(RootNodeFunctionCall arg) throws XQueryException;
	//	public void visit(SequenceTypeExpression arg) throws XQueryException;
	//	public void visit(SortedExpression arg) throws XQueryException;
	//	public void visit(Step arg) throws XQueryException;
	//	public void visit(TypeSwitchExpression arg) throws XQueryException;
	//	public void visit(UnOpMinusExpression arg) throws XQueryException;
	//	public void visit(Value arg) throws XQueryException;
	//	public void visit(ValueBoolean arg) throws XQueryException;
	//	public void visit(ValueDecimal arg) throws XQueryException;
	//	public void visit(ValueDouble arg) throws XQueryException;
	//	public void visit(ValueFloat arg) throws XQueryException;
	//	public void visit(ValueInteger arg) throws XQueryException;
	//	public void visit(ValueString arg) throws XQueryException;
	//	public void visit(ValueParameter arg) throws XQueryException;
	//	public void visit(ValueText arg) throws XQueryException;
	//	public void visit(Variable arg) throws XQueryException;
	//	public void visit(XMLComment arg) throws XQueryException;
	//	public void visit(XMLProcessingInstruction arg) throws XQueryException;
	//	public void visit(XQueryBinaryOperatorExpression arg) throws XQueryException;
	//	public void visit(XQueryBooleanOperatorExpression arg) throws XQueryException;
	//  public void visit(XQueryExpression arg) throws XQueryException {
	//	public void visit(XQueryExpressionSequence arg) throws XQueryException;
	//	public void visit(XQueryFile arg) throws XQueryException;
	//	public void visit(XQueryListBinaryOperatorExpression arg) throws XQueryException;
	//	public void visit(XQueryListUnaryOperatorExpression arg) throws XQueryException;
	//	public void visit(XQueryUnaryOperatorExpression arg) throws XQueryException;
	//	public void visit(XQueryUnit arg) throws XQueryException;
	//	public void visit(XQueryVoid arg) throws XQueryException;
	//	
	//	// primitive functions
	//	
	//	public void visit(FunctionABS arg) throws XQueryException ;
	//	public void visit(FunctionAVG arg) throws XQueryException ;
	//	public void visit(FunctionCEILING arg) throws XQueryException;
	//	public void visit(FunctionCOLLECTION arg) throws XQueryException;
	//	public void visit(FunctionCONCAT arg) throws XQueryException;
	//	public void visit(FunctionCONTAINS arg) throws XQueryException;
	//	public void visit(FunctionCOUNT arg) throws XQueryException;
	//	public void visit(FunctionCURRENT_DATE arg) throws XQueryException;
	//	public void visit(FunctionCURRENT_DATETIME arg) throws XQueryException;
	//	public void visit(FunctionCURRENT_TIME arg) throws XQueryException;
	//	public void visit(FunctionDATA arg) throws XQueryException;
	//	public void visit(FunctionDATE arg) throws XQueryException;
	//	public void visit(FunctionDATETIME arg) throws XQueryException;
	//	public void visit(FunctionDECIMAL arg) throws XQueryException;
	//	public void visit(FunctionDISTINCT_VALUES arg) throws XQueryException;
	//	public void visit(FunctionDOC arg) throws XQueryException;
	//	public void visit(FunctionDOUBLE arg) throws XQueryException;
	//	public void visit(FunctionEMPTY arg) throws XQueryException;
	//	public void visit(FunctionENDS_WITH arg) throws XQueryException;
	//	public void visit(FunctionEXISTS arg) throws XQueryException;
	//	public void visit(FunctionGET_MONTH_FROM_DATE arg) throws XQueryException;
	//	public void visit(FunctionGET_YEAR_FROM_DATE arg) throws XQueryException;
	//	public void visit(FunctionFALSE arg) throws XQueryException;
	//	public void visit(FunctionFLOOR arg) throws XQueryException;
	//	public void visit(FunctionINTEGER arg) throws XQueryException;
	//	public void visit(FunctionLOCAL_NAME arg) throws XQueryException;
	//	public void visit(FunctionLOWER_CASE arg) throws XQueryException;
	//	public void visit(FunctionMATCHES arg) throws XQueryException;
	//	public void visit(FunctionMAX arg) throws XQueryException;
	//	public void visit(FunctionMIN arg) throws XQueryException;
	//	public void visit(FunctionNAMESPACE_URI arg) throws XQueryException;
	//	public void visit(FunctionNOT arg) throws XQueryException;
	//	public void visit(FunctionNUMBER arg) throws XQueryException;
	//	public void visit(FunctionROUND arg) throws XQueryException;
	//	public void visit(FunctionSTARTS_WITH arg) throws XQueryException;
	//	public void visit(FunctionSTRING arg) throws XQueryException;
	//	public void visit(FunctionSTRING_LENGTH arg) throws XQueryException;
	//	public void visit(FunctionSUBSTRING arg) throws XQueryException;
	//	public void visit(FunctionSUM arg) throws XQueryException;
	//	public void visit(FunctionTIME arg) throws XQueryException;
	//	public void visit(FunctionTRUE arg) throws XQueryException;
	//	public void visit(FunctionUPPER_CASE arg) throws XQueryException;
	//	public void visit(FunctionPART arg) throws XQueryException;
	//	public void visit(FunctionTUPLE arg) throws XQueryException;
	//	public void visit(FunctionDEEP_EQUALS arg) throws XQueryException;
}
