/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */

package org.xquark.xquery.parser;

public class QName extends XQueryExpression implements Cloneable {

	private static final String RCSRevision = "$Revision: 1.1 $";
	private static final String RCSName = "$Name:  $";

	protected String namespace = null;
	protected String prefixname = null;
	// cannot be null or empty string
	protected String localname = null;

	// #############################################################################
	// VISITOR STUFF
	// #############################################################################

	public void accept(ParserVisitor visitor) throws XQueryException {
		visitor.visit(this);
	}

	// #############################################################################
	// CONSTRUCTOR STUFF
	// #############################################################################

	public QName(String namespace, String prefixname, String localname, TypeVisitor typevisitor) throws TypeException, XQueryException {
		setNameSpace(namespace);
		setPrefixName(prefixname);
		setLocalName(localname);
		// erase   05/09/2003 not useful
		/*
		if (typevisitor != null)
			accept(typevisitor);
		*/
	}

	// #############################################################################
	// ATTRIBUTE ACCESS STUFF
	// #############################################################################

	public String getLocalName() {
		return localname;
	}
	public void setLocalName(String localname) throws XQueryException {
		if (localname == null || localname.equals("")) {
			if (prefixname == null || prefixname.equals(""))
				throw new XQueryException("localname of QName cannot be empty string");
			this.localname = prefixname;
			setPrefixName(null);
		} else
			this.localname = localname;
	}

	public String getPrefixName() {
		return prefixname;
	}
	public void setPrefixName(String prefixname) {
		this.prefixname = prefixname;
	}

	public String getNameSpace() {
		return namespace;
	}
	public void setNameSpace(String namespace) {
		this.namespace = namespace;
	}

	//public String getName()	{ return pprint(); }

	public String getName() {
		return localname;
	}
	public void setName(String name) {
		localname = name;
	}

	public boolean equals(XQueryExpression expr) {
		if (expr instanceof QName) {
			if (!localname.equals(((QName) expr).getLocalName()))
				return false;
			if (namespace == null && ((QName) expr).getNameSpace() == null)
				return true;
			if (namespace != null && namespace.equals(((QName) expr).getNameSpace()))
				return true;
		}
		if (expr instanceof Value) {
			if (namespace == null && localname.equals(((Value) expr).getValue()))
				return true;
		}
		return false;
	}

	// #############################################################################
	// PRINT STUFF
	// #############################################################################

	public String print() {
		if (prefixname != null && (!prefixname.equals("*") || !localname.equals("*")))
			return prefixname + ":" + localname;
		return localname;
	}

	public String printWithNameSpace() {
		if (namespace != null)
			return "{" + namespace + "}" + localname;
		// if (prefixname != null) return  prefixname + ":" + localname;
		return localname;
	}

	// #############################################################################
	// CLONE STUFF
	// #############################################################################

	//    public synchronized Object clone()
	//    throws CloneNotSupportedException {
	//        try {
	//            QName newobj = (QName)super.clone();
	//            newobj.setNameSpace((getNameSpace() == null) ? null : new String(getPrefixName()));
	//            newobj.setPrefixName((getPrefixName() == null) ? null : new String(getPrefixName()));
	//            newobj.setLocalName((getLocalName() == null) ? null : new String(getLocalName()));
	//            return  newobj;
	//        }
	//        catch (Exception e) {
	//            throw new CloneNotSupportedException("Error in cloning QName");
	//        }
	//    }
	//    
	//    public synchronized Object clone(HashMap mapvars)
	//    throws CloneNotSupportedException {
	//        try {
	//            QName newobj = (QName)super.clone(mapvars);
	//            if (newobj == null) return null ;
	//            newobj.setNameSpace((getNameSpace() == null) ? null : new String(getPrefixName()));
	//            newobj.setPrefixName((getPrefixName() == null) ? null : new String(getPrefixName()));
	//            newobj.setLocalName((getLocalName() == null) ? null : new String(getLocalName()));
	//            return  newobj;
	//        }
	//        catch (Exception e) {
	//            throw new CloneNotSupportedException("Error in cloning QName");
	//        }
	//    }

}
