/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */

package org.xquark.xquery.parser;

import java.util.ArrayList;

import org.xquark.xquery.parser.util.Constants;

public class QuantifiedExpression extends XQueryListUnaryOperatorExpression implements Cloneable {

	private static final String RCSRevision = "$Revision: 1.1 $";
	private static final String RCSName = "$Name:  $";

	// cannot be -1
	int kind = -1;
	// cannot be null
	ArrayList variables = null;

	// #############################################################################
	// VISITOR STUFF
	// #############################################################################

	public void accept(ParserVisitor visitor) throws XQueryException {
		visitor.visit(this);
	}

	// #############################################################################
	// CONSTRUCTOR STUFF
	// #############################################################################

	public QuantifiedExpression(int kind, ArrayList variables, XQueryExpression constraintExpression, TypeVisitor typevisitor) throws TypeException, XQueryException {
		super(constraintExpression);
		setKind(kind);
		setVariables(variables);
		if (typevisitor != null)
			accept(typevisitor);
	}

	// #############################################################################
	// ATTRIBUTE ACCESS STUFF
	// #############################################################################

	public void setParentUnit(XQueryUnit parentUnit) {
		if (parentUnit == null)
			return;
		this.parentUnit = parentUnit;
		for (int i = 0; i < variables.size(); i++)
			 ((Variable) variables.get(i)).setParentUnit(parentUnit);
		super.setParentUnit(parentUnit);
	}

	public int getKind() {
		return kind;
	}
	public void setKind(int kind) throws XQueryException {
		if (kind == -1 || kind > Constants.MAX_QUANTIFIER)
			throw new XQueryException("kind of QuantifiedExpression has incorrect value " + kind);
		this.kind = kind;
	}

	//public XQueryExpression getMainExpresson() { return variable.getExpression(); }
	public XQueryExpression getConstraintExpresson() {
		return expression;
	}

	// OLIVIER - W3C WD 16 August 2002
	public void setConstraintExpresson(XQueryExpression constraintExpression) {
		expression = constraintExpression;
	}

	public Variable getVariable() {
		return (Variable) variables.get(0);
	}
	public ArrayList getVariables() {
		return variables;
	}
	public void setVariables(ArrayList variables) throws XQueryException {
		if (variables == null || variables.isEmpty())
			throw new XQueryException("variables of QuantifiedExpression cannot be null or empty");
		this.variables = variables;
		for (int i = 0; i < variables.size(); i++) {
			((Variable) variables.get(i)).setParentExpression(this);
			((Variable) variables.get(i)).setParentUnit(parentUnit);
		}
	}

	// #############################################################################
	// PRINT STUFF
	// #############################################################################

	public String print() {
		StringBuffer buf = new StringBuffer();
		buf.append(Constants.QUANTIFIERSSTRINGS[kind]);
		buf.append(" ");
		for (int i = 0; i < variables.size(); i++) {
			if (i != 0)
				buf.append(" , ");
			buf.append(((Variable) variables.get(i)).printBinding());
		}
		buf.append(" satisfies ");
		buf.append(expression.pprint());
		return buf.toString();
	}

	// #############################################################################
	// CLONE STUFF
	// #############################################################################
	public void addParentExpression(XQueryExpression parentExpression) {
		addParentExpression(parentExpression);
		for (int i = 0; i < variables.size(); i++)
			 ((Variable) variables.get(i)).addParentExpression(parentExpression);
		expression.addParentExpression(parentExpression);
	}

//	public synchronized Object clone() throws CloneNotSupportedException {
//		try {
//			QuantifiedExpression newobj = (QuantifiedExpression) super.clone();
//			newobj.setKind(getKind());
//			newobj.setVariables((ArrayList) getVariables().clone());
//			return newobj;
//		}
//		catch (Exception e) {
//			//System.err.println(e.getMessage()) ;
//			throw new CloneNotSupportedException("Error in cloning QuantifiedExpression");
//		}
//	}
//
//	public synchronized Object clone(HashMap mapvars) throws CloneNotSupportedException {
//		try {
//			QuantifiedExpression newobj = (QuantifiedExpression) super.clone(mapvars);
//			if (newobj == null)
//				return null;
//			newobj.setKind(getKind());
//			newobj.setVariables(XQueryExpressionTools.totalClone(getVariables(), mapvars));
//			return newobj;
//		}
//		catch (Exception e) {
//			//System.err.println(e.getMessage()) ;
//			throw new CloneNotSupportedException("Error in cloning QuantifiedExpression");
//		}
//	}
}
