/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */

/*
 * SequenceTypeExpression.java
 *
 * Created on 26 septembre 2002, 14:29
 */

package org.xquark.xquery.parser;

import org.xquark.xquery.parser.util.Constants;

public class SequenceTypeExpression extends XQueryExpression implements Cloneable {
	
	private static final String RCSRevision = "$Revision: 1.1 $";
	private static final String RCSName     = "$Name:  $";
	
	protected int type = -1;
	protected ItemType itemType = null;
	protected int occurrence = -1;
	
	// #############################################################################
	// VISITOR STUFF
	// #############################################################################
	
	public void accept(ParserVisitor visitor) throws XQueryException {
		visitor.visit(this);
	}
	
	// #############################################################################
	// CONSTRUCTOR STUFF
	// #############################################################################
	
	public SequenceTypeExpression(int type, ItemType itemType, int occurrence, TypeVisitor typevisitor) throws TypeException, XQueryException {
		setType(type);
		setItemType(itemType);
		setOccurrence(occurrence);
		if (typevisitor != null) accept(typevisitor);
	}
	
	// #############################################################################
	// ATTRIBUTE ACCESS STUFF
	// #############################################################################
	
	public void setParentUnit(XQueryUnit parentUnit) {
		if (parentUnit == null) return;
		this.parentUnit = parentUnit;
		super.setParentUnit(parentUnit);
		if (itemType != null) itemType.setParentUnit(parentUnit);
	}
	
	public void setType(int type) { this.type = type; }
	
	public int getType() { return type; }
	
	public void setItemType(ItemType itemType) { this.itemType = itemType; }
	
	public ItemType getItemType() { return itemType; }
	
	public void setOccurrence(int occurrence) { this.occurrence = occurrence; }
	
	public int getOccurrence() { return occurrence; }
	
	public String getNameSpace() { return itemType.getTypeName().getNameSpace(); }
	
	public String getLocalName() { return itemType.getTypeName().getLocalName(); }
	
	// #############################################################################
	// PRINT STUFF
	// #############################################################################
	
	public String print() {
		StringBuffer buf = new StringBuffer();
		if (occurrence != -1) buf.append("( ");
		if (type != -1) {
			buf.append(Constants.SEQUENCETYPE_OPERATORS[type]);
			buf.append(" ");
		}
		if (itemType != null) {
			buf.append(itemType.print());
			buf.append(" ");
		}
		if (occurrence != -1) {
			buf.append(")");
			buf.append(Constants.OCCURRENCE_INDICATORS[occurrence]);
		}
		return buf.toString();
	}
	
	// #############################################################################
	// CLONE STUFF
	// #############################################################################
	
//	public synchronized Object clone() throws CloneNotSupportedException {
//		SequenceTypeExpression newobj = (SequenceTypeExpression)super.clone();
//		newobj.setType(getType());
//		newobj.setItemType(getItemType());
//		newobj.setOccurrence(getOccurrence());
//		return newobj;
//	}
//	
//	public synchronized Object clone(HashMap mapvars) throws CloneNotSupportedException {
//		SequenceTypeExpression newobj = (SequenceTypeExpression)super.clone(mapvars);
//		if (newobj == null) return null;
//		newobj.setType(getType());
//		newobj.setItemType(getItemType());
//		newobj.setOccurrence(getOccurrence());
//		return newobj;
//	}
}
