/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */

package org.xquark.xquery.parser;

import java.util.ArrayList;

public class SortedExpression extends XQueryListUnaryOperatorExpression implements Cloneable {
	private static final String RCSRevision = "$Revision: 1.1 $";
	private static final String RCSName = "$Name:  $";

	// cannot be null or empty
	protected ArrayList sortClauses = null;

	// OLIVIER - W3C WD 16 August 2002
	protected boolean stable = false; // by default, implementation-defined

	// #############################################################################
	// CONSTRUCTOR STUFF
	// #############################################################################
	public SortedExpression(XQueryExpression expression, ArrayList sortClauses, boolean stable, TypeVisitor typevisitor) throws TypeException, XQueryException {
		super(expression);

		// OLIVIER - W3C WD 16 August 2002
		setStable(stable);
		setSortClauses(sortClauses);

		if (typevisitor != null) {
			accept(typevisitor);
		}
	}

	// #############################################################################
	// VISITOR STUFF
	// #############################################################################
	public void accept(ParserVisitor visitor) throws XQueryException {
		visitor.visit(this);
	}

	// #############################################################################
	// ATTRIBUTE ACCESS STUFF
	// #############################################################################
	public void setParentUnit(XQueryUnit parentUnit) {
		if (parentUnit == null) {
			return;
		}

		this.parentUnit = parentUnit;
		super.setParentUnit(parentUnit);

		for (int i = 0; i < sortClauses.size(); i++)
			 ((XQueryExpression) sortClauses.get(i)).setParentUnit(parentUnit);
	}

	public ArrayList getSortClauses() {
		return sortClauses;
	}

	public void setSortClauses(ArrayList sortClauses) throws XQueryException {
		// cannot be null or empty
		if ((sortClauses == null) || sortClauses.isEmpty()) {
			throw new XQueryException("sortClauses of SortedExpression cannot be null or empty");
		}

		this.sortClauses = sortClauses;

		for (int i = 0; i < this.sortClauses.size(); i++) {
			XQueryExpression expr = (XQueryExpression) this.sortClauses.get(i);
			expr.setParentUnit(parentUnit);
			expr.setParentExpression(this);
		}
	}

	// OLIVIER - W3C WD 16 August 2002
	public void setStable(boolean stable) {
		this.stable = stable;
	}

	public boolean getStable() {
		return stable;
	}

	// #############################################################################
	// PRINT STUFF
	// #############################################################################
	public String print() {
		StringBuffer buf = new StringBuffer();
		buf.append(expression.pprint());

		// OLIVIER - W3C WD 16 August 2002
		if (stable) {
			buf.append(" stable");
		}

		buf.append(" sortby (");

		for (int j = 0; j < sortClauses.size(); j++) {
			if (j != 0) {
				buf.append(" ,");
			}

			XQueryExpression expression = (XQueryExpression) sortClauses.get(j);
			buf.append(" " + expression.pprintWithOrder());
		}

		buf.append(" )");

		return buf.toString();
	}

	// #############################################################################
	// CLONE STUFF
	// #############################################################################
	public void addParentExpression(XQueryExpression parentExpression) {
		addParentExpression(parentExpression);
		expression.addParentExpression(parentExpression);

		for (int i = 0; i < sortClauses.size(); i++)
			 ((XQueryExpression) sortClauses.get(i)).addParentExpression(parentExpression);
	}

//	public synchronized Object clone() throws CloneNotSupportedException {
//		try {
//			SortedExpression newobj = (SortedExpression) super.clone();
//
//			// OLIVIER - W3C WD 16 August 2002
//			newobj.setStable(getStable());
//			newobj.setSortClauses((getSortClauses() == null) ? null : XQueryExpressionTools.totalClone(getSortClauses()));
//
//			return newobj;
//		} catch (Exception e) {
//			//System.err.println (e.getMessage ()) ;
//			throw new CloneNotSupportedException("Error in cloning SortedExpression");
//		}
//	}
//
//	public synchronized Object clone(HashMap mapvars) throws CloneNotSupportedException {
//		try {
//			SortedExpression newobj = (SortedExpression) super.clone(mapvars);
//
//			if (newobj == null) {
//				return null;
//			}
//
//			newobj.setSortClauses((getSortClauses() == null) ? null : XQueryExpressionTools.totalClone(getSortClauses(), mapvars));
//
//			return newobj;
//		} catch (Exception e) {
//			System.err.println(e.getMessage());
//			throw new XQueryException("Error in cloning SortedExpression");
//		}
//	}
}
