/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */

package org.xquark.xquery.parser;

import java.util.ArrayList;

import org.xquark.schema.Type;
import org.xquark.xpath.*;

public class Step extends XQueryUnaryOperatorExpression implements StepExpr, Cloneable {
	private static final String RCSRevision = "$Revision: 1.1 $";
	private static final String RCSName = "$Name:  $";

	// cannot be null
	protected ArrayList predicates = null;
	//protected int separator = -1;
	/**
	 * 
	 */
	private boolean hasSeparator = false;

	/**
	 * 
	 */
	private byte axis = Axis.NONE;

	/**
	 * 
	 */
	private String localName = null;

	/**
	 * 
	 */
	private String nameSpace = null;

	/**
	 * 
	 */
	private byte wild = StepExpr.NO_WILDCARD;

	/**
	 * 
	 */
	private byte kindtest = NodeKind.NONE;

	/**
	 * 
	 */
	private Type type = null;

	/**
	 * 
	 */
	private boolean isTypeNillable = false;

	// #############################################################################
	// VISITOR STUFF
	// #############################################################################

	public void accept(ParserVisitor visitor) throws XQueryException {
		visitor.visit(this);
	}

	// #############################################################################
	// CONSTRUCTOR STUFF
	// #############################################################################

	public Step(boolean hasSeparator, byte axis, XQueryExpression expression, ArrayList predicates, TypeVisitor typevisitor) throws TypeException, XQueryException {
		super(expression);
		setHasSeparator(hasSeparator);
		setAxis(axis);
		setPredicates(predicates);
		setInterfaceProperties();
		// change   05/09/2003 not useful
		/*
		if (typevisitor != null)
			accept(typevisitor);
		*/
	}

	// #############################################################################
	// ATTRIBUTE ACCESS STUFF
	// #############################################################################

	private void setInterfaceProperties() {
		// depending on expression		
		if (expression instanceof QName) {
			QName qname = (QName) expression;
			localName = qname.getLocalName();
			nameSpace = qname.getNameSpace();
			kindtest = NodeKind.ELEMENT;
			byte nodeKind = NodeKind.NONE;
			switch (axis) {
				case Axis.ATTRIBUTE :
					nodeKind = NodeKind.ATTRIBUTE;
					break;
				case Axis.NAMESPACE :
					nodeKind = NodeKind.NAMESPACE;
					break;
				default :
					nodeKind = NodeKind.ELEMENT;
			}
			xnode = new XNode(nameSpace, localName, nodeKind);
			// set wild
			if (NAME_WILDCARD.equals(qname.getName()))
				wild = StepExpr.WILDCARD;
			else if (NAME_WILDCARD.equals(localName))
				wild = StepExpr.LOCALNAME_WILDCARD;
			else if (NAME_WILDCARD.equals(nameSpace))
				wild = StepExpr.NAMESPACE_WILDCARD;
		} else if (expression instanceof NodeType) {
			// set kindtest
			kindtest = ((NodeType) expression).getKind();
			xnode = new XNode(kindtest);
		}
	}

	public void setParentUnit(XQueryUnit parentUnit) {
		if (parentUnit == null)
			return;
		this.parentUnit = parentUnit;
		super.setParentUnit(parentUnit);
		if (predicates != null)
			for (int i = 0; i < predicates.size(); i++)
				 ((XQueryExpression) predicates.get(i)).setParentUnit(parentUnit);
	}

	public boolean hasSeparator() {
		return hasSeparator;
	}
	public void setHasSeparator(boolean hasSeparator) {
		this.hasSeparator = hasSeparator;
	}

	public byte getAxis() {
		return axis;
	}
	public void setAxis(byte axis) {
		this.axis = axis;
	}

	public ArrayList getPredicates() {
		return predicates;
	}
	public void setPredicates(ArrayList predicates) {
		this.predicates = predicates;
		if (predicates != null)
			for (int i = 0; i < this.predicates.size(); i++) {
				XQueryExpression expr = (XQueryExpression) this.predicates.get(i);
				expr.setParentUnit(parentUnit);
				expr.setParentExpression(this);
			}
	}

	public void addPredicates(ArrayList predicates) {
		if (predicates == null || predicates.isEmpty())
			return;
		if (this.predicates == null || this.predicates.isEmpty()) {
			setPredicates(predicates);
			return;
		}
		this.predicates.addAll(predicates);
		setPredicates(this.predicates);
	}

	public void addPredicate(XQueryExpression predicate) {
		if (predicate == null)
			return;
		if (predicates == null)
			predicates = new ArrayList(1);
		predicates.add(predicate);
		predicate.setParentUnit(parentUnit);
		predicate.setParentExpression(this);
	}

	public void addPredicate(int index, XQueryExpression predicate) {
		if (predicates == null)
			predicates = new ArrayList(1);
		predicates.add(index, predicate);
		predicate.setParentUnit(parentUnit);
		predicate.setParentExpression(this);
	}

	//	public String getName() {
	//		return expression.getName();
	//	}

	// #############################################################################
	// PRINT STUFF
	// #############################################################################

	public String print() {
		StringBuffer buf = new StringBuffer();
		if (parenthesis)
			buf.append("( ");
		if (hasSeparator)
			buf.append('/');
		buf.append(Axis.AXISSTRINGS[axis]);
		buf.append(expression.print());
		if (predicates != null)
			for (int i = 0; i < predicates.size(); i++)
				buf.append(((XQueryExpression) predicates.get(i)).pprint());
		if (parenthesis)
			buf.append(" )");
		return buf.toString();
	}

	public String toHashCodeString() {
		StringBuffer buf = new StringBuffer();
		if (axis != Axis.NONE)
			buf.append('/');
		buf.append(Axis.AXISSTRINGS[axis]);
		switch (kindtest) {
			case NodeKind.ATTRIBUTE :
//				buf.append(NodeKind.NODEKINDSTRINGS[kindtest]);
//				buf.append('(');
				if (isWildcard())
					buf.append('*');
				else if (localName != null) {
//					buf.append('@');
					if (nameSpace != null) {
						buf.append('{');
						buf.append(nameSpace); 
						buf.append('}');
					}
					buf.append(localName);
//					if (type != null) {
//						buf.append(" ,");
//						buf.append(type.toString());
//					}
//				} else if (type != null) {
//					buf.append(type.toString());
				}
//				buf.append(')');
				break;
			case NodeKind.COMMENT :
				buf.append(NodeKind.NODEKINDSTRINGS[kindtest]);
				buf.append('(');
				buf.append(')');
				break;
			case NodeKind.DOCUMENT :
				buf.append(NodeKind.NODEKINDSTRINGS[kindtest]);
				buf.append('(');
				if (localName != null || type != null) {
//					buf.append(NodeKind.NODEKINDSTRINGS[NodeKind.ELEMENT]);
//					buf.append('(');
					if (isWildcard())
						buf.append('*');
					else if (localName != null) {
						if (nameSpace != null) {
							buf.append('{');
							buf.append(nameSpace); 
							buf.append('}');
						}
						buf.append(localName);
//						if (type != null) {
//							buf.append(" ,");
//							buf.append(type.toString());
//							if (isTypeNillable) {
//								buf.append(" nillable");
//							}
//						}
//					} else if (type != null) {
//						buf.append(type.toString());
					}
//					buf.append(')');
				}
				buf.append(')');
				break;
			case NodeKind.ELEMENT :
//				buf.append(NodeKind.NODEKINDSTRINGS[kindtest]);
//				buf.append('(');
				if (isWildcard())
					buf.append('*');
				else if (localName != null) {
					if (nameSpace != null) {
						buf.append('{');
						buf.append(nameSpace); 
						buf.append('}');
					}
					buf.append(localName);
//					if (type != null) {
//						buf.append(" ,");
//						buf.append(type.toString());
//						if (isTypeNillable) {
//							buf.append(" nillable");
//						}
//					}
//				} else if (type != null) {
//					buf.append(type.toString());
				}
//				buf.append(')');
				break;
				//case NodeKind.NAMESPACE :
			case NodeKind.NODE :
				buf.append(NodeKind.NODEKINDSTRINGS[kindtest]);
				buf.append('(');
				buf.append(')');
				break;
			case NodeKind.PI :
				buf.append(NodeKind.NODEKINDSTRINGS[kindtest]);
				buf.append('(');
				if (localName != null)
					buf.append(localName);
				buf.append(')');
				break;
			case NodeKind.TEXT :
				buf.append(NodeKind.NODEKINDSTRINGS[kindtest]);
				buf.append('(');
				buf.append(')');
				break;
		}
		return buf.toString();
	}
//	public String printWithoutChild() {
//		StringBuffer buf = new StringBuffer();
//		if (hasSeparator)
//			buf.append('/');
//		// this change was made to fit YU CHENG problems but it should not have been done.
//		// if (axis != -1) buf.append(Constants.AXISSTRINGS[axis]);
//		if (axis != Axis.NONE && axis != Axis.CHILD)
//			buf.append(Axis.AXISSTRINGS[axis]);
//		buf.append(expression.pprint());
//		if (predicates != null)
//			for (int i = 0; i < predicates.size(); i++)
//				buf.append(((XQueryExpression) predicates.get(i)).pprint());
//		return buf.toString();
//	}

	// #############################################################################
	// CLONE STUFF
	// #############################################################################
	public void addParentExpression(XQueryExpression parentExpression) {
		addParentExpression(parentExpression);
		expression.addParentExpression(parentExpression);
		if (predicates != null)
			for (int i = 0; i < predicates.size(); i++)
				 ((XQueryExpression) predicates.get(i)).addParentExpression(parentExpression);
	}

	//    public synchronized Object clone()
	//    throws CloneNotSupportedException {
	//        try {
	//            Step newobj = (Step)super.clone();
	//            newobj.setPredicates((getPredicates() == null) ? null : XQueryExpressionTools.totalClone(getPredicates()));
	//            newobj.setSeparator(getSeparator());
	//            newobj.setAxis(getAxis());
	//            return  newobj;
	//        }
	//        catch (Exception e) {
	//            //System.err.println(e.getMessage()) ;
	//            throw new CloneNotSupportedException("Error in cloning Step");
	//        }
	//    }
	//    
	//    public synchronized Object clone(HashMap mapvars)
	//    throws CloneNotSupportedException {
	//        try {
	//            Step newobj = (Step)super.clone(mapvars);
	//            if (newobj == null) return null ;
	//            newobj.setPredicates((getPredicates() == null) ? null : XQueryExpressionTools.totalClone(getPredicates(), mapvars));
	//            newobj.setSeparator(getSeparator());
	//            newobj.setAxis(getAxis());
	//            return  newobj;
	//        }
	//        catch (Exception e) {
	//            //System.err.println(e.getMessage()) ;
	//            throw new CloneNotSupportedException("Error in cloning Step");
	//        }
	//    }

	// #############################################################################
	// RESTRUCTURATION STUFF
	// #############################################################################

	public boolean hasPredicates() {
		if (predicates != null && !predicates.isEmpty())
			return true;
		return false;
	}

	public boolean hasPositionPredicates() {
		if (predicates != null)
			for (int i = 0; i < predicates.size(); i++) {
				XQueryExpression predExpr = (XQueryExpression) predicates.get(i);
				// test the type of the predicates
				if (predExpr instanceof ValueInteger)
					return true;
			}
		return false;
	}

	// #############################################################################
	// INTERFACE STUFF
	// #############################################################################

	/**
	 * 
	 * @return the namespace if any null otherwise
	 */
	public String getNameSpace() {
		return nameSpace;
	}

	/**
	 * 
	 * @return the localname if any null otherwise
	 */
	public String getLocalName() {
		return localName;
	}

	/**
	 * 
	 * @return the constant representing the wilcard
	 */
	public byte getWildCardType() {
		return wild;
	}

	/**
	 * 
	 * @return false if no wildcard is present
	 */
	public boolean hasWildcard() {
		return wild != StepExpr.NO_WILDCARD;
	}

	/**
	 * 
	 * @return true if step if *
	 */
	public boolean isWildcard() {
		return wild == StepExpr.WILDCARD;
	}

	/**
	 * 
	 * @return true if step is qname with * as namespace
	 */
	public boolean isNameSpaceWildcard() {
		return isWildcard() || wild == StepExpr.NAMESPACE_WILDCARD;
	}

	/**
	 * 
	 * @return true if step is qname with * as localname
	 */
	public boolean isLocalNameWildcard() {
		return isWildcard() || wild == StepExpr.LOCALNAME_WILDCARD;
	}

	/**
	 * 
	 * @return the constant representing the kind of the step
	 */
	public byte getKindTest() {
		return kindtest;
	}

	/**
	 * 
	 * @return the (schema) type used in element or attribute test
	 */
	public org.xquark.schema.Type getType() {
		return type;
	}

	/**
	 * 
	 * @return true if the type used in element test is nillable
	 */
	public boolean isTypeNillable() {
		return isTypeNillable;
	}

	//	methods for path resolving

	public static final String NAME_WILDCARD = "*";
	public XNode xnode = null;

	//	public StepExprImpl(byte axis, XNode xnode) {
	//		this.axis = axis;
	//		this.xnode = xnode;
	//	}
	//
	//	public StepExprImpl(XNode xnode) {
	//		this.axis = Axis.CHILD;
	//		this.xnode = xnode;
	//	}

	public XNode getXNode() {
		return xnode;
	}

	public boolean match(XNode node) {
		if (node == null)
			return false;
		if (isTypeTest()) { // true if local name == null
			return typeMatch(node);
		} else // namespace matching
			{
			if (typeMatch(node) == false) {
				return false;
			}
			if (getNamespace() == null) {
				if (node.getNamespace() == null) {
					// namespace match
				} else {
					return false;
				}
			} else if (isNamespaceWildcard()) {
				// namespace match
			} else if (node.getNamespace() == null) {
				return false;
			} else if (getNamespace().equals(node.getNamespace())) {
				// namespace match
			} else {
				return false;
			}

			// matching local name
			if (this.getLocalName() == null) {
				if (node.getLocalName() == null) {
					// local name match
				} else {
					return false;
				}
			} else if (this.isLocalNameWildcard()) {
				// local name match
			} else if (node.getLocalName() == null) {
				return false;
			} else if (this.getLocalName().equals(node.getLocalName())) {
				// local name match
			} else {
				return false;
			}
			return true;
		}
	}
	public boolean isTypeTest() {
		return xnode.getLocalName() == null;
	}
	private boolean typeMatch(XNode node) {
		return ((getTypeTest() == node.getType()) || (getTypeTest() == NodeKind.NODE));
	}
	public byte getTypeTest() {
		return xnode.getType();
	}
	public String getNamespace() {
		return xnode.getNamespace();
	}
	public boolean isNamespaceWildcard() {
		return isNameTest() && (xnode.getNamespace() != null) && xnode.getNamespace().equals(NAME_WILDCARD);
	}
	public boolean isNameTest() {
		return xnode.getLocalName() != null;
	}

}
