/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */

package org.xquark.xquery.parser;

import java.util.*;

public class TypeSwitchExpression extends XQueryBinaryOperatorExpression implements Cloneable {

	private static final String RCSRevision = "$Revision: 1.1 $";
	private static final String RCSName = "$Name:  $";

	// cannot be null
	protected HashMap cases = null;
	// can be null
	protected Variable defaultVariable = null;

	// #############################################################################
	// VISITOR STUFF
	// #############################################################################

	public void accept(ParserVisitor visitor) throws XQueryException {
		visitor.visit(this);
	}

	// #############################################################################
	// CONSTRUCTOR STUFF
	// #############################################################################

	public TypeSwitchExpression(XQueryExpression switchExpression, HashMap cases, Variable defaultVariable, XQueryExpression defaultExpression, TypeVisitor typevisitor) throws TypeException, XQueryException {
		super(switchExpression, defaultExpression);
		setCases(cases);
		setDefaultVariable(defaultVariable);
		if (typevisitor != null)
			accept(typevisitor);
	}

	// #############################################################################
	// ATTRIBUTE ACCESS STUFF
	// #############################################################################

	public void setParentUnit(XQueryUnit parentUnit) {
		if (parentUnit == null)
			return;
		this.parentUnit = parentUnit;
		super.setParentUnit(parentUnit);
		Set entries = cases.entrySet();
		Iterator it = entries.iterator();
		while (it.hasNext()) {
			Map.Entry entry = (Map.Entry) it.next();
			((QName) entry.getKey()).setParentUnit(parentUnit);
			((XQueryExpression) entry.getValue()).setParentUnit(parentUnit);
		}
		if (defaultVariable != null)
			defaultVariable.setParentUnit(parentUnit);
	}

	public HashMap getCases() {
		return cases;
	}

	public void setCases(HashMap cases) throws XQueryException {
		// cannot be null
		if (cases == null)
			throw new XQueryException("Cases of TypeSwitchExpression cannot be null.");
		this.cases = cases;
		Set entries = cases.entrySet();
		Iterator it = entries.iterator();
		while (it.hasNext()) {
			Map.Entry entry = (Map.Entry) it.next();
			QName qname = (QName) entry.getKey();
			qname.setParentUnit(parentUnit);
			qname.setParentExpression(this);
			XQueryExpression expr = (XQueryExpression) entry.getValue();
			expr.setParentUnit(parentUnit);
			expr.setParentExpression(this);
		}
	}

	public Variable getDefaultVariable() {
		return defaultVariable;
	}

	public void setDefaultVariable(Variable defaultVariable) throws XQueryException {
		this.defaultVariable = defaultVariable;
		if (defaultVariable != null) {
			this.defaultVariable.setParentUnit(parentUnit);
			this.defaultVariable.setParentExpression(this);
		}
	}

	// #############################################################################
	// PRINT STUFF
	// #############################################################################

	public String print() {
		StringBuffer buf = new StringBuffer();
		buf.append("typeswitch (" + expression1.pprint() + ") ");
		ArrayList casesAsList = new ArrayList(cases.values());
		for (int i = 0; i < casesAsList.size(); i++)
			buf.append(((CaseClauseExpression) casesAsList.get(i)).print());
		buf.append(" default ");
		if (defaultVariable != null)
			buf.append(defaultVariable.pprint());
		buf.append(" return " + expression2.pprint());
		return buf.toString();
	}

	// #############################################################################
	// CLONE STUFF
	// #############################################################################
	public void addParentExpression(XQueryExpression parentExpression) {
		addParentExpression(parentExpression);
		if (defaultVariable != null)
			defaultVariable.addParentExpression(parentExpression);
		Set entries = cases.entrySet();
		Iterator it = entries.iterator();
		while (it.hasNext()) {
			Map.Entry entry = (Map.Entry) it.next();
			((QName) entry.getKey()).addParentExpression(parentExpression);
			((XQueryExpression) entry.getValue()).addParentExpression(parentExpression);
		}
	}

//	public synchronized Object clone() throws CloneNotSupportedException {
//		try {
//			TypeSwitchExpression newobj = (TypeSwitchExpression) super.clone();
//			newobj.setCases((getCases() == null) ? null : XQueryExpressionTools.totalCloneHashMap(getCases()));
//			newobj.setDefaultVariable((getDefaultVariable() == null) ? null : (Variable) getDefaultVariable().clone());
//			return newobj;
//		}
//		catch (Exception e) {
//			//System.err.println(e.getMessage()) ;
//			throw new CloneNotSupportedException("Error in cloning Step");
//		}
//	}
//
//	public synchronized Object clone(HashMap mapvars) throws CloneNotSupportedException {
//		try {
//			TypeSwitchExpression newobj = (TypeSwitchExpression) super.clone(mapvars);
//			if (newobj == null)
//				return null;
//			newobj.setCases((getCases() == null) ? null : XQueryExpressionTools.totalCloneHashMap(getCases(), mapvars));
//			newobj.setDefaultVariable((getDefaultVariable() == null) ? null : (Variable) getDefaultVariable().clone(mapvars));
//			return newobj;
//		}
//		catch (Exception e) {
//			//System.err.println(e.getMessage()) ;
//			throw new CloneNotSupportedException("Error in cloning Step");
//		}
//	}

}
