/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */


package org.xquark.xquery.parser;

import java.util.ArrayList;

import org.xquark.xpath.Axis;
import org.xquark.xquery.parser.primitivefunctions.fnfunctions.FunctionCOLLECTION;
import org.xquark.xquery.parser.primitivefunctions.fnfunctions.FunctionDISTINCT_VALUES;
import org.xquark.xquery.parser.util.UnitEnvironment;

public class TypingLocatedExpressionsVisitor extends DefaultParserVisitor {
    private static final String RCSRevision = "$Revision: 1.1 $";
    private static final String RCSName     = "$Name:  $";
    
    ArrayList resVect = new ArrayList(1);
    UnitEnvironment environment = null;
    
    public TypingLocatedExpressionsVisitor(UnitEnvironment environment) throws XQueryException {
        if (environment == null) throw new XQueryException("Environment of TypingLocatedExpressionsVisitor cannot be null");
        this.environment = environment;
    }
    
    public void clearResult() { resVect.clear(); }
    public void setResult(ArrayList resVect) { if (resVect == null) this.resVect.clear(); else this.resVect =  resVect; }
    public ArrayList getResult() { return resVect; }
    
    //	public void visit(AggregateFunctionCall arg) throws XQueryException
    //	public void visit(AttributeValuePair arg) throws XQueryException;
    //	public void visit(BinOpANDExpression arg) throws XQueryException;
    //	public void visit(BinOpORExpression arg) throws XQueryException;
    //	public void visit(CastTreatExpression arg) throws XQueryException;
    //	public void visit(ContextDeclaration arg) throws XQueryException;
    //	public void visit(Dereference arg) throws XQueryException;
    //	public void visit(Element arg) throws XQueryException
    
    public void visit(FLWRExpression arg) throws XQueryException {
        //org.xquark.xml.misc.Debug.print(this,"Start TypingLocatedExpressionsVisitor -> FLWRExpression ( " + arg.print() + " ) - resVect = " + resVect);
        ArrayList variables = arg.getVariables();
        environment.addVariables(variables);
        arg.getReturnClause().accept(this);
        environment.removeVariables(variables);
        //org.xquark.xml.misc.Debug.print(this,"End   TypingLocatedExpressionsVisitor -> FLWRExpression ( " + arg.print() + " ) - resVect = " + resVect);
    }
    //	public void visit(FunctionCall arg) throws XQueryException
    //  public void visit(FunctionDefinition arg) throws XQueryException
    //	public void visit(ITEExpression arg) throws XQueryException
    //	public void visit(LibraryFunctionCall arg) throws XQueryException
    //	public void visit(ListOpAFTERExpression arg) throws XQueryException;
    //	public void visit(ListOpArithExpression arg) throws XQueryException;
    //	public void visit(ListOpBEFOREExpression arg) throws XQueryException;
    //	public void visit(ListOpCompExpression arg) throws XQueryException;
    //	public void visit(ListOpEXCEPTExpression arg) throws XQueryException;
    //	public void visit(ListOpINTERSECTExpression arg) throws XQueryException;
    //	public void visit(ListOpUNIONExpression arg) throws XQueryException;
    
    public void visit(LocatedExpression arg, int stepNum) throws XQueryException {
        //org.xquark.xml.misc.Debug.print(this,"Start TypingLocatedExpressionsVisitor -> LocatedExpression ( " + arg.print() + " ) - resVect = " + resVect);
        ArrayList steps = arg.getSteps();
        if (stepNum < 0 || steps.size() < stepNum) return;
        Step tmpStep = (Step)steps.get(stepNum);
        TypingLocatedExpressionsVisitor TypingLocatedExpressionsVisitor = new TypingLocatedExpressionsVisitor(environment);
        tmpStep.accept(TypingLocatedExpressionsVisitor);
        ArrayList vectStep = TypingLocatedExpressionsVisitor.getResult();
        if (vectStep.isEmpty()) {
            throw new TypeException("Could not extract path from step : " + tmpStep);
        }
        resVect.addAll(vectStep);
        //org.xquark.xml.misc.Debug.print(this,"End   TypingLocatedExpressionsVisitor -> LocatedExpression ( " + arg.print() + " ) - resVect = " + resVect);
    }
    
    public void visit(LocatedExpression arg) throws XQueryException {
        //org.xquark.xml.misc.Debug.print(this,"Start TypingLocatedExpressionsVisitor -> LocatedExpression ( " + arg.print() + " ) - resVect = " + resVect);
        ArrayList tmpVect = new ArrayList();
        ArrayList steps = arg.getSteps();
        TypingLocatedExpressionsVisitor TypingLocatedExpressionsVisitor = new TypingLocatedExpressionsVisitor(environment);
        for (int i = 0; i < steps.size(); i++) {
            Step tmpStep = (Step)steps.get(i);
            TypingLocatedExpressionsVisitor.clearResult();
            tmpStep.accept(TypingLocatedExpressionsVisitor);
            ArrayList vectStep = TypingLocatedExpressionsVisitor.getResult();
            if (vectStep.isEmpty()) {
                throw new TypeException("Could not extract path from step : " + tmpStep);
            }
            tmpVect.addAll(vectStep);
        }
        resVect.addAll(tmpVect);
        //org.xquark.xml.misc.Debug.print(this,"End   TypingLocatedExpressionsVisitor -> LocatedExpression ( " + arg.print() + " ) - resVect = " + resVect);
    }
    public void visit(NodeType arg) throws XQueryException {
        //org.xquark.xml.misc.Debug.print(this,"Start TypingLocatedExpressionsVisitor -> NodeType ( " + arg.print() + " ) - resVect = " + resVect);
        //int kind = arg.getKind();
        //if (kind != Constants.NODE_NODETYPE) arg.setKind(Constants.NODE_NODETYPE);
        resVect.add(new Step(true,Axis.CHILD,arg,null,null));
        //ArrayListUtil.add(resVect,arg.print());
        //if (kind != Constants.NODE_NODETYPE) arg.setKind(kind);
        //org.xquark.xml.misc.Debug.print(this,"End   TypingLocatedExpressionsVisitor -> NodeType ( " + arg.print() + " ) - resVect = " + resVect);
    }
    //	public void visit(Parameter arg) throws XQueryException;
    //	public void visit(PrimitiveFunctionCall arg) throws XQueryException;
    public void visit(QName arg) throws XQueryException {
        //org.xquark.xml.misc.Debug.print(this,"Start TypingLocatedExpressionsVisitor -> QName ( " + arg.print() + " ) - resVect = " + resVect);
        resVect.add(new Step(true,Axis.CHILD,arg,null,null));
        //ArrayListUtil.add(resVect,arg.printWithNameSpace());
        //		ArrayListUtil.add(resVect,arg.printWithNameSpace());
        //org.xquark.xml.misc.Debug.print(this,"End   TypingLocatedExpressionsVisitor -> QName ( " + arg.print() + " ) - resVect = " + resVect);
    }
    //	public void visit(QuantifiedExpression arg) throws XQueryException;
    //	public void visit(RangeExpression arg) throws XQueryException;
    //	public void visit(RootNodeFunctionCall arg) throws XQueryException
    public void visit(SortedExpression arg) throws XQueryException {
        XQueryExpression expression = arg.getExpression();
        TypingLocatedExpressionsVisitor TypingLocatedExpressionsVisitor = new TypingLocatedExpressionsVisitor(environment);
        expression.accept(TypingLocatedExpressionsVisitor);
        resVect = TypingLocatedExpressionsVisitor.getResult();
    }
    
    public void visit(Step arg) throws XQueryException {
        //org.xquark.xml.misc.Debug.print(this,"Start TypingLocatedExpressionsVisitor -> Step ( " + arg.print() + " ) - resVect = " + resVect);
        // int separator = arg.getSeparator();
        // int axis = arg.getAxis();
        XQueryExpression expression = arg.getExpression();
        // StringBuffer buf = new StringBuffer();
        
        // if (separator != -1) buf.append(Constants.SEPARATORSTRINGS[separator]);
        // if (axis != -1) buf.append(Constants.AXISSTRINGS[axis]);
        TypingLocatedExpressionsVisitor TypingLocatedExpressionsVisitor = new TypingLocatedExpressionsVisitor(environment);
        expression.accept(TypingLocatedExpressionsVisitor);
        ArrayList tmpList = TypingLocatedExpressionsVisitor.getResult();
        if (tmpList.isEmpty()) resVect.add(arg);
        else {
        	// ADD   22/04/03
        	// setting qtype if any
        	((Step)tmpList.get(tmpList.size()-1)).setQType(arg.getQType());
        	// end ADD
            ((Step)tmpList.get(0)).setAxis(arg.getAxis());
            resVect.addAll(tmpList);
        }
        //org.xquark.xml.misc.Debug.print(this,"End   TypingLocatedExpressionsVisitor -> Step ( " + arg.print() + " ) - resVect = " + resVect);
    }
    //	public void visit(TypeSwitchExpression arg) throws XQueryException;
    //	public void visit(UnOpMinusExpression arg) throws XQueryException;
    //	public void visit(Value arg) throws XQueryException;
    //	public void visit(ValueBoolean arg) throws XQueryException;
    //	public void visit(ValueFloat arg) throws XQueryException;
    //	public void visit(ValueInteger arg) throws XQueryException;
    //	public void visit(ValueString arg) throws XQueryException;
    public void visit(Variable arg) throws XQueryException {
        //org.xquark.xml.misc.Debug.print(this,"Start TypingLocatedExpressionsVisitor -> Variable ( " + arg.print() + " ) - resVect = " + resVect);
        XQueryExpression expression = arg.getExpression();
        if (environment != null && environment.getReplaceVar()) expression.accept(this);
        //org.xquark.xml.misc.Debug.print(this,"End   TypingLocatedExpressionsVisitor -> Variable ( " + arg.print() + " ) - resVect = " + resVect);
    }
    //	public void visit(XQueryBinaryOperatorExpression arg) throws XQueryException
    //	public void visit(XQueryBooleanOperatorExpression arg) throws XQueryException;
    //	public void visit(XQueryExpression arg) throws XQueryException
    /*
        public void visit(XQueryExpressionSequence arg) throws XQueryException {
        //org.xquark.xml.misc.Debug.print(this,"Start TypingLocatedExpressionsVisitor -> XQueryExpressionSequence ( " + arg.print() + " ) - resVect = " + resVect);
        ArrayList subExpressions = arg.getSubExpressions();
        if (subExpressions != null)
            for (int i = 0; i < subExpressions.size(); i++)
                ((XQueryExpression)subExpressions.get(i)).accept(this);
        //org.xquark.xml.misc.Debug.print(this,"End   TypingLocatedExpressionsVisitor -> XQueryExpressionSequence ( " + arg.print() + " ) - resVect = " + resVect);
    }
     */
    //	public void visit(XQueryListBinaryOperatorExpression arg) throws XQueryException;
    //	public void visit(XQueryListUnaryOperatorExpression arg) throws XQueryException;
    //	public void visit(XQueryUnaryOperatorExpression arg) throws XQueryException
    //	public void visit(XQueryVoid arg) throws XQueryException;
    
    // primitive functions
    
    
    public void visit(FunctionCOLLECTION arg) throws XQueryException {
        //org.xquark.xml.misc.Debug.print(this,"Start TypingLocatedExpressionsVisitor -> FunctionDISTINCT_VALUES ( " + arg.print() + " ) - resVect = " + resVect);
        resVect.add(new Step(false,Axis.CHILD,arg,null,null));
        //org.xquark.xml.misc.Debug.print(this,"End   TypingLocatedExpressionsVisitor -> FunctionDISTINCT_VALUES ( " + arg.print() + " ) - resVect = " + resVect);
    }
    
    
    public void visit(FunctionDISTINCT_VALUES arg) throws XQueryException {
        //org.xquark.xml.misc.Debug.print(this,"Start TypingLocatedExpressionsVisitor -> FunctionDISTINCT_VALUES ( " + arg.print() + " ) - resVect = " + resVect);
        TypingLocatedExpressionsVisitor TypingLocatedExpressionsVisitor = new TypingLocatedExpressionsVisitor(environment);
        arg.getArgument(0).accept(TypingLocatedExpressionsVisitor);
        ArrayList tmplist = TypingLocatedExpressionsVisitor.getResult();
        resVect.addAll(tmplist);
        //org.xquark.xml.misc.Debug.print(this,"End   TypingLocatedExpressionsVisitor -> FunctionDISTINCT_VALUES ( " + arg.print() + " ) - resVect = " + resVect);
    }
    
    
}
