/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */

/*
 * ValidateExpression.java
 *
 * Created on 26 septembre 2002, 17:22
 */

package org.xquark.xquery.parser;


public class ValidateExpression extends XQueryExpression implements Cloneable {
	
	private static final String RCSRevision = "$Revision: 1.1 $";
	private static final String RCSName     = "$Name:  $";
	
	protected XQueryExpression schemaContext = null;
	// cannot be null
	protected XQueryExpression validateExpression = null;
	
	// #############################################################################
	// VISITOR STUFF
	// #############################################################################
	
	public void accept(ParserVisitor visitor) throws XQueryException {
		visitor.visit(this);
	}
	
	// #############################################################################
	// CONSTRUCTOR STUFF
	// #############################################################################
	
	public ValidateExpression(XQueryExpression schemaContext, XQueryExpression validateExpression, TypeVisitor typeVisitor) throws TypeException, XQueryException {
		setSchemaContext(schemaContext);
		setValidateExpression(validateExpression);
		if (typeVisitor != null) accept(typeVisitor);
	}
	
	// #############################################################################
	// ATTRIBUTE ACCESS STUFF
	// #############################################################################
	
	public void setParentUnit(XQueryUnit parentUnit) {
		if (parentUnit == null) return;
		this.parentUnit = parentUnit;
		super.setParentUnit(parentUnit);
		if (schemaContext != null) schemaContext.setParentUnit(parentUnit);
		validateExpression.setParentUnit(parentUnit);
	}
	
	public void setSchemaContext(XQueryExpression schemaContext) {
		this.schemaContext = schemaContext;
		if (schemaContext != null) {
			this.schemaContext.setParentUnit(parentUnit);
			this.schemaContext.setParentExpression(this);
		}
	}
	
	public XQueryExpression getSchemaContext() { return schemaContext; }
	
	public void setValidateExpression(XQueryExpression validateExpression) throws XQueryException {
		// cannot be null
		if (validateExpression == null) throw new XQueryException("validateExpression cannot be null.");
		this.validateExpression = validateExpression;
		this.validateExpression.setParentUnit(parentUnit);
		this.validateExpression.setParentExpression(this);
	}
	
	public XQueryExpression getValidateExpression() { return validateExpression; }
	
	// #############################################################################
	// PRINT STUFF
	// #############################################################################
	
	public String print() {
		StringBuffer buf = new StringBuffer();
		buf.append("validate");
		if (schemaContext != null) buf.append(" " + schemaContext.print());
		buf.append(" { " + validateExpression + " }");
		return buf.toString();
	}
	
	// #############################################################################
	// CLONE STUFF
	// #############################################################################
	
//	public synchronized Object clone() throws CloneNotSupportedException {
//		try {
//			ValidateExpression newobj = (ValidateExpression)super.clone();
//			newobj.setSchemaContext(getSchemaContext());
//			newobj.setValidateExpression(getValidateExpression());
//			return newobj;
//		}
//		catch (XQueryException e) {
//			throw new CloneNotSupportedException("Error in cloning Element");
//		}
//	}
//	
//	public synchronized Object clone(HashMap mapvars) throws CloneNotSupportedException {
//		try {
//			ValidateExpression newobj = (ValidateExpression)super.clone(mapvars);
//			if (newobj == null) return null;
//			newobj.setSchemaContext(getSchemaContext());
//			newobj.setValidateExpression(getValidateExpression());
//			return newobj;
//		}
//		catch (XQueryException e) {
//			throw new CloneNotSupportedException("Error in cloning Element");
//		}
//	}
}
