/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */

package org.xquark.xquery.parser;

import java.util.ArrayList;

import org.xquark.xquery.parser.util.Constants;

public class Variable extends XQueryExpression implements Cloneable {
	private static final String RCSRevision = "$Revision: 1.3 $";
	private static final String RCSName = "$Name:  $";

	// OLIVIER - W3C WD 16 August 2002
	protected SequenceTypeExpression typeDeclaration = null;
	// cannot be null or empty string
	protected QName varName = null;
	protected XQueryExpression expression = null;
	// cannot be -1
	protected int bindingtype = -1;
	// boolean telling if the root of this variable is a document
	//protected boolean hasDocumentRoot = false;
	// paths beginning with variable
	protected ArrayList xpaths = null;
	protected FLWRExpression parentFLWR = null;
	//   - W3C WD 26 November 2002
	protected Variable positionVar = null;

	// #############################################################################
	// VISITOR STUFF
	// #############################################################################

	public void accept(ParserVisitor visitor) throws XQueryException {
		visitor.visit(this);
	}

	// #############################################################################
	// CONSTRUCTOR STUFF
	// #############################################################################

	public Variable(QName varName, TypeVisitor typevisitor) throws TypeException, XQueryException {
		setVarName(varName);
		// if (typevisitor != null) accept(typevisitor);
	}

	// #############################################################################
	// ATTRIBUTES ACCESS STUFF
	// #############################################################################

	public FLWRExpression getParentFLWRExpression() {
		return parentFLWR;
	}

	public void setParentFLWRExpression(FLWRExpression parentFLWR) {
		this.parentFLWR = parentFLWR;
	}

	public void setParentExpression(XQueryExpression expression) {
		if (expression == null)
			return;
		if (parentExpression == null)
			parentExpression = new ArrayList(1);
		// else parentExpression.clear();
		if (!parentExpression.contains(expression))
			parentExpression.add(expression);
	}

	public void setParentUnit(XQueryUnit parentUnit) {
		if (parentUnit == null)
			return;
		this.parentUnit = parentUnit;
		// OLIVIER - W3C WD 16 August 2002
		if (typeDeclaration != null)
			typeDeclaration.setParentUnit(parentUnit);
		varName.setParentUnit(parentUnit);
		if (expression != null)
			expression.setParentUnit(parentUnit);
	}

	public void setExpression(XQueryExpression expression, TypeVisitor typevisitor) throws TypeException, XQueryException {
		this.expression = expression;
		if (this.expression != null) {
			expression.setParentExpression(this);
			expression.setParentUnit(parentUnit);
//			if (expression instanceof LocatedExpression) {
//				XQueryExpression tmpexpr = ((LocatedExpression)expression).getStep(0).getExpression();
//				if (tmpexpr instanceof Variable)
//					hasDocumentRoot = ((Variable)tmpexpr).hasDocumentRoot();
//				else if (tmpexpr instanceof FunctionDOC)
//					hasDocumentRoot = true;
//				else
//					hasDocumentRoot = false;
//			}
//			else if (expression instanceof Variable)
//				hasDocumentRoot = ((Variable)expression).hasDocumentRoot();
//			else
//				hasDocumentRoot = false;
		}
//		else
//			hasDocumentRoot = false;
		if (typevisitor != null)
			accept(typevisitor);
	}

	public void setExpression(XQueryExpression expression) throws XQueryException {
		this.expression = expression;
		if (this.expression != null) {
			expression.setParentExpression(this);
			expression.setParentUnit(parentUnit);
//			if (expression instanceof LocatedExpression) {
//				XQueryExpression tmpexpr = ((LocatedExpression)expression).getStep(0).getExpression();
//				if (tmpexpr instanceof Variable)
//					hasDocumentRoot = ((Variable)tmpexpr).hasDocumentRoot();
//				else if (tmpexpr instanceof FunctionDOC)
//					hasDocumentRoot = true;
//				else
//					hasDocumentRoot = false;
//			}
//			else if (expression instanceof Variable)
//				hasDocumentRoot = ((Variable)expression).hasDocumentRoot();
//			else
//				hasDocumentRoot = false;
		}
//		else
//			hasDocumentRoot = false;
	}

	public void setBindingType(int bindingtype) throws XQueryException {
		if (bindingtype == -1 || bindingtype > Constants.MAX_BINDINGTYPE)
			throw new XQueryException("bindingtype of Variable has incorrect value " + bindingtype);
		this.bindingtype = bindingtype;
	}

	public XQueryExpression getExpression() {
		return expression;
	}
	public int getBindingType() {
		return bindingtype;
	}

	public boolean isInput() {
		if (expression == null) return false;
		if (expression instanceof RootNodeFunctionCall || (expression instanceof LocatedExpression && ((LocatedExpression)expression).getExpression() instanceof RootNodeFunctionCall))
			return true;
		return false;
	}
	
	public ArrayList getSubPathList() {
		return xpaths;
	}

	// attention if a variable is present this method only returns the variable
	// attention this method erases the NodeTypes found at the end of a path
	public ArrayList getUniqueSubPathList() throws XQueryException {
		if (xpaths == null)
			return null;
		ArrayList tmpList = null;
		if (xpaths.contains(this)) {
			tmpList = new ArrayList(1);
			tmpList.add(this);
		}
		else {
			tmpList = new ArrayList();
			for (int i = 0; i < xpaths.size(); i++) {
				XQueryExpression tmpExpr = (XQueryExpression) xpaths.get(i);
				if (tmpExpr instanceof LocatedExpression) {
					ArrayList steps = ((LocatedExpression) tmpExpr).getSteps();
					if (((Step) steps.get(steps.size() - 1)).getExpression() instanceof NodeType) {
						try {
							tmpExpr = (LocatedExpression) tmpExpr.clone();
						}
						catch (CloneNotSupportedException e) {
							throw new XQueryException(e.getMessage());
						}
						steps = ((LocatedExpression) tmpExpr).getSteps();
						steps.remove(steps.size() - 1);
						tmpExpr.setQType(((Step) steps.get(steps.size() - 1)).getQType());
					}
				}
				if (tmpList.isEmpty())
					tmpList.add(tmpExpr);
				else {
					boolean found = false;
					for (int j = 0; j < tmpList.size(); j++)
						if (tmpExpr.toSimpleString().equals(((XQueryExpression) tmpList.get(j)).toSimpleString())) {
							found = true;
							break;
						}
					if (!found)
						tmpList.add(tmpExpr);
				}
			}
		}
		return tmpList;
	}

	/**
	 * Returns the PathNode list of the defining expression
	 *
	 */
	public ArrayList getXTrees() {
		return this.getExpression().getXTrees();
	}

	// OLIVIER - W3C WD 16 August 2002
	public void setTypeDeclaration(SequenceTypeExpression typeDeclaration) {
		this.typeDeclaration = typeDeclaration;
		if (typeDeclaration != null) {
			this.typeDeclaration.setParentUnit(parentUnit);
			this.typeDeclaration.setParentExpression(this);
		}
	}
	public SequenceTypeExpression getTypeDeclaration() {
		return typeDeclaration;
	}

	public void setVarName(QName varName) throws XQueryException {
		if (varName == null || varName.getName().equals(""))
			throw new XQueryException("name of Variable cannot be null or empty string");
		this.varName = varName;
		this.varName.setParentUnit(parentUnit);
		this.varName.setParentExpression(this);
	}

	public QName getVarName() {
		return varName;
	}
	public String getName() {
		return varName.getName();
	}
	public void setName(String name) {
		varName.setName(name);
	}

	public void addSubPath(XQueryExpression expr) {
		if (xpaths == null)
			xpaths = new ArrayList();
		if (expr != null && !xpaths.contains(expr))
			xpaths.add(expr);
	}

	// #############################################################################
	// PRINT STUFF
	// #############################################################################

//	public String toString() {
//		// OLIVIER - W3C WD 16 August 2002
//		if (typeDeclaration == null)
//			return "$" + varName.print();
//		else
//			return typeDeclaration.print() + " $" + varName.print();
//	}

	public String print() {
		// OLIVIER - W3C WD 16 August 2002
		if (typeDeclaration == null)
			return "$" + varName.print();
		else
			return typeDeclaration.print() + " $" + varName.print();
	}

	public String printBinding() {
		//if (xpaths != null)
		//	for (int i=0;i<xpaths.size();i++)
		//		System.out.println("Path associated to " + this + " : " + xpaths.get(i));

		StringBuffer buf = new StringBuffer();
		if (bindingtype == Constants.FOR_BINDINGTYPE) {
			buf.append(" for ");
			buf.append(print());
			buf.append(" in ");
			if (expression != null)
				buf.append(expression);
		}
		else if (bindingtype == Constants.LET_BINDINGTYPE) {
			buf.append(" let ");
			buf.append(print());
			buf.append(" := ");
			if (expression != null)
				buf.append(expression);
		}
		else if (bindingtype == Constants.OUTERFOR_BINDINGTYPE) {
			buf.append(" outerfor ");
			buf.append(print());
			buf.append(" in ");
			if (expression != null)
				buf.append(expression);
		}
		else if (bindingtype == Constants.OUTERLET_BINDINGTYPE) {
			buf.append(" outerlet ");
			buf.append(print());
			buf.append(" := ");
			if (expression != null)
				buf.append(expression);
		}
		else if (bindingtype == Constants.QUANTIFIER_BINDINGTYPE) {
			buf.append(print());
			buf.append(" in ");
			if (expression != null)
				buf.append(expression);
		}
		else if (bindingtype == Constants.TYPESWITCH_BINDINGTYPE) {
			if (expression != null)
				buf.append(expression);
			buf.append(" ");
			buf.append(print());
		}
		if (positionVar != null) {
			buf.append(" at ");
			buf.append(positionVar);
		}
		return buf.toString();
	}

	// #############################################################################
	// CLONE STUFF
	// #############################################################################
	public void addParentExpression(XQueryExpression parentExpression) {
		addParentExpression(parentExpression);
		varName.addParentExpression(parentExpression);
		if (expression != null)
			expression.addParentExpression(parentExpression);
	}

	/**
	 * Clone the structure
	 */
//	public synchronized Object clone() throws CloneNotSupportedException {
//		/*
//		 try {
//		    Variable newobj = (Variable)super.clone();
//		    newobj.setVarName((getVarName() == null) ? null : (QName) getVarName().clone());
//		    newobj.setExpression((getExpression() == null) ? null : (XQueryExpression) getExpression().clone());
//		    newobj.setBindingType(getBindingType()) ;
//		    return  newobj;
//		}
//		catch (Exception e) {
//		    System.err.println(e.getMessage()) ;
//		    throw new CloneNotSupportedException("Error in cloning Variable");
//		}
//		 */
//		return this;
//	}
//
//	public synchronized Object clone(HashMap mapvars) throws CloneNotSupportedException {
//		try {
//			// if this variables has already be cloned somewhere,
//			// don't clone it again.
//			Object obj = null;
//
//			///*DEBUG*/System.out.print ("CLONING " + this + "(" + this.hashCode () + ") in " + mapvars+ "(") ;
//			///*DEBUG*/Iterator it = mapvars.keySet ().iterator () ;
//			///*DEBUG*/while (it.hasNext ())
//			///*DEBUG*/{
//			///*DEBUG*/	Variable vari = (Variable) it.next () ;
//			///*DEBUG*/	System.out.print (" " + vari.hashCode ()) ;
//			///*DEBUG*/}
//			///*DEBUG*/System.out.println (")") ;
//
//			if ((obj = mapvars.get(this)) != null) {
//				return obj;
//			}
//			Variable newobj = (Variable) super.clone(mapvars);
//			if (newobj == null)
//				return null;
//
//			// OLIVIER - W3C WD 16 August 2002
//			newobj.setTypeDeclaration((getTypeDeclaration() == null) ? null : (SequenceTypeExpression) (getTypeDeclaration().clone()));
//
//			newobj.setVarName((getVarName() == null) ? null : (QName) (getVarName().clone())); // pas de verification de variable
//
//			//newobj.setName((getName() == null) ? null : getName() + "_new") ; // pas de verification de variable
//			newobj.setName((getName() == null) ? null : getName()); // pas de verification de variable // CHANGE   31/07/2002
//
//			newobj.setExpression((getExpression() == null) ? null : (XQueryExpression) (getExpression().clone(mapvars)));
//			//OUf, le chef a dit qu'au niveau du mediateur, AlgebraExpression est toujours a nul !
//			//Donc pas de clonage recursif
//			//NON --->	newobj.setAlgebraExpression((getAlgebraExpression () == null) ? null : (Expression) getAlgebraExpression ().clone (mapvars));
//			newobj.setBindingType(getBindingType());
//			mapvars.put(this, newobj);
//
//			return newobj;
//
//		}
//		catch (Exception e) {
//			//System.err.println(e.getMessage()) ;
//			throw new CloneNotSupportedException("Error in cloning Variable");
//		}
//	}

}
