/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */


package  org.xquark.xquery.parser;



// START_COMMON

public abstract class XQueryBinaryOperatorExpression extends XQueryExpression implements Cloneable {
    private static final String RCSRevision = "$Revision: 1.1 $";
    private static final String RCSName     = "$Name:  $";
    
    XQueryExpression expression1 = null;
    XQueryExpression expression2 = null;
    
    // #############################################################################
    // VISITOR STUFF
    // #############################################################################
    
    public void accept(ParserVisitor visitor) throws XQueryException {
        visitor.visit(this);
    }
    
    // #############################################################################
    // CONSTRUCTOR STUFF
    // #############################################################################
    
    //	public XQueryBinaryOperatorExpression()
    //	{
    //	}
    
    public XQueryBinaryOperatorExpression(XQueryExpression expression1, XQueryExpression expression2) throws XQueryException {
        setExpression1(expression1);
        setExpression2(expression2);
    }
    
    // #############################################################################
    // ATTRIBUTE ACCESS STUFF
    // #############################################################################
    
    public void setParentUnit(XQueryUnit parentUnit) {
        if (parentUnit == null) return;
        this.parentUnit = parentUnit;
        expression1.setParentUnit(parentUnit);
        expression2.setParentUnit(parentUnit);
    }
    
    public XQueryExpression getExpression1() { return expression1; }
    public void setExpression1(XQueryExpression expression1) throws XQueryException {
        if (expression1 == null) throw new XQueryException("expression1 of XQueryBinaryOperatorExpression cannot be null");
        this.expression1 = expression1;
        this.expression1.setParentUnit(parentUnit);
        this.expression1.setParentExpression(this);
    }
    
    public XQueryExpression getExpression2() { return expression2; }
    public void setExpression2(XQueryExpression expression2) throws XQueryException {
        if (expression2 == null) throw new XQueryException("expression2 of XQueryBinaryOperatorExpression cannot be null");
        this.expression2 = expression2;
        this.expression2.setParentUnit(parentUnit);
        this.expression2.setParentExpression(this);
    }
    
    public XQueryExpression getLeftOperand() { return expression1; }
    
    public XQueryExpression getRightOperand() { return expression2; }
    
    // #############################################################################
    // CLONE STUFF
    // #############################################################################
    public void addParentExpression(XQueryExpression parentExpression) {
        addParentExpression(parentExpression) ;
        expression1.addParentExpression(parentExpression);
        expression2.addParentExpression(parentExpression);
    }
    
//    public synchronized Object clone()
//    throws CloneNotSupportedException {
//        try {
//            XQueryBinaryOperatorExpression newobj = (XQueryBinaryOperatorExpression)super.clone();
//            newobj.setExpression1((getExpression1() == null) ? null : (XQueryExpression) getExpression1().clone());
//            newobj.setExpression2((getExpression2() == null) ? null : (XQueryExpression) getExpression2().clone());
//            return  newobj;
//        }
//        catch (Exception e) {
//            //System.err.println(e.getMessage()) ;
//            throw new CloneNotSupportedException("Error in cloning XQueryBinaryOperatorExpression");
//        }
//    }
//    
//    public synchronized Object clone(HashMap mapvars)
//    throws CloneNotSupportedException {
//        try {
//            XQueryBinaryOperatorExpression newobj = (XQueryBinaryOperatorExpression)super.clone(mapvars);
//            newobj.setExpression1((getExpression1() == null) ? null : (XQueryExpression) getExpression1().clone(mapvars));
//            newobj.setExpression2((getExpression2() == null) ? null : (XQueryExpression) getExpression2().clone(mapvars));
//            return  newobj;
//        }
//        catch (Exception e) {
//            throw new CloneNotSupportedException("Error in cloning XQueryBinaryOperatorExpression");
//        }
//    }
    
    
}



