/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */


package org.xquark.xquery.parser;

import java.util.ArrayList;

import org.xquark.xquery.parser.util.ArrayListUtil;

public class XQueryExpressionSequence extends XQueryExpression {
    private static final String RCSRevision = "$Revision: 1.2 $";
    private static final String RCSName     = "$Name:  $";
    
    ArrayList subExpressions = null; // ArrayList of XQueryExpression
    
    // #############################################################################
    // VISITOR STUFF
    // #############################################################################
    
    public void accept(ParserVisitor visitor) throws XQueryException {
        visitor.visit(this);
    }
    
    // #############################################################################
    // CONSTRUCTOR STUFF
    // #############################################################################
    
    public XQueryExpressionSequence(ArrayList subExpressions, TypeVisitor typevisitor) throws TypeException, XQueryException {
        setSubExpressions(subExpressions);
        if (typevisitor != null) accept(typevisitor);
    }
    
    // #############################################################################
    // ATTRIBUTE ACCESS STUFF
    // #############################################################################
    
    public void setParentUnit(XQueryUnit parentUnit) {
        if (parentUnit == null) return;
        this.parentUnit = parentUnit;
        for (int j=0;j<subExpressions.size();j++)
            ((XQueryExpression)subExpressions.get(j)).setParentUnit(parentUnit);
    }
    
    public ArrayList getSubExpressions() { return subExpressions; }
    public void setSubExpressions(ArrayList subExpressions) throws XQueryException {
        if (subExpressions == null || subExpressions.isEmpty()) throw new XQueryException("subExpressions of XQueryExpressionSequence cannot be null or empty");
        this.subExpressions = subExpressions;
        for (int j=0;j<this.subExpressions.size();j++) {
            XQueryExpression expression = (XQueryExpression)this.subExpressions.get(j);
            expression.setParentExpression(this);
            expression.setParentUnit(parentUnit);
        }
    }
    
    public void addSubExpressions(XQueryExpression subExpression) throws XQueryException {
        subExpressions.add(subExpression);
        subExpression.setParentExpression(this);
        subExpression.setParentUnit(parentUnit);
    }
    
    public void addSubExpressions(int index,XQueryExpression subExpression) throws XQueryException {
        subExpressions.add(index,subExpression);
        subExpression.setParentExpression(this);
        subExpression.setParentUnit(parentUnit);
    }
    
	public ArrayList getSourceNames() {
		if (subExpressions == null) return null;
		ArrayList tmplist = null;
		for (int i = 0; i < subExpressions.size(); i++) {
			XQueryExpression expressioni = (XQueryExpression) subExpressions.get(i);
			if (expressioni.getSourceNames() != null) {
				if (tmplist == null) tmplist = (ArrayList)expressioni.getSourceNames().clone();
				else ArrayListUtil.add(tmplist, expressioni.getSourceNames());
			}
		}
		return tmplist;
	}

	public ArrayList getUrls() {
		if (subExpressions == null) return null;
		ArrayList tmplist = null;
		for (int i = 0; i < subExpressions.size(); i++) {
			XQueryExpression expressioni = (XQueryExpression) subExpressions.get(i);
			if (expressioni.getUrls() != null) {
				if (tmplist == null) tmplist = (ArrayList)expressioni.getUrls().clone();
				else ArrayListUtil.add(tmplist, expressioni.getUrls());
			}
		}
		return tmplist;
	}

    // #############################################################################
    // PRINT STUFF
    // #############################################################################
    
    public String print() {
        StringBuffer buf = new StringBuffer();
        for (int j=0;j<subExpressions.size();j++) {
            if (j!=0) buf.append(", ");
            buf.append(((XQueryExpression)subExpressions.get(j)).pprint());
        }
        return buf.toString();
    }
    
    // #############################################################################
    // CLONE STUFF
    // #############################################################################
    public void addParentExpression(XQueryExpression parentExpression) {
        addParentExpression(parentExpression) ;
        for (int j=0;j<subExpressions.size();j++)
            ((XQueryExpression)subExpressions.get(j)).addParentExpression(parentExpression);
    }
    
//    public synchronized Object clone() throws CloneNotSupportedException {
//        try {
//            XQueryExpressionSequence newobj = (XQueryExpressionSequence)super.clone();
//            newobj.setSubExpressions((getSubExpressions() == null) ? null : XQueryExpressionTools.totalClone(getSubExpressions()));
//            return  newobj;
//        }
//        catch (Exception e) {
//            throw new CloneNotSupportedException("Error in cloning List");
//        }
//    }
//    
//    public synchronized Object clone(HashMap mapvars)
//    throws CloneNotSupportedException {
//        try {
//            XQueryExpressionSequence newobj = (XQueryExpressionSequence)super.clone(mapvars);
//            if (newobj == null) return null ;
//            newobj.setSubExpressions((getSubExpressions() == null) ? null : XQueryExpressionTools.totalClone(getSubExpressions(), mapvars));
//            return  newobj;
//        }
//        catch (Exception e) {
//            throw new CloneNotSupportedException("Error in cloning List");
//        }
//    }
    
    // added   18/03/2002 --> no explanation needed!
//    public void setRoot(boolean root) {
//        if (root) {
//            if (subExpressions.size() == 1) ((XQueryExpression)subExpressions.get(0)).setRoot(true);
//        }
//        else
//            for (int j=0;j<subExpressions.size();j++) ((XQueryExpression)subExpressions.get(j)).setRoot(root);
//    }
    
}
