/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */


package org.xquark.xquery.parser;

import java.util.ArrayList;

import org.xquark.schema.SchemaManager;
import org.xquark.xquery.parser.util.ArrayListUtil;

public class XQueryFile implements Cloneable {
	private static final String RCSRevision = "$Revision: 1.2 $";
	private static final String RCSName = "$Name:  $";

	protected ArrayList units = null;
	// to manage typing
	//protected MetadataAccess metadata = null;
	protected SchemaManager schemaManager = null;

	public XQueryFile(ArrayList units, TypeVisitor typevisitor) {
		if (typevisitor != null) {
			this.schemaManager = typevisitor.getSchemaManager();
			//this.metadata = typevisitor.getMetaData();
		}
		setUnits(units);
	}

	// #############################################################################
	// VISITOR STUFF
	// #############################################################################

	public void accept(ParserVisitor visitor) throws XQueryException {
		visitor.visit(this);
	}

	// #############################################################################
	// ATTRIBUTE ACCESS STUFF
	// #############################################################################

	// metadata
//	public void setMetaData(MetadataAccess metadata) {
//		this.metadata = metadata;
//	}
//	public MetadataAccess getMetaData() {
//		return metadata;
//	}

	// schemaManager
	public void setSchemaManager(SchemaManager schemaManager) {
		this.schemaManager = schemaManager;
	}
	public SchemaManager getSchemaManager() {
		return schemaManager;
	}

	// units
	public ArrayList getUnits() {
		return units;
	}
	public void setUnits(ArrayList units) {
		this.units = units;
		if (units != null)
			for (int i = 0; i < units.size(); i++)
				 ((XQueryUnit) units.get(i)).setParentFile(this);
	}

	/**
	 * Methode ajoute conformement  l'accord No.1 du 18 septembre 2001.
	 *
	 */
	public XQueryUnit getUnit(int i) {
		return (XQueryUnit) units.get(i);
	}

	// #############################################################################
	// SOURCES STUFF
	// #############################################################################

	public ArrayList getSourceNames() {
		ArrayList sources = new ArrayList();
		for (int i = 0; i < units.size(); i++) {
			ArrayListUtil.add(sources, ((XQueryUnit) units.get(i)).getSourceNames());
		}
		return sources;
	}

	public ArrayList getUrls() {
		ArrayList urls = new ArrayList();
		for (int i = 0; i < units.size(); i++) {
			ArrayListUtil.add(urls, ((XQueryUnit) units.get(i)).getUrls());
		}
		return urls;
	}

	// #############################################################################
	// PRINT STUFF
	// #############################################################################

	public String pprint() {
		StringBuffer buf = new StringBuffer();
		if (units != null)
			for (int i = 0; i < units.size(); i++) {
				if (i != 0)
					buf.append("; ");
				buf.append(" " + ((XQueryUnit) units.get(i)).pprint());
			}
		return buf.toString();
	}

	public String toString() {
		return pprint();
	}

	// #############################################################################
	// CLONE STUFF
	// #############################################################################

	//    public synchronized Object clone() throws CloneNotSupportedException {
	//        try {
	//            XQueryFile newobj = (XQueryFile)super.clone();
	//            newobj.setUnits((getUnits() == null) ? null : XQueryExpressionTools.totalClone(getUnits()));
	//            return  newobj;
	//        }
	//        catch (Exception e) {
	//            throw new CloneNotSupportedException("Error in cloning XQueryFile");
	//        }
	//    }
	//    
	//    public synchronized Object clone(HashMap mapvars)
	//    throws CloneNotSupportedException {
	//        try {
	//            XQueryFile newobj = (XQueryFile)super.clone();
	//            if (newobj == null) return null ;
	//            newobj.setUnits((getUnits() == null) ? null : XQueryExpressionTools.totalClone(getUnits(), mapvars));
	//            return  newobj;
	//        }
	//        catch (Exception e) {
	//            //System.err.println (e.getMessage ()) ;
	//            throw new CloneNotSupportedException("Error in cloning XQueryFile");
	//        }
	//    }

}
