/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */

package org.xquark.xquery.parser.primitivefunctions.xsfunctions;

import java.util.ArrayList;

import org.xquark.xquery.parser.*;

/*
Returns the value of $srcval represented as a string. If no argument is supplied, $srcval defaults to the context item (.).
If $srcval is the empty sequence, the zero-length string is returned.
If $srcval is a node, the function returns the string value of the node, as obtained using the string-value accessor defined in the [XQuery 1.0 and XPath 2.0 Data Model].
If $srcval is an atomic value, then the function returns the same string as is returned by the expression cast as xs:string ($srcval), except in the cases listed below:
If the type of $srcval is xs:anyURI, the URI is converted to a string without any escaping of special characters.
*/

public class FunctionSTRING extends PrimitiveFunctionCall {
    
    private static final String RCSRevision = "$Revision: 1.1 $";
    private static final String RCSName     = "$Name:  $";
    
    // #############################################################################
    // VISITOR STUFF
    // #############################################################################
    
    public void accept(ParserVisitor visitor) throws XQueryException {
        visitor.visit(this);
    }
    
    // #############################################################################
    // CONSTRUCTOR STUFF
    // #############################################################################
    
    public FunctionSTRING(ArrayList args, TypeVisitor typevisitor) throws TypeException, XQueryException {
        super(new QName(null,null,"string",null),args);
        if (typevisitor != null) accept(typevisitor);
    }
    
    public void setArguments(ArrayList arguments) throws XQueryException {
        if (arguments == null || arguments.size() != 1) throw new XQueryException("Only partial specification, function string should one argument");
        super.setArguments(arguments);
    }
    
    // #############################################################################
    // CLONE STUFF
    // #############################################################################
    public synchronized Object clone()
    throws CloneNotSupportedException {
        try {
            FunctionSTRING newobj = (FunctionSTRING)super.clone();
            return  newobj;
        }
        catch (Exception e) {
            //System.err.println(e.getMessage()) ;
            throw new CloneNotSupportedException("Error in cloning FunctionSTRING");
        }
    }
    
}


