/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */


package  org.xquark.xquery.parser.util;

import java.util.ArrayList;
import java.util.ListIterator;

import org.xquark.xquery.parser.QName;
import org.xquark.xquery.parser.Variable;

public class ArrayListUtil {
    private static final String RCSRevision = "$Revision: 1.1 $";
    private static final String RCSName     = "$Name:  $";

	/*
	Returns a ArrayList being the result of a cross product of a string and an ArrayList.
	*/
	public static void crossBefore (ArrayList al, String str) 
	{
		if (str == null || al == null || al.isEmpty()) return;
		for (int i = 0; i < al.size(); i++) 
			al.set(i,str + (String)al.get(i));
	}

	/*
	Returns a ArrayList being the result of a cross product of a vector and a string.
	*/
	public static void crossAfter (ArrayList al, String str) 
	{
		if (str == null || al == null || al.isEmpty()) return ;
		for (int i = 0; i < al.size(); i++) 
			al.set(i,(String)al.get(i) + str);
	}


	/*
	Returns a ArrayList being the result of a cross product of two vectors.
	Duplicates are removed.
	*/
	public static void crossAfter (ArrayList al, ArrayList add) 
	{
		if (al == null || add == null || add.isEmpty()) return;
		int i, j, diff;
		if (al.isEmpty())
		{
			for (j = 0; j < add.size(); j++) 
			{
				String strAdd = (String)add.get(j);
				if (!al.contains(strAdd)) al.add(strAdd);
			}
			return;
		}
		int size = al.size();
		diff = 0;

		for (i = 0; i < size; i++) 
			for (j = 0; j < add.size(); j++) 
			{
				String strAdd = (String)al.get(i) + (String)add.get(j);
				if (!al.contains(strAdd)) 
					al.add(size + (i + 1)*j - diff, strAdd);
				else diff++;
			}
		for (i=0;i<size;i++) al.remove(0);
	}


	/*
	Returns a ArrayList being the result of a cross product of two vectors
	and inserting between both elements a given string.
	Duplicates are removed.
	*/
	public static void crossAfter (ArrayList al, ArrayList add, String str) 
	{
		if (str == null || str.length() == 0)
		{
			crossAfter(al,add);
			return;
		}
		if (al == null || add == null || add.isEmpty()) return;
		int i, j, diff;
		if (al.isEmpty())
		{
			for (j = 0; j < add.size(); j++) 
			{
				String strAdd = str + (String)add.get(j);
				if (!al.contains(strAdd)) al.add(strAdd);
			}
			return;
		}
			
		int size = al.size();
		diff = 0;

		for (i = 0; i < size; i++) 
			for (j = 0; j < add.size(); j++) 
			{
				String strAdd = (String)al.get(i) + str + (String)add.get(j);
				if (!al.contains(strAdd)) 
					al.add(size + (i + 1)*j - diff, strAdd);
				else diff++;
			}
		for (i=0;i<size;i++) al.remove(0);
	}

	/*
	Adds an element not already present to a vector
	*/
	public static void add (ArrayList al, Object o) 
	{
		if (o == null) return ;
		if (al == null) return;
		if (!al.contains(o)) al.add(o);
	}

	/*
	Adds to vector to another without duplicates
	*/
	public static void add (ArrayList al, ArrayList add) 
	{
		if (al == null || add == null || add.isEmpty()) return;
		ListIterator li = add.listIterator();
		while (li.hasNext()) 
		{
			Object obj = li.next();
			if (!al.contains(obj)) al.add(obj);
		}
	}

	public static boolean areSame (ArrayList v1, ArrayList v2)
	{
		if ((v1 == null) && (v2 == null))
			return true ;
		if ((v1 == null) && (v2 != null))
			return false ;
		if ((v1 != null) && (v2 == null))
			return false ;
		if ((v1.size ()) != (v2.size ()))
			return false ;
		for (int i = 0 ; i < v1.size () ; i ++)
		{
			 if (!v2.contains (v1.get (i)))
				return false ;
		}
		return true ;
	}

	// is v2 included in v1
	public static boolean contains (ArrayList v1, ArrayList v2)
	{
		if ((v1 == null) && (v2 == null))
			return true ;
		if ((v1 == null) && (v2 != null))
			return false ;
		if ((v1 != null) && (v2 == null))
			return true ;
		if ((v1.size ()) < (v2.size ()))
			return false ;
		for (int i = 0 ; i < v2.size () ; i ++)
		{
			Object vi = v2.get (i) ;

			 if (!containsVariable (v1, (Variable) vi))
			 {
				return false ;
			}
		}
		return true ;
	}

	private static boolean isSameVariable (Variable var1, Variable var2) 
	{
		if ((var1 == null) && (var2 == null)) return true ;
		if ((var1 != null) && (var2 == null)) return false ;
		if ((var1 == null) && (var2 != null)) return false ;

		QName qn1 = var1.getVarName () ;
		QName qn2 = var2.getVarName () ;

		if ((qn1 == null) && (qn2 == null)) return true ;
		if ((qn1 != null) && (qn2 == null)) return false ;
		if ((qn1 == null) && (qn2 != null)) return false ;

		String ns1 = qn1.getNameSpace () ;
		String ns2 = qn2.getNameSpace () ;

		if ((ns1 != null) && (ns2 == null)) return false ;
		if ((ns1 == null) && (ns2 != null)) return false ;
		if ((ns1 != null) && (ns2 != null) && (! ns1.equals (ns2))) return false ;


		String pn1 = qn1.getPrefixName () ;
		String pn2 = qn2.getPrefixName () ;

		if ((pn1 != null) && (pn2 == null)) return false ;
		if ((pn1 == null) && (pn2 != null)) return false ;
		if ((pn1 != null) && (pn2 != null) && (! pn1.equals (pn2))) return false ;

		String ln1 = qn1.getLocalName () ;
		String ln2 = qn2.getLocalName () ;

		if ((ln1 != null) && (ln2 == null)) return false ;
		if ((ln1 == null) && (ln2 != null)) return false ;
		if ((ln1 != null) && (ln2 != null) && (! ln1.equals (ln2))) return false ;

		return true ;
	}


	public static boolean containsVariable (ArrayList al, Variable v)
	{
		for (int i = 0 ; i < al.size () ; i ++)
		{
			Variable vi  = (Variable) al.get (i) ;
			if (isSameVariable (vi, v))
				return true ;
		}
		return false ;
	}

	// is v2 included in v1
	public static boolean containsVariable (ArrayList v1, ArrayList v2)
	{
		if ((v1 == null) && (v2 == null))
			return true ;
		if ((v1 == null) && (v2 != null))
			return false ;
		if ((v1 != null) && (v2 == null))
			return true ;
		if ((v1.size ()) < (v2.size ()))
			return false ;
		for (int i = 0 ; i < v2.size () ; i ++)
		{
			Variable vi = (Variable) v2.get (i) ;

			 if (!containsVariable (v1, vi))
				return false ;
		}
		return true ;
	}


}

// end of file

