/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */


package  org.xquark.xquery.parser.util;




import java.util.Enumeration;
import java.util.Vector;




public class VectorUtil {
    private static final String RCSRevision = "$Revision: 1.1 $";
    private static final String RCSName     = "$Name:  $";

	/*
	Returns a Vector being the result of a cross product of a string and a Vector.
	Duplicates are removed.
	*/
	public static Vector crossBefore (Vector v, String str) 
	{
		if (str == null) 
		{
			return  v;
		}

		if (v == null) 
		{
			Vector resVect = new Vector();
			resVect.addElement(str);
			return  resVect;
		}

		for (int i = 0; i < v.size(); i++) 
		{
			v.setElementAt(str + (String)v.elementAt(i), i);
		}
		return  v;
	}


	/*
	Returns a Vector being the result of a cross product of a vector and a string.
	Duplicates are removed.
	*/
	public static Vector crossAfter (Vector v, String str) 
	{
		if (str == null) 
		{
			return  v;
		}

		if (v == null) 
		{
			Vector resVect = new Vector();
			resVect.addElement(str);
			return  resVect;
		}

		for (int i = 0; i < v.size(); i++) 
		{
			v.setElementAt((String)v.elementAt(i) + str, i);
		}
		return  v;
	}


	/*
	Returns a Vector being the result of a cross product of two vectors.
	Duplicates are removed.
	*/
	public static Vector crossAfter (Vector v, Vector vadd) 
	{
		if (vadd == null || vadd.isEmpty()) 
		{
			return  v;
		}
		if (v == null || v.isEmpty()) 
		{
			return  vadd;
		}
		int i, j, diff;
		int size = v.size();
		diff = 0;

		for (i = 0; i < size; i++) 
		{
			for (j = 0; j < vadd.size(); j++) 
			{
				String strAdd = (String)v.elementAt(i) + (String)vadd.elementAt(j);
				if (!v.contains(strAdd)) 
				{
					v.add(size + (i + 1)*j - diff, strAdd);
				}
				else 
				{
					diff++;
				}
			}
		}

		for (i = 0; i < size; i++) 
		{
			v.removeElementAt(0);
		}
		return  v;
	}


	/*
	Returns a Vector being the result of a cross product of two vectors
	and inserting between both elements a given string.
	Duplicates are removed.
	*/
	public static Vector crossAfter (Vector v, Vector vadd, String str) 
	{
		if (vadd == null || vadd.isEmpty()) 
		{
			return  v;
		}
		if (v == null || v.isEmpty()) 
		{
			return  vadd;
		}
		int i, j, diff;
		int size = v.size();
		diff = 0;

		for (i = 0; i < size; i++) 
		{
			for (j = 0; j < vadd.size(); j++) 
			{
				String strAdd = (String)v.elementAt(i) + str + (String)vadd.elementAt(j);
				if (!v.contains(strAdd)) 
				{
					v.add(size + (i + 1)*j - diff, strAdd);
				}
				else 
				{
					diff++;
				}
			}
		}

		for (i = 0; i < size; i++) 
		{
			v.removeElementAt(0);
		}
		return  v;
	}


	/*
	Adds an element not already present to a vector
	*/
	public static Vector add (Vector v, Object o) 
	{
		if (o == null) 
		{
			return  v;
		}

		if (v == null) 
		{
			v = new Vector();
			v.addElement(o);
		}
		else 
		{
			if (!v.contains(o)) 
			{
				v.add(o);
			}
		}

		return  v;
	}


	/*
	Adds to vector to another without duplicates
	*/
	public static Vector add (Vector v, Vector vadd) 
	{
		if (vadd == null || vadd.isEmpty()) 
		{
			return  v;
		}

		if (v == null || v.isEmpty()) 
		{
			v = vadd;
		}
		else 
		{
			for (Enumeration e = vadd.elements(); e.hasMoreElements();) 
			{
				Object obj = e.nextElement();
				if (!v.contains(obj)) 
				{
					v.add(obj);
				}
			}
		}

		return  v;
	}


	//	public static Vector addWithoutSub( Vector v, Vector vadd )
	//	{
	//		if (vadd == null) return v;
	//		if (v == null) v = vadd;
	//		else 
	//		{
	//			for (Enumeration enumvadd = vadd.elements();enumvadd.hasMoreElements();)
	//			{
	//				Object objAdd = enumvadd.nextElement();
	//				String tmpStrAdd = (String) objAdd;
	//				if (v.contains(objAdd)) continue;
	//				for (Enumeration enumv = v.elements();enumv.hasMoreElements();)
	//				{
	//					String tmpStr = (String)enumv.nextElement();
	//					if (tmpStr.startsWith(tmpStrAdd) break;
	//					if (tmpStrAdd.startsWith(tmpStr))
	//					{
	//						v.remove(tmpStr);
	//						v.add(tmpStrAdd);
	//					}
	//				}					
	//				v.add(obj);
	//			}
	//		}
	//		return v;	
	//	}

	/**
	 * Supposing two vector of Strings v1 and v2, says if there
	 * is at least one String of v1 common with v2.
	 */
	public static boolean hasCommonString (Vector v1, Vector v2)
	{
		if ((v1 == null) || (v2 == null))
			return true ;
		if ((v2.size () == 0))
			return true ;
		for (int i = 0 ; i < v1.size () ; i ++)
		{
			String s1 = (String) v1.elementAt (i) ;
			for (int j = 0 ; j < v2.size () ; j ++)
			{
			   String s2 = (String) v2.elementAt (j) ;
				if (s1.equalsIgnoreCase (s2))
					return true ;
			}
		}
		return false ;
	}

	public static boolean areSame (Vector v1, Vector v2)
	{
		if ((v1 == null) && (v2 == null))
			return true ;
		if ((v1 == null) && (v2 != null))
			return false ;
		if ((v1 != null) && (v2 == null))
			return false ;
		if ((v1.size ()) != (v2.size ()))
			return false ;
		for (int i = 0 ; i < v1.size () ; i ++)
		{
			 if (!v2.contains (v1.elementAt (i)))
				return false ;
		}
		return true ;
	}

	// is v2 included in v1
	public static boolean contains (Vector v1, Vector v2)
	{
		if ((v1 == null) && (v2 == null))
			return true ;
		if ((v1 == null) && (v2 != null))
			return false ;
		if ((v1 != null) && (v2 == null))
			return true ;
		if ((v1.size ()) < (v2.size ()))
			return false ;
		for (int i = 0 ; i < v2.size () ; i ++)
		{
			 if (!v1.contains (v2.elementAt (i)))
				return false ;
		}
		return true ;
	}


}



