/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * Copyright (C) 2003 XQuark Group.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */

package org.xquark.xquery.typing;

import org.xquark.schema.SimpleType;
import org.xquark.schema.datatypes.PrimitiveType;
import org.xquark.xquery.parser.XQueryException;

public class QTypeAtom extends QTypeItem implements QTypeVisitable {
	private static final String RCSRevision = "$Revision: 1.2 $";
	private static final String RCSName = "$Name:  $";

	public QTypeAtom(SimpleType simpleType) {
		setType(simpleType);
		subclass = ATOM;
		occurence = OCC_1_1;
	}
	public QTypeAtom(SimpleType simpleType, byte occurence) {
		setType(simpleType);
		subclass = ATOM;
		this.occurence = occurence;
	}

	// #############################################################################
	// VISITOR STUFF
	// #############################################################################

	public void accept(QTypeVisitor visitor) throws XQueryException {
		visitor.visit(this);
	}

	// get/set
//	public SimpleType getSimpleType() {
//		return (SimpleType)type;
//	}

	// equals

	public boolean equals(Object qtype) {
		//org.xquark.xml.misc.Debug.print(this,"QTypeAtom::equals");
		if (!(qtype instanceof QTypeAtom))
			return false;
//		if (((QTypeAtom) qtype).getOccurence() != occurence)
//			return false;
		return compare(type, ((QTypeAtom) qtype).getSimpleType());
	}

	// tells whether qtype is numeric
	public boolean isNumeric() {
		if (type == null)
			return false;
		else
			return isNumeric(type);
	}

	// tells whether qtype is boolean
	public boolean isBoolean() {
		if (type == null)
			return false;
		else
			return isBoolean(type);
	}

	// tells whether qtype is integer
	public boolean isInteger() {
		if (type == null)
			return false;
		else
			return isInteger(type);
	}

	// tells whether qtype is string
	public boolean isString() {
		if (type == null)
			return false;
		else
			return isString(type);
	}

	// tells whether qtype is an atom
	public boolean isAtom() {
		return true;
	}

//	// tells whether qtype can react to text()
//	public boolean isText() {
//		return false;
//	}

	// tells whether qtype is a date or collection of dates
	public boolean isDate() {
		if (type == null)
			return false;
		else
			return isDate(type);
	}

//	// tells whether qtype is a attribte or collection or sequence
//	public boolean isAttribute() {
//		return false;
//	}

//	// tells whether qtype is a ID, IDREF or IDREFS type
//	public boolean isIDAttribute() {
//		return false;
//	}

//	// tells whether qtype is a node
//	public boolean isNode() {
//		return false;
//	}

//	// tells whether qtype is a node with simple type
//	public boolean isSimpleTypeNode() {
//		return false;
//	}

//	// tells whether qtype is a collection
//	public boolean isCollection() {
//		return false;
//	}

//	// tells whether qtype is a document
//	public boolean isDocument() {
//		return false;
//	}

//	// tells whether qtype is multiple
//	public boolean isMultiple() {
//		return false;
//	}

	// tells whether qtype is atomic (atom or element with simple content
	public boolean isAtomic() {
		return !isMultiple();
	}

//	// tells whether qtype is mixed
//	public boolean isMixed() {
//		return false;
//	}

//	// tells whether qtype is element
//	public boolean isElement() {
//		return false;
//	}

	public boolean canBeComparedTo(QType compType) {
		if (compType == null)
			return false;
		if (!(compType instanceof QTypeAtom))
			return compType.canBeComparedTo(this);
		if (type == null)
			return true;
		if (((QTypeAtom) compType).getSimpleType() == null)
			return true;
		return canBeCompared(((SimpleType)type).getPrimitive().getType(), ((QTypeAtom) compType).getSimpleType().getPrimitive().getType());
	}

		public boolean canBeCastedInto(int castType) {
			if (castType == -1)
				return false;
			int myCastType = getCastType();
			if (myCastType == -1)
				return false;
			return QType.castTab[myCastType][castType];
		}

		public int getCastType() {
			if (type == null)
				return QType.PRIMITIVE_anySimpleType;
			switch (((SimpleType)type).getPrimitive().getType()) {
				//case PrimitiveType.VOID : return -1;
				case PrimitiveType.STRING :
					return QType.PRIMITIVE_string;
				case PrimitiveType.BOOLEAN :
					return QType.PRIMITIVE_boolean;
				case PrimitiveType.DECIMAL :
					return QType.PRIMITIVE_decimal;
				case PrimitiveType.DOUBLE :
					return QType.PRIMITIVE_double;
				case PrimitiveType.FLOAT :
					return QType.PRIMITIVE_float;
				case PrimitiveType.DURATION :
					return QType.PRIMITIVE_duration;
				case PrimitiveType.DATE_TIME :
					return QType.PRIMITIVE_dateTime;
				case PrimitiveType.TIME :
					return QType.PRIMITIVE_time;
				case PrimitiveType.DATE :
					return QType.PRIMITIVE_date;
				case PrimitiveType.GYEAR_MONTH :
					return QType.PRIMITIVE_gYearMonth;
				case PrimitiveType.GYEAR :
					return QType.PRIMITIVE_gYear;
				case PrimitiveType.GMONTH_DAY :
					return QType.PRIMITIVE_gMonthDay;
				case PrimitiveType.GDAY :
					return QType.PRIMITIVE_gDay;
				case PrimitiveType.GMONTH :
					return QType.PRIMITIVE_gMonth;
				case PrimitiveType.HEX_BINARY :
					return QType.PRIMITIVE_hexBinary;
				case PrimitiveType.BASE64_BINARY :
					return QType.PRIMITIVE_base64Binary;
				case PrimitiveType.ANY_URI :
					return QType.PRIMITIVE_anyURI;
				case PrimitiveType.QNAME :
					return QType.PRIMITIVE_Qname;
				case PrimitiveType.NOTATION :
					return QType.PRIMITIVE_NOTATION;
			}
			return -1;
		}
	/*
	// not bounded values !!!
	public static final int PRIMITIVE_anySimpleType = 1;
	public static final int PRIMITIVE_dayTimeDuration = 10;
	public static final int PRIMITIVE_yearMonthDuration = 21;
	*/

	// clone
	public Object clone() throws CloneNotSupportedException {
		QTypeAtom newObj = new QTypeAtom((SimpleType)type,occurence);
		return newObj;
	}
	// print
	public String toSimpleString() {
		if (type == null)
			return "QTypeAtom(anySimpleType)";
		else
			return "QTypeAtom(" + type.getName() + ")";
	}

	public String toString() {
		/*
		StringWriter strWriter = new StringWriter();
		try {
		    OutputFormat format = new OutputFormat(Method.XML, "iso-8859-1", true);
		    XMLSerializer serializer = new XMLSerializer(strWriter, format);
		    SchemaReader schemaReader = new SchemaReader(null);
		    schemaReader.setContentHandler(serializer.asContentHandler());
		    schemaReader.parse(simpleType);
		}
		catch (Exception e) { // empty on purpose
		}
		*/
		return "QTypeAtom(\n" + type + ")";
	}

}
