/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * Copyright (C) 2003 XQuark Group.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */

package org.xquark.xquery.typing;

import org.xquark.schema.ComplexType;
import org.xquark.schema.SchemaConstants;
import org.xquark.schema.Type;
import org.xquark.xquery.parser.*;

public class QTypeElement extends QTypeNode implements QTypeVisitable {
    private static final String RCSRevision = "$Revision: 1.2 $";
    private static final String RCSName     = "$Name:  $";

    
    XQueryExpression name = null;
    
    public QTypeElement(XQueryExpression name, Type type) {
        this.name = name;
        setType(type);
		subclass = ELEMENT;
		occurence = OCC_1_1;
		}
	public QTypeElement(XQueryExpression name, Type type, byte occurence) {
		this.name = name;
		setType(type);
		subclass = ELEMENT;
		this.occurence = occurence;
		}
    
	// #############################################################################
	// VISITOR STUFF
	// #############################################################################

	public void accept(QTypeVisitor visitor) throws XQueryException {
		visitor.visit(this);
	}

    // get/set
//    public Type getType() {
//        return type;
//    }
            
    public XQueryExpression getName() {
        return name;
    }
    
//    public void setName(XQueryExpression name) throws TypeException {
//        //if (name == null) throw new TypeException("name of QTypeElement cannot be null");
//        this.name = name;
//    }
    
    // equals
    public boolean equals(Object qtype) {
        //org.xquark.xml.misc.Debug.print(this,"QTypeElement::equals");
        if (!(qtype instanceof QTypeElement)) return false;
//		if (((QTypeElement) qtype).getOccurence() != occurence)
//			return false;
        //org.xquark.xml.misc.Debug.print(this,"compare: "+compare(type, ((QTypeElement) qtype).getType()));
        //org.xquark.xml.misc.Debug.print(this,"names: "+name.equals(((QTypeElement) qtype).getName()));
        //org.xquark.xml.misc.Debug.print(this,"name1: "+name);
        //org.xquark.xml.misc.Debug.print(this,"name2: "+((QTypeElement) qtype).getName());
        return compare(type, ((QTypeElement) qtype).getType()) && name.equals(((QTypeElement) qtype).getName());
    }
    
    // tells whether qtype is numeric
    public boolean isNumeric() {
        return isNumeric(type);
        /*
         if (type.isSimpleType()) return isNumeric(type);
        if (((ComplexType) type).getContentType() == SchemaConstants.MIXED) return true;
        if (((ComplexType) type).isExtension()) return true;
        else return false;
         */
    }
    
    // tells whether qtype is boolean
    public boolean isBoolean() { return isBoolean(type); }
    
    // tells whether qtype is integer
    public boolean isInteger() { return isInteger(type); }
    
    // tells whether qtype is string
    public boolean isString() { 
        if (this.isMixed()) return true; // this is temporary
        return isString(type); 
    }
    
    // tells whether qtype is an atom
    public boolean isAtom() { return false; }
    //        if (type.isSimpleType()) return true;
    //        // else if (((ComplexType) type).getContentType() == SchemaConstants.MIXED) return true;
    //        else if (((ComplexType) type).isExtension()) return true;
    //        else return false;
    //    }
    
    // tells whether qtype can react to text()
    public boolean isText() { return true; }
    
    // tells whether qtype is a date or collection of dates
    public boolean isDate() { return isDate(type); }
        
    // tells whether qtype is a node
    public boolean isNode() { return true; }
    
    // tells whether qtype is a node with simple type
    public boolean isSimpleTypeNode() {    	
        if (type.getValueType() != null) return true;
        if (type.getName() != null && type.getName().equals(Type.ANY_TYPE)) return true;
        return false;
    }

    // tells whether qtype is multiple
//    public boolean isMultiple() {
//    	if (type.getName() != null && type.getName().equals("anyType"))
//    		return true; 
//    	return false; 
//    }
    
    // tells whether qtype is atomic (atom or element with simple content
    public boolean isAtomic()  {
    	if (type.getName() != null && type.getName().equals(TypeVisitor.SC_ANYTYPE))
    		return true;
        if (this.isMixed()) return !isMultiple(); // this is temporary -- add  2002/09/02
        if (type.getValueType() != null) return !isMultiple();
        return false;
    }
    
    // tells whether qtype is mixed
    public boolean isMixed() {
        if (type instanceof ComplexType && ((ComplexType) type).getContentType() == SchemaConstants.MIXED) return true;
        return false;
    }
    
    // tells whether qtype is element
    public boolean isElement() { return true; }

    /*
     public boolean canBeComparedTo(QType compType) {
        if (compType == null) return false;
        if (!(compType instanceof QTypeElement)) return false;
        if (type.isSimpleType() && compType.getType().isSimpleType() &&
            ((SimpleType)type).getPrimitive().getType() == ((SimpleType)compType.getType()).getPrimitive().getType())
            return true;
        return false;
    }
     */
    
    //public Object getSubType(String namespace, String localname, boolean isAttribute, boolean inDepth, ArrayList history) throws TypeException {
    // Olivier: 2002/11/19
//    public Object getSubType(QName qname, int axis, boolean inDepth, ArrayList history, ArrayList reslist, SchemaManager schemamanager) throws TypeException {
//        String namespace = qname.getNameSpace();
//        String localname = qname.getLocalName();
//        // for now not used in sub typing, only for sort expressions
//        /*
//        if (name instanceof QName) {
//            QName qname = (QName)name;
//            nameStr = qname.getNameSpace() + ":" + qname.getLocalName();
//            if (((namespace == null && qname.getNameSpace() == null) ||
//                (namespace != null && namespace.equals(qname.getNameSpace())) ||
//                (namespace != null && namespace.equals("*"))) &&
//                (localname.equals(qname.getLocalName())  || localname.equals("*"))) return this;
//        }
//                 */
//		Object obj = null;
//		if (axis == Axis.SELF) {
//			QName tmpqname = (QName)name;
//			if (((namespace == null && tmpqname.getNameSpace() == null) ||
//				(namespace != null && (namespace.equals("*") || namespace.equals(tmpqname.getNameSpace())))) &&
//				(localname.equals(tmpqname.getLocalName())  || localname.equals("*"))) {
//					if (!inDepth) return this;
//					else obj = this;
//					} 
//			else {
//				if (!inDepth) return null;
//			}
//		}       
//		else if (axis == Axis.ATTRIBUTE) {
//			if (type.getName() != null && type.getName().equals(TypeVisitor.SC_ANYTYPE)) {
//				// TODO maybe test if the namespace exists ...
//				return new QTypeAttribute(qname,schemamanager.getAnySimpleType());
//			}
//			obj = type.getAttributeDeclaration(namespace,localname);
//			// change   04/06/2003
//			//if (obj == null && !inDepth) {
//			if (obj == null) {
//				//if (namespace == null) namespace = "";
//				if (localname == null) localname = "";
//				Collection decls = type.getAttributeDeclarations();
//				if (decls != null) {
//					Iterator it = decls.iterator();
//					ArrayList list = new ArrayList();
//					while (it.hasNext()) {
//						AttributeDeclaration decl = (AttributeDeclaration)it.next();
//						if (((namespace == null && decl.getNamespace() == null) ||
//						(namespace != null && namespace.equals(decl.getNamespace())) ||
//						(namespace != null && namespace.equals("*"))) &&
//						(localname.equals(decl.getName()) || localname.equals("*"))) {
//							QType tmpQType = null;
//							try {
//								tmpQType = getQTypeFromDeclaration(decl);
//							}
//							catch (Exception e) {}
//							if (tmpQType != null) list.add(tmpQType);
//						}
//					}
//					if (!list.isEmpty())
//						//if (list.size() > 1) obj = new QTypeSequence(list);
//						if (list.size() > 1) obj = new QTypeUnion(list);
//						else obj = list.get(0);
//				}
//				// change   04/06/2003
//				//return obj;
//				if (!inDepth) return obj;
//			}
//		}
//		else if (axis == Axis.CHILD) {
//			if (type.getName() != null && type.getName().equals(TypeVisitor.SC_ANYTYPE)) {
//				// TODO maybe test if the namespace exists ...
//				return new QTypeElement(qname,type);
//			}
//			obj = type.getElementDeclaration(namespace,localname);
//			if (obj == null && !inDepth) {
//				//if (namespace == null) namespace = "";
//				if (localname == null) localname = "";
//				Collection decls = type.getElementDeclarations();
//				if (decls != null) {
//					Iterator it = decls.iterator();
//					ArrayList list = new ArrayList();
//					while (it.hasNext()) {
//						Declaration decl = (Declaration)it.next();
//						if (((namespace == null && decl.getNamespace() == null) ||
//						(namespace != null && namespace.equals(decl.getNamespace())) ||
//						(namespace != null && namespace.equals("*"))) &&
//						(localname.equals(decl.getName()) || localname.equals("*"))) {
//							QType tmpQType = null;
//							try {
//								tmpQType = getQTypeFromDeclaration(decl);
//							}
//							catch (Exception e) {}
//							if (tmpQType != null) list.add(tmpQType);
//						}
//					}
//					if (!list.isEmpty())
//						// if (list.size() > 1) obj = new QTypeSequence(list);
//						if (list.size() > 1) obj = new QTypeUnion(list);
//						else obj = list.get(0);
//				}
//				return obj;
//			}
//		}
//		//if (!inDepth || obj != null || type.getValueType() != null) return obj;
//        if (!inDepth || type.getValueType() != null) return obj;
//        if (obj != null) {
//        	if (obj instanceof Declaration) {
//				try {
//					obj = this.getQTypeFromDeclaration((Declaration)obj);
//				} catch (Exception e) {
//					// do nothing
//					obj = null;
//				}
//        	}
//        	if (obj != null)
//        		reslist.add(obj);
//        }
//		//String nameStr = qname.getNameSpace() + ":" + qname.getLocalName();
//        //history.add(nameStr);
//        Collection decls = type.getElementDeclarations();
//        Iterator it = decls.iterator();
//        while (it.hasNext()) {
//            Declaration decl = (Declaration)it.next();
//            String declName = decl.getNamespace() + ":" + decl.getName();
//            if (history.contains(declName)) {  // this is to handle recursions
//                // history.remove(declName);
//                continue;
//            }
//            history.add(declName);
//            QType tmpQType = null;
//            try {
//                tmpQType = getQTypeFromDeclaration(decl);
//            }
//            catch (Exception e) {
//            	// do nothing
//            }
//            if (tmpQType != null) {
//            	
//                obj = tmpQType.getSubType(qname, axis, inDepth, history, reslist, schemamanager);
//				//if (obj != null) return obj;
//                if (obj != null) reslist.add(obj);
//                
//            }
//            history.remove(declName);
//        }
//        //history.remove(nameStr);
//        return null;
//    }
    
//    public QType applyTEXTFunction(boolean inDepth, ArrayList history, SchemaManager schemamanager) throws TypeException {
//    	QType qtype = null;
//        if (type.getValueType() != null)
//			qtype = new QTypeAtom((SimpleType) schemamanager.getType(SchemaConstants.XMLSCHEMA_URI,TypeVisitor.SC_STRING));
//        else if (type.getContentType() == Type.MIXED) {
//			qtype = new QTypeAtom((SimpleType) schemamanager.getType(SchemaConstants.XMLSCHEMA_URI,TypeVisitor.SC_STRING),OCC_0_N);
//        }
//        else if (type.getContentType() == Type.TEXT_ONLY)
//            qtype = new QTypeAtom((SimpleType) schemamanager.getType(SchemaConstants.XMLSCHEMA_URI,TypeVisitor.SC_STRING));
//        if (!inDepth)
//        	return qtype;
////		if (reslist.isEmpty()) {
////			// add text
////			reslist.add(new QTypeAtom(schemamanager.getStringType()));
////		}
////		// add current type
////		reslist.add(this);
////		// first get all elements
//		Collection decls = type.getElementDeclarations();
//		if (decls != null) {
//			Iterator it = decls.iterator();
//			while (it.hasNext()) {
//				Declaration decl = (Declaration)it.next();
//				String declName = decl.getNamespace() + ":" + decl.getName();
//				if (history.contains(declName)) {  // this is to handle recursions
//					// history.remove(declName);
//					continue;
//				}
//				history.add(declName);
//				QType tmpQType = null;
//				try {
//					tmpQType = getQTypeFromDeclaration(decl);
//				}
//				catch (Exception e) {}
//				if (tmpQType != null) {
//					tmpQType.applyTEXTFunction(inDepth, history, schemamanager);
//					if (tmpQType != null) {
//						if (tmpQType.isMultiple())
//							return tmpQType;
//						else
//							qtype = tmpQType;
//					}
//				}
//				history.remove(declName);
//			}
//		}       
//        return qtype;
//    }
    
    public QType applyDATAFunction() throws TypeException {
        if (type.getValueType() != null) return new QTypeAtom(type.getValueType());
        if (type.getContentType() == Type.MIXED) {
            //SchemaManager sm = new SchemaManager();
            return new QTypeAtom(null);
        }
        return null;
    }
    
//    public QType applySHALLOWFunction() throws TypeException {
//        if (type.getValueType() != null) return this;
//        // type instanceof ComplexType
//        ComplexType newCT = new ComplexType(null,null,null);
//        ContentModel newCM = new ContentModel();
//        newCM.setContentType(SchemaConstants.EMPTY);
//        newCT.setContentModel(newCM);
//        
//        ComplexType ct = (ComplexType)type;
//        Collection collAttr = ct.getAttributeDeclarations();
//        if (collAttr != null) {
//            Iterator it = collAttr.iterator();
//            try {
//                while (it.hasNext()) newCT.register((AttributeDeclaration)it.next());
//            }
//            catch (Exception e) {
//                throw new TypeException(e.getMessage());
//            }
//        }
//        return new QTypeElement(name,newCT);
//    }
    
//    public QType applyFILTERFunction() throws TypeException {
//        if (type.getValueType() != null) return this;
//        // type instanceof ComplexType
//        SchemaManager sm = new SchemaManager();
//        return new QTypeElement(name,sm.getType(SchemaConstants.XMLSCHEMA_URI, SchemaConstants.ANY_TYPE));
//    }
    
//    public QType applyNODEFunction(int axis, ArrayList history, ArrayList reslist, SchemaManager schemamanager) throws TypeException {
//    	if (axis == Axis.SELF) return this;
//    	else if (axis == Axis.CHILD) {
//	        if (type.getValueType() != null) return null;
//	        ComplexType complexType = (ComplexType) type;
//	        Collection col = complexType.getElementDeclarations();
//	        if (col != null) {
//	            Iterator it = col.iterator();
//	            ArrayList list = new ArrayList();
//	            while(it.hasNext()) {
//	                ElementDeclaration eltDecl = (ElementDeclaration)it.next();
//	                try {
//	                    QType tmpQType = getQTypeFromDeclaration(eltDecl);
//	                    if (tmpQType != null) list.add(tmpQType);
//	                }
//	                catch (Exception e) {}
//	            }
//	            //        if (!list.isEmpty()) return new QTypeSequence(list);
//	            if (!list.isEmpty()) {
//	            	if (list.size() > 1)
//	            		return new QTypeUnion(list);
//	            	return (QType)list.get(0);
//	        	} 
//	            if (col.isEmpty()) {
//	                if (compareQNames(complexType.getNamespace(),complexType.getName(), SchemaConstants.XMLSCHEMA_URI, SchemaConstants.ANY_TYPE))
//	                    return new QTypeElement(name, complexType);
//	            }
//	        }
//    	}
//    	else if (axis == Axis.DESCENDANT_OR_SELF) {
//    		// make union of processing instruction, comment, element, text
//    		// TODO
//    		// for now only text and element
//    		if (reslist.isEmpty()) {
//				// add text
//				reslist.add(new QTypeAtom(schemamanager.getStringType()));
//    		}
//			// add current type
//			reslist.add(this);
//    		// first get all elements
//			Collection decls = type.getElementDeclarations();
//			if (decls != null) {
//				Iterator it = decls.iterator();
//				while (it.hasNext()) {
//					Declaration decl = (Declaration)it.next();
//					String declName = decl.getNamespace() + ":" + decl.getName();
//					if (history.contains(declName)) {  // this is to handle recursions
//						// history.remove(declName);
//						continue;
//					}
//					history.add(declName);
//					QType tmpQType = null;
//					try {
//						tmpQType = getQTypeFromDeclaration(decl);
//					}
//					catch (Exception e) {}
//					if (tmpQType != null) {
//						tmpQType.applyNODEFunction(axis, history, reslist, schemamanager);
//						//if (obj != null) return obj;
//						//if (obj != null) reslist.add(obj);
//                
//					}
//					history.remove(declName);
//				}
//			}
//		}
//		return null;
//    }
 
	// clone
	public Object clone() throws CloneNotSupportedException {
		QTypeElement newObj = new QTypeElement(name,type,occurence);
		return newObj;
	}
    public String toSimpleString() {
        return "QTypeElement(" + name + " ," + type.getName() + ")";
    }
    
    public String toString() {
        /*
                StringWriter strWriter = new StringWriter();
        try {
            OutputFormat format = new OutputFormat(Method.XML, "iso-8859-1", true);
            XMLSerializer serializer = new XMLSerializer(strWriter, format);
            SchemaReader schemaReader = new SchemaReader(null);
            schemaReader.setContentHandler(serializer.asContentHandler());
            schemaReader.parse(type);
        }
        catch (Exception e) { // empty on purpose
        }
         */
        return "QTypeElement(" + name + " ,\n" + type + ")";
    }
    
}
