/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */

package org.xquark.xquery.xdbc;

import java.util.Collections;
import java.util.Iterator;

import org.xml.sax.helpers.AttributesImpl;
import org.xquark.util.XMLDocumentFragmentor;
import org.xquark.xml.xdbc.*;
import org.xquark.xpath.XTreeReader;

/**
 * Implementation of XMLDataSourceMetaData for the method getMetaData()
 */
public abstract class AbstractXMLDataSourceMetaData implements XMLDataSourceMetaData {
	private static final String RCSRevision = "$Revision: 1.1 $";
	private static final String RCSName     = "$Name:  $";
	public static final String METADATA_ELEMENT_NAME = "metadata";
	public static final String SOURCE_ATTR = "source";
	public static final String SCHEMAS_ELEMENT_NAME = "schemas";
	public static final String COLLECTIONS_ELEMENT_NAME = "collections";
	
	public static final String[] PROPERTY_LIST = {
		"xmldbc.maxConnections", "xmldbc.maxStatementLength",
		"xmldbc.maxActiveStatements", "xmldbc.maxResultSetLength",
		"xmldbc.maxUserNameLength", "xmldbc.dataSourceVersion",
		"xmldbc.vendor.name", "xmldbc.vendor.product",
		"xmldbc.vendor.version", "xmldbc.driver.version",
		"xmldbc.driver.product", "xmldbc.xpath",
		"xmldbc.xpath.version", "xmldbc.xquery",
		"xmldbc.xquery.version", "xmldbc.xquery.outerJoin",
		"xmldbc.twoPhaseCommit", "xmldbc.xquery.equality",
		"xmldbc.xquery.result", "xmldbc.xmlCollection",
		"xmldbc.xmlCollection.id", "xmldbc.xmlCollection.iterator",
		"xmldbc.xquery.multiCollection"
	};
	
	XMLConnection connection;
	
	protected AbstractXMLDataSourceMetaData(XMLConnection connection) {
		this.connection = connection;
	}
	
	public java.util.Set getCapabilities() {
		return Collections.EMPTY_SET;
	}
	
	public XMLDocument getMetaData() throws XMLDBCException {
		return new MetaDataDocument();
	}
	
	/**
	 * To get the XMLConnection object that produced this XMLDataSourceMetaData object.
	 * @return the XMLConnection object that produced this object.
	 * @throws XMLDBCException if a data source access error occurs.
	 */
	public XMLConnection getConnection() throws XMLDBCException {
		return connection;
	}
	
	public void setConnection(XMLConnection connection) throws XMLDBCException {
		this.connection = connection;
	}
	
	public String[] getPropertyList() {
		return PROPERTY_LIST;
	}
	
	///////////////////////////////////////////////////////////////////////////
	//                          INNER CLASSES
	///////////////////////////////////////////////////////////////////////////
	private class MetaDataDocument extends AbstractXMLDocument {
		private static final String RCSRevision = "$Revision: 1.1 $";
		private static final String RCSName     = "$Name:  $";
		
		private static final String METADATA_ID = "XMLDBC DataSource Metadata";
		
		public MetaDataDocument() {
			super(METADATA_ID, null, true);
		}
		
		public void remove() throws XMLDBCException, XMLDBCNotSupportedException {
			throw new XMLDBCNotSupportedException("This document is read-only");
		}
		
		public void getAsSAX() throws XMLDBCException, org.xml.sax.SAXException {
			AttributesImpl atts = new AttributesImpl();
			
			// start doc
			contentHandler.startDocument();
			
			// start root element
			contentHandler.startPrefixMapping(XTreeReader.SOURCE_METADATA_PREFIX, XTreeReader.SOURCE_METADATA_NAMESPACE);
			atts.addAttribute("", SOURCE_ATTR, "", "CDATA", connection == null? "":connection.getURL());
			contentHandler.startElement(XTreeReader.SOURCE_METADATA_NAMESPACE, METADATA_ELEMENT_NAME, "", atts);
			atts.clear();
			
			// serialize schemas
			contentHandler.startElement("", SCHEMAS_ELEMENT_NAME, "", atts);
			
			XMLDocumentFragmentor fragmentor = new XMLDocumentFragmentor(contentHandler, lexicalHandler);
			
			Iterator it = AbstractXMLDataSourceMetaData.this.getSchemaNamespaces().iterator();
			XMLDocument fragment;
			
			while (it.hasNext()) {
				fragment = AbstractXMLDataSourceMetaData.this.getSchema((String)it.next());
				if (fragment == null) continue ;
				fragment.setContentHandler(fragmentor);
				fragment.setLexicalHandler(fragmentor);
				fragment.getAsSAX();
			}
			
			contentHandler.endElement("", SCHEMAS_ELEMENT_NAME, "");
			
			// serialize path sets
			contentHandler.startElement("", COLLECTIONS_ELEMENT_NAME, "", atts);
			
			it = AbstractXMLDataSourceMetaData.this.getCollectionNames().iterator();
			
			while (it.hasNext()) {
				fragment = AbstractXMLDataSourceMetaData.this.getPathSet((String)it.next());
				if (fragment == null)
					continue ;
				fragment.setContentHandler(fragmentor);
				fragment.setLexicalHandler(fragmentor);
				fragment.getAsSAX();
			}
			
			contentHandler.endElement("", COLLECTIONS_ELEMENT_NAME, "");
			
			// end root element
			contentHandler.endElement(XTreeReader.SOURCE_METADATA_NAMESPACE, METADATA_ELEMENT_NAME, "");
			contentHandler.endPrefixMapping(XTreeReader.SOURCE_METADATA_PREFIX);
			
			// end doc
			contentHandler.endDocument();
		}
		
	}
}
