/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */

package org.xquark.xquery.xdbc;

import java.io.StringWriter;
import java.io.Writer;
import java.util.ArrayList;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.w3c.dom.Document;
import org.xml.sax.ContentHandler;
import org.xml.sax.ErrorHandler;
import org.xml.sax.SAXException;
import org.xml.sax.ext.LexicalHandler;
import org.xquark.serialize.XMLSerializer;
import org.xquark.util.DOMBuilder;
import org.xquark.util.NamespaceContextStack;
import org.xquark.xml.xdbc.*;
import org.xquark.xquery.parser.*;

public class XResultSetImpl implements XMLResultSet {
	private static final String RCSRevision = "$Revision: 1.2 $";
	private static final String RCSName = "$Name:  $";
	protected ContentHandler contentHandler = null;
	protected LexicalHandler lexicalhandler = null;
	protected ErrorHandler errorhandler = null;
	protected XMLStatement statement = null;
	protected XDBCResultSetInterface xdbcResulSet = null;
	protected ReconstructionVisitor visitor = null;
	protected XQueryExpression reconstructExpression = null;
	private boolean noNext = false;
	//	private boolean hasNext = true;
	private ArrayList varList = null;

	private int position = 0;
	private boolean firstHasNext = true;

	private XMLSerializer serializer = null;
	private DOMBuilder builder = null;
	private Document document = null;
	private static DocumentBuilder DOMCreator;
	/**********************************************************************************************/

	/**
	 * @see XMLResultSet#setContentHandler
	 */
	public void setContentHandler(ContentHandler contentHandler) {
		//org.xquark.xml.misc.Debug.print(this, "START - XResultSetImpl setContentHandler");
		if (contentHandler == null)
			throw new NullPointerException("ContentHandler cannot be null.");
		this.contentHandler = contentHandler;
		xdbcResulSet.setContentHandler(contentHandler);
		visitor.setContentHandler(contentHandler);
		//org.xquark.xml.misc.Debug.print(this, "STOP  - XResultSetImpl setContentHandler");
	}

	/**
	 * @see ResultSet#setLexicalHandler
	 */
	public void setLexicalHandler(LexicalHandler lexicalhandler) {
		//org.xquark.xml.misc.Debug.print(this, "START - XResultSetImpl setLexicalHandler");
		if (lexicalhandler == null)
			throw new NullPointerException("LexicalHandler cannot be null.");
		this.lexicalhandler = lexicalhandler;
		xdbcResulSet.setLexicalHandler(lexicalhandler);
		visitor.setLexicalHandler(lexicalhandler);
		//org.xquark.xml.misc.Debug.print(this, "STOP  - XResultSetImpl setLexicalHandler");
	}

	/**
	 * @see ResultSet#setErrorHandler
	 */
	public void setErrorHandler(ErrorHandler errorhandler) {
		//org.xquark.xml.misc.Debug.print(this, "START - XResultSetImpl setErrorHandler");
		if (errorhandler == null)
			throw new NullPointerException("ErrorHandler cannot be null.");
		this.errorhandler = errorhandler;
		xdbcResulSet.setErrorHandler(errorhandler);
		visitor.setErrorHandler(errorhandler);
		//org.xquark.xml.misc.Debug.print(this, "STOP  - XResultSetImpl setErrorHandler");
	}

	/**
	 * @see ResultSet#setErrorHandler
	 */
	public ContentHandler getContentHandler() {
		return contentHandler;
	}

	/**
	 * @see ResultSet#setErrorHandler
	 */
	public LexicalHandler getLexicalHandler() {
		return lexicalhandler;
	}

	/**
	 * @see ResultSet#setErrorHandler
	 */
	public ErrorHandler getErrorHandler() {
		return errorhandler;
	}

	/**********************************************************************************************/

	/**
	 * @see XMLResultSet#getResultCount
	 */
	public int getResultCount() throws XMLDBCException {
		return -1;
	}

	/**
	 * @see XMLResultSet#getStatement
	 */
	public XMLStatement getStatement() throws XMLDBCException {
		return statement;
	}

	/**
	 * @see XMLResultSet#close
	 */
	public void close() throws XMLDBCException {
		xdbcResulSet.close();
	}

	/**********************************************************************************************/

	/**
	 * @see XMLResultSet#isBeforeFirst
	 */
	public boolean isBeforeFirst() throws XMLDBCException {
		return 0 == position ? true : false;
	}

	/**
	 * @see XMLResultSet#nextAsDOM
	 */
	public Document nextAsDocument() throws XMLDBCException {
		try {
			createDOMBuilder();
			Document document = DOMCreator.newDocument();
			if (builder == null)
				builder = new DOMBuilder(document);
			else
				builder.setDocument(document);
			setContentHandler(builder);
			setLexicalHandler(builder);
			nextAsSAX();
			return document;
		} catch (Exception e) {
			throw new XMLDBCException(e.getMessage(), e);
		}
	}

	public org.w3c.dom.Node nextAsDOM() throws XMLDBCException {
		return nextAsDocument().getFirstChild();
	}

	public void nextAsDOM(org.w3c.dom.Element parent) throws XMLDBCException {
		try {
			if (builder == null)
				builder = new DOMBuilder(parent);
			else
				builder.setElement(parent);
			setContentHandler(builder);
			setLexicalHandler(builder);
			nextAsSAX();
		} catch (Exception e) {
			throw new XMLDBCException(e.getMessage(), e);
		}
	}

	/**
	 * @see XMLResultSet#nextAsString
	 */
	public java.lang.String nextAsString() throws XMLDBCException {
		StringWriter stringWriter = new StringWriter();
		nextAsStream(stringWriter);
		return stringWriter.toString();
	}

	public void nextAsStream(Writer out) throws XMLDBCException {
		try {
			if (serializer == null)
				serializer = new XMLSerializer(out);
			else {
				serializer.reset();
				serializer.setWriter(out);
			}
			setContentHandler(serializer);
			setLexicalHandler(serializer);
			nextAsSAX();
		} catch (Exception e) {
			throw new XMLDBCException(e.getMessage(), e);
		}
	}

	/**
	 * @see XMLResultSet#nextAsSAX
	 */
	public void nextAsSAX() throws XMLDBCException, SAXException {
		//Debug.print(this, "START XDBCResultSet::NEXTASSAX");
		firstHasNext = false;
		try {
			if (!visitor.getNoNext() && hasNext()) {
				xdbcResulSet.next();
			}
			if (reconstructExpression != null && visitor != null && !visitor.getNoReconstructor()) {
				// for prefix mapping
				NamespaceContextStack declarations = reconstructExpression.getParentUnit().getDeclarations();
				if (contentHandler != null && declarations != null) {
					visitor.setNamespaceDecl(declarations);
				}
				reconstructExpression.accept(visitor);
				visitor.reInit();
			}
			position++;
		} catch (Exception e) {
			throw new XMLDBCException(e.getMessage(), e);
		}
		//Debug.print(this, "STOP  XDBCResultSet::NEXTASSAX");
	}

	/**
	 * @see XMLResultSet#hasNext
	 */
	public boolean hasNext() throws XMLDBCException {
		boolean hasnext = xdbcResulSet.hasNext();
		if (hasnext == false && reconstructExpression.getRoot() && firstHasNext) {
			visitor.setNoResultSet(true);
			hasnext = true;
		}
		//		firstHasNext = false; 	commented by yucheng , moved to nextAsSAX() (authorized by  )
		return hasnext;
	}

	/**
	 * @see XMLResultSet#getPosition
	 */
	public int getPosition() throws XMLDBCException {
		return 0 == position ? -1 : position;
	}

	/**********************************************************************************************/

	//	public XResultSetImpl(XDBCResultSetInterface xdbcResulSet, XQueryExpression reconstructExpression, ArrayList varList) {
	//		//org.xquark.xml.misc.Debug.print(this, "START - XResultSetImpl constructor");
	//		this.xdbcResulSet = xdbcResulSet;
	//		this.reconstructExpression = reconstructExpression;
	//		this.varList = varList;
	//		visitor = new ReconstructionVisitor(this, varList);
	//		//org.xquark.xml.misc.Debug.print(this, "STOP  - XResultSetImpl constructor");
	//	}

	public XResultSetImpl(XDBCResultSetInterface xdbcResulSet, XQueryExpression reconstructExpression, ArrayList varList, XMLStatement statement) {
		//org.xquark.xml.misc.Debug.print(this, "START - XResultSetImpl constructor");
		this.xdbcResulSet = xdbcResulSet;
		this.reconstructExpression = reconstructExpression;
		this.varList = varList;
		this.statement = statement;
		visitor = new ReconstructionVisitor(this, varList);
		//org.xquark.xml.misc.Debug.print(this, "STOP  - XResultSetImpl constructor");
	}

	public XDBCResultSetInterface getXDBCResultSet() {
		return xdbcResulSet;
	}

	//	public void initReconstructionVisitor() throws XDBCException
	//	{
	//	//org.xquark.xml.misc.Debug.print(this, "START - XResultSetImpl initReconstructionVisitor");
	//		if (handler == null) throw new XDBCException("Handler must be initialize, before this visitor.");
	//		visitor = new ReconstructionVisitor(handler, this, varList);
	//	//org.xquark.xml.misc.Debug.print(this, "STOP  - XResultSetImpl initReconstructionVisitor");
	//	}

	public boolean hasRootTag() {
		if (varList != null && !varList.isEmpty()) {
			Variable tmpVar = (Variable) varList.get(0);
			XQueryExpression tmpExpr = tmpVar.getExpression();
			if ((reconstructExpression instanceof Element) && ((tmpExpr instanceof ValueInteger) || reconstructExpression.getLoop()))
				return true;
		}
		return false;
	}

	/**
	 * Returns the result set prefix map (as prefix-namespace pairs).
	 * This map is used to produce each result in the set.
	 * @return the result set prefix map.
	 * @throws XMLDBCException if a data source access error occurs.
	 * @deprecated
	 */
	public java.util.Map getPrefixMap() throws XMLDBCException {
		return java.util.Collections.EMPTY_MAP;
	}

	/**
	 * Returns the fragments contained in the result set as a single XML document, with
	 * a document root element having the specified namespace, local name and qualified name.
	 * Note: this convenience method can only be called if the cursor is before the
	 * first element. It will process the complete result set in one step.
	 * @param namespace the namespace of the document root element.
	 * @param localName the local name of the document root element.
	 * @param qName the qualified name of the document root element.
	 * @return an XML document containing the fragments of the result set, as children of
	 * the specified document root element.
	 * @throws XMLDBCException if a data source access error occurs.
	 */
	public XMLDocument getFragmentsAsDocument(String namespace, String localName, String qName) throws XMLDBCException {
		if (isBeforeFirst()) {
			try {
				XMLDocumentImpl xmlDocument = new XMLDocumentImpl(this, namespace, localName, qName);
				xmlDocument.setContentHandler(contentHandler);
				xmlDocument.setLexicalHandler(lexicalhandler);
				xmlDocument.setErrorHandler(errorhandler);
				return xmlDocument;
			} catch (Exception e) {
				throw new XMLDBCException(e.getMessage(), e);
			}
		} else {
			throw new XMLDBCException("The cursor moved, it is not more before the first element.");
		}
	}

	/**
	 * Returns the result set metadata (the XML Schema that models the returned data)
	 * as an XMLDocument.
	 * This schema describes the XML type of each result in the set.
	 * @return the result set metadata (a XML Schema).
	 * @throws XMLDBCException if a data source access error occurs.
	 */
	public XMLDocument getMetaData() throws XMLDBCException {
		return null;
	}

	private synchronized void createDOMBuilder() {
		if (DOMCreator == null) {
			DocumentBuilderFactory DOMFactory = DocumentBuilderFactory.newInstance();
			DOMFactory.setNamespaceAware(true);
			try {
				DOMCreator = DOMFactory.newDocumentBuilder();
			} catch (ParserConfigurationException pce) {
				throw new RuntimeException("Could not create an empty DOM level 2 document using JAXP. Check a JAXP DOM implementation is in your classpath.");
			}
		}
	}
}
