/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * Copyright (C) 2003 XQuark Group.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */

package org.xquark.xquery.parser.normalize;

//For all contants used in the structure:
import java.util.*;

import org.xquark.xpath.Axis;
import org.xquark.xpath.NodeKind;
import org.xquark.xquery.parser.*;
import org.xquark.xquery.parser.primitivefunctions.fnfunctions.*;
import org.xquark.xquery.parser.primitivefunctions.xsfunctions.FunctionSTRING;
import org.xquark.xquery.parser.util.Constants;

/**
 * This class is an implementation of org.xquark.xml.xqueryparser.common.ParserVisitor interface.<BR>
 * It extends org.xquark.xml.xqueryparser.common.DefaultParserVisitor class.<BR>
 * In each node of the XQuery structure, the ExpressionNormalizationVisitor identified if a normalization
 * rule/concept can be applied and construct a new normalized XQuery structure.<BR>
 * This visitor maintains the type of each node of the XQuery structure if they were typified in entry.
 * <BR><BR>
 * - You MUST visit a XQueryFile node to begin the normalization. If you have a XQueryExpression, you must build
 * a XQueryUnit with expresion and a XQueryFile with the XqueryUnit.<BR>
 */

/* TODO List:
- chercher et enlever les "REMOVE" et debug (sans casse):
- chercher et enlever les appel _trace(..) et untrace()
- chercher et faire les "TODO"
- Faire modifier le type visiteur pour deplacer le test de retypage (force + qtype != null) au tout debut (pour
    ne pas perdre de temps au retypage.
*/

public class ExpressionNormalizationVisitor extends org.xquark.xquery.parser.DefaultParserVisitor {

	/** CVS/QA stuff. */
	private static final String RCSRevision = "$Revision: 1.10 $";
	/** CVS/QA stuff. */
	private static final String RCSName = "$Name:  $";

	/*
	In this class, in this following order, you can found:
	
	    1) Protected properties
	    2) Protected constants
	    3) Constructors
	    4) Protected and package methods (except debug methods)
	    5) Public methods (except visit methods)
	    6) Protected debug methods
	    7) Public/protected visit methods in parameter type alphabetical order:
	*/

	/*************************** Protected properties *******************************/

	/** The type visitor. */
	protected TypeVisitor typeVisitor = null;

	/** The current normalised XQueryFile. */
	protected XQueryFile XQFile = null;

	/** The current normalised XQueryUnit. */
	protected XQueryUnit XQUnit = null;

	/** The current normalised XQueryExpression. */
	protected XQueryExpression expr = null;

	/** The visitor context (contains some questions/responses). */
	protected VisitorContext context = null;

	/** Absolute depth in the XQuery structure (in the tree). */
	protected int depth = -1;

	/** To know if a normalize rule was applied. */
	protected boolean normalized;

	/** Hash map of all variables declarations (to clone partial xquery expressions). */
	protected HashMap variablesDeclarations = null;

	/*************************** Protected constants *******************************/

	/** The default XML namespace prefix. */
	protected final String DEFAULT_XML_PREFIX = "xml";
	/** The default XML namespace url. */
	protected final String DEFAULT_XML_URL = "http://www.w3.org/XML/1998/namespace";

	/** The default Schema namespace prefix. */
	protected final String DEFAULT_SCHEMA_PREFIX = "xs";
	/** The default Schema namespace url. */
	protected final String DEFAULT_SCHEMA_URL = "http://www.w3.org/2001/XMLSchema";

	/** The default Schema instance namespace prefix. */
	protected final String DEFAULT_SCHEMA_INSTANCE_PREFIX = "xsi";
	/** The default Schema instance namespace url. */
	protected final String DEFAULT_SCHEMA_INSTANCE_URL = "http://www.w3.org/2001/XMLSchema-instance";

	/** The default function namespace prefix. */
	protected final String DEFAULT_FUNCTION_PREFIX = "xf";
	/** The default function namespace url. */
	protected final String DEFAULT_FUNCTION_URL = "http://www.w3.org/2002/08/xquery-functions";

	/** The default operator namespace prefix. */
	protected final String DEFAULT_OPERATOR_PREFIX = "op";
	/** The default operator namespace url. */
	protected final String DEFAULT_OPERATOR_URL = "http://www.w3.org/2002/08/xquery-operators";

	/*************************** Constructors *******************************/

	/**
	 * To construct a new visitor to create a canonical request from a valid XQuery request.
	 * @throws NormalizeException
	 */
	public ExpressionNormalizationVisitor() throws NormalizeException {
		this(new VisitorContext(), -1, null);
	}

	/**
	 * To construct a new visitor to create a canonical request from a valid XQuery request.
	 * @param typeVisitor a type visitor maintain typing of the XQuery structure.
	 * @throws NormalizeException
	 */
	public ExpressionNormalizationVisitor(TypeVisitor typeVisitor) throws NormalizeException {
		this(new VisitorContext(), -1, typeVisitor);
	}

	/**
	 * To construct a new visitor to create a canonical request from a valid XQuery request.
	 * @param depth the absolute depth in the XQuery structure (it is a tree).
	 * @throws NormalizeException
	 */
	protected ExpressionNormalizationVisitor(int depth) throws NormalizeException {
		this(new VisitorContext(), depth, null);
	}

	/**
	 * To construct a new visitor to create a canonical request from a valid XQuery request.
	 * @param depth the absolute depth in the XQuery structure (it is a tree).
	 * @param typeVisitor a type visitor maintain typing of the XQuery structure.
	 * @throws NormalizeException
	 */
	protected ExpressionNormalizationVisitor(int depth, TypeVisitor typeVisitor) throws NormalizeException {
		this(new VisitorContext(), depth, typeVisitor);
	}

	/**
	 * To construct a new visitor to create a canonical request from a valid XQuery request.
	 * @param ctx a context to set as this visitor context.
	 * @param depth the absolute depth in the XQuery structure (it is a tree).
	 * @param typeVisitor a type visitor maintain typing of the XQuery structure.
	 * @throws NormalizeException
	 */
	protected ExpressionNormalizationVisitor(VisitorContext ctx, int depth, TypeVisitor typeVisitor) throws NormalizeException {
		this.context = ctx;
		this.expr = null;
		this.depth = depth + 1;
		this.normalized = false;
		this.typeVisitor = typeVisitor;
	}

	/*************************** Protected and package methods *******************************/

	//Only for non regression tests! to rename variables in the same way each time.
	static int varCounter;

	// add LARS 15/09/2003
	private Variable createVariable(int bindingType, XQueryExpression expr) throws XQueryException {
		if (typeVisitor != null && expr != null)
			expr.accept(typeVisitor);
		Variable var = new Variable(new QName(null, null, getUniqueVarName(null), typeVisitor), null);
		var.setBindingType(bindingType);
		var.setExpression(expr, typeVisitor);
		return var;
	}

	private FLWRExpression createDummyFLWR(XQueryExpression whereClause, XQueryExpression returnClause) throws XQueryException, TypeException {
		Variable var = createVariable(Constants.FOR_BINDINGTYPE, new ValueInteger("1", typeVisitor));
		ArrayList vars = new ArrayList(1);
		vars.add(var);
		XQueryUnit unit = returnClause.getParentUnit();
		FLWRExpression flwr = new FLWRExpression(vars, null, whereClause, returnClause, typeVisitor);
		flwr.setParentUnit(unit);
		return flwr;
	}
	// stop add LARS 15/09/2003

	/**
	 * Gets an unique variable name.
	 * @param oldName The old variable name.
	 * @return an unique variable name.
	 */
	static synchronized String getUniqueVarName(String oldName) {
		if ((oldName == null) /*|| (traceLevel != -100)*/) {
			//Only for non regression tests!
			return ("var_norm" + (new Integer(++varCounter)).toString());
		} else {
			return oldName;
		}
	}

	/**
	 * To set (in a simple way) parenthesis, braces and/or brackets of a sequence.
	 * Apply some rules on ONE expression: like [(E)]=> [E], {(E)} => {E}...
	 * @param aSeq an XQuery expression (not only a sequence).
	 * @param parenthesis true if there is parenthesis, false otherwise.
	 * @param braces true if there is braces, false otherwise.
	 * @param brackets true if there is brackets, false otherwise.
	 */
	protected void setSequenceDelimiters(XQueryExpression aSeq, boolean parenthesis, boolean braces, boolean brackets) {
		if (brackets && parenthesis) {
			//if [(...)] then [...]
			aSeq.setParenthesis(false);
			aSeq.setBrace(braces);
			aSeq.setBracket(brackets);
		} else if (braces && parenthesis) {
			//if {(...)} then {...}
			aSeq.setParenthesis(false);
			aSeq.setBrace(true);
			aSeq.setBracket(brackets);
		} else {
			//General case:
			aSeq.setParenthesis(parenthesis);
			aSeq.setBrace(braces);
			aSeq.setBracket(brackets);
		}
	}

	/**
	 * To get the current context.
	 * @return the current context.
	 */
	protected VisitorContext getContext() {
		return this.context;
	}

	/**
	 * To set the current context.
	 * @param the new context.
	 */
	protected void setContext(VisitorContext ctx) {
		this.context = ctx;
	}

	/**
	 * To get the current depth.
	 * @return the current depth.
	 */
	protected int getDepth() {
		return this.depth;
	}

	/*************************** Public methods *******************************/

	/**
	 * To re-initialize this object, like call constructor ExpressionNormalizationVisitor(-1).
	 * @throws NormalizeException
	 */
	public void reInit() throws NormalizeException {
		this.context = new VisitorContext();
		this.expr = null;
		this.depth = 0;
		this.normalized = false;
	}

	/**
	 * Gets the current XQueryFile normalized.
	 * @return the current XQueryFile normalized.
	 */
	public XQueryFile getXQFile() {
		return this.XQFile;
	}

	/**
	 * Gets the current XQueryUnit normalized.
	 * @return the current XQueryUnit normalized.
	 */
	public XQueryUnit getXQUnit() {
		return this.XQUnit;
	}

	/**
	 * Gets the current XQuery expression normalized.
	 * @return the current XQuery expression normalized.
	 */
	public XQueryExpression getExpr() {
		return this.expr;
	}

	/**
	 * To know if a normalize rule was applied.
	 * @return true if a normalize rule was applied, false otherwise.
	 */
	public boolean wasNormalized() {
		return this.normalized;
	}
	
	/********************************************************************************/
	/********************** Public/protected visit methods **************************/
	/********************************************************************************/

	/**
	 * This visit method normalizes a XQueryFile : all its XQuery units.<BR>
	 * It manage also multiple pass to normalize units.
	 * @param arg a XQueryFile to normalize.
	 */
	public void visit(XQueryFile arg) throws NormalizeException {
		try {
			ExpressionNormalizationVisitor recurseVisitor = new ExpressionNormalizationVisitor(this.typeVisitor);
			//Normalize each units, whith management of multiple normalize pass:
			ArrayList units = arg.getUnits(); //Could be null !
			if (units != null) {
				//TOREMOVE: only for debug:
				varCounter = 0;
				int unitsLen = units.size();
				for (int i = unitsLen - 1; i >= 0; i--) {
					int pass = 1;
					boolean removed = false;
					//Loop for each pass while at least a rule was apply.
					do {
						recurseVisitor.reInit();
						//Depth <- 0, normalized flag <- false
						 ((XQueryUnit) units.get(i)).accept(recurseVisitor);
						XQueryUnit newUnit = recurseVisitor.getXQUnit();
						if (newUnit != null) {
							newUnit.setParentFile(arg);
							units.set(i, newUnit);
						} else {
							units.remove(i);
							removed = true;
						}
						if (pass >= 32) {
							throw new NormalizeException(0, "Infinite loop (>= 32 passes) in normalization !");
						}
					} while (recurseVisitor.wasNormalized() && !removed);
				}
			}
			this.XQFile = arg;
		} catch (Exception e) {
			if (e instanceof NormalizeException)
				throw (NormalizeException) e;
			else
				throw new NormalizeException(0, "Unexpected exception in XQueryFile :\n==>" + e.toString() + "<==\n", e);
		}
	}

	/**
	 * This visit method normalizes a XQueryUnit : its context declaration, its function definition
	 * and its sub expressions.<BR>
	 * It also answers the questions of the current context.
	 * @param arg a XQueryUnit to normalize.
	 */
	public void visit(XQueryUnit arg) throws NormalizeException {
		try {
			ExpressionNormalizationVisitor recurseVisitor = new ExpressionNormalizationVisitor(this.depth, this.typeVisitor);
			Iterator keys;
			Object oneKey;
			//Normalize function definitions:
			HashMap functionDef = arg.getFunctions();
			if ((functionDef != null) && (functionDef.keySet() != null)) {
				keys = functionDef.keySet().iterator();
				while (keys.hasNext()) {
					oneKey = keys.next();
					((FunctionDefinition) functionDef.get(oneKey)).accept(recurseVisitor);
					XQueryExpression newFunctionDef = recurseVisitor.getExpr();
					if (newFunctionDef == null) {
						functionDef.remove(oneKey);
					} else {
						//Set parent unit :
						newFunctionDef.setParentUnit(arg);
						functionDef.put(oneKey, newFunctionDef);
					}
					//Update the normalized flag:
					this.normalized = this.normalized | recurseVisitor.wasNormalized();
				}
			}
			//Normalize expressions:
			ArrayList expressions = arg.getExpressions();
			int exprLen = expressions.size();
			for (int i = exprLen - 1; i >= 0; i--) {
				((XQueryExpression) expressions.get(i)).accept(recurseVisitor);
				XQueryExpression newExpr = recurseVisitor.getExpr();
				if (newExpr == null) {
					expressions.remove(i);
				} else {
					//Set parent unit :
					newExpr.setParentUnit(arg);
					expressions.set(i, newExpr);
				}
				//Update the normalized flag:
				this.normalized = this.normalized | recurseVisitor.wasNormalized();
			}
			if (expressions.size() == 0) {
				arg = null;
			}
			this.XQUnit = arg;
		} catch (Exception e) {
			if (e instanceof NormalizeException)
				throw (NormalizeException) e;
			else
				throw new NormalizeException(0, "Unexpected exception in XQueryUnit :\n==>" + e.toString() + "<==\n", e);
		}
	}

	/**
	 * This visit method normalizes a XQueryExpression (abstract class). It do nothing : this is the default
	 * behaviour for all unsupported expressions type.<BR>
	 * @param arg a XQueryExpression to normalize.
	 */
	public void visit(XQueryExpression arg) throws NormalizeException {
		try {
			this.expr = arg;
		} catch (Exception e) {
			if (e instanceof NormalizeException)
				throw (NormalizeException) e;
			else
				throw new NormalizeException(0, "Unexpected exception in (abstract) XQueryExpression :\n==>" + e.toString() + "<==\n", e);
		}
	}

	/******** Unary/Binary operator ********/

	/**
	 * This visit method normalizes a Dereference LE(E1)=>E2 and its sub expressions (E1, E2).<BR>
	 * This method apply theses rules:<BR>
	 * <OL>
	 * <LI>N(LE(E1)=>E2) => N(LE(E1))=>N(E2)</LI>
	 * <LI>N(null=>E2) => null</LI>
	 * <LI>N(LE(E1)=>null) => null</LI>
	 * </OL>
	 * It possibly answers the questions of the current context.
	 * @param arg a Dereference to normalize.
	 */
	//	public void visit(Dereference arg) throws NormalizeException {
	//		try {
	//			ExpressionNormalizationVisitor recurseVisitor = new ExpressionNormalizationVisitor(this.depth, this.typeVisitor);
	//			VisitorContext localCtx = new VisitorContext();
	//			localCtx.merge(this.context, false);
	//			//Clean all questions which depth is smaller than current depth:
	//			localCtx.filter(this.depth);
	//			recurseVisitor.setContext(localCtx);
	//			//Normalize first expression:
	//			XQueryExpression expr1 = arg.getExpression1();
	//			expr1.accept(recurseVisitor);
	//			localCtx.update(recurseVisitor.getContext());
	//			expr1 = recurseVisitor.getExpr();
	//			//Update the normalized flag:
	//			this.normalized = recurseVisitor.wasNormalized();
	//			if (expr1 != null) {
	//				//Test if sub expression was normalized (and update it) :
	//				if (recurseVisitor.wasNormalized()) {
	//					arg.setExpression1(expr1);
	//				}
	//				//Normalize second expression:
	//				XQueryExpression expr2 = arg.getExpression2();
	//				expr2.accept(recurseVisitor);
	//				localCtx.update(recurseVisitor.getContext());
	//				expr2 = recurseVisitor.getExpr();
	//				//Update the normalized flag:
	//				this.normalized = this.normalized | recurseVisitor.wasNormalized();
	//				if (expr2 != null) {
	//					//Test if sub expression was normalized (and update it) :
	//					if (recurseVisitor.wasNormalized()) {
	//						arg.setExpression2(expr2);
	//					}
	//				} else {
	//					arg = null;
	//				}
	//			} else {
	//				arg = null;
	//			}
	//			this.expr = arg;
	//			//Re-type this expression :
	//			if (this.normalized && (this.expr != null) && (this.typeVisitor != null)) {
	//				this.expr.accept(this.typeVisitor);
	//			}
	//			//Update response context with current context to pass the answers of parent questions:
	//			this.context.update(localCtx);
	//		} catch (Exception e) {
	//			if (e instanceof NormalizeException)
	//				throw (NormalizeException) e;
	//			else
	//				throw new NormalizeException(0, "Unexpected exception in Dereference :\n==>" + e.toString() + "<==\n", e);
	//		}
	//	}

	/**
	 * This visit method normalizes an UnOpMinusExpression and its expression (its value).<BR>
	 * This method apply theses rules:<BR>
	 * <OL>
	 * <LI>N(-E) => -N(E)</LI>
	 * <LI>N(-null) => null</LI>
	 * </OL>
	 * It possibly answers the questions of the current context.
	 * @param arg an UnOpMinusExpression to normalize.
	 */
	public void visit(UnOpMinusExpression arg) throws NormalizeException {
		try {
			ExpressionNormalizationVisitor recurseVisitor = new ExpressionNormalizationVisitor(this.depth, this.typeVisitor);
			VisitorContext localCtx = new VisitorContext();
			localCtx.merge(this.context, false);
			//Clean all questions which depth is smaller than current depth:
			localCtx.filter(this.depth);
			recurseVisitor.setContext(localCtx);
			//Normalize expression of this UnOpMinusExpression:
			XQueryExpression anExpr = arg.getExpression();
			anExpr.accept(recurseVisitor);
			localCtx.update(recurseVisitor.getContext());
			anExpr = recurseVisitor.getExpr();
			//Update the normalized flag:
			this.normalized = recurseVisitor.wasNormalized();
			//Test if sub expression was normalized:
			if (this.normalized) {
				if (anExpr != null) {
					//Update current UnOpMinusExpression:
					arg.setExpression(anExpr);
					//Re-type this expression :
					if (this.typeVisitor != null) {
						arg.accept(this.typeVisitor);
					}
				} else {
					arg = null;
				}
			}
			this.expr = arg;
			//Update response context with current context to pass the answers of parent questions:
			this.context.update(localCtx);
		} catch (Exception e) {
			if (e instanceof NormalizeException)
				throw (NormalizeException) e;
			else
				throw new NormalizeException(0, "Unexpected exception in UnOpMinusExpression :\n==>" + e.toString() + "<==\n", e);
		}
	}

	/**
	 * This visit method normalizes a BinOpANDExpression (E1 and E2) and its sub expressions (E1 and E2).<BR>
	 * This method apply theses rules:<BR>
	 * <OL>
	 * <LI>N(E1 and E2) => N(E1) and N(E2)</LI>
	 * <LI>N(true() and true()) => true()</LI>
	 * <LI>N(false()|null and E) => false()</LI>
	 * <LI>N(E and false()|null) => false()</LI>
	 * </OL>
	 * It also answers the questions of the current context.
	 * @param arg a BinOpANDExpression to normalize.
	 */
	public void visit(BinOpANDExpression arg) throws NormalizeException {
		try {
			ExpressionNormalizationVisitor recurseVisitor = new ExpressionNormalizationVisitor(this.depth, this.typeVisitor);
			VisitorContext localCtx = new VisitorContext();
			localCtx.merge(this.context, false);
			//Clean all questions which depth is smaller than current depth:
			localCtx.filter(this.depth);
			recurseVisitor.setContext(localCtx);
			//For multivalued expression (put in exists function) :
			localCtx.setMemo((this.depth + 1), VisitorContext._MULTIVALUED_NEEDEXISTSFUNCTION, null, true);
			//Normalize first expression:
			XQueryExpression expr1 = arg.getExpression1();
			expr1.accept(recurseVisitor);
			localCtx.update(recurseVisitor.getContext());
			expr1 = recurseVisitor.getExpr();
			//Update the normalized flag:
			this.normalized = recurseVisitor.wasNormalized();
			//Test if sub expression was normalized:
			if (this.normalized) {
				if (expr1 != null) {
					//Update current UnOpMinusExpression:
					arg.setExpression1(expr1);
				} else {
					arg = null;
				}
			}
			//Normalize second expression:
			XQueryExpression expr2 = null;
			if (arg != null) {
				expr2 = arg.getExpression2();
				expr2.accept(recurseVisitor);
				localCtx.update(recurseVisitor.getContext());
				expr2 = recurseVisitor.getExpr();
				//Update the normalized flag:
				this.normalized = this.normalized | recurseVisitor.wasNormalized();
				//Test if sub expression was normalized:
				if (recurseVisitor.wasNormalized()) {
					if (expr2 != null) {
						//Update current UnOpMinusExpression:
						arg.setExpression2(expr2);
					} else {
						arg = null;
					}
				}
			}
			//For multivalued expression (put in exists function) :
			localCtx.removeMemo((this.depth + 1), VisitorContext._MULTIVALUED_NEEDEXISTSFUNCTION);
			//Evaluation of this AND expression :
			if (arg == null) {
				//Here : we can evaluate this boolean expression :
				this.expr = new FunctionFALSE(null, this.typeVisitor);
				this.normalized = true;
			} else if ((expr1 instanceof FunctionFALSE) || (expr2 instanceof FunctionFALSE)) {
				//Here : we can evaluate this boolean expression :
				this.expr = new FunctionFALSE(null, this.typeVisitor);
				this.normalized = true;
			} else if ((expr1 instanceof FunctionTRUE) && (expr2 instanceof FunctionTRUE)) {
				//Here : we can evaluate this boolean expression :
				this.expr = new FunctionTRUE(null, this.typeVisitor);
				this.normalized = true;
			} else if (expr1 instanceof FunctionTRUE) {
				this.expr = expr2;
				this.normalized = true;
			} else if (expr2 instanceof FunctionTRUE) {
				this.expr = expr1;
				this.normalized = true;
			} else {
				this.expr = arg;
			}
			//Re-type this expression :
			if (this.normalized && (this.expr != null) && (this.typeVisitor != null)) {
				this.expr.accept(this.typeVisitor);
			}
			//Update response context with current context to pass the answers of parent questions:
			this.context.update(localCtx);
		} catch (Exception e) {
			if (e instanceof NormalizeException)
				throw (NormalizeException) e;
			else
				throw new NormalizeException(0, "Unexpected exception in BinOpANDExpression :\n==>" + e.toString() + "<==\n", e);
		}
	}

	/**
	 * This visit method normalizes a BinOpORExpression (E1 or E2) and its sub expressions (E1 and E2).<BR>
	 * This method apply theses rules:<BR>
	 * <OL>
	 * <LI>N(E1 or E2) => N(E1) or N(E2)</LI>
	 * <LI>N(false()|null or false()|null) => false()</LI>
	 * <LI>N(true() or E) => true()</LI>
	 * <LI>N(E or true()) => true()</LI>
	 * </OL>
	 * It also answers the questions of the current context.
	 * @param arg a BinOpORExpression to normalize.
	 */
	public void visit(BinOpORExpression arg) throws NormalizeException {
		try {
			ExpressionNormalizationVisitor recurseVisitor = new ExpressionNormalizationVisitor(this.depth, this.typeVisitor);
			VisitorContext localCtx = new VisitorContext();
			localCtx.merge(this.context, false);
			//Clean all questions which depth is smaller than current depth:
			localCtx.filter(this.depth);
			recurseVisitor.setContext(localCtx);
			//For multivalued expression (put in exists function) :
			localCtx.setMemo((this.depth + 1), VisitorContext._MULTIVALUED_NEEDEXISTSFUNCTION, null, true);
			//Normalize first expression:
			XQueryExpression expr1 = arg.getExpression1();
			expr1.accept(recurseVisitor);
			localCtx.update(recurseVisitor.getContext());
			expr1 = recurseVisitor.getExpr();
			//Update the normalized flag:
			this.normalized = recurseVisitor.wasNormalized();
			if ((this.normalized) && (expr1 != null)) {
				arg.setExpression1(expr1);
			}
			//Normalize second expression:
			XQueryExpression expr2 = arg.getExpression2();
			expr2.accept(recurseVisitor);
			localCtx.update(recurseVisitor.getContext());
			expr2 = recurseVisitor.getExpr();
			//Update the normalized flag:
			this.normalized = this.normalized | recurseVisitor.wasNormalized();
			if ((recurseVisitor.wasNormalized()) && (expr2 != null)) {
				arg.setExpression2(expr2);
			}
			//For multivalued expression (put in exists function) :
			localCtx.removeMemo((this.depth + 1), VisitorContext._MULTIVALUED_NEEDEXISTSFUNCTION);
			//Evaluation of this OR expression :
			if (expr1 == null) {
				this.normalized = true;
				if (expr2 == null) {
					this.expr = new FunctionFALSE(null, this.typeVisitor);
				} else if (expr2 instanceof FunctionFALSE) {
					this.expr = new FunctionFALSE(null, this.typeVisitor);
				} else if (expr2 instanceof FunctionTRUE) {
					this.expr = new FunctionTRUE(null, this.typeVisitor);
				} else {
					this.expr = expr2;
				}
			} else if (expr1 instanceof FunctionFALSE) {
				this.normalized = true;
				if (expr2 == null) {
					this.expr = new FunctionFALSE(null, this.typeVisitor);
				} else if (expr2 instanceof FunctionFALSE) {
					this.expr = new FunctionFALSE(null, this.typeVisitor);
				} else if (expr2 instanceof FunctionTRUE) {
					this.expr = new FunctionTRUE(null, this.typeVisitor);
				} else {
					this.expr = expr2;
				}
			} else if (expr1 instanceof FunctionTRUE) {
				this.normalized = true;
				this.expr = new FunctionTRUE(null, this.typeVisitor);
			} else {
				if (expr2 == null) {
					this.expr = expr1;
					this.normalized = true;
				} else if (expr2 instanceof FunctionFALSE) {
					this.expr = expr1;
					this.normalized = true;
				} else if (expr2 instanceof FunctionTRUE) {
					this.expr = new FunctionTRUE(null, this.typeVisitor);
					this.normalized = true;
				} else {
					this.expr = arg;
				}
			}
			//Re-type this expression :
			if (this.normalized && (this.expr != null) && (this.typeVisitor != null)) {
				this.expr.accept(this.typeVisitor);
			}
			//Update response context with current context to pass the answers of parent questions:
			this.context.update(localCtx);
		} catch (Exception e) {
			if (e instanceof NormalizeException)
				throw (NormalizeException) e;
			else
				throw new NormalizeException(0, "Unexpected exception in BinOpORExpression :\n==>" + e.toString() + "<==\n", e);
		}
	}

	/**
	 * This visit method normalizes a ListOpAFTERExpression (E1 after E2) and its sub expressions (E1 and E2).<BR>
	 * It also answers the questions of the current context.
	 * @param arg a ListOpAFTERExpression to normalize.
	 */
	//	public void visit(ListOpAFTERExpression arg) throws NormalizeException {
	//		try {
	//			throw new NormalizeException(0, "ListOpAFTERExpression not supported : (" + arg + ")");
	//			ExpressionNormalizationVisitor recurseVisitor = new ExpressionNormalizationVisitor(this.depth, this.typeVisitor);
	//			VisitorContext localCtx = new VisitorContext();
	//			localCtx.merge(this.context, false);
	//			//Clean all questions which depth is smaller than current depth:
	//			localCtx.filter(this.depth);
	//			recurseVisitor.setContext(localCtx);
	//			//Normalize first expression:
	//			XQueryExpression expr1 = arg.getExpression1();
	//			expr1.accept(recurseVisitor);
	//			localCtx.update(recurseVisitor.getContext());
	//			expr1 = recurseVisitor.getExpr();
	//			//Update the normalized flag:
	//			this.normalized = recurseVisitor.wasNormalized();
	//			//Normalize second expression:
	//			XQueryExpression expr2 = arg.getExpression2();
	//			expr2.accept(recurseVisitor);
	//			localCtx.update(recurseVisitor.getContext());
	//			expr2 = recurseVisitor.getExpr();
	//			//Update the normalized flag:
	//			this.normalized = this.normalized | recurseVisitor.wasNormalized();
	//			//Test if sub expressions were normalized:
	//			if (this.normalized) {
	//			    //Update current ListOpAFTERExpression (and type this expression):
	//			    arg = new ListOpAFTERExpression(expr1, expr2, this.typeVisitor);
	//			}
	//			this.expr = arg;
	//			//Update response context with current context to pass the answers of parent questions:
	//			this.context.update(localCtx);
	//		} catch (Exception e) {
	//			if (e instanceof NormalizeException)
	//				throw (NormalizeException) e;
	//			else
	//				throw new NormalizeException(0, "Unexpected exception in ListOpAFTERExpression :\n==>" + e.toString() + "<==\n", e);
	//		}
	//	}

	/**
	 * This visit method normalizes a ListOpArithExpression (E1 operator E2) and its sub expressions (E1 and E2).<BR>
	 * This method apply theses rules:<BR>
	 * <OL>
	 * <LI>N(E1 operator E2) => N(E1) operator N(E2)</LI>
	 * <LI>N(null operator E) =>  null (where E cannot be evaluated)</LI>
	 * <LI>N(E operator null) =>  null (where E cannot be evaluated)</LI>
	 * <LI>N(null operator null) =>  null</LI>
	 * </OL>
	 * It also answers the questions of the current context.
	 * @param arg a ListOpArithExpression to normalize.
	 */
	public void visit(ListOpArithExpression arg) throws NormalizeException {
		try {
			ExpressionNormalizationVisitor recurseVisitor = new ExpressionNormalizationVisitor(this.depth, this.typeVisitor);
			VisitorContext localCtx = new VisitorContext();
			localCtx.merge(this.context, false);
			//Clean all questions which depth is smaller than current depth:
			localCtx.filter(this.depth);
			recurseVisitor.setContext(localCtx);
			//Normalize first expression:
			XQueryExpression expr1 = arg.getExpression1();
			expr1.accept(recurseVisitor);
			localCtx.update(recurseVisitor.getContext());
			expr1 = recurseVisitor.getExpr();
			//Update the normalized flag:
			this.normalized = recurseVisitor.wasNormalized();
			if (this.normalized) {
				if (expr1 != null) {
					arg.setExpression1(expr1);
				} else {
					arg = null;
				}
			}
			//Normalize second expression:
			XQueryExpression expr2 = null;
			if (arg != null) {
				expr2 = arg.getExpression2();
				expr2.accept(recurseVisitor);
				localCtx.update(recurseVisitor.getContext());
				expr2 = recurseVisitor.getExpr();
				//Update the normalized flag:
				this.normalized = this.normalized | recurseVisitor.wasNormalized();
				if (recurseVisitor.wasNormalized()) {
					if (expr2 != null) {
						arg.setExpression2(expr2);
					} else {
						arg = null;
					}
				}
			}
			this.expr = arg;
			//Re-type this expression :
			if (this.normalized && (this.expr != null) && (this.typeVisitor != null)) {
				this.expr.accept(this.typeVisitor);
			}
			//Update response context with current context to pass the answers of parent questions:
			this.context.update(localCtx);
		} catch (Exception e) {
			if (e instanceof NormalizeException)
				throw (NormalizeException) e;
			else
				throw new NormalizeException(0, "Unexpected exception in ListOpArithExpression :\n==>" + e.toString() + "<==\n", e);
		}
	}

	/**
	 * This visit method normalizes a ListOpBEFOREExpression (E1 before E2) and its sub expressions (E1 and E2).<BR>
	 * It also answers the questions of the current context.
	 * @param arg a ListOpBEFOREExpression to normalize.
	 */
	//	public void visit(ListOpBEFOREExpression arg) throws NormalizeException {
	//		try {
	//			throw new NormalizeException(0, "ListOpBEFOREExpression not supported : (" + arg + ")");
	//			ExpressionNormalizationVisitor recurseVisitor = new ExpressionNormalizationVisitor(this.depth, this.typeVisitor);
	//			VisitorContext localCtx = new VisitorContext();
	//			localCtx.merge(this.context, false);
	//			//Clean all questions which depth is smaller than current depth:
	//			localCtx.filter(this.depth);
	//			recurseVisitor.setContext(localCtx);
	//			//Normalize first expression:
	//			XQueryExpression expr1 = arg.getExpression1();
	//			expr1.accept(recurseVisitor);
	//			localCtx.update(recurseVisitor.getContext());
	//			expr1 = recurseVisitor.getExpr();
	//			//Update the normalized flag:
	//			this.normalized = recurseVisitor.wasNormalized();
	//			//Normalize second expression:
	//			XQueryExpression expr2 = arg.getExpression2();
	//			expr2.accept(recurseVisitor);
	//			localCtx.update(recurseVisitor.getContext());
	//			expr2 = recurseVisitor.getExpr();
	//			//Update the normalized flag:
	//			this.normalized = this.normalized | recurseVisitor.wasNormalized();
	//			//Test if sub expressions were normalized:
	//			if (this.normalized) {
	//			    //Update current ListOpBEFOREExpression (and type this expression):
	//			    arg = new ListOpBEFOREExpression(expr1, expr2, this.typeVisitor);
	//			}
	//			this.expr = arg;
	//			//Update response context with current context to pass the answers of parent questions:
	//			this.context.update(localCtx);
	//		} catch (Exception e) {
	//			if (e instanceof NormalizeException)
	//				throw (NormalizeException) e;
	//			else
	//				throw new NormalizeException(0, "Unexpected exception in ListOpBEFOREExpression :\n==>" + e.toString() + "<==\n", e);
	//		}
	//	}

	/**
	 * This visit method normalizes a ListOpCompExpression (E1 "<"|">"|">="|"<="|"="|"!="|"=="|"!==" E2)
	 * and its sub expressions (E1 and E2).<BR>
	 * This method apply theses rules:<BR>
	 * <OL>
	 * <LI>N(E1 comparison E2) => N(E1) comparison N(E2)</LI>
	 * <LI>N(null comparison E) =>  null (where E cannot be evaluated)</LI>
	 * <LI>N(E comparison null) =>  null (where E cannot be evaluated)</LI>
	 * <LI>N(null comparison null) =>  null</LI>
	 * </OL>
	 * It also answers the questions of the current context.
	 * @param arg a ListOpCompExpression to normalize.
	 */
	public void visit(ListOpCompExpression arg) throws NormalizeException {
		try {
			ExpressionNormalizationVisitor recurseVisitor = new ExpressionNormalizationVisitor(this.depth, this.typeVisitor);
			VisitorContext localCtx = new VisitorContext();
			localCtx.merge(this.context, false);
			//Clean all questions which depth is smaller than current depth:
			localCtx.filter(this.depth);
			recurseVisitor.setContext(localCtx);
			//Normalize first expression:
			XQueryExpression expr1 = arg.getExpression1();
			expr1.accept(recurseVisitor);
			localCtx.update(recurseVisitor.getContext());
			expr1 = recurseVisitor.getExpr();
			//Update the normalized flag:
			this.normalized = recurseVisitor.wasNormalized();
			if (this.normalized) {
				if (expr1 != null) {
					arg.setExpression1(expr1);
				} else {
					arg = null;
				}
			}
			//Normalize second expression:
			XQueryExpression expr2 = null;
			if (arg != null) {
				expr2 = arg.getExpression2();
				expr2.accept(recurseVisitor);
				localCtx.update(recurseVisitor.getContext());
				expr2 = recurseVisitor.getExpr();
				//Update the normalized flag:
				this.normalized = this.normalized | recurseVisitor.wasNormalized();
				if (recurseVisitor.wasNormalized()) {
					if (expr2 != null) {
						arg.setExpression2(expr2);
					} else {
						arg = null;
					}
				}
			}
			this.expr = arg;
			//Re-type this expression :
			if (this.normalized && (this.expr != null) && (this.typeVisitor != null)) {
				this.expr.accept(this.typeVisitor);
			}
			//Update response context with current context to pass the answers of parent questions:
			this.context.update(localCtx);
		} catch (Exception e) {
			if (e instanceof NormalizeException)
				throw (NormalizeException) e;
			else
				throw new NormalizeException(0, "Unexpected exception in ListOpCompExpression :\n==>" + e.toString() + "<==\n", e);
		}
	}

	/**
	 * This visit method normalizes a ListOpEXCEPTExpression (E1 except E2) and its sub expressions (E1 and E2).<BR>
	 * This method apply theses rules:<BR>
	 * <OL>
	 * <LI>N(E1 except E2) => N(E1) except N(E2)</LI>
	 * <LI>N(null except E) =>  null (where E cannot be evaluated)</LI>
	 * <LI>N(E except null) =>  N(E) (where E cannot be evaluated)</LI>
	 * <LI>N(null except null) =>  null</LI>
	 * </OL>
	 * It also answers the questions of the current context.
	 * @param arg a ListOpEXCEPTExpression to normalize.
	 */
	//	public void visit(ListOpEXCEPTExpression arg) throws NormalizeException {
	//		try {
	//			ExpressionNormalizationVisitor recurseVisitor = new ExpressionNormalizationVisitor(this.depth, this.typeVisitor);
	//			VisitorContext localCtx = new VisitorContext();
	//			localCtx.merge(this.context, false);
	//			//Clean all questions which depth is smaller than current depth:
	//			localCtx.filter(this.depth);
	//			recurseVisitor.setContext(localCtx);
	//			//Normalize first expression:
	//			XQueryExpression expr1 = arg.getExpression1();
	//			expr1.accept(recurseVisitor);
	//			localCtx.update(recurseVisitor.getContext());
	//			expr1 = recurseVisitor.getExpr();
	//			//Update the normalized flag:
	//			this.normalized = recurseVisitor.wasNormalized();
	//			if (this.normalized) {
	//				if (expr1 != null) {
	//					arg.setExpression1(expr1);
	//				} else {
	//					arg = null;
	//				}
	//			}
	//			//Normalize second expression:
	//			XQueryExpression expr2 = null;
	//			if (arg != null) {
	//				expr2 = arg.getExpression2();
	//				expr2.accept(recurseVisitor);
	//				localCtx.update(recurseVisitor.getContext());
	//				expr2 = recurseVisitor.getExpr();
	//				//Update the normalized flag:
	//				this.normalized = this.normalized | recurseVisitor.wasNormalized();
	//				if (recurseVisitor.wasNormalized()) {
	//					if (expr2 != null) {
	//						arg.setExpression2(expr2);
	//						this.expr = arg;
	//					} else {
	//						this.expr = expr1;
	//					}
	//				}
	//			} else {
	//				this.expr = arg;
	//			}
	//			//Re-type this expression :
	//			if (this.normalized && (this.expr != null) && (this.typeVisitor != null)) {
	//				this.expr.accept(this.typeVisitor);
	//			}
	//			//Update response context with current context to pass the answers of parent questions:
	//			this.context.update(localCtx);
	//		} catch (Exception e) {
	//			if (e instanceof NormalizeException)
	//				throw (NormalizeException) e;
	//			else
	//				throw new NormalizeException(0, "Unexpected exception in ListOpEXCEPTExpression :\n==>" + e.toString() + "<==\n", e);
	//		}
	//	}

	/**
	 * This visit method normalizes a ListOpINTERSECTExpression (E1 intersect E2) and its sub expressions (E1 and E2).<BR>
	 * This method apply theses rules:<BR>
	 * <OL>
	 * <LI>N(E1 intersect E2) => N(E1) intersect N(E2)</LI>
	 * <LI>N(null intersect E) =>  null (where E cannot be evaluated)</LI>
	 * <LI>N(E intersect null) =>  null (where E cannot be evaluated)</LI>
	 * <LI>N(null intersect null) =>  null</LI>
	 * </OL>
	 * It also answers the questions of the current context.
	 * @param arg a ListOpINTERSECTExpression to normalize.
	 */
	//	public void visit(ListOpINTERSECTExpression arg) throws NormalizeException {
	//		try {
	//			ExpressionNormalizationVisitor recurseVisitor = new ExpressionNormalizationVisitor(this.depth, this.typeVisitor);
	//			VisitorContext localCtx = new VisitorContext();
	//			localCtx.merge(this.context, false);
	//			//Clean all questions which depth is smaller than current depth:
	//			localCtx.filter(this.depth);
	//			recurseVisitor.setContext(localCtx);
	//			//Normalize first expression:
	//			XQueryExpression expr1 = arg.getExpression1();
	//			expr1.accept(recurseVisitor);
	//			localCtx.update(recurseVisitor.getContext());
	//			expr1 = recurseVisitor.getExpr();
	//			//Update the normalized flag:
	//			this.normalized = recurseVisitor.wasNormalized();
	//			if (this.normalized) {
	//				if (expr1 != null) {
	//					arg.setExpression1(expr1);
	//				} else {
	//					arg = null;
	//				}
	//			}
	//			//Normalize second expression:
	//			XQueryExpression expr2 = null;
	//			if (arg != null) {
	//				expr2 = arg.getExpression2();
	//				expr2.accept(recurseVisitor);
	//				localCtx.update(recurseVisitor.getContext());
	//				expr2 = recurseVisitor.getExpr();
	//				//Update the normalized flag:
	//				this.normalized = this.normalized | recurseVisitor.wasNormalized();
	//				if (recurseVisitor.wasNormalized()) {
	//					if (expr2 != null) {
	//						arg.setExpression2(expr2);
	//					} else {
	//						arg = null;
	//					}
	//				}
	//			}
	//			this.expr = arg;
	//			//Re-type this expression :
	//			if (this.normalized && (this.expr != null) && (this.typeVisitor != null)) {
	//				this.expr.accept(this.typeVisitor);
	//			}
	//			//Update response context with current context to pass the answers of parent questions:
	//			this.context.update(localCtx);
	//		} catch (Exception e) {
	//			if (e instanceof NormalizeException)
	//				throw (NormalizeException) e;
	//			else
	//				throw new NormalizeException(0, "Unexpected exception in ListOpINTERSECTExpression :\n==>" + e.toString() + "<==\n", e);
	//		}
	//	}

	/**
	 * This visit method normalizes a ListOpUNIONExpression (E1 union E2) and its sub expressions.<BR>
	 * This method apply theses rules:<BR>
	 * <OL>
	 * <LI>N(E1 union E2) => N(E1) intersect N(E2)</LI>
	 * <LI>N(null union E) =>  N(E) (where E cannot be evaluated)</LI>
	 * <LI>N(E union null) =>  N(E) (where E cannot be evaluated)</LI>
	 * <LI>N(null union null) =>  null</LI>
	 * </OL>
	 * It also answers the questions of the current context.
	 * @param arg a ListOpUNIONExpression to normalize.
	 */
	// erase for now   05/09/2003

	public void visit(ListOpUNIONExpression arg) throws NormalizeException {
		try {
			ExpressionNormalizationVisitor recurseVisitor = new ExpressionNormalizationVisitor(this.depth, this.typeVisitor);
			VisitorContext localCtx = new VisitorContext();
			localCtx.merge(this.context, false);
			//Clean all questions which depth is smaller than current depth:
			localCtx.filter(this.depth);
			recurseVisitor.setContext(localCtx);
			//Normalize first expression:
			XQueryExpression expr1 = arg.getExpression1();
			expr1.accept(recurseVisitor);
			localCtx.update(recurseVisitor.getContext());
			expr1 = recurseVisitor.getExpr();
			//Update the normalized flag:
			this.normalized = recurseVisitor.wasNormalized();
			if ((this.normalized) && (expr1 != null)) {
				arg.setExpression1(expr1);
			}
			//Normalize second expression:
			XQueryExpression expr2 = arg.getExpression2();
			expr2.accept(recurseVisitor);
			localCtx.update(recurseVisitor.getContext());
			expr2 = recurseVisitor.getExpr();
			//Update the normalized flag:
			this.normalized = this.normalized | recurseVisitor.wasNormalized();
			if (recurseVisitor.wasNormalized() && (expr2 != null)) {
				arg.setExpression2(expr2);
			}
			if (expr1 == null) {
				this.expr = expr2;
			} else if (expr2 == null) {
				this.expr = expr1;
			} else {
				this.expr = arg;
			}
			//Re-type this expression :
			if (this.normalized && (this.expr != null) && (this.typeVisitor != null)) {
				this.expr.accept(this.typeVisitor);
			}
			/*
			// CHANGE LARS 07/10/2003 to avoid union of FLWRs
			//For FLWR expression:
			if (this.context.existsMemo(this.depth, VisitorContext._LISTOPUNIONEXPRESSION_GETEXPRESSIONS) && (expr1 != null) && (expr2 != null)) {
				ArrayList exprs = new ArrayList();
				exprs.add(expr1);
				exprs.add(expr2);
				localCtx.setMemo(this.depth, VisitorContext._LISTOPUNIONEXPRESSION_GETEXPRESSIONS, exprs, true);
			}
			*/
			//Update response context with current context to pass the answers of parent questions:
			this.context.update(localCtx);
		} catch (Exception e) {
			if (e instanceof NormalizeException)
				throw (NormalizeException) e;
			else
				throw new NormalizeException(0, "Unexpected exception in ListOpUNIONExpression :\n==>" + e.toString() + "<==\n", e);
		}
	}

	/**
	 * This visit method normalizes a QuantifiedExpression (some|every $var in E1 satisfies E2) and
	 * its sub expressions (E1 and E2).<BR>
	 * This method apply theses rules:<BR>
	 * <OL>
	 * <LI>N(every|some $var in E satisfies null|false())
	 *  => false()</LI>
	 * <LI>N(every|some $var in E satisfies true())
	 *  => true()</LI>
	 * <LI>N(every $var in null satisfies E)
	 *  => true()</LI>
	 * <LI>N(some $var in null satisfies E)
	 *  => false()</LI>
	 * <LI>N(some %xi in Ei, $y in (
	 *  for $z in E2 where C2 return R2)
	 * satisfies C1(%xi, $y))
	 *  => some %xi in Ei, $z in E2
	 * satisfies C1(%xi, R2) and C2</LI>
	 * </OL>
	 * It possibly answers the questions of the current context.
	 * @param arg a QuantifiedExpression to normalize.
	 */
	public void visit(QuantifiedExpression arg) throws NormalizeException {
		try {
			ExpressionNormalizationVisitor recurseVisitor = new ExpressionNormalizationVisitor(this.depth, this.typeVisitor);
			VisitorContext localCtx = new VisitorContext();
			localCtx.merge(this.context, false);
			//Clean all questions which depth is smaller than current depth :
			localCtx.filter(this.depth);
			//To inform the sub variable to normalize its subexpression because it is a variable declaration :
			localCtx.setMemo(this.depth + 1, VisitorContext._VARIABLE_DECLARATION, null, true);
			this.normalized = false;
			//Questions for the rule SOME(FLWR) :
			localCtx.setMemo(this.depth + 2, VisitorContext._XQUERYEXPRESSIONSEQUENCE_EXISTS, null, true);
			localCtx.setMemo(this.depth + 2, VisitorContext._FLWREXPRESSION_GETVARCLAUSE4QUANTIFIED, null, true);
			localCtx.setMemo(this.depth + 2, VisitorContext._FLWREXPRESSION_GETWHERECLAUSE4QUANTIFIED, null, true);
			localCtx.setMemo(this.depth + 2, VisitorContext._FLWREXPRESSION_GETRETURNCLAUSE4QUANTIFIED, null, true);
			localCtx.setMemo(this.depth + 3, VisitorContext._FLWREXPRESSION_GETVARCLAUSE4QUANTIFIED, null, true);
			localCtx.setMemo(this.depth + 3, VisitorContext._FLWREXPRESSION_GETWHERECLAUSE4QUANTIFIED, null, true);
			localCtx.setMemo(this.depth + 3, VisitorContext._FLWREXPRESSION_GETRETURNCLAUSE4QUANTIFIED, null, true);
			//Some or Every variables ?
			int kind = arg.getKind();
			//Normalize each Variables :
			ArrayList variables = arg.getVariables();
			ArrayList newQEVariables = new ArrayList();
			ArrayList innerWhereClauses = new ArrayList();
			Variable currentVariable, newVariable;
			HashMap toSubstitute = new HashMap();
			//Add existing substitutions hashmap :
			if (localCtx.existsMemo(VisitorContext._ANY_DEPTH, VisitorContext._VARIABLE_SUBSTITUTION4QUANTIFIED)) {
				HashMap substToAdd = (HashMap) localCtx.getMemo(VisitorContext._ANY_DEPTH, VisitorContext._VARIABLE_SUBSTITUTION4QUANTIFIED);
				toSubstitute.putAll(substToAdd);
			}
			//------------------ LOOP FOR QUANTIFIED VARIABLES :
			for (int i = 0; i < variables.size(); i++) {
				currentVariable = (Variable) variables.get(i);
				//Normalize this variable :
				recurseVisitor.setContext(localCtx);
				currentVariable.accept(recurseVisitor);
				localCtx.update(recurseVisitor.getContext());
				newVariable = (Variable) recurseVisitor.getExpr();
				//Update the normalized flag :
				this.normalized = this.normalized | recurseVisitor.wasNormalized();
				if (newVariable == null) {
					toSubstitute.put(currentVariable.getVarName(), null);
				} else {
					//Here : there is still the variable :
					//Test if rule SOME(FLWR) can be apply :
					if ((kind == Constants.SOME_QUANTIFIER) && ((localCtx.getMemo(this.depth + 2, VisitorContext._FLWREXPRESSION_GETVARCLAUSE4QUANTIFIED) != null) || ((localCtx.getMemo(this.depth + 2, VisitorContext._XQUERYEXPRESSIONSEQUENCE_EXISTS) != null) && (localCtx.getMemo(this.depth + 3, VisitorContext._FLWREXPRESSION_GETVARCLAUSE4QUANTIFIED) != null)))) {
						boolean existSequence = (localCtx.getMemo(this.depth + 2, VisitorContext._XQUERYEXPRESSIONSEQUENCE_EXISTS) != null);
						//Get inner FLWR parts :
						Variable innerVar = (Variable) (existSequence ? localCtx.getMemo(this.depth + 3, VisitorContext._FLWREXPRESSION_GETVARCLAUSE4QUANTIFIED) : localCtx.getMemo(this.depth + 2, VisitorContext._FLWREXPRESSION_GETVARCLAUSE4QUANTIFIED));
						//Replace current var by this var :
						innerVar.setBindingType(Constants.QUANTIFIER_BINDINGTYPE);
						newQEVariables.add(innerVar);
						XQueryExpression innerReturn = (XQueryExpression) (existSequence ? localCtx.getMemo(this.depth + 3, VisitorContext._FLWREXPRESSION_GETRETURNCLAUSE4QUANTIFIED) : localCtx.getMemo(this.depth + 2, VisitorContext._FLWREXPRESSION_GETRETURNCLAUSE4QUANTIFIED));
						//Add return clause in substitution hashmap :
						toSubstitute.put(currentVariable.getVarName(), innerReturn);
						localCtx.setMemo(VisitorContext._ANY_DEPTH, VisitorContext._VARIABLE_SUBSTITUTION4QUANTIFIED, toSubstitute, true);
						XQueryExpression innerWhere = (XQueryExpression) (existSequence ? localCtx.getMemo(this.depth + 3, VisitorContext._FLWREXPRESSION_GETWHERECLAUSE4QUANTIFIED) : localCtx.getMemo(this.depth + 2, VisitorContext._FLWREXPRESSION_GETWHERECLAUSE4QUANTIFIED));
						//Add where clause :
						if (innerWhere != null)
							innerWhereClauses.add(innerWhere);
						//Clean context :
						localCtx.setMemo(this.depth + 2, VisitorContext._XQUERYEXPRESSIONSEQUENCE_EXISTS, null, true);
						if (existSequence) {
							localCtx.setMemo(this.depth + 3, VisitorContext._FLWREXPRESSION_GETVARCLAUSE4QUANTIFIED, null, true);
							localCtx.setMemo(this.depth + 3, VisitorContext._FLWREXPRESSION_GETWHERECLAUSE4QUANTIFIED, null, true);
							localCtx.setMemo(this.depth + 3, VisitorContext._FLWREXPRESSION_GETRETURNCLAUSE4QUANTIFIED, null, true);
						} else {
							localCtx.setMemo(this.depth + 2, VisitorContext._FLWREXPRESSION_GETVARCLAUSE4QUANTIFIED, null, true);
							localCtx.setMemo(this.depth + 2, VisitorContext._FLWREXPRESSION_GETWHERECLAUSE4QUANTIFIED, null, true);
							localCtx.setMemo(this.depth + 2, VisitorContext._FLWREXPRESSION_GETRETURNCLAUSE4QUANTIFIED, null, true);
						}
					} else {
						newQEVariables.add(newVariable);
					}
				}
			}
			//Remove questions :
			localCtx.removeMemo(this.depth + 2, VisitorContext._XQUERYEXPRESSIONSEQUENCE_EXISTS);
			localCtx.removeMemo(this.depth + 2, VisitorContext._FLWREXPRESSION_GETVARCLAUSE4QUANTIFIED);
			localCtx.removeMemo(this.depth + 2, VisitorContext._FLWREXPRESSION_GETWHERECLAUSE4QUANTIFIED);
			localCtx.removeMemo(this.depth + 2, VisitorContext._FLWREXPRESSION_GETRETURNCLAUSE4QUANTIFIED);
			localCtx.removeMemo(this.depth + 3, VisitorContext._FLWREXPRESSION_GETVARCLAUSE4QUANTIFIED);
			localCtx.removeMemo(this.depth + 3, VisitorContext._FLWREXPRESSION_GETWHERECLAUSE4QUANTIFIED);
			localCtx.removeMemo(this.depth + 3, VisitorContext._FLWREXPRESSION_GETRETURNCLAUSE4QUANTIFIED);
			//Clean context (if we found a variable, this is not a declaration now) :
			localCtx.removeMemo(this.depth + 1, VisitorContext._VARIABLE_DECLARATION);
			//------------------ END LOOP FOR QUANTIFIED VARIABLES.
			if (newQEVariables.size() == 0) {
				//Expressions of variables where eliminated. (this.normalized is true !)
				if (kind == Constants.EVERY_QUANTIFIER) {
					this.expr = new FunctionTRUE(null, this.typeVisitor);
				} else { //kind == Constants.SOME_QUANTIFIER
					this.expr = new FunctionFALSE(null, this.typeVisitor);
				}
			} else {
				//Update variables :
				arg.setVariables(newQEVariables);
				//--------------------- Normalize constraint :
				//Update context :
				recurseVisitor.setContext(localCtx);
				//Get constraint :
				XQueryExpression constraint = arg.getConstraintExpresson();
				//Add inner where clauses to constraint :
				if (innerWhereClauses.size() > 0) {
					constraint.setParenthesis(true);
					this.normalized = true;
					do {
						constraint = new BinOpANDExpression((XQueryExpression) innerWhereClauses.get(0), constraint, this.typeVisitor);
						constraint.setParenthesis(true);
						innerWhereClauses.remove(0);
					} while (innerWhereClauses.size() > 0);
				}
				//For multivalued expression (put in exists function) :
				localCtx.setMemo((this.depth + 1), VisitorContext._MULTIVALUED_NEEDEXISTSFUNCTION, null, true);
				//Normalize constraint :
				constraint.accept(recurseVisitor);
				localCtx.update(recurseVisitor.getContext());
				constraint = recurseVisitor.getExpr();
				//Update the normalized flag :
				this.normalized = this.normalized | recurseVisitor.wasNormalized();
				//For multivalued expression (put in exists function) :
				localCtx.removeMemo((this.depth + 1), VisitorContext._MULTIVALUED_NEEDEXISTSFUNCTION);
				if (this.normalized) {
					if (constraint == null) {
						this.expr = new FunctionFALSE(null, this.typeVisitor);
					} else if ((constraint instanceof FunctionFALSE) || (constraint instanceof FunctionTRUE)) {
						this.expr = constraint;
						this.normalized = true;
					} else {
						arg.setExpression(constraint);
						this.expr = arg;
					}
				} else {
					this.expr = arg;
				}
				//Remove substitution memo :
				localCtx.removeMemo(VisitorContext._ANY_DEPTH, VisitorContext._VARIABLE_SUBSTITUTION4QUANTIFIED);
			}
			//Re-type this expression :
			if (this.normalized && (this.expr != null) && (this.typeVisitor != null)) {
				this.expr.accept(this.typeVisitor);
			}
			//Update response context with current context to pass the answers of parent questions :
			this.context.update(localCtx);
		} catch (Exception e) {
			if (e instanceof NormalizeException)
				throw (NormalizeException) e;
			else
				throw new NormalizeException(0, "Unexpected exception in QuantifiedExpression :\n==>" + e.toString() + "<==\n", e);
		}
	}

	/**
	 * This visit method normalizes a RangeExpression [E1 to E2] and its sub expressions (E1 and E2).<BR>
	 * This method apply theses rules:<BR>
	 * <OL>
	 * <LI>N([E1 to E2]) => [N(E1) to N(E2)]</LI>
	 * <LI>N([E1 to E2]) => null if E1 == null or E2 == null</LI>
	 * </OL>
	 * It possibly answers the questions of the current context.
	 * @param arg a RangeExpression to normalize.
	 */
	//	public void visit(RangeExpression arg) throws NormalizeException {
	//		try {
	//			ExpressionNormalizationVisitor recurseVisitor = new ExpressionNormalizationVisitor(this.depth, this.typeVisitor);
	//			VisitorContext localCtx = new VisitorContext();
	//			localCtx.merge(this.context, false);
	//			//Clean all questions which depth is smaller than current depth:
	//			localCtx.filter(this.depth);
	//			recurseVisitor.setContext(localCtx);
	//			//Normalize first sub expression :
	//			XQueryExpression expr1 = arg.getExpression1();
	//			expr1.accept(recurseVisitor);
	//			localCtx.update(recurseVisitor.getContext());
	//			expr1 = recurseVisitor.getExpr();
	//			//Update the normalized flag:
	//			this.normalized = recurseVisitor.wasNormalized();
	//			if (this.normalized) {
	//				if (expr1 != null) {
	//					arg.setExpression1(expr1);
	//				} else {
	//					arg = null;
	//				}
	//			}
	//			if (expr1 != null) {
	//				//Normalize second sub expression :
	//				XQueryExpression expr2 = arg.getExpression2();
	//				expr2.accept(recurseVisitor);
	//				localCtx.update(recurseVisitor.getContext());
	//				expr2 = recurseVisitor.getExpr();
	//				//Update the normalized flag:
	//				this.normalized = this.normalized | recurseVisitor.wasNormalized();
	//				if (recurseVisitor.wasNormalized()) {
	//					if (expr2 != null) {
	//						arg.setExpression2(expr2);
	//					} else {
	//						arg = null;
	//					}
	//				}
	//			}
	//			this.expr = arg;
	//			//Re-type this expression :
	//			if (this.normalized && (this.expr != null) && (this.typeVisitor != null)) {
	//				this.expr.accept(this.typeVisitor);
	//			}
	//			//Update response context with current context to pass the answers of parent questions:
	//			this.context.update(localCtx);
	//		} catch (Exception e) {
	//			if (e instanceof NormalizeException)
	//				throw (NormalizeException) e;
	//			else
	//				throw new NormalizeException(0, "Unexpected exception in RangeExpression :\n==>" + e.toString() + "<==\n", e);
	//		}
	//	}

	/******** Values ********/

	/**
	 * This visit method normalizes a Value (do nothing: this is a terminal node).<BR>
	 * It possibly answers the questions of the current context.
	 * @param arg a Value to normalize.
	 */
	//	public void visit(Value arg) throws NormalizeException {
	//		try {
	//			this.expr = arg;
	//			//Re-type this expression :
	//			if (this.typeVisitor != null) {
	//				this.expr.accept(this.typeVisitor);
	//			}
	//		} catch (Exception e) {
	//			if (e instanceof NormalizeException)
	//				throw (NormalizeException) e;
	//			else
	//				throw new NormalizeException(0, "Unexpected exception in Value :\n==>" + e.toString() + "<==\n", e);
	//		}
	//	}

	/**
	 * This visit method normalizes a ValueBoolean (do nothing: this is a terminal node).<BR>
	 * It possibly answers the questions of the current context.
	 * @param arg a ValueBoolean to normalize.
	 */
	//	public void visit(ValueBoolean arg) throws NormalizeException {
	//		try {
	//			this.expr = arg;
	//			//Re-type this expression :
	//			if (this.typeVisitor != null) {
	//				this.expr.accept(this.typeVisitor);
	//			}
	//		} catch (Exception e) {
	//			if (e instanceof NormalizeException)
	//				throw (NormalizeException) e;
	//			else
	//				throw new NormalizeException(0, "Unexpected exception in ValueBoolean :\n==>" + e.toString() + "<==\n", e);
	//		}
	//	}

	/**
	 * This visit method normalizes a ValueDecimal (do nothing: this is a terminal node).<BR>
	 * It possibly answers the questions of the current context.
	 * @param arg a ValueDecimal to normalize.
	 */
	//	public void visit(ValueDecimal arg) throws NormalizeException {
	//		try {
	//			this.expr = arg;
	//			//Re-type this expression :
	//			if (this.typeVisitor != null) {
	//				this.expr.accept(this.typeVisitor);
	//			}
	//		} catch (Exception e) {
	//			if (e instanceof NormalizeException)
	//				throw (NormalizeException) e;
	//			else
	//				throw new NormalizeException(0, "Unexpected exception in ValueDecimal :\n==>" + e.toString() + "<==\n", e);
	//		}
	//	}

	/**
	 * This visit method normalizes a ValueDouble (do nothing: this is a terminal node).<BR>
	 * It possibly answers the questions of the current context.
	 * @param arg a ValueDouble to normalize.
	 */
	//	public void visit(ValueDouble arg) throws NormalizeException {
	//		try {
	//			this.expr = arg;
	//			//Re-type this expression :
	//			if (this.typeVisitor != null) {
	//				this.expr.accept(this.typeVisitor);
	//			}
	//		} catch (Exception e) {
	//			if (e instanceof NormalizeException)
	//				throw (NormalizeException) e;
	//			else
	//				throw new NormalizeException(0, "Unexpected exception in ValueDouble :\n==>" + e.toString() + "<==\n", e);
	//		}
	//	}

	/**
	 * This visit method normalizes a ValueFloat (do nothing: this is a terminal node).<BR>
	 * It possibly answers the questions of the current context.
	 * @param arg a ValueFloat to normalize.
	 */
	//	public void visit(ValueFloat arg) throws NormalizeException {
	//		try {
	//			this.expr = arg;
	//			//Re-type this expression :
	//			if (this.typeVisitor != null) {
	//				this.expr.accept(this.typeVisitor);
	//			}
	//		} catch (Exception e) {
	//			if (e instanceof NormalizeException)
	//				throw (NormalizeException) e;
	//			else
	//				throw new NormalizeException(0, "Unexpected exception in ValueFloat :\n==>" + e.toString() + "<==\n", e);
	//		}
	//	}

	/**
	 * This visit method normalizes a ValueInteger (do nothing: this is a terminal node).<BR>
	 * It possibly answers the questions of the current context.
	 * @param arg a ValueInteger to normalize.
	 */
	//	public void visit(ValueInteger arg) throws NormalizeException {
	//		try {
	//			this.expr = arg;
	//			//Re-type this expression :
	//			if (this.typeVisitor != null) {
	//				this.expr.accept(this.typeVisitor);
	//			}
	//		} catch (Exception e) {
	//			if (e instanceof NormalizeException)
	//				throw (NormalizeException) e;
	//			else
	//				throw new NormalizeException(0, "Unexpected exception in ValueInteger :\n==>" + e.toString() + "<==\n", e);
	//		}
	//	}

	/**
	 * This visit method normalizes a ValueString (do nothing: this is a terminal node).<BR>
	 * It possibly answers the questions of the current context.
	 * @param arg a ValueString to normalize.
	 */
	//	public void visit(ValueString arg) throws NormalizeException {
	//		try {
	//			this.expr = arg;
	//			//Re-type this expression :
	//			if (this.typeVisitor != null) {
	//				this.expr.accept(this.typeVisitor);
	//			}
	//		} catch (Exception e) {
	//			if (e instanceof NormalizeException)
	//				throw (NormalizeException) e;
	//			else
	//				throw new NormalizeException(0, "Unexpected exception in ValueString :\n==>" + e.toString() + "<==\n", e);
	//		}
	//	}

	/**
	 * This visit method normalizes a ValueText (do nothing: this is a terminal node).<BR>
	 * It possibly answers the questions of the current context.
	 * @param arg a ValueText to normalize.
	 */
	//	public void visit(ValueText arg) throws NormalizeException {
	//		try {
	//			this.expr = arg;
	//			//Re-type this expression :
	//			if (this.typeVisitor != null) {
	//				this.expr.accept(this.typeVisitor);
	//			}
	//		} catch (Exception e) {
	//			if (e instanceof NormalizeException)
	//				throw (NormalizeException) e;
	//			else
	//				throw new NormalizeException(0, "Unexpected exception in ValueText :\n==>" + e.toString() + "<==\n", e);
	//		}
	//	}

	/******** Functions definition & call ********/

	/**
	 * This visit method normalizes a FunctionDefinition, its parameters and its expressions.<BR>
	 * This method apply theses rules:<BR>
	 * <OL>
	 * <LI>N(define function name(parameters) return type {E})
	 * => define function name(parameters) return type {N(E)}</LI>
	 * </OL>
	 * It possibly answers the questions of the current context.
	 * @param arg a FunctionDefinition to normalize.
	 */
	//	public void visit(FunctionDefinition arg) throws NormalizeException {
	//		try {
	//			ExpressionNormalizationVisitor recurseVisitor = new ExpressionNormalizationVisitor(this.depth, this.typeVisitor);
	//			VisitorContext localCtx = new VisitorContext();
	//			localCtx.merge(this.context, false);
	//			//Clean all questions which depth is smaller than current depth:
	//			localCtx.filter(this.depth);
	//			recurseVisitor.setContext(localCtx);
	//			//Get and normalize all expressions:
	//			ArrayList expressions = arg.getExpressions();
	//			int exprLen = expressions.size();
	//			for (int i = exprLen - 1; i >= 0; i--) {
	//				((XQueryExpression) expressions.get(i)).accept(recurseVisitor);
	//				localCtx.update(recurseVisitor.getContext());
	//				if (recurseVisitor.getExpr() != null) {
	//					expressions.set(i, recurseVisitor.getExpr());
	//				} else {
	//					expressions.remove(i);
	//				}
	//				//Update the normalized flag:
	//				this.normalized = this.normalized | recurseVisitor.wasNormalized();
	//			}
	//			if (expressions.size() == 0) {
	//				XQueryVoid emptyExpr = new XQueryVoid(this.typeVisitor);
	//				//TOREMOVE ? only for testing !
	//				emptyExpr.setParenthesis(true);
	//				expressions.add(emptyExpr);
	//			}
	//			if (this.normalized) {
	//				arg.setExpressions(expressions);
	//				//Re-type this expression :
	//				if (this.typeVisitor != null) {
	//					arg.accept(this.typeVisitor);
	//				}
	//			}
	//			this.expr = arg;
	//			//Update response context with current context to pass the answers of parent questions:
	//			this.context.update(localCtx);
	//		} catch (Exception e) {
	//			if (e instanceof NormalizeException)
	//				throw (NormalizeException) e;
	//			else
	//				throw new NormalizeException(0, "Unexpected exception in FunctionDefinition :\n==>" + e.toString() + "<==\n", e);
	//		}
	//	}

	/**
	 * This visit method normalizes an InternalFunctionCall and its arguments.<BR>
	 * This method apply theses rules:<BR>
	 * <OL>
	 * <LI>N(define function name(parameters) return type {E})
	 * => define function name(parameters) return type {N(E)}</LI>
	 * </OL>
	 * It possibly answers the questions of the current context.
	 * @param arg an InternalFunctionCall to normalize.
	 */
	//	public void visit(InternalFunctionCall arg) throws NormalizeException {
	//		try {
	//			throw new NormalizeException(0, "InternalFunctionCall not supported : (" + arg + ")");
	//			VisitorContext localCtx = new VisitorContext();
	//			localCtx.merge(this.context, false);
	//			//Clean all questions which depth is smaller than current depth:
	//			localCtx.filter(this.depth);
	//			//Set one information memo for not create useless sequence:
	//			localCtx.setMemo(this.depth+1, VisitorContext._INCLUDEINPARENTHESIS,null,true);
	//			//Get and normalize all parameters:
	//			ArrayList params = arg.getArguments();
	//			if (params != null) {
	//			    ExpressionNormalizationVisitor recurseVisitor = new ExpressionNormalizationVisitor(this.depth, this.typeVisitor);
	//			    recurseVisitor.setContext(localCtx);
	//			    for (int i = 0 ; i < params.size() ; i++) {
	//			        ((XQueryExpression)params.get(i)).accept(recurseVisitor);
	//			        localCtx.update(recurseVisitor.getContext());
	//			        params.set(i, recurseVisitor.getExpr());
	//			        //Update the normalized flag:
	//			        this.normalized = this.normalized | recurseVisitor.wasNormalized();
	//			    }
	//			}
	//			//Test if sub expressions were normalized:
	//			if (this.normalized) {
	//			    //Update current InternalFunctionCall :
	//			    arg.setArguments(params);
	//			    //Re-type this expression :
	//			    if (this.typeVisitor != null) {
	//			        arg.accept(this.typeVisitor);
	//			    }
	//			
	//			}
	//			this.expr = arg;
	//			//Answer questions:
	//			//For LocatedExpression:
	//			if ((localCtx.existsMemo(this.depth, VisitorContext._INTERNALFUNCTIONCALL_NAMEDUNORDERED_PARAMETERS))
	//			    && (arg.getFuncName().getName().equals("unordered"))) {
	//			    localCtx.setMemo(this.depth, VisitorContext._INTERNALFUNCTIONCALL_NAMEDUNORDERED_PARAMETERS,
	//			        ((InternalFunctionCall)this.expr).getArguments(),
	//			        true);
	//			} else if (localCtx.existsMemo(this.depth, VisitorContext.TODO !!!".NodeTest")) {
	//			    //Useless? Only if comment() and processing-instructions() are supported!
	//			    localCtx.setMemo(this.depth, VisitorContext.TODO !!!".NodeTest", this.expr, true);
	//			}
	//			//Update response context with current context to pass the answers of parent questions:
	//			this.context.update(localCtx);
	//		} catch (Exception e) {
	//			if (e instanceof NormalizeException)
	//				throw (NormalizeException) e;
	//			else
	//				throw new NormalizeException(0, "Unexpected exception in InternalFunctionCall :\n==>" + e.toString() + "<==\n", e);
	//		}
	//	}

	/**
	 * This visit method normalizes a LibraryFunctionCall (i.e. userFunctionNameCall(E) and its
	 * arguments (E can be a sequence of parameters and can be empty).<BR>
	 * This method apply theses rules:<BR>
	 * <OL>
	 * <LI>N(userFunctionNameCall(E)) => userFunctionNameCall(N(E))</LI>
	 * <LI>N(userFunctionNameCall()) => userFunctionNameCall()</LI>
	 * <LI>N(userFunctionNameCall(null)) => userFunctionNameCall(())</LI>
	 * </OL>
	 * It possibly answers the questions of the current context.
	 * @param arg a LibraryFunctionCall to normalize.
	 */
	//	public void visit(LibraryFunctionCall arg) throws NormalizeException {
	//		try {
	//			ExpressionNormalizationVisitor recurseVisitor = new ExpressionNormalizationVisitor(this.depth, this.typeVisitor);
	//			VisitorContext localCtx = new VisitorContext();
	//			localCtx.merge(this.context, false);
	//			//Clean all questions which depth is smaller than current depth:
	//			localCtx.filter(this.depth);
	//			//Set one information memo for not create useless sequence:
	//			localCtx.setMemo(this.depth + 1, VisitorContext._INCLUDEINPARENTHESIS, null, true);
	//			recurseVisitor.setContext(localCtx);
	//			//Get and normalize all parameters:
	//			ArrayList params = arg.getArguments();
	//			if ((params != null) && (params.size() > 0)) {
	//				int paramLen = params.size();
	//				for (int i = paramLen - 1; i >= 0; i--) {
	//					((XQueryExpression) params.get(i)).accept(recurseVisitor);
	//					localCtx.update(recurseVisitor.getContext());
	//					if (recurseVisitor.getExpr() != null) {
	//						params.set(i, recurseVisitor.getExpr());
	//					} else {
	//						XQueryVoid emptyExpr = new XQueryVoid(this.typeVisitor);
	//						//TOREMOVE ? only for testing !
	//						emptyExpr.setParenthesis(true);
	//						params.set(i, emptyExpr);
	//					}
	//					//Update the normalized flag:
	//					this.normalized = this.normalized | recurseVisitor.wasNormalized();
	//				}
	//				if (this.normalized) {
	//					if (params.size() == 0) {
	//						arg = null;
	//					} else {
	//						arg.setArguments(params);
	//						//Re-type this expression :
	//						if (this.typeVisitor != null) {
	//							arg.accept(this.typeVisitor);
	//						}
	//					}
	//				}
	//			}
	//			this.expr = arg;
	//			//Update response context with current context to pass the answers of parent questions:
	//			this.context.update(localCtx);
	//		} catch (Exception e) {
	//			if (e instanceof NormalizeException)
	//				throw (NormalizeException) e;
	//			else
	//				throw new NormalizeException(0, "Unexpected exception in LibraryFunctionCall :\n==>" + e.toString() + "<==\n", e);
	//		}
	//	}

	/******** Primitive functions ********/

	/**
	 * This method contains the factorized code of the methods "visit" of some PrimitiveFunctionCall subclasses.
	 * @param arg a subclasse of PrimitiveFunctionCall.
	 * @param aggregate true if this is an aggregate function, false otherwise.
	 * @return normalized parameters of this function.
	 */
	protected ArrayList visitPrimitiveFunctionCall(PrimitiveFunctionCall arg, boolean aggregate) throws NormalizeException {
		try {
			//Get and normalize all parameters:
			ArrayList params = arg.getArguments();
			if ((params != null) && (params.size() > 0)) {
				ExpressionNormalizationVisitor recurseVisitor = new ExpressionNormalizationVisitor(this.depth, this.typeVisitor);
				VisitorContext localCtx = new VisitorContext();
				localCtx.merge(this.context, false);
				//Clean all questions which depth is smaller than current depth:
				localCtx.filter(this.depth);
				//Set one information memo for not create useless sequence:
				localCtx.setMemo(this.depth + 1, VisitorContext._INCLUDEINPARENTHESIS, null, true);
				if (aggregate) {
					//To inform the possible variables which they are in an aggregate function :
					localCtx.setMemo(VisitorContext._ANY_DEPTH, VisitorContext._INAGGREGATEFUNCTION, null, true);
				}
				recurseVisitor.setContext(localCtx);
				for (int i = params.size() - 1; i >= 0; i--) {
					((XQueryExpression) params.get(i)).accept(recurseVisitor);
					localCtx.update(recurseVisitor.getContext());
					if (recurseVisitor.getExpr() != null) {
						params.set(i, recurseVisitor.getExpr());
					} else {
						params.set(i, new XQueryVoid(this.typeVisitor));
					}
					//Update the normalized flag:
					this.normalized = this.normalized | recurseVisitor.wasNormalized();
				}
				//Update response context with current context to pass the answers of parent questions:
				this.context.update(localCtx);
			}
			return params;
		} catch (Exception e) {
			if (e instanceof NormalizeException)
				throw (NormalizeException) e;
			else
				throw new NormalizeException(0, "Unexpected exception in visitPrimitiveFunctionCall(" + arg.getClass().getName() + ") :\n==>" + e.toString() + "<==\n", e);
		}
	}

	// Default primitive function call
	public void visit(PrimitiveFunctionCall arg) throws NormalizeException {
		ArrayList params = this.visitPrimitiveFunctionCall(arg, false);
		if (params != null) {
			try {
				boolean norm = false;
				//We suppose that the parser did his work and that the number of parameters is correct !
				for (int i = 0; i < params.size(); i++) {
					if (params.get(i) instanceof XQueryVoid) {
						this.normalized = true;
						norm = true;
						this.expr = new XQueryVoid(this.typeVisitor);
						break;
					}
				}
				if (!norm && this.normalized) {
					//Update its parameters :
					arg.setArguments(params);
					//Re-type this expression :
					if (this.typeVisitor != null) {
						arg.accept(this.typeVisitor);
					}
					this.expr = arg;
				} else {
					this.expr = arg;
				}
			} catch (Exception e) {
				throw new NormalizeException(0, "Unexpected exception in PrimitiveFunctionCall(" + arg.getClass().getName() + ") :\n==>" + e.toString() + "<==\n", e);
			}
		} else
			this.expr = arg;
	}

	/**
	 * This visit method normalizes a FunctionABS <CODE>double? abs(double? E)</CODE> and its sub expression (E).<BR>
	 * This method apply theses rules:<BR>
	 * <OL>
	 * <LI>N(abs(E)) => abs(N(E))</LI>
	 * <LI>N(abs(()|null)) => ()</LI>
	 * </OL>
	 * It possibly answers the questions of the current context.
	 * @param arg a FunctionABS to normalize.
	 */
	//	public void visit(FunctionABS arg) throws NormalizeException {
	//	    ArrayList params = this.visitPrimitiveFunctionCall(arg, false);
	//	    try {
	//	        //We suppose that the parser did his work and that the number of parameters is correct !
	//	        if (params.get(0) instanceof XQueryVoid) {
	//	            this.normalized = true;
	//	            this.expr = new XQueryVoid(this.typeVisitor);
	//	        } else if (this.normalized) {
	//    	        //Update its parameters :
	//        	    arg.setArguments(params);
	//        	    //Re-type this expression :
	//        	    if (this.typeVisitor != null) {
	//        	        arg.accept(this.typeVisitor);
	//        	    }
	//        	    this.expr = arg;
	//            } else {
	//                this.expr = arg;
	//            }
	//        } catch (Exception e) {
	//            throw new NormalizeException(0,"Unexpected exception in FunctionABS("+arg.getClass().getName()+") :\n==>"+e.toString()+"<==\n",e);
	//        }
	//	}

	/**
	 * This visit method normalizes a FunctionAVG <CODE>double? avg(item* E)</CODE> and its sub expression (E).<BR>
	 * This method apply theses rules:<BR>
	 * <OL>
	 * <LI>N(avg(E)) => avg(N(E))</LI>
	 * <LI>N(avg(()|null)) => ()</LI>
	 * </OL>
	 * It possibly answers the questions of the current context.
	 * @param arg a FunctionAVG to normalize.
	 */
	//	public void visit(FunctionAVG arg) throws NormalizeException {
	//	    ArrayList params = this.visitPrimitiveFunctionCall(arg, true);
	//	    try {
	//	        //We suppose that the parser did his work and that the number of parameters is correct !
	//    	    if (params.get(0) instanceof XQueryVoid) {
	//    	        this.normalized = true;
	//	            this.expr = new XQueryVoid(this.typeVisitor);
	//	        } else if (this.normalized) {
	//    	        //Update its parameters :
	//        	    arg.setArguments(params);
	//        	    //Re-type this expression :
	//        	    if (this.typeVisitor != null) {
	//        	        arg.accept(this.typeVisitor);
	//        	    }
	//        	    this.expr = arg;
	//            } else {
	//                this.expr = arg;
	//            }
	//        } catch (Exception e) {
	//            throw new NormalizeException(0,"Unexpected exception in FunctionAVG("+arg.getClass().getName()+") :\n==>"+e.toString()+"<==\n",e);
	//        }
	//	}

	/**
	 * This visit method normalizes a FunctionCEILING <CODE>double? ceiling(double? E)</CODE> and its sub expression (E).<BR>
	 * This method apply theses rules:<BR>
	 * <OL>
	 * <LI>N(ceiling(E)) => ceiling(N(E))</LI>
	 * <LI>N(ceiling(()|null)) => ()</LI>
	 * </OL>
	 * It possibly answers the questions of the current context.
	 * @param arg a FunctionCEILING to normalize.
	 */
	//	public void visit(FunctionCEILING arg) throws NormalizeException {
	//  	    ArrayList params = this.visitPrimitiveFunctionCall(arg, false);
	//  	    try {
	//  	        //We suppose that the parser did his work and that the number of parameters is correct !
	//    	    if (params.get(0) instanceof XQueryVoid) {
	//    	        this.normalized = true;
	//	            this.expr = new XQueryVoid(this.typeVisitor);
	//	        } else if (this.normalized) {
	//    	        //Update its parameters :
	//        	    arg.setArguments(params);
	//        	    //Re-type this expression :
	//        	    if (this.typeVisitor != null) {
	//        	        arg.accept(this.typeVisitor);
	//        	    }
	//        	    this.expr = arg;
	//            } else {
	//                this.expr = arg;
	//            }
	//        } catch (Exception e) {
	//            throw new NormalizeException(0,"Unexpected exception in FunctionCEILING("+arg.getClass().getName()+") :\n==>"+e.toString()+"<==\n",e);
	//        }
	//	}

	// Default root node function call
	//	public void visit(RootNodeFunctionCall arg) throws NormalizeException {
	//		try {
	//			//Nothing to do :
	//			this.expr = arg;
	//		} catch (Exception e) {
	//			if (e instanceof NormalizeException)
	//				throw (NormalizeException) e;
	//			else
	//				throw new NormalizeException(0, "Unexpected exception in RootNodeFunctionCall :\n==>" + e.toString() + "<==\n", e);
	//		}
	//	}
	/**
	 * This visit method normalizes a FunctionCOLLECTION <CODE>node* collection(string E)</CODE> (do nothing: this is a terminal node).<BR>
	 * It also answers the questions of the current context.
	 * @param arg a FunctionCOLLECTION to normalize.
	 */
	//	public void visit(FunctionCOLLECTION arg) throws NormalizeException {
	//	    try {
	//    	    //Nothing to do :
	//    	    this.expr = arg;
	//    	    //untrace();
	//	    } catch (Exception e) {
	//			if (e instanceof NormalizeException)
	//			    throw (NormalizeException)e;
	//			else
	//			    throw new NormalizeException(0,"Unexpected exception in FunctionCOLLECTION :\n==>"+e.toString()+"<==\n",e);
	//        }
	//	}

	/**
	 * This visit method normalizes a FunctionDOCUMENT <CODE>node* collection(string E)</CODE> (do nothing: this is a terminal node).<BR>
	 * It also answers the questions of the current context.
	 * @param arg a FunctionDOCUMENT to normalize.
	 */
	//	public void visit(FunctionDOCUMENT arg) throws NormalizeException {
	//		try {
	//			//Nothing to do :
	//			this.expr = arg;
	//		} catch (Exception e) {
	//			if (e instanceof NormalizeException)
	//				throw (NormalizeException)e;
	//			else
	//				throw new NormalizeException(0,"Unexpected exception in FunctionDOCUMENT :\n==>"+e.toString()+"<==\n",e);
	//		}
	//	}

	/**
	 * This visit method normalizes a FunctionCONCAT <CODE>string concat()</CODE> or
	 * <CODE>string concat(string? E1)</CODE> or <CODE>string concat(string? E1, string? E2,...)</CODE>
	 * and its sub expressions (Ei).<BR>
	 * This method apply theses rules:<BR>
	 * <OL>
	 * <LI>N(concat(E)) => concat(N(E))</LI>
	 * <LI>N(concat(E1, E2, ...)) => concat(N(E1), N(E2), ...)</LI>
	 * <LI>N(concat(()|null))) => "" (empty string)</LI>
	 * </OL>
	 * It possibly answers the questions of the current context.
	 * @param arg a FunctionCONCAT to normalize.
	 */
	public void visit(FunctionCONCAT arg) throws NormalizeException {
		ArrayList params = this.visitPrimitiveFunctionCall(arg, false);
		try {
			//We suppose that the parser did his work and that the number of parameters is correct !
			if ((params.size() == 0) || ((params.size() == 1) && (params.get(0) instanceof XQueryVoid))) {
				this.normalized = true;
				this.expr = new ValueString("", this.typeVisitor);
			} else if (this.normalized) {
				//Update its parameters :
				arg.setArguments(params);
				//Re-type this expression :
				if (this.typeVisitor != null) {
					arg.accept(this.typeVisitor);
				}
				this.expr = arg;
			} else {
				this.expr = arg;
			}
		} catch (Exception e) {
			throw new NormalizeException(0, "Unexpected exception in FunctionCONCAT(" + arg.getClass().getName() + ") :\n==>" + e.toString() + "<==\n", e);
		}
	}

	/**
	 * This visit method normalizes a FunctionCONTAINS <CODE>boolean? contains(string? E1, string? E2)</CODE>
	 * and its sub expressions (E1 and E2).<BR>
	 * This method apply theses rules:<BR>
	 * <OL>
	 * <LI>N(contains(E1, E2)) => contains(N(E1), N(E2))</LI>
	 * <LI>N(contains(()|null, E2)) => ()</LI>
	 * <LI>N(contains(E1, ()|null)) => ()</LI>
	 * </OL>
	 * It also answers the questions of the current context.
	 * @param arg a FunctionCONTAINS to normalize.
	 */
	//	public void visit(FunctionCONTAINS arg) throws NormalizeException {
	//	    ArrayList params = this.visitPrimitiveFunctionCall(arg, false);
	//  	    try {
	//    	    //We suppose that the parser did his work and that the number of parameters is correct !
	//    	    if ((params.get(0) instanceof XQueryVoid) || (params.get(1) instanceof XQueryVoid)) {
	//    	        this.normalized = true;
	//	            this.expr = new XQueryVoid(this.typeVisitor);
	//	        } else if (this.normalized) {
	//    	        //Update its parameters :
	//        	    arg.setArguments(params);
	//        	    //Re-type this expression :
	//        	    if (this.typeVisitor != null) {
	//        	        arg.accept(this.typeVisitor);
	//        	    }
	//        	    this.expr = arg;
	//            } else {
	//                this.expr = arg;
	//            }
	//        } catch (Exception e) {
	//            throw new NormalizeException(0,"Unexpected exception in FunctionCONTAINS("+arg.getClass().getName()+") :\n==>"+e.toString()+"<==\n",e);
	//        }
	//	}

	/**
	 * This visit method normalizes a FunctionCOUNT <CODE>unsignedInt count(item* E)</CODE> and its sub expression (E).<BR>
	 * This method apply theses rules:<BR>
	 * <OL>
	 * <LI>N(count(E)) => count(N(E))</LI>
	 * </OL>
	 * It possibly answers the questions of the current context.
	 * @param arg a FunctionCOUNT to normalize.
	 */
	//	public void visit(FunctionCOUNT arg) throws NormalizeException {
	//	    ArrayList params = this.visitPrimitiveFunctionCall(arg, true);
	//  	    try {
	//    	    //We suppose that the parser did his work and that the number of parameters is correct !
	//    	    if (this.normalized) {
	//    	        //Update its parameters :
	//        	    arg.setArguments(params);
	//        	    //Re-type this expression :
	//        	    if (this.typeVisitor != null) {
	//        	        arg.accept(this.typeVisitor);
	//        	    }
	//        	    this.expr = arg;
	//            } else {
	//                this.expr = arg;
	//            }
	//        } catch (Exception e) {
	//            throw new NormalizeException(0,"Unexpected exception in FunctionCOUNT("+arg.getClass().getName()+") :\n==>"+e.toString()+"<==\n",e);
	//        }
	//	}

	/**
	 * This visit method normalizes a FunctionCURRENT_DATETIME <CODE>dateTime current-dateTime()</CODE>.<BR>
	 * It possibly answers the questions of the current context.
	 * @param arg a FunctionCURRENT_DATETIME to normalize.
	 */
	//	public void visit(FunctionCURRENT_DATETIME arg) throws NormalizeException {
	//		//Nothing to do !
	//		this.expr = arg;
	//	}

	/**
	 * This visit method normalizes a FunctionDATA <CODE>atomic value* data(node E)</CODE> and its sub expression (E).<BR>
	 * This method apply theses rules:<BR>
	 * <OL>
	 * <LI>N(data(E)) => data(N(E))</LI>
	 * </OL>
	 * It possibly answers the questions of the current context.
	 * @param arg a FunctionDATA to normalize.
	 */
	//	public void visit(FunctionDATA arg) throws NormalizeException {
	//	    ArrayList params = this.visitPrimitiveFunctionCall(arg, false);
	//  	    try {
	//    	    //We suppose that the parser did his work and that the number of parameters is correct !
	//    	    if (this.normalized) {
	//    	        //Update its parameters :
	//        	    arg.setArguments(params);
	//        	    //Re-type this expression :
	//        	    if (this.typeVisitor != null) {
	//        	        arg.accept(this.typeVisitor);
	//        	    }
	//        	    this.expr = arg;
	//            } else {
	//                this.expr = arg;
	//            }
	//        } catch (Exception e) {
	//            throw new NormalizeException(0,"Unexpected exception in FunctionDATA("+arg.getClass().getName()+") :\n==>"+e.toString()+"<==\n",e);
	//        }
	//	}

	/**
	 * This visit method normalizes a FunctionDATE <CODE>date date(string E)</CODE> and its sub expression (E).<BR>
	 * This method apply theses rules:<BR>
	 * <OL>
	 * <LI>N(date(E)) => date(N(E))</LI>
	 * </OL>
	 * It possibly answers the questions of the current context.
	 * @param arg a FunctionDATE to normalize.
	 */
	//	public void visit(FunctionDATE arg) throws NormalizeException {
	//	    ArrayList params = this.visitPrimitiveFunctionCall(arg, false);
	//  	    try {
	//    	    //We suppose that the parser did his work and that the number of parameters is correct !
	//    	    if (this.normalized) {
	//    	        //Update its parameters :
	//        	    arg.setArguments(params);
	//        	    //Re-type this expression :
	//        	    if (this.typeVisitor != null) {
	//        	        arg.accept(this.typeVisitor);
	//        	    }
	//        	    this.expr = arg;
	//            } else {
	//                this.expr = arg;
	//            }
	//        } catch (Exception e) {
	//            throw new NormalizeException(0,"Unexpected exception in FunctionDATE("+arg.getClass().getName()+") :\n==>"+e.toString()+"<==\n",e);
	//        }
	//	}

	/**
	 * This visit method normalizes a FunctionDATETIME <CODE>dateTime dateTime(string E)</CODE> and its sub expression (E).<BR>
	 * This method apply theses rules:<BR>
	 * <OL>
	 * <LI>N(dateTime(E)) => dateTime(N(E))</LI>
	 * </OL>
	 * It possibly answers the questions of the current context.
	 * @param arg a FunctionDATETIME to normalize.
	 */
	//	public void visit(FunctionDATETIME arg) throws NormalizeException {
	//	    ArrayList params = this.visitPrimitiveFunctionCall(arg, false);
	//  	    try {
	//    	    //We suppose that the parser did his work and that the number of parameters is correct !
	//    	    if (this.normalized) {
	//    	        //Update its parameters :
	//        	    arg.setArguments(params);
	//        	    //Re-type this expression :
	//        	    if (this.typeVisitor != null) {
	//        	        arg.accept(this.typeVisitor);
	//        	    }
	//        	    this.expr = arg;
	//            } else {
	//                this.expr = arg;
	//            }
	//        } catch (Exception e) {
	//            throw new NormalizeException(0,"Unexpected exception in FunctionDATETIME("+arg.getClass().getName()+") :\n==>"+e.toString()+"<==\n",e);
	//        }
	//	}

	/**
	 * This visit method normalizes a FunctionDECIMAL <CODE>decimal decimal(string E)</CODE> and its sub expression (E).<BR>
	 * This method apply theses rules:<BR>
	 * <OL>
	 * <LI>N(decimal(E)) => decimal(N(E))</LI>
	 * </OL>
	 * It possibly answers the questions of the current context.
	 * @param arg a FunctionDECIMAL to normalize.
	 */
	//	public void visit(FunctionDECIMAL arg) throws NormalizeException {
	//	    ArrayList params = this.visitPrimitiveFunctionCall(arg, false);
	//  	    try {
	//    	    //We suppose that the parser did his work and that the number of parameters is correct !
	//    	    if (this.normalized) {
	//    	        //Update its parameters :
	//        	    arg.setArguments(params);
	//        	    //Re-type this expression :
	//        	    if (this.typeVisitor != null) {
	//        	        arg.accept(this.typeVisitor);
	//        	    }
	//        	    this.expr = arg;
	//            } else {
	//                this.expr = arg;
	//            }
	//        } catch (Exception e) {
	//            throw new NormalizeException(0,"Unexpected exception in FunctionDECIMAL("+arg.getClass().getName()+") :\n==>"+e.toString()+"<==\n",e);
	//        }
	//	}

	/**
	 * This visit method normalizes a FunctionVALUE_EQUALS <CODE>string? upper-case(string? E)</CODE> and its sub expression (E).<BR>
	 * This method apply theses rules:<BR>
	 * <OL>
	 * <LI>N(value-equals(E1, E2)) => value-equals(N(E1), N(E2))</LI>
	 * <LI>N(value-equals(()|null, E2)) => ()</LI>
	 * <LI>N(value-equals(E1, ()|null)) => ()</LI>
	 * </OL>
	 * It possibly answers the questions of the current context.
	 * @param arg a FunctionVALUE_EQUALS to normalize.
	 */
	//	public void visit(FunctionDEEP_EQUALS arg) throws NormalizeException {
	//		//trace("<o-"+(nodeNumber++)+">FunctionVALUE_EQUALS<o>(hc="+arg.hashCode()+") : "+arg.toString()+" QType="+this.typeToString(arg.getQType()),0);
	//		ArrayList params = this.visitPrimitiveFunctionCall(arg, false);
	//		try {
	//			//We suppose that the parser did his work and that the number of parameters is correct !
	//			if ((params.get(0) instanceof XQueryVoid) || (params.get(1) instanceof XQueryVoid)) {
	//				this.normalized = true;
	//				this.expr = new XQueryVoid(this.typeVisitor);
	//			} else if (this.normalized) {
	//				//Update its parameters :
	//				arg.setArguments(params);
	//				//Re-type this expression :
	//				if (this.typeVisitor != null) {
	//					arg.accept(this.typeVisitor);
	//				}
	//				this.expr = arg;
	//			} else {
	//				this.expr = arg;
	//			}
	//		} catch (Exception e) {
	//			throw new NormalizeException(0,"Unexpected exception in FunctionVALUE_EQUALS("+arg.getClass().getName()+") :\n==>"+e.toString()+"<==\n",e);
	//		}
	//		//untrace();
	//	}

	/**
	 * This visit method normalizes a FunctionDISTINCT_VALUES <CODE>item* distinct-values(item* E)</CODE> and its sub expression (E).<BR>
	 * It possibly answers the questions of the current context.
	 * This method apply theses rules:<BR>
	 * <OL>
	 * <LI>N(distinct-values(E)) => distinct-values(N(E))</LI>
	 * <LI>N(distinct-values(()|null)) => ()</LI>
	 * </OL>
	 * @param arg a FunctionDISTINCT_VALUES to normalize.
	 */
	//	public void visit(FunctionDISTINCT_VALUES arg) throws NormalizeException {
	//	    //trace("<o-"+(nodeNumber++)+">FunctionDISTINCT_VALUES<o>(hc="+arg.hashCode()+") : "+arg.toString()+" QType="+this.typeToString(arg.getQType()),0);
	//	    ArrayList params = this.visitPrimitiveFunctionCall(arg, false);
	//  	    try {
	//    	    //We suppose that the parser did his work and that the number of parameters is correct !
	//    	    if (params.get(0) instanceof XQueryVoid) {
	//    	        this.normalized = true;
	//	            this.expr = new XQueryVoid(this.typeVisitor);
	//	        } else if (this.normalized) {
	//    	        //Update its parameters :
	//        	    arg.setArguments(params);
	//        	    //Re-type this expression :
	//        	    if (this.typeVisitor != null) {
	//        	        arg.accept(this.typeVisitor);
	//        	    }
	//        	    this.expr = arg;
	//            } else {
	//                this.expr = arg;
	//            }
	//        } catch (Exception e) {
	//            throw new NormalizeException(0,"Unexpected exception in FunctionDISTINCT_VALUES("+arg.getClass().getName()+") :\n==>"+e.toString()+"<==\n",e);
	//        }
	//	    //untrace();
	//	}

	/**
	 * This visit method normalizes a FunctionDOUBLE <CODE>double double(string E)</CODE> and its sub expression (E).<BR>
	 * This method apply theses rules:<BR>
	 * <OL>
	 * <LI>N(double(E)) => double(N(E))</LI>
	 * </OL>
	 * It possibly answers the questions of the current context.
	 * @param arg a FunctionDOUBLE to normalize.
	 */
	//	public void visit(FunctionDOUBLE arg) throws NormalizeException {
	//	    //trace("<o-"+(nodeNumber++)+">FunctionDOUBLE<o>(hc="+arg.hashCode()+") : "+arg.toString()+" QType="+this.typeToString(arg.getQType()),0);
	//	    ArrayList params = this.visitPrimitiveFunctionCall(arg, false);
	//  	    try {
	//    	    //We suppose that the parser did his work and that the number of parameters is correct !
	//    	    if (this.normalized) {
	//    	        //Update its parameters :
	//        	    arg.setArguments(params);
	//        	    //Re-type this expression :
	//        	    if (this.typeVisitor != null) {
	//        	        arg.accept(this.typeVisitor);
	//        	    }
	//        	    this.expr = arg;
	//            } else {
	//                this.expr = arg;
	//            }
	//        } catch (Exception e) {
	//            throw new NormalizeException(0,"Unexpected exception in FunctionDOUBLE("+arg.getClass().getName()+") :\n==>"+e.toString()+"<==\n",e);
	//        }
	//	    //untrace();
	//	}

	/**
	 * This visit method normalizes a FunctionEMPTY <CODE>boolean empty(item* E)</CODE> and its sub expression (E).<BR>
	 * This method apply theses rules:<BR>
	 * <OL>
	 * <LI>N(empty(E)) => empty(N(E))</LI>
	 * <LI>N(empty(()|null)) => true()</LI>
	 * </OL>
	 * It possibly answers the questions of the current context.
	 * @param arg a FunctionEMPTY to normalize.
	 */
	public void visit(FunctionEMPTY arg) throws NormalizeException {
		//trace("<o-"+(nodeNumber++)+">FunctionEMPTY<o>(hc="+arg.hashCode()+") : "+arg.toString()+" QType="+this.typeToString(arg.getQType()),0);
		ArrayList params = this.visitPrimitiveFunctionCall(arg, false);
		try {
			//We suppose that the parser did his work and that the number of parameters is correct !
			if (params.get(0) instanceof XQueryVoid) {
				this.normalized = true;
				this.expr = new FunctionTRUE(null, this.typeVisitor);
			} else if (this.normalized) {
				//Update its parameters :
				arg.setArguments(params);
				//Re-type this expression :
				if (this.typeVisitor != null) {
					arg.accept(this.typeVisitor);
				}
				this.expr = arg;
			} else {
				this.expr = arg;
			}
		} catch (Exception e) {
			throw new NormalizeException(0, "Unexpected exception in FunctionEMPTY(" + arg.getClass().getName() + ") :\n==>" + e.toString() + "<==\n", e);
		}
		//untrace();
	}

	/**
	 * This visit method normalizes a FunctionENDS_WITH <CODE>boolean? ends-with(string E1, string E2)</CODE>
	 * and its sub expressions (E1, E2).<BR>
	 * This method apply theses rules:<BR>
	 * <OL>
	 * <LI>N(ends-with(E1, E2)) => ends-with(N(E1), N(E2))</LI>
	 * <LI>N(ends-with(()|null, E2)) => ()</LI>
	 * <LI>N(ends-with(E1, ()|null)) => ()</LI>
	 * </OL>
	 * It possibly answers the questions of the current context.
	 * @param arg a FunctionENDS_WITH to normalize.
	 */
	//	public void visit(FunctionENDS_WITH arg) throws NormalizeException {
	//	    //trace("<o-"+(nodeNumber++)+">FunctionENDS_WITH<o>(hc="+arg.hashCode()+") : "+arg.toString()+" QType="+this.typeToString(arg.getQType()),0);
	//	    ArrayList params = this.visitPrimitiveFunctionCall(arg, false);
	//  	    try {
	//    	    //We suppose that the parser did his work and that the number of parameters is correct !
	//    	    if ((params.get(0) instanceof XQueryVoid) || (params.get(1) instanceof XQueryVoid)) {
	//    	        this.normalized = true;
	//	            this.expr = new XQueryVoid(this.typeVisitor);
	//	        } else if (this.normalized) {
	//    	        //Update its parameters :
	//        	    arg.setArguments(params);
	//        	    //Re-type this expression :
	//        	    if (this.typeVisitor != null) {
	//        	        arg.accept(this.typeVisitor);
	//        	    }
	//        	    this.expr = arg;
	//            } else {
	//                this.expr = arg;
	//            }
	//        } catch (Exception e) {
	//            throw new NormalizeException(0,"Unexpected exception in FunctionENDS_WITH("+arg.getClass().getName()+") :\n==>"+e.toString()+"<==\n",e);
	//        }
	//	    //untrace();
	//	}

	/**
	 * This visit method normalizes a FunctionEXISTS <CODE>boolean? exists(item* E)</CODE>
	 * and its sub expression (E).<BR>
	 * This method apply theses rules:<BR>
	 * <OL>
	 * <LI>N(exists(E)) => exists(N(E))</LI>
	 * <LI>N(exists(()|null)) => false()</LI>
	 * </OL>
	 * It possibly answers the questions of the current context.
	 * @param arg a FunctionEXISTS to normalize.
	 */

	// add LARS 15/09/2003
	private GetBoundVariablesVisitor gbvv = new GetBoundVariablesVisitor(true, false);

	public void visit(FunctionEXISTS arg) throws NormalizeException {
		//trace("<o-"+(nodeNumber++)+">FunctionEXISTS<o>(hc="+arg.hashCode()+") : "+arg.toString()+" QType="+this.typeToString(arg.getQType()),0);
		ArrayList params = this.visitPrimitiveFunctionCall(arg, false);
		try {
			//We suppose that the parser did his work and that the number of parameters is correct !
			if (params.get(0) instanceof XQueryVoid) {
				this.normalized = true;
				this.expr = new FunctionFALSE(null, this.typeVisitor);
				return;
			}
			// add LARS 08/10/2003
			// TODO : attention if computed ?
			if (params.get(0) instanceof Element) {
				this.normalized = true;
				this.expr = new FunctionTRUE(null, this.typeVisitor);
				return;
			}
			// add LARS 15/09/2003 to be generalized??? 
			// TODO : generalize this?
			if (params.get(0) instanceof XQueryExpressionSequence) {
				ArrayList subExprs = ((XQueryExpressionSequence) params.get(0)).getSubExpressions();
				if (subExprs == null || subExprs.isEmpty()) {
					this.normalized = true;
					this.expr = new FunctionFALSE(null, this.typeVisitor);
					return;
				}
			}
			gbvv.reset();
			((XQueryExpression) params.get(0)).accept(gbvv);
			if (gbvv.getVariables() == null || gbvv.getVariables().isEmpty()) {
				this.normalized = true;
				this.expr = new FunctionTRUE(null, this.typeVisitor);
				return;
			}
			if (this.normalized) {
				//Update its parameters :
				arg.setArguments(params);
				//Re-type this expression :
				if (this.typeVisitor != null) {
					arg.accept(this.typeVisitor);
				}
				this.expr = arg;
			} else {
				this.expr = arg;
			}
		} catch (Exception e) {
			throw new NormalizeException(0, "Unexpected exception in FunctionEXISTS(" + arg.getClass().getName() + ") :\n==>" + e.toString() + "<==\n", e);
		}
		//untrace();
	}

	/**
	 * This visit method normalizes a FunctionFALSE <CODE>boolean false()</CODE> and its sub expression (E).<BR>
	 * It possibly answers the questions of the current context.
	 * @param arg a FunctionFALSE to normalize.
	 */
	//	public void visit(FunctionFALSE arg) throws NormalizeException {
	//		//trace("<o-"+(nodeNumber++)+">FunctionFALSE<o>(hc="+arg.hashCode()+") : "+arg.toString()+" QType="+this.typeToString(arg.getQType()),0);
	//		//Nothing to do :
	//		this.expr = arg;
	//		//untrace();
	//	}

	/**
	 * This visit method normalizes a FunctionFLOAT <CODE>float float(string E)</CODE> and its sub expression (E).<BR>
	 * This method apply theses rules:<BR>
	 * <OL>
	 * <LI>N(float(E)) => float(N(E))</LI>
	 * </OL>
	 * It possibly answers the questions of the current context.
	 * @param arg a FunctionFLOAT to normalize.
	 */
	//	public void visit(FunctionFLOAT arg) throws NormalizeException {
	//	    //trace("<o-"+(nodeNumber++)+">FunctionFLOAT<o>(hc="+arg.hashCode()+") : "+arg.toString()+" QType="+this.typeToString(arg.getQType()),0);
	//	    ArrayList params = this.visitPrimitiveFunctionCall(arg, false);
	//  	    try {
	//    	    //We suppose that the parser did his work and that the number of parameters is correct !
	//    	    if (this.normalized) {
	//    	        //Update its parameters :
	//        	    arg.setArguments(params);
	//        	    //Re-type this expression :
	//        	    if (this.typeVisitor != null) {
	//        	        arg.accept(this.typeVisitor);
	//        	    }
	//        	    this.expr = arg;
	//            } else {
	//                this.expr = arg;
	//            }
	//        } catch (Exception e) {
	//            throw new NormalizeException(0,"Unexpected exception in FunctionFLOAT("+arg.getClass().getName()+") :\n==>"+e.toString()+"<==\n",e);
	//        }
	//	    //untrace();
	//	}

	/**
	 * This visit method normalizes a FunctionFLOOR <CODE>double? floor(double? E)</CODE> and its sub expression (E).<BR>
	 * This method apply theses rules:<BR>
	 * <OL>
	 * <LI>N(floor(E)) => floor(N(E))</LI>
	 * <LI>N(floor(()|null)) => ()</LI>
	 * </OL>
	 * It possibly answers the questions of the current context.
	 * @param arg a FunctionFLOOR to normalize.
	 */
	//	public void visit(FunctionFLOOR arg) throws NormalizeException {
	//	    //trace("<o-"+(nodeNumber++)+">FunctionFLOOR<o>(hc="+arg.hashCode()+") : "+arg.toString()+" QType="+this.typeToString(arg.getQType()),0);
	//	    ArrayList params = this.visitPrimitiveFunctionCall(arg, false);
	//  	    try {
	//    	    //We suppose that the parser did his work and that the number of parameters is correct !
	//    	    if (params.get(0) instanceof XQueryVoid) {
	//    	        this.normalized = true;
	//	            this.expr = new XQueryVoid(this.typeVisitor);
	//	        } else if (this.normalized) {
	//    	        //Update its parameters :
	//        	    arg.setArguments(params);
	//        	    //Re-type this expression :
	//        	    if (this.typeVisitor != null) {
	//        	        arg.accept(this.typeVisitor);
	//        	    }
	//        	    this.expr = arg;
	//            } else {
	//                this.expr = arg;
	//            }
	//        } catch (Exception e) {
	//            throw new NormalizeException(0,"Unexpected exception in FunctionFLOOR("+arg.getClass().getName()+") :\n==>"+e.toString()+"<==\n",e);
	//        }
	//	    //untrace();
	//	}

	/**
	 * This visit method normalizes a FunctionGET_MONTH_FROM_DATE <CODE>float float(string E)</CODE> and its sub expression (E).<BR>
	 * This method apply theses rules:<BR>
	 * <OL>
	 * <LI>N(float(E)) => float(N(E))</LI>
	 * </OL>
	 * It possibly answers the questions of the current context.
	 * @param arg a FunctionGET_MONTH_FROM_DATE to normalize.
	 */
	//	public void visit(FunctionGET_MONTH_FROM_DATE arg) throws NormalizeException {
	//		//trace("<o-"+(nodeNumber++)+">FunctionGET_MONTH_FROM_DATE<o>(hc="+arg.hashCode()+") : "+arg.toString()+" QType="+this.typeToString(arg.getQType()),0);
	//		ArrayList params = this.visitPrimitiveFunctionCall(arg, false);
	//		try {
	//			//We suppose that the parser did his work and that the number of parameters is correct !
	//			if (this.normalized) {
	//				//Update its parameters :
	//				arg.setArguments(params);
	//				//Re-type this expression :
	//				if (this.typeVisitor != null) {
	//					arg.accept(this.typeVisitor);
	//				}
	//				this.expr = arg;
	//			} else {
	//				this.expr = arg;
	//			}
	//		} catch (Exception e) {
	//			throw new NormalizeException(0,"Unexpected exception in FunctionGET_MONTH_FROM_DATE("+arg.getClass().getName()+") :\n==>"+e.toString()+"<==\n",e);
	//		}
	//		//untrace();
	//	}

	/**
	 * This visit method normalizes a FunctionGET_YEAR_FROM_DATE <CODE>float float(string E)</CODE> and its sub expression (E).<BR>
	 * This method apply theses rules:<BR>
	 * <OL>
	 * <LI>N(float(E)) => float(N(E))</LI>
	 * </OL>
	 * It possibly answers the questions of the current context.
	 * @param arg a FunctionGET_YEAR_FROM_DATE to normalize.
	 */
	//	public void visit(FunctionGET_YEAR_FROM_DATE arg) throws NormalizeException {
	//		//trace("<o-"+(nodeNumber++)+">FunctionGET_YEAR_FROM_DATE<o>(hc="+arg.hashCode()+") : "+arg.toString()+" QType="+this.typeToString(arg.getQType()),0);
	//		ArrayList params = this.visitPrimitiveFunctionCall(arg, false);
	//		try {
	//			//We suppose that the parser did his work and that the number of parameters is correct !
	//			if (this.normalized) {
	//				//Update its parameters :
	//				arg.setArguments(params);
	//				//Re-type this expression :
	//				if (this.typeVisitor != null) {
	//					arg.accept(this.typeVisitor);
	//				}
	//				this.expr = arg;
	//			} else {
	//				this.expr = arg;
	//			}
	//		} catch (Exception e) {
	//			throw new NormalizeException(0,"Unexpected exception in FunctionGET_YEAR_FROM_DATE("+arg.getClass().getName()+") :\n==>"+e.toString()+"<==\n",e);
	//		}
	//		//untrace();
	//	}

	/**
	 * This visit method normalizes a FunctionINPUT. Not yet supported !
	 * @param arg a FunctionINPUT to normalize.
	 */
	//	public void visit(FunctionINPUT arg) throws NormalizeException {
	//		//trace("<o-"+(nodeNumber++)+">FunctionFLOOR<o>(hc="+arg.hashCode()+") : "+arg.toString()+" QType="+this.typeToString(arg.getQType()),0);
	//		throw new NormalizeException(0, "FunctionINPUT not supported : (" + arg + ")");
	//		////untrace();
	//	}

	/**
	 * This visit method normalizes a FunctionINTEGER <CODE>integer integer(string E)</CODE> and its sub expression (E).<BR>
	 * This method apply theses rules:<BR>
	 * <OL>
	 * <LI>N(integer(E)) => integer(N(E))</LI>
	 * </OL>
	 * It possibly answers the questions of the current context.
	 * @param arg a FunctionINTEGER to normalize.
	 */
	//	public void visit(FunctionINTEGER arg) throws NormalizeException {
	//	    //trace("<o-"+(nodeNumber++)+">FunctionINTEGER<o>(hc="+arg.hashCode()+") : "+arg.toString()+" QType="+this.typeToString(arg.getQType()),0);
	//	    ArrayList params = this.visitPrimitiveFunctionCall(arg, false);
	//  	    try {
	//    	    //We suppose that the parser did his work and that the number of parameters is correct !
	//    	    if (this.normalized) {
	//    	        //Update its parameters :
	//        	    arg.setArguments(params);
	//        	    //Re-type this expression :
	//        	    if (this.typeVisitor != null) {
	//        	        arg.accept(this.typeVisitor);
	//        	    }
	//        	    this.expr = arg;
	//            } else {
	//                this.expr = arg;
	//            }
	//        } catch (Exception e) {
	//            throw new NormalizeException(0,"Unexpected exception in FunctionINTEGER("+arg.getClass().getName()+") :\n==>"+e.toString()+"<==\n",e);
	//        }
	//	    //untrace();
	//	}

	/**
	 * This visit method normalizes a FunctionLOCAL_NAME <CODE>string name(E)</CODE> and its sub expression (E).<BR>
	 * This method apply theses rules:<BR>
	 * <OL>
	 * <LI>N(name(E)) => name(N(E))</LI>
	 * <LI>N(name(()|null)) => ()</LI>
	 * </OL>
	 * It possibly answers the questions of the current context.
	 * @param arg a FunctionLOCAL_NAME to normalize.
	 */
	public void visit(FunctionLOCAL_NAME arg) throws NormalizeException {
		//trace("<o-"+(nodeNumber++)+">FunctionLOCAL_NAME<o>(hc="+arg.hashCode()+") : "+arg.toString()+" QType="+this.typeToString(arg.getQType()),0);
		ArrayList params = this.visitPrimitiveFunctionCall(arg, false);
		try {
			//We suppose that the parser did his work and that the number of parameters is correct !
			if (params.get(0) instanceof XQueryVoid) {
				this.normalized = true;
				this.expr = new ValueString("", this.typeVisitor);
			} else if (this.normalized) {
				//Update its parameters :
				arg.setArguments(params);
				//Re-type this expression :
				if (this.typeVisitor != null) {
					arg.accept(this.typeVisitor);
				}
				this.expr = arg;
			} else {
				this.expr = arg;
			}
		} catch (Exception e) {
			throw new NormalizeException(0, "Unexpected exception in FunctionLOCAL_NAME(" + arg.getClass().getName() + ") :\n==>" + e.toString() + "<==\n", e);
		}
		//untrace();
	}

	/**
	 * This visit method normalizes a FunctionLOWER_CASE <CODE>string? lower-case(string? E)</CODE> and its sub expression (E).<BR>
	 * This method apply theses rules:<BR>
	 * <OL>
	 * <LI>N(lower-case(E)) => lower-case(N(E))</LI>
	 * <LI>N(lower-case(()|null)) => ()</LI>
	 * </OL>
	 * It possibly answers the questions of the current context.
	 * @param arg a FunctionLOWER_CASE to normalize.
	 */
	//	public void visit(FunctionLOWER_CASE arg) throws NormalizeException {
	//	    //trace("<o-"+(nodeNumber++)+">FunctionLOWER_CASE<o>(hc="+arg.hashCode()+") : "+arg.toString()+" QType="+this.typeToString(arg.getQType()),0);
	//	    ArrayList params = this.visitPrimitiveFunctionCall(arg, false);
	//  	    try {
	//    	    //We suppose that the parser did his work and that the number of parameters is correct !
	//    	    if (params.get(0) instanceof XQueryVoid) {
	//    	        this.normalized = true;
	//	            this.expr = new XQueryVoid(this.typeVisitor);
	//	        } else if (this.normalized) {
	//    	        //Update its parameters :
	//        	    arg.setArguments(params);
	//        	    //Re-type this expression :
	//        	    if (this.typeVisitor != null) {
	//        	        arg.accept(this.typeVisitor);
	//        	    }
	//        	    this.expr = arg;
	//            } else {
	//                this.expr = arg;
	//            }
	//        } catch (Exception e) {
	//            throw new NormalizeException(0,"Unexpected exception in FunctionLOWER_CASE("+arg.getClass().getName()+") :\n==>"+e.toString()+"<==\n",e);
	//        }
	//	    //untrace();
	//	}

	/**
	 * This visit method normalizes a FunctionMATCH <CODE>boolean? match(string? E1, string? E2)</CODE>
	 * and its sub expressions (E1 and E2).<BR>
	 * This method apply theses rules:<BR>
	 * <OL>
	 * <LI>N(match(E1, E2)) => match(N(E1), N(E2))</LI>
	 * <LI>N(match(()|null, E2)) => ()</LI>
	 * <LI>N(match(E1, ()|null)) => ()</LI>
	 * </OL>
	 * It possibly answers the questions of the current context.
	 * @param arg a FunctionMATCH to normalize.
	 */
	//	public void visit(FunctionMATCHES arg) throws NormalizeException {
	//	    //trace("<o-"+(nodeNumber++)+">FunctionMATCH<o>(hc="+arg.hashCode()+") : "+arg.toString()+" QType="+this.typeToString(arg.getQType()),0);
	//	    ArrayList params = this.visitPrimitiveFunctionCall(arg, true);
	//  	    try {
	//    	    //We suppose that the parser did his work and that the number of parameters is correct !
	//    	    if ((params.get(0) instanceof XQueryVoid) || (params.get(1) instanceof XQueryVoid)) {
	//    	        this.normalized = true;
	//	            this.expr = new XQueryVoid(this.typeVisitor);
	//	        } else if (this.normalized) {
	//    	        //Update its parameters :
	//        	    arg.setArguments(params);
	//        	    //Re-type this expression :
	//        	    if (this.typeVisitor != null) {
	//        	        arg.accept(this.typeVisitor);
	//        	    }
	//        	    this.expr = arg;
	//            } else {
	//                this.expr = arg;
	//            }
	//        } catch (Exception e) {
	//            throw new NormalizeException(0,"Unexpected exception in FunctionMATCH("+arg.getClass().getName()+") :\n==>"+e.toString()+"<==\n",e);
	//        }
	//	    //untrace();
	//	}

	/**
	 * This visit method normalizes a FunctionMAX <CODE>anySimpleType? max(item* E)</CODE> and its sub expression (E).<BR>
	 * This method apply theses rules:<BR>
	 * <OL>
	 * <LI>N(max(E)) => max(N(E))</LI>
	 * <LI>N(max(()|null)) => ()</LI>
	 * </OL>
	 * It possibly answers the questions of the current context.
	 * @param arg a FunctionMAX to normalize.
	 */
	//	public void visit(FunctionMAX arg) throws NormalizeException {
	//	    //trace("<o-"+(nodeNumber++)+">FunctionMAX<o>(hc="+arg.hashCode()+") : "+arg.toString()+" QType="+this.typeToString(arg.getQType()),0);
	//	    ArrayList params = this.visitPrimitiveFunctionCall(arg, true);
	//  	    try {
	//    	    //We suppose that the parser did his work and that the number of parameters is correct !
	//    	    if (params.get(0) instanceof XQueryVoid) {
	//    	        this.normalized = true;
	//	            this.expr = new XQueryVoid(this.typeVisitor);
	//	        } else if (this.normalized) {
	//    	        //Update its parameters :
	//        	    arg.setArguments(params);
	//        	    //Re-type this expression :
	//        	    if (this.typeVisitor != null) {
	//        	        arg.accept(this.typeVisitor);
	//        	    }
	//        	    this.expr = arg;
	//            } else {
	//                this.expr = arg;
	//            }
	//        } catch (Exception e) {
	//            throw new NormalizeException(0,"Unexpected exception in FunctionMAX("+arg.getClass().getName()+") :\n==>"+e.toString()+"<==\n",e);
	//        }
	//	    //untrace();
	//	}

	/**
	 * This visit method normalizes a FunctionMIN <CODE>anySimpleType? min(item* E)</CODE> and its sub expression (E).<BR>
	 * This method apply theses rules:<BR>
	 * <OL>
	 * <LI>N(min(E)) => min(N(E))</LI>
	 * <LI>N(min(()|null)) => ()</LI>
	 * </OL>
	 * It possibly answers the questions of the current context.
	 * @param arg a FunctionMIN to normalize.
	 */
	//	public void visit(FunctionMIN arg) throws NormalizeException {
	//	    //trace("<o-"+(nodeNumber++)+">FunctionMIN<o>(hc="+arg.hashCode()+") : "+arg.toString()+" QType="+this.typeToString(arg.getQType()),0);
	//	    ArrayList params = this.visitPrimitiveFunctionCall(arg, true);
	//  	    try {
	//    	    //We suppose that the parser did his work and that the number of parameters is correct !
	//    	    if (params.get(0) instanceof XQueryVoid) {
	//    	        this.normalized = true;
	//	            this.expr = new XQueryVoid(this.typeVisitor);
	//	        } else if (this.normalized) {
	//    	        //Update its parameters :
	//        	    arg.setArguments(params);
	//        	    //Re-type this expression :
	//        	    if (this.typeVisitor != null) {
	//        	        arg.accept(this.typeVisitor);
	//        	    }
	//        	    this.expr = arg;
	//            } else {
	//                this.expr = arg;
	//            }
	//        } catch (Exception e) {
	//            throw new NormalizeException(0,"Unexpected exception in FunctionMIN("+arg.getClass().getName()+") :\n==>"+e.toString()+"<==\n",e);
	//        }
	//	    //untrace();
	//	}

	//    /**
	//     * This visit method normalizes a FunctionNAME <CODE>string name(E)</CODE> and its sub expression (E).<BR>
	//     * This method apply theses rules:<BR>
	//     * <OL>
	//     * <LI>N(name(E)) => name(N(E))</LI>
	//     * <LI>N(name(()|null)) => ()</LI>
	//     * </OL>
	//     * It possibly answers the questions of the current context.
	//     * @param arg a FunctionNAME to normalize.
	//     */
	//	public void visit(FunctionNAME arg) throws NormalizeException {
	//	    //trace("<o-"+(nodeNumber++)+">FunctionNAME<o>(hc="+arg.hashCode()+") : "+arg.toString()+" QType="+this.typeToString(arg.getQType()),0);
	//	    ArrayList params = this.visitPrimitiveFunctionCall(arg, false);
	//  	    try {
	//    	    //We suppose that the parser did his work and that the number of parameters is correct !
	//    	    if (params.get(0) instanceof XQueryVoid) {
	//    	        this.normalized = true;
	//	            this.expr = new ValueString("", this.typeVisitor);
	//	        } else if (this.normalized) {
	//    	        //Update its parameters :
	//        	    arg.setArguments(params);
	//        	    //Re-type this expression :
	//        	    if (this.typeVisitor != null) {
	//        	        arg.accept(this.typeVisitor);
	//        	    }
	//        	    this.expr = arg;
	//            } else {
	//                this.expr = arg;
	//            }
	//        } catch (Exception e) {
	//            throw new NormalizeException(0,"Unexpected exception in FunctionNAME("+arg.getClass().getName()+") :\n==>"+e.toString()+"<==\n",e);
	//        }
	//	    //untrace();
	//	}

	/**
	 * This visit method normalizes a FunctionNOT <CODE>boolean not(item* E)</CODE> and its sub expression (E).<BR>
	 * This method apply theses rules:<BR>
	 * <OL>
	 * <LI>N(not(E)) => not(N(E))</LI>
	 * <LI>N(not(()|null|false())) => true()</LI>
	 * <LI>N(not(true())) => false()</LI>
	 * </OL>
	 * It possibly answers the questions of the current context.
	 * @param arg a FunctionNOT to normalize.
	 */
	public void visit(FunctionNOT arg) throws NormalizeException {
		//trace("<o-"+(nodeNumber++)+">FunctionNOT<o>(hc="+arg.hashCode()+") : "+arg.toString()+" QType="+this.typeToString(arg.getQType()),0);
		//For multivalued expression (put in exists function) :
		this.context.setMemo((this.depth + 1), VisitorContext._MULTIVALUED_NEEDEXISTSFUNCTION, null, true);
		ArrayList params = this.visitPrimitiveFunctionCall(arg, false);
		//For multivalued expression (put in exists function) :
		this.context.removeMemo((this.depth + 1), VisitorContext._MULTIVALUED_NEEDEXISTSFUNCTION);
		//Apply specifics rules :
		try {
			if (this.normalized) {
				if ((params.get(0) == null) || (params.get(0) instanceof FunctionFALSE) || (params.get(0) instanceof XQueryVoid)) {
					this.expr = new FunctionTRUE(null, this.typeVisitor);
				} else if (params.get(0) instanceof FunctionTRUE) {
					this.expr = new FunctionFALSE(null, this.typeVisitor);
				} else {
					arg.setArguments(params);
					this.expr = arg;
				}
				//Re-type this expression :
				if (this.typeVisitor != null) {
					this.expr.accept(this.typeVisitor);
				}
			} else {
				if ((params.get(0) instanceof FunctionFALSE) || (params.get(0) instanceof XQueryVoid)) {
					this.expr = new FunctionTRUE(null, this.typeVisitor);
					this.normalized = true;
				} else if (params.get(0) instanceof FunctionTRUE) {
					this.expr = new FunctionFALSE(null, this.typeVisitor);
					this.normalized = true;
				} else {
					this.expr = arg;
				}
			}
		} catch (Exception e) {
			if (e instanceof NormalizeException)
				throw (NormalizeException) e;
			else
				throw new NormalizeException(0, "Unexpected exception in FunctionNOT(" + arg.getClass().getName() + ") :\n==>" + e.toString() + "<==\n", e);
		}
		//untrace();
	}

	/**
	 * This visit method normalizes a FunctionNUMBER <CODE>double number()</CODE> or
	 * <CODE>double number(node E)</CODE> and its sub expression (E).<BR>
	 * This method apply theses rules:<BR>
	 * <OL>
	 * <LI>N(number(E)) => number(N(E))</LI>
	 * <LI>N(number(()|null)) => ()</LI>
	 * </OL>
	 * It possibly answers the questions of the current context.
	 * @param arg a FunctionNUMBER to normalize.
	 */
	//	public void visit(FunctionNUMBER arg) throws NormalizeException {
	//	    //trace("<o-"+(nodeNumber++)+">FunctionNUMBER<o>(hc="+arg.hashCode()+") : "+arg.toString()+" QType="+this.typeToString(arg.getQType()),0);
	//	    ArrayList params = this.visitPrimitiveFunctionCall(arg, false);
	//  	    try {
	//    	    //We suppose that the parser did his work and that the number of parameters is correct !
	//    	    if ((params != null) && (params.size() != 0) && (params.get(0) instanceof XQueryVoid)) {
	//    	        this.normalized = true;
	//	            this.expr = new XQueryVoid(this.typeVisitor);
	//	        } else if (this.normalized) {
	//    	        //Update its parameters :
	//        	    arg.setArguments(params);
	//        	    //Re-type this expression :
	//        	    if (this.typeVisitor != null) {
	//        	        arg.accept(this.typeVisitor);
	//        	    }
	//        	    this.expr = arg;
	//            } else {
	//                this.expr = arg;
	//            }
	//        } catch (Exception e) {
	//            throw new NormalizeException(0,"Unexpected exception in FunctionNUMBER"+arg.getClass().getName()+") :\n==>"+e.toString()+"<==\n",e);
	//        }
	//	    //untrace();
	//	}

	/**
	 * This visit method normalizes a FunctionPART (internal function, not xquery => do nothing !)
	 * @param arg a FunctionPART to normalize.
	 */
	//	public void visit(FunctionPART arg) throws NormalizeException {
	//		//trace("<o-"+(nodeNumber++)+">FunctionPART<o>(hc="+arg.hashCode()+") : "+arg.toString()+" QType="+this.typeToString(arg.getQType()),0);
	//		//Do nothing :
	//		this.expr = arg;
	//		//untrace();
	//	}

	/**
	 * This visit method normalizes a FunctionROUND <CODE>double? round(double? E)</CODE> and its sub expression (E).<BR>
	 * This method apply theses rules:<BR>
	 * <OL>
	 * <LI>N(round(E)) => round(N(E))</LI>
	 * <LI>N(round(()|null)) => ()</LI>
	 * </OL>
	 * It possibly answers the questions of the current context.
	 * @param arg a FunctionROUND to normalize.
	 */
	//	public void visit(FunctionROUND arg) throws NormalizeException {
	//	    //trace("<o-"+(nodeNumber++)+">FunctionROUND<o>(hc="+arg.hashCode()+") : "+arg.toString()+" QType="+this.typeToString(arg.getQType()),0);
	//	    ArrayList params = this.visitPrimitiveFunctionCall(arg, false);
	//  	    try {
	//    	    //We suppose that the parser did his work and that the number of parameters is correct !
	//    	    if (params.get(0) instanceof XQueryVoid) {
	//    	        this.normalized = true;
	//	            this.expr = new XQueryVoid(this.typeVisitor);
	//	        } else if (this.normalized) {
	//    	        //Update its parameters :
	//        	    arg.setArguments(params);
	//        	    //Re-type this expression :
	//        	    if (this.typeVisitor != null) {
	//        	        arg.accept(this.typeVisitor);
	//        	    }
	//        	    this.expr = arg;
	//            } else {
	//                this.expr = arg;
	//            }
	//        } catch (Exception e) {
	//            throw new NormalizeException(0,"Unexpected exception in FunctionROUND("+arg.getClass().getName()+") :\n==>"+e.toString()+"<==\n",e);
	//        }
	//	    //untrace();
	//	}

	/**
	 * This visit method normalizes a FunctionSTARTS_WITH <CODE>boolean? starts-with(string? E1, string? E2)</CODE>
	 * and its sub expressions (E1 and E2).<BR>
	 * This method apply theses rules:<BR>
	 * <OL>
	 * <LI>N(starts-with(E, E2)) => starts-with(N(E1), N(E2))</LI>
	 * <LI>N(starts-with(()|null, E2)) => ()</LI>
	 * <LI>N(starts-with(E1, ()|null)) => ()</LI>
	 * </OL>
	 * It possibly answers the questions of the current context.
	 * @param arg a FunctionSTARTS_WITH to normalize.
	 */
	//	public void visit(FunctionSTARTS_WITH arg) throws NormalizeException {
	//	    //trace("<o-"+(nodeNumber++)+">FunctionSTARTS_WITH<o>(hc="+arg.hashCode()+") : "+arg.toString()+" QType="+this.typeToString(arg.getQType()),0);
	//	    ArrayList params = this.visitPrimitiveFunctionCall(arg, false);
	//  	    try {
	//    	    //We suppose that the parser did his work and that the number of parameters is correct !
	//    	    if ((params.get(0) instanceof XQueryVoid) || (params.get(1) instanceof XQueryVoid)) {
	//    	        this.normalized = true;
	//	            this.expr = new XQueryVoid(this.typeVisitor);
	//	        } else if (this.normalized) {
	//    	        //Update its parameters :
	//        	    arg.setArguments(params);
	//        	    //Re-type this expression :
	//        	    if (this.typeVisitor != null) {
	//        	        arg.accept(this.typeVisitor);
	//        	    }
	//        	    this.expr = arg;
	//            } else {
	//                this.expr = arg;
	//            }
	//        } catch (Exception e) {
	//            throw new NormalizeException(0,"Unexpected exception in FunctionSTARTS_WITH("+arg.getClass().getName()+") :\n==>"+e.toString()+"<==\n",e);
	//        }
	//	    //untrace();
	//	}

	/**
	 * This visit method normalizes a FunctionSTRING <CODE>string string(item E)</CODE> and its sub expression (E).<BR>
	 * This method apply theses rules:<BR>
	 * <OL>
	 * <LI>N(string(E)) => string(N(E))</LI>
	 * <LI>N(string(()|null)) => ""</LI>
	 * </OL>
	 * It possibly answers the questions of the current context.
	 * @param arg a FunctionSTRING to normalize.
	 */
	public void visit(FunctionSTRING arg) throws NormalizeException {
		//trace("<o-"+(nodeNumber++)+">FunctionSTRING<o>(hc="+arg.hashCode()+") : "+arg.toString()+" QType="+this.typeToString(arg.getQType()),0);
		ArrayList params = this.visitPrimitiveFunctionCall(arg, false);
		try {
			//We suppose that the parser did his work and that the number of parameters is correct !
			if ((params.size() != 0) && (params.get(0) instanceof XQueryVoid)) {
				this.normalized = true;
				this.expr = new ValueString("", this.typeVisitor);
			} else if (this.normalized) {
				//Update its parameters :
				arg.setArguments(params);
				//Re-type this expression :
				if (this.typeVisitor != null) {
					arg.accept(this.typeVisitor);
				}
				this.expr = arg;
			} else {
				this.expr = arg;
			}
		} catch (Exception e) {
			throw new NormalizeException(0, "Unexpected exception in FunctionSTRING(" + arg.getClass().getName() + ") :\n==>" + e.toString() + "<==\n", e);
		}
		//untrace();
	}

	/**
	 * This visit method normalizes a FunctionSTRING_LENGTH <CODE>integer? string-length(string? E)</CODE>
	 * and its sub expression (E).<BR>
	 * This method apply theses rules:<BR>
	 * <OL>
	 * <LI>N(string-length(E)) => string-length(N(E))</LI>
	 * <LI>N(string-length(()|null)) => ()</LI>
	 * </OL>
	 * It possibly answers the questions of the current context.
	 * @param arg a FunctionSTRING_LENGTH to normalize.
	 */
	//	public void visit(FunctionSTRING_LENGTH arg) throws NormalizeException {
	//	    //trace("<o-"+(nodeNumber++)+">FunctionSTRING_LENGTH<o>(hc="+arg.hashCode()+") : "+arg.toString()+" QType="+this.typeToString(arg.getQType()),0);
	//	    ArrayList params = this.visitPrimitiveFunctionCall(arg, false);
	//  	    try {
	//    	    //We suppose that the parser did his work and that the number of parameters is correct !
	//    	    if (params.get(0) instanceof XQueryVoid) {
	//    	        this.normalized = true;
	//	            this.expr = new XQueryVoid(this.typeVisitor);
	//	        } else if (this.normalized) {
	//    	        //Update its parameters :
	//        	    arg.setArguments(params);
	//        	    //Re-type this expression :
	//        	    if (this.typeVisitor != null) {
	//        	        arg.accept(this.typeVisitor);
	//        	    }
	//        	    this.expr = arg;
	//            } else {
	//                this.expr = arg;
	//            }
	//        } catch (Exception e) {
	//            throw new NormalizeException(0,"Unexpected exception in FunctionSTRING_LENGTH("+arg.getClass().getName()+") :\n==>"+e.toString()+"<==\n",e);
	//        }
	//	    //untrace();
	//	}

	/**
	 * This visit method normalizes a FunctionSUBSTRING <CODE>string? substring(string? E1, decimal? E2)</CODE> or
	 * <CODE>string? substring(string? E1, decimal? E2, decimal? E3)</CODE> and its sub expressions (E1, E2 and E3).<BR>
	 * This method apply theses rules:<BR>
	 * <OL>
	 * <LI>N(substring(E1, E2)) => substring(N(E1), N(E2))</LI>
	 * <LI>N(substring(E1, E2, E3)) => substring(N(E1), N(E2), N(E3))</LI>
	 * <LI>N(substring(()|null, E2, E3) => ()</LI>
	 * <LI>N(substring(E1, ()|null, E3) => ()</LI>
	 * <LI>N(substring(E1, E2, ()|null) => ()</LI>
	 * </OL>
	 * It possibly answers the questions of the current context.
	 * @param arg a FunctionSUBSTRING to normalize.
	 */
	//	public void visit(FunctionSUBSTRING arg) throws NormalizeException {
	//	    //trace("<o-"+(nodeNumber++)+">FunctionSUBSTRING<o>(hc="+arg.hashCode()+") : "+arg.toString()+" QType="+this.typeToString(arg.getQType()),0);
	//	    ArrayList params = this.visitPrimitiveFunctionCall(arg, false);
	//  	    try {
	//    	    //We suppose that the parser did his work and that the number of parameters is correct !
	//    	    if ((params.get(0) instanceof XQueryVoid)
	//    	        || (params.get(1) instanceof XQueryVoid)
	//    	        || ((params.size() == 3) && (params.get(2) instanceof XQueryVoid))) {
	//    	        this.normalized = true;
	//	            this.expr = new XQueryVoid(this.typeVisitor);
	//	        } else if (this.normalized) {
	//    	        //Update its parameters :
	//        	    arg.setArguments(params);
	//        	    //Re-type this expression :
	//        	    if (this.typeVisitor != null) {
	//        	        arg.accept(this.typeVisitor);
	//        	    }
	//        	    this.expr = arg;
	//            } else {
	//                this.expr = arg;
	//            }
	//        } catch (Exception e) {
	//            throw new NormalizeException(0,"Unexpected exception in FunctionSUBSTRING("+arg.getClass().getName()+") :\n==>"+e.toString()+"<==\n",e);
	//        }
	//	    //untrace();
	//	}

	/**
	 * This visit method normalizes a FunctionSUM <CODE>double sum(item* E)</CODE> and its sub expression (E).<BR>
	 * This method apply theses rules:<BR>
	 * <OL>
	 * <LI>N(sum(E)) => sum(N(E))</LI>
	 * <LI>N(sum(()|null)) => 0</LI>
	 * </OL>
	 * It possibly answers the questions of the current context.
	 * @param arg a FunctionSUM to normalize.
	 */
	public void visit(FunctionSUM arg) throws NormalizeException {
		//trace("<o-"+(nodeNumber++)+">FunctionSUM<o>(hc="+arg.hashCode()+") : "+arg.toString()+" QType="+this.typeToString(arg.getQType()),0);
		ArrayList params = this.visitPrimitiveFunctionCall(arg, true);
		try {
			//We suppose that the parser did his work and that the number of parameters is correct !
			if (params.get(0) instanceof XQueryVoid) {
				this.normalized = true;
				this.expr = new ValueDouble("0", this.typeVisitor);
			} else if (this.normalized) {
				//Update its parameters :
				arg.setArguments(params);
				//Re-type this expression :
				if (this.typeVisitor != null) {
					arg.accept(this.typeVisitor);
				}
				this.expr = arg;
			} else {
				this.expr = arg;
			}
		} catch (Exception e) {
			throw new NormalizeException(0, "Unexpected exception in FunctionSUM(" + arg.getClass().getName() + ") :\n==>" + e.toString() + "<==\n", e);
		}
		//untrace();
	}

	/**
	 * This visit method normalizes a FunctionTIME <CODE>time time()</CODE> and its sub expression (E).<BR>
	 * It possibly answers the questions of the current context.
	 * @param arg a FunctionTIME to normalize.
	 */
	//   DANGER differences between FunctionDATE and FuncionTIME ??????
	//	public void visit(FunctionTIME arg) throws NormalizeException {
	//	    //trace("<o-"+(nodeNumber++)+">FunctionTIME<o>(hc="+arg.hashCode()+") : "+arg.toString()+" QType="+this.typeToString(arg.getQType()),0);
	//	    //Nothing to do :
	//	    this.expr = arg;
	//	    //untrace();
	//	}

	/**
	 * This visit method normalizes a FunctionTRUE <CODE>boolean true()</CODE> and its sub expression (E).<BR>
	 * It possibly answers the questions of the current context.
	 * @param arg a FunctionTRUE to normalize.
	 */
	//	public void visit(FunctionTRUE arg) throws NormalizeException {
	//		//trace("<o-"+(nodeNumber++)+">FunctionTRUE<o>(hc="+arg.hashCode()+") : "+arg.toString()+" QType="+this.typeToString(arg.getQType()),0);
	//		//Nothing to do :
	//		this.expr = arg;
	//		//untrace();
	//	}

	/**
	 * This visit method normalizes a FunctionTUPLE (internal function, not xquery => do nothing !)
	 * @param arg a FunctionTUPLE to normalize.
	 */
	//	public void visit(FunctionTUPLE arg) throws NormalizeException {
	//		//trace("<o-"+(nodeNumber++)+">FunctionTUPLE<o>(hc="+arg.hashCode()+") : "+arg.toString()+" QType="+this.typeToString(arg.getQType()),0);
	//		//Do nothing :
	//		this.expr = arg;
	//		//untrace();
	//	}

	/**
	 * This visit method normalizes a FunctionUPPER_CASE <CODE>string? upper-case(string? E)</CODE> and its sub expression (E).<BR>
	 * This method apply theses rules:<BR>
	 * <OL>
	 * <LI>N(upper-case(E)) => upper-case(N(E))</LI>
	 * <LI>N(upper-case(()|null)) => ()</LI>
	 * </OL>
	 * It possibly answers the questions of the current context.
	 * @param arg a FunctionUPPER_CASE to normalize.
	 */
	//	public void visit(FunctionUPPER_CASE arg) throws NormalizeException {
	//	    //trace("<o-"+(nodeNumber++)+">FunctionUPPER_CASE<o>(hc="+arg.hashCode()+") : "+arg.toString()+" QType="+this.typeToString(arg.getQType()),0);
	//        ArrayList params = this.visitPrimitiveFunctionCall(arg, false);
	//  	    try {
	//    	    //We suppose that the parser did his work and that the number of parameters is correct !
	//    	    if (params.get(0) instanceof XQueryVoid) {
	//    	        this.normalized = true;
	//	            this.expr = new XQueryVoid(this.typeVisitor);
	//	        } else if (this.normalized) {
	//    	        //Update its parameters :
	//        	    arg.setArguments(params);
	//        	    //Re-type this expression :
	//        	    if (this.typeVisitor != null) {
	//        	        arg.accept(this.typeVisitor);
	//        	    }
	//        	    this.expr = arg;
	//            } else {
	//                this.expr = arg;
	//            }
	//        } catch (Exception e) {
	//            throw new NormalizeException(0,"Unexpected exception in FunctionUPPER_CASE("+arg.getClass().getName()+") :\n==>"+e.toString()+"<==\n",e);
	//        }
	//	    //untrace();
	//	}

	/******** XPath/Element ********/

	/**
	 * This visit method normalizes an AttributeValuePair (attributeName="E") and its sub expressions
	 * (attributeName and E).<BR>
	 * This method apply theses rules:<BR>
	 * <OL>
	 * <LI>N(attributeName = "E") => N(attributeName) = "N(E)"</LI>
	 * <LI>N(attributeName = "null") => attributeName = ""</LI>
	 * <LI>N(null = "E") => null</LI>
	 * </OL>
	 * It possibly answers the questions of the current context.
	 * @param arg an AttributeValuePair to normalize.
	 */
	public void visit(AttributeValuePair arg) throws NormalizeException {
		try {
			//trace("<o-"+(nodeNumber++)+">AttributeValuePair<o>(hc="+arg.hashCode()+") : "+arg.toString()+" QType="+this.typeToString(arg.getQType()),0);
			ExpressionNormalizationVisitor recurseVisitor = new ExpressionNormalizationVisitor(this.depth, this.typeVisitor);
			VisitorContext localCtx = new VisitorContext();
			localCtx.merge(this.context, false);
			//Clean all questions which depth is smaller than current depth:
			localCtx.filter(this.depth);
			recurseVisitor.setContext(localCtx);
			//Normalize attribut name:
			XQueryExpression attrName = arg.getAttributeName();
			attrName.accept(recurseVisitor);
			localCtx.update(recurseVisitor.getContext());
			attrName = recurseVisitor.getExpr();
			//Update the normalized flag:
			this.normalized = recurseVisitor.wasNormalized();
			if (this.normalized) {
				if (attrName != null) {
					attrName.setBrace(false);
					arg.setAttibuteName(attrName);
				} else {
					arg = null;
				}
			}
			if (arg != null) {
				//Normalize attribut value:
				XQueryExpression attrValue = arg.getAttributeValue();
				attrValue.accept(recurseVisitor);
				localCtx.update(recurseVisitor.getContext());
				attrValue = recurseVisitor.getExpr();
				//Update the normalized flag:
				this.normalized = this.normalized | recurseVisitor.wasNormalized();
				if (recurseVisitor.wasNormalized()) {
					if (attrValue != null) {
						arg.setAttributeValue(attrValue);
					} else {
						arg.setAttributeValue(new ValueString("", this.typeVisitor));
					}
					//Re-type this expression :
					if (this.typeVisitor != null) {
						arg.accept(this.typeVisitor);
					}
				}
			}
			this.expr = arg;
			//Update response context with current context to pass the answers of parent questions:
			this.context.update(localCtx);
			//untrace();
		} catch (Exception e) {
			if (e instanceof NormalizeException)
				throw (NormalizeException) e;
			else
				throw new NormalizeException(0, "Unexpected exception in AttributeValuePair :\n==>" + e.toString() + "<==\n", e);
		}
	}

	/**
	 * This visit method normalizes an CData (<![CDATA[text]]>).<BR>
	 * It possibly answers the questions of the current context.
	 * @param arg a CData to normalize.
	 */
	//	public void visit(CData arg) throws NormalizeException {
	//		try {
	//			//trace("<o-"+(nodeNumber++)+">CData<o>(hc="+arg.hashCode()+") : "+arg.toString()+" QType="+this.typeToString(arg.getQType()),0);
	//			//Nothing to do :
	//			this.expr = arg;
	//			//untrace();
	//		} catch (Exception e) {
	//			if (e instanceof NormalizeException)
	//				throw (NormalizeException) e;
	//			else
	//				throw new NormalizeException(0, "Unexpected exception in CData :\n==>" + e.toString() + "<==\n", e);
	//		}
	//	}

	/**
	 * This visit method normalizes an Document (document(E)).<BR>
	 * It possibly answers the questions of the current context.
	 * @param arg a Document to normalize.
	 */
	//	public void visit(Document arg) throws NormalizeException {
	//		try {
	//			//trace("<o-"+(nodeNumber++)+">Document<o>(hc="+arg.hashCode()+") : "+arg.toString()+" QType="+this.typeToString(arg.getQType()),0);
	//			throw new NormalizeException(0, "Document not supported : (" + arg + ")");
	//			/*//Nothing to do :
	//			this.expr = arg;
	//			//untrace();*/
	//		} catch (Exception e) {
	//			if (e instanceof NormalizeException)
	//				throw (NormalizeException) e;
	//			else
	//				throw new NormalizeException(0, "Unexpected exception in Document :\n==>" + e.toString() + "<==\n", e);
	//		}
	//	}

	/**
	 * This visit method normalizes an Element (<tag>E</tag>) and its sub expressions (E and tag).<BR>
	 * This method apply theses rules:<BR>
	 * <OL>
	 * <LI>N(<tag>E</tag>) => <N(tag)>N(E)</N(tag)> with N(E) = N(E1, E2, E3...) where :<BR>
	 * - if Ei = ValueText or Element => Ei<BR>
	 * - if Ei = enclosedExpression (ex: FLWR, Var, LocatedExpression...) like {Ei1, Ei2...Ein} => {Ei1}{Ei2}...{Ein}</LI>
	 * <LI>N(<tag>null</tag>) => <tag></tag></LI>
	 * <LI>N(<null>E</null>) => null</LI>
	 * </OL>
	 * It possibly answers the questions of the current context.
	 * @param arg an Element to normalize.
	 */
	public void visit(Element arg) throws NormalizeException {
		try {
			//trace("<o-"+(nodeNumber++)+">Element<o>(hc="+arg.hashCode()+") : "+arg.toString()+" QType="+this.typeToString(arg.getQType()),0);
			ExpressionNormalizationVisitor recurseVisitor = new ExpressionNormalizationVisitor(this.depth, this.typeVisitor);
			VisitorContext localCtx = new VisitorContext();
			localCtx.merge(this.context, false);
			//Clean all questions which depth is smaller than current depth:
			localCtx.filter(this.depth);
			recurseVisitor.setContext(localCtx);
			//Normalize tag:
			XQueryExpression tagExpr = arg.getStartTag();
			tagExpr.accept(recurseVisitor);
			localCtx.update(recurseVisitor.getContext());
			tagExpr = recurseVisitor.getExpr();
			//Update the normalized flag:
			this.normalized = recurseVisitor.wasNormalized();
			if (this.normalized) {
				if (tagExpr != null) {
					if (tagExpr instanceof ValueText) {
						tagExpr.setBrace(false);
						arg.setStartTag(tagExpr);
					} else {
						throw new NormalizeException(0, "Tag name of Element is not correctly defined. Its type is " + tagExpr.getClass().getName() + ".");
					}
				} else {
					arg = null;
				}
			}
			ArrayList expressions = null;
			ArrayList attributes = null;
			if (arg != null) {
				//Normalize attributs (if exists):
				attributes = arg.getAttributes();
				if (attributes != null) {
					int attrLen = attributes.size();
					for (int i = attrLen - 1; i >= 0; i--) {
						((AttributeValuePair) attributes.get(i)).accept(recurseVisitor);
						localCtx.update(recurseVisitor.getContext());
						if (recurseVisitor.getExpr() != null) {
							attributes.set(i, recurseVisitor.getExpr());
						} else {
							attributes.remove(i);
						}
						//Update the normalized flag:
						this.normalized = this.normalized | recurseVisitor.wasNormalized();
					}
					if (this.normalized) {
						arg.setAttributes(attributes);
					}
				}
				//Get sub expressions:
				expressions = arg.getSubExpressions();
				if (expressions != null) {
					//Normalize sub expressions :
					int exprLen = expressions.size();
					for (int i = exprLen - 1; i >= 0; i--) {
						((XQueryExpression) expressions.get(i)).accept(recurseVisitor);
						localCtx.update(recurseVisitor.getContext());
						if (recurseVisitor.getExpr() != null) {
							expressions.set(i, recurseVisitor.getExpr());
						} else {
							expressions.remove(i);
						}
						//Update the normalized flag:
						this.normalized = this.normalized | recurseVisitor.wasNormalized();
					}
					//Normalize sub expressions according to type of sub expressions (ex: enclosed expr. {E}):
					if (expressions.size() > 0) { //Can not be null !
						//To normalize sub sequences of expressions :
						ArrayList newExpressions = new ArrayList();
						for (int i = 0; i < expressions.size(); i++) {
							if (expressions.get(i) instanceof XQueryExpressionSequence) {
								ArrayList subExpr = ((XQueryExpressionSequence) expressions.get(i)).getSubExpressions();
								for (int j = 0; j < subExpr.size(); j++) {
									if ((subExpr.get(j) instanceof Element) || (subExpr.get(j) instanceof Value)) {
										newExpressions.add(subExpr.get(j));
									} else {
										ArrayList newSubExpr = new ArrayList();
										newSubExpr.add(subExpr.get(j));
										XQueryExpressionSequence aSeq = new XQueryExpressionSequence(newSubExpr, this.typeVisitor);
										this.setSequenceDelimiters(aSeq, false, true, false);
										newExpressions.add(aSeq);
									}
								}
							} else {
								newExpressions.add(expressions.get(i));
							}
						}
						arg.setSubExpressions(newExpressions);
					} else {
						arg.setSubExpressions(expressions);
					}
				}
			}
			this.expr = arg;
			//Re-type this expression :
			if (this.normalized && (this.expr != null) && (this.typeVisitor != null)) {
				this.expr.accept(this.typeVisitor);
			}
			//Answer questions of the current context (we must use the normalized Element to answer):
			//For FLWRExpression:
			if (localCtx.existsMemo(this.depth, VisitorContext._ELEMENT_EXISTS)) {
				localCtx.setMemo(this.depth, VisitorContext._ELEMENT_EXISTS, Boolean.TRUE, true);
			}
			//Update response context with current context to pass the answers of parent questions:
			this.context.update(localCtx);
			//untrace();
		} catch (Exception e) {
			if (e instanceof NormalizeException)
				throw (NormalizeException) e;
			else
				throw new NormalizeException(0, "Unexpected exception in Element :\n==>" + e.toString() + "<==\n", e);
		}
	}

	/**
	 * This visit method normalizes an XMLComment (<!-- comment -->).<BR>
	 * It possibly answers the questions of the current context.
	 * @param arg a XMLComment to normalize.
	 */
	//	public void visit(XMLComment arg) throws NormalizeException {
	//		try {
	//			//trace("<o-"+(nodeNumber++)+">XMLComment<o>(hc="+arg.hashCode()+") : "+arg.toString()+" QType="+this.typeToString(arg.getQType()),0);
	//			//Nothing to do :
	//			this.expr = arg;
	//			//untrace();
	//		} catch (Exception e) {
	//			if (e instanceof NormalizeException)
	//				throw (NormalizeException) e;
	//			else
	//				throw new NormalizeException(0, "Unexpected exception in XMLComment :\n==>" + e.toString() + "<==\n", e);
	//		}
	//	}

	/**
	 * This visit method normalizes an XMLProcessingInstruction (<?processing instruction?>).<BR>
	 * It possibly answers the questions of the current context.
	 * @param arg a XMLProcessingInstruction to normalize.
	 */
	//	public void visit(XMLProcessingInstruction arg) throws NormalizeException {
	//		try {
	//			//trace("<o-"+(nodeNumber++)+">XMLProcessingInstruction<o>(hc="+arg.hashCode()+") : "+arg.toString()+" QType="+this.typeToString(arg.getQType()),0);
	//			//Nothing to do :
	//			this.expr = arg;
	//			//untrace();
	//		} catch (Exception e) {
	//			if (e instanceof NormalizeException)
	//				throw (NormalizeException) e;
	//			else
	//				throw new NormalizeException(0, "Unexpected exception in XMLProcessingInstruction :\n==>" + e.toString() + "<==\n", e);
	//		}
	//	}

	/******** Other expressions ********/

	/**
	 * This visit method normalizes an CaseClauseExpression.<BR>
	 * It possibly answers the questions of the current context.
	 * @param arg a CaseClauseExpression to normalize.
	 */
	//	public void visit(CaseClauseExpression arg) throws NormalizeException {
	//		try {
	//			//trace("<o-"+(nodeNumber++)+">CaseClauseExpression<o>(hc="+arg.hashCode()+") : "+arg.toString()+" QType="+this.typeToString(arg.getQType()),0);
	//			throw new NormalizeException(0, "CaseClauseExpression not supported : (" + arg + ")");
	//			/*this.expr = arg;
	//			//Re-type this expression :
	//			if (this.typeVisitor != null) {
	//			    this.expr.accept(this.typeVisitor);
	//			}
	//			//untrace();*/
	//		} catch (Exception e) {
	//			if (e instanceof NormalizeException)
	//				throw (NormalizeException) e;
	//			else
	//				throw new NormalizeException(0, "Unexpected exception in CaseClauseExpression :\n==>" + e.toString() + "<==\n", e);
	//		}
	//	}

	/**
	 * This visit method normalizes an CastTreatExpression.<BR>
	 * It possibly answers the questions of the current context.
	 * @param arg a CastTreatExpression to normalize.
	 */
	//	public void visit(CastTreatExpression arg) throws NormalizeException {
	//		try {
	//			//trace("<o-"+(nodeNumber++)+">CastTreatExpression<o>(hc="+arg.hashCode()+") : "+arg.toString()+" QType="+this.typeToString(arg.getQType()),0);
	//			throw new NormalizeException(0, "CastTreatExpression not supported : (" + arg + ")");
	//			/*this.expr = arg;
	//			//Re-type this expression :
	//			if (this.typeVisitor != null) {
	//			    this.expr.accept(this.typeVisitor);
	//			}
	//			//untrace();*/
	//		} catch (Exception e) {
	//			if (e instanceof NormalizeException)
	//				throw (NormalizeException) e;
	//			else
	//				throw new NormalizeException(0, "Unexpected exception in CastTreatExpression :\n==>" + e.toString() + "<==\n", e);
	//		}
	//	}

	/**
	 * To analyse a FLWRExpression (and its subnodes) to determine if a rules can be applied.<BR>
	 * This method apply theses rules:<BR>
	 * <OL>
	 * <LI>N(for %xi in Ei,let %yi := E'i where C return R) =>
	 *  for %xi in N(Ei), let %yi := N(E'i) where N(C) return N(R)</LI>
	 * <LI>N(for %xi in Ei,let %yi := E'i where null return R) => null</LI>
	 * <LI>N(for %xi in Ei,let %yi := E'i where C return null) => null</LI>
	 * <LI>N(for $x in null for %xi in Ei($x) let %yi := E'i($x) where C($x) return R($x)) => null</LI>
	 * <LI>N(let $y := null for %xi in Ei($y)  let %yi := E'i($y)  where C($y) return R($y)) =>
	 *  for %xi in Ei(null)  let %yi := E'i(null)  where C(null) return R(null)</LI>
	 * <LI>N(for %xi in Ei, $y in EC(%xi) where C(%xi,$y)
	 *  return R(%xi,$y)) =>
	 *  N(for %xi in N(Ei), where N(C(%xi,EC(%xi)))
	 *  return N(R(%xi,EC(%xi))))</LI>
	 * <LI>N(for $x in EC where C($x) return R($x)) => N(R(EC)) if N(C(EC)) = true</LI>
	 * <LI>N(for $x in EC return R($x)) => N(R(EC))</LI>
	 * <LI>N(for %xi in Ei, $y in (
	 *  for $z in E2(%xi)
	 *  where C2(%xi, $z)
	 *  return R2(%xi, $z))
	 *  where C1(%xi, $y)
	 *  return R1(%xi, $y)) =>
	 *  for %xi in N(Ei), $z in N(E2(%xi))
	 *  where N(C2(%xi, $z) and C1(%xi, R2(%xi, $z)))
	 *  return N(R1(%xi, R2(%xi, $z))) if R2(%xi, $z) is mono valued.</LI>
	 * <LI>N(for %xi in Ei, $y in (
	 *  for $z in E2(%xi)
	 *  where C2(%xi, $z)
	 *  return R2(%xi, $z))
	 *  where C1(%xi, $y)
	 *  return R1(%xi, $y)) =>
	 *  for %xi in N(Ei), $z in N(E2(%xi))
	 *  where N(C2(%xi, $z) and C1(%xi, R2(%xi, $z)))
	 *  return N(R1(%xi, R2(%xi, $z))) if R2(%xi, $z) is multi valued or not typed</LI>
	 * <LI>N(for %xi in Ei, $z in (E2(%xi) UNION E3(%xi)) where C(%xi, $z) return R(%xi, $z)) =>
	 *  (for %xi in N(Ei), $z in N(E2(%xi)) where N(C(%xi, $z)) return N(R(%xi, $z)))
	 *  UNION
	 *  (for %xi in N(Ei), $z in N(E3(%xi)) where N(C(%xi, $z)) return N(R(%xi, $z)))</LI>
	 *  <LI>N(let %x := Ex,
	 *  let $y := FLWR
	 *  for %z in Ez($y)
	 *  where some $t in E(%x,$y,%z) satisfies C(%x,$y,%z)
	 *  return R(%x,$y,%z))=>
	 *  let %x := N(Ex),
	 *  let $y := N(FLWR)
	 *  for %z in N(Ez(N(FLWR)))
	 *  where N(some $t in E(%x,N(FLWR),%z) satisfies C(%x,N(FLWR),%z))
	 *  return N(R(%x,N(FLWR),%z))
	 *  (no substitution of $y if it is in an aggregate function like count, avg, max, min or sum)</LI>
	 * <LI>N(let %x := Ex,
	 *  let $y := FLWR
	 *  for %z in Ez($y)
	 *  where C(%x,$y,%z)
	 *  return R(%x,$y,%z)) (with C <> QuantifiedExpression)=>
	 *  let %x := N(Ex),
	 *  let $y := N(FLWR)
	 *  for %z in N(Ez(N(FLWR)))
	 *  where some $t in N(FLWR) satisfies N(C(%x,$t,%z))
	 *  return N(R(%x,N(FLWR),%z))
	 *  (no substitution of $y if it is in an aggregate function like count, avg, max, min or sum)</LI>
	 * <LI>N(C(E)) C = condition and E = LE || FLWR || VAR => N(C(EXISTS(E)))</LI>
	 * </OL>
	 * (See class header to learn more about these symbols).<BR>
	 * It possibly answers the questions of the current context.
	 * @param arg a FLWRExpression to normalize.
	 */
	public void visit(FLWRExpression arg) throws NormalizeException {
		try {
			this.expr = null;
			ExpressionNormalizationVisitor recurseVisitor = new ExpressionNormalizationVisitor(this.depth, this.typeVisitor);
			VisitorContext localCtx = new VisitorContext();
			//For rule WHERE(FLWR) : push the data
			ArrayList whereFLWRStack = null;
			if (this.context.existsMemo(VisitorContext._ANY_DEPTH, VisitorContext._FLWREXPRESSION_GETANDREPLACEFLWRINWHERE)) {
				whereFLWRStack = (ArrayList) this.context.getMemo(VisitorContext._ANY_DEPTH, VisitorContext._FLWREXPRESSION_GETANDREPLACEFLWRINWHERE);
				this.context.removeMemo(VisitorContext._ANY_DEPTH, VisitorContext._FLWREXPRESSION_GETANDREPLACEFLWRINWHERE);
			}
			localCtx.merge(this.context, false);
			//Clean all questions which depth is smaller than current depth:
			localCtx.filter(this.depth);
			//Set common context :
			localCtx.setMemo(this.depth + 1, VisitorContext._VARIABLE_DECLARATION, null, true);
			// Order not a question !
			localCtx.setMemo(this.depth + 2, VisitorContext._XQUERYEXPRESSIONSEQUENCE_EXISTS, null, true);
			localCtx.setMemo(this.depth + 2, VisitorContext._ELEMENT_EXISTS, null, true);
			localCtx.setMemo(this.depth + 3, VisitorContext._ELEMENT_EXISTS, null, true);
			//Prepare FOR context :
			VisitorContext localForCtx = new VisitorContext();
			localForCtx.setMemo(this.depth + 2, VisitorContext._FLWREXPRESSION_GETVARCLAUSE, null, true);
			localForCtx.setMemo(this.depth + 2, VisitorContext._FLWREXPRESSION_GETWHERECLAUSE, null, true);
			localForCtx.setMemo(this.depth + 2, VisitorContext._FLWREXPRESSION_GETRETURNCLAUSE, null, true);
			localForCtx.setMemo(this.depth + 3, VisitorContext._FLWREXPRESSION_GETVARCLAUSE, null, true);
			localForCtx.setMemo(this.depth + 3, VisitorContext._FLWREXPRESSION_GETWHERECLAUSE, null, true);
			localForCtx.setMemo(this.depth + 3, VisitorContext._FLWREXPRESSION_GETRETURNCLAUSE, null, true);
			localForCtx.setMemo(this.depth + 2, VisitorContext._LISTOPUNIONEXPRESSION_GETEXPRESSIONS, null, true);
			localForCtx.setMemo(this.depth + 3, VisitorContext._LISTOPUNIONEXPRESSION_GETEXPRESSIONS, null, true);
			//			localForCtx.setMemo(this.depth + 2, VisitorContext._ADDED_VARIABLE_BEFORE, null, true);
			//			localForCtx.setMemo(this.depth + 2, VisitorContext._ADDED_VARIABLE_BEFORE_VAR, null, true);
			localForCtx.merge(localCtx, false);
			//Prepare LET context :
			VisitorContext localLetCtx = new VisitorContext();
			//To deactivate	LET(FLWR) rule :
			//For   :
			//localLetCtx.setMemo(this.depth+2, VisitorContext._FLWREXPRESSION_GETFLWR, null, true);
			//localLetCtx.setMemo(this.depth+3, VisitorContext._FLWREXPRESSION_GETFLWR, null, true);
			localLetCtx.merge(localCtx, false);
			//Create structure to store rule informations :
			HashMap substFor = new HashMap();
			ArrayList newWhere = new ArrayList();
			HashMap exprUnion = new HashMap();
			HashMap substLet = new HashMap();
			HashMap substVarTmp = new HashMap();
			//Local flags:
			boolean allVariablesAreEliminated = false;
			boolean eliminateThisFLWR = false;
			//Get all variables for and/or lets:
			ArrayList variables = arg.getVariables();
			//We supposed that there is at least a variable.
			//--------------------------------------------- NORMALIZE FOR AND LET CLAUSES
			// REMOVE LARS ArrayList newFLWRVariables = new ArrayList();
			Variable currentVar = null;
			//Normalize each variables and search which rule(s) to apply :
			for (int i = 0;(i < variables.size()) && !eliminateThisFLWR; i++) {
				currentVar = (Variable) variables.get(i);
				if (currentVar.getBindingType() == Constants.FOR_BINDINGTYPE) {
					//++++++++++++++++++++++++++++++++++++++ FOR variables :
					recurseVisitor.setContext(localForCtx);
					//Normalize sub expression:
					currentVar.accept(recurseVisitor);
					localForCtx.update(recurseVisitor.getContext());
					//Update the normalized flag:
					this.normalized = this.normalized | recurseVisitor.wasNormalized();
					if (recurseVisitor.getExpr() == null) {
						//To eliminate this variable.
						eliminateThisFLWR = true;
					} else { //recurseVisitor.getExpr() != null
						boolean existSequence = (localForCtx.getMemo(this.depth + 2, VisitorContext._XQUERYEXPRESSIONSEQUENCE_EXISTS) != null);
						//Choice which rule(s) can be apply :
						if ((localForCtx.getMemo(this.depth + 2, VisitorContext._ELEMENT_EXISTS) != null) || ((localForCtx.getMemo(this.depth + 3, VisitorContext._ELEMENT_EXISTS) != null) && existSequence && (((Integer) localForCtx.getMemo(this.depth + 2, VisitorContext._XQUERYEXPRESSIONSEQUENCE_EXISTS)).intValue() <= 1))) {
							// ADD LARS
							variables.remove(i);
							i--;
							//This is a FOR(EC) or FOR(SEQ(EC)) but not FOR(SEQ(EC1,...,ECn)).
							//Rule FOR(EC) :
							substFor.put(currentVar.getVarName(), currentVar.getExpression());
							//Update substitution orders :
							substVarTmp.clear();
							substVarTmp.putAll(substFor);
							substVarTmp.putAll(substLet);
							localForCtx.setMemo(VisitorContext._ANY_DEPTH, VisitorContext._VARIABLE_SUBSTITUTION4FLWR, substVarTmp, true);
							localLetCtx.setMemo(VisitorContext._ANY_DEPTH, VisitorContext._VARIABLE_SUBSTITUTION4FLWR, substVarTmp, true);
							//Clean context :
							if (existSequence) {
								localForCtx.setMemo(this.depth + 2, VisitorContext._XQUERYEXPRESSIONSEQUENCE_EXISTS, null, true);
								localForCtx.setMemo(this.depth + 3, VisitorContext._ELEMENT_EXISTS, null, true);
							} else {
								localForCtx.setMemo(this.depth + 2, VisitorContext._ELEMENT_EXISTS, null, true);
							}
						} else if ((localForCtx.getMemo(this.depth + 2, VisitorContext._FLWREXPRESSION_GETVARCLAUSE) != null) || ((localForCtx.getMemo(this.depth + 3, VisitorContext._FLWREXPRESSION_GETVARCLAUSE) != null) && existSequence)) {
							//Rule FOR(FLWR) :
							ArrayList innerVars = (ArrayList) (existSequence ? localForCtx.getMemo(this.depth + 3, VisitorContext._FLWREXPRESSION_GETVARCLAUSE) : localForCtx.getMemo(this.depth + 2, VisitorContext._FLWREXPRESSION_GETVARCLAUSE));
							XQueryExpression innerWhere = (XQueryExpression) (existSequence ? localForCtx.getMemo(this.depth + 3, VisitorContext._FLWREXPRESSION_GETWHERECLAUSE) : localForCtx.getMemo(this.depth + 2, VisitorContext._FLWREXPRESSION_GETWHERECLAUSE));
							XQueryExpression innerReturn = (XQueryExpression) (existSequence ? localForCtx.getMemo(this.depth + 3, VisitorContext._FLWREXPRESSION_GETRETURNCLAUSE) : localForCtx.getMemo(this.depth + 2, VisitorContext._FLWREXPRESSION_GETRETURNCLAUSE));
							//Add inner var :
							// REMOVE LARS newFLWRVariables.addAll(innerVars);
							// ADD LARS
							arg.addVariables(i, innerVars);
							i += innerVars.size();
							//Choose between FOR(FLWR_1) and FOR(FLWR_2) :
							if ((arg.getQType() == null) || (innerReturn.getQType().isMultiple())) {
								//FOR(FLWR) where return of inner FLWR is multi valued (or not typed) :
								currentVar.setExpression(innerReturn, typeVisitor);
								// REMOVE LARS newFLWRVariables.add(currentVar);
								this.normalized = true;
							} else {
								// ADD LARS
								variables.remove(i);
								i--;
								//FOR(FLWR) where return of inner FLWR is mono valued :
								substFor.put(currentVar.getVarName(), innerReturn);
								//Update substitution orders :
								substVarTmp.clear();
								substVarTmp.putAll(substFor);
								substVarTmp.putAll(substLet);
								localForCtx.setMemo(VisitorContext._ANY_DEPTH, VisitorContext._VARIABLE_SUBSTITUTION4FLWR, substVarTmp, true);
								localLetCtx.setMemo(VisitorContext._ANY_DEPTH, VisitorContext._VARIABLE_SUBSTITUTION4FLWR, substVarTmp, true);
							}
							if (innerWhere != null) {
								newWhere.add(innerWhere);
							}
							//Clean context :
							if (existSequence) {
								localForCtx.setMemo(this.depth + 2, VisitorContext._XQUERYEXPRESSIONSEQUENCE_EXISTS, null, true);
								localForCtx.setMemo(this.depth + 3, VisitorContext._FLWREXPRESSION_GETVARCLAUSE, null, true);
								localForCtx.setMemo(this.depth + 3, VisitorContext._FLWREXPRESSION_GETWHERECLAUSE, null, true);
								localForCtx.setMemo(this.depth + 3, VisitorContext._FLWREXPRESSION_GETRETURNCLAUSE, null, true);
							} else {
								localForCtx.setMemo(this.depth + 2, VisitorContext._FLWREXPRESSION_GETVARCLAUSE, null, true);
								localForCtx.setMemo(this.depth + 2, VisitorContext._FLWREXPRESSION_GETWHERECLAUSE, null, true);
								localForCtx.setMemo(this.depth + 2, VisitorContext._FLWREXPRESSION_GETRETURNCLAUSE, null, true);
							}
						} else if ((localForCtx.getMemo(this.depth + 2, VisitorContext._LISTOPUNIONEXPRESSION_GETEXPRESSIONS) != null) || ((localForCtx.getMemo(this.depth + 3, VisitorContext._LISTOPUNIONEXPRESSION_GETEXPRESSIONS) != null) && existSequence)) {
							//Rule FOR(UNION) :
							ArrayList innerUnionExpr = (ArrayList) (existSequence ? localForCtx.getMemo(this.depth + 3, VisitorContext._LISTOPUNIONEXPRESSION_GETEXPRESSIONS) : localForCtx.getMemo(this.depth + 2, VisitorContext._LISTOPUNIONEXPRESSION_GETEXPRESSIONS));
							currentVar.setExpression((XQueryExpression) innerUnionExpr.get(0), typeVisitor);
							exprUnion.put(currentVar.getVarName(), innerUnionExpr.get(1));
							//Clean context :
							if (existSequence) {
								localForCtx.setMemo(this.depth + 2, VisitorContext._XQUERYEXPRESSIONSEQUENCE_EXISTS, null, true);
								localForCtx.setMemo(this.depth + 3, VisitorContext._LISTOPUNIONEXPRESSION_GETEXPRESSIONS, null, true);
							} else {
								localForCtx.setMemo(this.depth + 2, VisitorContext._LISTOPUNIONEXPRESSION_GETEXPRESSIONS, null, true);
							}
							//						} else if (localForCtx.getMemo(this.depth + 2, VisitorContext._ADDED_VARIABLE_BEFORE) != null) {
							//							Variable addVar = (Variable) localForCtx.getMemo(this.depth + 2, VisitorContext._ADDED_VARIABLE_BEFORE);
							//							Variable beforeVar = (Variable) localForCtx.getMemo(this.depth + 2, VisitorContext._ADDED_VARIABLE_BEFORE_VAR);
							//							int varPos = newFLWRVariables.indexOf(beforeVar);
							//							if (varPos != -1)
							//							newFLWRVariables.add(newFLWRVariables.indexOf(beforeVar), addVar);
							//							newFLWRVariables.add(currentVar);
							//							i++;
							//							localForCtx.setMemo(this.depth + 2, VisitorContext._ADDED_VARIABLE_BEFORE, null, true);
							//							localForCtx.setMemo(this.depth + 2, VisitorContext._ADDED_VARIABLE_BEFORE_VAR, null, true);
						} else {
							//outln("No rules detected !");
							//Default case :
							// REMOVE LARS newFLWRVariables.add(currentVar);
						}
					}
				} else if (currentVar.getBindingType() == Constants.LET_BINDINGTYPE) {
					//++++++++++++++++++++++++++++++++++++++ LET variables :
					recurseVisitor.setContext(localLetCtx);
					//Normalize sub expression:
					currentVar.accept(recurseVisitor);
					localLetCtx.update(recurseVisitor.getContext());
					//Update the normalized flag:
					this.normalized = this.normalized | recurseVisitor.wasNormalized();
					if (recurseVisitor.getExpr() == null) {
						// ADD LARS
						variables.remove(i);
						i--;
						substLet.put(currentVar.getVarName(), null);
						//To eliminate this variable.
						substVarTmp.clear();
						substVarTmp.putAll(substLet);
						substVarTmp.putAll(substFor);
						localForCtx.setMemo(VisitorContext._ANY_DEPTH, VisitorContext._VARIABLE_SUBSTITUTION4FLWR, substVarTmp, true);
						localLetCtx.setMemo(VisitorContext._ANY_DEPTH, VisitorContext._VARIABLE_SUBSTITUTION4FLWR, substVarTmp, true);
					} else { //recurseVisitor.getExpr() != null
						boolean existSequence = (localLetCtx.getMemo(this.depth + 2, VisitorContext._XQUERYEXPRESSIONSEQUENCE_EXISTS) != null);
						//Choice which rule(s) can be apply :
						if ((localLetCtx.getMemo(this.depth + 2, VisitorContext._FLWREXPRESSION_GETFLWR) != null) || ((localLetCtx.getMemo(this.depth + 3, VisitorContext._FLWREXPRESSION_GETFLWR) != null) && existSequence)) {
							//Rule LET(FLWR) :
							//outln("Rule LET(FLWR) detected !");
							//FOR   :
							/*if (substLet.size() >= 1) {
							    throw new NormalizeException("More than two \"LET $var := (FLWR)\" in one query is not supported for the moment !");
							}*/
							FLWRExpression innerFLWR = (FLWRExpression) (existSequence ? localLetCtx.getMemo(this.depth + 3, VisitorContext._FLWREXPRESSION_GETFLWR) : localLetCtx.getMemo(this.depth + 2, VisitorContext._FLWREXPRESSION_GETFLWR));
							//We keep the current let variable :
							// REMOVE LARS newFLWRVariables.add(currentVar);
							substLet.put(currentVar.getVarName(), innerFLWR);
							//Update substitution orders (different of FOR because no substitution if aggregate function) :
							localForCtx.setMemo(VisitorContext._ANY_DEPTH, VisitorContext._VARIABLE_SUBSTITUTION4LETFLWR, substLet, true);
							localLetCtx.setMemo(VisitorContext._ANY_DEPTH, VisitorContext._VARIABLE_SUBSTITUTION4LETFLWR, substLet, true);
							//Clean context :
							if (existSequence) {
								localLetCtx.setMemo(this.depth + 2, VisitorContext._XQUERYEXPRESSIONSEQUENCE_EXISTS, null, true);
								localLetCtx.setMemo(this.depth + 3, VisitorContext._FLWREXPRESSION_GETFLWR, null, true);
							} else {
								localLetCtx.setMemo(this.depth + 2, VisitorContext._FLWREXPRESSION_GETFLWR, null, true);
							}
						} else {
							//Default case :
							// REMOVE LARS newFLWRVariables.add(currentVar);
						}
					}
				} else { //Not FOR or LET variables :
					throw new NormalizeException("The variable \"" + currentVar + "\" in " + arg.toString() + " has not a valid binding type (" + currentVar.getBindingType() + ") !");
				}
			} //End for loop !
			if (eliminateThisFLWR) {
				arg = null;
			} else {
				//Update variables :
				// RMOVE LARS
				/*
				if (newFLWRVariables.size() > 0) {
					arg.setVariables(newFLWRVariables);
				} else { //newVariables.size() == 0				
				*/
				// ADD LARS
				if (variables.size() == 0) {
					//We test if there is no substitution or there is only null substitutions :
					Object[] substForValues = substFor.values().toArray();
					boolean substForValuesAllNull = true;
					for (int sfv = 0;(sfv < substForValues.length) && substForValuesAllNull; sfv++) {
						if (substForValues[sfv] != null) {
							substForValuesAllNull = false;
						}
					}
					Object[] substLetValues = substLet.values().toArray();
					boolean substLetValuesAllNull = true;
					for (int slv = 0;(slv < substLetValues.length) && substForValuesAllNull && substLetValuesAllNull; slv++) {
						if (substLetValues[slv] != null) {
							substLetValuesAllNull = false;
						}
					}
					if (substForValuesAllNull && substLetValuesAllNull) {
						//Eliminate this FLWR :
						arg = null;
					} else {
						//There is at least one not null substitution :
						allVariablesAreEliminated = true;
					}
				}
			}
			if (arg != null) {
				//--------------------------------------------- NORMALIZE WHERE CLAUSE
				//Normalize where (and apply rule(s)) :
				XQueryExpression whereClause = arg.getWhereClause();
				//outln("WHERE CLAUSE = "+whereClause+" CTX \n"+);
				if (newWhere.size() > 0) {
					if (whereClause != null) {
						//Add inner where clause :
						for (int iwc = 0; iwc < newWhere.size(); iwc++) {
							whereClause = new BinOpANDExpression(whereClause, (XQueryExpression) newWhere.get(iwc), this.typeVisitor);
							whereClause.setParenthesis(true);
						}
					} else {
						whereClause = (XQueryExpression) newWhere.get(0);
						for (int iwc = 1; iwc < newWhere.size(); iwc++) {
							whereClause = new BinOpANDExpression(whereClause, (XQueryExpression) newWhere.get(iwc), this.typeVisitor);
							whereClause.setParenthesis(true);
						}
					}
				}
				if (whereClause != null) {
					VisitorContext localWhereCtx = new VisitorContext();
					//Test if where clause is quantified expression for rule LET(FLWR) :
					//Note : where clause is not yet normalize !
					boolean stop = false;
					XQueryExpression whereContent = whereClause;
					ArrayList subExpr = null;
					//Pass sequence(s) :
					while ((whereContent instanceof XQueryExpressionSequence) && !stop) {
						subExpr = ((XQueryExpressionSequence) whereContent).getSubExpressions();
						if (subExpr.size() != 1) {
							stop = true;
						} else {
							whereContent = (XQueryExpression) subExpr.get(0);
						}

					}
					//Change where clause in quantified expression for rule LET(FLWR) :
					XQueryExpression substExpr = null;
					boolean buildWhereQuantified = false;
					Variable newQuantifiedVar = null;
					HashMap substLetClone = (HashMap) substLet.clone();
					//Search for substitution of variable by FLWR
					Object[] keys = substLetClone.keySet().toArray();
					int substPos = -1;
					for (int k = 0; k < keys.length; k++) {
						Object substitutionExpr = substLetClone.get(keys[k]);
						if ((substitutionExpr != null) && (substitutionExpr instanceof FLWRExpression)) {
							if (substPos == -1) {
								substPos = k;
							} else {
								throw new NormalizeException(0, "CAN NOT BE MORE THAN ONE SUBSTITUTION FOR RULE LET(FLWR)!");
							}
						}
					}
					//Prepare to change where clause in quantified expression :
					if (!(whereContent instanceof QuantifiedExpression) && (substPos != -1) && (substLetClone.get(keys[substPos]) != null)) {
						substExpr = (XQueryExpression) ((XQueryExpression) substLetClone.get(keys[0])).clone();
						substExpr.setParenthesis(true);
						newQuantifiedVar = createVariable(Constants.QUANTIFIER_BINDINGTYPE, substExpr);
						substLetClone.put(keys[substPos], newQuantifiedVar);
						localWhereCtx.setMemo(VisitorContext._ANY_DEPTH, VisitorContext._VARIABLE_WASSUBSTITUTELETFLWR, Boolean.FALSE, true);
						buildWhereQuantified = true;
					} else {
						//outln("===> WARNING : IT IS A QUANTIFIED !");
					}
					//Merge local context for where clause with current context (and merge inner substitution hashmaps):
					localWhereCtx.merge(this.context, false);
					//Set contexts (add substitutions to do) :
					if (!substLetClone.isEmpty()) {
						if (localWhereCtx.existsMemo(VisitorContext._ANY_DEPTH, VisitorContext._VARIABLE_SUBSTITUTION4LETFLWR)) {
							HashMap substToAdd = (HashMap) localWhereCtx.getMemo(VisitorContext._ANY_DEPTH, VisitorContext._VARIABLE_SUBSTITUTION4LETFLWR);
							substLetClone.putAll(substToAdd);
						}
						localWhereCtx.setMemo(VisitorContext._ANY_DEPTH, VisitorContext._VARIABLE_SUBSTITUTION4LETFLWR, substLetClone, true);
					}
					if (!substFor.isEmpty()) {
						if (localWhereCtx.existsMemo(VisitorContext._ANY_DEPTH, VisitorContext._VARIABLE_SUBSTITUTION4FLWR)) {
							HashMap substToAdd = (HashMap) localWhereCtx.getMemo(VisitorContext._ANY_DEPTH, VisitorContext._VARIABLE_SUBSTITUTION4FLWR);
							substFor.putAll(substToAdd);
						}
						localWhereCtx.setMemo(VisitorContext._ANY_DEPTH, VisitorContext._VARIABLE_SUBSTITUTION4FLWR, substFor, true);
					}
					//For multivalued expression (put in exists function) :
					localWhereCtx.setMemo((this.depth + 1), VisitorContext._MULTIVALUED_NEEDEXISTSFUNCTION, null, true);
					//For rule WHERE(FLWR) :
					localWhereCtx.setMemo(VisitorContext._ANY_DEPTH, VisitorContext._FLWREXPRESSION_GETANDREPLACEFLWRINWHERE, new ArrayList(), true);
					recurseVisitor.setContext(localWhereCtx);
					//Normalize sub expression:
					whereClause.accept(recurseVisitor);
					this.context.update(recurseVisitor.getContext());
					localWhereCtx.update(recurseVisitor.getContext());
					//Update the normalized flag:
					this.normalized = this.normalized | recurseVisitor.wasNormalized();
					//For multivalued expression (put in exists function) :
					localWhereCtx.removeMemo((this.depth + 1), VisitorContext._MULTIVALUED_NEEDEXISTSFUNCTION);
					//Get possible new LET variables (rule WHERE(FLWR)) :
					ArrayList newLetVariables = (ArrayList) localWhereCtx.getMemo(VisitorContext._ANY_DEPTH, VisitorContext._FLWREXPRESSION_GETANDREPLACEFLWRINWHERE);
					localWhereCtx.removeMemo(VisitorContext._ANY_DEPTH, VisitorContext._FLWREXPRESSION_GETANDREPLACEFLWRINWHERE);
					//Add this LET variables in current FLWR :
					//Note : here newLetVariables always != null :
					while (!newLetVariables.isEmpty()) {
						arg.addVariable((Variable) newLetVariables.remove(0));
					}
					//Update where clause :
					if (recurseVisitor.getExpr() == null) {
						arg = null;
					} else {
						//End of LET(FLWR) rule. Build a new quantified expression :
						if (buildWhereQuantified) {
							if (((Boolean) localWhereCtx.getMemo(VisitorContext._ANY_DEPTH, VisitorContext._VARIABLE_WASSUBSTITUTELETFLWR)).booleanValue()) {
								ArrayList quantifiedVar = new ArrayList();
								quantifiedVar.add(newQuantifiedVar);
								arg.setWhereClause(new QuantifiedExpression(Constants.SOME_QUANTIFIER, quantifiedVar, recurseVisitor.getExpr(), this.typeVisitor));
							}
							//Set the initial substLet context (before rule LET(FLWR)) :
							//substLetClone.put(keys[0], substExpr);
						} else {
							arg.setWhereClause(recurseVisitor.getExpr());
						}
					}
					//For gc :
					localWhereCtx = null;
				}
				if (arg != null) {
					//--------------------------------------------- NORMALIZE RETURN CLAUSE
					//Normalize return (and apply rule(s)) :
					XQueryExpression returnClause = arg.getReturnClause();
					VisitorContext localReturnCtx = new VisitorContext();
					//outln("===> RETURN ("+nn+") ");
					//Merge local context for where clause with current context (and merge inner substitution hashmaps):
					localReturnCtx.merge(this.context, false);
					//Set contexts (add substitutions to do) :
					if (!substLet.isEmpty()) {
						if (localReturnCtx.existsMemo(VisitorContext._ANY_DEPTH, VisitorContext._VARIABLE_SUBSTITUTION4LETFLWR)) {
							HashMap substToAdd = (HashMap) localReturnCtx.getMemo(VisitorContext._ANY_DEPTH, VisitorContext._VARIABLE_SUBSTITUTION4LETFLWR);
							substLet.putAll(substToAdd);
						}
						localReturnCtx.setMemo(VisitorContext._ANY_DEPTH, VisitorContext._VARIABLE_SUBSTITUTION4LETFLWR, substLet, true);
					}
					if (!substFor.isEmpty()) {
						if (localReturnCtx.existsMemo(VisitorContext._ANY_DEPTH, VisitorContext._VARIABLE_SUBSTITUTION4FLWR)) {
							HashMap substToAdd = (HashMap) localReturnCtx.getMemo(VisitorContext._ANY_DEPTH, VisitorContext._VARIABLE_SUBSTITUTION4FLWR);
							substFor.putAll(substToAdd);
						}
						localReturnCtx.setMemo(VisitorContext._ANY_DEPTH, VisitorContext._VARIABLE_SUBSTITUTION4FLWR, substFor, true);
					}
					//Add context rules for RETURN(FLWR) :
					localReturnCtx.setMemo(this.depth + 1, VisitorContext._FLWREXPRESSION_GETVARCLAUSE, null, true);
					localReturnCtx.setMemo(this.depth + 1, VisitorContext._FLWREXPRESSION_GETWHERECLAUSE, null, true);
					localReturnCtx.setMemo(this.depth + 1, VisitorContext._FLWREXPRESSION_GETRETURNCLAUSE, null, true);
					localReturnCtx.setMemo(this.depth + 2, VisitorContext._FLWREXPRESSION_GETVARCLAUSE, null, true);
					localReturnCtx.setMemo(this.depth + 2, VisitorContext._FLWREXPRESSION_GETWHERECLAUSE, null, true);
					localReturnCtx.setMemo(this.depth + 2, VisitorContext._FLWREXPRESSION_GETRETURNCLAUSE, null, true);
					localReturnCtx.setMemo(this.depth + 1, VisitorContext._XQUERYEXPRESSIONSEQUENCE_EXISTS, null, true);
					//Normalize sub expression:
					recurseVisitor.setContext(localReturnCtx);
					returnClause.accept(recurseVisitor);
					this.context.update(recurseVisitor.getContext());
					//Update the normalized flag:
					this.normalized = this.normalized | recurseVisitor.wasNormalized();
					if (recurseVisitor.getExpr() != null) {
						//Rule RETURN(FLWR) :
						Integer cardSeqTmp = (Integer) localReturnCtx.getMemo(this.depth + 1, VisitorContext._XQUERYEXPRESSIONSEQUENCE_EXISTS);
						int cardSequence = 0;
						if (cardSeqTmp != null) {
							cardSequence = cardSeqTmp.intValue();
						}
						if ((localReturnCtx.getMemo(this.depth + 1, VisitorContext._FLWREXPRESSION_GETVARCLAUSE) != null) || ((localReturnCtx.getMemo(this.depth + 2, VisitorContext._FLWREXPRESSION_GETVARCLAUSE) != null) && (cardSequence == 1))) {
							//Get inner FLWR clauses :
							ArrayList innerVars = (ArrayList) ((cardSequence == 1) ? localReturnCtx.getMemo(this.depth + 2, VisitorContext._FLWREXPRESSION_GETVARCLAUSE) : localReturnCtx.getMemo(this.depth + 1, VisitorContext._FLWREXPRESSION_GETVARCLAUSE));
							XQueryExpression innerWhere = (XQueryExpression) ((cardSequence == 1) ? localReturnCtx.getMemo(this.depth + 2, VisitorContext._FLWREXPRESSION_GETWHERECLAUSE) : localReturnCtx.getMemo(this.depth + 1, VisitorContext._FLWREXPRESSION_GETWHERECLAUSE));
							XQueryExpression innerReturn = (XQueryExpression) ((cardSequence == 1) ? localReturnCtx.getMemo(this.depth + 2, VisitorContext._FLWREXPRESSION_GETRETURNCLAUSE) : localReturnCtx.getMemo(this.depth + 1, VisitorContext._FLWREXPRESSION_GETRETURNCLAUSE));
							// REMOVE LARS newFLWRVariables.addAll(innerVars);
							// add LARS 02/10/2003 this is hacking
							arg.addVariables(innerVars);
							//AND of where clauses :
							arg.addWhereClause(innerWhere, typeVisitor);
							//							XQueryExpression previousWhere = arg.getWhereClause();
							//							if (innerWhere != null) {
							//								if (previousWhere != null) {
							//									innerWhere.setParenthesis(true);
							//									innerWhere = new BinOpANDExpression(arg.getWhereClause(), innerWhere, this.typeVisitor);
							//
							//								}
							//							} else {
							//								innerWhere = previousWhere;
							//								if (innerWhere != null) {
							//									innerWhere.setParenthesis(true);
							//								}
							//							}
							//Set new where and return clauses :
							//							arg.setWhereClause(innerWhere);
							arg.setReturnClause(innerReturn);
							allVariablesAreEliminated = false;
							this.normalized = true;
						} else {
							arg.setReturnClause(recurseVisitor.getExpr());
						}
						//Update return clause :
						if (allVariablesAreEliminated) {
							//Special case where all variables are eliminated (but substituted by 'not null' expressions)
							if (arg.getWhereClause() == null) {
								//We keep only the return expression :
								this.expr = recurseVisitor.getExpr();
							} else if (arg.getWhereClause() instanceof FunctionTRUE) {
								//We keep only the return expression :
								this.expr = recurseVisitor.getExpr();
							} else if (arg.getWhereClause() instanceof FunctionFALSE) {
								//This FLWR is eliminated :
								arg = null;
							} else { //whereClause != null and !(whereClause instanceof FunctionTRUE or FunctionFALSE)
								//This FLWR change to ITEExpression :
								XQueryVoid emptyExpr = new XQueryVoid(this.typeVisitor);
								emptyExpr.setParenthesis(true);
								// change LARS 15/09/2003 to be removed when ITE is supported
								this.expr = createDummyFLWR(arg.getWhereClause(), recurseVisitor.getExpr());
								//this.expr = new ITEExpression(arg.getWhereClause(), recurseVisitor.getExpr(), emptyExpr, this.typeVisitor);
							}
						} else { //There is some variables :
							if (exprUnion.size() > 0) {
								//*** UNION of two FLWR
								Object[] keys = exprUnion.keySet().toArray();
								for (int ik = 0; ik < keys.length; ik++) {
									FLWRExpression newFLWR = (FLWRExpression) arg.clone();
									//Set context :
									VisitorContext localUnionFLWRCtx = new VisitorContext();
									Variable varSubstExpr = new Variable((QName) keys[ik], this.typeVisitor);
									varSubstExpr.setExpression((XQueryExpression) exprUnion.get(keys[ik]), typeVisitor);
									localUnionFLWRCtx.setMemo(VisitorContext._ANY_DEPTH, VisitorContext._VARIABLE_CHANGEEXPRESSION4FLWR, varSubstExpr, true);
									localUnionFLWRCtx.merge(this.context, false);
									recurseVisitor.setContext(localUnionFLWRCtx);
									//Normalize new FLWR:
									newFLWR.accept(recurseVisitor);
									this.context.update(recurseVisitor.getContext());
									//Update the normalized flag:
									this.normalized = this.normalized | recurseVisitor.wasNormalized();
									this.expr = arg;
									this.expr = new ListOpUNIONExpression(this.expr, newFLWR, this.typeVisitor);
								}
							}
						}
					} else { //recurseVisitor.getExpr() == null
						arg = null;
					}
					if (this.expr == null) {
						this.expr = arg;
					}
					//For gc :
					localReturnCtx = null;
				} else {
					//Where clause was eliminated :
					this.expr = null;
				}
			} else {
				//All variables was eliminated and all variables substitutions are by null :
				this.expr = null;
			}

			//			if (arg != null) {
			//				//Update variables :
			//				if (newFLWRVariables.size() > 0) {
			//					arg.setVariables(newFLWRVariables);
			//				} else { //newVariables.size() == 0
			//					//We test if there is no substitution or there is only null substitutions :
			//					Object[] substForValues = substFor.values().toArray();
			//					boolean substForValuesAllNull = true;
			//					for (int sfv = 0;(sfv < substForValues.length) && substForValuesAllNull; sfv++) {
			//						if (substForValues[sfv] != null) {
			//							substForValuesAllNull = false;
			//						}
			//					}
			//					Object[] substLetValues = substLet.values().toArray();
			//					boolean substLetValuesAllNull = true;
			//					for (int slv = 0;(slv < substLetValues.length) && substForValuesAllNull && substLetValuesAllNull; slv++) {
			//						if (substLetValues[slv] != null) {
			//							substLetValuesAllNull = false;
			//						}
			//					}
			//					if (substForValuesAllNull && substLetValuesAllNull) {
			//						//Eliminate this FLWR :
			//						arg = null;
			//					} else {
			//						//There is at least one not null substitution :
			//						allVariablesAreEliminated = true;
			//					}
			//				}
			//			}

			//For rule WHERE(FLWR) : pop the data
			if (whereFLWRStack != null) {
				this.context.setMemo(VisitorContext._ANY_DEPTH, VisitorContext._FLWREXPRESSION_GETANDREPLACEFLWRINWHERE, whereFLWRStack, true);
			}
			//For rule WHERE(FLWR) :
			if ((this.expr != null) && (this.expr instanceof FLWRExpression) && (this.context.existsMemo(VisitorContext._ANY_DEPTH, VisitorContext._FLWREXPRESSION_GETANDREPLACEFLWRINWHERE))) {
				//Replace  this FLWR by a new LET variable :
				ArrayList letVariables = (ArrayList) this.context.getMemo(VisitorContext._ANY_DEPTH, VisitorContext._FLWREXPRESSION_GETANDREPLACEFLWRINWHERE);
				//Create the new variable :
				Variable newLetVariable = createVariable(Constants.LET_BINDINGTYPE, this.expr);
				//Add in context :
				letVariables.add(newLetVariable);
				this.context.setMemo(VisitorContext._ANY_DEPTH, VisitorContext._FLWREXPRESSION_GETANDREPLACEFLWRINWHERE, letVariables, true);
				//Change current expression :
				this.expr = newLetVariable;
				this.normalized = true;
			}
			//For multivalued expression (put in exists function) :
			if ((this.expr != null) && (localCtx.existsMemo(this.depth, VisitorContext._MULTIVALUED_NEEDEXISTSFUNCTION))) {
				this.normalized = true;
				boolean brace = this.expr.getBrace();
				boolean bracket = this.expr.getBracket();
				boolean parenthesis = this.expr.getParenthesis();
				this.expr.setBrace(false);
				this.expr.setBracket(false);
				this.expr.setParenthesis(false);
				ArrayList parameter = new ArrayList();
				parameter.add(this.expr);
				this.expr = new FunctionEXISTS(parameter, this.typeVisitor);
				this.expr.setBrace(brace);
				this.expr.setBracket(bracket);
				this.expr.setParenthesis(parenthesis);
			}
			//Re-type this expression :
			if (this.normalized && (this.expr != null) && (this.typeVisitor != null)) {
				this.expr.accept(this.typeVisitor);
			}
			//--------------------------------------------- ANSWER QUESTION(S) OF CURRENT CONTEXT
			if ((this.expr != null) && (this.expr instanceof FLWRExpression)) {
				//Answer questions of the current context (we must use the normalized FLWRExpression to answer) :
				FLWRExpression thisFLWR = (FLWRExpression) this.expr;
				//For FWLRExpression :
				if ((this.context.existsMemo(this.depth, VisitorContext._FLWREXPRESSION_GETVARCLAUSE)) && (this.context.existsMemo(this.depth, VisitorContext._FLWREXPRESSION_GETWHERECLAUSE)) && (this.context.existsMemo(this.depth, VisitorContext._FLWREXPRESSION_GETRETURNCLAUSE))) {
					//Answers:
					this.context.setMemo(this.depth, VisitorContext._FLWREXPRESSION_GETVARCLAUSE, thisFLWR.getVariables(), true);
					this.context.setMemo(this.depth, VisitorContext._FLWREXPRESSION_GETWHERECLAUSE, thisFLWR.getWhereClause(), true);
					this.context.setMemo(this.depth, VisitorContext._FLWREXPRESSION_GETRETURNCLAUSE, thisFLWR.getReturnClause(), true);
				}
				//For FLWRExpression :
				if (this.context.existsMemo(this.depth, VisitorContext._FLWREXPRESSION_GETFLWR)) {
					//Answers:
					this.context.setMemo(this.depth, VisitorContext._FLWREXPRESSION_GETFLWR, thisFLWR, true);
				}
				//For QuantifiedExpression :
				if ((thisFLWR.getVariables().size() == 1) && (((Variable) thisFLWR.getVariables().get(0)).getBindingType() == Constants.FOR_BINDINGTYPE)) {
					if ((this.context.existsMemo(this.depth, VisitorContext._FLWREXPRESSION_GETVARCLAUSE4QUANTIFIED)) && (this.context.existsMemo(this.depth, VisitorContext._FLWREXPRESSION_GETWHERECLAUSE4QUANTIFIED)) && (this.context.existsMemo(this.depth, VisitorContext._FLWREXPRESSION_GETRETURNCLAUSE4QUANTIFIED))) {
						//Answers:
						this.context.setMemo(this.depth, VisitorContext._FLWREXPRESSION_GETVARCLAUSE4QUANTIFIED, thisFLWR.getVariables().get(0), true);
						this.context.setMemo(this.depth, VisitorContext._FLWREXPRESSION_GETWHERECLAUSE4QUANTIFIED, thisFLWR.getWhereClause(), true);
						this.context.setMemo(this.depth, VisitorContext._FLWREXPRESSION_GETRETURNCLAUSE4QUANTIFIED, thisFLWR.getReturnClause(), true);
					}
				}
			}
			//For gc :
			localCtx = null;
			localForCtx = null;
			localLetCtx = null;
			//TOREMOVE ?
			//Update response context with current context to pass the answers of parent questions:
			//this.context.update(localCtx);
			//outln("=====> New FLWR ("+nn+") = ("+this.normalized+"):\n\t>>>"+this.expr);
			//untrace();
		} catch (Exception e) {
			if (e instanceof NormalizeException)
				throw (NormalizeException) e;
			else
				throw new NormalizeException(0, "Unexpected exception in FLWRExpression :\n==>" + e.toString() + "<==\n", e);
		}
	}

	/**
	 * This visit method normalizes an InstanceOfExpression.<BR>
	 * It possibly answers the questions of the current context.
	 * @param arg a InstanceOfExpression to normalize.
	 */
	//	public void visit(InstanceOfExpression arg) throws NormalizeException {
	//		try {
	//			//trace("<o-"+(nodeNumber++)+">InstanceOfExpression<o>(hc="+arg.hashCode()+") : "+arg.toString()+" QType="+this.typeToString(arg.getQType()),0);
	//			throw new NormalizeException(0, "InstanceOfExpression not supported : (" + arg + ")");
	//			ExpressionNormalizationVisitor recurseVisitor = new ExpressionNormalizationVisitor(this.depth, this.typeVisitor);
	//			VisitorContext localCtx = new VisitorContext();
	//			localCtx.merge(this.context, false);
	//			//Clean all questions which depth is smaller than current depth:
	//			localCtx.filter(this.depth);
	//			recurseVisitor.setContext(localCtx);
	//			//Normalize sub expression:
	//			XQueryExpression expr = arg.getExpression();
	//			if (expr != null) {
	//			    expr.accept(recurseVisitor);
	//			    localCtx.update(recurseVisitor.getContext());
	//			    expr = recurseVisitor.getExpr();
	//			    //Update the normalized flag:
	//			    this.normalized = recurseVisitor.wasNormalized();
	//			}
	//			//Test if sub expression was normalized:
	//			if (this.normalized) {
	//			    //Update current InstanceOfExpression:
	//			    arg.setExpression(expr);
	//			    //Re-type this expression :
	//			    if (this.typeVisitor != null) {
	//			        arg.accept(this.typeVisitor);
	//			    }
	//			}
	//			this.expr = arg;
	//			//Update response context with current context to pass the answers of parent questions:
	//			this.context.update(localCtx);
	//			//untrace();
	//		} catch (Exception e) {
	//			if (e instanceof NormalizeException)
	//				throw (NormalizeException) e;
	//			else
	//				throw new NormalizeException(0, "Unexpected exception in InstanceOfExpression :\n==>" + e.toString() + "<==\n", e);
	//		}
	//	}

	/**
	 * This visit method normalizes a ITEExpression (If C then E1 else E2) and its sub expressions (C,E1 and E2).<BR>
	 * This method apply theses rules:<BR>
	 * <OL>
	 * <LI>N(if C then E1 else E2) => if N(C) then N(E1) else N(E2)</LI>
	 * <LI>N(if null|false() then E1 else E2) => N(E2)</LI>
	 * <LI>N(if true() then E1 else E2) => N(E1)</LI>
	 * </OL>
	 * It possibly answers the questions of the current context.
	 * @param arg a ITEExpression to normalize.
	 */
	public void visit(ITEExpression arg) throws NormalizeException {
		try {
			//trace("<o-"+(nodeNumber++)+">ITEExpression<o>(hc="+arg.hashCode()+") : "+arg.toString()+" QType="+this.typeToString(arg.getQType()),0);
			ExpressionNormalizationVisitor recurseVisitor = new ExpressionNormalizationVisitor(this.depth, this.typeVisitor);
			VisitorContext localCtx = new VisitorContext();
			localCtx.merge(this.context, false);
			//Clean all questions which depth is smaller than current depth:
			localCtx.filter(this.depth);
			recurseVisitor.setContext(localCtx);
			//For multivalued expression (put in exists function) :
			localCtx.setMemo((this.depth + 1), VisitorContext._MULTIVALUED_NEEDEXISTSFUNCTION, null, true);
			//Normalize condition :
			XQueryExpression ifClause = arg.getIfExpression();
			ifClause.accept(recurseVisitor);
			localCtx.update(recurseVisitor.getContext());
			ifClause = recurseVisitor.getExpr();
			//Update the normalized flag:
			this.normalized = recurseVisitor.wasNormalized();
			//For multivalued expression (put in exists function) :
			localCtx.removeMemo((this.depth + 1), VisitorContext._MULTIVALUED_NEEDEXISTSFUNCTION);
			byte ifValue = 0; //0 = undefined, 1 = true and 2 = false.
			if (this.normalized) {
				if (ifClause != null) {
					arg.setIfExpression(ifClause);
				} else {
					ifValue = 2;
				}
			}
			//Determine if the condition value is true, false or unknown :
			if (ifClause instanceof FunctionTRUE) {
				ifValue = 1;
			} else if (ifClause instanceof FunctionFALSE) {
				ifValue = 2;
			} else if (ifClause instanceof XQueryExpressionSequence) {
				ArrayList subExpr = ((XQueryExpressionSequence) ifClause).getSubExpressions();
				if (subExpr.size() == 1) {
					if (subExpr.get(0) instanceof FunctionTRUE) {
						ifValue = 1;
					} else if (subExpr.get(0) instanceof FunctionFALSE) {
						ifValue = 2;
					}
				}
			}
			XQueryExpression thenClause = arg.getThenExpression();
			XQueryExpression elseClause = arg.getElseExpression();
			if (ifValue <= 1) {
				//Normalize then clause :
				if (!(thenClause instanceof XQueryVoid)) {
					thenClause.accept(recurseVisitor);
					localCtx.update(recurseVisitor.getContext());
					thenClause = recurseVisitor.getExpr();
					//Update the normalized flag:
					this.normalized = this.normalized | recurseVisitor.wasNormalized();
				}
			}
			if ((ifValue == 0) || (ifValue == 2)) {
				//Normalize else clause :
				if (!(elseClause instanceof XQueryVoid)) {
					elseClause.accept(recurseVisitor);
					localCtx.update(recurseVisitor.getContext());
					elseClause = recurseVisitor.getExpr();
					//Update the normalized flag:
					this.normalized = this.normalized | recurseVisitor.wasNormalized();
				}
			}
			//Update result :
			if (this.normalized || (ifValue >= 1)) {
				if (ifValue == 1) {
					this.expr = thenClause;
				} else if (ifValue == 2) {
					this.expr = elseClause;
				} else {
					arg.setIfExpression(ifClause);
					arg.setThenExpression(thenClause);
					arg.setElseExpression(elseClause);
					this.expr = arg;
				}
				this.normalized = true;
			} else {
				this.expr = arg;
			}
			//Re-type this expression :
			if (this.normalized && (this.expr != null) && (this.typeVisitor != null)) {
				this.expr.accept(this.typeVisitor);
			}
			//Update response context with current context to pass the answers of parent questions:
			this.context.update(localCtx);
			//untrace();
		} catch (Exception e) {
			if (e instanceof NormalizeException)
				throw (NormalizeException) e;
			else
				throw new NormalizeException(0, "Unexpected exception in ITEExpression :\n==>" + e.toString() + "<==\n", e);
		}
	}

	/**
	 * This visit method normalizes an ItemType.<BR>
	 * It possibly answers the questions of the current context.
	 * @param arg a ItemType to normalize.
	 */
	//	public void visit(ItemType arg) throws NormalizeException {
	//		try {
	//			//trace("<o-"+(nodeNumber++)+">ItemType<o>(hc="+arg.hashCode()+") : "+arg.toString()+" QType="+this.typeToString(arg.getQType()),0);
	//			throw new NormalizeException(0, "ItemType not supported : (" + arg + ")");
	//			/*this.expr = arg;
	//			//Re-type this expression :
	//			if (this.typeVisitor != null) {
	//			    this.expr.accept(this.typeVisitor);
	//			}
	//			//untrace();*/
	//		} catch (Exception e) {
	//			if (e instanceof NormalizeException)
	//				throw (NormalizeException) e;
	//			else
	//				throw new NormalizeException(0, "Unexpected exception in ItemType :\n==>" + e.toString() + "<==\n", e);
	//		}
	//	}

	/**
	 * To analyse a LocatedExpression (and its subnodes) to determine if a rules can be applied.<BR>
	 * This method apply theses rules:<BR>
	 * <OL>
	 * <LI>N(step1/.../stepi[predi]/.../stepN) => N(step1)/.../N(step2)[N(predi)]/.../N(stepN)</LI>
	 * <LI>N(E) = N(S1/.../Si[Pi]..[Pi+m]/Si+1/.../Sn)<BR>
	 * =><BR>
	 * for $x1 in S1/.../Si<BR>
	 * for $x2 in $x1/Si+1/.../Sn<BR>
	 * where N([Pi]) and ... N([Pi+m])<BR>
	 * return $x2<BR>
	 * WHERE :<BR>
	 * If N(Pi) = QName :<BR>
	 *     => N([Pi]) = $x/Pi<BR>
	 * If N(Pi) = true() :<BR>
	 *     => N([Pi]) = null<BR>
	 * If N(Pi) = false() | null :<BR>
	 *     => N(E) = null<BR>
	 * If N(Pi) = relative LocatedExpression :<BR>
	 *     => N([Pi]) = $x/Pi<BR>
	 * If N(Pi) = absolute LocatedExpression :<BR>
	 *     => N([Pi]) = Pi<BR>
	 * If N(Pi) = FLWRExpression :<BR>
	 *     => N([Pi]) = Pi<BR>
	 * If N(Pi) = Variable :<BR>
	 *     => N([Pi]) = Pi</LI><BR>
	 * <LI>Apply XPath on all types of expressions...see org.xquark.xml.xqueryparser.normalize.LocatedExpressionReducer
	 * class (source and javadoc) for more informations.</LI>
	 * <LI>N(C(E)) C = condition and E = LE || FLWR || VAR => N(C(EXISTS(E)))</LI>
	 * </OL>
	 * It possibly answers the questions of the current context.
	 * @param arg a LocatedExpression to normalize.
	 * @see org.xquark.xml.xqueryparser.normalize.LocatedExpressionReducer
	 */

	private LocatedExpressionReducer ler = new LocatedExpressionReducer();

	public void visit(LocatedExpression arg) throws NormalizeException {
		try {
			//Steps can not be null and steps.size() must be greater or egals 1
			ArrayList steps = arg.getSteps();
			if ((steps == null) || (steps.size() < 1))
				throw new NormalizeException(0, "This located expression form \"" + arg + "\" is not supported !");
			if (((Step) steps.get(0)).hasSeparator()) {
				throw new NormalizeException(0, "This located expression form \"" + arg + "\" is not supported : first step with separator !");
			}
			// ADD LARS to avoid having a located expression with single step and no predicates
			if (!arg.getInPredicate() && steps.size() == 1 && !arg.hasPredicates() && (arg.getParentExpression() == null || arg.getParentExpression().isEmpty() || !(arg.getParentExpression().get(0) instanceof ListOpUNIONExpression))) {
				this.normalized = true;
				this.expr = ((Step) steps.get(0)).getExpression();
				return;
			}
			//Prepare Normalization :
			ExpressionNormalizationVisitor recurseVisitor = new ExpressionNormalizationVisitor(this.depth, this.typeVisitor);
			VisitorContext localCtx = new VisitorContext();
			localCtx.merge(this.context, false);
			//Clean all questions which depth is smaller than current depth:
			localCtx.filter(this.depth);
			recurseVisitor.setContext(localCtx);
			//Save the braces, brackets and parenthesis :
			boolean braces = arg.getBrace();
			boolean brackets = arg.getBracket();
			boolean parenthesis = arg.getParenthesis();
			//Set local context :
			// comment LARS 25/09.2003
			//localCtx.setMemo(this.depth + 1, VisitorContext._STEP_ISPARENTAXISSTEP, null, true);
			//Variables declarations :
			boolean eliminateThisLE = false;
			//comment LARS 25/09.2003
			//boolean doNotReduce = false;
			//For rule LE(PRED) => FLWR(LE) :
			//Push the possible Step context for LE (if this located expression is in a step predicate) :
			Variable newVar4LE_Predicate = null;
			if (localCtx.existsMemo(VisitorContext._ANY_DEPTH, VisitorContext._LOCATEDEXPRESSION_INPREDICATE)) {
				newVar4LE_Predicate = (Variable) localCtx.getMemo(VisitorContext._ANY_DEPTH, VisitorContext._LOCATEDEXPRESSION_INPREDICATE);
				localCtx.removeMemo(VisitorContext._ANY_DEPTH, VisitorContext._LOCATEDEXPRESSION_INPREDICATE);
				// ADD   12/05/03
				if (newVar4LE_Predicate != null && !arg.getStep(0).hasSeparator() && arg.getStep(0).getAxis() != Axis.NONE) {
					// erase fisrt step if self::node
					int stepindex = 0;
					if (arg.getExpression() instanceof NodeType && ((NodeType) arg.getExpression()).getKind() == NodeKind.NODE && arg.getStep(0).getAxis() == Axis.SELF) {
						steps.remove(0);
					} else {
						// put child axis to first step
						arg.getStepNum(0).setHasSeparator(true);
					}
					// add step with variable
					steps.add(0, new Step(false, Axis.NONE, newVar4LE_Predicate, null, null));
					arg.setContext(false);
				}
				// END ADD   12/05/03
			}
			ArrayList newLEVariables = new ArrayList();
			XQueryExpression newWhereExpr = null;
			ArrayList expression4VariableBuffer = new ArrayList();
			boolean hadPredicates = false;
			//Normalize all the steps :
			for (int i = 0;(i < steps.size()) && (!eliminateThisLE); i++) {
				//For rule LE(PRED) => FLWR(LE) :
				//Pass to step class the previous steps expressions for variable expression :
				// ICI ICI
				localCtx.setMemo(this.depth + 1, VisitorContext._STEP_GETPREDICATE, expression4VariableBuffer, true);
				//Test if current step has predicates :
				// CHANGE   12/05/03
				hadPredicates = ((Step) steps.get(i)).hasPredicates();
				//Normalize one step :
				recurseVisitor.setContext(localCtx);
				((Step) steps.get(i)).accept(recurseVisitor);
				localCtx.update(recurseVisitor.getContext());
				if (recurseVisitor.getExpr() != null) {
					steps.set(i, recurseVisitor.getExpr());
					//Test context :
					// comment LARS 25/09.2003
					//					if (localCtx.getMemo(this.depth + 1, VisitorContext._STEP_ISPARENTAXISSTEP) != null) {
					//						doNotReduce = true;
					//						localCtx.setMemo(this.depth + 1, VisitorContext._STEP_ISPARENTAXISSTEP, null, true);
					//					}
					//For rule LE(PRED) => FLWR(LE) :
					//Test context :
					if (hadPredicates) {
						if (localCtx.getMemo(this.depth + 1, VisitorContext._STEP_GETPREDICATE) == null) {
							//There is no more predicates for the current step!
							//The current step without its predicates was already added in
							//expression4VariableBuffer by Step
						} else { //Context _STEP_GETPREDICATE != null
							ArrayList variableAndPredicate = (ArrayList) localCtx.getMemo(this.depth + 1, VisitorContext._STEP_GETPREDICATE);
							//Get the new variable :
							Variable newVar = (Variable) variableAndPredicate.get(0);
							newLEVariables.add(newVar);
							//Get the new predicate for new where clause :
							XQueryExpression newPredicate = (XQueryExpression) variableAndPredicate.get(1);
							//Create/update new where clause :
							if (newWhereExpr == null) {
								newWhereExpr = newPredicate;
							} else {
								newWhereExpr = new BinOpANDExpression(newWhereExpr, newPredicate, this.typeVisitor);
							}
							//Re-initialization :
							localCtx.setMemo(this.depth + 1, VisitorContext._STEP_GETPREDICATE, null, true);
							expression4VariableBuffer = new ArrayList();
							expression4VariableBuffer.add(newVar);
							//The previous variable for the possible next variable.
						}
					} else {
						//Add the current Step in buffer :
						expression4VariableBuffer.add(recurseVisitor.getExpr());
					}

				} else {
					eliminateThisLE = true;
				}
				//Update the normalized flag:
				this.normalized = this.normalized | recurseVisitor.wasNormalized();
			} //End for each steps
			//Remove memo :
			localCtx.removeMemo(this.depth + 1, VisitorContext._STEP_ISPARENTAXISSTEP);
			localCtx.removeMemo(this.depth + 1, VisitorContext._STEP_GETPREDICATE);

			// add LARS 25/09/2003 test elimination !!!
			if (eliminateThisLE) {
				this.expr = null;
				return;
			}
			// end add LARS 25/09/2003 test elimination

			//Three case (by priority) :
			// 1- rule LE(PRED) => FLWR(LE)
			// 2- do not reduce because there is parent axis
			// 3- do steps reduction and apply step on expression
			if (newLEVariables.size() > 0) { //Case 1- rule LE(PRED) => FLWR(LE)
				//Create new FLWRExpression
				//Create last FOR variable with the last steps (if exist) :
				XQueryExpression newReturnExpr = null;
				if (expression4VariableBuffer.get(0) instanceof Variable) {
					Variable headerVariable = (Variable) expression4VariableBuffer.get(0);
					if (expression4VariableBuffer.size() > 1) {
						//Prepare to create a new located expression for last variable :
						Step headerStep = new Step(false, Axis.NONE, headerVariable, null, null);
						expression4VariableBuffer.set(0, headerStep);
						((Step) expression4VariableBuffer.get(1)).setHasSeparator(true);
						//Create the last variable :
						Variable lastVar = createVariable(Constants.FOR_BINDINGTYPE, new LocatedExpression(expression4VariableBuffer, this.typeVisitor));
						newLEVariables.add(lastVar);
						//Create the return clause :
						newReturnExpr = lastVar;
					} else { //Size <= 1, but in fact : size is >= 1 so size == 1
						newReturnExpr = headerVariable;
					}
				} else {
					//Create the last variable :
					Variable lastVar = createVariable(Constants.FOR_BINDINGTYPE, new LocatedExpression(expression4VariableBuffer, typeVisitor));
					newLEVariables.add(lastVar);
					//Create the return clause :
					newReturnExpr = lastVar;
				}
				//Create a FLWR expression instead of the located expression :
				this.normalized = true;
				this.expr = new FLWRExpression(newLEVariables, null, newWhereExpr, newReturnExpr, this.typeVisitor);
				// add   23/06/2003
				this.expr.setParentUnit(arg.getParentUnit());
			} else { //Case 2 or 3 :
				//				if (doNotReduce) {
				//					doNotReduce = true;
				//					//Case 2- do not reduce because there is parent axis
				//					//					//Get the first step :
				//					//					Step first = ((Step) steps.get(0));
				//					//					steps.remove(0);
				//					//					//Reduce only the steps 2 by 2 (but not the entire located expression because we do not know how
				//					//					//reduce LE with parent axis) :
				//					//					LocatedExpressionReducer ler = new LocatedExpressionReducer(first, steps, this.typeVisitor);
				//					//					if (ler.simplifySteps()) {
				//					//						//Eliminate this located expression :
				//					//						this.expr = null;
				//					//					}
				//					//					//Update the steps :
				//					//					steps = ler.getReducedSteps();
				//					//					steps.add(0, first);
				//					//					arg.setSteps(steps);
				//					//					this.expr = arg;
				//					//					this.expr.setBrace(braces);
				//					//					this.expr.setBracket(brackets);
				//					//					this.expr.setParenthesis(parenthesis);
				//					//					//Re-type this expression :
				//					//					if (this.typeVisitor != null) {
				//					//						this.expr.accept(this.typeVisitor);
				//					//					}
				//					//-------------------------------
				//					// change   22/08/2003
				//					//Get the first step :
				//					//Reduce only the steps 2 by 2 (but not the entire located expression because we do not know how
				//					//reduce LE with parent axis) :
				//
				//					//ler.reset(arg,this.typeVisitor);
				//					LocatedExpressionReducer.simplifySteps(arg.getSteps(), true, (arg.getExpression() instanceof Variable));
				//					//Update the steps :
				//					//arg.setSteps(ler.getReducedSteps());
				//					this.expr = arg;
				//					//Re-type this expression :
				//					if (this.typeVisitor != null) {
				//						this.expr.accept(this.typeVisitor);
				//					}
				//				} else { //Case 3- do steps reduction and apply step on expression
				//Get the first step :
				//Step first = ((Step) steps.get(0));
				//Analyse the first normalize step :
				// comment LARS 26/09/2003
				/*
				if (eliminateThisLE) {
					//Eliminate this located expression :
					this.expr = null;
				} else {
				*/
				//						if ((first.getAxis() != Axis.NONE) || (first.getPredicates() != null)) {
				//							//First step has no separator but has an axis or predicates :
				//							if (this.normalized) {
				//								arg.setSteps(steps);
				//								arg.setBrace(braces);
				//								arg.setBracket(brackets);
				//								arg.setParenthesis(parenthesis);
				//								//Re-type this expression :
				//								if (this.typeVisitor != null) {
				//									arg.accept(this.typeVisitor);
				//								}
				//							}
				//							this.expr = arg;
				//						} else {
				//							//Gets first step :
				//							XQueryExpression firstExpr = first.getExpression();
				//							steps.remove(0);
				//							//Reduce this located expression :
				//							LocatedExpressionReducer ler = new LocatedExpressionReducer(firstExpr, steps, this.typeVisitor);
				//							if (ler.reduceLocatedExpression()) {
				//								this.expr = ler.getReducedExpression();
				//								this.normalized = true;
				//							} else {
				//								Step newExprStep = new Step(false, Axis.NONE, firstExpr, null, null);
				//								arg.getSteps().add(0, newExprStep);
				//								this.expr = arg;
				//							}
				//							//Set braces, brackets and parenthesis, and re-type :
				//							if (this.expr != null) {
				//								this.expr.setBrace(braces);
				//								this.expr.setBracket(brackets);
				//								this.expr.setParenthesis(parenthesis);
				//								//Re-type this expression :
				//								if (this.typeVisitor != null) {
				//									this.expr.accept(this.typeVisitor);
				//								}
				//							}
				//------------------------------------
				// change   22/08/2003
				//Reduce this located expression :
				ler.reset(arg, this.typeVisitor);
				if (ler.reduceLocatedExpression()) {
					this.expr = ler.getReducedExpression();
					this.normalized = true;
					if (this.expr != null) {
						this.expr.setBrace(braces);
						this.expr.setBracket(brackets);
						this.expr.setParenthesis(parenthesis);
						//this.expr.accept(this.typeVisitor);
					}
				} else {
					this.expr = arg;
				}
				//						}
				//					}
				//				}
				//Answer/execute questions/orders found in local context :
				if ((newVar4LE_Predicate != null) && (this.expr instanceof LocatedExpression)) {
					//For rule LE(PRED) => FLWR(LE) :
					//Pop the Step context for LE :
					localCtx.setMemo(VisitorContext._ANY_DEPTH, VisitorContext._LOCATEDEXPRESSION_INPREDICATE, newVar4LE_Predicate, true);
					//Test if this located expression begins with relative step :
					ArrayList newSteps = ((LocatedExpression) this.expr).getSteps();
					//Any located expression has at least one step :
					Step firstStep = (Step) newSteps.get(0);
					if (!firstStep.hasSeparator() && (firstStep.getAxis() != Axis.NONE)) {
						//outln(nn+" firstStep "+firstStep+" "+firstStep.getAxis()+" "+firstStep.getSeparator());
						//This is a located which begins with a relative step as '.', '..', child::A, ...etc :
						//Add the given variable at the beginning :
						//Modifiy this located expression (add the variable at the beginning) :
						this.normalized = true;
						Step newFirstStep = new Step(false, Axis.NONE, newVar4LE_Predicate, null, null);
						firstStep.setHasSeparator(true);
						//To indicate that this LocatedExpression is no more contextual :
						this.expr.setContext(false);
						//Particular case : if first step is '.', eliminate it :
						boolean isSelfNode = false;
						if ((newSteps.size() >= 2) && (firstStep != null) && (firstStep.getPredicates() == null) && (firstStep.getAxis() == Axis.SELF)) {
							XQueryExpression test = firstStep.getExpression();
							if ((test != null) && (test instanceof NodeType) && (((NodeType) test).getKind() == NodeKind.NODE)) {
								isSelfNode = true;
							}
						}
						if (isSelfNode) {
							newSteps.set(0, newFirstStep);
						} else {
							newSteps.add(0, newFirstStep);
						}
					}
					//Re-type this expression :
					if (this.typeVisitor != null) {
						this.expr.accept(this.typeVisitor);
					}
				}
				// add LARS 26/09/2003

				else {
					// this part of the code nomalize the following pattern
					// there are no more predicates
					/*
					$x -> xp1
					$y -> $x/xp2/parent::node()/xp3
					It is assumed that xp2 is either empty or ending with descendant-or-self::node()
					case 1 : xp2 is empty and xp3 is empty the expression becomes
					$y -> xp1/parent::node()
					$x -> $y/child::(expression of last step of xp1)
					case 2 : xp2 is empty and xp3 is not empty the expression becomes
					$newVar -> xp1/parent::node()
					$x -> $newVar/child::(expression of last step of xp1)
					$y -> $newVar/xp3
					case 3 : xp2 is not empty and xp3 is empty the expression becomes					
					case 4 : xp2 is not empty and xp3 is not empty the expression becomes
					$x -> xp1
					$y -> $x/xp2/parent::node() union $x/xp2/descendant-or-self::node()
					$newVar -> $y/xp3
					where exists($y/child::node())
					
					
					
					$newVar -> xp1/parent::node()
					$x -> $newVar/child::(expression of last step of xp1)
					$y -> $newVar/xp3 union $x/descendant-or-self::node()
					where exists $y[child::node()]
					*/
					int parentIndex = arg.hasParentStep();
					if (parentIndex != -1 && arg.startsWithVariable()) {
						XQueryExpression argParent = (XQueryExpression) arg.getParentExpression().get(0);
						// parent step must be just after variable
						if (parentIndex != 1 && arg.getStepNum(parentIndex - 1).getAxis() != Axis.DESCENDANT_OR_SELF)
							throw new NormalizeException("Could not normalize located expression with parent::node : " + arg);
						// get variable starting located expression
						Variable var = (Variable) arg.getExpression();
						// verify that expression is located ending with child or descendant-or-self
						if (!(var.getExpression() instanceof LocatedExpression))
							throw new NormalizeException("Could not normalize located expression with parent::node : " + arg);
						// get parent FLWR
						FLWRExpression parentflwr = var.getParentFLWRExpression();
						// verify that variables are in the same FLWR otherwise create temporary variable
						if (!(argParent instanceof Variable) || parentflwr != ((Variable) argParent).getParentFLWRExpression()) {
							// add variable in parentflwr newVar -> arg
							// substitute arg with newVar
							Variable newVar = createVariable(Constants.LET_BINDINGTYPE, arg);
							HashMap substituteMap = new HashMap();
							substituteMap.put(arg, newVar);
							parentflwr.substitute(substituteMap, typeVisitor);
							parentflwr.addVariable(parentflwr.getVarPosition(var) + 1, newVar);
							this.normalized = true;
							this.expr = newVar;
						} else {
							// get starting variable
							Variable varParent = (Variable) argParent;
							// get expression of variable
							LocatedExpression varLoc = (LocatedExpression) var.getExpression();
							// verify last step
							ArrayList varLocSteps = varLoc.getSteps();
							Step lastStep = varLoc.getStepNum(varLocSteps.size() - 1);
							if (lastStep.getAxis() != Axis.CHILD && lastStep.getAxis() != Axis.DESCENDANT_OR_SELF)
								throw new NormalizeException("Could not normalize located expression with parent::node : " + arg);
							int varPos = parentflwr.getVarPosition(var);
							// do the stuff!!!
							if (parentIndex == 1) {
								// CASE 1
								if (parentIndex == steps.size() - 1) {
									// reset variable positions
									parentflwr.getVariables().remove(varPos);
									parentflwr.addVariable(parentflwr.getVarPosition(varParent) + 1, var);
									// reset arg steps
									arg.getSteps().remove(0);
									arg.getSteps().addAll(0, varLocSteps);
									this.expr = arg;
									// simplify var
									int retValue = LocatedExpressionReducer.simplifySteps(arg.getSteps(), false, typeVisitor);
									if (retValue == 0)
										throw new NormalizeException("Could not normalize located expression with parent::node : " + arg);
									// reset varLoc steps
									ArrayList newSteps = new ArrayList(2);
									newSteps.add(new Step(false, Axis.NONE, varParent, null, null));
									newSteps.add(new Step(true, Axis.CHILD, lastStep.getExpression(), null, null));
									varLoc.setSteps(newSteps);
								} else { // CASE 2
									// create variable newVar -> xp1/PN
									List xp3 = steps.subList(parentIndex + 1, steps.size());
									List xp2pn = steps.subList(1, parentIndex + 1);
									arg.setSteps((ArrayList) varLocSteps.clone());
									arg.getSteps().addAll(xp2pn);
									// simplify new variable
									int retValue = LocatedExpressionReducer.simplifySteps(arg.getSteps(), false, typeVisitor);
									if (retValue == 0)
										throw new NormalizeException("Could not normalize located expression with parent::node : " + arg);
									Variable newVar = createVariable(var.getBindingType(), arg);
									parentflwr.addVariable(varPos, newVar);
									// localCtx.setMemo(this.depth, VisitorContext._ADDED_VARIABLE_BEFORE, newVar, true);
									// localCtx.setMemo(this.depth, VisitorContext._ADDED_VARIABLE_BEFORE_VAR, var, true);
									// modify expression of var -> newVar/DCN
									Step newVarStep = new Step(false, Axis.NONE, newVar, null, null);
									ArrayList newSteps = new ArrayList(2);
									newSteps.add(newVarStep);
									newSteps.add(new Step(true, Axis.CHILD, lastStep.getExpression(), null, null));
									varLoc.setSteps(newSteps);
									// modify expression of arg = newVar/xp3
									newSteps = new ArrayList();
									newSteps.add(newVarStep.clone());
									newSteps.addAll(xp3);
									//varParent.setExpression(new LocatedExpression(newSteps,typeVisitor),typeVisitor);
									this.expr = new LocatedExpression(newSteps, typeVisitor);
								}
							} else {
								/*
								$x -> xp1
								$y -> $x/xp2/descendant-or-self::node()/parent::node()/xp3
								-------------------------------------------------
								$x -> xp1
								$newVar -> $x/xp2/parent::node() union $x/xp2/descendant-or-self::node()
								$y -> $newVar/xp3
								where exists($newVar/child::node())
								*/
								if (parentIndex == steps.size() - 1) {
									// set varParent expression
									ArrayList newSteps = new ArrayList();
									for (int i = 0; i < parentIndex - 1; i++)
										newSteps.add(((Step) steps.get(i)).clone());
									newSteps.add(steps.get(parentIndex));
									LocatedExpression locParent = new LocatedExpression(newSteps, false, typeVisitor);
									newSteps = new ArrayList();
									for (int i = 0; i < parentIndex - 1; i++)
										newSteps.add(steps.get(i));
									newSteps.add(steps.get(parentIndex - 1));
									LocatedExpression locDosNode = new LocatedExpression(newSteps, false, typeVisitor);
									this.expr = new ListOpUNIONExpression(locParent, locDosNode, typeVisitor);
									// add where clause
									newSteps = new ArrayList(2);
									newSteps.add(new Step(false, Axis.NONE, varParent, null, null));
									newSteps.add(new Step(true, Axis.CHILD, new NodeType(NodeKind.NODE, null, null), null, null));
									parentflwr.addWhereClause(new LocatedExpression(newSteps, false, typeVisitor, false), typeVisitor);
								} else {
									// set newVar expression
									ArrayList newSteps = new ArrayList();
									for (int i = 0; i < parentIndex - 1; i++)
										newSteps.add(((Step) steps.get(i)).clone());
									newSteps.add(steps.get(parentIndex));
									LocatedExpression locParent = new LocatedExpression(newSteps, false, typeVisitor);
									newSteps = new ArrayList();
									for (int i = 0; i < parentIndex - 1; i++)
										newSteps.add(steps.get(i));
									newSteps.add(steps.get(parentIndex - 1));
									LocatedExpression locDosNode = new LocatedExpression(newSteps, false, typeVisitor);
									XQueryExpression newVarExpr = new ListOpUNIONExpression(locParent, locDosNode, typeVisitor);
									Variable newVar = createVariable(varParent.getBindingType(), newVarExpr);
									parentflwr.addVariable(parentflwr.getVarPosition(varParent), newVar);
									// set varParent expression
									newSteps = new ArrayList();
									newSteps.add(new Step(false, Axis.NONE, newVar, null, null));
									newSteps.addAll(steps.subList(parentIndex + 1, steps.size()));
									this.expr = new LocatedExpression(newSteps, false, typeVisitor);
									// add where clause
									newSteps = new ArrayList(2);
									newSteps.add(new Step(false, Axis.NONE, newVar, null, null));
									newSteps.add(new Step(true, Axis.CHILD, new NodeType(NodeKind.NODE, null, null), null, null));
									parentflwr.addWhereClause(new LocatedExpression(newSteps, false, typeVisitor, false), typeVisitor);
								}
								/*
								// remove descendant-or-self::node()
								Step dosnodeStep = (Step) steps.get(parentIndex - 1);
								dosnodeStep.setHasSeparator(false);
								steps.remove(parentIndex - 1);
								// remove parent::node()
								Step parentStep = (Step) steps.get(parentIndex - 1);
								parentStep.setHasSeparator(false);
								steps.remove(parentIndex - 1);
								//create step list
								ArrayList newSteps = new ArrayList(1);
								newSteps.add(new Step(false, Axis.CHILD, new NodeType(NodeKind.NODE, null, null), null, null));
								//create predicate list
								ArrayList predicates = new ArrayList(1);
								LocatedExpression predLoc = new LocatedExpression(newSteps, true, typeVisitor, false);
								predLoc.setBracket(true);
								predicates.add(predLoc);
								// add new step
								newSteps = new ArrayList(1);
								newSteps.add(parentStep);
								LocatedExpression locParent = new LocatedExpression(newSteps, false, null, false);
								newSteps = new ArrayList(1);
								newSteps.add(dosnodeStep);
								LocatedExpression locDosNode = new LocatedExpression(newSteps, false, null, false);
								steps.add(parentIndex - 1, new Step(true, Axis.NONE, new ListOpUNIONExpression(locParent, locDosNode, null), predicates, null));
								*/

								/*
								// CASE 3
								if (parentIndex == steps.size() - 1) {
									throw new NormalizeException("CASE 3 : Could not normalize located expression with parent::node : " + arg);
								} else { // CASE 4
									throw new NormalizeException("CASE 4 : Could not normalize located expression with parent::node : " + arg);
								}
								*/
							}
							this.normalized = true;
						}
					}
				}

				//For multivalued expression :
				if ((this.expr != null) && (this.expr instanceof LocatedExpression) && (localCtx.existsMemo(this.depth, VisitorContext._MULTIVALUED_NEEDEXISTSFUNCTION))) {
					//Put this expression in exists function :
					this.normalized = true;
					boolean brace = this.expr.getBrace();
					boolean bracket = this.expr.getBracket();
					boolean parenthesis2 = this.expr.getParenthesis();
					this.expr.setBrace(false);
					this.expr.setBracket(false);
					this.expr.setParenthesis(false);
					ArrayList parameter = new ArrayList();
					parameter.add(this.expr);
					this.expr = new FunctionEXISTS(parameter, this.typeVisitor);
					this.expr.setBrace(brace);
					this.expr.setBracket(bracket);
					this.expr.setParenthesis(parenthesis2);
				}
			}
			//outln("("+nn+")==>LOCATED EXPRESSION AFTER (final) ("+this.normalized+")= "+this.expr);
			//if (this.expr != null) {printClass(this.expr);outln("QTYPE = "+this.typeToString(this.expr.getQType()));}

			localCtx.removeMemo(VisitorContext._ANY_DEPTH, VisitorContext._LOCATEDEXPRESSION_INPREDICATE);

			//Update response context with current context to pass the answers of parent questions:
			this.context.update(localCtx);
			//untrace();
		} catch (Exception e) {
			if (e instanceof NormalizeException)
				throw (NormalizeException) e;
			else
				throw new NormalizeException(0, "Unexpected exception in LocatedExpression :\n==>" + e.toString() + "<==\n", e);
		}
	}

	/**
	 * This visit method normalizes a NodeType (do nothing: this is a terminal node). A NodeType is a node like :<BR>
	 * <OL>
	 * <LI>node()</LI>
	 * <LI>text()</LI>
	 * <LI>comment()</LI>
	 * <LI>comment()</LI>
	 * <LI>processing-instruction("...")</LI>
	 * </OL>
	 * It possibly answers the questions of the current context.
	 * @param arg a NodeType to normalize.
	 */
	public void visit(NodeType arg) throws NormalizeException {
		try {
			//trace("<o-"+(nodeNumber++)+">NodeType<o>(hc="+arg.hashCode()+") : "+arg.toString()+" QType="+this.typeToString(arg.getQType()),0);
			//Set new expression:
			this.expr = arg;
			//Answer/execute questions/orders found in local context :
			boolean brace = this.expr.getBrace();
			boolean bracket = this.expr.getBracket();
			boolean parenthesis = this.expr.getParenthesis();
			//For LocatedExpression :
			if (this.context.existsMemo(VisitorContext._ANY_DEPTH, VisitorContext._LOCATEDEXPRESSION_INPREDICATE)) {
				//For rule LE(PRED) => FLWR(LE) :
				Variable newVar4LE_Predicate = (Variable) this.context.getMemo(VisitorContext._ANY_DEPTH, VisitorContext._LOCATEDEXPRESSION_INPREDICATE);
				//Put variable in front of this QName (and transform it in a child Step) :
				this.normalized = true;
				ArrayList newSteps = new ArrayList();
				Step firstStep = new Step(false, Axis.NONE, newVar4LE_Predicate, null, null);
				newSteps.add(firstStep);
				this.expr.setBrace(false);
				this.expr.setBracket(false);
				this.expr.setParenthesis(false);
				//To indicate that this QName is no more contextual :
				this.expr.setContext(false);
				Step secondStep = new Step(true, Axis.CHILD, this.expr, null, null);
				newSteps.add(secondStep);
				LocatedExpression newLocated = new LocatedExpression(newSteps, this.typeVisitor);
				this.expr = newLocated;
				//For multivalued expression (put in exists function) :
				if (this.context.existsMemo(this.depth, VisitorContext._MULTIVALUED_NEEDEXISTSFUNCTION)) {
					//Put the located expression in function EXISTS :
					ArrayList parameter = new ArrayList();
					parameter.add(this.expr);
					this.expr = new FunctionEXISTS(parameter, this.typeVisitor);
				}
				this.expr.setBrace(brace);
				this.expr.setBracket(bracket);
				this.expr.setParenthesis(parenthesis);
			}
			//Re-type this expression :
			if (this.typeVisitor != null) {
				this.expr.accept(this.typeVisitor);
			}
			//untrace();
		} catch (Exception e) {
			if (e instanceof NormalizeException)
				throw (NormalizeException) e;
			else
				throw new NormalizeException(0, "Unexpected exception in NodeType :\n==>" + e.toString() + "<==\n", e);
		}
	}

	/**
	 * This visit method normalizes a Parameter and its sub expression.<BR>
	 * It possibly answers the questions of the current context.
	 * @param arg a Parameter to normalize.
	 */
	//	public void visit(Parameter arg) throws NormalizeException {
	//		//TODO
	//		throw new NormalizeException(0, "visit(Parameter) is not implemented !");
	//		try {
	//		    //trace("<o-"+(nodeNumber++)+">Parameter<o>(hc="+arg.hashCode()+") : "+arg.toString()+" QType="+this.typeToString(arg.getQType()),0);
	//		    ExpressionNormalizationVisitor recurseVisitor = new ExpressionNormalizationVisitor(this.depth, this.typeVisitor);
	//		    VisitorContext localCtx = new VisitorContext();
	//		    localCtx.merge(this.context, false);
	//		    //Clean all questions which depth is smaller than current depth:
	//		    localCtx.filter(this.depth);
	//		    recurseVisitor.setContext(localCtx);
	//		    //Normalize parameter type name:
	//		    QName paramType = arg.getDataType();
	//		    if (paramType != null) {
	//		        paramType.accept(recurseVisitor);
	//		        localCtx.update(recurseVisitor.getContext());
	//		        paramType = (QName)recurseVisitor.getExpr();
	//		        //Update the normalized flag:
	//		        this.normalized = recurseVisitor.wasNormalized();
	//		    }
	//		    //Normalize variable:
	//		    Variable var = arg.getVariable();
	//		    if (var != null) {
	//		        var.accept(recurseVisitor);
	//		        localCtx.update(recurseVisitor.getContext());
	//		        var = (Variable)recurseVisitor.getExpr();
	//		        //Update the normalized flag:
	//		        this.normalized = this.normalized | recurseVisitor.wasNormalized();
	//		    }
	//		    //Test if sub expressions were normalized:
	//		    if (this.normalized) {
	//		        //Update current Parameter:
	//		        arg.setDataType(paramType);
	//		        arg.setVariable(var);
	//		        //Re-type this expression :
	//		        if (this.typeVisitor != null) {
	//		            arg.accept(this.typeVisitor);
	//		        }
	//			}
	//			this.expr = arg;
	//		    //Update response context with current context to pass the answers of parent questions:
	//		    this.context.update(localCtx);
	//		    //untrace();
	//		} catch (Exception e) {
	//			if (e instanceof NormalizeException)
	//			    throw (NormalizeException)e;
	//			else
	//			    throw new NormalizeException(0,"Unexpected exception in Parameter :\n==>"+e.toString()+"<==\n",e);
	//		}
	//	}

	/**
	 * This visit method normalizes a QName (do nothing: this is a terminal node).<BR>
	 * It possibly answers the questions of the current context.
	 * @param arg a QName to normalize.
	 */
	public void visit(QName arg) throws NormalizeException {
		try {
			//trace("<o-"+(nodeNumber++)+">QName<o>(hc="+arg.hashCode()+") : "+arg.toString()+" QType="+this.typeToString(arg.getQType()),0);
			//Set new expression:
			this.expr = arg;
			//Answer/execute questions/orders found in local context :
			boolean brace = this.expr.getBrace();
			boolean bracket = this.expr.getBracket();
			boolean parenthesis = this.expr.getParenthesis();
			//For LocatedExpression :
			if (this.context.existsMemo(VisitorContext._ANY_DEPTH, VisitorContext._LOCATEDEXPRESSION_INPREDICATE)) {
				//For rule LE(PRED) => FLWR(LE) :
				Variable newVar4LE_Predicate = (Variable) this.context.getMemo(VisitorContext._ANY_DEPTH, VisitorContext._LOCATEDEXPRESSION_INPREDICATE);
				//Put variable in front of this QName (and transform it in a child Step) :
				this.normalized = true;
				ArrayList newSteps = new ArrayList();
				Step firstStep = new Step(false, Axis.NONE, newVar4LE_Predicate, null, null);
				newSteps.add(firstStep);
				this.expr.setBrace(false);
				this.expr.setBracket(false);
				this.expr.setParenthesis(false);
				//To indicate that this QName is no more contextual :
				this.expr.setContext(false);
				Step secondStep = new Step(true, Axis.CHILD, this.expr, null, null);
				newSteps.add(secondStep);
				LocatedExpression newLocated = new LocatedExpression(newSteps, this.typeVisitor);
				this.expr = newLocated;
				//For multivalued expression (put in exists function) :
				if (this.context.existsMemo(this.depth, VisitorContext._MULTIVALUED_NEEDEXISTSFUNCTION)) {
					//Put the located expression in function EXISTS :
					ArrayList parameter = new ArrayList();
					parameter.add(this.expr);
					this.expr = new FunctionEXISTS(parameter, this.typeVisitor);
				}
				this.expr.setBrace(brace);
				this.expr.setBracket(bracket);
				this.expr.setParenthesis(parenthesis);
			}
			//Re-type this expression :
			/*
			if (this.typeVisitor != null) {
				this.expr.accept(this.typeVisitor);
			}
			*/
			//untrace();
		} catch (Exception e) {
			if (e instanceof NormalizeException)
				throw (NormalizeException) e;
			else
				throw new NormalizeException(0, "Unexpected exception in QName :\n==>" + e.toString() + "<==\n", e);
		}
	}

	/**
	 * This visit method normalizes an SequenceTypeExpression.<BR>
	 * It possibly answers the questions of the current context.
	 * @param arg a SequenceTypeExpression to normalize.
	 */
	public void visit(SequenceTypeExpression arg) throws NormalizeException {
		try {
			//trace("<o-"+(nodeNumber++)+">SequenceTypeExpression<o>(hc="+arg.hashCode()+") : "+arg.toString()+" QType="+this.typeToString(arg.getQType()),0);
			throw new NormalizeException(0, "SequenceTypeExpression not supported : (" + arg + ")");
			/*this.expr = arg;
			//Re-type this expression :
			if (this.typeVisitor != null) {
			    this.expr.accept(this.typeVisitor);
			}
			//untrace();*/
		} catch (Exception e) {
			if (e instanceof NormalizeException)
				throw (NormalizeException) e;
			else
				throw new NormalizeException(0, "Unexpected exception in SequenceTypeExpression :\n==>" + e.toString() + "<==\n", e);
		}
	}

	/**
	 * This visit method normalizes a SortedExpression (E sortby (sort clause(s))) and its sub expressions (E and
	 * sort clauses).<BR>
	 * This method apply theses rules:<BR>
	 * <OL>
	 * <LI>N(E sortby (C1, C2...Cn)) => N(E) sortby (N(C1), N(C2)...N(Cn))</LI>
	 * <LI>N(null sortby (Ci)) => null</LI>
	 * <LI>N(E sortby (null)) => N(E)</LI>
	 * </OL>
	 * It possibly answers the questions of the current context.
	 * @param arg a SortedExpression to normalize.
	 */
	public void visit(SortedExpression arg) throws NormalizeException {
		try {
			//trace("<o-"+(nodeNumber++)+">SortedExpression<o>(hc="+arg.hashCode()+") : "+arg.toString()+" QType="+this.typeToString(arg.getQType()),0);
			ExpressionNormalizationVisitor recurseVisitor = new ExpressionNormalizationVisitor(this.depth, this.typeVisitor);
			VisitorContext localCtx = new VisitorContext();
			localCtx.merge(this.context, false);
			//Clean all questions which depth is smaller than current depth:
			localCtx.filter(this.depth);
			recurseVisitor.setContext(localCtx);
			//Normalize sub expression:
			XQueryExpression exprSeq = arg.getExpression();
			exprSeq.accept(recurseVisitor);
			localCtx.update(recurseVisitor.getContext());
			exprSeq = recurseVisitor.getExpr();
			//Update the normalized flag:
			this.normalized = recurseVisitor.wasNormalized();
			if (recurseVisitor.wasNormalized()) {
				if (exprSeq != null) {
					arg.setExpression(exprSeq);
				} else {
					arg = null;
				}
			}
			if (arg != null) {
				//Normalize sort clauses:
				ArrayList sortClauses = arg.getSortClauses();
				if ((sortClauses != null) && (sortClauses.size() != 0)) {
					//Normalize each clause :
					int[] direction;
					int sortClauseLen = sortClauses.size();
					XQueryExpression aClause = null;
					for (int i = sortClauseLen - 1; i >= 0; i--) {
						//Get order:
						direction = ((XQueryExpression) sortClauses.get(i)).getOrder();
						//Normalize this clause :
						 ((XQueryExpression) sortClauses.get(i)).accept(recurseVisitor);
						localCtx.update(recurseVisitor.getContext());
						aClause = recurseVisitor.getExpr();
						//Update the normalized flag:
						this.normalized = this.normalized | recurseVisitor.wasNormalized();
						if (recurseVisitor.wasNormalized()) {
							aClause = recurseVisitor.getExpr();
							if (aClause != null) {
								aClause.setOrder(direction);
								sortClauses.set(i, aClause);
							} else {
								sortClauses.remove(i);
							}
						}
					}
					if (sortClauses.size() == 0) {
						this.expr = exprSeq;
					} else {
						arg.setSortClauses(sortClauses);
						this.expr = arg;
					}
				} else {
					this.expr = exprSeq;
				}
				//Re-type this expression :
				if (this.normalized && (this.expr != null) && (this.typeVisitor != null)) {
					this.expr.accept(this.typeVisitor);
				}
			} else {
				this.expr = null;
			}
			//Update response context with current context to pass the answers of parent questions:
			this.context.update(localCtx);
			//untrace();
		} catch (Exception e) {
			if (e instanceof NormalizeException)
				throw (NormalizeException) e;
			else
				throw new NormalizeException(0, "Unexpected exception in SortedExpression :\n==>" + e.toString() + "<==\n", e);
		}
	}

	/**
	 * This visit method normalizes a Step and its sub expression.<BR>
	 * This method apply theses rules:<BR>
	 * <OL>
	 * <LI>N(/Axis::E1[E2]) => /Axis::N(E1)[N(E2)]</LI>
	 * <LI>N(/Axis::null[E]) => null</LI>
	 * <LI>N(/Axis::E[null|false()|(empty sequence)]) => null</LI>
	 * <LI>N(/Axis::E[true()|(not empty sequence)]) => /Axis::N(E)</LI>
	 * <LI>N(/Axis::E[P1][P2][P3]...) => /Axis::N(E)[N(P1 AND P2 AND P3 AND ...)]</LI>
	 * </OL>
	 * It possibly answers the questions of the current context.
	 * @param arg a Step to normalize.
	 */
	public void visit(Step arg) throws NormalizeException {
		try {
			//outln(nn+" "+arg.getSeparator()+" "+arg.getAxis());
			ExpressionNormalizationVisitor recurseVisitor = new ExpressionNormalizationVisitor(this.depth, this.typeVisitor);
			VisitorContext localCtx = new VisitorContext();
			localCtx.merge(this.context, false);
			//Clean all questions which depth is smaller than current depth:
			localCtx.filter(this.depth);
			recurseVisitor.setContext(localCtx);
			//Get Step separator and axis:
			//            boolean separator = arg.hasSeparator();
			//            if (separator > Constants.MAX_SEPARATOR)
			//                throw new NormalizeException(0,"Separator of a step must be always \"/\" or nothing.");
			// int axis = arg.getAxis();
			// erase LARS not needed should have been tested before
			//Only supported axis :
			//			if (!((axis == Axis.NONE) || (axis == Axis.CHILD) || (axis == Axis.DESCENDANT_OR_SELF) || (axis == Axis.SELF) || (axis == Axis.ATTRIBUTE) || (axis == Axis.DESCENDANT) || (axis == Axis.PARENT))) {
			//				throw new NormalizeException(0, "Axis " + Axis.AXISSTRINGS[axis] + " not supported in step " + arg + ".");
			//			}
			//Normalize sub expression:
			XQueryExpression subExpr = arg.getExpression();
			subExpr.accept(recurseVisitor);
			localCtx.update(recurseVisitor.getContext());
			subExpr = recurseVisitor.getExpr();
			//Update the normalized flag:
			this.normalized = recurseVisitor.wasNormalized();
			if (recurseVisitor.wasNormalized()) {
				if (subExpr != null) {
					arg.setExpression(subExpr);
				} else {
					arg = null;
				}
			}
			//For LE(PRED) (See LocatedExpression) :
			//Note this arraylist is empty :
			ArrayList expression4VariableBuffer = (ArrayList) localCtx.getMemo(this.depth, VisitorContext._STEP_GETPREDICATE);
			localCtx.setMemo(this.depth, VisitorContext._STEP_GETPREDICATE, null, true);
			if (arg != null) {
				//Get predicates if exists :
				ArrayList predicates = arg.getPredicates();
				if ((predicates != null) && (predicates.size() > 0)) {
					//For rule LE(PRED) :
					Variable newVar = null;
					//Always true if steps are only in LocatedExpression :
					if (localCtx.existsMemo(this.depth, VisitorContext._STEP_GETPREDICATE)) {
						//Add the current step without its predicate :
						arg.setPredicates(null);
						expression4VariableBuffer.add(arg);
						//Create expression of the variable :
						if (expression4VariableBuffer.get(0) instanceof Variable) {
							Variable headerVariable = (Variable) expression4VariableBuffer.get(0);
							Step headerStep = new Step(false, Axis.NONE, headerVariable, null, null);
							expression4VariableBuffer.set(0, headerStep);
						}
						//Create new variable :
						// ADD LARS to avoid located expression with a single step without predicates
						if (expression4VariableBuffer.size() == 1)
							newVar = createVariable(Constants.FOR_BINDINGTYPE, ((Step) expression4VariableBuffer.get(0)).getExpression());
						else
							newVar = createVariable(Constants.FOR_BINDINGTYPE, new LocatedExpression(expression4VariableBuffer, this.typeVisitor));
						//Prepare context order for sub expressions in predicates :
						localCtx.setMemo(VisitorContext._ANY_DEPTH, VisitorContext._LOCATEDEXPRESSION_INPREDICATE, newVar, true);
					}
					//Normalize all predicates:
					int predicatesLen = predicates.size();
					boolean eliminateThis = false;
					XQueryExpression lastPredicate = null;
					for (int i = predicatesLen - 1;(i >= 0) && (arg != null); i--) {
						//For multivalued expression (put in exists function) :
						localCtx.setMemo((this.depth + 1), VisitorContext._MULTIVALUED_NEEDEXISTSFUNCTION, null, true);
						//Normalize the current predicate :
						 ((XQueryExpression) predicates.get(i)).accept(recurseVisitor);
						localCtx.update(recurseVisitor.getContext());
						//Update the normalized flag:
						this.normalized = this.normalized | recurseVisitor.wasNormalized();
						if (recurseVisitor.getExpr() != null) {
							XQueryExpression aPredicate = recurseVisitor.getExpr();
							//Test if predicate or step can be eliminated :
							if (aPredicate instanceof FunctionTRUE) {
								//Case predicate == true() :
								this.normalized = true;
								aPredicate = null;
							} else if (aPredicate instanceof FunctionFALSE) {
								//Case predicate == false() :
								this.normalized = true;
								arg = null;
							} else if (aPredicate instanceof XQueryExpressionSequence) {
								//Case predicate == sequence :
								int size = ((XQueryExpressionSequence) aPredicate).getSubExpressions().size();
								this.normalized = true;
								if (size == 0) {
									arg = null; //Empty sequence == false
								} else {
									aPredicate = null;
									//Not empty sequence == true
								}
							}
							//Concat predicates (AND) :
							if (arg != null) {
								if (aPredicate != null) {
									aPredicate.setBracket(false);
									if (!(aPredicate instanceof XQueryExpressionSequence)) {
										aPredicate.setParenthesis(true);
									}
									if ((predicatesLen >= 2) && (lastPredicate != null)) {
										this.normalized = true;
										lastPredicate = new BinOpANDExpression(aPredicate, lastPredicate, this.typeVisitor);
									} else {
										lastPredicate = aPredicate;
									}
								}
							}
						} else {
							this.normalized = true;
							arg = null;
						}
					} //End for
					//For multivalued expression (put in exists function) :
					localCtx.removeMemo((this.depth + 1), VisitorContext._MULTIVALUED_NEEDEXISTSFUNCTION);
					if (arg != null) {
						if (lastPredicate != null) {
							//For LocatedExpression rule LE(PRED) :
							ArrayList variableAndPredicate = new ArrayList();
							variableAndPredicate.add(newVar);
							variableAndPredicate.add(lastPredicate);
							localCtx.setMemo(this.depth, VisitorContext._STEP_GETPREDICATE, variableAndPredicate, true);
							//Remove predicate of the current step :
							this.normalized = true;
						}
						arg.setPredicates(null);
					}
				}
				//Remove memo :
				//For LocatedExpression rule LE(PRED) :
				if (localCtx.existsMemo(this.depth, VisitorContext._STEP_GETPREDICATE)) {
					localCtx.removeMemo(VisitorContext._ANY_DEPTH, VisitorContext._LOCATEDEXPRESSION_INPREDICATE);
				}
			}
			this.expr = arg;
			//Answer/execute questions of the current context (we must use the normalized Step to answer):
			if (this.expr != null) {
				//For LocatedExpression:
				//Answer question : it is a parent axis step ?
				if ((localCtx.existsMemo(this.depth, VisitorContext._STEP_ISPARENTAXISSTEP)) && (this.expr instanceof Step) && (((Step) this.expr).getAxis() == Axis.PARENT)) {
					localCtx.setMemo(this.depth, VisitorContext._STEP_ISPARENTAXISSTEP, Boolean.TRUE, true);
				}
			}
			//Re-type this expression :
			if (this.normalized && (this.expr != null) && (this.typeVisitor != null)) {
				this.expr.accept(this.typeVisitor);
			}
			//outln("("+nn+") STEP AFTER ("+this.normalized+") = "+this.expr);
			//Update response context with current context to pass the answers of parent questions:
			this.context.update(localCtx);
			//untrace();
		} catch (Exception e) {
			if (e instanceof NormalizeException)
				throw (NormalizeException) e;
			else
				throw new NormalizeException(0, "Unexpected exception in Step :\n==>" + e.toString() + "<==\n", e);
		}
	}

	/**
	 * This visit method normalizes a TypeSwitchExpression and its sub expressions.<BR>
	 * This method apply theses rules:<BR>
	 * <OL>
	 * <LI>N(typeswitch(E1) as $x (case Type return E2)+ default return E3)
	 *  => typeswitch(N(E1)) as $x (case Type return N(E2))+ default return N(E3)</LI>
	 * <LI>N(typeswitch(null) as $x (case Type return E2)+ default return E3)
	 *  => typeswitch(()) as $x (case Type return N(E2))+ default return N(E3))</LI>
	 * <LI>N(typeswitch(E1) as $x (case Type return null)+ default return E3)
	 *  => typeswitch(N(E1)) as $x (case Type return ())+ default return N(E3))</LI>
	 * <LI>N(typeswitch(E1) as $x (case Type return E2)+ default return ())
	 *  => typeswitch(N(E1)) as $x (case Type return N(E2))+ default return ())</LI>
	 * </OL>
	 * It possibly answers the questions of the current context.
	 * @param arg a TypeSwitchExpression to normalize.
	 */
	//	public void visit(TypeSwitchExpression arg) throws NormalizeException {
	//		try {
	//			//trace("<o-"+(nodeNumber++)+">TypeSwitchExpression<o>(hc="+arg.hashCode()+") : "+arg.toString()+" QType="+this.typeToString(arg.getQType()),0);
	//			throw new NormalizeException(0, "TypeSwitchExpression not supported : (" + arg + ")");
	//			/*ExpressionNormalizationVisitor recurseVisitor = new ExpressionNormalizationVisitor(this.depth, this.typeVisitor);
	//			VisitorContext localCtx = new VisitorContext();
	//			localCtx.merge(this.context, false);
	//			//Clean all questions which depth is smaller than current depth:
	//			localCtx.filter(this.depth);
	//			//Set one information memo for not create useless sequence:
	//			localCtx.setMemo(this.depth+1, VisitorContext._INCLUDEINPARENTHESIS,null,true);
	//			localCtx.setMemo(this.depth+1), VisitorContext._VARIABLE_DECLARATION,null,true);
	//			recurseVisitor.setContext(localCtx);
	//			//Normalize switch expression:
	//			Variable var = arg.getVariable();
	//			var.accept(recurseVisitor);
	//			localCtx.update(recurseVisitor.getContext());
	//			var = (Variable)recurseVisitor.getExpr();
	//			//Update the normalized flag:
	//			this.normalized = recurseVisitor.wasNormalized();
	//			//Remove local memo:
	//			//localCtx.removeMemo(this.depth+1, VisitorContext._INCLUDEINPARENTHESIS);
	//			if (recurseVisitor.wasNormalized()) {
	//			    if (var.getExpression() == null) {
	//			        var.setExpression(new XQueryVoid(this.typeVisitor));
	//			    }
	//			    //Set variable :
	//			    arg.setVariable(var);
	//			}
	//			//outln(" VAR = "+arg.getVariable()+" "+((Variable)arg.getVariable()).getExpression());
	//			//Normalize all cases clauses:
	//			HashMap caseClauses = arg.getCaseHash();
	//			Object[] keys = caseClauses.keySet().toArray();
	//			int keysLen = keys.length;
	//			for (int i = 0 ; i < keysLen ; i++) {
	//			    ((XQueryExpression)caseClauses.get(keys[i])).accept(recurseVisitor);
	//			    localCtx.update(recurseVisitor.getContext());
	//			    caseClauses.put(keys[i], recurseVisitor.getExpr());
	//			    //Update the normalized flag:
	//			    this.normalized = this.normalized | recurseVisitor.wasNormalized();
	//			}
	//			if (recurseVisitor.wasNormalized()) {
	//			    //TODO
	//			}
	//			//Normalize default expression:
	//			XQueryExpression defaultExpr = arg.getExpression2();
	//			defaultExpr.accept(recurseVisitor);
	//			localCtx.update(recurseVisitor.getContext());
	//			defaultExpr = recurseVisitor.getExpr();
	//			//Update the normalized flag:
	//			this.normalized = this.normalized | recurseVisitor.wasNormalized();
	//			if (recurseVisitor.wasNormalized()) {
	//			    if (defaultExpr == null) {
	//			        defaultExpr = new XQueryVoid(this.typeVisitor);
	//			    }
	//			    //Set Default clause :
	//			    arg.setExpression2(defaultExpr);
	//			}
	//			this.expr = arg;
	//			//Re-type this expression :
	//			if (this.normalized && (this.expr != null) && (this.typeVisitor != null)) {
	//			    this.expr.accept(this.typeVisitor);
	//			}
	//			//Update response context with current context to pass the answers of parent questions:
	//			this.context.update(localCtx);
	//			//untrace();*/
	//		} catch (Exception e) {
	//			if (e instanceof NormalizeException)
	//				throw (NormalizeException) e;
	//			else
	//				throw new NormalizeException(0, "Unexpected exception in TypeSwitchExpression :\n==>" + e.toString() + "<==\n", e);
	//		}
	//	}

	/**
	 * This visit method normalizes a Variable and its expression (its value).<BR>
	 * If it is a variable declaration, we normalize its expression.
	 * This method apply theses rules:<BR>
	 * <OL>
	 * <LI>N(Var(E)) => Var(N(E)) if this instance of Var is a declaration</LI>
	 * <LI>N(Var(E1)) and substitution order of Var(E1) by E2 => N(E2)</LI>
	 * <LI>N(Var(E1)) and substitution order of E1 by E2 => Var(N(E2))</LI>
	 * <LI>N(C(E)) C = condition and E = LE || FLWR || VAR => N(C(EXISTS(E)))</LI>
	 * </OL>
	 * It possibly answers the questions of the current context.
	 * @param arg a Variable to normalize.
	 */
	public void visit(Variable arg) throws NormalizeException {
		try {
			ExpressionNormalizationVisitor recurseVisitor = new ExpressionNormalizationVisitor(this.depth, this.typeVisitor);
			VisitorContext localCtx = new VisitorContext();
			localCtx.merge(this.context, false);
			//outln("VAR CONTEXT = "+localCtx);
			//Clean all questions which depth is smaller than current depth:
			localCtx.filter(this.depth);
			recurseVisitor.setContext(localCtx);
			//Rename variable :
//			if (traceLevel != -100) {
//				arg.setName(ExpressionNormalizationVisitor.getUniqueVarName(arg.getName()));
//			}
			//Normalize expression of this variable if this is a variable declaration (one time):
			if (localCtx.existsMemo(this.depth, VisitorContext._VARIABLE_DECLARATION)) {
				(arg.getExpression()).accept(recurseVisitor);
				localCtx.update(recurseVisitor.getContext());
				if (recurseVisitor.getExpr() != null) {
					arg.setExpression(recurseVisitor.getExpr(), typeVisitor);
				} else {
					arg = null;
				}
				//Update the normalized flag:
				this.normalized = recurseVisitor.wasNormalized();
			}
			this.expr = arg;
			//Answer/execute questions of the current context (we must use the normalized Variable to answer):
			if (this.expr != null) {
				//For FLWRExpression:
				//Execute orders : change expression of variable :
				if (localCtx.existsMemo(VisitorContext._ANY_DEPTH, VisitorContext._VARIABLE_CHANGEEXPRESSION4FLWR)) {
					Variable changeExpr = (Variable) localCtx.getMemo(VisitorContext._ANY_DEPTH, VisitorContext._VARIABLE_CHANGEEXPRESSION4FLWR);
					if (changeExpr.getVarName().equals(arg.getVarName())) {
						//outln(" ("+nn+") Change Expr for FLWR  (var = "+arg.getVarName()+") : "
						//+arg.getExpression()+" by "+(XQueryExpression)changeExpr.getExpression().clone());
						//We MUST clone the expression because it can be change many time !
						arg.setExpression((XQueryExpression) changeExpr.getExpression().clone(), typeVisitor);
						this.expr = arg;
						//Indicate that we normalized this expression:
						this.normalized = true;
					}

				}
				HashMap substVariables = null;
				//For FLWRExpression:
				//Execute orders : substitution of variable by an expression :
				if (this.expr != null) {
					if (localCtx.existsMemo(VisitorContext._ANY_DEPTH, VisitorContext._VARIABLE_SUBSTITUTION4FLWR)) {
						substVariables = (HashMap) localCtx.getMemo(VisitorContext._ANY_DEPTH, VisitorContext._VARIABLE_SUBSTITUTION4FLWR);
						if (substVariables.containsKey(arg.getVarName())) {
							XQueryExpression substExpr = (XQueryExpression) substVariables.get(arg.getVarName());
							//outln(" ("+nn+") ON SUBSTITUE(1) "+this.expr+" par "+substExpr);
							if (substExpr != null) {
								//We MUST clone the substitution expression because it can be substitute many time !
								this.expr = (XQueryExpression) substExpr.clone();
							} else { //substExpr == null
								this.expr = null;
							}
							//Indicate that we normalized this expression:
							this.normalized = true;
						}
					}
					if (localCtx.existsMemo(VisitorContext._ANY_DEPTH, VisitorContext._VARIABLE_SUBSTITUTION4LETFLWR)) {
						substVariables = (HashMap) localCtx.getMemo(VisitorContext._ANY_DEPTH, VisitorContext._VARIABLE_SUBSTITUTION4LETFLWR);
						if (substVariables.containsKey(arg.getVarName())) {
							XQueryExpression substExpr = (XQueryExpression) substVariables.get(arg.getVarName());
							//outln(" ("+nn+") ON SUBSTITUE(2) "+this.expr+" par "+substExpr+" AGREGAT ? "+localCtx.existsMemo(VisitorContext._ANY_DEPTH, VisitorContext._INAGGREGATEFUNCTION));
							if ((substExpr != null) && !localCtx.existsMemo(VisitorContext._ANY_DEPTH, VisitorContext._INAGGREGATEFUNCTION)) {
								//We MUST clone the substitution expression because it can be substitute many time !
								this.expr = (XQueryExpression) substExpr.clone();
								if (!this.expr.getBracket() && !this.expr.getBrace()) {
									this.expr.setParenthesis(true);
								}
								//Indicate that we normalized this expression:
								this.normalized = true;
								//To inform FLWR (where clause) of this variable substitution :
								if (localCtx.existsMemo(VisitorContext._ANY_DEPTH, VisitorContext._VARIABLE_WASSUBSTITUTELETFLWR)) {
									localCtx.setMemo(VisitorContext._ANY_DEPTH, VisitorContext._VARIABLE_WASSUBSTITUTELETFLWR, Boolean.TRUE, true);
								}
							} else if (substExpr == null) {
								this.expr = null;
								//Indicate that we normalized this expression:
								this.normalized = true;
							}
						}

					}
				}
				//For QuantifiedExpression:
				//Execute orders
				if ((this.expr != null) && (localCtx.existsMemo(VisitorContext._ANY_DEPTH, VisitorContext._VARIABLE_SUBSTITUTION4QUANTIFIED))) {
					substVariables = (HashMap) localCtx.getMemo(VisitorContext._ANY_DEPTH, VisitorContext._VARIABLE_SUBSTITUTION4QUANTIFIED);
					//outln(" ("+nn+") SUBSTITUTION DE VARIABLE(3) ? Current Varname = "+arg.getVarName()+" in "+substVariables);
					if (substVariables.containsKey(arg.getVarName())) {
						XQueryExpression substExpr = (XQueryExpression) substVariables.get(arg.getVarName());
						//outln(" ("+nn+") ON SUBSTITUE(3) "+this.expr+" par "+substExpr);
						if (substExpr != null) {
							//We MUST clone the substitution expression because it can be substitute many time !
							this.expr = (XQueryExpression) substExpr.clone();
						} else { //substExpr == null
							this.expr = null;
						}
						//Indicate that we normalized this expression:
						this.normalized = true;
					}
				}
			}
			//For multivalued expression (put in exists function) :
			if ((this.expr != null) && (localCtx.existsMemo(this.depth, VisitorContext._MULTIVALUED_NEEDEXISTSFUNCTION))) {
				this.normalized = true;
				boolean brace = this.expr.getBrace();
				boolean bracket = this.expr.getBracket();
				boolean parenthesis = this.expr.getParenthesis();
				this.expr.setBrace(false);
				this.expr.setBracket(false);
				this.expr.setParenthesis(false);
				ArrayList parameter = new ArrayList();
				parameter.add(this.expr);
				this.expr = new FunctionEXISTS(parameter, this.typeVisitor);
				this.expr.setBrace(brace);
				this.expr.setBracket(bracket);
				this.expr.setParenthesis(parenthesis);
			}
			//Re-type this expression :
			if (this.normalized && (this.expr != null) && (this.typeVisitor != null)) {
				this.expr.accept(this.typeVisitor);
			}
			//Update response context with current context to pass the answers of parent questions:
			this.context.update(localCtx);
			//outln(" ("+nn+") CTX VAR : "+localCtx);
			//outln(" ("+nn+") NEW VARIABLE (hc="+this.expr.hashCode()+") = "+this.expr+" "+this.normalized); //+" "+((Variable)this.expr).getExpression());
		} catch (Exception e) {
			if (e instanceof NormalizeException)
				throw (NormalizeException) e;
			else
				throw new NormalizeException(0, "Unexpected exception in Variable :\n==>" + e.toString() + "<==\n", e);
		}
	}

	/**
	 * This visit method normalizes a XQueryExpressionSequence (E1, E2, E3...).<BR>
	 * This method apply theses rules:<BR>
	 * <OL>
	 * <!-- 1 --><LI>N(SEQ(E)) => SEQ(N(E))</LI>
	 * <!-- 2 --><LI>N(SEQ(null)) => null</LI>
	 * <!-- 3 --><LI>N(SEQ(SEQ(...SEQ(E)...))) => SEQ(N(E)) with E <> SEQ</LI>
	 * <!-- 4 --><LI>N(SEQ(E1, SEQ(), SEQ(E2,E3), SEQ(E4))) => SEQ(N(E1), N(E2), N(E3), N(E4)) (sequence linearization)</LI>
	 * <!-- 5 --><LI>N("{"E"}") AND E <- EC (subsitution by EC expression) => N(EC)</LI>
	 * <!-- 6 --><LI>N("("E")") AND (DEPTH = 0 OR parent expression is a parenthesed) AND E is not a sequence (multivalued) => N(E)</LI>
	 * <!-- 7 --><LI>N("{""("E")""}") => "{"N(E)"}"</LI>
	 * <!-- 8 --><LI>N(FC"(""("E")"")") AND E is not a sequence => FC"("N(E)")"</LI>
	 * </OL>
	 * It possibly answers the questions of the current context.
	 * @param arg a XQueryExpressionSequence to normalize.
	 */
	public void visit(XQueryExpressionSequence arg) throws NormalizeException {
		try {
			//NOTE: method getExpressions() can not return null or anempty ArrayList (see XQueryExpressionSequence).
			ExpressionNormalizationVisitor recurseVisitor = new ExpressionNormalizationVisitor(this.depth, this.typeVisitor);
			VisitorContext localCtx = new VisitorContext();
			//Get parent context content:
			localCtx.merge(this.context, false);
			//Clean all questions which depth is smaller than current depth:
			localCtx.filter(this.depth);
			// Useless : if (localCtx.existsMemo(this.depth, VisitorContext._XQUERYEXPRESSIONSEQUENCE_ELEMENT_CHILDREN))
			//localCtx.setMemo(this.depth+1, VisitorContext._ELEMENT_CHILDREN,null, true);
			recurseVisitor.setContext(localCtx);
			//Get sub expression(s) :
			ArrayList expressions = arg.getSubExpressions();
			boolean brace = arg.getBrace();
			boolean isASequence = true;
			//Normalization rule 3 (and implicitly rule 1):
			while ((expressions != null) && (expressions.size() == 1) && (expressions.get(0) instanceof XQueryExpressionSequence)) {
				//We ignore current sequence (useless sequence) :
				arg = (XQueryExpressionSequence) expressions.get(0);
				expressions = arg.getSubExpressions();
				brace = brace | arg.getBrace();
				//Update the normalized flag:
				this.normalized = true;
			}
			//Here: arg is the new XQueryExpressionSequence to analyse and expressions are its child(s) expression(s).
			//To apply normalization rule 4 (not by recursiv way):
			if (expressions.size() > 1) {
				Object[] childExpr = expressions.toArray();
				ArrayList subExprTmp;
				expressions.clear();
				for (int i = 0; i < childExpr.length; i++) {
					if (childExpr[i] instanceof XQueryExpressionSequence) {
						//Note: subexpression of a sequence can not be null or empty (see XQueryExpressionSequence) !
						subExprTmp = ((XQueryExpressionSequence) childExpr[i]).getSubExpressions();
						//Add all children in result expressions ArrayList :
						for (int j = 0; j < subExprTmp.size(); j++) {
							expressions.add(subExprTmp.get(j));
						}
						//Update the normalized flag:
						this.normalized = true;
					} else if (!(childExpr[i] instanceof XQueryVoid)) {
						//Add child expression just "as it is":
						expressions.add(childExpr[i]);
					} //Else : ignore current expression.
				}
			}
			//Normalize the (linearized) sequence of expression(s) after the rules 3 and 4 were applied :
			ArrayList childNormalizedExpr = new ArrayList();
			//For multivalued expression (put in exists function) :
			boolean mustBeInExistsFunction = false;
			if (localCtx.existsMemo(this.depth, VisitorContext._MULTIVALUED_NEEDEXISTSFUNCTION)) {
				localCtx.removeMemo(this.depth, VisitorContext._MULTIVALUED_NEEDEXISTSFUNCTION);
				mustBeInExistsFunction = true;
			}
			//Normalize child expressions:
			for (int i = 0; i < expressions.size(); i++) {
				//For multivalued expression (put in exists function) :
				if (mustBeInExistsFunction) {
					localCtx.setMemo((this.depth + 1), VisitorContext._MULTIVALUED_NEEDEXISTSFUNCTION, null, true);
				}
				((XQueryExpression) expressions.get(i)).accept(recurseVisitor);
				localCtx.update(recurseVisitor.getContext());
				if (recurseVisitor.getExpr() != null) {
					childNormalizedExpr.add(recurseVisitor.getExpr());
				}
				//Update the normalized flag:
				this.normalized = this.normalized | recurseVisitor.wasNormalized();
			}
			//For multivalued expression (put in exists function) :
			localCtx.removeMemo((this.depth + 1), VisitorContext._MULTIVALUED_NEEDEXISTSFUNCTION);
			//For rule 5 :
			if ((childNormalizedExpr.size() == 1) && (arg.getBrace()) && (childNormalizedExpr.get(0) instanceof Element)) {
				//Return the sub expression without braces :
				this.expr = (XQueryExpression) childNormalizedExpr.get(0);
				isASequence = false;
				//Indicate that at least a rule was applied:
				this.normalized = true;
			} else if ((childNormalizedExpr.size() == 1) && (arg.getParenthesis()) && (this.depth == 1)) {
				//Rule 6 :
				//Return the sub expression without parenthesis:
				this.expr = (XQueryExpression) childNormalizedExpr.get(0);
				isASequence = false;
				//Indicate that at least a rule was applied:
				this.normalized = true;
			} else {
				//Test if sub expressions were normalized:
				if (this.normalized) {
					if (childNormalizedExpr.size() == 0) {
						expr = null;
					} else if (childNormalizedExpr.size() == 1) {
						expr = (XQueryExpression) childNormalizedExpr.get(0);
					} else {
						//Update current XQueryExpressionSequence:
						expr = new XQueryExpressionSequence(childNormalizedExpr, this.typeVisitor);
					}
				} else
					this.expr = arg;
			}
			//Set delimiters :
			if (this.expr != null) {
				//Set parenthesis, braces...:
				if (isASequence) {
					if (brace) {
						//Normalization rule 7
						this.setSequenceDelimiters(this.expr, true, true, arg.getBracket());
					} else {
						this.setSequenceDelimiters(this.expr, true, arg.getBrace(), arg.getBracket());
					}
				}
				//Execute order:
				//For rule 8 (order, not question):
				if ((this.expr instanceof XQueryExpressionSequence) && (localCtx.existsMemo(this.depth, VisitorContext._INCLUDEINPARENTHESIS))) {
					ArrayList subExpr = ((XQueryExpressionSequence) this.expr).getSubExpressions();
					if (subExpr.size() == 1) {
						this.expr = (XQueryExpression) subExpr.get(0);
					}
				}

			}
			//Re-type and answer question(s) :
			if (this.expr != null) {
				//Re-type this expression :
				if (this.normalized && (this.typeVisitor != null)) {
					this.expr.accept(this.typeVisitor);
				}
				//Answer questions of the current context (we must use the normalized XQueryExpressionSequence to answer):
				//For FWLRExpression :
				if ((localCtx.existsMemo(this.depth, VisitorContext._XQUERYEXPRESSIONSEQUENCE_EXISTS)) && (this.expr instanceof XQueryExpressionSequence)) {
					int sequenceSize = ((XQueryExpressionSequence) this.expr).getSubExpressions().size();
					localCtx.setMemo(this.depth, VisitorContext._XQUERYEXPRESSIONSEQUENCE_EXISTS, new Integer(sequenceSize), true);
				}
			}
			//Update response context with current context to pass the answers of parent questions:
			this.context.update(localCtx);
		} catch (Exception e) {
			if (e instanceof NormalizeException)
				throw (NormalizeException) e;
			else
				throw new NormalizeException(0, "Unexpected exception in XQueryExpressionSequence :\n==>" + e.toString() + "<==\n", e);
		}
	}

	/**
	 * This visit method normalizes a XQueryVoid (return null because XQueryVoid is an empty sequence : () ).<BR>
	 * It possibly answers the questions of the current context.
	 * @param arg a XQueryVoid to normalize.
	 */
	public void visit(XQueryVoid arg) throws NormalizeException {
		try {
			//Set new expression:
			this.expr = null;
			//Not re-type this expression !! it is null !
		} catch (Exception e) {
			if (e instanceof NormalizeException)
				throw (NormalizeException) e;
			else
				throw new NormalizeException(0, "Unexpected exception in XQueryVoid :\n==>" + e.toString() + "<==\n", e);
		}
	}
}
